Package: dendRoAnalyst
Type: Package
Date: 2026-05-18
Title: A Tool for Processing and Analyzing Dendrometer Data
Version: 0.1.6
Authors@R: c(
    person("Sugam", "Aryal", email = "sugam.aryal@fau.de", role = c("aut", "cre", "dtc")),
    person("Martin", "Häusser", email = "martin.haeusser@fau.de", role = "aut"),
    person("Jussi", "Grießinger", email = "jussi.griessinger@fau.de", role = "aut"),
    person("Ze-Xin", "Fan", email = "fanzexin@xtbg.org.cn", role = "aut"),
    person("Achim", "Bräuning", email = "achim.braeuning@fau.de", role = c("aut", "dgs")))
Maintainer: Sugam Aryal <sugam.aryal@fau.de>
Description: There are various functions for managing and cleaning data before the application of different approaches. This includes identifying and erasing sudden jumps in dendrometer data not related to environmental change, identifying the time gaps of recordings, and changing the temporal resolution of data to different frequencies. Furthermore, the package calculates daily statistics of dendrometer data, including the daily amplitude of tree growth. Various approaches can be applied to separate radial growth from daily cyclic shrinkage and expansion due to uptake and loss of stem water. In addition, it identifies periods of consecutive days with user-defined climatic conditions in daily meteorological data, then check what trees are doing during that period. 
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0), base
Imports: stats, tidyverse, dplyr, ggplot2, lubridate, readxl, tibble,
        tidyr, zoo, forecast, mgcv, minpack.lm, pspline, moments,
        signal, readr, boot, rlang, changepoint, WaveletComp
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-20 13:29:58 UTC; xe40husu
Author: Sugam Aryal [aut, cre, dtc],
  Martin Häusser [aut],
  Jussi Grießinger [aut],
  Ze-Xin Fan [aut],
  Achim Bräuning [aut, dgs]
Repository: CRAN
Date/Publication: 2026-05-20 14:40:02 UTC
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Built: R 4.6.0; ; 2026-05-22 00:32:02 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,SC_output)
S3method(plot,SC_output_clim)
S3method(plot,ZG_output)
S3method(plot,ZG_output_clim)
S3method(plot,clim_twd_stats)
S3method(plot,clim_twd_test)
S3method(plot,daily_output)
S3method(plot,daily_output_clim)
S3method(plot,dm_detrended)
S3method(plot,dm_epoch)
S3method(plot,dm_growth_evaluation)
S3method(plot,dm_growth_fit)
S3method(plot,dm_na_interpolation)
S3method(plot,dm_standardized)
S3method(plot,dm_wavelet)
S3method(plot,dm_wavelet_coherence)
S3method(plot,dm_wavelet_reconstruct)
S3method(plot,mean_dm_detrended)
S3method(plot,mov.cor)
S3method(plot,mov.cor.boot)
S3method(plot,mov_cor_dm)
S3method(plot,network_interpolation)
S3method(plot,summary_mov_cor_dm)
S3method(print,dm_growth_fit)
S3method(print,mov_cor_dm)
S3method(print,summary.clim_twd_stats)
S3method(print,summary.clim_twd_test)
S3method(print,summary.dm_epoch)
S3method(print,summary.dm_growth_fit)
S3method(print,summary.dm_wavelet)
S3method(print,summary.dm_wavelet_reconstruct)
S3method(print,summary_mov_cor_dm)
S3method(summary,clim_twd_stats)
S3method(summary,clim_twd_test)
S3method(summary,dm_epoch)
S3method(summary,dm_growth_fit)
S3method(summary,dm_wavelet)
S3method(summary,dm_wavelet_reconstruct)
S3method(summary,mov_cor_dm)
export(clim.twd)
export(clim.twd.stats)
export(clim.twd.test)
export(daily.data)
export(dendro.resample)
export(dendro.truncate)
export(dm.detrend.fit)
export(dm.fit.gompertz)
export(dm.growth.evaluate)
export(dm.growth.fit)
export(dm.growth.fit.double)
export(dm.na.interpolation)
export(dm_add_climate)
export(dm_daily_clim)
export(dm_epoch_extract)
export(dm_epoch_test)
export(dm_event_climate)
export(dm_event_climate_summary)
export(dm_event_climate_test)
export(dm_event_times)
export(dm_join_daily_clim)
export(dm_join_phase_clim)
export(dm_join_subdaily_clim)
export(dm_plot_climate)
export(dm_plot_climate_compare)
export(dm_standardize)
export(dm_subdaily_clim)
export(dm_wavelet)
export(dm_wavelet_coherence)
export(dm_wavelet_reconstruct)
export(i.jump.locator)
export(jump.locator)
export(mean_detrended.dm)
export(mov.cor.dm)
export(network.interpolation)
export(phase.sc)
export(phase.zg)
export(plot_dm_assessment)
export(plot_dm_gaps)
export(plot_dm_interpolation)
export(plot_event_climate_box)
export(plot_event_climate_relation)
export(plot_mov.cor)
export(read.climate)
export(read.dendrometer)
export(reso_dm)
export(smooth_dm)
export(twd.maxima)
importFrom(boot,boot)
importFrom(changepoint,cpt.mean)
importFrom(changepoint,cpts)
importFrom(dplyr,"%>%")
importFrom(dplyr,`%>%`)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_column)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,lag)
importFrom(dplyr,last)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice_max)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(forecast,na.interp)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_contour)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_c)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(lubridate,floor_date)
importFrom(lubridate,hour)
importFrom(lubridate,minute)
importFrom(lubridate,minutes)
importFrom(lubridate,month)
importFrom(lubridate,parse_date_time)
importFrom(lubridate,second)
importFrom(lubridate,seconds)
importFrom(lubridate,yday)
importFrom(lubridate,year)
importFrom(lubridate,ymd)
importFrom(lubridate,ymd_hms)
importFrom(minpack.lm,nlsLM)
importFrom(moments,kurtosis)
importFrom(pspline,smooth.Pspline)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(stats,aov)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pairwise.t.test)
importFrom(stats,pairwise.wilcox.test)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,runmed)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,smooth.spline)
importFrom(stats,t.test)
importFrom(stats,ts)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyverse,tidyverse_packages)
importFrom(tools,file_ext)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(zoo,na.approx)
importFrom(zoo,na.locf)
importFrom(zoo,na.spline)
importFrom(zoo,rollapply)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  warning = FALSE,
  message = FALSE
)

library(dendRoAnalyst)

RUN_HEAVY <- FALSE
RUN_INTERACTIVE <- FALSE

## ----data---------------------------------------------------------------------
data(gf_nepa17)
data(nepa17)
data(ktm_clim_hourly)
data(ktm_rain17)

head(gf_nepa17[, 1:3])
head(ktm_clim_hourly)

## ----objects------------------------------------------------------------------
dm_hourly <- gf_nepa17
dm_daily_demo <- nepa17[1:1000, ]
dm_phase_demo <- gf_nepa17[1:800, ]
clim_hourly <- ktm_clim_hourly
clim_rain <- ktm_rain17

## ----read-dendrometer, eval = FALSE-------------------------------------------
# raw_dm <- read.dendrometer(
#   file = "inst/extdata/my_dendrometer_file.csv",
#   detect_resolution = TRUE,
#   return_report = TRUE,
#   quiet = FALSE
# )

## ----read-climate-------------------------------------------------------------
clim_std <- read.climate(
  x = clim_hourly,
  time_col = "TIME",
  vars = c("temp", "prec", "VPD", "RH"),
  verbose = FALSE
)
head(clim_std)

## ----reso---------------------------------------------------------------------
reso_dm(dm_hourly[[1]])

## ----jump-manual--------------------------------------------------------------
jump_free_manual <- jump.locator(
  df = dm_daily_demo[, 1:3],
  detection_method = "manual",
  manual_threshold = 1,
  adjustment_method = "window_median"
)
head(jump_free_manual)

## ----jump-auto, eval = FALSE--------------------------------------------------
# jump_free_auto <- jump.locator(
#   df = dm_daily_demo[, 1:3],
#   detection_method = "auto",
#   auto_method_penalty = 10,
#   adjustment_method = "window_median"
# )

## ----jump-interactive, eval = FALSE-------------------------------------------
# # Interactive graphical workflow
# if (RUN_INTERACTIVE) {
#   i.jump.locator(df = dm_daily_demo[, 1:3])
# }

## ----gap-detect---------------------------------------------------------------
gap_demo <- dm_daily_demo[, 1:3]
gap_demo[150:170, 2] <- NA

gap_info <- dm.na.interpolation(
  df = gap_demo,
  resolution = 60,
  fill = FALSE,
  assess = FALSE
)
head(gap_info$gap_info)

## ----gap-fill-----------------------------------------------------------------
gap_fill <- dm.na.interpolation(
  df = gap_demo,
  resolution = 60,
  fill = TRUE,
  method = "spline",
  assess = TRUE,
  assess_lengths_hours = c(6, 12)
)
head(gap_fill$data)

## ----gap-plots, fig.height=5--------------------------------------------------
plot(gap_fill)
plot(gap_fill)
plot(gap_fill, original = gap_demo)
plot(gap_fill)

## ----resample-----------------------------------------------------------------
resampled_hourly <- dendro.resample(
  df = dm_hourly,
  by = "H",
  value = "mean",
  method = "auto"
)
head(resampled_hourly)

## ----truncate-----------------------------------------------------------------
truncated_demo <- dendro.truncate(dm_hourly, CalYear = 2017, DOY = c(1, 120))
head(truncated_demo)

## ----smooth-------------------------------------------------------------------
res_minutes <- reso_dm(dm_phase_demo[[1]])
smoothed_medmean <- smooth_dm(
  time = dm_phase_demo[[1]],
  dm = dm_phase_demo[[2]],
  resolution_min = res_minutes,
  method = "median_mean",
  window_hours = 3
)
head(smoothed_medmean)

## ----smooth-other, eval = FALSE-----------------------------------------------
# smoothed_pspline <- smooth_dm(
#   time = dm_phase_demo[[1]],
#   dm = dm_phase_demo[[2]],
#   resolution_min = res_minutes,
#   method = "pspline",
#   window_hours = 6
# )

## ----daily-data---------------------------------------------------------------
daily_out <- daily.data(df = dm_daily_demo, TreeNum = 1)
head(daily_out)

## ----daily-plots--------------------------------------------------------------
plot(daily_out)
plot(daily_out, type = "timing")
plot(daily_out, type = "change")

## ----daily-more-plots, eval = FALSE-------------------------------------------
# plot(daily_out, type = "timing_violin", Year = 2017, DOY = c(1, 6))
# plot(daily_out, type = "boxplot", stat = "amplitude", by = "month_of_year")
# plot(daily_out, type = "heatmap")

## ----daily-clim---------------------------------------------------------------
clim_day <- dm_daily_clim(
  clim_std,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec"),
  lag_vars = c("VPD_max", "temp_mean"),
  lagmean_vars = c("temp_mean", "VPD_mean"),
  lagsum_vars = c("prec_sum"),
  lag_days = c(1, 3, 7)
)
head(clim_day)

## ----daily-join---------------------------------------------------------------
daily_clim_joined <- dm_join_daily_clim(daily_out, clim_day)
head(daily_clim_joined)

## ----add-climate-daily--------------------------------------------------------
daily_clim_added <- dm_add_climate(
  daily_out,
  clim_hourly,
  scale = "daily",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(daily_clim_added)

## ----sc-out-------------------------------------------------------------------
sc_out <- phase.sc(df = dm_phase_demo, TreeNum = 1)
sc_out_sm <- phase.sc(df = dm_phase_demo, TreeNum = 1, smoothing = 12)
head(sc_out$SC_phase)
head(sc_out$SC_cycle)

## ----sc-plots-----------------------------------------------------------------
plot(sc_out)
plot(sc_out, type = "ribbon")
plot(sc_out, type = "increment")

## ----sc-more-plots, eval = FALSE----------------------------------------------
# plot(sc_out, type = "transition", x_axis = "doy", Year = 2017, DOY = c(50, 100))
# plot(sc_out, type = "balance", temporal = "daily")
# plot(sc_out, type = "frequency", temporal = "monthly")
# plot(sc_out, type = "heatmap", temporal = "daily")
# plot(sc_out, type = "boxplot", stat = "Magnitude", temporal = "monthly")

## ----subdaily-clim------------------------------------------------------------
clim_sub <- dm_subdaily_clim(
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  sum_vars  = c("prec"),
  lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)
head(clim_sub)

## ----sc-joins-----------------------------------------------------------------
sc_phase_clim <- dm_join_phase_clim(
  sc_out,
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(sc_phase_clim$SC_cycle)

sc_point_clim <- dm_join_subdaily_clim(sc_out_sm, clim_sub)
head(sc_point_clim$SC_phase)

sc_added_clim <- dm_add_climate(
  sc_out_sm,
  clim_hourly,
  scale = "subdaily",
  sub_mean_vars = c("temp", "VPD", "RH"),
  sub_sum_vars  = c("prec"),
  sub_lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)
head(sc_added_clim$SC_phase)

## ----zg-out-------------------------------------------------------------------
zg_out <- phase.zg(df = dm_phase_demo, TreeNum = 1)
zg_out_sm <- phase.zg(df = dm_phase_demo, TreeNum = 1, smoothing = 6)
head(zg_out$ZG_phase)
head(zg_out$ZG_cycle)

## ----twd-maxima---------------------------------------------------------------
max_twd_tbl <- twd.maxima(dm_phase_demo, TreeNum = 1, smoothing = 5)
head(max_twd_tbl)

## ----zg-plots-----------------------------------------------------------------
plot(zg_out)
plot(zg_out, temporal = "daily")
plot(zg_out, temporal = "monthly", type = "twd")

## ----zg-more-plots, eval = FALSE----------------------------------------------
# plot(zg_out, temporal = "monthly", type = "abr")
# plot(zg_out, type = "transition", x_axis = "doy", Year = 2017, DOY = c(50, 100))
# plot(zg_out, type = "boxplot", stat = "max.twd", temporal = "daily")

## ----zg-clim-joins------------------------------------------------------------
zg_phase_clim <- dm_join_phase_clim(
  zg_out,
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(zg_phase_clim$ZG_cycle)

zg_point_clim <- dm_join_subdaily_clim(zg_out_sm, clim_sub)
head(zg_point_clim$ZG_phase)

zg_phase_added <- dm_add_climate(
  zg_out,
  clim_hourly,
  scale = "phase",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(zg_phase_added$ZG_cycle)

## ----climate-phase-plots------------------------------------------------------
plot(zg_phase_added, climate_var = "VPD_mean_phase", scale = "cycle", type = "boxplot", temporal = 'daily')

## ----climate-phase-compare, eval = FALSE--------------------------------------
# plot(zg_phase_added, climate_var = "temp_mean_phase", scale = "cycle", type = "timeseries")
# 
# plot(
#   zg_phase_added,
#   climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
#   scale = "cycle",
#   type = "boxplot"
# )
# 
# plot(
#   zg_phase_added,
#   scale = "cycle",
#   type = "cor_heatmap",
#   numeric_vars = c("Duration_h", "max.twd", "ABr.value",
#                    "temp_mean_phase", "VPD_mean_phase", "RH_mean_phase")
# )

## ----epoch--------------------------------------------------------------------
event_tbl <- dm_event_times(zg_out, event = "all")
head(event_tbl)

## ----epoch-extract, eval = FALSE----------------------------------------------
# epoch_extract <- dm_epoch_extract(
#   x = zg_out,
#   clim = clim_hourly,
#   event = "phase_start",
#   phase = "all",
#   response_var = c("temp", "VPD"),
#   before = 24,
#   after = 24,
#   unit = "hours"
# )
# head(epoch_extract)

## ----epoch-test, eval = FALSE-------------------------------------------------
# epoch_test <- dm_epoch_test(
#   x = zg_out,
#   clim = clim_hourly,
#   event = "phase_start",
#   phase = "all",
#   response_var = c("temp", "VPD"),
#   before = 24,
#   after = 24,
#   unit = "hours",
#   n_boot = 99
# )
# 
# epoch_summary <- summary(epoch_test)   # summary.dm_epoch()
# print(epoch_summary)                   # print.summary.dm_epoch()
# plot(epoch_test)

## ----event-climate, eval = FALSE----------------------------------------------
# evt_zg <- dm_event_climate(
#   zg_out,
#   clim_hourly,
#   event = "phase_start",
#   phase = "all",
#   windows = c(0, 3, 6, 12, 24),
#   mean_vars = c("temp", "VPD", "RH"),
#   max_vars  = c("VPD"),
#   sum_vars  = c("prec")
# )
# head(evt_zg)

## ----event-climate-summary, eval = FALSE--------------------------------------
# evt_summary <- dm_event_climate_summary(
#   evt_zg,
#   climate_var = "VPD_mean_prev_6h",
#   group_var = "Phase",
#   by = "month_of_year"
# )
# head(evt_summary)

## ----event-climate-test, eval = FALSE-----------------------------------------
# evt_test <- dm_event_climate_test(
#   evt_zg,
#   climate_var = "VPD_mean_prev_6h",
#   group_var = "Phase",
#   by = "month_of_year",
#   method = "auto"
# )
# head(evt_test$summary)
# head(evt_test$overall_tests)
# head(evt_test$pairwise_tests)

## ----event-climate-plots, eval = FALSE----------------------------------------
# plot(
#   evt_zg,
#   climate_var = "VPD_mean_prev_6h",
#   group_var = "Phase",
#   facet_by = "month_of_year",
#   geom = "both",
#   add_test = TRUE
# )
# 
# evt_maxtwd <- dm_event_climate(
#   zg_out,
#   clim_hourly,
#   event = "max_twd",
#   windows = c(0, 3, 6, 12, 24),
#   mean_vars = c("temp", "VPD", "RH"),
#   max_vars  = c("VPD"),
#   sum_vars  = c("prec")
# )
# 
# plot(
#   evt_maxtwd,
#   climate_var = "VPD_mean_prev_6h",
#   response_var = "max.twd",
#   group_var = "Phase"
# )

## ----legacy-gompertz, eval = FALSE--------------------------------------------
# gomp_legacy <- dm.fit.gompertz(df = dm_hourly, TreeNum = 1, CalYear = 2017, verbose = FALSE)
# head(gomp_legacy)

## ----growth-fit, eval = FALSE-------------------------------------------------
# fit_single <- dm.growth.fit(
#   df = dm_hourly,
#   TreeNum = 1:2,
#   method = "gompertz",
#   year_mode = "yearly",
#   verbose = FALSE
# )
# 
# print(fit_single)                 # print.dm_growth_fit()
# fit_single_summary <- summary(fit_single)   # summary.dm_growth_fit()
# print(fit_single_summary)         # print.summary.dm_growth_fit()

## ----growth-fit-plots, eval = FALSE-------------------------------------------
# plot(fit_single, type = "fit")
# plot(fit_single, type = "season")
# plot(fit_single, type = "residuals")
# plot(fit_single, type = "timing")
# plot(fit_single, type = "overlay")
# plot(fit_single, type = "parameters")

## ----growth-fit-double, eval = FALSE------------------------------------------
# fit_double <- dm.growth.fit.double(
#   df = dm_hourly,
#   TreeNum = 1:2,
#   method = "gompertz",
#   year_mode = "yearly",
#   verbose = FALSE
# )
# print(fit_double)

## ----growth-evaluate, eval = FALSE--------------------------------------------
# growth_eval <- dm.growth.evaluate(
#   df = dm_hourly,
#   TreeNum = 1:2,
#   methods = c("gompertz", "loess", "spline", "double_gompertz"),
#   year_mode = "yearly",
#   verbose = FALSE
# )
# head(growth_eval)

## ----growth-evaluate-plots, eval = FALSE--------------------------------------
# plot(growth_eval, metric = "rmse", type = "boxplot")
# plot(growth_eval, metric = "r2", type = "heatmap")

## ----detrend, eval = FALSE----------------------------------------------------
# detrended_fit <- dm.detrend.fit(fit_single)
# plot(detrended_fit, type = "compare")
# plot(detrended_fit, type = "fit")
# plot(detrended_fit, type = "residual")
# plot(detrended_fit, type = "detrended")
# plot(detrended_fit, type = "boxplot")

## ----mean-detrended, eval = FALSE---------------------------------------------
# mean_det <- mean_detrended.dm(detrended_fit)
# head(mean_det)
# plot(mean_det)

## ----standardize--------------------------------------------------------------
std_nh <- dm_standardize(
  df = dm_hourly,
  season_type = "NH",
  method = "robust_amplitude"
)
head(std_nh$data)
head(std_nh$parameters)

## ----standardize-plot---------------------------------------------------------
plot(std_nh)

## ----mov-cor, eval = FALSE----------------------------------------------------
# mov_out <- mov.cor.dm(
#   df = dm_hourly,
#   Clim = clim_hourly,
#   TreeNum = 1,
#   win_size = 15,
#   cor_method = "pearson",
#   boot = FALSE
# )
# 
# print(mov_out)                # print.mov_cor_dm()
# mov_summary <- summary(mov_out)   # summary.mov_cor_dm()
# print(mov_summary)            # print.summary_mov_cor_dm()

## ----mov-cor-plots, eval = FALSE----------------------------------------------
# plot(mov_out, type = "heatmap")
# plot(mov_out, type = "line", annotate_peak = TRUE)
# plot(mov_out, type = "facet")
# plot(mov_out, type = "peak")
# plot(mov_out, type = "line")
# plot(mov_summary, type = "peak_corr")

## ----mov-cor-boot, eval = FALSE-----------------------------------------------
# mov_boot <- mov.cor.dm(
#   df = dm_hourly,
#   Clim = clim_hourly,
#   TreeNum = 1,
#   win_size = 15,
#   cor_method = "pearson",
#   boot = TRUE,
#   R = 99,
#   set_seed = 1
# )
# plot(mov_boot, type = "line", show_ci = TRUE)

## ----clim-twd, eval = FALSE---------------------------------------------------
# rel_out <- clim.twd(
#   df = dm_hourly,
#   Clim = clim_rain,
#   thresholdClim = "<10",
#   thresholdDays = ">5",
#   showPlot = FALSE
# )

## ----clim-twd-stats, eval = FALSE---------------------------------------------
# clim_stats <- clim.twd.stats(
#   rel_out,
#   response = "full_period_change",
#   group_by = "tree"
# )
# 
# clim_stats_summary <- summary(clim_stats)   # summary.clim_twd_stats()
# print(clim_stats_summary)                   # print.summary.clim_twd_stats()
# plot(clim_stats, type = "trajectory")

## ----clim-twd-test, eval = FALSE----------------------------------------------
# tree_info <- data.frame(
#   tree = c("T2", "T3"),
#   species = c("Pinus", "Pinus"),
#   site = c("Ktm", "Ktm")
# )
# 
# clim_test <- clim.twd.test(
#   rel_out,
#   tree_info = tree_info,
#   parameter = "lag_to_return_zero",
#   compare_by = "species",
#   within = "year"
# )
# 
# clim_test_summary <- summary(clim_test)   # summary.clim_twd_test()
# print(clim_test_summary)                  # print.summary.clim_twd_test()
# plot(clim_test)

## ----network-interp, eval = FALSE---------------------------------------------
# df1 <- dm_hourly
# df1[40:50, "T2"] <- NA
# 
# ref_net <- cbind(dm_hourly, dm_hourly[, 2:3], dm_hourly[, 2:3])
# colnames(ref_net) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")
# 
# net_interp <- network.interpolation(
#   df1,
#   ref_net,
#   niMethod = "proportional",
#   n_boot = 100,
#   return_flags = TRUE,
#   return_pi = TRUE,
#   assess = TRUE,
#   assess_lengths_steps = c(1, 2, 4),
#   correct_gap_jumps = TRUE,
#   jump_threshold = 0.05
# )
# head(net_interp)
# # to get the assessment stats we can use attr() function
# attr(out, "network_validation_summary")
# attr(out, "network_validation_points")
# attr(out, "network_validation_seasonal")

## ----network-interp-plots, eval = FALSE---------------------------------------
# plot(net_interp)
# plot(net_interp, type = "uncertainty")
# plot(net_interp, type = "availability")
# plot(net_interp, type = "summary", summary_metric = "n_gap_jumps_removed")
# plot(net_interp, type = "validation", validation_metric = "MAPE")
# plot(net_interp, type = "coverage")
# plot(net_interp, type = "compare")
# plot(net_interp, type = "seasonal_error", seasonal_metric = "mean_abs_error")

## ----wavelet, eval = FALSE----------------------------------------------------
# wv <- dm_wavelet(
#   x = dm_hourly,
#   TreeNum = 1:2,
#   source = "raw",
#   make_pval = TRUE,
#   verbose = FALSE
# )
# 
# wv_summary <- summary(wv)   # summary.dm_wavelet()
# print(wv_summary)           # print.summary.dm_wavelet()

## ----wavelet-plots, eval = FALSE----------------------------------------------
# plot(wv, type = "series")
# plot(wv, type = "average")
# plot(wv, type = "power")
# plot(wv, series = names(wv$results)[1], type = "power")

## ----wavelet-reconstruct, eval = FALSE----------------------------------------
# rec_extract <- dm_wavelet_reconstruct(
#   wv,
#   mode = "extract",
#   lower_hours = 20,
#   upper_hours = 28,
#   only_sig = TRUE
# )
# 
# rec_remove <- dm_wavelet_reconstruct(
#   wv,
#   mode = "remove",
#   lower_hours = 20,
#   upper_hours = 28,
#   only_sig = TRUE
# )
# 
# rec_extract_summary <- summary(rec_extract)   # summary.dm_wavelet_reconstruct()
# print(rec_extract_summary)                    # print.summary.dm_wavelet_reconstruct()

## ----wavelet-reconstruct-plots, eval = FALSE----------------------------------
# plot(rec_extract)
# plot(rec_remove)

## ----wavelet-coherence, eval = FALSE------------------------------------------
# wc <- dm_wavelet_coherence(
#   dm = dm_hourly,
#   clim = clim_hourly,
#   TreeNum = 1,
#   clim_vars = c("temp", "VPD"),
#   source = "raw",
#   dm_fun = "mean",
#   clim_funs = c(temp = "mean", VPD = "mean"),
#   loess_span = 0,
#   make.pval = TRUE,
#   n.sim = 10,
#   verbose = FALSE
# )

## ----wavelet-coherence-plots, eval = FALSE------------------------------------
# plot(wc)
# plot(wc, type = "cross_power")
# plot(wc, type = "average_coherence")
# plot(wc, type = "phase")
# plot(wc, type = "series")

#' @title Daily statistics of dendrometer data
#'
#' @description
#' Computes daily statistics from high-frequency dendrometer time series.
#' For each day, it extracts the minimum and maximum values, their times
#' of occurrence, daily mean and median, daily amplitude, the signed lag
#' between the time of maximum and minimum, the day-to-day change in the
#' daily maximum, and a daily status indicating whether the day is growing,
#' shrinking, or stable relative to the previous day.
#'
#' @details
#' The function requires a data frame with a time column in the first column
#' and one or more dendrometer series in the following columns. The user
#' selects the series using \code{TreeNum}.
#'
#' The returned object has class \code{"daily_output"}, so it can be plotted
#' directly with \code{plot()}.
#'
#' The column \code{Max_diff} is computed as:
#' \deqn{Max\_diff_t = Max_t - Max_{t-1}}
#'
#' The column \code{Day_status} is derived from \code{Max_diff}:
#' \itemize{
#'   \item \code{"growing"} if \code{Max_diff > 0}
#'   \item \code{"shrinking"} if \code{Max_diff < 0}
#'   \item \code{"stable"} if \code{Max_diff = 0}
#' }
#'
#' The first day has \code{NA} for \code{Max_diff} and \code{Day_status}.
#'
#' @references
#' King G, Fonti P, Nievergelt D, Büntgen U, Frank D (2013)
#' Climatic drivers of hourly to yearly tree radius variations along a 6°C natural warming gradient.
#' \emph{Agricultural and Forest Meteorology} 168:36–46.
#' \doi{10.1016/j.agrformet.2012.08.002}
#'
#' @param df A data frame with the first column containing date-time stamps
#'   convertible to \code{POSIXct} and subsequent columns containing
#'   dendrometer measurements.
#' @param TreeNum Integer. The index of the tree (column) to analyze.
#'   \code{TreeNum = 1} refers to the first dendrometer column after the
#'   time column.
#'
#' @return A tibble of class \code{"daily_output"} containing:
#' \describe{
#'   \item{DATE}{Calendar date.}
#'   \item{Min}{Daily minimum value.}
#'   \item{Time_min}{Time of day of minimum value.}
#'   \item{Max}{Daily maximum value.}
#'   \item{Time_max}{Time of day of maximum value.}
#'   \item{mean}{Daily mean value.}
#'   \item{median}{Daily median value.}
#'   \item{amplitude}{Daily amplitude = Max - Min.}
#'   \item{Time_min_h}{Time of minimum expressed in decimal hours.}
#'   \item{Time_max_h}{Time of maximum expressed in decimal hours.}
#'   \item{lag_h}{Signed difference in hours: Time_max_h - Time_min_h.}
#'   \item{Remarks}{\code{"*"} if \code{Time_max > Time_min}, otherwise \code{""}.}
#'   \item{Max_diff}{Difference between today's maximum and the previous day's maximum.}
#'   \item{Day_status}{\code{"growing"}, \code{"shrinking"}, \code{"stable"}, or \code{NA}.}
#' }
#'
#' @note
#' The object returned by \code{daily.data()} can be plotted using
#' \code{plot()} because it is assigned class \code{"daily_output"}.
#'
#' @importFrom lubridate ymd_hms
#' @importFrom dplyr mutate group_by summarise rename lag case_when if_else %>%
#' @importFrom tibble tibble
#' @importFrom stats median
#'
#' @examples
#' \donttest{
#' data(nepa17)
#' daily_stats <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
#' head(daily_stats, 10)
#' }
#'
#' @export
daily.data <- function(df, TreeNum) {
  if (TreeNum > ncol(df) - 1) {
    stop("The tree number is not valid for provided dataset.")
  }
  TIME <- DATE <- dm <- time <- Time_min <- Time_max <- Time_max_h <- Time_min_h <- Max <- NULL


  tn <- TreeNum + 1

  if (!inherits(df[[1]], "Date") && !inherits(df[[1]], "POSIXct")) {
    df[[1]] <- lubridate::ymd_hms(df[[1]])
  }

  time_to_hours <- function(x) {
    x <- as.character(x)
    bad <- is.na(x) | x == ""
    hh <- suppressWarnings(as.numeric(substr(x, 1, 2)))
    mm <- suppressWarnings(as.numeric(substr(x, 4, 5)))
    ss <- suppressWarnings(as.numeric(substr(x, 7, 8)))
    out <- hh + mm / 60 + ss / 3600
    out[bad] <- NA_real_
    out
  }

  dat <- tibble::tibble(
    TIME = as.POSIXct(df[[1]]),
    dm = df[[tn]]
  ) %>%
    dplyr::mutate(
      DATE = as.Date(TIME),
      time = format(TIME, "%H:%M:%S")
    )

  result <- dat %>%
    dplyr::group_by(DATE) %>%
    dplyr::summarise(
      Min = if (all(is.na(dm))) NA_real_ else min(dm, na.rm = TRUE),
      Time_min = if (all(is.na(dm))) {
        NA_character_
      } else {
        time[which.min(replace(dm, is.na(dm), Inf))][1]
      },
      Max = if (all(is.na(dm))) NA_real_ else max(dm, na.rm = TRUE),
      Time_max = if (all(is.na(dm))) {
        NA_character_
      } else {
        time[which.max(replace(dm, is.na(dm), -Inf))][1]
      },
      mean = if (all(is.na(dm))) NA_real_ else mean(dm, na.rm = TRUE),
      median = if (all(is.na(dm))) NA_real_ else stats::median(dm, na.rm = TRUE),
      amplitude = if (all(is.na(dm))) NA_real_ else max(dm, na.rm = TRUE) - min(dm, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::mutate(
      Time_min_h = time_to_hours(Time_min),
      Time_max_h = time_to_hours(Time_max),
      lag_h = Time_max_h - Time_min_h,
      Remarks = dplyr::if_else(
        !is.na(Time_max_h) & !is.na(Time_min_h) & Time_max_h > Time_min_h,
        "*", ""
      ),
      Max_diff = Max - dplyr::lag(Max),
      Day_status = dplyr::case_when(
        is.na(Max_diff) ~ NA_character_,
        Max_diff > 0 ~ "growing",
        Max_diff < 0 ~ "shrinking",
        TRUE ~ "stable"
      )
    )

  attr(result, "tree_num") <- TreeNum
  attr(result, "tree_col") <- names(df)[tn]

  class(result) <- c("daily_output", class(result))
  result
}

#' Wavelet analysis of dendrometer series
#'
#' @description
#' Performs continuous wavelet analysis on dendrometer time series.
#'
#' The function can work with:
#' \itemize{
#'   \item raw dendrometer series from a data frame,
#'   \item detrended dendrometer series from \code{dm.detrend.fit()} output,
#'   \item first differences of either raw or detrended series.
#' }
#'
#' It automatically:
#' \itemize{
#'   \item identifies the temporal resolution of the input series,
#'   \item regularizes the series to a complete time grid,
#'   \item optionally interpolates missing values,
#'   \item performs wavelet analysis separately for each selected tree/series,
#'   \item converts wavelet periods to hours for summary and plotting.
#' }
#'
#' @param x Input data. Either:
#'   \describe{
#'     \item{data.frame}{First column must be time, remaining selected numeric
#'       columns are dendrometer series.}
#'     \item{dm_detrended object}{Uses \code{x$detrended_data} as input.}
#'   }
#' @param TreeNum Either \code{"all"} to use all available dendrometer series,
#'   a numeric vector selecting series by position, or a character vector of
#'   series names.
#' @param source Which series to analyze. One of:
#'   \describe{
#'     \item{`"auto"`}{Uses raw series for a data frame input and detrended
#'       series for a \code{dm_detrended} object.}
#'     \item{`"raw"`}{Use raw dendrometer series from a data frame input.}
#'     \item{`"detrended"`}{Use detrended series from a \code{dm_detrended} object.}
#'     \item{`"first_diff"`}{Use first differences of the available input series.}
#'   }
#' @param detrended_col For \code{dm_detrended} input, which table to use:
#'   \code{"detrended_data"}.
#' @param na_action How to handle missing values after completing the regular time
#'   grid. One of:
#'   \describe{
#'     \item{`"interpolate"`}{Linearly interpolate internal gaps and carry ends
#'       forward/backward.}
#'     \item{`"fail"`}{Stop if missing values are found.}
#'   }
#' @param loess_span Smoothing span passed to \code{WaveletComp::analyze.wavelet()}.
#' @param dj Frequency resolution parameter passed to
#'   \code{WaveletComp::analyze.wavelet()}.
#' @param lowerPeriod Optional lower period bound in native time units of the
#'   detected input resolution. If \code{NULL}, the default from
#'   \code{WaveletComp::analyze.wavelet()} is used.
#' @param upperPeriod Optional upper period bound in native time units of the
#'   detected input resolution. If \code{NULL}, the default from
#'   \code{WaveletComp::analyze.wavelet()} is used.
#' @param make_pval Logical; if \code{TRUE}, compute Monte Carlo significance.
#' @param n_sim Number of simulations used when \code{make_pval = TRUE}.
#' @param verbose Logical; if \code{TRUE}, prints a completion message.
#'
#' @return
#' An object of class \code{"dm_wavelet"} with elements:
#' \describe{
#'   \item{call}{The matched function call.}
#'   \item{input_type}{Either \code{"raw"} or \code{"dm_detrended"}.}
#'   \item{source}{Series source actually used.}
#'   \item{series}{Character vector of analyzed series names.}
#'   \item{time_unit}{Detected time unit of the input series.}
#'   \item{dt}{Detected time step in native units.}
#'   \item{dt_hours}{Detected time step expressed in hours.}
#'   \item{resolution_seconds}{Detected time step in seconds.}
#'   \item{data_used}{Regularized time series used for analysis.}
#'   \item{results}{Named list of wavelet results, one per series. Each entry
#'     contains the original \code{WaveletComp} object plus time and
#'     period-in-hours information.}
#'   \item{settings}{Analysis settings.}
#' }
#'
#' @importFrom dplyr left_join arrange %>%
#' @importFrom tibble as_tibble tibble
#' @importFrom lubridate ymd_hms parse_date_time
#' @importFrom zoo na.approx na.locf
#' @export
dm_wavelet <- function(x,
                       TreeNum = "all",
                       source = c("auto", "raw", "detrended", "first_diff"),
                       detrended_col = c("detrended_data"),
                       na_action = c("interpolate", "fail"),
                       loess_span = 0.75,
                       dj = 1 / 20,
                       lowerPeriod = NULL,
                       upperPeriod = NULL,
                       make_pval = TRUE,
                       n_sim = 10,
                       verbose = TRUE) {

  source <- match.arg(source)
  detrended_col <- match.arg(detrended_col)
  na_action <- match.arg(na_action)

  if (!requireNamespace("WaveletComp", quietly = TRUE)) {
    stop("Package 'WaveletComp' is required for dm_wavelet().")
  }

  cl <- match.call()

  dmw_parse_time <- function(tt) {
    if (inherits(tt, "POSIXct") || inherits(tt, "POSIXt")) return(as.POSIXct(tt))
    if (inherits(tt, "Date")) return(as.POSIXct(tt))
    out <- suppressWarnings(lubridate::ymd_hms(tt, quiet = TRUE))
    if (all(is.na(out))) {
      out <- suppressWarnings(
        lubridate::parse_date_time(
          tt,
          orders = c("ymd HMS", "ymd HM", "ymd",
                     "dmy HMS", "dmy HM", "dmy",
                     "mdy HMS", "mdy HM", "mdy"),
          quiet = TRUE
        )
      )
    }
    out
  }

  dmw_select_cols <- function(dat, TreeNum, meta_cols = character(0)) {
    series_cols <- setdiff(names(dat), meta_cols)
    series_cols <- series_cols[vapply(dat[series_cols], is.numeric, logical(1))]

    if (length(series_cols) == 0) {
      stop("No numeric dendrometer series available.")
    }

    if (is.character(TreeNum) && length(TreeNum) == 1 && tolower(TreeNum) == "all") {
      keep <- series_cols
    } else if (is.numeric(TreeNum)) {
      idx <- as.integer(TreeNum)
      if (any(is.na(idx)) || any(idx < 1) || any(idx > length(series_cols))) {
        stop("'TreeNum' numeric values are out of range.")
      }
      keep <- series_cols[idx]
    } else if (is.character(TreeNum)) {
      miss <- setdiff(TreeNum, series_cols)
      if (length(miss) > 0) {
        warning("These requested series were not found and were ignored: ",
                paste(miss, collapse = ", "))
      }
      keep <- intersect(TreeNum, series_cols)
      if (length(keep) == 0) {
        stop("None of the requested TreeNum names were found.")
      }
    } else {
      stop("'TreeNum' must be 'all', numeric indices, or character names.")
    }

    keep
  }

  dmw_infer_dt <- function(time_vec) {
    dsec <- as.numeric(diff(time_vec), units = "secs")
    dsec <- dsec[is.finite(dsec) & dsec > 0]

    if (length(dsec) == 0) {
      stop("Could not infer time step from TIME column.")
    }

    med <- stats::median(dsec, na.rm = TRUE)

    if (med >= 86400 && abs(med / 86400 - round(med / 86400)) < 1e-6) {
      return(list(dt = med / 86400, unit = "days", sec = med, hours = med / 3600))
    }
    if (med >= 3600 && abs(med / 3600 - round(med / 3600)) < 1e-6) {
      return(list(dt = med / 3600, unit = "hours", sec = med, hours = med / 3600))
    }
    if (med >= 60 && abs(med / 60 - round(med / 60)) < 1e-6) {
      return(list(dt = med / 60, unit = "mins", sec = med, hours = med / 3600))
    }

    list(dt = med, unit = "secs", sec = med, hours = med / 3600)
  }

  dmw_period_to_hours <- function(period, time_unit) {
    if (time_unit == "days") return(period * 24)
    if (time_unit == "hours") return(period)
    if (time_unit == "mins") return(period / 60)
    if (time_unit == "secs") return(period / 3600)
    period
  }

  dmw_complete_regular <- function(dat, dt_sec) {
    tz_use <- attr(dat$TIME, "tzone")
    if (is.null(tz_use) || length(tz_use) == 0) tz_use <- "UTC"

    full_time <- seq(
      from = min(dat$TIME, na.rm = TRUE),
      to = max(dat$TIME, na.rm = TRUE),
      by = dt_sec
    )

    tibble::tibble(TIME = as.POSIXct(full_time, origin = "1970-01-01", tz = tz_use)) %>%
      dplyr::left_join(dat, by = "TIME") %>%
      dplyr::arrange(.data$TIME)
  }

  dmw_fill_na <- function(z) {
    z2 <- zoo::na.approx(z, na.rm = FALSE)
    z2 <- zoo::na.locf(z2, na.rm = FALSE)
    z2 <- zoo::na.locf(z2, fromLast = TRUE, na.rm = FALSE)
    z2
  }

  if (inherits(x, "dm_detrended")) {
    input_type <- "dm_detrended"
    if (source == "raw") {
      stop("source = 'raw' is not valid for a dm_detrended object.")
    }
    dat0 <- tibble::as_tibble(x[[detrended_col]])
    meta_cols <- intersect(
      c("TIME", "doy", "season_label", "season_start", "season_end", "season_day"),
      names(dat0)
    )
    keep_cols <- dmw_select_cols(dat0, TreeNum = TreeNum, meta_cols = meta_cols)
    dat <- dat0[, c("TIME", keep_cols), drop = FALSE]
    source_used <- if (source == "auto") "detrended" else source
  } else if (is.data.frame(x)) {
    input_type <- "raw"
    if (source == "detrended") {
      stop("source = 'detrended' requires a dm_detrended object.")
    }
    dat0 <- tibble::as_tibble(x)
    names(dat0)[1] <- "TIME"
    keep_cols <- dmw_select_cols(dat0, TreeNum = TreeNum, meta_cols = "TIME")
    dat <- dat0[, c("TIME", keep_cols), drop = FALSE]
    source_used <- if (source == "auto") "raw" else source
  } else {
    stop("'x' must be either a data.frame or an object of class 'dm_detrended'.")
  }

  dat$TIME <- dmw_parse_time(dat$TIME)
  if (any(is.na(dat$TIME))) {
    stop("Some timestamps in the first column could not be parsed.")
  }

  dat <- dat[order(dat$TIME), , drop = FALSE]

  dt_info <- dmw_infer_dt(dat$TIME)
  dat_reg <- dmw_complete_regular(dat, dt_sec = dt_info$sec)

  series_names <- setdiff(names(dat_reg), "TIME")

  for (ss in series_names) {
    if (na_action == "interpolate") {
      dat_reg[[ss]] <- dmw_fill_na(dat_reg[[ss]])
    } else {
      if (any(is.na(dat_reg[[ss]]))) {
        stop("Missing values found after regularization. Use na_action = 'interpolate' or pre-clean the series.")
      }
    }
  }

  if (source_used == "first_diff") {
    for (ss in series_names) {
      dat_reg[[ss]] <- c(NA_real_, diff(dat_reg[[ss]]))
    }
    dat_reg <- dat_reg[-1, , drop = FALSE]
  }

  for (ss in series_names) {
    if (any(is.na(dat_reg[[ss]]))) {
      stop("Series '", ss, "' still contains missing values after preprocessing.")
    }
    if (sum(is.finite(dat_reg[[ss]])) < 8) {
      stop("Series '", ss, "' is too short for wavelet analysis after preprocessing.")
    }
  }

  res_list <- vector("list", length(series_names))
  names(res_list) <- series_names

  for (ss in series_names) {
    tmp <- data.frame(series = as.numeric(dat_reg[[ss]]))

    wave_args <- list(
      my.data = tmp,
      my.series = "series",
      loess.span = loess_span,
      dt = dt_info$dt,
      dj = dj,
      make.pval = make_pval,
      n.sim = n_sim
    )

    if (!is.null(lowerPeriod)) wave_args$lowerPeriod <- lowerPeriod
    if (!is.null(upperPeriod)) wave_args$upperPeriod <- upperPeriod

    wt <- do.call(WaveletComp::analyze.wavelet, wave_args)

    period_hours <- dmw_period_to_hours(wt$Period, dt_info$unit)

    res_list[[ss]] <- list(
      wavelet = wt,
      time = dat_reg$TIME,
      series = dat_reg[[ss]],
      period_native = wt$Period,
      period_hours = period_hours,
      avg_power = if (!is.null(wt$Power.avg)) {
        as.numeric(wt$Power.avg)
      } else {
        rowMeans(wt$Power, na.rm = TRUE)
      }
    )
  }

  out <- list(
    call = cl,
    input_type = input_type,
    source = source_used,
    series = series_names,
    time_unit = dt_info$unit,
    dt = dt_info$dt,
    dt_hours = dt_info$hours,
    resolution_seconds = dt_info$sec,
    data_used = dat_reg,
    results = res_list,
    settings = list(
      TreeNum = TreeNum,
      loess_span = loess_span,
      dj = dj,
      lowerPeriod = lowerPeriod,
      upperPeriod = upperPeriod,
      make_pval = make_pval,
      n_sim = n_sim,
      na_action = na_action
    )
  )

  class(out) <- "dm_wavelet"

  if (isTRUE(verbose)) {
    message(
      "dm_wavelet completed for ", length(series_names),
      " series. Detected resolution: ", dt_info$dt, " ", dt_info$unit,
      " (", round(dt_info$hours, 4), " hours)."
    )
  }

  out
}


#' Summarize a dm_wavelet object
#'
#' @description
#' Summarizes wavelet-analysis results produced by \code{dm_wavelet()}.
#'
#' For each analyzed series, the summary reports:
#' \itemize{
#'   \item the dominant period in hours,
#'   \item the corresponding maximum average wavelet power,
#'   \item the analyzed period range in hours,
#'   \item the detected temporal resolution.
#' }
#'
#' @param object An object of class \code{"dm_wavelet"}.
#' @param top_n Integer. Number of strongest periods to report per series.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An object of class \code{"summary.dm_wavelet"}.
#' @method summary dm_wavelet
#' @export
summary.dm_wavelet <- function(object, top_n = 3, ...) {

  if (!inherits(object, "dm_wavelet")) {
    stop("'object' must be an object of class 'dm_wavelet'.")
  }

  if (!is.numeric(top_n) || length(top_n) != 1 || is.na(top_n) || top_n < 1) {
    stop("'top_n' must be a positive integer.")
  }
  top_n <- as.integer(top_n)

  series_rows <- vector("list", length(object$results))
  top_rows <- vector("list", length(object$results))
  nm <- names(object$results)

  for (i in seq_along(object$results)) {
    ss <- nm[i]
    res <- object$results[[i]]

    per_h <- as.numeric(res$period_hours)
    avg_pow <- as.numeric(res$avg_power)

    ok <- is.finite(per_h) & is.finite(avg_pow)

    if (sum(ok) == 0) {
      series_rows[[i]] <- tibble::tibble(
        series = ss,
        n_obs = length(res$series),
        dominant_period_hours = NA_real_,
        max_average_power = NA_real_,
        min_period_hours = NA_real_,
        max_period_hours = NA_real_,
        dt = object$dt,
        dt_hours = object$dt_hours,
        time_unit = object$time_unit
      )

      top_rows[[i]] <- tibble::tibble(
        series = ss,
        rank = integer(0),
        period_hours = numeric(0),
        average_power = numeric(0)
      )
      next
    }

    per_h <- per_h[ok]
    avg_pow <- avg_pow[ok]
    ord <- order(avg_pow, decreasing = TRUE)
    ord_top <- head(ord, top_n)

    series_rows[[i]] <- tibble::tibble(
      series = ss,
      n_obs = length(res$series),
      dominant_period_hours = per_h[ord[1]],
      max_average_power = avg_pow[ord[1]],
      min_period_hours = min(per_h, na.rm = TRUE),
      max_period_hours = max(per_h, na.rm = TRUE),
      dt = object$dt,
      dt_hours = object$dt_hours,
      time_unit = object$time_unit
    )

    top_rows[[i]] <- tibble::tibble(
      series = ss,
      rank = seq_along(ord_top),
      period_hours = per_h[ord_top],
      average_power = avg_pow[ord_top]
    )
  }

  out <- list(
    overview = tibble::tibble(
      input_type = object$input_type,
      source = object$source,
      n_series = length(object$series),
      dt = object$dt,
      dt_hours = object$dt_hours,
      time_unit = object$time_unit
    ),
    series_summary = dplyr::bind_rows(series_rows),
    top_periods = dplyr::bind_rows(top_rows),
    settings = object$settings
  )

  class(out) <- "summary.dm_wavelet"
  out
}


#' @method print summary.dm_wavelet
#' @export
print.summary.dm_wavelet <- function(x, ...) {
  cat("dm_wavelet summary\n")
  cat("------------------\n")
  print(x$overview)

  cat("\nSeries summary:\n")
  print(x$series_summary)

  cat("\nTop periods (hours):\n")
  print(x$top_periods)

  invisible(x)
}


#' @title Plot method for wavelet analysis output
#'
#' @description
#' S3 plotting method for objects returned by \code{dm_wavelet()}.
#'
#' The function provides three main visualization types:
#' \itemize{
#'   \item \code{"series"}: plots the analyzed input dendrometer series over time,
#'   \item \code{"average"}: plots the average wavelet power spectrum against period,
#'   \item \code{"power"}: plots the full wavelet power spectrum as a time-period image.
#' }
#'
#' Periods are shown in \strong{hours}, regardless of the original temporal
#' resolution of the input series. For power plots, the x-axis shows the actual
#' time series timestamps and the y-axis shows the wavelet period in hours.
#'
#' @param x An object of class \code{"dm_wavelet"} returned by
#'   \code{dm_wavelet()}.
#' @param y Unused.
#' @param series Optional character vector giving one or more series names to
#'   plot. If \code{NULL}, the first available series is used.
#' @param type Character string specifying the plot type. One of:
#'   \describe{
#'     \item{`"power"`}{Wavelet power spectrum as a raster image.}
#'     \item{`"average"`}{Average wavelet power spectrum across the full time series.}
#'     \item{`"series"`}{Original analyzed series over time.}
#'   }
#' @param facet Logical. If \code{TRUE} and more than one series is selected,
#'   panels are faceted by series.
#' @param log_period Logical. If \code{TRUE}, the period axis is shown on a
#'   log10 scale where applicable.
#' @param log_power Logical. If \code{TRUE} and \code{type = "power"}, the
#'   plotted wavelet power is transformed using \code{log10(power + eps)} to
#'   improve contrast.
#' @param clip_quantile Optional numeric vector of length 2 giving lower and
#'   upper quantiles used to clip wavelet power before plotting, for example
#'   \code{c(0.01, 0.99)}. This is useful when a few extreme values dominate the
#'   colour scale. Use \code{NULL} to disable clipping.
#' @param show_sig Logical. If \code{TRUE}, overlays significance information
#'   when available. For \code{type = "average"}, significant periods are marked
#'   as points. For \code{type = "power"}, significant regions are shown as
#'   contour lines.
#' @param show_coi Logical. If \code{TRUE} and \code{type = "power"}, the cone of
#'   influence (COI) is added as a shaded overlay.
#' @param coi_fill Fill colour used for the cone of influence shading.
#' @param coi_alpha Numeric transparency of the cone of influence shading.
#' @param siglvl Numeric significance threshold between 0 and 1. Default is
#'   \code{0.05}.
#' @param sig_color Colour used for significance overlays.
#' @param sig_size Line width or point size used for significance overlays.
#' @param main Optional plot title. If \code{NULL}, a default title is used.
#' @param ... Further arguments passed to or from other methods. Currently unused.
#'
#' @details
#' For \code{type = "power"}, the function uses the wavelet power matrix stored
#' in the \code{"dm_wavelet"} object and converts the Fourier periods to hours.
#' The power plot may also show:
#' \itemize{
#'   \item significance contours, when p-values are available,
#'   \item the cone of influence (COI), when COI information is available.
#' }
#'
#' For \code{type = "average"}, the function plots the average wavelet power
#' spectrum and may optionally indicate significant periods when average-spectrum
#' p-values are available.
#'
#' For \code{type = "series"}, the original analyzed series are plotted without
#' any wavelet transformation.
#'
#' @return
#' A \code{ggplot2} object.
#'
#' @seealso
#' \code{\link{dm_wavelet}}, \code{\link[ggplot2]{ggplot}}
#'
#' @examples
#' \donttest{
#' wv <- dm_wavelet(
#'   x = gf_nepa17,
#'   TreeNum = 1:2,
#'   source = "raw",
#'   make_pval = TRUE,
#'   verbose = FALSE
#' )
#'
#' # original series
#' plot(wv, type = "series")
#'
#' # average wavelet power
#' plot(wv, type = "average")
#'
#' # full power spectrum
#' plot(wv, type = "power")
#'
#' # one selected series
#' plot(wv, series = names(wv$results)[1], type = "power")
#'
#' # stronger contrast in the power plot
#' plot(
#'   wv,
#'   type = "power",
#'   log_power = TRUE,
#'   clip_quantile = c(0.05, 0.95)
#' )
#' }
#'
#' @method plot dm_wavelet
#' @importFrom dplyr bind_rows
#' @importFrom tibble tibble
#' @importFrom ggplot2 ggplot aes geom_raster geom_line geom_point geom_contour
#'   geom_polygon facet_wrap theme_bw theme element_text labs scale_y_log10
#'   scale_x_log10 scale_fill_viridis_c coord_cartesian
#' @export
plot.dm_wavelet <- function(x,
                            y = NULL,
                            series = NULL,
                            type = c("power", "average", "series"),
                            facet = TRUE,
                            log_period = TRUE,
                            log_power = TRUE,
                            clip_quantile = c(0.01, 0.99),
                            show_sig = TRUE,
                            show_coi = TRUE,
                            coi_fill = "white",
                            coi_alpha = 0.45,
                            siglvl = 0.05,
                            sig_color = "black",
                            sig_size = 0.4,
                            main = NULL,
                            ...) {
  TIME <- value <- period_hours <- average_power <- power_plot <- pval <- NULL

  if (!inherits(x, "dm_wavelet")) {
    stop("'x' must be an object of class 'dm_wavelet'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_wavelet().")
  }

  type <- match.arg(type)

  if (is.null(series)) {
    series <- x$series[1]
  }

  miss <- setdiff(series, x$series)
  if (length(miss) > 0) {
    stop("These requested series were not found: ", paste(miss, collapse = ", "))
  }

  if (!is.numeric(siglvl) || length(siglvl) != 1 || is.na(siglvl) ||
      siglvl <= 0 || siglvl >= 1) {
    stop("'siglvl' must be a single number between 0 and 1.")
  }

  dmw_period_to_hours <- function(period, time_unit) {
    if (time_unit == "days") return(period * 24)
    if (time_unit == "hours") return(period)
    if (time_unit == "mins") return(period / 60)
    if (time_unit == "secs") return(period / 3600)
    period
  }

  if (type == "series") {
    df_list <- lapply(series, function(ss) {
      res <- x$results[[ss]]
      tibble::tibble(
        TIME = res$time,
        value = as.numeric(res$series),
        series = ss
      )
    })

    dat <- dplyr::bind_rows(df_list)

    p <- ggplot2::ggplot(dat, ggplot2::aes(x = TIME, y = value, colour = series)) +
      ggplot2::geom_line() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (length(series) > 1) "right" else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Series value",
        colour = "Series",
        title = if (is.null(main)) "Input series used for wavelet analysis" else main
      )

    if (length(series) > 1 && isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  if (type == "average") {
    df_list <- lapply(series, function(ss) {
      res <- x$results[[ss]]
      wt <- res$wavelet

      out <- tibble::tibble(
        period_hours = as.numeric(res$period_hours),
        average_power = as.numeric(res$avg_power),
        series = ss
      )

      if (!is.null(wt$Power.avg.pval)) {
        pavg <- as.numeric(wt$Power.avg.pval)
        n <- min(length(pavg), nrow(out))
        out$avg_pval <- NA_real_
        out$avg_pval[seq_len(n)] <- pavg[seq_len(n)]
      } else {
        out$avg_pval <- NA_real_
      }

      out
    })

    dat <- dplyr::bind_rows(df_list)

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = period_hours, y = average_power, colour = series)
    ) +
      ggplot2::geom_line() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (length(series) > 1) "right" else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Period (hours)",
        y = "Average wavelet power",
        colour = "Series",
        title = if (is.null(main)) "Average wavelet power spectrum" else main
      )

    if (isTRUE(log_period) && all(dat$period_hours > 0, na.rm = TRUE)) {
      p <- p + ggplot2::scale_x_log10()
    }

    if (isTRUE(show_sig) && "avg_pval" %in% names(dat) && any(is.finite(dat$avg_pval))) {
      sig_dat <- dat[is.finite(dat$avg_pval) & dat$avg_pval <= siglvl, , drop = FALSE]
      if (nrow(sig_dat) > 0) {
        p <- p + ggplot2::geom_point(
          data = sig_dat,
          mapping = ggplot2::aes(x = period_hours, y = average_power),
          inherit.aes = FALSE,
          colour = sig_color,
          size = max(1, sig_size * 2)
        )
      }
    }

    if (length(series) > 1 && isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  if (type == "power") {
    df_list <- lapply(series, function(ss) {
      res <- x$results[[ss]]
      wt <- res$wavelet
      pow <- as.matrix(wt$Power)

      n_p <- min(nrow(pow), length(res$period_hours))
      n_t <- min(ncol(pow), length(res$time))

      pow <- pow[seq_len(n_p), seq_len(n_t), drop = FALSE]
      per <- as.numeric(res$period_hours[seq_len(n_p)])
      tt <- res$time[seq_len(n_t)]

      df <- expand.grid(
        period_hours = per,
        TIME = tt,
        KEEP.OUT.ATTRS = FALSE,
        stringsAsFactors = FALSE
      )
      df$power <- c(pow)
      df$series <- ss

      if (!is.null(wt$Power.pval)) {
        pmat <- as.matrix(wt$Power.pval)
        pmat <- pmat[seq_len(n_p), seq_len(n_t), drop = FALSE]
        df$pval <- c(pmat)
      } else {
        df$pval <- NA_real_
      }

      df
    })

    dat <- dplyr::bind_rows(df_list)

    # keep only valid positive periods
    dat <- dat[is.finite(dat$period_hours) & dat$period_hours > 0, , drop = FALSE]

    # fixed y range from actual wavelet periods only
    y_rng <- range(dat$period_hours, na.rm = TRUE)

    if (!is.null(clip_quantile)) {
      if (!is.numeric(clip_quantile) || length(clip_quantile) != 2 ||
          any(is.na(clip_quantile)) ||
          clip_quantile[1] < 0 || clip_quantile[2] > 1 ||
          clip_quantile[1] >= clip_quantile[2]) {
        stop("'clip_quantile' must be NULL or a numeric vector like c(0.01, 0.99).")
      }

      qv <- stats::quantile(dat$power, probs = clip_quantile, na.rm = TRUE, names = FALSE)
      dat$power_plot <- pmin(pmax(dat$power, qv[1]), qv[2])
    } else {
      dat$power_plot <- dat$power
    }

    if (isTRUE(log_power)) {
      eps <- max(min(dat$power_plot[dat$power_plot > 0], na.rm = TRUE) / 10, 1e-12)
      if (!is.finite(eps)) eps <- 1e-12
      dat$power_plot <- log10(dat$power_plot + eps)
      fill_lab <- "log10(Power)"
    } else {
      fill_lab <- "Power"
    }

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = TIME, y = period_hours, fill = power_plot)
    ) +
      ggplot2::geom_raster() +
      ggplot2::scale_fill_viridis_c(option = "D", na.value = "grey85") +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "right",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Period (hours)",
        fill = fill_lab,
        title = if (is.null(main)) "Wavelet power spectrum" else main
      )

    if (isTRUE(show_coi)) {
      coi_list <- lapply(series, function(ss) {
        res <- x$results[[ss]]
        wt <- res$wavelet

        if (is.null(wt$coi.1) || is.null(wt$coi.2) ||
            is.null(wt$axis.1) || is.null(wt$axis.2)) {
          return(NULL)
        }

        tz_use <- attr(res$time, "tzone")
        if (is.null(tz_use) || length(tz_use) == 0) tz_use <- "UTC"

        x_num <- stats::approx(
          x = as.numeric(wt$axis.1),
          y = as.numeric(res$time),
          xout = as.numeric(wt$coi.1),
          rule = 2
        )$y

        period_native <- 2^as.numeric(wt$coi.2)
        period_hours <- dmw_period_to_hours(period_native, x$time_unit)

        # clip COI to actual plotted y range
        period_hours <- pmin(pmax(period_hours, y_rng[1]), y_rng[2])

        tibble::tibble(
          TIME = as.POSIXct(x_num, origin = "1970-01-01", tz = tz_use),
          period_hours = period_hours,
          series = ss
        )
      })

      coi_dat <- dplyr::bind_rows(coi_list)

      if (nrow(coi_dat) > 0) {
        p <- p + ggplot2::geom_polygon(
          data = coi_dat,
          mapping = ggplot2::aes(x = TIME, y = period_hours, group = series),
          inherit.aes = FALSE,
          fill = coi_fill,
          alpha = coi_alpha,
          colour = NA
        )
      }
    }

    if (isTRUE(show_sig) && "pval" %in% names(dat) && any(is.finite(dat$pval))) {
      sig_dat <- dat[is.finite(dat$pval), , drop = FALSE]
      if (nrow(sig_dat) > 0) {
        p <- p + ggplot2::geom_contour(
          data = sig_dat,
          mapping = ggplot2::aes(
            x = TIME,
            y = period_hours,
            z = pval
          ),
          breaks = siglvl,
          colour = sig_color,
          linewidth = sig_size,
          inherit.aes = FALSE
        )
      }
    }

    # force the y axis to the real period range
    if (isTRUE(log_period) && all(y_rng > 0)) {
      p <- p + ggplot2::scale_y_log10(limits = y_rng)
    } else {
      p <- p + ggplot2::coord_cartesian(ylim = y_rng, expand = FALSE)
    }

    if (length(series) > 1 && isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_x", ncol = 1)
    }

    return(p)
  }

  stop("Unknown plot type.")
}

#' @title Join daily climate summaries to daily dendrometer statistics
#'
#' @description
#' Joins daily climate summaries to the output of \code{daily.data()} by
#' calendar date.
#'
#' The climate input can be:
#' \itemize{
#'   \item the output of \code{dm_daily_clim()}
#'   \item the output of \code{read.climate()}
#'   \item a raw climate data frame
#'   \item a valid climate file path readable by \code{read.climate()}
#' }
#'
#' If the supplied climate input is not already a daily climate table, it will
#' first be converted to daily summaries with \code{dm_daily_clim()} using
#' default daily means for numeric variables.
#'
#' @param daily_obj Output of \code{daily.data()} with class \code{"daily_output"}.
#' @param clim_daily Climate input. This can be a daily climate table, a
#'   standardized climate object returned by \code{read.climate()}, a raw
#'   climate data frame, or a valid climate file path.
#'
#' @return
#' The \code{daily.data()} output with climate columns appended. The returned
#' object has class \code{c("daily_output_clim", "daily_output", ...)}.
#'
#' @examples
#' \donttest{
#' data(nepa17)
#' data(ktm_clim_hourly)
#'
#' dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
#'
#' clim_std <- read.climate(
#'   ktm_clim_hourly,
#'   time_col = "TIME",
#'   vars = c("temp", "prec", "VPD", "RH"),
#'   verbose = FALSE
#' )
#'
#' clim_day <- dm_daily_clim(
#'   clim_std,
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' dd_clim <- dm_join_daily_clim(dd, clim_day)
#' head(dd_clim)
#' }
#'
#' @importFrom dplyr left_join
#' @importFrom tibble as_tibble
#' @export
dm_join_daily_clim <- function(daily_obj, clim_daily) {

  if (!inherits(daily_obj, "daily_output")) {
    stop("'daily_obj' must be an object returned by daily.data().")
  }

  # prepare climate input
  clim <- NULL

  if (inherits(clim_daily, "daily_clim")) {
    clim <- tibble::as_tibble(clim_daily)

  } else if (inherits(clim_daily, "dm_clim") ||
             is.data.frame(clim_daily) ||
             (is.character(clim_daily) && length(clim_daily) == 1 && file.exists(clim_daily))) {

    # standardize raw climate first if needed
    if (!inherits(clim_daily, "dm_clim")) {
      clim_daily <- read.climate(clim_daily, verbose = FALSE)
    }

    # if there is no DATE column, create daily summaries
    clim <- dm_daily_clim(clim_daily)

  } else {
    stop(
      "'clim_daily' must be one of: ",
      "output of dm_daily_clim(), output of read.climate(), ",
      "a raw climate data frame, or a valid climate file path."
    )
  }

  if (!"DATE" %in% names(clim)) {
    if ("TIME" %in% names(clim)) {
      clim$DATE <- as.Date(clim$TIME)
    } else {
      stop("Climate input must contain a 'DATE' column or a 'TIME' column.")
    }
  }

  # preserve useful attributes from daily_obj
  old_class <- class(daily_obj)
  old_attrs <- attributes(daily_obj)
  out <- dplyr::left_join(tibble::as_tibble(daily_obj), tibble::as_tibble(clim), by = "DATE")

  # restore selected attributes
  if (!is.null(old_attrs$tree_num)) attr(out, "tree_num") <- old_attrs$tree_num
  if (!is.null(old_attrs$tree_col)) attr(out, "tree_col") <- old_attrs$tree_col

  class(out) <- unique(c("daily_output_clim", old_class))
  out
}

# =========================================================
# Internal helpers
# =========================================================

.dm_parse_time_candidate <- function(x, tz = "UTC") {
  if (inherits(x, "POSIXct")) return(as.POSIXct(x, tz = tz))
  if (inherits(x, "Date")) return(as.POSIXct(x, tz = tz))

  if (is.factor(x)) x <- as.character(x)

  # numeric: try Excel serial first, then Unix seconds, then Unix milliseconds
  if (is.numeric(x)) {
    out <- rep(as.POSIXct(NA, origin = "1970-01-01", tz = tz), length(x))
    ok <- is.finite(x)

    if (any(ok)) {
      xr <- range(x[ok], na.rm = TRUE)

      # Excel serial dates (roughly 1954-2149)
      if (xr[1] > 20000 && xr[2] < 100000) {
        out[ok] <- as.POSIXct(x[ok] * 86400, origin = "1899-12-30", tz = tz)
        return(out)
      }

      # Unix milliseconds
      if (xr[1] > 1e12 && xr[2] < 5e13) {
        out[ok] <- as.POSIXct(x[ok] / 1000, origin = "1970-01-01", tz = tz)
        return(out)
      }

      # Unix seconds
      if (xr[1] > 1e8 && xr[2] < 5e9) {
        out[ok] <- as.POSIXct(x[ok], origin = "1970-01-01", tz = tz)
        return(out)
      }
    }

    return(out)
  }

  x <- trimws(as.character(x))
  x[x == ""] <- NA_character_

  # try multiple common date-time/date formats
  parsed <- suppressWarnings(
    lubridate::parse_date_time(
      x,
      orders = c(
        "ymd HMS", "ymd HM", "ymd",
        "Ymd HMS", "Ymd HM", "Ymd",
        "dmy HMS", "dmy HM", "dmy",
        "mdy HMS", "mdy HM", "mdy"
      ),
      tz = tz,
      quiet = TRUE
    )
  )

  as.POSIXct(parsed, tz = tz)
}

.dm_auto_find_time_col <- function(dat, tz = "UTC", min_success = 0.6) {
  nms <- names(dat)
  scores <- rep(NA_real_, length(nms))
  parsed_list <- vector("list", length(nms))

  for (i in seq_along(nms)) {
    v <- dat[[i]]
    parsed <- .dm_parse_time_candidate(v, tz = tz)
    parsed_list[[i]] <- parsed

    valid_input <- if (is.character(v) || is.factor(v)) {
      !is.na(v) & trimws(as.character(v)) != ""
    } else {
      !is.na(v)
    }

    denom <- sum(valid_input)
    success <- if (denom == 0) 0 else sum(!is.na(parsed) & valid_input) / denom

    # slight preference to names that look like time/date columns
    name_bonus <- if (grepl("time|date|datetime|timestamp", nms[i], ignore.case = TRUE)) 0.05 else 0
    scores[i] <- success + name_bonus
  }

  best <- which.max(scores)
  best_success <- if (length(best) == 0) 0 else scores[best]

  if (length(best) == 0 || best_success < min_success) {
    stop(
      "Could not detect a valid climate time column automatically. ",
      "Please provide 'time_col' explicitly. Accepted formats are POSIXct, Date, ",
      "or character timestamps such as 'yyyy-mm-dd HH:MM:SS'."
    )
  }

  list(
    time_col = nms[best],
    parsed_time = parsed_list[[best]]
  )
}

.dm_maybe_numeric <- function(x, dec = ".") {
  if (is.numeric(x)) return(x)
  if (inherits(x, "POSIXct") || inherits(x, "Date")) return(x)

  z <- trimws(as.character(x))
  z[z == ""] <- NA_character_

  if (!identical(dec, ".")) {
    z <- gsub(dec, ".", z, fixed = TRUE)
  }

  num <- suppressWarnings(as.numeric(z))

  ok_in <- sum(!is.na(z))
  ok_out <- sum(!is.na(num))

  # only coerce if most non-missing values can be converted
  if (ok_in > 0 && ok_out / ok_in >= 0.8) {
    return(num)
  }

  x
}

.dm_resolution_hours <- function(time) {
  d <- diff(as.numeric(time)) / 3600
  d <- d[is.finite(d) & d > 0]
  if (length(d) == 0) return(NA_real_)
  stats::median(d)
}

.dm_is_irregular <- function(time) {
  d <- diff(as.numeric(time)) / 60
  d <- d[is.finite(d) & d > 0]
  if (length(d) <= 1) return(FALSE)
  length(unique(round(d, 6))) > 1
}

.dm_safe_agg <- function(x, FUN) {
  if (length(x) == 0 || all(is.na(x))) return(NA_real_)
  FUN(x, na.rm = TRUE)
}

.dm_roll_right <- function(x, n, FUN) {
  n <- as.integer(n)
  out <- rep(NA_real_, length(x))

  if (length(x) == 0 || n <= 0) return(out)
  if (n > length(x)) return(out)

  for (i in seq_along(x)) {
    if (i >= n) {
      out[i] <- .dm_safe_agg(x[(i - n + 1):i], FUN)
    }
  }
  out
}

.dm_lag_vec <- function(x, n) {
  n <- as.integer(n)
  if (n <= 0) return(x)
  if (n >= length(x)) return(rep(NA, length(x)))
  c(rep(NA, n), x[seq_len(length(x) - n)])
}

.dm_prepare_climate <- function(clim_input, verbose = FALSE) {
  if (inherits(clim_input, "dm_clim")) {
    out <- tibble::as_tibble(clim_input)
    out$TIME <- as.POSIXct(out$TIME)
    out <- out[order(out$TIME), , drop = FALSE]
    return(out)
  }
  read.climate(clim_input, verbose = verbose)
}



#' @title Read and standardize climate data for dendrometer analyses
#'
#' @description
#' A robust climate-data reader designed to be as flexible as read.dendrometer().
#' It accepts data frames and common file formats, auto-detects separators and
#' decimal marks, parses many datetime formats, supports Excel serial dates,
#' supports separate date + time columns, detects the time column automatically,
#' sorts timestamps, removes duplicates, converts numeric-like climate variables,
#' and returns a standardized tibble with a POSIXct TIME column.
#'
#' @param x A data frame or path to a file. Supported file extensions are
#'   csv, txt, tsv, tab, dat, xls, xlsx, rds, rda, and RData.
#' @param time_col Backward-compatible explicit time/datetime column name or index.
#'   If date_col is also supplied, this is treated as the time-of-day column.
#' @param vars Optional character vector of climate variables to keep.
#' @param sep Optional field separator for text files. If NULL, it is auto-detected.
#' @param dec Optional decimal mark for text files. If NULL, it is auto-detected.
#' @param header Logical; passed to text-file readers.
#' @param sheet Sheet name or index for Excel files.
#' @param tz Time zone for parsed timestamps.
#' @param drop_duplicate_time Logical; if TRUE, duplicated timestamps are removed.
#' @param min_time_success Minimum parsing success proportion for automatic time detection.
#' @param verbose Logical; print an import summary.
#' @param datetime_col Optional explicit datetime column name or index. Prefer this
#'   when the file has one combined timestamp column.
#' @param date_col Optional explicit date column name or index. Can be combined with
#'   time_col, or parsed alone with midnight appended when assume_midnight = TRUE.
#' @param range Optional Excel cell range.
#' @param na Strings to treat as missing values.
#' @param assume_midnight Logical; if TRUE, date-only values are assigned 00:00:00.
#' @param orders Optional lubridate parse_date_time() orders.
#' @param excel_dates One of auto, none, 1900, or 1904.
#' @param drop_empty_cols Logical; if TRUE, columns that are completely empty are removed.
#' @param trim_names Logical; if TRUE, trim whitespace from column names.
#' @param detect_resolution Logical; if TRUE, attach simple time-resolution diagnostics.
#' @param return_report Logical; if TRUE, return list(data = ..., report = ...).
#' @param quiet Logical; suppress messages. By default this is the inverse of verbose.
#'
#' @return A tibble of class dm_clim with TIME in the first column. An import
#'   report is attached as attr(x, "import_report").
#'
#' @export
read.climate <- function(
    x,
    time_col = NULL,
    vars = NULL,
    sep = NULL,
    dec = NULL,
    header = TRUE,
    sheet = 1,
    tz = "UTC",
    drop_duplicate_time = TRUE,
    min_time_success = 0.6,
    verbose = TRUE,
    datetime_col = NULL,
    date_col = NULL,
    range = NULL,
    na = c("", "NA", "NaN", "nan", "null", "NULL", "-9999", "-999", "N/A"),
    assume_midnight = TRUE,
    orders = NULL,
    excel_dates = c("auto", "none", "1900", "1904"),
    drop_empty_cols = TRUE,
    trim_names = TRUE,
    detect_resolution = FALSE,
    return_report = FALSE,
    quiet = !verbose
) {
  excel_dates <- match.arg(excel_dates)

  # -------------------------------------------------------------------------
  # Helpers
  # -------------------------------------------------------------------------
  `%||%` <- function(a, b) if (!is.null(a)) a else b

  default_orders <- function() {
    c(
      "Y-m-d H:M:S", "Y-m-d H:M", "Y-m-d",
      "Y/m/d H:M:S", "Y/m/d H:M", "Y/m/d",
      "Y.m.d H:M:S", "Y.m.d H:M", "Y.m.d",
      "d.m.Y H:M:S", "d.m.Y H:M", "d.m.Y",
      "d-m-Y H:M:S", "d-m-Y H:M", "d-m-Y",
      "d/m/Y H:M:S", "d/m/Y H:M", "d/m/Y",
      "m/d/Y H:M:S", "m/d/Y H:M", "m/d/Y",
      "Y-m-d I:M p", "Y-m-d I:M:S p",
      "m/d/Y I:M p", "m/d/Y I:M:S p",
      "d/m/Y I:M p", "d/m/Y I:M:S p",
      "Ymd HMS", "Ymd HM", "Ymd",
      "dmY HMS", "dmY HM", "dmY",
      "mdY HMS", "mdY HM", "mdY"
    )
  }

  get_col_index <- function(data, col, arg_name) {
    if (is.null(col)) return(NULL)

    if (is.character(col)) {
      idx <- match(col, names(data))
      if (is.na(idx)) {
        idx <- match(tolower(col), tolower(names(data)))
      }
      if (is.na(idx)) stop(arg_name, " '", col, "' not found in columns.", call. = FALSE)
      return(idx)
    }

    idx <- suppressWarnings(as.integer(col))
    if (!is.finite(idx) || idx < 1 || idx > ncol(data)) {
      stop(arg_name, " index out of bounds.", call. = FALSE)
    }
    idx
  }

  nonempty <- function(z) {
    if (is.factor(z)) z <- as.character(z)
    if (is.character(z)) return(!is.na(z) & trimws(z) != "")
    !is.na(z)
  }

  sniff_sep <- function(lines) {
    lines <- lines[nzchar(trimws(lines))]
    if (!length(lines)) return(",")

    candidates <- c("," = ",", ";" = ";", "\\t" = "\t", "|" = "|")

    scores <- vapply(candidates, function(s) {
      counts <- vapply(
        lines,
        function(line) length(strsplit(line, s, fixed = TRUE)[[1]]) - 1L,
        integer(1)
      )
      if (max(counts, na.rm = TRUE) == 0) return(0)
      median(counts, na.rm = TRUE) + stats::sd(counts, na.rm = TRUE) * -0.1
    }, numeric(1))

    if (max(scores, na.rm = TRUE) <= 0) "" else candidates[[which.max(scores)]]
  }

  sniff_dec <- function(lines, sep) {
    if (!length(lines)) return(".")
    if (identical(sep, ",")) return(".")

    if (identical(sep, "")) {
      tokens <- unlist(strsplit(lines, "[[:space:]]+"), use.names = FALSE)
    } else {
      tokens <- unlist(strsplit(lines, sep, fixed = TRUE), use.names = FALSE)
    }
    tokens <- trimws(tokens)
    tokens <- tokens[nzchar(tokens)]

    comma_hits <- sum(grepl("^-?[0-9]+,[0-9]+$", tokens))
    dot_hits <- sum(grepl("^-?[0-9]+\\.[0-9]+$", tokens))

    if (comma_hits > dot_hits) "," else "."
  }

  append_midnight_if_needed <- function(x_chr) {
    if (!assume_midnight) return(x_chr)

    x_chr <- trimws(x_chr)
    only_date <- grepl("^\\d{4}[-/.]\\d{1,2}[-/.]\\d{1,2}$", x_chr) |
      grepl("^\\d{1,2}[-/.]\\d{1,2}[-/.]\\d{4}$", x_chr) |
      grepl("^\\d{8}$", x_chr)

    x_chr[only_date] <- paste0(x_chr[only_date], " 00:00:00")
    x_chr
  }

  excel_serial_to_posix <- function(x_num, tz, system = c("1900", "1904")) {
    system <- match.arg(system)
    origin <- if (system == "1900") "1899-12-30" else "1904-01-01"
    as.POSIXct(x_num * 86400, origin = origin, tz = tz)
  }

  choose_excel_system <- function(x_num, tz, excel_dates) {
    systems <- if (excel_dates == "auto") c("1900", "1904") else excel_dates

    best <- list(
      system = NA_character_,
      dt = rep(as.POSIXct(NA, tz = tz), length(x_num)),
      score = -Inf
    )

    for (sys in systems) {
      dt <- suppressWarnings(excel_serial_to_posix(x_num, tz = tz, system = sys))
      yrs <- suppressWarnings(as.integer(format(dt, "%Y")))
      score <- sum(!is.na(dt) & yrs >= 1980 & yrs <= 2100)
      if (score > best$score) {
        best$system <- sys
        best$dt <- dt
        best$score <- score
      }
    }

    best
  }

  detect_format_string <- function(x_chr) {
    x_chr <- trimws(as.character(x_chr))
    samp <- x_chr[!is.na(x_chr) & nzchar(x_chr)]
    samp <- utils::head(samp, 50)
    if (!length(samp)) return(NA_character_)

    pats <- list(
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%Y-%m-%d %H:%M:%S",
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2}$" = "%Y-%m-%d %H:%M",
      "^\\d{4}-\\d{1,2}-\\d{1,2}$" = "%Y-%m-%d",
      "^\\d{4}/\\d{1,2}/\\d{1,2} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%Y/%m/%d %H:%M:%S",
      "^\\d{4}/\\d{1,2}/\\d{1,2} \\d{1,2}:\\d{1,2}$" = "%Y/%m/%d %H:%M",
      "^\\d{4}/\\d{1,2}/\\d{1,2}$" = "%Y/%m/%d",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%d.%m.%Y %H:%M:%S",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4} \\d{1,2}:\\d{1,2}$" = "%d.%m.%Y %H:%M",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4}$" = "%d.%m.%Y",
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%m/%d/%Y %H:%M:%S or %d/%m/%Y %H:%M:%S",
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2}$" = "%m/%d/%Y %H:%M or %d/%m/%Y %H:%M",
      "^\\d{1,2}/\\d{1,2}/\\d{4}$" = "%m/%d/%Y or %d/%m/%Y",
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2} [APap][Mm]$" = "%Y-%m-%d %I:%M %p",
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2} [APap][Mm]$" = "%m/%d/%Y %I:%M %p or %d/%m/%Y %I:%M %p"
    )

    hits <- vapply(names(pats), function(rx) mean(grepl(rx, samp)), numeric(1))
    if (max(hits) > 0.9) pats[[which.max(hits)]] else NA_character_
  }

  parse_datetime_vector <- function(z) {
    if (is.null(orders)) orders_local <- default_orders() else orders_local <- orders

    if (inherits(z, "POSIXct")) {
      return(list(
        dt = as.POSIXct(z, tz = tz),
        method = "POSIXct",
        detected_format = "%Y-%m-%d %H:%M:%S",
        excel_system = NA_character_
      ))
    }

    if (inherits(z, "POSIXlt")) {
      return(list(
        dt = as.POSIXct(z, tz = tz),
        method = "POSIXlt",
        detected_format = "%Y-%m-%d %H:%M:%S",
        excel_system = NA_character_
      ))
    }

    if (inherits(z, "Date")) {
      return(list(
        dt = as.POSIXct(z, tz = tz),
        method = "Date",
        detected_format = "%Y-%m-%d",
        excel_system = NA_character_
      ))
    }

    z_chr <- trimws(as.character(z))
    z_chr[z_chr %in% na] <- NA_character_
    z_chr <- append_midnight_if_needed(z_chr)

    detected_format <- detect_format_string(z_chr)

    dt_text <- suppressWarnings(lubridate::parse_date_time(
      z_chr,
      orders = orders_local,
      tz = tz,
      quiet = TRUE
    ))
    dt_text <- as.POSIXct(dt_text, tz = tz)
    score_text <- sum(!is.na(dt_text))

    dt_unix <- rep(as.POSIXct(NA, tz = tz), length(z_chr))
    score_unix <- -Inf
    unix_method <- NA_character_

    z_num <- suppressWarnings(as.numeric(z_chr))
    numeric_like <- is.numeric(z) || all(is.na(z_chr) | !nzchar(z_chr) | grepl("^-?[0-9]+([.]?[0-9]+)?$", z_chr))

    if (numeric_like && any(is.finite(z_num), na.rm = TRUE)) {
      rng <- range(z_num[is.finite(z_num)], na.rm = TRUE)

      if (rng[1] > 1e12 && rng[2] < 5e13) {
        dt_unix <- as.POSIXct(z_num / 1000, origin = "1970-01-01", tz = tz)
        score_unix <- sum(!is.na(dt_unix))
        unix_method <- "unix_milliseconds"
      } else if (rng[1] > 1e8 && rng[2] < 5e9) {
        dt_unix <- as.POSIXct(z_num, origin = "1970-01-01", tz = tz)
        score_unix <- sum(!is.na(dt_unix))
        unix_method <- "unix_seconds"
      }
    }

    dt_excel <- rep(as.POSIXct(NA, tz = tz), length(z_chr))
    score_excel <- -Inf
    excel_system <- NA_character_

    if (excel_dates != "none" && numeric_like && any(is.finite(z_num), na.rm = TRUE)) {
      chosen <- choose_excel_system(z_num, tz = tz, excel_dates = excel_dates)
      dt_excel <- chosen$dt
      excel_system <- chosen$system
      score_excel <- chosen$score
    }

    scores <- c(text = score_text, unix = score_unix, excel = score_excel)
    best <- names(which.max(scores))

    if (best == "excel" && score_excel > 0) {
      return(list(
        dt = dt_excel,
        method = "excel_serial",
        detected_format = paste0("Excel serial date (", excel_system, " system)"),
        excel_system = excel_system
      ))
    }

    if (best == "unix" && score_unix > 0) {
      return(list(
        dt = dt_unix,
        method = unix_method,
        detected_format = unix_method,
        excel_system = NA_character_
      ))
    }

    list(
      dt = dt_text,
      method = "text",
      detected_format = detected_format,
      excel_system = NA_character_
    )
  }

  format_time_of_day <- function(t) {
    if (inherits(t, "POSIXct") || inherits(t, "POSIXlt")) return(format(t, "%H:%M:%S"))
    if (inherits(t, "hms")) return(as.character(t))

    if (is.numeric(t)) {
      out <- rep(NA_character_, length(t))
      ok <- is.finite(t)

      # Excel fractional day, e.g. 0.5 = 12:00:00
      frac <- ok & t >= 0 & t < 1
      secs <- round(t[frac] * 86400)
      out[frac] <- sprintf("%02d:%02d:%02d", secs %/% 3600, (secs %% 3600) %/% 60, secs %% 60)

      # Decimal hours, e.g. 13.5 = 13:30:00
      hrs <- ok & t >= 1 & t < 24
      secs2 <- round(t[hrs] * 3600)
      out[hrs] <- sprintf("%02d:%02d:%02d", secs2 %/% 3600, (secs2 %% 3600) %/% 60, secs2 %% 60)
      return(out)
    }

    z <- trimws(as.character(t))
    z[z %in% na] <- NA_character_
    z
  }

  combine_date_time <- function(date_vec, time_vec = NULL) {
    if (is.null(time_vec)) return(date_vec)

    # Numeric Excel date + numeric fractional day/time.
    if (is.numeric(date_vec) && is.numeric(time_vec)) {
      ok_time <- is.finite(time_vec) & time_vec >= 0 & time_vec < 1
      out <- date_vec
      out[ok_time] <- date_vec[ok_time] + time_vec[ok_time]
      return(out)
    }

    date_chr <- trimws(as.character(date_vec))
    time_chr <- format_time_of_day(time_vec)
    paste(date_chr, time_chr)
  }

  success_rate <- function(parsed, original) {
    valid_input <- nonempty(original)
    denom <- sum(valid_input)
    if (denom == 0) return(0)
    sum(!is.na(parsed) & valid_input) / denom
  }

  has_subdaily_variation <- function(dt) {
    ok <- !is.na(dt)
    if (sum(ok) == 0) return(FALSE)
    any(format(dt[ok], "%H:%M:%S") != "00:00:00")
  }

  looks_like_date_name <- function(nm) grepl("date|day|datum|fecha", nm, ignore.case = TRUE)
  looks_like_time_name <- function(nm) grepl("time|hour|heure|zeit|timestamp|datetime", nm, ignore.case = TRUE)

  auto_find_time <- function(dat) {
    nms <- names(dat)
    best <- list(
      mode = NA_character_,
      datetime_col = NA_integer_,
      date_col = NA_integer_,
      time_col = NA_integer_,
      parsed = rep(as.POSIXct(NA, tz = tz), nrow(dat)),
      score = -Inf,
      success = 0,
      method = NA_character_,
      detected_format = NA_character_,
      excel_system = NA_character_
    )

    # 1) Single-column datetime candidates.
    for (i in seq_along(nms)) {
      parsed <- parse_datetime_vector(dat[[i]])
      success <- success_rate(parsed$dt, dat[[i]])
      bonus <- 0
      if (looks_like_time_name(nms[i]) || looks_like_date_name(nms[i])) bonus <- bonus + 0.08
      if (has_subdaily_variation(parsed$dt)) bonus <- bonus + 0.03
      score <- success + bonus

      if (score > best$score) {
        best <- list(
          mode = "datetime_col",
          datetime_col = i,
          date_col = NA_integer_,
          time_col = NA_integer_,
          parsed = parsed$dt,
          score = score,
          success = success,
          method = parsed$method,
          detected_format = parsed$detected_format,
          excel_system = parsed$excel_system
        )
      }
    }

    # 2) Separate date + time candidates.
    # Prefer columns with date/time-like names, but also check all columns if needed.
    date_candidates <- which(vapply(nms, looks_like_date_name, logical(1)))
    time_candidates <- which(vapply(nms, looks_like_time_name, logical(1)))

    if (!length(date_candidates)) date_candidates <- seq_along(nms)
    if (!length(time_candidates)) time_candidates <- seq_along(nms)

    for (di in date_candidates) {
      for (ti in setdiff(time_candidates, di)) {
        combined <- combine_date_time(dat[[di]], dat[[ti]])
        parsed <- parse_datetime_vector(combined)
        success <- success_rate(parsed$dt, combined)
        bonus <- 0.04
        if (looks_like_date_name(nms[di])) bonus <- bonus + 0.06
        if (looks_like_time_name(nms[ti])) bonus <- bonus + 0.06
        if (has_subdaily_variation(parsed$dt)) bonus <- bonus + 0.05
        score <- success + bonus

        if (score > best$score) {
          best <- list(
            mode = "date_col + time_col",
            datetime_col = NA_integer_,
            date_col = di,
            time_col = ti,
            parsed = parsed$dt,
            score = score,
            success = success,
            method = parsed$method,
            detected_format = parsed$detected_format,
            excel_system = parsed$excel_system
          )
        }
      }
    }

    if (!is.finite(best$score) || best$success < min_time_success) {
      stop(
        "Could not detect a valid climate time column automatically. ",
        "Use datetime_col = 'your_datetime_column' or date_col = 'date' and time_col = 'time'.\n",
        "Detected columns were: ", paste(nms, collapse = ", "),
        call. = FALSE
      )
    }

    best
  }

  maybe_numeric <- function(v) {
    if (is.numeric(v)) return(v)
    if (inherits(v, "POSIXct") || inherits(v, "Date")) return(v)

    z <- trimws(as.character(v))
    z[z %in% na] <- NA_character_
    if (!is.null(dec) && !identical(dec, ".")) z <- gsub(dec, ".", z, fixed = TRUE)
    num <- suppressWarnings(as.numeric(z))

    ok_in <- sum(!is.na(z))
    ok_out <- sum(!is.na(num))

    if (ok_in > 0 && ok_out / ok_in >= 0.8) num else v
  }

  compute_resolution_diag <- function(dt) {
    dt_ok <- dt[!is.na(dt)]
    dt_ok <- sort(unique(dt_ok))

    if (length(dt_ok) < 2) {
      return(list(
        n_timestamps = length(dt_ok),
        start_time = if (length(dt_ok)) min(dt_ok) else as.POSIXct(NA, tz = tz),
        end_time = if (length(dt_ok)) max(dt_ok) else as.POSIXct(NA, tz = tz),
        dominant_step_seconds = NA_real_,
        dominant_step_minutes = NA_real_,
        irregular_intervals = NA_integer_,
        estimated_missing_steps = NA_integer_
      ))
    }

    diffs_sec <- round(as.numeric(diff(dt_ok), units = "secs"))
    diffs_sec <- diffs_sec[is.finite(diffs_sec) & diffs_sec > 0]
    tab <- sort(table(diffs_sec), decreasing = TRUE)
    dominant <- as.numeric(names(tab)[1])

    list(
      n_timestamps = length(dt_ok),
      start_time = min(dt_ok),
      end_time = max(dt_ok),
      dominant_step_seconds = dominant,
      dominant_step_minutes = dominant / 60,
      irregular_intervals = sum(diffs_sec != dominant),
      estimated_missing_steps = sum(pmax(round(diffs_sec / dominant) - 1L, 0L), na.rm = TRUE)
    )
  }

  make_report <- function() {
    list(
      source = input_source,
      file = if (is.character(x) && length(x) == 1) normalizePath(x, winslash = "/", mustWork = FALSE) else NA_character_,
      file_type = file_ext,
      separator = sep_used,
      decimal_mark = dec_used,
      datetime_source = datetime_source,
      datetime_column_name = datetime_name,
      date_column_name = date_name,
      time_column_name = time_name,
      datetime_parse_method = parse_method,
      datetime_format_detected = detected_format,
      excel_date_system = excel_system_used,
      tz = tz,
      n_rows_input = n_rows_input,
      n_rows_output = nrow(out),
      n_cols_output = ncol(out),
      climate_variables = setdiff(names(out), "TIME"),
      n_bad_datetime = n_bad_datetime,
      bad_datetime_rows = utils::head(which(is.na(out$TIME)), 20L),
      n_duplicated_timestamps = n_dup,
      duplicated_timestamps_dropped = drop_duplicate_time,
      resolution_diagnostics = resolution_diag
    )
  }

  # -------------------------------------------------------------------------
  # Read input
  # -------------------------------------------------------------------------
  input_source <- if (is.data.frame(x)) "data_frame" else "file"
  file_ext <- NA_character_
  sep_used <- sep %||% NA_character_
  dec_used <- dec %||% NA_character_

  if (is.data.frame(x)) {
    dat <- tibble::as_tibble(x)
  } else if (is.character(x) && length(x) == 1 && file.exists(x)) {
    file_ext <- tolower(tools::file_ext(x))

    if (file_ext %in% c("xls", "xlsx")) {
      if (!requireNamespace("readxl", quietly = TRUE)) {
        stop("Package 'readxl' is required to read Excel climate files.", call. = FALSE)
      }
      if (is.null(range)) {
        dat <- readxl::read_excel(path = x, sheet = sheet, na = na)
      } else {
        dat <- readxl::read_excel(path = x, sheet = sheet, range = range, na = na)
      }
      dat <- tibble::as_tibble(dat)
    } else if (file_ext %in% c("rds")) {
      dat <- readRDS(x)
      if (!is.data.frame(dat)) stop("RDS file must contain a data frame.", call. = FALSE)
      dat <- tibble::as_tibble(dat)
    } else if (file_ext %in% c("rda", "rdata")) {
      env <- new.env(parent = emptyenv())
      objs <- load(x, envir = env)
      if (length(objs) != 1) stop("RData/RDA file must contain exactly one object.", call. = FALSE)
      dat <- env[[objs]]
      if (!is.data.frame(dat)) stop("RData/RDA object must be a data frame.", call. = FALSE)
      dat <- tibble::as_tibble(dat)
    } else if (file_ext %in% c("csv", "txt", "tsv", "tab", "dat")) {
      lines <- readLines(x, n = 100L, warn = FALSE)
      lines <- lines[nzchar(trimws(lines))]
      if (!length(lines)) stop("Climate file appears to be empty.", call. = FALSE)

      if (is.null(sep)) {
        sep <- if (file_ext %in% c("tsv", "tab")) "\t" else sniff_sep(lines)
      }
      if (is.null(dec)) {
        data_lines <- if (length(lines) > 1) lines[-1] else lines
        dec <- sniff_dec(data_lines, sep = sep)
      }
      sep_used <- sep
      dec_used <- dec

      if (requireNamespace("readr", quietly = TRUE) && !identical(sep, "")) {
        dat <- readr::read_delim(
          file = x,
          delim = sep,
          na = na,
          locale = readr::locale(decimal_mark = dec),
          col_types = readr::cols(.default = readr::col_guess()),
          show_col_types = FALSE,
          progress = FALSE,
          trim_ws = TRUE
        )
        dat <- tibble::as_tibble(dat)
      } else {
        dat <- utils::read.table(
          file = x,
          sep = sep,
          dec = dec,
          header = header,
          na.strings = na,
          stringsAsFactors = FALSE,
          check.names = FALSE,
          fill = TRUE,
          comment.char = "",
          quote = "\"'"
        )
        dat <- tibble::as_tibble(dat)
      }
    } else {
      stop(
        "Unsupported climate file format: ", file_ext,
        ". Use csv, txt, tsv, tab, dat, xls, xlsx, rds, rda, or RData.",
        call. = FALSE
      )
    }
  } else {
    stop("'x' must be a data frame or a valid file path.", call. = FALSE)
  }

  if (trim_names) names(dat) <- trimws(names(dat))
  if (drop_empty_cols && ncol(dat) > 0) {
    keep <- vapply(dat, function(col) any(nonempty(col)), logical(1))
    dat <- dat[, keep, drop = FALSE]
  }

  n_rows_input <- nrow(dat)

  if (ncol(dat) < 2) {
    stop(
      "Climate input must contain at least one time column and one climate variable.\n",
      "Only ", ncol(dat), " column(s) were read. This usually means the delimiter was wrong.\n",
      "Try sep = ';', sep = ',', sep = '\\t', or check whether the file has a header.",
      call. = FALSE
    )
  }

  # -------------------------------------------------------------------------
  # Build TIME
  # -------------------------------------------------------------------------
  datetime_source <- NA_character_
  datetime_name <- NA_character_
  date_name <- NA_character_
  time_name <- NA_character_
  parse_method <- NA_character_
  detected_format <- NA_character_
  excel_system_used <- NA_character_
  drop_idx <- integer(0)

  if (!is.null(datetime_col) && !is.null(date_col)) {
    stop("Use either datetime_col or date_col/time_col, not both.", call. = FALSE)
  }

  if (!is.null(date_col)) {
    date_idx <- get_col_index(dat, date_col, "date_col")
    time_idx <- get_col_index(dat, time_col, "time_col")

    combined <- if (is.null(time_idx)) dat[[date_idx]] else combine_date_time(dat[[date_idx]], dat[[time_idx]])
    parsed <- parse_datetime_vector(combined)
    success <- success_rate(parsed$dt, combined)
    if (success < min_time_success) {
      stop("The supplied date_col/time_col could not be parsed reliably.", call. = FALSE)
    }

    TIME <- parsed$dt
    datetime_source <- if (is.null(time_idx)) "date_col" else "date_col + time_col"
    date_name <- names(dat)[date_idx]
    time_name <- if (is.null(time_idx)) NA_character_ else names(dat)[time_idx]
    datetime_name <- "TIME"
    parse_method <- parsed$method
    detected_format <- parsed$detected_format
    excel_system_used <- parsed$excel_system
    drop_idx <- sort(unique(c(date_idx, time_idx)))
  } else if (!is.null(datetime_col) || !is.null(time_col)) {
    # Backward-compatible: time_col alone means combined datetime column.
    dt_col <- datetime_col %||% time_col
    dt_idx <- get_col_index(dat, dt_col, if (!is.null(datetime_col)) "datetime_col" else "time_col")

    parsed <- parse_datetime_vector(dat[[dt_idx]])
    success <- success_rate(parsed$dt, dat[[dt_idx]])
    if (success < min_time_success) {
      stop("The supplied time/datetime column could not be parsed reliably.", call. = FALSE)
    }

    TIME <- parsed$dt
    datetime_source <- "datetime_col"
    datetime_name <- names(dat)[dt_idx]
    parse_method <- parsed$method
    detected_format <- parsed$detected_format
    excel_system_used <- parsed$excel_system
    drop_idx <- dt_idx
  } else {
    det <- auto_find_time(dat)
    TIME <- det$parsed
    datetime_source <- det$mode
    datetime_name <- if (!is.na(det$datetime_col)) names(dat)[det$datetime_col] else "TIME"
    date_name <- if (!is.na(det$date_col)) names(dat)[det$date_col] else NA_character_
    time_name <- if (!is.na(det$time_col)) names(dat)[det$time_col] else NA_character_
    parse_method <- det$method
    detected_format <- det$detected_format
    excel_system_used <- det$excel_system
    drop_idx <- if (det$mode == "date_col + time_col") {
      sort(unique(c(det$date_col, det$time_col)))
    } else {
      det$datetime_col
    }
  }

  if (all(is.na(TIME))) {
    stop(
      "Could not parse the climate time information. Use datetime_col = '...' ",
      "or date_col = '...' and time_col = '...'.",
      call. = FALSE
    )
  }

  other <- dat[, setdiff(seq_len(ncol(dat)), drop_idx), drop = FALSE]
  out <- tibble::as_tibble(cbind(tibble::tibble(TIME = TIME), other))

  # Keep selected variables.
  if (!is.null(vars)) {
    miss <- setdiff(vars, names(out))
    if (length(miss) > 0) {
      stop("Requested climate variables not found: ", paste(miss, collapse = ", "), call. = FALSE)
    }
    out <- out[, c("TIME", vars), drop = FALSE]
  }

  # Convert climate variables to numeric where sensible.
  for (nm in setdiff(names(out), "TIME")) {
    out[[nm]] <- maybe_numeric(out[[nm]])
  }

  # Sort and remove duplicate timestamps.
  out <- out[order(out$TIME, na.last = TRUE), , drop = FALSE]
  n_bad_datetime <- sum(is.na(out$TIME))

  dup <- duplicated(out$TIME) & !is.na(out$TIME)
  n_dup <- sum(dup)
  if (n_dup > 0 && drop_duplicate_time) {
    out <- out[!dup, , drop = FALSE]
    if (!quiet) warning("Duplicated climate timestamps were removed; first occurrence kept.", call. = FALSE)
  }

  if (ncol(out) < 2) {
    stop("After removing the time column, no climate variables remain.", call. = FALSE)
  }

  numeric_vars <- names(out)[vapply(out, is.numeric, logical(1))]
  numeric_vars <- setdiff(numeric_vars, "TIME")

  if (!length(numeric_vars) && !quiet) {
    warning(
      "No climate variables appear numeric after reading. Check sep, dec, and column formatting.",
      call. = FALSE
    )
  }

  resolution_diag <- if (detect_resolution) compute_resolution_diag(out$TIME) else NULL

  attr(out, "timezone") <- tz
  attr(out, "time_col_original") <- datetime_name
  attr(out, "date_col_original") <- date_name
  attr(out, "clock_time_col_original") <- time_name

  class(out) <- unique(c("dm_clim", class(out)))

  report <- make_report()
  attr(out, "import_report") <- report

  if (!quiet) {
    msg <- paste0(
      "Climate data standardized.\n",
      "Rows: ", nrow(out), "\n",
      "Time source: ", datetime_source,
      if (!is.na(datetime_name)) paste0(" [", datetime_name, "]") else "",
      if (!is.na(date_name)) paste0(" date=[", date_name, "]") else "",
      if (!is.na(time_name)) paste0(" time=[", time_name, "]") else "",
      "\nVariables: ", paste(setdiff(names(out), "TIME"), collapse = ", "), "\n",
      "Numeric variables: ", if (length(numeric_vars)) paste(numeric_vars, collapse = ", ") else "none", "\n",
      "Time range: ", format(min(out$TIME, na.rm = TRUE)), " to ", format(max(out$TIME, na.rm = TRUE))
    )
    message(msg)

    if (!is.na(sep_used)) message("Detected/used separator: '", sep_used, "'.")
    if (!is.na(dec_used)) message("Detected/used decimal mark: '", dec_used, "'.")
    if (!is.na(detected_format)) message("Detected datetime format: ", detected_format)
  }

  if (return_report) {
    return(list(data = out, report = report))
  }

  out
}



#' @title Daily climate summaries for dendrometer analyses
#'
#' @description
#' Computes daily climate summaries from climate time series so they can be
#' related to daily dendrometer summaries from \code{daily.data()}.
#'
#' The input can be a standardized climate object returned by
#' \code{read.climate()}, a raw data frame, or a valid file path accepted by
#' \code{read.climate()}.
#'
#' In addition to same-day climate summaries, the function can also compute
#' lagged and antecedent daily climate features from the summarized daily series:
#' \itemize{
#'   \item lagged values (e.g. previous 1 or 3 days)
#'   \item antecedent means over previous \code{n} days
#'   \item antecedent sums over previous \code{n} days
#' }
#'
#' @details
#' Lagged and antecedent features are calculated from the already summarized
#' daily climate columns. For example, if \code{VPD} is included in
#' \code{max_vars}, the daily summary column will be \code{VPD_max}. If this
#' column is listed in \code{lag_vars} and \code{lag_days = 1}, then the
#' additional column \code{VPD_max_lag_1d} is created.
#'
#' Antecedent means and sums exclude the current day. For example:
#' \deqn{x\_lagmean\_3d(t) = mean(x_{t-3}, x_{t-2}, x_{t-1})}
#' \deqn{x\_lagsum\_7d(t) = sum(x_{t-7}, \ldots, x_{t-1})}
#'
#' @param clim_df Climate input. This can be:
#'   \itemize{
#'     \item a standardized object returned by \code{read.climate()}
#'     \item a raw data frame with a time column in the first column or in a
#'           column named \code{TIME}
#'     \item a valid file path readable by \code{read.climate()}
#'   }
#' @param mean_vars Character vector of variables to summarize by mean.
#' @param min_vars Character vector of variables to summarize by minimum.
#' @param max_vars Character vector of variables to summarize by maximum.
#' @param sum_vars Character vector of variables to summarize by sum.
#' @param median_vars Character vector of variables to summarize by median.
#' @param lag_vars Character vector of summarized daily climate columns for
#'   which simple lagged values should be computed, e.g.
#'   \code{c("VPD_max", "SWC_mean")}.
#' @param lagmean_vars Character vector of summarized daily climate columns for
#'   which antecedent means should be computed, e.g.
#'   \code{c("Tair_mean", "VPD_mean")}.
#' @param lagsum_vars Character vector of summarized daily climate columns for
#'   which antecedent sums should be computed, e.g.
#'   \code{c("P_sum", "Rad_sum")}.
#' @param lag_days Integer vector giving lag/antecedent window sizes in days,
#'   e.g. \code{c(1, 3, 7)}.
#'
#' @return A tibble of class \code{"daily_clim"} with one row per day.
#'
#' @examples
#' \donttest{
#' data(ktm_clim_hourly)
#' clim_day <- dm_daily_clim(
#'   ktm_clim_hourly,
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec"),
#'   lag_vars = c("VPD_max", "temp_mean"),
#'   lagmean_vars = c("temp_mean", "VPD_mean", "RH_mean"),
#'   lagsum_vars = c("prec_sum"),
#'   lag_days = c(1, 3, 7)
#' )
#' head(clim_day, 5)
#' }
#'
#' @export
dm_daily_clim <- function(clim_df,
                          mean_vars = NULL,
                          min_vars = NULL,
                          max_vars = NULL,
                          sum_vars = NULL,
                          median_vars = NULL,
                          lag_vars = NULL,
                          lagmean_vars = NULL,
                          lagsum_vars = NULL,
                          lag_days = c(1, 3, 7)) {
  TIME <- DATE <- NULL

  dat <- .dm_prepare_climate(clim_df, verbose = FALSE)
  numeric_vars <- setdiff(names(dat)[vapply(dat, is.numeric, logical(1))], "TIME")

  if (all(c(
    is.null(mean_vars), is.null(min_vars), is.null(max_vars),
    is.null(sum_vars), is.null(median_vars)
  ))) {
    mean_vars <- numeric_vars
  }

  check_vars <- function(vars, allowed, arg_name) {
    if (is.null(vars)) return(character(0))
    miss <- setdiff(vars, allowed)
    if (length(miss) > 0) {
      stop(sprintf(
        "Unknown or invalid variable(s) in %s: %s",
        arg_name, paste(miss, collapse = ", ")
      ))
    }
    vars
  }

  # raw climate vars to summarize
  mean_vars   <- check_vars(mean_vars, numeric_vars, "mean_vars")
  min_vars    <- check_vars(min_vars, numeric_vars, "min_vars")
  max_vars    <- check_vars(max_vars, numeric_vars, "max_vars")
  sum_vars    <- check_vars(sum_vars, numeric_vars, "sum_vars")
  median_vars <- check_vars(median_vars, numeric_vars, "median_vars")

  # helper for antecedent windows excluding current day
  antecedent_stat <- function(x, n, FUN) {
    out <- rep(NA_real_, length(x))
    n <- as.integer(n)

    if (n <= 0) return(out)
    if (length(x) <= n) return(out)

    for (i in seq_along(x)) {
      if (i > n) {
        out[i] <- .dm_safe_agg(x[(i - n):(i - 1)], FUN)
      }
    }
    out
  }

  # build daily summary first
  out <- dat %>%
    dplyr::mutate(DATE = as.Date(TIME)) %>%
    dplyr::group_by(DATE) %>%
    dplyr::summarise(
      dplyr::across(
        dplyr::all_of(mean_vars),
        ~ .dm_safe_agg(.x, mean),
        .names = "{.col}_mean"
      ),
      dplyr::across(
        dplyr::all_of(min_vars),
        ~ .dm_safe_agg(.x, min),
        .names = "{.col}_min"
      ),
      dplyr::across(
        dplyr::all_of(max_vars),
        ~ .dm_safe_agg(.x, max),
        .names = "{.col}_max"
      ),
      dplyr::across(
        dplyr::all_of(sum_vars),
        ~ .dm_safe_agg(.x, sum),
        .names = "{.col}_sum"
      ),
      dplyr::across(
        dplyr::all_of(median_vars),
        ~ .dm_safe_agg(.x, stats::median),
        .names = "{.col}_median"
      ),
      .groups = "drop"
    )

  # validate requested lag-based daily summary columns
  summary_cols <- setdiff(names(out), "DATE")
  lag_vars     <- check_vars(lag_vars, summary_cols, "lag_vars")
  lagmean_vars <- check_vars(lagmean_vars, summary_cols, "lagmean_vars")
  lagsum_vars  <- check_vars(lagsum_vars, summary_cols, "lagsum_vars")

  if (length(lag_days) > 0) {
    if (any(!is.finite(lag_days)) || any(lag_days <= 0)) {
      stop("'lag_days' must contain positive finite integers.")
    }
    lag_days <- sort(unique(as.integer(lag_days)))
  }

  # simple lags
  if (length(lag_vars) > 0 && length(lag_days) > 0) {
    for (v in lag_vars) {
      for (d in lag_days) {
        out[[paste0(v, "_lag_", d, "d")]] <- dplyr::lag(out[[v]], n = d)
      }
    }
  }

  # antecedent means
  if (length(lagmean_vars) > 0 && length(lag_days) > 0) {
    for (v in lagmean_vars) {
      for (d in lag_days) {
        out[[paste0(v, "_lagmean_", d, "d")]] <- antecedent_stat(out[[v]], d, mean)
      }
    }
  }

  # antecedent sums
  if (length(lagsum_vars) > 0 && length(lag_days) > 0) {
    for (v in lagsum_vars) {
      for (d in lag_days) {
        out[[paste0(v, "_lagsum_", d, "d")]] <- antecedent_stat(out[[v]], d, sum)
      }
    }
  }

  class(out) <- c("daily_clim", class(out))
  out
}


#' @title Subdaily climate features for dendrometer analyses
#'
#' @description
#' Computes rolling-window and lagged climate features at subdaily resolution
#' for direct linkage with point-level dendrometer outputs such as
#' \code{ZG_phase} and \code{SC_phase}.
#'
#' The input can be a standardized climate object returned by
#' \code{read.climate()}, a raw data frame, or a valid file path accepted by
#' \code{read.climate()}.
#'
#' @details
#' The function learns the temporal resolution automatically from the median
#' time step in the \code{TIME} column. It works with hourly as well as
#' minute-resolution data (for example 60-, 30-, 15-, 10-, or 5-minute data).
#'
#' Rolling windows and lags are provided in hours and may be fractional:
#' \itemize{
#'   \item \code{0.25} = 15 minutes
#'   \item \code{0.5} = 30 minutes
#'   \item \code{1} = 1 hour
#'   \item \code{3} = 3 hours
#' }
#'
#' If the user requests a rolling window or lag that is smaller than the
#' inferred climate resolution, the function stops with an error.
#'
#' If a requested window is not an exact multiple of the inferred resolution,
#' it is rounded to the nearest number of time steps and a warning is issued.
#'
#' @param clim_df Climate input. This can be:
#'   \itemize{
#'     \item a standardized object returned by \code{read.climate()}
#'     \item a raw data frame with a time column in the first column or in a
#'           column named \code{TIME}
#'     \item a valid file path readable by \code{read.climate()}
#'   }
#' @param mean_vars Variables for rolling means.
#' @param sum_vars Variables for rolling sums.
#' @param lag_vars Variables for lagged features.
#' @param roll_hours Numeric vector of rolling-window sizes in hours.
#'   Fractional values are allowed, e.g. \code{0.5} for 30 minutes.
#' @param lag_hours Numeric vector of lag sizes in hours.
#'   Fractional values are allowed, e.g. \code{0.25} for 15 minutes.
#' @examples
#' \donttest{
#' data(ktm_clim_hourly)
#'
#' clim_sub <- dm_subdaily_clim(
#'   ktm_clim_hourly,
#'   mean_vars = c("temp", "VPD", "RH"),
#'   sum_vars  = c("prec"),
#'   lag_vars  = c("temp", "VPD", "RH"),
#'   roll_hours = c(1, 3, 6, 24),
#'   lag_hours  = c(1, 3, 6, 24)
#' )
#'
#' head(clim_sub)
#' attr(clim_sub, "resolution_hours")
#' }
#' @return
#' A tibble of class \code{"subdaily_clim"} with timestamp-level climate
#' features added. The inferred temporal resolution in hours is stored in
#' \code{attr(x, "resolution_hours")}.
#'
#' @export
dm_subdaily_clim <- function(clim_df,
                             mean_vars = NULL,
                             sum_vars = NULL,
                             lag_vars = NULL,
                             roll_hours = c(3, 6, 24),
                             lag_hours = c(1, 3, 6, 24)) {

  dat <- .dm_prepare_climate(clim_df, verbose = FALSE)
  numeric_vars <- setdiff(names(dat)[vapply(dat, is.numeric, logical(1))], "TIME")

  if (all(c(is.null(mean_vars), is.null(sum_vars), is.null(lag_vars)))) {
    mean_vars <- numeric_vars
    lag_vars <- numeric_vars
  }

  check_vars <- function(vars, arg_name) {
    if (is.null(vars)) return(character(0))
    miss <- setdiff(vars, numeric_vars)
    if (length(miss) > 0) {
      stop(sprintf("Unknown or non-numeric variable(s) in %s: %s", arg_name, paste(miss, collapse = ", ")))
    }
    vars
  }

  mean_vars <- check_vars(mean_vars, "mean_vars")
  sum_vars  <- check_vars(sum_vars, "sum_vars")
  lag_vars  <- check_vars(lag_vars, "lag_vars")

  res_h <- .dm_resolution_hours(dat$TIME)
  if (is.na(res_h)) {
    stop("Could not infer temporal resolution from the climate time column.")
  }

  if (.dm_is_irregular(dat$TIME)) {
    warning("Climate data appear to have irregular temporal resolution. Rolling/lagged features are based on the median resolution.")
  }

  if (length(roll_hours) > 0 && any(roll_hours < res_h)) {
    stop(
      sprintf(
        "Requested rolling window(s) below the inferred climate resolution (%.6f h): %s",
        res_h,
        paste(roll_hours[roll_hours < res_h], collapse = ", ")
      )
    )
  }

  if (length(lag_hours) > 0 && any(lag_hours < res_h)) {
    stop(
      sprintf(
        "Requested lag(s) below the inferred climate resolution (%.6f h): %s",
        res_h,
        paste(lag_hours[lag_hours < res_h], collapse = ", ")
      )
    )
  }

  out <- dat

  # rolling means
  if (length(mean_vars) > 0 && length(roll_hours) > 0) {
    for (v in mean_vars) {
      for (h in roll_hours) {
        n_steps <- max(1L, as.integer(round(h / res_h)))
        if (abs((h / res_h) - n_steps) > 0.05) {
          warning(sprintf(
            "Rolling window %sh for '%s' is not an exact multiple of the inferred resolution (%.6f h); rounded to %s steps.",
            h, v, res_h, n_steps
          ))
        }
        out[[paste0(v, "_rollmean_", h, "h")]] <- .dm_roll_right(out[[v]], n_steps, mean)
      }
    }
  }

  # rolling sums
  if (length(sum_vars) > 0 && length(roll_hours) > 0) {
    for (v in sum_vars) {
      for (h in roll_hours) {
        n_steps <- max(1L, as.integer(round(h / res_h)))
        if (abs((h / res_h) - n_steps) > 0.05) {
          warning(sprintf(
            "Rolling window %sh for '%s' is not an exact multiple of the inferred resolution (%.6f h); rounded to %s steps.",
            h, v, res_h, n_steps
          ))
        }
        out[[paste0(v, "_rollsum_", h, "h")]] <- .dm_roll_right(out[[v]], n_steps, sum)
      }
    }
  }

  # lags
  if (length(lag_vars) > 0 && length(lag_hours) > 0) {
    for (v in lag_vars) {
      for (h in lag_hours) {
        n_steps <- max(1L, as.integer(round(h / res_h)))
        if (abs((h / res_h) - n_steps) > 0.05) {
          warning(sprintf(
            "Lag window %sh for '%s' is not an exact multiple of the inferred resolution (%.6f h); rounded to %s steps.",
            h, v, res_h, n_steps
          ))
        }
        out[[paste0(v, "_lag_", h, "h")]] <- .dm_lag_vec(out[[v]], n_steps)
      }
    }
  }

  attr(out, "resolution_hours") <- res_h
  class(out) <- c("subdaily_clim", class(out))
  out
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage("Welcome to dendRoAnalyst 0.1.6!\n",
          "To check the changes in this version, check NEWS file.\n",
          "Please see the publication: Aryal, S., Haeusser, M., Griessinger, J., Fan, Z., and Braeuning, A. (2020). 'dendRoAnalyst': A tool for processing and analysing dendrometer data. Dendrochronologia, 64, 125772.")
}

#' @title Dendrometer data from Kathmandu
#' @description Dendrometer data from three Chir pine trees collected in hourly resolution for 2 years.
#' @format A data frame with 14534 rows and 3 variables:
#' \describe{
#'   \item{\code{Time}}{datetime time of data recording}
#'   \item{\code{T2}}{double reading for first tree}
#'   \item{\code{T3}}{double reading for second tree}
#'}
"nepa"

#' Fit dendrometer growth curves by vegetation season
#'
#' @description
#' Fits cumulative growth curves to daily dendrometer series using one of
#' several supported methods: generalized additive model \code{"gam"},
#' Gompertz \code{"gompertz"}, logistic \code{"logistic"}, Richards
#' \code{"richards"}, local regression \code{"loess"}, or smoothing spline
#' \code{"spline"}.
#'
#' The function:
#' \itemize{
#'   \item resamples dendrometer data to daily maxima using [dendro.resample()],
#'   \item assigns each daily observation to a vegetation season,
#'   \item optionally converts daily stem-size values to cumulative growth
#'     with \code{fit_GRO = TRUE},
#'   \item fits one curve per series and season with
#'     \code{year_mode = "yearly"} or one pooled curve across retained seasons
#'     with \code{year_mode = "pooled"},
#'   \item estimates growing-season timing from the fitted cumulative-growth
#'     curve,
#'   \item additionally identifies active-growth timing from the fitted
#'     growth-rate curve.
#' }
#'
#' For Gompertz, logistic, and Richards fits, the asymptote parameter
#' \code{a}, representing maximum seasonal growth, can optionally be fixed to
#' the observed maximum cumulative growth for that series and vegetation season.
#' This is controlled by \code{fix_a_to_observed_max}.
#'
#' @param df A data frame or tibble. The first column must be a timestamp
#'   \code{Date}, \code{POSIXct}, or parseable date-time string. All selected
#'   remaining columns must be numeric dendrometer series.
#' @param TreeNum Either \code{"all"} to use all dendrometer series, a numeric
#'   vector selecting dendrometer columns by position, or a character vector
#'   with series names.
#' @param method Fitting method. One of \code{"gam"}, \code{"gompertz"},
#'   \code{"logistic"}, \code{"richards"}, \code{"loess"}, or \code{"spline"}.
#' @param years Either \code{"all"} to fit all available vegetation seasons, or
#'   a character vector of season labels to retain.
#' @param year_mode Either \code{"yearly"} to fit one curve per vegetation
#'   season, or \code{"pooled"} to fit one pooled curve across all retained
#'   seasons.
#' @param fit_GRO Logical. If \code{TRUE}, processed daily series are converted
#'   to cumulative growth using a cumulative maximum within each vegetation
#'   season.
#' @param site_mode Vegetation season definition. One of \code{"NH"},
#'   \code{"SH"}, or \code{"CS"}.
#' @param custom_veg_season Numeric vector of length two giving the start and
#'   end day-of-year for custom vegetation seasons in \code{"CS"} mode.
#'   Cross-year seasons are supported, for example \code{c(275, 274)}.
#' @param growth_fraction Numeric vector of length two giving the lower and
#'   upper fractions of final fitted seasonal growth used to define cumulative
#'   growth onset and cessation, for example \code{c(0.1, 0.9)}.
#' @param min_unique_growth Minimum number of unique non-missing cumulative
#'   growth values required before a fit is attempted.
#' @param rate_threshold_fraction Numeric scalar between 0 and 1. Active-growth
#'   dates are derived from the fitted growth-rate curve as the first and last
#'   days where fitted growth rate exceeds this fraction of the peak fitted
#'   growth rate.
#' @param fix_a_to_observed_max Logical. If \code{TRUE} and
#'   \code{method = "gompertz"}, \code{"logistic"}, or \code{"richards"}, the
#'   asymptote parameter \code{a}, representing maximum seasonal growth, is fixed
#'   to the observed maximum cumulative growth of the corresponding series and
#'   vegetation season. This is most appropriate with
#'   \code{year_mode = "yearly"}. For \code{year_mode = "pooled"}, one pooled
#'   maximum is used.
#' @param fixed_a_multiplier Numeric multiplier applied to the observed maximum
#'   when \code{fix_a_to_observed_max = TRUE}. The default is \code{1}, meaning
#'   \code{a} is fixed exactly to the observed seasonal maximum. Values slightly
#'   above 1, for example \code{1.01} or \code{1.05}, allow the fitted asymptote
#'   to sit just above the observed maximum.
#' @param start_value_gompertz_parameters Optional named list with starting
#'   values for Gompertz fits. Supported names are \code{a}, \code{b}, and
#'   \code{k}. If \code{fix_a_to_observed_max = TRUE}, supplied \code{a} is
#'   ignored.
#' @param start_value_richards_parameters Optional named list with starting
#'   values for Richards and logistic fits. Supported names are \code{a},
#'   \code{k}, \code{t0}, and \code{v}. If
#'   \code{fix_a_to_observed_max = TRUE}, supplied \code{a} is ignored.
#' @param loess_span Span used when \code{method = "loess"}.
#' @param spline_df Degrees of freedom used when \code{method = "spline"}.
#' @param verbose Logical. If \code{TRUE}, prints a short completion message.
#'
#' @return
#' An object of class \code{"dm_growth_fit"} with elements:
#' \describe{
#'   \item{call}{The matched function call.}
#'   \item{original_daily_data}{Raw daily dendrometer data after resampling to
#'     daily maxima and assigning vegetation seasons, but before centering and
#'     optional cumulative-growth transformation.}
#'   \item{processed_data}{Daily processed data used for fitting.}
#'   \item{fitted_data}{Daily fitted values on the full vegetation-season grid.}
#'   \item{fit_statistics}{Table with fit-level statistics and estimated timing.}
#'   \item{fit_parameters}{Table with fit-level model parameters and convergence
#'     information.}
#'   \item{season_table}{Vegetation seasons retained for fitting.}
#' }
#'
#' @details
#' The first column of \code{df} is treated as the time column and renamed to
#' \code{"TIME"} internally. If it is not already a date-time object, the
#' function attempts to parse it using [lubridate::parse_date_time()].
#'
#' \code{growth_start_*} and \code{growth_end_*} are based on cumulative fitted
#' growth. \code{rate_start_*} and \code{rate_end_*} are based on the fitted
#' growth-rate curve.
#'
#' For Gompertz, logistic, and Richards models, \code{a} is the upper asymptote.
#' When \code{fix_a_to_observed_max = TRUE}, this parameter is not estimated by
#' nonlinear least squares. Instead, it is fixed to:
#'
#' \deqn{a = max(y_{obs}) \times fixed\_a\_multiplier}
#'
#' where \eqn{y_{obs}} is the observed cumulative growth for that series and
#' vegetation season.
#'
#' @examples
#' \donttest{
#' # fit <- dm.growth.fit(
#' #   df = dendro_data,
#' #   TreeNum = "all",
#' #   method = "gompertz",
#' #   year_mode = "yearly",
#' #   fit_GRO = TRUE,
#' #   fix_a_to_observed_max = TRUE
#' # )
#'
#' # fit2 <- dm.growth.fit(
#' #   df = dendro_data,
#' #   TreeNum = "all",
#' #   method = "richards",
#' #   year_mode = "yearly",
#' #   fit_GRO = TRUE,
#' #   fix_a_to_observed_max = TRUE,
#' #   fixed_a_multiplier = 1.02
#' # )
#' }
#'
#' @seealso [summary.dm_growth_fit()], [print.dm_growth_fit()]
#'
#' @importFrom dplyr %>% select all_of any_of bind_cols bind_rows filter arrange
#'   mutate across group_by ungroup distinct left_join summarise
#' @importFrom tibble as_tibble tibble
#' @importFrom lubridate parse_date_time year yday month
#' @importFrom stats lm coef qlogis predict smooth.spline loess na.omit
#' @importFrom utils modifyList head
#' @export
dm.growth.fit <- function(df,
                          TreeNum = "all",
                          method = c("gam", "gompertz", "logistic", "richards", "loess", "spline"),
                          years = "all",
                          year_mode = c("yearly", "pooled"),
                          fit_GRO = TRUE,
                          site_mode = c("NH", "SH", "CS"),
                          custom_veg_season = c(275, 274),
                          growth_fraction = c(0.1, 0.9),
                          min_unique_growth = 10,
                          rate_threshold_fraction = 0.1,
                          fix_a_to_observed_max = FALSE,
                          fixed_a_multiplier = 1,
                          start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_),
                          start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1),
                          loess_span = 0.2,
                          spline_df = 10,
                          verbose = TRUE) {

  cl <- match.call()

  TIME <- doy <- season_label <- season_start <- season_end <- season_day <- season_length <- any_observed <- NULL

  method <- match.arg(method)
  year_mode <- match.arg(year_mode)
  site_mode <- match.arg(site_mode)

  dmgf_validate_fit_growth_inputs(
    df = df,
    growth_fraction = growth_fraction,
    min_unique_growth = min_unique_growth,
    custom_veg_season = custom_veg_season,
    rate_threshold_fraction = rate_threshold_fraction,
    fixed_a_multiplier = fixed_a_multiplier
  )

  df <- tibble::as_tibble(df)
  names(df)[1] <- "TIME"

  if (!inherits(df$TIME, "Date") &&
      !inherits(df$TIME, "POSIXct") &&
      !inherits(df$TIME, "POSIXt")) {
    df$TIME <- suppressWarnings(
      lubridate::parse_date_time(
        df$TIME,
        orders = c(
          "ymd HMS", "ymd HM", "ymd",
          "dmy HMS", "dmy HM", "dmy",
          "mdy HMS", "mdy HM", "mdy"
        ),
        quiet = TRUE
      )
    )
  }

  if (inherits(df$TIME, "Date")) {
    df$TIME <- as.POSIXct(df$TIME)
  }

  if (any(is.na(df$TIME))) {
    stop("Some timestamps in the first column could not be parsed.")
  }

  obs_years <- sort(unique(lubridate::year(as.Date(df$TIME))))

  if (site_mode == "SH" && length(obs_years) < 2) {
    stop("For site_mode = 'SH', df must contain observations from at least two calendar years.")
  }

  if (site_mode == "CS" &&
      custom_veg_season[1] > custom_veg_season[2] &&
      length(obs_years) < 2) {
    stop(
      "For site_mode = 'CS' with custom_veg_season[1] > custom_veg_season[2], ",
      "df must contain observations from at least two calendar years."
    )
  }

  all_series <- names(df)[-1]

  if (length(all_series) == 0) {
    stop("df must contain a time column followed by at least one dendrometer column.")
  }

  if (is.character(TreeNum) && length(TreeNum) == 1 && tolower(TreeNum) == "all") {
    keep_cols <- all_series
  } else if (is.numeric(TreeNum)) {
    tree_idx <- as.integer(TreeNum)

    if (any(is.na(tree_idx)) ||
        any(tree_idx < 1) ||
        any(tree_idx > length(all_series))) {
      stop(
        "'TreeNum' numeric values must be between 1 and ", length(all_series),
        ". Here, 1 refers to the first dendrometer series after the TIME column."
      )
    }

    keep_cols <- all_series[tree_idx]
  } else if (is.character(TreeNum)) {
    missing_cols <- setdiff(TreeNum, all_series)

    if (length(missing_cols) > 0) {
      warning(
        "These TreeNum column names were not found and were ignored: ",
        paste(missing_cols, collapse = ", ")
      )
    }

    keep_cols <- intersect(TreeNum, all_series)

    if (length(keep_cols) == 0) {
      stop("None of the requested TreeNum column names were found in df.")
    }
  } else {
    stop("TreeNum must be 'all', a numeric vector, or a character vector of column names.")
  }

  df <- dplyr::select(df, TIME, dplyr::all_of(keep_cols))

  num_cols <- names(df)[vapply(df, is.numeric, logical(1))]
  num_cols <- setdiff(num_cols, "TIME")

  if (length(num_cols) == 0) {
    stop("No numeric dendrometer series found to fit.")
  }

  df <- dplyr::select(df, TIME, dplyr::all_of(num_cols))

  dm_daily <- dendro.resample(
    df = df,
    by = "D",
    value = "max",
    method = "aggregate"
  )

  season_tbl_obs <- dmgf_build_season_table(
    time_vec = dm_daily$TIME,
    site_mode = site_mode,
    custom_veg_season = custom_veg_season
  )

  observed_daily <- dplyr::bind_cols(dm_daily, season_tbl_obs) %>%
    dplyr::filter(!is.na(season_label)) %>%
    dplyr::arrange(TIME)

  original_daily_data <- observed_daily %>%
    dplyr::select(
      TIME, doy, season_label, season_start, season_end, season_day,
      dplyr::all_of(num_cols)
    )

  season_table <- observed_daily %>%
    dplyr::distinct(season_label, season_start, season_end) %>%
    dplyr::arrange(season_start)

  if (!(is.character(years) && length(years) == 1 && tolower(years) == "all")) {
    season_table <- season_table %>%
      dplyr::filter(season_label %in% as.character(years))
  }

  if (nrow(season_table) == 0) {
    stop("No seasons remained after applying site_mode/custom_veg_season/years.")
  }

  full_grids <- lapply(seq_len(nrow(season_table)), function(i) {
    ss <- season_table$season_start[i]
    se <- season_table$season_end[i]
    sl <- season_table$season_label[i]
    tt <- seq(ss, se, by = "day")

    tibble::tibble(
      TIME = tt,
      doy = lubridate::yday(tt),
      season_label = sl,
      season_start = ss,
      season_end = se,
      season_day = seq_along(tt),
      season_length = length(tt)
    )
  })

  full_grid <- dplyr::bind_rows(full_grids)

  dat <- full_grid %>%
    dplyr::left_join(
      observed_daily %>%
        dplyr::select(TIME, season_label, dplyr::all_of(num_cols)),
      by = c("TIME", "season_label")
    ) %>%
    dplyr::arrange(season_start, TIME)

  dat_proc <- dat %>%
    dplyr::group_by(season_label) %>%
    dplyr::arrange(TIME, .by_group = TRUE) %>%
    dplyr::mutate(
      dplyr::across(
        dplyr::all_of(num_cols),
        ~ . - dmgf_first_non_na(.)
      )
    ) %>%
    dplyr::ungroup()

  if (isTRUE(fit_GRO)) {
    dat_proc <- dat_proc %>%
      dplyr::group_by(season_label) %>%
      dplyr::arrange(TIME, .by_group = TRUE) %>%
      dplyr::mutate(
        dplyr::across(
          dplyr::all_of(num_cols),
          dmgf_cummax_na
        )
      ) %>%
      dplyr::ungroup()
  }

  dat_proc$any_observed <- rowSums(
    !is.na(as.data.frame(dat_proc[, num_cols, drop = FALSE]))
  ) > 0

  fitted_dat <- dat_proc %>%
    dplyr::select(
      TIME, doy, season_label, season_start, season_end,
      season_day, season_length, any_observed
    )

  for (cc in num_cols) {
    fitted_dat[[cc]] <- NA_real_
  }

  param_rows <- list()
  row_id <- 1L

  if (year_mode == "yearly") {
    fit_ids <- unique(dat_proc$season_label)

    for (series_name in num_cols) {
      for (sid in fit_ids) {
        idx <- which(dat_proc$season_label == sid)

        x_full <- dat_proc$season_day[idx]
        y_full <- dat_proc[[series_name]][idx]

        ss <- unique(dat_proc$season_start[idx])[1]
        full_len <- unique(dat_proc$season_length[idx])[1]

        good <- is.finite(y_full)

        x_obs <- x_full[good]
        y_obs <- y_full[good]

        fit_res <- dmgf_fit_one_curve(
          x_obs = x_obs,
          y_obs = y_obs,
          x_pred = x_full,
          season_start_date = ss,
          season_length = full_len,
          season_id = NULL,
          method = method,
          loess_span = loess_span,
          spline_df = spline_df,
          growth_fraction = growth_fraction,
          min_unique_growth = min_unique_growth,
          rate_threshold_fraction = rate_threshold_fraction,
          fix_a_to_observed_max = fix_a_to_observed_max,
          fixed_a_multiplier = fixed_a_multiplier,
          start_value_gompertz_parameters = start_value_gompertz_parameters,
          start_value_richards_parameters = start_value_richards_parameters
        )

        fitted_dat[[series_name]][idx] <- fit_res$pred

        param_rows[[row_id]] <- tibble::tibble(
          series = series_name,
          fit_id = sid,
          year_mode = year_mode,
          method = method,
          site_mode = site_mode,
          season_start = ss,
          season_end = unique(dat_proc$season_end[idx])[1],
          included_seasons = NA_character_,
          season_length = fit_res$params$season_length,
          n_obs = fit_res$params$n_obs,
          n_days_observed = fit_res$params$n_days_observed,
          first_obs_day = fit_res$params$first_obs_day,
          last_obs_day = fit_res$params$last_obs_day,
          n_missing_days = fit_res$params$n_missing_days,
          extrapolated = fit_res$params$extrapolated,
          anchor_added = fit_res$params$anchor_added,
          growth_start_day = fit_res$params$growth_start_day,
          growth_end_day = fit_res$params$growth_end_day,
          growth_start_season_day = fit_res$params$growth_start_season_day,
          growth_end_season_day = fit_res$params$growth_end_season_day,
          growth_start_date = fit_res$params$growth_start_date,
          growth_end_date = fit_res$params$growth_end_date,
          peak_rate = fit_res$params$peak_rate,
          rate_start_day = fit_res$params$rate_start_day,
          rate_end_day = fit_res$params$rate_end_day,
          rate_start_season_day = fit_res$params$rate_start_season_day,
          rate_end_season_day = fit_res$params$rate_end_season_day,
          rate_start_date = fit_res$params$rate_start_date,
          rate_end_date = fit_res$params$rate_end_date,
          converged = fit_res$params$converged,
          fixed_a_used = fit_res$params$fixed_a_used,
          fixed_a_value = fit_res$params$fixed_a_value,
          a = fit_res$params$a,
          b = fit_res$params$b,
          k = fit_res$params$k,
          t0 = fit_res$params$t0,
          v = fit_res$params$v,
          edf = fit_res$params$edf,
          span = fit_res$params$span,
          spline_df = fit_res$params$spline_df,
          spar = fit_res$params$spar,
          model_note = fit_res$params$model_note
        )

        row_id <- row_id + 1L
      }
    }
  }

  if (year_mode == "pooled") {
    included_seasons <- paste(unique(dat_proc$season_label), collapse = ", ")

    for (series_name in num_cols) {
      x_full <- dat_proc$season_day
      y_full <- dat_proc[[series_name]]
      sid_full <- dat_proc$season_label

      good <- is.finite(y_full)

      x_obs <- x_full[good]
      y_obs <- y_full[good]
      sid_obs <- sid_full[good]

      fit_res <- dmgf_fit_one_curve(
        x_obs = x_obs,
        y_obs = y_obs,
        x_pred = x_full,
        season_start_date = as.Date(NA),
        season_length = NA_real_,
        season_id = sid_obs,
        method = method,
        loess_span = loess_span,
        spline_df = spline_df,
        growth_fraction = growth_fraction,
        min_unique_growth = min_unique_growth,
        rate_threshold_fraction = rate_threshold_fraction,
        fix_a_to_observed_max = fix_a_to_observed_max,
        fixed_a_multiplier = fixed_a_multiplier,
        start_value_gompertz_parameters = start_value_gompertz_parameters,
        start_value_richards_parameters = start_value_richards_parameters
      )

      fitted_dat[[series_name]] <- fit_res$pred

      param_rows[[row_id]] <- tibble::tibble(
        series = series_name,
        fit_id = "pooled",
        year_mode = year_mode,
        method = method,
        site_mode = site_mode,
        season_start = as.Date(NA),
        season_end = as.Date(NA),
        included_seasons = included_seasons,
        season_length = fit_res$params$season_length,
        n_obs = fit_res$params$n_obs,
        n_days_observed = fit_res$params$n_days_observed,
        first_obs_day = fit_res$params$first_obs_day,
        last_obs_day = fit_res$params$last_obs_day,
        n_missing_days = fit_res$params$n_missing_days,
        extrapolated = fit_res$params$extrapolated,
        anchor_added = fit_res$params$anchor_added,
        growth_start_day = fit_res$params$growth_start_day,
        growth_end_day = fit_res$params$growth_end_day,
        growth_start_season_day = fit_res$params$growth_start_season_day,
        growth_end_season_day = fit_res$params$growth_end_season_day,
        growth_start_date = fit_res$params$growth_start_date,
        growth_end_date = fit_res$params$growth_end_date,
        peak_rate = fit_res$params$peak_rate,
        rate_start_day = fit_res$params$rate_start_day,
        rate_end_day = fit_res$params$rate_end_day,
        rate_start_season_day = fit_res$params$rate_start_season_day,
        rate_end_season_day = fit_res$params$rate_end_season_day,
        rate_start_date = fit_res$params$rate_start_date,
        rate_end_date = fit_res$params$rate_end_date,
        converged = fit_res$params$converged,
        fixed_a_used = fit_res$params$fixed_a_used,
        fixed_a_value = fit_res$params$fixed_a_value,
        a = fit_res$params$a,
        b = fit_res$params$b,
        k = fit_res$params$k,
        t0 = fit_res$params$t0,
        v = fit_res$params$v,
        edf = fit_res$params$edf,
        span = fit_res$params$span,
        spline_df = fit_res$params$spline_df,
        spar = fit_res$params$spar,
        model_note = fit_res$params$model_note
      )

      row_id <- row_id + 1L
    }
  }

  parameter_table <- dplyr::bind_rows(param_rows)

  stats_cols <- c(
    "series", "fit_id", "year_mode", "method", "site_mode",
    "season_start", "season_end", "included_seasons",
    "season_length", "n_obs", "n_days_observed",
    "first_obs_day", "last_obs_day", "n_missing_days",
    "extrapolated", "anchor_added",
    "growth_start_day", "growth_end_day",
    "growth_start_season_day", "growth_end_season_day",
    "growth_start_date", "growth_end_date",
    "peak_rate",
    "rate_start_day", "rate_end_day",
    "rate_start_season_day", "rate_end_season_day",
    "rate_start_date", "rate_end_date"
  )

  param_cols <- c(
    "series", "fit_id", "year_mode", "method", "site_mode",
    "converged", "fixed_a_used", "fixed_a_value",
    "a", "b", "k", "t0", "v",
    "edf", "span", "spline_df", "spar", "model_note"
  )

  fit_statistics <- parameter_table %>%
    dplyr::select(dplyr::any_of(stats_cols))

  fit_parameters <- parameter_table %>%
    dplyr::select(dplyr::any_of(param_cols))

  out <- list(
    call = cl,
    original_daily_data = original_daily_data,
    processed_data = dat_proc,
    fitted_data = fitted_dat,
    fit_statistics = fit_statistics,
    fit_parameters = fit_parameters,
    season_table = season_table
  )

  class(out) <- "dm_growth_fit"

  if (isTRUE(verbose)) {
    message(
      "dm.growth.fit completed: ",
      nrow(fit_parameters), " fit(s) across ",
      length(unique(fit_parameters$series)), " series."
    )
  }

  out
}


# helpers ----------------------------------------------------------------------

#' Validate inputs for dendrometer growth fitting
#'
#' @param df Input data frame.
#' @param growth_fraction Numeric vector of two growth fractions.
#' @param min_unique_growth Minimum number of unique growth values.
#' @param custom_veg_season Custom vegetation season DOY vector.
#' @param rate_threshold_fraction Fraction of peak rate used for rate window.
#' @param fixed_a_multiplier Multiplier used when fixing asymptote \code{a}.
#'
#' @return Invisibly returns \code{TRUE} if checks pass.
#'
#' @keywords internal
dmgf_validate_fit_growth_inputs <- function(df,
                                            growth_fraction,
                                            min_unique_growth,
                                            custom_veg_season,
                                            rate_threshold_fraction,
                                            fixed_a_multiplier) {
  if (!is.data.frame(df)) {
    stop("df must be a data.frame or tibble.")
  }

  if (ncol(df) < 2) {
    stop("df must contain a time column and at least one dendrometer series.")
  }

  if (!is.numeric(growth_fraction) ||
      length(growth_fraction) != 2 ||
      any(!is.finite(growth_fraction)) ||
      growth_fraction[1] < 0 ||
      growth_fraction[2] > 1 ||
      growth_fraction[1] >= growth_fraction[2]) {
    stop("growth_fraction must be a numeric vector like c(0.1, 0.9).")
  }

  if (!is.numeric(min_unique_growth) ||
      length(min_unique_growth) != 1 ||
      !is.finite(min_unique_growth) ||
      min_unique_growth < 2) {
    stop("min_unique_growth must be a single finite number >= 2.")
  }

  if (!is.numeric(custom_veg_season) ||
      length(custom_veg_season) != 2 ||
      any(!is.finite(custom_veg_season))) {
    stop("custom_veg_season must be a numeric vector of length 2.")
  }

  if (!is.numeric(rate_threshold_fraction) ||
      length(rate_threshold_fraction) != 1 ||
      !is.finite(rate_threshold_fraction) ||
      rate_threshold_fraction <= 0 ||
      rate_threshold_fraction >= 1) {
    stop("rate_threshold_fraction must be a single number between 0 and 1.")
  }

  if (!is.numeric(fixed_a_multiplier) ||
      length(fixed_a_multiplier) != 1 ||
      !is.finite(fixed_a_multiplier) ||
      fixed_a_multiplier <= 0) {
    stop("fixed_a_multiplier must be a single positive finite number.")
  }

  invisible(TRUE)
}


#' Check whether a value is a finite numeric scalar
#'
#' @param x Object to test.
#'
#' @return Logical scalar.
#'
#' @keywords internal
dmgf_is_scalar_finite <- function(x) {
  is.numeric(x) && length(x) == 1 && !is.na(x) && is.finite(x)
}


#' Return the first non-missing value
#'
#' @param x Numeric vector.
#'
#' @return First non-missing value, or \code{NA_real_}.
#'
#' @keywords internal
dmgf_first_non_na <- function(x) {
  idx <- which(!is.na(x))[1]

  if (length(idx) == 0 || is.na(idx)) {
    return(NA_real_)
  }

  x[idx]
}


#' Cumulative maximum while preserving missing values
#'
#' @param x Numeric vector.
#'
#' @return Numeric vector.
#'
#' @keywords internal
dmgf_cummax_na <- function(x) {
  out <- x
  idx <- which(!is.na(x))

  if (length(idx) > 0) {
    out[idx] <- cummax(x[idx])
  }

  out
}


#' Convert year and day-of-year to Date
#'
#' @param year Calendar year.
#' @param doy Day of year.
#'
#' @return Date.
#'
#' @keywords internal
dmgf_doy_to_date <- function(year, doy) {
  as.Date(sprintf("%04d-01-01", year)) + (doy - 1)
}


#' Build vegetation-season table
#'
#' @param time_vec Time vector.
#' @param site_mode One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.
#' @param custom_veg_season Custom vegetation season DOY range.
#'
#' @return Tibble with season labels and season timing.
#'
#' @keywords internal
dmgf_build_season_table <- function(time_vec,
                                    site_mode = c("NH", "SH", "CS"),
                                    custom_veg_season = c(275, 274)) {
  site_mode <- match.arg(site_mode)

  dates <- as.Date(time_vec)
  yr <- lubridate::year(dates)
  doy <- lubridate::yday(dates)
  mo <- lubridate::month(dates)

  n <- length(dates)

  season_label <- rep(NA_character_, n)
  season_start <- as.Date(rep(NA_character_, n))
  season_end <- as.Date(rep(NA_character_, n))
  season_day <- rep(NA_integer_, n)

  if (site_mode == "NH") {
    season_start <- as.Date(sprintf("%04d-01-01", yr))
    season_end <- as.Date(sprintf("%04d-12-31", yr))
    season_label <- as.character(yr)
    season_day <- as.integer(dates - season_start) + 1L
  }

  if (site_mode == "SH") {
    start_year <- ifelse(mo >= 7, yr, yr - 1L)
    end_year <- start_year + 1L

    season_start <- as.Date(sprintf("%04d-07-01", start_year))
    season_end <- as.Date(sprintf("%04d-06-30", end_year))
    season_label <- sprintf("%04d/%04d", start_year, end_year)
    season_day <- as.integer(dates - season_start) + 1L
  }

  if (site_mode == "CS") {
    start_doy <- as.integer(custom_veg_season[1])
    end_doy <- as.integer(custom_veg_season[2])

    if (start_doy <= end_doy) {
      inside <- doy >= start_doy & doy <= end_doy

      season_start[inside] <- dmgf_doy_to_date(yr[inside], start_doy)
      season_end[inside] <- dmgf_doy_to_date(yr[inside], end_doy)
      season_label[inside] <- as.character(yr[inside])
      season_day[inside] <- as.integer(dates[inside] - season_start[inside]) + 1L
    } else {
      inside_late <- doy >= start_doy
      inside_early <- doy <= end_doy

      if (any(inside_late)) {
        sy <- yr[inside_late]
        ey <- sy + 1L

        season_start[inside_late] <- dmgf_doy_to_date(sy, start_doy)
        season_end[inside_late] <- dmgf_doy_to_date(ey, end_doy)
        season_label[inside_late] <- sprintf("%04d/%04d", sy, ey)
        season_day[inside_late] <- as.integer(dates[inside_late] - season_start[inside_late]) + 1L
      }

      if (any(inside_early)) {
        ey <- yr[inside_early]
        sy <- ey - 1L

        season_start[inside_early] <- dmgf_doy_to_date(sy, start_doy)
        season_end[inside_early] <- dmgf_doy_to_date(ey, end_doy)
        season_label[inside_early] <- sprintf("%04d/%04d", sy, ey)
        season_day[inside_early] <- as.integer(dates[inside_early] - season_start[inside_early]) + 1L
      }
    }
  }

  tibble::tibble(
    season_label = season_label,
    season_start = season_start,
    season_end = season_end,
    season_day = season_day
  )
}


#' Add zero-growth anchor points
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param season_id Optional season identifier for pooled fits.
#'
#' @return List with \code{x} and \code{y}.
#'
#' @keywords internal
dmgf_add_anchor_points <- function(x, y, season_id = NULL) {
  if (length(x) == 0) {
    return(list(x = x, y = y))
  }

  if (is.null(season_id)) {
    if (min(x, na.rm = TRUE) > 1 && !any(x == 1)) {
      x <- c(1, x)
      y <- c(0, y)
    }

    ord <- order(x)

    return(list(
      x = x[ord],
      y = y[ord]
    ))
  }

  season_id <- as.character(season_id)

  xs <- list()
  ys <- list()
  uids <- unique(season_id)

  for (i in seq_along(uids)) {
    sid <- uids[i]
    idx <- season_id == sid

    x_i <- x[idx]
    y_i <- y[idx]

    if (length(x_i) == 0) {
      next
    }

    if (min(x_i, na.rm = TRUE) > 1 && !any(x_i == 1)) {
      x_i <- c(1, x_i)
      y_i <- c(0, y_i)
    }

    ord <- order(x_i)

    xs[[i]] <- x_i[ord]
    ys[[i]] <- y_i[ord]
  }

  list(
    x = unlist(xs, use.names = FALSE),
    y = unlist(ys, use.names = FALSE)
  )
}


#' Infer logistic starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial asymptote.
#'
#' @return Named list with \code{a}, \code{k}, and \code{t0}.
#'
#' @keywords internal
dmgf_infer_logistic_starts <- function(x, y, a0) {
  frac <- pmin(pmax(y / a0, 1e-6), 1 - 1e-6)
  z <- stats::qlogis(frac)
  ok <- is.finite(x) & is.finite(z)

  k0 <- 0.03
  t00 <- stats::median(x, na.rm = TRUE)

  if (sum(ok) >= 2) {
    lm_fit <- try(stats::lm(z[ok] ~ x[ok]), silent = TRUE)

    if (!inherits(lm_fit, "try-error")) {
      cf <- stats::coef(lm_fit)

      if (length(cf) == 2 && all(is.finite(cf))) {
        k_try <- abs(unname(cf[2]))

        if (is.finite(k_try) && k_try > 1e-4) {
          k0 <- k_try

          t0_try <- -unname(cf[1]) / k0

          if (is.finite(t0_try)) {
            t00 <- t0_try
          }
        }
      }
    }
  }

  list(
    a = a0,
    k = k0,
    t0 = t00
  )
}


#' Infer Gompertz starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial asymptote.
#'
#' @return Named list with \code{a}, \code{b}, and \code{k}.
#'
#' @keywords internal
dmgf_infer_gompertz_starts <- function(x, y, a0) {
  frac <- pmin(pmax(y / a0, 1e-6), 1 - 1e-6)
  z <- log(-log(frac))
  ok <- is.finite(x) & is.finite(z)

  b0 <- 0.5
  k0 <- 0.01

  if (sum(ok) >= 2) {
    lm_fit <- try(stats::lm(z[ok] ~ x[ok]), silent = TRUE)

    if (!inherits(lm_fit, "try-error")) {
      cf <- stats::coef(lm_fit)

      if (length(cf) == 2 && all(is.finite(cf))) {
        b_try <- unname(cf[1])
        k_try <- -unname(cf[2])

        if (is.finite(b_try)) {
          b0 <- b_try
        }

        if (is.finite(k_try) && k_try > 1e-4) {
          k0 <- k_try
        }
      }
    }
  }

  list(
    a = a0,
    b = b0,
    k = k0
  )
}


#' Empty model-parameter template
#'
#' @return Named list of empty model parameters.
#'
#' @keywords internal
dmgf_empty_model_params <- function() {
  list(
    converged = FALSE,
    fixed_a_used = FALSE,
    fixed_a_value = NA_real_,
    a = NA_real_,
    b = NA_real_,
    k = NA_real_,
    t0 = NA_real_,
    v = NA_real_,
    edf = NA_real_,
    span = NA_real_,
    spline_df = NA_real_,
    spar = NA_real_,
    model_note = NA_character_
  )
}


#' Dispatch growth-curve model fitting
#'
#' @param method Fitting method.
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param loess_span Span for loess.
#' @param spline_df Degrees of freedom for spline.
#' @param extrapolated Logical flag for extrapolation.
#' @param fixed_a_value Optional fixed asymptote.
#' @param fix_a_to_observed_max Logical.
#' @param start_value_gompertz_parameters Starting values for Gompertz.
#' @param start_value_richards_parameters Starting values for logistic/Richards.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model <- function(method,
                           x,
                           y,
                           x_pred,
                           loess_span = 0.2,
                           spline_df = 10,
                           extrapolated = FALSE,
                           fixed_a_value = NA_real_,
                           fix_a_to_observed_max = FALSE,
                           start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_),
                           start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1)) {
  switch(
    method,
    gam = dmgf_fit_model_gam(
      x = x,
      y = y,
      x_pred = x_pred,
      extrapolated = extrapolated
    ),
    gompertz = dmgf_fit_model_gompertz(
      x = x,
      y = y,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_gompertz_parameters = start_value_gompertz_parameters
    ),
    logistic = dmgf_fit_model_logistic(
      x = x,
      y = y,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_richards_parameters = start_value_richards_parameters
    ),
    richards = dmgf_fit_model_richards(
      x = x,
      y = y,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_richards_parameters = start_value_richards_parameters
    ),
    loess = dmgf_fit_model_loess(
      x = x,
      y = y,
      x_pred = x_pred,
      loess_span = loess_span,
      extrapolated = extrapolated
    ),
    spline = dmgf_fit_model_spline(
      x = x,
      y = y,
      x_pred = x_pred,
      spline_df = spline_df,
      extrapolated = extrapolated
    ),
    stop("Unknown method: ", method)
  )
}


#' Fit GAM growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param extrapolated Logical flag for extrapolation.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_gam <- function(x, y, x_pred, extrapolated = FALSE) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    if (!requireNamespace("mgcv", quietly = TRUE)) {
      stop("Package 'mgcv' is required for method = 'gam'. Please install it.")
    }

    x <- as.numeric(x)
    y <- as.numeric(y)
    x_pred <- as.numeric(x_pred)

    dat <- data.frame(x = x, y = y)
    ux <- sort(unique(dat$x))

    if (length(ux) < 4) {
      mod <- stats::lm(y ~ x, data = dat)
      pred <- stats::predict(mod, newdata = data.frame(x = x_pred))
      pred <- as.numeric(pred)
      pred <- pmax(pred, 0)
      pred <- cummax(pred)

      params$converged <- TRUE
      params$model_note <- "lm fallback used because too few unique x values for GAM."

      return(list(pred = pred, params = params))
    }

    k_gam <- min(10L, length(ux) - 1L)
    k_gam <- max(4L, k_gam)

    mod <- mgcv::gam(
      y ~ s(x, bs = "cs", k = k_gam),
      data = dat,
      method = "REML",
      select = TRUE
    )

    pred <- stats::predict(
      mod,
      newdata = data.frame(x = x_pred),
      type = "response"
    )

    pred <- as.numeric(pred)
    pred <- pmax(pred, 0)
    pred <- cummax(pred)

    st <- summary(mod)$s.table

    if (!is.null(st)) {
      st <- as.matrix(st)

      if ("edf" %in% colnames(st)) {
        params$edf <- unname(st[1, "edf"])
      }
    }

    params$converged <- TRUE

    if (isTRUE(extrapolated)) {
      params$model_note <- "GAM predictions outside observed DOY range may be unstable."
    }

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit Gompertz growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param fixed_a_value Optional fixed asymptote value.
#' @param fix_a_to_observed_max Logical.
#' @param start_value_gompertz_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_gompertz <- function(x,
                                    y,
                                    x_pred,
                                    fixed_a_value = NA_real_,
                                    fix_a_to_observed_max = FALSE,
                                    start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_)) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgf_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      start_guess <- dmgf_infer_gompertz_starts(x, y_adj, a_fixed_fit)
      start_guess <- utils::modifyList(start_guess, start_value_gompertz_parameters)

      if (!dmgf_is_scalar_finite(start_guess$b)) {
        start_guess$b <- 0.5
      }

      if (!dmgf_is_scalar_finite(start_guess$k)) {
        start_guess$k <- 0.01
      }

      dat <- data.frame(x = x, y = y_adj)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit * exp(-exp(b - k * x)),
        data = dat,
        start = list(
          b = start_guess$b,
          k = start_guess$k
        ),
        control = minpack.lm::nls.lm.control(maxiter = 500)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- as.numeric(pred)

      cf <- stats::coef(mod)

      params$a <- a_fixed_original
      params$b <- unname(cf["b"])
      params$k <- unname(cf["k"])
      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$model_note <- "Gompertz asymptote a fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    start_guess <- dmgf_infer_gompertz_starts(x, y_adj, a0)
    start_guess <- utils::modifyList(start_guess, start_value_gompertz_parameters)

    if (!dmgf_is_scalar_finite(start_guess$a)) {
      start_guess$a <- a0
    }

    if (!dmgf_is_scalar_finite(start_guess$b)) {
      start_guess$b <- 0.5
    }

    if (!dmgf_is_scalar_finite(start_guess$k)) {
      start_guess$k <- 0.01
    }

    dat <- data.frame(x = x, y = y_adj)

    mod <- minpack.lm::nlsLM(
      y ~ a * exp(-exp(b - k * x)),
      data = dat,
      start = start_guess,
      control = minpack.lm::nls.lm.control(maxiter = 500)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- as.numeric(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$b <- unname(cf["b"])
    params$k <- unname(cf["k"])
    params$converged <- TRUE

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit logistic growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param fixed_a_value Optional fixed asymptote value.
#' @param fix_a_to_observed_max Logical.
#' @param start_value_richards_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_logistic <- function(x,
                                    y,
                                    x_pred,
                                    fixed_a_value = NA_real_,
                                    fix_a_to_observed_max = FALSE,
                                    start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1)) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgf_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      start_guess <- dmgf_infer_logistic_starts(x, y_adj, a_fixed_fit)

      if (dmgf_is_scalar_finite(start_value_richards_parameters$k)) {
        start_guess$k <- start_value_richards_parameters$k
      }

      if (dmgf_is_scalar_finite(start_value_richards_parameters$t0)) {
        start_guess$t0 <- start_value_richards_parameters$t0
      }

      dat <- data.frame(x = x, y = y_adj)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit / (1 + exp(-(k * (x - t0)))),
        data = dat,
        start = list(
          k = start_guess$k,
          t0 = start_guess$t0
        ),
        control = minpack.lm::nls.lm.control(maxiter = 500)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- as.numeric(pred)

      cf <- stats::coef(mod)

      params$a <- a_fixed_original
      params$k <- unname(cf["k"])
      params$t0 <- unname(cf["t0"])
      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$model_note <- "Logistic asymptote a fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    start_guess <- dmgf_infer_logistic_starts(x, y_adj, a0)

    if (dmgf_is_scalar_finite(start_value_richards_parameters$a)) {
      start_guess$a <- start_value_richards_parameters$a
    }

    if (dmgf_is_scalar_finite(start_value_richards_parameters$k)) {
      start_guess$k <- start_value_richards_parameters$k
    }

    if (dmgf_is_scalar_finite(start_value_richards_parameters$t0)) {
      start_guess$t0 <- start_value_richards_parameters$t0
    }

    dat <- data.frame(x = x, y = y_adj)

    mod <- minpack.lm::nlsLM(
      y ~ a / (1 + exp(-(k * (x - t0)))),
      data = dat,
      start = start_guess,
      control = minpack.lm::nls.lm.control(maxiter = 500)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- as.numeric(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$k <- unname(cf["k"])
    params$t0 <- unname(cf["t0"])
    params$converged <- TRUE

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit Richards growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param fixed_a_value Optional fixed asymptote value.
#' @param fix_a_to_observed_max Logical.
#' @param start_value_richards_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_richards <- function(x,
                                    y,
                                    x_pred,
                                    fixed_a_value = NA_real_,
                                    fix_a_to_observed_max = FALSE,
                                    start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1)) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgf_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      logi_start <- dmgf_infer_logistic_starts(x, y_adj, a_fixed_fit)

      start_vals <- list(
        k = if (dmgf_is_scalar_finite(start_value_richards_parameters$k)) {
          start_value_richards_parameters$k
        } else {
          logi_start$k
        },
        t0 = if (dmgf_is_scalar_finite(start_value_richards_parameters$t0)) {
          start_value_richards_parameters$t0
        } else {
          logi_start$t0
        },
        v = if (dmgf_is_scalar_finite(start_value_richards_parameters$v)) {
          start_value_richards_parameters$v
        } else {
          1
        }
      )

      dat <- data.frame(x = x, y = y_adj)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit / ((1 + v * exp(-k * (x - t0)))^(1 / v)),
        data = dat,
        start = start_vals,
        control = minpack.lm::nls.lm.control(maxiter = 500)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- as.numeric(pred)

      cf <- stats::coef(mod)

      params$a <- a_fixed_original
      params$k <- unname(cf["k"])
      params$t0 <- unname(cf["t0"])
      params$v <- unname(cf["v"])
      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$model_note <- "Richards asymptote a fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    logi_start <- dmgf_infer_logistic_starts(x, y_adj, a0)

    start_vals <- list(
      a = if (dmgf_is_scalar_finite(start_value_richards_parameters$a)) {
        start_value_richards_parameters$a
      } else {
        logi_start$a
      },
      k = if (dmgf_is_scalar_finite(start_value_richards_parameters$k)) {
        start_value_richards_parameters$k
      } else {
        logi_start$k
      },
      t0 = if (dmgf_is_scalar_finite(start_value_richards_parameters$t0)) {
        start_value_richards_parameters$t0
      } else {
        logi_start$t0
      },
      v = if (dmgf_is_scalar_finite(start_value_richards_parameters$v)) {
        start_value_richards_parameters$v
      } else {
        1
      }
    )

    dat <- data.frame(x = x, y = y_adj)

    mod <- minpack.lm::nlsLM(
      y ~ a / ((1 + v * exp(-k * (x - t0)))^(1 / v)),
      data = dat,
      start = start_vals,
      control = minpack.lm::nls.lm.control(maxiter = 500)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- as.numeric(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$k <- unname(cf["k"])
    params$t0 <- unname(cf["t0"])
    params$v <- unname(cf["v"])
    params$converged <- TRUE

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit loess growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param loess_span Loess span.
#' @param extrapolated Logical flag for extrapolation.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_loess <- function(x,
                                 y,
                                 x_pred,
                                 loess_span = 0.2,
                                 extrapolated = FALSE) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    dat <- data.frame(x = x, y = y)

    mod <- stats::loess(
      y ~ x,
      data = dat,
      span = loess_span,
      degree = 2
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred))
    pred <- as.numeric(pred)

    params$span <- loess_span
    params$edf <- if (!is.null(mod$enp)) mod$enp else NA_real_
    params$converged <- TRUE

    if (isTRUE(extrapolated)) {
      params$model_note <- "loess extrapolation outside observed DOY range may be unreliable."
    }

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit smoothing-spline growth curve
#'
#' @param x Observed x values.
#' @param y Observed y values.
#' @param x_pred Prediction x values.
#' @param spline_df Spline degrees of freedom.
#' @param extrapolated Logical flag for extrapolation.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgf_fit_model_spline <- function(x,
                                  y,
                                  x_pred,
                                  spline_df = 10,
                                  extrapolated = FALSE) {
  params <- dmgf_empty_model_params()

  res <- tryCatch({
    mod <- stats::smooth.spline(x, y, df = spline_df)
    pred <- stats::predict(mod, x_pred)$y
    pred <- as.numeric(pred)

    params$spline_df <- mod$df
    params$spar <- mod$spar
    params$converged <- TRUE

    if (isTRUE(extrapolated)) {
      params$model_note <- "Spline predictions outside observed DOY range may be uncertain."
    }

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Reduce prediction curve to unique x values
#'
#' @param x_pred Prediction x values.
#' @param pred Predicted y values.
#'
#' @return List with unique \code{x} and mean \code{pred}.
#'
#' @keywords internal
dmgf_unique_prediction_curve <- function(x_pred, pred) {
  ok <- is.finite(x_pred) & is.finite(pred)

  if (sum(ok) == 0) {
    return(list(x = numeric(0), pred = numeric(0)))
  }

  xp <- as.numeric(x_pred[ok])
  pp <- as.numeric(pred[ok])

  ux <- sort(unique(xp))
  mp <- vapply(
    ux,
    function(xx) mean(pp[xp == xx]),
    numeric(1)
  )

  list(
    x = ux,
    pred = mp
  )
}


#' Calculate non-negative fitted growth rate
#'
#' @param pred Fitted cumulative growth.
#'
#' @return Numeric growth-rate vector.
#'
#' @keywords internal
dmgf_growth_rate <- function(pred) {
  pred <- as.numeric(pred)

  if (length(pred) < 2) {
    return(rep(NA_real_, length(pred)))
  }

  pred_mono <- cummax(pmax(pred, 0))
  rate <- c(NA_real_, diff(pred_mono))
  rate[!is.finite(rate)] <- NA_real_
  rate <- pmax(rate, 0)

  rate
}


#' Estimate active-growth window from growth rate
#'
#' @param rate Fitted growth-rate vector.
#' @param x_pred Prediction x values.
#' @param threshold_fraction Fraction of peak growth rate.
#'
#' @return List with peak rate, start day, and end day.
#'
#' @keywords internal
dmgf_rate_window <- function(rate,
                             x_pred,
                             threshold_fraction = 0.1) {
  out <- list(
    peak_rate = NA_real_,
    start_day = NA_real_,
    end_day = NA_real_
  )

  ok <- is.finite(rate) & is.finite(x_pred)

  if (sum(ok) == 0) {
    return(out)
  }

  r <- rate[ok]
  x <- x_pred[ok]

  peak_rate <- suppressWarnings(max(r, na.rm = TRUE))

  if (!is.finite(peak_rate) || peak_rate <= 0) {
    return(out)
  }

  thr <- threshold_fraction * peak_rate
  above <- which(r >= thr)

  if (length(above) == 0) {
    return(out)
  }

  out$peak_rate <- peak_rate
  out$start_day <- min(x[above], na.rm = TRUE)
  out$end_day <- max(x[above], na.rm = TRUE)

  out
}


#' Fit one growth curve
#'
#' @description
#' Internal helper used by [dm.growth.fit()] to fit one curve for one series and
#' one season, or one pooled curve.
#'
#' @param x_obs Observed x values.
#' @param y_obs Observed growth values.
#' @param x_pred Prediction x values.
#' @param season_start_date Season start date.
#' @param season_length Season length.
#' @param season_id Optional season ID for pooled fits.
#' @param method Fitting method.
#' @param loess_span Loess span.
#' @param spline_df Spline degrees of freedom.
#' @param growth_fraction Growth fractions used for cumulative timing.
#' @param min_unique_growth Minimum number of unique growth values.
#' @param rate_threshold_fraction Fraction of peak rate used for rate timing.
#' @param fix_a_to_observed_max Logical.
#' @param fixed_a_multiplier Multiplier for fixed asymptote.
#' @param start_value_gompertz_parameters Starting values for Gompertz.
#' @param start_value_richards_parameters Starting values for logistic/Richards.
#'
#' @return List with fitted predictions and parameters.
#'
#' @keywords internal
dmgf_fit_one_curve <- function(x_obs,
                               y_obs,
                               x_pred,
                               season_start_date = as.Date(NA),
                               season_length = NA_real_,
                               season_id = NULL,
                               method,
                               loess_span,
                               spline_df,
                               growth_fraction,
                               min_unique_growth,
                               rate_threshold_fraction = 0.1,
                               fix_a_to_observed_max = FALSE,
                               fixed_a_multiplier = 1,
                               start_value_gompertz_parameters,
                               start_value_richards_parameters) {

  pred_full <- rep(NA_real_, length(x_pred))

  params <- list(
    converged = FALSE,
    fixed_a_used = FALSE,
    fixed_a_value = NA_real_,
    n_obs = sum(is.finite(x_obs) & is.finite(y_obs)),
    n_days_observed = NA_integer_,
    first_obs_day = NA_real_,
    last_obs_day = NA_real_,
    season_length = season_length,
    n_missing_days = NA_real_,
    extrapolated = NA,
    anchor_added = FALSE,
    a = NA_real_,
    b = NA_real_,
    k = NA_real_,
    t0 = NA_real_,
    v = NA_real_,
    edf = NA_real_,
    span = NA_real_,
    spline_df = NA_real_,
    spar = NA_real_,
    model_note = NA_character_,
    growth_start_day = NA_real_,
    growth_end_day = NA_real_,
    growth_start_season_day = NA_real_,
    growth_end_season_day = NA_real_,
    growth_start_date = as.Date(NA),
    growth_end_date = as.Date(NA),
    peak_rate = NA_real_,
    rate_start_day = NA_real_,
    rate_end_day = NA_real_,
    rate_start_season_day = NA_real_,
    rate_end_season_day = NA_real_,
    rate_start_date = as.Date(NA),
    rate_end_date = as.Date(NA)
  )

  good <- is.finite(x_obs) & is.finite(y_obs)

  if (sum(good) == 0) {
    params$model_note <- "No observed data in this vegetation year."
    return(list(pred = pred_full, params = params))
  }

  x_raw <- x_obs[good]
  y_raw <- y_obs[good]
  sid_raw <- if (is.null(season_id)) NULL else season_id[good]

  params$n_days_observed <- length(x_raw)
  params$first_obs_day <- min(x_raw, na.rm = TRUE)
  params$last_obs_day <- max(x_raw, na.rm = TRUE)

  if (is.finite(season_length)) {
    params$n_missing_days <- season_length - length(unique(x_raw))
    params$extrapolated <- params$first_obs_day > 1 ||
      params$last_obs_day < season_length
  }

  if (length(unique(y_raw)) < min_unique_growth) {
    params$model_note <- paste0(
      "Too few unique cumulative-growth values (",
      length(unique(y_raw)), " < ", min_unique_growth, ")."
    )

    return(list(pred = pred_full, params = params))
  }

  anchored <- dmgf_add_anchor_points(
    x = x_raw,
    y = y_raw,
    season_id = sid_raw
  )

  x_fit <- anchored$x
  y_fit <- anchored$y

  params$anchor_added <- length(x_fit) > length(x_raw)

  dyn_range <- max(y_fit, na.rm = TRUE) - min(y_fit, na.rm = TRUE)

  if (!is.finite(dyn_range) || dyn_range <= 0) {
    params$model_note <- "Observed cumulative-growth range is too small."
    return(list(pred = pred_full, params = params))
  }

  fixed_a_value <- NA_real_

  if (isTRUE(fix_a_to_observed_max) &&
      method %in% c("gompertz", "logistic", "richards")) {
    fixed_a_value <- suppressWarnings(max(y_raw, na.rm = TRUE)) * fixed_a_multiplier

    if (!is.finite(fixed_a_value) || fixed_a_value <= 0) {
      fixed_a_value <- NA_real_
    }
  }

  fit_res <- dmgf_fit_model(
    method = method,
    x = x_fit,
    y = y_fit,
    x_pred = x_pred,
    loess_span = loess_span,
    spline_df = spline_df,
    extrapolated = isTRUE(params$extrapolated),
    fixed_a_value = fixed_a_value,
    fix_a_to_observed_max = fix_a_to_observed_max,
    start_value_gompertz_parameters = start_value_gompertz_parameters,
    start_value_richards_parameters = start_value_richards_parameters
  )

  pred_full <- as.numeric(fit_res$pred)

  params[names(fit_res$params)] <- fit_res$params

  timing_curve <- dmgf_unique_prediction_curve(
    x_pred = x_pred,
    pred = pred_full
  )

  timing_x <- timing_curve$x
  timing_pred <- timing_curve$pred

  if (length(timing_pred) > 0) {
    timing_pred_mono <- cummax(pmax(timing_pred, 0))
    final_val <- tail(stats::na.omit(timing_pred_mono), 1)

    if (length(final_val) == 1 &&
        is.finite(final_val) &&
        final_val > 0) {
      low_val <- final_val * growth_fraction[1]
      high_val <- final_val * growth_fraction[2]

      st_pos <- suppressWarnings(
        min(timing_x[timing_pred_mono > low_val], na.rm = TRUE)
      )

      en_pos <- suppressWarnings(
        max(timing_x[timing_pred_mono < high_val], na.rm = TRUE)
      )

      if (is.finite(st_pos)) {
        params$growth_start_season_day <- st_pos

        if (!is.na(season_start_date)) {
          params$growth_start_date <- season_start_date + (st_pos - 1)
          params$growth_start_day <- lubridate::yday(params$growth_start_date)
        }
      }

      if (is.finite(en_pos)) {
        params$growth_end_season_day <- en_pos

        if (!is.na(season_start_date)) {
          params$growth_end_date <- season_start_date + (en_pos - 1)
          params$growth_end_day <- lubridate::yday(params$growth_end_date)
        }
      }
    }

    rate <- dmgf_growth_rate(timing_pred)

    rate_win <- dmgf_rate_window(
      rate = rate,
      x_pred = timing_x,
      threshold_fraction = rate_threshold_fraction
    )

    params$peak_rate <- rate_win$peak_rate

    if (is.finite(rate_win$start_day)) {
      params$rate_start_season_day <- rate_win$start_day

      if (!is.na(season_start_date)) {
        params$rate_start_date <- season_start_date + (rate_win$start_day - 1)
        params$rate_start_day <- lubridate::yday(params$rate_start_date)
      }
    }

    if (is.finite(rate_win$end_day)) {
      params$rate_end_season_day <- rate_win$end_day

      if (!is.na(season_start_date)) {
        params$rate_end_date <- season_start_date + (rate_win$end_day - 1)
        params$rate_end_day <- lubridate::yday(params$rate_end_date)
      }
    }
  }

  list(
    pred = pred_full,
    params = params
  )
}


# S3 methods -------------------------------------------------------------------

#' Print a dm_growth_fit object
#'
#' @description
#' Prints a compact overview of an object returned by [dm.growth.fit()] or
#' [dm.growth.fit.double()].
#'
#' @param x An object of class \code{"dm_growth_fit"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#'
#' @method print dm_growth_fit
#' @export
print.dm_growth_fit <- function(x, ...) {
  fit_parameters <- x$fit_parameters
  fit_statistics <- x$fit_statistics

  if (is.null(fit_parameters) || nrow(fit_parameters) == 0) {
    cat("<dm_growth_fit>\n")
    cat("No fit results available.\n")
    return(invisible(x))
  }

  methods_used <- if ("method" %in% names(fit_parameters)) {
    unique(stats::na.omit(fit_parameters$method))
  } else {
    character(0)
  }

  n_series <- if ("series" %in% names(fit_parameters)) {
    length(unique(stats::na.omit(fit_parameters$series)))
  } else {
    NA_integer_
  }

  n_fits <- nrow(fit_parameters)

  converged_count <- if ("converged" %in% names(fit_parameters)) {
    sum(fit_parameters$converged %in% TRUE, na.rm = TRUE)
  } else {
    NA_integer_
  }

  fixed_a_count <- if ("fixed_a_used" %in% names(fit_parameters)) {
    sum(fit_parameters$fixed_a_used %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  fallback_count <- if ("fallback_used" %in% names(fit_parameters)) {
    sum(fit_parameters$fallback_used %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  two_pulse_count <- if (!is.null(fit_statistics) &&
                         "two_pulse_detected" %in% names(fit_statistics)) {
    sum(fit_statistics$two_pulse_detected %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  cat("Call:\n")
  print(x$call)
  cat("\n")
  cat("<dm_growth_fit>\n")
  cat("Series:", n_series, "\n")
  cat("Fits:", n_fits, "\n")
  cat("Methods used:", paste(methods_used, collapse = ", "), "\n")
  cat("Converged fits:", converged_count, "\n")
  cat("Fixed-a fits:", fixed_a_count, "\n")

  if ("fallback_used" %in% names(fit_parameters)) {
    cat("Fallback to single curve:", fallback_count, "\n")
  }

  if (!is.null(fit_statistics) &&
      "two_pulse_detected" %in% names(fit_statistics)) {
    cat("True two-pulse fits:", two_pulse_count, "\n")
  }

  invisible(x)
}


#' Summarize a dm_growth_fit object
#'
#' @description
#' Summarizes an object returned by [dm.growth.fit()] or
#' [dm.growth.fit.double()].
#'
#' @param object An object of class \code{"dm_growth_fit"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An object of class \code{"summary.dm_growth_fit"}.
#'
#' @method summary dm_growth_fit
#' @export
summary.dm_growth_fit <- function(object, ...) {
  fit_parameters <- object$fit_parameters
  fit_statistics <- object$fit_statistics
  season_table <- object$season_table

  if (is.null(fit_parameters)) {
    fit_parameters <- tibble::tibble()
  }

  if (is.null(fit_statistics)) {
    fit_statistics <- tibble::tibble()
  }

  if (is.null(season_table)) {
    season_table <- tibble::tibble()
  }

  methods_used <- if ("method" %in% names(fit_parameters)) {
    unique(stats::na.omit(fit_parameters$method))
  } else {
    character(0)
  }

  fixed_a_count <- if ("fixed_a_used" %in% names(fit_parameters)) {
    sum(fit_parameters$fixed_a_used %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  fallback_count <- if ("fallback_used" %in% names(fit_parameters)) {
    sum(fit_parameters$fallback_used %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  nonfallback_count <- if ("fallback_used" %in% names(fit_parameters)) {
    sum(fit_parameters$fallback_used %in% FALSE, na.rm = TRUE)
  } else {
    NA_integer_
  }

  two_pulse_count <- if ("two_pulse_detected" %in% names(fit_statistics)) {
    sum(fit_statistics$two_pulse_detected %in% TRUE, na.rm = TRUE)
  } else {
    0L
  }

  not_two_pulse_count <- if ("two_pulse_detected" %in% names(fit_statistics)) {
    sum(fit_statistics$two_pulse_detected %in% FALSE, na.rm = TRUE)
  } else {
    NA_integer_
  }

  out <- list(
    call = object$call,
    n_series = if ("series" %in% names(fit_parameters)) {
      length(unique(stats::na.omit(fit_parameters$series)))
    } else {
      0L
    },
    n_fits = nrow(fit_parameters),
    methods_used = methods_used,
    site_mode = if ("site_mode" %in% names(fit_parameters)) {
      unique(stats::na.omit(fit_parameters$site_mode))
    } else {
      character(0)
    },
    converged = if ("converged" %in% names(fit_parameters)) {
      sum(fit_parameters$converged %in% TRUE, na.rm = TRUE)
    } else {
      0L
    },
    failed = if ("converged" %in% names(fit_parameters)) {
      sum(fit_parameters$converged %in% FALSE, na.rm = TRUE)
    } else {
      0L
    },
    fixed_a_used = fixed_a_count,
    fallback_used = fallback_count,
    no_fallback = nonfallback_count,
    true_two_pulse_fits = two_pulse_count,
    not_two_pulse_fits = not_two_pulse_count,
    season_table = season_table,
    statistics_head = utils::head(fit_statistics, 10),
    parameter_head = utils::head(fit_parameters, 10),
    fit_statistics = fit_statistics,
    fit_parameters = fit_parameters
  )

  class(out) <- "summary.dm_growth_fit"

  out
}


#' Print a summary.dm_growth_fit object
#'
#' @description
#' Prints a formatted summary of a \code{"summary.dm_growth_fit"} object.
#'
#' @param x An object of class \code{"summary.dm_growth_fit"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#'
#' @method print summary.dm_growth_fit
#' @export
print.summary.dm_growth_fit <- function(x, ...) {
  cat("Call:\n")
  print(x$call)
  cat("\n")
  cat("dm.growth.fit summary\n")
  cat("---------------------\n")
  cat("Number of fitted series:", x$n_series, "\n")
  cat("Number of fits:", x$n_fits, "\n")
  cat("Methods used:", paste(x$methods_used, collapse = ", "), "\n")
  cat("Site mode:", paste(x$site_mode, collapse = ", "), "\n")
  cat("Converged fits:", x$converged, "\n")
  cat("Failed fits:", x$failed, "\n")
  cat("Fixed-a fits:", x$fixed_a_used, "\n")

  if (!is.null(x$fallback_used)) {
    cat("Fits using fallback to single curve:", x$fallback_used, "\n")
  }

  if (!is.null(x$true_two_pulse_fits)) {
    cat("True two-pulse fits:", x$true_two_pulse_fits, "\n")
  }

  cat("\nSeason table:\n")
  print(x$season_table)

  cat("\nFit statistics preview:\n")
  print(x$statistics_head)

  cat("\nParameter overview:\n")
  print(x$parameter_head)

  invisible(x)
}

#' @title Detect Temporal Resolution and Irregularities in Time Series
#'
#' @description
#' Determines the average temporal resolution (in minutes) of a time series vector (e.g., dendrometer timestamps)
#' and detects inconsistencies in time intervals. If the time column is character-formatted, it is automatically
#' converted to POSIXct. Any inconsistent intervals are flagged and printed.
#'
#' @details
#' This function is helpful for checking if a time series (especially dendrometer data) is regularly sampled.
#' It handles both regular and irregular timestamps and gives feedback if the resolution changes.
#'
#' @param input_time A vector of class \code{POSIXct}, \code{Date}, or \code{character}
#' representing time stamps.
#'
#' @return A single integer: the estimated (rounded) average resolution in minutes.
#'         If multiple intervals are detected, a warning and index positions are printed.
#'
#' @examples
#' \dontrun{
#' # Regular 30-minute time sequence
#' time_seq <- seq.POSIXt(from = as.POSIXct("2023-06-01 00:00:00"),
#'                        by = "30 min", length.out = 100)
#' reso_dm(time_seq)  # Should return 30
#'
#' # With character time input
#' time_char <- format(time_seq, format = "%Y-%m-%d %H:%M:%S")
#' reso_dm(time_char)  # Auto converts to POSIXct
#'
#' # Introduce an irregular step
#' time_seq[51] <- time_seq[50] + 60  # One-time 1-hour jump
#' reso_dm(time_seq)  # Should print warning and irregular step index
#' }
#'
#' @export
reso_dm <- function(input_time) {
  # Convert character time to POSIXct
  if (is.character(input_time)) {
    input_time <- lubridate::ymd_hms(input_time, quiet = TRUE)
    if (any(is.na(input_time))) {
      stop("Some timestamps could not be converted to POSIXct. Please check format.")
    }
  }

  # Validate input
  if (!inherits(input_time, "POSIXct") && !inherits(input_time, "Date")) {
    stop("Input must be a POSIXct, Date, or character vector.")
  }

  # Reference time and diff calculation
  reference <- input_time[1]
  time_min <- as.integer(difftime(input_time, reference, units = "mins"))
  diff_time <- diff(time_min)

  unique_steps <- unique(diff_time)
  avg_reso <- mean(diff_time, na.rm = TRUE)

  # Detect irregularities
  irregular_idx <- which(diff_time != stats::median(diff_time, na.rm = TRUE))

  if (length(unique_steps) > 1) {
    warning("Inconsistent time intervals detected in time series.")
    message("Unique step sizes (in minutes): ", paste(sort(unique_steps), collapse = ", "))
    if (length(irregular_idx)) {
      message("Irregular intervals found at positions: ", paste(irregular_idx, collapse = ", "))
    }
  }

  return(round(avg_reso))
}

#' @title Dendrometer data from Kathmandu version 2
#' @description Dendrometer data from three Chir pine trees collected in hourly resolution for 2 years with separated time.
#' @format A data frame with 14534 rows and 8 variables:
#' \describe{
#'   \item{\code{year}}{numeric year of data recording}
#'   \item{\code{month}}{numeric months of data recording}
#'   \item{\code{day}}{numeric days of data recording}
#'   \item{\code{hours}}{numeric hours of data recording}
#'   \item{\code{minutes}}{numeric minutes of data recording}
#'   \item{\code{seconds}}{numeric seconds of data recording}
#'   \item{\code{T2}}{double reading for first tree}
#'   \item{\code{T3}}{double reading for second tree}
#'}
"nepa2"

#' Plot growth-fitting evaluation statistics
#'
#' @description
#' Creates ggplot2-based comparison plots for objects returned by
#' [dm.growth.evaluate()].
#'
#' @param x An object of class \code{"dm_growth_evaluation"}.
#' @param metric Evaluation metric to plot. One of \code{"rmse"},
#'   \code{"mae"}, \code{"bias"}, \code{"abs_bias"}, \code{"r2"},
#'   \code{"correlation"}, \code{"nrmse"}, \code{"rss"},
#'   \code{"aic_approx"}, or \code{"bic_approx"}.
#' @param type Plot type. One of \code{"boxplot"}, \code{"mean"}, or
#'   \code{"heatmap"}.
#' @param order_methods Logical. If \code{TRUE}, methods are ordered by their
#'   average value for the chosen metric.
#' @param decreasing Logical or \code{NULL}. Controls method ordering. If
#'   \code{NULL}, metrics where smaller values are better are ordered ascending,
#'   and metrics where larger values are better are ordered descending.
#' @param show_points Logical. If \code{TRUE}, add individual fit points to
#'   \code{"boxplot"} and \code{"mean"} displays.
#' @param show_errorbar Logical. If \code{TRUE}, show standard-error bars when
#'   \code{type = "mean"}.
#' @param heatmap_label Character string used to label rows in the heatmap. One
#'   of \code{"series_fit"}, \code{"series"}, or \code{"fit_id"}.
#' @param na.rm Logical. If \code{TRUE}, remove missing metric values before
#'   plotting.
#' @param ... Further arguments passed to plotting methods.
#'
#' @return A \code{ggplot} object.
#'
#' @seealso [dm.growth.evaluate()]
#'
#' @method plot dm_growth_evaluation
#' @export
plot.dm_growth_evaluation <- function(x,
                                      metric = c(
                                        "rmse", "mae", "bias", "abs_bias",
                                        "r2", "correlation", "nrmse",
                                        "rss", "aic_approx", "bic_approx"
                                      ),
                                      type = c("boxplot", "mean", "heatmap"),
                                      order_methods = TRUE,
                                      decreasing = NULL,
                                      show_points = TRUE,
                                      show_errorbar = TRUE,
                                      heatmap_label = c("series_fit", "series", "fit_id"),
                                      na.rm = TRUE,
                                      ...) {
  avg_metric <- mean_metric <- method <- metric_value <- n <- row_label <- NULL
  sd_metric <- se_metric <- NULL
  metric <- match.arg(metric)
  type <- match.arg(type)
  heatmap_label <- match.arg(heatmap_label)

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_growth_evaluation().")
  }

  dat <- tibble::as_tibble(x)

  required_cols <- c("method", "series", "fit_id")
  missing_required <- setdiff(required_cols, names(dat))
  if (length(missing_required) > 0) {
    stop(
      "The evaluation table is missing required columns: ",
      paste(missing_required, collapse = ", "), "."
    )
  }

  if (metric == "abs_bias") {
    if (!"bias" %in% names(dat)) {
      stop("Column 'bias' is required to compute 'abs_bias'.")
    }
    dat$metric_value <- abs(dat$bias)
    metric_label <- "Absolute bias"
  } else {
    if (!metric %in% names(dat)) {
      stop("Column '", metric, "' not found in evaluation table.")
    }
    dat$metric_value <- dat[[metric]]
    metric_label <- switch(
      metric,
      rmse = "RMSE",
      mae = "MAE",
      bias = "Bias",
      r2 = "R2",
      correlation = "Correlation",
      nrmse = "NRMSE",
      rss = "RSS",
      aic_approx = "Approx. AIC",
      bic_approx = "Approx. BIC",
      metric
    )
  }

  if (isTRUE(na.rm)) {
    dat <- dat[is.finite(dat$metric_value), , drop = FALSE]
  }

  if (nrow(dat) == 0) {
    stop("No finite values available for metric '", metric, "'.")
  }

  smaller_is_better <- metric %in% c("rmse", "mae", "abs_bias", "nrmse", "rss", "aic_approx", "bic_approx")

  if (is.null(decreasing)) {
    decreasing <- !smaller_is_better
  }

  if (isTRUE(order_methods)) {
    method_order <- dat %>%
      dplyr::group_by(method) %>%
      dplyr::summarise(avg_metric = mean(metric_value, na.rm = TRUE), .groups = "drop") %>%
      dplyr::arrange(if (isTRUE(decreasing)) dplyr::desc(avg_metric) else avg_metric) %>%
      dplyr::pull(method)

    dat$method <- factor(dat$method, levels = method_order)
  }

  if (type == "boxplot") {
    p <- ggplot2::ggplot(dat, ggplot2::aes(x = method, y = metric_value)) +
      ggplot2::geom_boxplot(outlier.shape = NA) +
      ggplot2::labs(
        x = "Method",
        y = metric_label,
        title = paste("Comparison of", metric, "across growth-fitting methods")
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      )

    if (isTRUE(show_points)) {
      p <- p + ggplot2::geom_jitter(width = 0.15, height = 0, alpha = 0.7, size = 1.8)
    }

    return(p)
  }

  if (type == "mean") {
    sum_dat <- dat %>%
      dplyr::group_by(method) %>%
      dplyr::summarise(
        n = sum(is.finite(metric_value)),
        mean_metric = mean(metric_value, na.rm = TRUE),
        sd_metric = stats::sd(metric_value, na.rm = TRUE),
        se_metric = sd_metric / sqrt(n),
        .groups = "drop"
      )

    if (isTRUE(order_methods)) {
      sum_dat$method <- factor(sum_dat$method, levels = levels(dat$method))
    }

    p <- ggplot2::ggplot(sum_dat, ggplot2::aes(x = method, y = mean_metric)) +
      ggplot2::geom_col() +
      ggplot2::labs(
        x = "Method",
        y = paste("Mean", metric_label),
        title = paste("Mean", metric, "by growth-fitting method")
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      )

    if (isTRUE(show_errorbar)) {
      p <- p + ggplot2::geom_errorbar(
        ggplot2::aes(
          ymin = mean_metric - se_metric,
          ymax = mean_metric + se_metric
        ),
        width = 0.2
      )
    }

    if (isTRUE(show_points)) {
      p <- p + ggplot2::geom_point(
        data = dat,
        mapping = ggplot2::aes(x = method, y = metric_value),
        inherit.aes = FALSE,
        alpha = 0.5,
        position = ggplot2::position_jitter(width = 0.12, height = 0)
      )
    }

    return(p)
  }

  if (type == "heatmap") {
    dat$row_label <- switch(
      heatmap_label,
      series_fit = paste(dat$series, dat$fit_id, sep = " | "),
      series = dat$series,
      fit_id = dat$fit_id
    )

    row_levels <- dat %>%
      dplyr::group_by(row_label) %>%
      dplyr::summarise(avg_metric = mean(metric_value, na.rm = TRUE), .groups = "drop") %>%
      dplyr::arrange(if (isTRUE(decreasing)) dplyr::desc(avg_metric) else avg_metric) %>%
      dplyr::pull(row_label)

    dat$row_label <- factor(dat$row_label, levels = row_levels)

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = method, y = row_label, fill = metric_value)
    ) +
      ggplot2::geom_tile() +
      ggplot2::labs(
        x = "Method",
        y = switch(
          heatmap_label,
          series_fit = "Series | Fit",
          series = "Series",
          fit_id = "Fit ID"
        ),
        fill = metric_label,
        title = paste("Heatmap of", metric, "across growth-fitting methods")
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      )

    return(p)
  }

  stop("Unknown plot type.")
}

#' @title Plot method for zero-growth output
#'
#' @description
#' Unified S3 plotting method for objects returned by \code{phase.zg()}.
#' It supports time-series views of GRO and TWD, phase ribbons and transitions,
#' TWD-only plots, ABr summaries, phase-balance views, and boxplots of
#' phase-level statistics. For boxplots, phase summaries are first assembled to
#' the requested daily or monthly time scale, and rows with missing x-axis
#' grouping labels are removed before plotting.
#'
#' The plotting method operates on the two tables returned by
#' \code{phase.zg()}:
#' \itemize{
#'   \item \code{ZG_phase}: point-level time series with \code{TIME},
#'         \code{dm}, \code{Phases}, \code{TWD}, and \code{GRO};
#'   \item \code{ZG_cycle}: phase-level summaries including
#'         \code{max.twd}, \code{AUC.load}, \code{AUC.total}, and
#'         \code{ABr.value}.
#' }
#'
#' @details
#' \strong{ABr formula shown by this plot method.}
#'
#' When \code{type = "abr"} or when \code{stat = "ABr.value"} is used in
#' boxplots, the displayed values come directly from \code{phase.zg()}:
#'
#' \deqn{
#'   ABr = \max(TWD) \times \left(\frac{AUC_{load}}{\max(TWD)}\right)^{\beta}
#' }
#'
#' where
#'
#' \deqn{AUC_{load} = \int_{t_{start}}^{t_{max}} TWD(t)\,dt}
#'
#' and
#'
#' \deqn{AUC_{total} = \int_{t_{start}}^{t_{end}} TWD(t)\,dt.}
#'
#' This function does not recompute ABr or AUC values; it only visualizes
#' the values already stored in the \code{ZG_output} object.
#'
#' \strong{Aggregation behavior.}
#'
#' For \code{temporal = "daily"} or \code{"monthly"}, point-level series
#' are aggregated before plotting. TWD and GRO can be summarized using
#' \code{"mean"} or \code{"max"}, depending on \code{twd_fun} and
#' \code{gro_fun}.
#'
#' For phase-level summaries in \code{ZG_cycle}, the plot method aggregates
#' statistics by daily or monthly period. In those aggregated summaries:
#' \itemize{
#'   \item \code{Duration_h}, \code{Magnitude}, \code{AUC.load},
#'         \code{AUC.total}, and \code{ABr.value} are summed;
#'   \item \code{max.twd} is taken as the maximum;
#'   \item \code{Avg.twd} and \code{STD.twd} are averaged;
#'   \item \code{rate} is recalculated from aggregated magnitude and duration.
#' }
#'
#' \strong{Phase balance.}
#'
#' For \code{type = "balance"}, two balance styles are available through
#' \code{balance_mode}. With \code{balance_mode = "time_of_day"}, the plot
#' draws TWD and GRO intervals on a 0--24 hour y-axis, preserving the order in
#' which phases occurred within each day, similar to the balance option in
#' \code{plot.SC_output()}. With \code{balance_mode = "duration"}, the plot
#' draws stacked bars showing the total number of hours spent in each phase.
#' Missing or irregular intervals can be handled with \code{balance_gap}.
#'
#' \strong{Boxplot statistics.}
#'
#' The following phase-level variables can be visualized in
#' \code{type = "boxplot"}. For \code{box_group = "period"}, boxplots are
#' drawn on a continuous date axis spanning the selected multi-annual data
#' window, so empty days or months remain visible as gaps rather than being
#' removed from the x-axis:
#' \itemize{
#'   \item \code{"Duration_h"}
#'   \item \code{"Magnitude"}
#'   \item \code{"rate"}
#'   \item \code{"max.twd"}
#'   \item \code{"Avg.twd"}
#'   \item \code{"STD.twd"}
#'   \item \code{"AUC.load"}
#'   \item \code{"AUC.total"}
#'   \item \code{"ABr.value"}
#' }
#'
#' \strong{Phase handling.}
#'
#' For \code{phase = "auto"}, TWD-specific statistics
#' (\code{max.twd}, \code{Avg.twd}, \code{STD.twd}, \code{AUC.load},
#' \code{AUC.total}, \code{ABr.value}) automatically select TWD phases,
#' whereas GRO-specific variables (\code{Magnitude}, \code{rate}) select GRO
#' phases.
#'
#' @param x Object of class \code{"ZG_output"} returned by \code{phase.zg()}.
#' @param y Unused.
#' @param DOY Optional numeric vector of length 2 giving start and end
#'   day-of-year for truncating the plotting window.
#' @param Year Optional numeric year used together with \code{DOY}.
#' @param view Optional backward-compatible argument. Use \code{"boxplot"}
#'   to force boxplot mode.
#' @param type Plot type. One of \code{"gro_twd"}, \code{"phase_ribbon"},
#'   \code{"transition"}, \code{"twd"}, \code{"abr"},
#'   \code{"phase_summary"}, \code{"balance"}, or \code{"boxplot"}.
#' @param temporal Temporal scale. One of \code{"raw"}, \code{"daily"},
#'   or \code{"monthly"}.
#' @param x_axis X-axis style for time-based plots. One of \code{"time"} or
#'   \code{"doy"}. When \code{"doy"} is used, timestamps are converted to
#'   day-of-year. For \code{temporal = "monthly"}, \code{"doy"} is not
#'   meaningful and time is used instead.
#' @param stat For \code{type = "boxplot"}, one of
#'   \code{"Duration_h"}, \code{"Magnitude"}, \code{"rate"},
#'   \code{"max.twd"}, \code{"Avg.twd"}, \code{"STD.twd"},
#'   \code{"AUC.load"}, \code{"AUC.total"}, or \code{"ABr.value"}.
#' @param phase For \code{type = "boxplot"}, one of \code{"auto"},
#'   \code{"TWD"}, \code{"GRO"}, or \code{"all"}.
#' @param box_group For \code{type = "boxplot"}, grouping mode:
#'   \code{"period"}, \code{"month_of_year"}, or \code{"doy"}.
#'   \code{"period"} uses exact dates/months from the already aggregated
#'   daily or monthly phase summaries; the other two pool values across
#'   repeated seasonal positions. Rows with missing grouping labels are
#'   removed before plotting.
#' @param twd_fun Aggregation function for point-level TWD when
#'   \code{temporal != "raw"}. One of \code{"mean"} or \code{"max"}.
#' @param gro_fun Aggregation function for point-level GRO when
#'   \code{temporal != "raw"}. One of \code{"max"} or \code{"mean"}.
#' @param transition_linetype Line type used for transition markers.
#' @param transition_alpha Transparency of transition markers.
#' @param cols Vector of two colours for TWD and GRO. It may be named
#'   (\code{c(TWD = "red", GRO = "blue")}) or unnamed length 2.
#' @param singleton_as_points Logical. If \code{TRUE}, the boxplot panel
#'   falls back to a point plot when every group contains only one value.
#' @param balance_mode For \code{type = "balance"}, one of
#'   \code{"time_of_day"} or \code{"duration"}. \code{"time_of_day"} draws
#'   continuous TWD/GRO intervals along the y-axis from 0 to 24 h, preserving
#'   when phases occurred within each day. \code{"duration"} draws stacked bars
#'   of total hours spent in each phase per day or month.
#' @param balance_gap For \code{type = "balance"}, one of
#'   \code{"carry_forward"} or \code{"observed_only"}.
#'   \code{"carry_forward"} assigns gaps to the previous observed phase;
#'   \code{"observed_only"} counts only observed intervals.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' # data(gf_nepa17)
#' # zg <- phase.zg(df = gf_nepa17[1:500, ], TreeNum = 1, beta = 0.1)
#'
#' # Raw GRO and TWD time series
#' # plot(zg)
#'
#' # Daily aggregated series
#' # plot(zg, temporal = "daily")
#'
#' # Monthly TWD plot
#' # plot(zg, temporal = "monthly", type = "twd")
#'
#' # ABr bar plot
#' # plot(zg, temporal = "monthly", type = "abr")
#'
#' # Daily phase timing, similar to plot.SC_output balance
#' # plot(zg, type = "balance", temporal = "daily",
#' #      balance_mode = "time_of_day")
#'
#' # Stacked phase-duration balance
#' # plot(zg, type = "balance", temporal = "daily",
#' #      balance_mode = "duration")
#'
#' # Transition plot in day-of-year coordinates
#' # plot(zg, type = "transition", x_axis = "doy",
#' #      DOY = c(50, 100), Year = 2017)
#'
#' # Boxplots of peak TWD
#' # plot(zg, type = "boxplot", stat = "max.twd",
#' #      temporal = "daily", box_group = "doy")
#'
#' # Boxplots of AUC.load
#' # plot(zg, type = "boxplot", stat = "AUC.load",
#' #      temporal = "monthly", box_group = "month_of_year")
#'
#' # Boxplots of ABr.value
#' # plot(zg, type = "boxplot", stat = "ABr.value",
#' #      temporal = "monthly", box_group = "month_of_year")
#' # plot(zg, view = "boxplot", stat = "ABr.value", temporal = "monthly")
#' }
#'
#' @method plot ZG_output
#' @importFrom dplyr %>% mutate rename group_by summarise arrange if_else filter bind_rows
#' @importFrom tidyr pivot_longer
#' @importFrom tibble tibble
#' @importFrom lubridate floor_date yday month
#' @importFrom ggplot2 ggplot aes geom_line geom_area geom_rect geom_point
#'   geom_col geom_boxplot geom_vline facet_wrap theme_minimal theme theme_bw
#'   element_text labs scale_color_manual scale_fill_manual position_jitter
#'   scale_x_date scale_x_discrete scale_y_continuous coord_cartesian
#' @export
plot.ZG_output <- function(x,
                           y = NULL,
                           DOY = NULL,
                           Year = NULL,
                           view = NULL,
                           type = c("gro_twd", "phase_ribbon", "transition",
                                    "twd", "abr", "phase_summary",
                                    "balance", "boxplot"),
                           temporal = c("raw", "daily", "monthly"),
                           x_axis = c("time", "doy"),
                           stat = c("Duration_h", "Magnitude", "rate",
                                    "max.twd", "Avg.twd", "STD.twd",
                                    "AUC.load", "AUC.total", "ABr.value"),
                           phase = c("auto", "TWD", "GRO", "all"),
                           box_group = c("period", "month_of_year", "doy"),
                           twd_fun = c("mean", "max"),
                           gro_fun = c("max", "mean"),
                           transition_linetype = "dashed",
                           transition_alpha = 0.55,
                           cols = c(TWD = "red", GRO = "blue"),
                           singleton_as_points = TRUE,
                           balance_mode = c("time_of_day", "duration"),
                           balance_gap = c("carry_forward", "observed_only"),
                           ...) {

  TIME <- dm <- GRO <- TWD <- Variable <- Value <- NULL
  Start <- End <- Duration_h <- Magnitude <- rate <- max.twd <- Max.twd.time <- NULL
  ABr.value <- Avg.twd <- STD.twd <- AUC.load <- AUC.total <- period <- Phase <- NULL
  group_label <- value <- hours <- Phases <- Metric <- xmin <- xmax <- X <- NULL
  group_date <- seg_start <- seg_end <- day <- ymin <- ymax <- unit <- total_hours <- NULL

  if (!inherits(x, "ZG_output")) {
    stop("Input must be an object of class 'ZG_output'.", call. = FALSE)
  }

  if (is.null(names(cols))) {
    if (length(cols) != 2) {
      stop(
        "'cols' must be a named vector with TWD and GRO, ",
        "or an unnamed vector of length 2.",
        call. = FALSE
      )
    }
    names(cols) <- c("TWD", "GRO")
  }

  if (!all(c("TWD", "GRO") %in% names(cols))) {
    stop("'cols' must contain names 'TWD' and 'GRO'.", call. = FALSE)
  }

  if (!is.null(view)) {
    if (!view %in% c("series", "boxplot")) {
      stop("'view' must be either 'series' or 'boxplot'.", call. = FALSE)
    }
    if (view == "boxplot") {
      type <- "boxplot"
    }
  }

  type <- match.arg(type)
  temporal <- match.arg(temporal)
  x_axis <- match.arg(x_axis)
  stat <- match.arg(stat)
  phase <- match.arg(phase)
  box_group <- match.arg(box_group)
  twd_fun <- match.arg(twd_fun)
  gro_fun <- match.arg(gro_fun)
  balance_mode <- match.arg(balance_mode)
  balance_gap <- match.arg(balance_gap)

  if (type == "boxplot" && temporal == "raw") {
    stop("For type = 'boxplot', temporal must be 'daily' or 'monthly'.", call. = FALSE)
  }

  .time_unit <- function(x) {
    switch(
      x,
      raw = NA_character_,
      daily = "day",
      monthly = "month"
    )
  }

  .next_period_end <- function(period_start, unit) {
    if (unit == "month") {
      seq(period_start, length.out = 2, by = "1 month")[2]
    } else {
      seq(period_start, length.out = 2, by = "1 day")[2]
    }
  }

  .x_values <- function(tt, temporal, x_axis) {
    if (x_axis == "time") {
      return(list(x = tt, xlab = "Time"))
    }

    if (temporal == "monthly") {
      warning(
        "x_axis = 'doy' is not meaningful for temporal = 'monthly'; using time instead.",
        call. = FALSE
      )
      return(list(x = tt, xlab = "Time"))
    }

    list(
      x = lubridate::yday(tt),
      xlab = "Day of year"
    )
  }

  .phase_factor <- function(x) {
    factor(x, levels = c(1, 2), labels = c("TWD", "GRO"))
  }

  .safe_sum <- function(z) {
    if (all(is.na(z))) NA_real_ else sum(z, na.rm = TRUE)
  }

  .safe_mean <- function(z) {
    if (all(is.na(z))) NA_real_ else mean(z, na.rm = TRUE)
  }

  .safe_max <- function(z) {
    if (all(is.na(z))) NA_real_ else max(z, na.rm = TRUE)
  }

  .safe_time_of_max <- function(values, times) {
    if (all(is.na(values))) {
      return(as.POSIXct(NA, tz = attr(times, "tzone")))
    }
    idx <- which.max(replace(values, is.na(values), -Inf))
    times[idx][1]
  }

  .dominant_phase <- function(z) {
    if (sum(z == 2L, na.rm = TRUE) >= sum(z == 1L, na.rm = TRUE)) {
      2L
    } else {
      1L
    }
  }

  .hour_decimal <- function(tt) {
    day_start <- lubridate::floor_date(tt, unit = "day")
    as.numeric(difftime(tt, day_start, units = "hours"))
  }

  .truncate_zg <- function(obj, DOY = NULL, Year = NULL) {
    if (is.null(DOY) && is.null(Year)) {
      td <- obj$ZG_phase %>%
        dplyr::rename(TIME = 1, dm = 2, Phases = 3, TWD = 4, GRO = 5)
    } else {
      if (is.null(DOY) || is.null(Year) || length(DOY) != 2) {
        stop("Provide both DOY = c(start, end) and Year, or neither.", call. = FALSE)
      }

      td <- dendro.truncate(obj$ZG_phase, CalYear = Year, DOY = DOY) %>%
        dplyr::rename(TIME = 1, dm = 2, Phases = 3, TWD = 4, GRO = 5)
    }

    if (nrow(td) == 0) {
      stop("No data available for the requested plotting period.", call. = FALSE)
    }

    cyc <- obj$ZG_cycle %>%
      dplyr::filter(
        End >= min(td$TIME, na.rm = TRUE),
        Start <= max(td$TIME, na.rm = TRUE)
      )

    list(td = td, cyc = cyc)
  }

  .aggregate_points <- function(td, temporal, twd_fun, gro_fun) {
    if (temporal == "raw") {
      td$Phase <- .phase_factor(td$Phases)
      return(td)
    }

    unit <- .time_unit(temporal)

    agg_twd <- function(z) {
      if (all(is.na(z))) {
        return(NA_real_)
      }
      if (twd_fun == "mean") {
        mean(z, na.rm = TRUE)
      } else {
        max(z, na.rm = TRUE)
      }
    }

    agg_gro <- function(z) {
      if (all(is.na(z))) {
        return(NA_real_)
      }
      if (gro_fun == "max") {
        max(z, na.rm = TRUE)
      } else {
        mean(z, na.rm = TRUE)
      }
    }

    td %>%
      dplyr::mutate(period = lubridate::floor_date(TIME, unit = unit)) %>%
      dplyr::group_by(period) %>%
      dplyr::summarise(
        dm = mean(dm, na.rm = TRUE),
        GRO = agg_gro(GRO),
        TWD = agg_twd(TWD),
        Phases = .dominant_phase(Phases),
        .groups = "drop"
      ) %>%
      dplyr::rename(TIME = period) %>%
      dplyr::mutate(Phase = .phase_factor(Phases))
  }

  .aggregate_cycles <- function(cyc, temporal) {
    if (temporal == "raw") {
      return(cyc)
    }

    unit <- .time_unit(temporal)

    cyc %>%
      dplyr::mutate(period = lubridate::floor_date(Start, unit = unit)) %>%
      dplyr::group_by(period, Phases) %>%
      dplyr::summarise(
        Start = min(Start, na.rm = TRUE),
        End = max(End, na.rm = TRUE),
        Duration_h = sum(Duration_h, na.rm = TRUE),
        Magnitude = .safe_sum(Magnitude),
        rate = {
          mag <- .safe_sum(Magnitude)
          dur <- sum(Duration_h, na.rm = TRUE)
          if (is.na(mag) || dur == 0) {
            NA_real_
          } else {
            mag * 1000 / dur
          }
        },
        max.twd = .safe_max(max.twd),
        Max.twd.time = .safe_time_of_max(max.twd, Max.twd.time),
        AUC.load = .safe_sum(AUC.load),
        AUC.total = .safe_sum(AUC.total),
        ABr.value = .safe_sum(ABr.value),
        Avg.twd = .safe_mean(Avg.twd),
        STD.twd = .safe_mean(STD.twd),
        DOY = lubridate::yday(min(Start, na.rm = TRUE)),
        .groups = "drop"
      )
  }

  .balance_segments <- function(td_raw, temporal, gap_mode = "carry_forward") {
    unit <- if (temporal == "monthly") "month" else "day"

    td_i <- td_raw %>%
      dplyr::filter(!is.na(TIME), !is.na(Phases)) %>%
      dplyr::arrange(TIME)

    if (nrow(td_i) < 2) {
      stop("At least two time steps are required to calculate phase balance.", call. = FALSE)
    }

    diffs_sec <- as.numeric(diff(td_i$TIME), units = "secs")
    diffs_sec <- diffs_sec[is.finite(diffs_sec) & diffs_sec > 0]

    if (length(diffs_sec) == 0) {
      stop("Could not determine the temporal resolution of the phase series.", call. = FALSE)
    }

    res_sec <- stats::median(diffs_sec, na.rm = TRUE)

    td_i$interval_start <- td_i$TIME
    td_i$interval_end <- dplyr::lead(td_i$TIME)
    td_i$interval_end[nrow(td_i)] <- td_i$TIME[nrow(td_i)] + res_sec

    if (gap_mode == "observed_only") {
      interval_sec <- as.numeric(
        difftime(td_i$interval_end, td_i$interval_start, units = "secs")
      )
      long_gap <- is.finite(interval_sec) & interval_sec > res_sec * 1.5
      td_i$interval_end[long_gap] <- td_i$interval_start[long_gap] + res_sec
    }

    rows <- list()
    k <- 0L

    for (i in seq_len(nrow(td_i))) {
      st <- td_i$interval_start[i]
      en <- td_i$interval_end[i]
      ph <- td_i$Phases[i]

      if (is.na(st) || is.na(en) || is.na(ph) || en <= st) {
        next
      }

      while (st < en) {
        period_start <- lubridate::floor_date(st, unit = unit)
        period_end <- .next_period_end(period_start, unit = unit)

        seg_end <- if (en < period_end) {
          en
        } else {
          period_end
        }

        if (seg_end <= st) {
          break
        }

        k <- k + 1L
        rows[[k]] <- tibble::tibble(
          period = period_start,
          seg_start = st,
          seg_end = seg_end,
          Phases = ph,
          hours = as.numeric(difftime(seg_end, st, units = "hours"))
        )

        st <- seg_end
      }
    }

    if (length(rows) == 0) {
      stop("No valid phase-duration segments could be calculated.", call. = FALSE)
    }

    dplyr::bind_rows(rows) %>%
      dplyr::mutate(
        Phase = .phase_factor(Phases),
        unit = unit
      )
  }

  out <- .truncate_zg(x, DOY = DOY, Year = Year)
  td_raw <- out$td
  cyc_raw <- out$cyc

  td <- .aggregate_points(
    td_raw,
    temporal = temporal,
    twd_fun = twd_fun,
    gro_fun = gro_fun
  )

  cyc <- .aggregate_cycles(cyc_raw, temporal = temporal)

  if (type %in% c("gro_twd", "phase_ribbon", "transition", "twd")) {
    td_plot <- td
  }

  if (type == "gro_twd") {
    data_long <- tidyr::pivot_longer(
      td_plot,
      cols = c(GRO, TWD),
      names_to = "Variable",
      values_to = "Value"
    ) %>%
      dplyr::mutate(
        Value = dplyr::if_else(Variable == "TWD", -Value, Value),
        Variable = factor(Variable, levels = c("GRO", "TWD"))
      ) %>%
      dplyr::arrange(TIME)

    xv <- .x_values(data_long$TIME, temporal, x_axis)
    data_long$X <- xv$x
    data_long <- dplyr::filter(data_long, !is.na(X))

    p <- ggplot2::ggplot(data_long, ggplot2::aes(x = X, y = Value)) +
      ggplot2::geom_line(
        data = dplyr::filter(data_long, Variable == "GRO"),
        color = unname(cols["GRO"]),
        linewidth = 0.5
      ) +
      ggplot2::geom_area(
        data = dplyr::filter(data_long, Variable == "TWD"),
        fill = unname(cols["TWD"]),
        alpha = 0.5
      ) +
      ggplot2::facet_wrap(~ Variable, scales = "free_y", ncol = 1) +
      ggplot2::labs(
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Value"
        } else {
          paste0("Value (", temporal, ")")
        }
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "phase_ribbon") {
    td_r <- td_plot %>% dplyr::filter(!is.na(Phases))
    xv <- .x_values(td_r$TIME, temporal, x_axis)
    td_r$X <- xv$x
    td_r <- dplyr::filter(td_r, !is.na(X))

    rr <- rle(td_r$Phases)
    idx_end <- cumsum(rr$lengths)
    idx_start <- c(1, head(idx_end + 1, -1))

    ribbons <- tibble::tibble(
      xmin = td_r$X[idx_start],
      xmax = td_r$X[idx_end],
      Phase = factor(rr$values, levels = c(1, 2), labels = c("TWD", "GRO"))
    )

    p <- ggplot2::ggplot(td_r, ggplot2::aes(x = X)) +
      ggplot2::geom_rect(
        data = ribbons,
        ggplot2::aes(xmin = xmin, xmax = xmax, ymin = -Inf, ymax = Inf, fill = Phase),
        inherit.aes = FALSE,
        alpha = 0.15
      ) +
      ggplot2::geom_line(ggplot2::aes(y = dm), linewidth = 0.4) +
      ggplot2::geom_line(
        ggplot2::aes(y = GRO),
        linetype = 2,
        linewidth = 0.5,
        color = unname(cols["GRO"])
      ) +
      ggplot2::scale_fill_manual(
        values = c(TWD = unname(cols["TWD"]), GRO = "forestgreen")
      ) +
      ggplot2::labs(
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Stem radius / growth line (mm)"
        } else {
          paste0("Stem radius / growth line (", temporal, ")")
        },
        fill = "Phase"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        legend.position = "top",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "transition") {
    td_tr <- td_plot %>%
      dplyr::filter(!is.na(Phases)) %>%
      dplyr::arrange(TIME)

    xv <- .x_values(td_tr$TIME, temporal, x_axis)
    td_tr$X <- xv$x
    td_tr <- dplyr::filter(td_tr, !is.na(X))

    trans_idx <- which(diff(td_tr$Phases) != 0) + 1L
    transitions <- if (length(trans_idx) > 0) {
      td_tr[trans_idx, , drop = FALSE]
    } else {
      td_tr[0, , drop = FALSE]
    }

    p <- ggplot2::ggplot(td_tr, ggplot2::aes(x = X, y = dm)) +
      ggplot2::geom_line(color = "grey35", linewidth = 0.5) +
      ggplot2::geom_point(ggplot2::aes(color = Phase), size = 1.8) +
      ggplot2::geom_vline(
        data = transitions,
        ggplot2::aes(xintercept = X, color = Phase),
        linetype = transition_linetype,
        alpha = transition_alpha,
        show.legend = FALSE
      ) +
      ggplot2::geom_point(
        data = transitions,
        ggplot2::aes(x = X, y = dm, color = Phase),
        size = 3,
        inherit.aes = FALSE
      ) +
      ggplot2::scale_color_manual(
        values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"])),
        name = "New phase"
      ) +
      ggplot2::labs(
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Stem radius (mm)"
        } else {
          paste0("Stem radius (", temporal, ")")
        },
        title = "Zero-growth phase transitions"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "twd") {
    twd_peaks <- cyc %>%
      dplyr::filter(Phases == 1L, !is.na(max.twd), !is.na(Max.twd.time))

    xv <- .x_values(td_plot$TIME, temporal, x_axis)
    td_plot$X <- xv$x
    td_plot <- dplyr::filter(td_plot, !is.na(X))

    if (nrow(twd_peaks) > 0) {
      twd_peaks$X <- .x_values(twd_peaks$Max.twd.time, temporal, x_axis)$x
      twd_peaks <- dplyr::filter(twd_peaks, !is.na(X))
    }

    p <- ggplot2::ggplot(td_plot, ggplot2::aes(x = X, y = TWD)) +
      ggplot2::geom_area(fill = unname(cols["TWD"]), alpha = 0.35) +
      ggplot2::geom_line(color = unname(cols["TWD"]), linewidth = 0.4) +
      ggplot2::geom_point(
        data = twd_peaks,
        ggplot2::aes(x = X, y = max.twd),
        inherit.aes = FALSE,
        color = "black",
        size = 1.8
      ) +
      ggplot2::labs(
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Tree water deficit (mm)"
        } else {
          paste0("Tree water deficit (", temporal, ")")
        }
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "abr") {
    abr_dat <- cyc %>%
      dplyr::filter(Phases == 1L, !is.na(ABr.value))

    abr_time <- if (temporal != "raw" && "period" %in% names(abr_dat)) {
      abr_dat$period
    } else {
      abr_dat$Start
    }

    xv <- .x_values(abr_time, temporal, x_axis)
    abr_dat$X <- xv$x
    abr_dat <- dplyr::filter(abr_dat, !is.na(X))

    p <- ggplot2::ggplot(abr_dat, ggplot2::aes(x = X, y = ABr.value)) +
      ggplot2::geom_col(fill = "firebrick", alpha = 0.8) +
      ggplot2::labs(
        x = if (x_axis == "doy" && temporal != "monthly") {
          "Day of year"
        } else if (temporal == "raw") {
          "Start of TWD phase"
        } else {
          "Aggregation period"
        },
        y = if (temporal == "raw") {
          "ABr value"
        } else {
          paste0("ABr value (", temporal, ")")
        }
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "phase_summary") {
    sum_dat <- cyc %>%
      dplyr::mutate(
        Phase = .phase_factor(Phases),
        Metric = ifelse(Phases == 1L, max.twd, Magnitude)
      )

    p <- ggplot2::ggplot(sum_dat, ggplot2::aes(x = Duration_h, y = Metric, color = Phase)) +
      ggplot2::geom_point(size = 2, alpha = 0.8) +
      ggplot2::facet_wrap(~ Phase, scales = "free_y") +
      ggplot2::scale_color_manual(
        values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"]))
      ) +
      ggplot2::labs(
        x = "Phase duration (h)",
        y = "Phase metric (max.twd for TWD, Magnitude for GRO)",
        color = "Phase"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        legend.position = "top",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "balance") {
    unit <- if (temporal == "monthly") "month" else "day"

    if (temporal == "raw") {
      warning("For type = 'balance', raw data are summarised to daily balance.", call. = FALSE)
      unit <- "day"
    }

    if (balance_mode == "duration") {
      bal_segments <- .balance_segments(
        td_raw = td_raw,
        temporal = if (unit == "month") "monthly" else "daily",
        gap_mode = balance_gap
      )

      bal <- bal_segments %>%
        dplyr::group_by(period, Phase) %>%
        dplyr::summarise(hours = sum(hours, na.rm = TRUE), .groups = "drop")

      if (unit == "day") {
        totals <- bal %>%
          dplyr::group_by(period) %>%
          dplyr::summarise(total_hours = sum(hours, na.rm = TRUE), .groups = "drop")

        incomplete_days <- totals %>%
          dplyr::filter(abs(total_hours - 24) > 1e-6)

        if (nrow(incomplete_days) > 0) {
          warning(
            "Some daily balance totals do not equal 24 h. ",
            "This usually indicates an incomplete first/last day, missing timestamps, ",
            "or truncated data. Use balance_gap = 'carry_forward' to allocate gaps ",
            "to the previous phase, or balance_gap = 'observed_only' to count only observed intervals.",
            call. = FALSE
          )
        }
      }

      if (x_axis == "doy" && unit == "day") {
        bal$X <- lubridate::yday(bal$period)
        x_lab <- "Day of year"
        use_date_scale <- FALSE
      } else {
        if (x_axis == "doy" && unit == "month") {
          warning(
            "x_axis = 'doy' is not meaningful for monthly balance; using time instead.",
            call. = FALSE
          )
        }
        bal$X <- as.Date(bal$period)
        x_lab <- if (unit == "month") "Month" else "Date"
        use_date_scale <- TRUE
      }

      bal <- dplyr::filter(bal, !is.na(X))

      p <- ggplot2::ggplot(bal, ggplot2::aes(x = X, y = hours, fill = Phase)) +
        ggplot2::geom_col() +
        ggplot2::scale_fill_manual(
          values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"])),
          name = "Phase"
        ) +
        ggplot2::labs(
          title = "Phase balance",
          x = x_lab,
          y = if (unit == "month") "Hours per month" else "Hours per day"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          legend.position = "top",
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12)
        )

      if (use_date_scale) {
        p <- p + ggplot2::scale_x_date()
      }

      if (unit == "day") {
        p <- p + ggplot2::coord_cartesian(ylim = c(0, 24))
      }

      print(p)
      return(invisible(p))
    }

    if (balance_mode == "time_of_day") {
      if (unit == "month") {
        warning(
          "balance_mode = 'time_of_day' is designed for daily phase timing. ",
          "Using daily phase timing even though temporal = 'monthly'.",
          call. = FALSE
        )
      }

      seg <- .balance_segments(
        td_raw = td_raw,
        temporal = "daily",
        gap_mode = balance_gap
      ) %>%
        dplyr::mutate(
          day = as.Date(period),
          ymin = .hour_decimal(seg_start),
          ymax = dplyr::if_else(
            as.Date(seg_end) > day & abs(.hour_decimal(seg_end)) < 1e-8,
            24,
            .hour_decimal(seg_end)
          )
        ) %>%
        dplyr::filter(!is.na(ymin), !is.na(ymax), ymax > ymin)

      totals <- seg %>%
        dplyr::group_by(day) %>%
        dplyr::summarise(total_hours = sum(ymax - ymin, na.rm = TRUE), .groups = "drop")

      incomplete_days <- totals %>%
        dplyr::filter(abs(total_hours - 24) > 1e-6)

      if (nrow(incomplete_days) > 0) {
        warning(
          "Some daily balance totals do not equal 24 h. ",
          "This usually indicates an incomplete first/last day, missing timestamps, ",
          "or truncated data. The plot still preserves the actual within-day timing ",
          "of observed or gap-filled phase intervals.",
          call. = FALSE
        )
      }

      if (x_axis == "doy") {
        seg$X <- lubridate::yday(seg$day)
        seg$xmin <- seg$X - 0.45
        seg$xmax <- seg$X + 0.45
        x_lab <- "Day of year"
        use_date_scale <- FALSE
      } else {
        seg$X <- seg$day
        seg$xmin <- seg$day - 0.45
        seg$xmax <- seg$day + 0.45
        x_lab <- "Date"
        use_date_scale <- TRUE
      }

      p <- ggplot2::ggplot(seg) +
        ggplot2::geom_rect(
          ggplot2::aes(
            xmin = xmin,
            xmax = xmax,
            ymin = ymin,
            ymax = ymax,
            fill = Phase
          ),
          color = NA
        ) +
        ggplot2::scale_fill_manual(
          values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"])),
          name = "Phase"
        ) +
        ggplot2::scale_y_continuous(
          limits = c(0, 24),
          breaks = seq(0, 24, by = 6)
        ) +
        ggplot2::labs(
          title = "Daily phase timing",
          x = x_lab,
          y = "Hour of day"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          legend.position = "top",
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12)
        )

      if (use_date_scale) {
        p <- p + ggplot2::scale_x_date()
      }

      print(p)
      return(invisible(p))
    }
  }

  if (type == "boxplot") {
    cyc_box <- cyc

    if (phase == "auto") {
      if (stat %in% c("max.twd", "Avg.twd", "STD.twd", "AUC.load", "AUC.total", "ABr.value")) {
        phase <- "TWD"
      } else if (stat %in% c("Magnitude", "rate")) {
        phase <- "GRO"
      } else {
        phase <- "all"
      }
    }

    if (phase == "TWD") {
      cyc_box <- dplyr::filter(cyc_box, Phases == 1L)
    }

    if (phase == "GRO") {
      cyc_box <- dplyr::filter(cyc_box, Phases == 2L)
    }

    cyc_box <- cyc_box %>%
      dplyr::mutate(Phase = .phase_factor(Phases))

    box_time <- if ("period" %in% names(cyc_box)) {
      cyc_box$period
    } else {
      cyc_box$Start
    }

    use_continuous_time <- FALSE

    if (box_group == "period") {
      use_continuous_time <- TRUE

      if (temporal == "daily") {
        full_dates <- seq(
          as.Date(min(td_raw$TIME, na.rm = TRUE)),
          as.Date(max(td_raw$TIME, na.rm = TRUE)),
          by = "day"
        )
        cyc_box$group_date <- as.Date(box_time)
        x_lab <- "Date"
        x_limits <- range(full_dates)
        box_width <- 0.8
        point_jitter_width <- 0.2
        date_breaks <- if (length(full_dates) > 120) "3 months" else "1 month"
        date_labels <- if (length(unique(format(full_dates, "%Y"))) > 1) "%Y-%m" else "%m-%d"
      } else {
        full_dates <- seq(
          as.Date(lubridate::floor_date(min(td_raw$TIME, na.rm = TRUE), "month")),
          as.Date(lubridate::floor_date(max(td_raw$TIME, na.rm = TRUE), "month")),
          by = "month"
        )
        cyc_box$group_date <- as.Date(lubridate::floor_date(box_time, "month"))
        x_lab <- "Date"
        x_limits <- range(full_dates)
        box_width <- 20
        point_jitter_width <- 5
        date_breaks <- if (length(full_dates) > 24) "6 months" else "1 month"
        date_labels <- if (length(unique(format(full_dates, "%Y"))) > 1) "%Y-%m" else "%b"
      }
    } else if (box_group == "month_of_year") {
      cyc_box$group_label <- factor(
        as.character(lubridate::month(box_time, label = TRUE, abbr = TRUE)),
        levels = month.abb
      )
      x_lab <- "Month of year"
    } else {
      full_levels <- as.character(1:366)
      cyc_box$group_label <- factor(
        as.character(lubridate::yday(box_time)),
        levels = full_levels
      )
      x_lab <- "Day of year"
    }

    cyc_box$value <- cyc_box[[stat]]

    if (use_continuous_time) {
      cyc_box <- cyc_box %>%
        dplyr::filter(!is.na(value), !is.na(group_date))
    } else {
      cyc_box <- cyc_box %>%
        dplyr::filter(!is.na(value), !is.na(group_label))
    }

    if (nrow(cyc_box) == 0) {
      stop("No non-missing values available for the selected statistic/phase.", call. = FALSE)
    }

    if (use_continuous_time) {
      count_key <- format(cyc_box$group_date, "%Y-%m-%d")
      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        count_key <- paste(count_key, cyc_box$Phase, sep = "_")
      }
    } else {
      count_key <- as.character(cyc_box$group_label)
      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        count_key <- paste(count_key, cyc_box$Phase, sep = "_")
      }
    }

    counts <- table(count_key)
    singleton_groups <- all(counts <= 1)

    if (singleton_groups) {
      if (box_group == "period") {
        warning(
          "Each group has only one value, so boxplots collapse to lines. ",
          "This is expected for box_group = 'period'. ",
          "Use box_group = 'month_of_year' for pooled seasonal boxplots.",
          call. = FALSE
        )
      } else if (box_group == "doy" && length(unique(format(cyc_box$Start, "%Y"))) <= 1) {
        warning(
          "box_group = 'doy' needs repeated day-of-year values across multiple years. ",
          "Your selected data appear to cover only one year, so each DOY has one value.",
          call. = FALSE
        )
      } else {
        warning("Each boxplot group has only one value; boxes collapse to lines.", call. = FALSE)
      }
    }

    if (singleton_groups && isTRUE(singleton_as_points)) {
      if (use_continuous_time) {
        p <- ggplot2::ggplot(
          cyc_box,
          ggplot2::aes(x = group_date, y = value, color = Phase)
        ) +
          ggplot2::geom_point(
            position = ggplot2::position_jitter(width = point_jitter_width, height = 0),
            size = 2,
            alpha = 0.8
          ) +
          ggplot2::scale_color_manual(
            values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"]))
          ) +
          ggplot2::scale_x_date(
            limits = x_limits,
            date_breaks = date_breaks,
            date_labels = date_labels
          ) +
          ggplot2::labs(
            x = x_lab,
            y = stat,
            color = "Phase"
          ) +
          ggplot2::theme_minimal() +
          ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
            axis.title = ggplot2::element_text(size = 14),
            axis.text = ggplot2::element_text(size = 11)
          )
      } else {
        p <- ggplot2::ggplot(
          cyc_box,
          ggplot2::aes(x = group_label, y = value, color = Phase)
        ) +
          ggplot2::geom_point(
            position = ggplot2::position_jitter(width = 0.12, height = 0),
            size = 2,
            alpha = 0.8
          ) +
          ggplot2::scale_color_manual(
            values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"]))
          ) +
          ggplot2::labs(
            x = x_lab,
            y = stat,
            color = "Phase"
          ) +
          ggplot2::theme_minimal() +
          ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
            axis.title = ggplot2::element_text(size = 14),
            axis.text = ggplot2::element_text(size = 11)
          ) +
          ggplot2::scale_x_discrete(drop = FALSE)
      }

      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        p <- p + ggplot2::facet_wrap(~ Phase, scales = "free_y")
      }

      print(p)
      return(invisible(p))
    }

    if (use_continuous_time) {
      p <- ggplot2::ggplot(
        cyc_box,
        ggplot2::aes(
          x = group_date,
          y = value,
          fill = Phase,
          group = interaction(group_date, Phase, drop = TRUE)
        )
      ) +
        ggplot2::geom_boxplot(width = box_width, outlier.shape = NA) +
        ggplot2::geom_point(
          position = ggplot2::position_jitter(width = point_jitter_width, height = 0),
          size = 1.2,
          alpha = 0.6
        ) +
        ggplot2::scale_fill_manual(
          values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"]))
        ) +
        ggplot2::scale_x_date(
          limits = x_limits,
          date_breaks = date_breaks,
          date_labels = date_labels
        ) +
        ggplot2::labs(
          x = x_lab,
          y = stat,
          fill = "Phase"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        )
    } else {
      p <- ggplot2::ggplot(
        cyc_box,
        ggplot2::aes(x = group_label, y = value, fill = Phase)
      ) +
        ggplot2::geom_boxplot(outlier.shape = NA) +
        ggplot2::geom_point(
          position = ggplot2::position_jitter(width = 0.15, height = 0),
          size = 1.2,
          alpha = 0.6
        ) +
        ggplot2::scale_fill_manual(
          values = c(TWD = unname(cols["TWD"]), GRO = unname(cols["GRO"]))
        ) +
        ggplot2::labs(
          x = x_lab,
          y = stat,
          fill = "Phase"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        ) +
        ggplot2::scale_x_discrete(drop = FALSE)
    }

    if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
      p <- p + ggplot2::facet_wrap(~ Phase, scales = "free_y")
    }

    print(p)
    return(invisible(p))
  }
}

# =========================
# Superposed epoch analysis
# =========================

#' Extract event times from phase.zg() or phase.sc() output
#'
#' @description
#' Extracts event times from objects returned by \code{phase.zg()} or
#' \code{phase.sc()}. The resulting table can be passed directly to
#' [dm_epoch_extract()] for superposed epoch analysis.
#'
#' @param x Object of class \code{"ZG_output"} or \code{"SC_output"}.
#' @param event Event type to extract. Use \code{"all"} to return all supported
#'   events. For ZG output, supported events are \code{"TWD_start"},
#'   \code{"TWD_end"}, \code{"GRO_start"}, \code{"GRO_end"},
#'   \code{"MaxTWD"}, \code{"phase_start"}, and \code{"phase_end"}.
#'   For SC output, supported events are \code{"Shrinkage_start"},
#'   \code{"Shrinkage_end"}, \code{"Expansion_start"}, \code{"Expansion_end"},
#'   \code{"Increment_start"}, \code{"Increment_end"}, \code{"phase_start"},
#'   and \code{"phase_end"}.
#' @param phase Optional phase filter for generic events \code{"phase_start"}
#'   or \code{"phase_end"}. For ZG output, use \code{"TWD"} or \code{"GRO"}.
#'   For SC output, use \code{"Shrinkage"}, \code{"Expansion"}, or
#'   \code{"Increment"}.
#' @param min_duration Optional minimum \code{Duration_h} filter.
#' @param min_magnitude Optional minimum \code{Magnitude} filter.
#' @param min_max_twd Optional minimum \code{max.twd} filter for ZG output.
#' @param remove_na_times Logical. If \code{TRUE}, rows with missing
#'   \code{event_time} are removed.
#'
#' @return
#' A tibble of events with class \code{c("dm_events", ...)}.
#'
#' @importFrom tibble as_tibble tibble
#' @importFrom dplyr bind_rows
#' @export
dm_event_times <- function(x,
                           event = "all",
                           phase = NULL,
                           min_duration = NULL,
                           min_magnitude = NULL,
                           min_max_twd = NULL,
                           remove_na_times = TRUE) {

  if (inherits(x, "ZG_output")) {
    cyc <- tibble::as_tibble(x$ZG_cycle)

    if (!all(c("Phases", "Start", "End") %in% names(cyc))) {
      stop("ZG_output does not contain the expected ZG_cycle columns.")
    }

    cyc$phase_label <- ifelse(cyc$Phases == 1L, "TWD", "GRO")

    if (!is.null(min_duration) && "Duration_h" %in% names(cyc)) {
      cyc <- cyc[is.na(cyc$Duration_h) | cyc$Duration_h >= min_duration, , drop = FALSE]
    }

    if (!is.null(min_magnitude) && "Magnitude" %in% names(cyc)) {
      cyc <- cyc[is.na(cyc$Magnitude) | cyc$Magnitude >= min_magnitude, , drop = FALSE]
    }

    if (!is.null(min_max_twd) && "max.twd" %in% names(cyc)) {
      cyc <- cyc[is.na(cyc$max.twd) | cyc$max.twd >= min_max_twd, , drop = FALSE]
    }

    allowed <- c(
      "all",
      "TWD_start", "TWD_end", "GRO_start", "GRO_end",
      "MaxTWD",
      "phase_start", "phase_end"
    )

    if (!event %in% allowed) {
      stop("Unsupported event for ZG_output.")
    }

    out_list <- list()

    if (event %in% c("all", "TWD_start")) {
      z <- cyc[cyc$Phases == 1L, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- "TWD_start"
        z$event_time <- z$Start
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (event %in% c("all", "TWD_end")) {
      z <- cyc[cyc$Phases == 1L, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- "TWD_end"
        z$event_time <- z$End
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (event %in% c("all", "GRO_start")) {
      z <- cyc[cyc$Phases == 2L, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- "GRO_start"
        z$event_time <- z$Start
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (event %in% c("all", "GRO_end")) {
      z <- cyc[cyc$Phases == 2L, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- "GRO_end"
        z$event_time <- z$End
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (event %in% c("all", "MaxTWD")) {
      if (!"Max.twd.time" %in% names(cyc)) {
        stop("ZG_cycle does not contain column 'Max.twd.time'.")
      }

      z <- cyc[cyc$Phases == 1L, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- "MaxTWD"
        z$event_time <- z$Max.twd.time
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (event %in% c("phase_start", "phase_end")) {
      if (is.null(phase)) {
        stop("For event = 'phase_start' or 'phase_end', please supply phase = 'TWD' or 'GRO'.")
      }

      phase <- match.arg(phase, c("TWD", "GRO"))
      ph_num <- ifelse(phase == "TWD", 1L, 2L)

      z <- cyc[cyc$Phases == ph_num, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- paste0(
          phase,
          "_",
          ifelse(event == "phase_start", "start", "end")
        )
        z$event_time <- if (event == "phase_start") z$Start else z$End
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (length(out_list) == 0) {
      out <- tibble::tibble()
    } else {
      out <- dplyr::bind_rows(out_list)
    }

  } else if (inherits(x, "SC_output")) {
    cyc <- tibble::as_tibble(x$SC_cycle)

    if (!all(c("Phases", "Start", "End") %in% names(cyc))) {
      stop("SC_output does not contain the expected SC_cycle columns.")
    }

    phase_map <- c(
      "1" = "Shrinkage",
      "2" = "Expansion",
      "3" = "Increment"
    )

    cyc$phase_label <- unname(phase_map[as.character(cyc$Phases)])

    if (!is.null(min_duration) && "Duration_h" %in% names(cyc)) {
      cyc <- cyc[is.na(cyc$Duration_h) | cyc$Duration_h >= min_duration, , drop = FALSE]
    }

    if (!is.null(min_magnitude) && "Magnitude" %in% names(cyc)) {
      cyc <- cyc[is.na(cyc$Magnitude) | abs(cyc$Magnitude) >= min_magnitude, , drop = FALSE]
    }

    allowed <- c(
      "all",
      "Shrinkage_start", "Shrinkage_end",
      "Expansion_start", "Expansion_end",
      "Increment_start", "Increment_end",
      "phase_start", "phase_end"
    )

    if (!event %in% allowed) {
      stop("Unsupported event for SC_output.")
    }

    out_list <- list()

    for (ph in c("Shrinkage", "Expansion", "Increment")) {
      ph_num <- which(c("Shrinkage", "Expansion", "Increment") == ph)

      if (event %in% c("all", paste0(ph, "_start"))) {
        z <- cyc[cyc$Phases == ph_num, , drop = FALSE]
        if (nrow(z) > 0) {
          z$event_type <- paste0(ph, "_start")
          z$event_time <- z$Start
          out_list[[length(out_list) + 1L]] <- z
        }
      }

      if (event %in% c("all", paste0(ph, "_end"))) {
        z <- cyc[cyc$Phases == ph_num, , drop = FALSE]
        if (nrow(z) > 0) {
          z$event_type <- paste0(ph, "_end")
          z$event_time <- z$End
          out_list[[length(out_list) + 1L]] <- z
        }
      }
    }

    if (event %in% c("phase_start", "phase_end")) {
      if (is.null(phase)) {
        stop("For event = 'phase_start' or 'phase_end', please supply phase = 'Shrinkage', 'Expansion', or 'Increment'.")
      }

      phase <- match.arg(phase, c("Shrinkage", "Expansion", "Increment"))
      ph_num <- which(c("Shrinkage", "Expansion", "Increment") == phase)

      z <- cyc[cyc$Phases == ph_num, , drop = FALSE]
      if (nrow(z) > 0) {
        z$event_type <- paste0(
          phase,
          "_",
          ifelse(event == "phase_start", "start", "end")
        )
        z$event_time <- if (event == "phase_start") z$Start else z$End
        out_list[[length(out_list) + 1L]] <- z
      }
    }

    if (length(out_list) == 0) {
      out <- tibble::tibble()
    } else {
      out <- dplyr::bind_rows(out_list)
    }

  } else {
    stop("'x' must be output of phase.zg() or phase.sc().")
  }

  if (nrow(out) == 0) {
    out <- tibble::tibble(
      event_id = integer(0),
      event_time = as.POSIXct(character(0)),
      event_type = character(0),
      phase = character(0)
    )
  } else {
    out$phase <- out$phase_label

    if (isTRUE(remove_na_times)) {
      out <- out[!is.na(out$event_time), , drop = FALSE]
    }

    out <- out[order(out$event_time), , drop = FALSE]
    out$event_id <- seq_len(nrow(out))

    out <- out[
      ,
      c(
        "event_id", "event_time", "event_type", "phase",
        setdiff(names(out), c("event_id", "event_time", "event_type", "phase"))
      ),
      drop = FALSE
    ]
  }

  class(out) <- c("dm_events", class(out))

  out
}


# -------------------------
# internal helpers
# -------------------------

#' Parse date-time values for SEA
#'
#' @param x A vector of date-time, date, or character values.
#'
#' @return A \code{POSIXct} vector.
#'
#' @importFrom lubridate ymd_hms parse_date_time
#' @keywords internal
dmea_parse_time <- function(x) {
  if (inherits(x, "POSIXct")) {
    return(as.POSIXct(x))
  }

  if (inherits(x, "POSIXt")) {
    return(as.POSIXct(x))
  }

  if (inherits(x, "Date")) {
    return(as.POSIXct(x))
  }

  out <- suppressWarnings(lubridate::ymd_hms(x, quiet = TRUE))

  if (all(is.na(out))) {
    out <- suppressWarnings(
      lubridate::parse_date_time(
        x,
        orders = c(
          "ymd HMS", "ymd HM", "ymd",
          "dmy HMS", "dmy HM", "dmy",
          "mdy HMS", "mdy HM", "mdy"
        ),
        quiet = TRUE
      )
    )
  }

  out
}


#' Create a lubridate period string
#'
#' @param step Numeric step size.
#' @param unit Unit. One of \code{"mins"}, \code{"hours"}, or \code{"days"}.
#'
#' @return A character period string for [lubridate::floor_date()].
#'
#' @keywords internal
dmea_unit_string <- function(step, unit) {
  if (unit == "mins") {
    return(paste(step, "minutes"))
  }

  if (unit == "hours") {
    return(paste(step, "hours"))
  }

  if (unit == "days") {
    return(paste(step, "days"))
  }

  stop("Unsupported unit.")
}


#' Convert SEA step size to seconds
#'
#' @param step Numeric step size.
#' @param unit Unit. One of \code{"mins"}, \code{"hours"}, or \code{"days"}.
#'
#' @return Numeric number of seconds.
#'
#' @keywords internal
dmea_step_seconds <- function(step, unit) {
  if (unit == "mins") {
    return(step * 60)
  }

  if (unit == "hours") {
    return(step * 3600)
  }

  if (unit == "days") {
    return(step * 86400)
  }

  stop("Unsupported unit.")
}


#' Apply an aggregation function
#'
#' @param x Numeric vector.
#' @param fun Function or character string. Supported character values are
#'   \code{"mean"}, \code{"sum"}, \code{"max"}, \code{"min"}, and
#'   \code{"median"}.
#'
#' @return Numeric scalar.
#'
#' @keywords internal
dmea_apply_fun <- function(x, fun) {
  if (all(is.na(x))) {
    return(NA_real_)
  }

  if (is.function(fun)) {
    return(fun(x))
  }

  fun <- match.arg(fun, c("mean", "sum", "max", "min", "median"))

  switch(
    fun,
    mean = mean(x, na.rm = TRUE),
    sum = sum(x, na.rm = TRUE),
    max = max(x, na.rm = TRUE),
    min = min(x, na.rm = TRUE),
    median = median(x, na.rm = TRUE)
  )
}


#' Filter timestamps by calendar year and day-of-year
#'
#' @description
#' Internal helper used by the SEA functions to select timestamps by calendar
#' year and day-of-year.
#'
#' @param times A vector of date-time values.
#' @param Year Optional numeric vector of calendar years to retain.
#' @param DOY Optional numeric vector of length one or two. If length one, only
#'   that day-of-year is retained. If length two, the inclusive DOY range is
#'   retained. If the first value is larger than the second, the range is treated
#'   as a cross-year DOY window, for example \code{DOY = c(300, 60)} keeps
#'   \code{DOY >= 300} or \code{DOY <= 60}.
#'
#' @return A logical vector with the same length as \code{times}.
#'
#' @importFrom lubridate year yday
#' @keywords internal
dmea_time_filter_index <- function(times, Year = NULL, DOY = NULL) {
  tt <- dmea_parse_time(times)

  keep <- !is.na(tt)

  if (!is.null(Year)) {
    if (!is.numeric(Year) || length(Year) < 1L || any(is.na(Year))) {
      stop("'Year' must be a non-empty numeric vector without NA values.")
    }

    if (any(Year != floor(Year))) {
      stop("'Year' must contain whole calendar years.")
    }

    Year <- as.integer(Year)

    keep <- keep & lubridate::year(tt) %in% Year
  }

  if (!is.null(DOY)) {
    if (!is.numeric(DOY) || !length(DOY) %in% c(1L, 2L) || any(is.na(DOY))) {
      stop("'DOY' must be a numeric vector of length one or two without NA values.")
    }

    if (any(DOY != floor(DOY))) {
      stop("'DOY' must contain whole day-of-year values.")
    }

    if (any(DOY < 1 | DOY > 366)) {
      stop("'DOY' values must be between 1 and 366.")
    }

    DOY <- as.integer(DOY)

    yy <- lubridate::yday(tt)

    if (length(DOY) == 1L) {
      keep <- keep & yy == DOY[1L]
    } else {
      doy_start <- DOY[1L]
      doy_end <- DOY[2L]

      if (doy_start <= doy_end) {
        keep <- keep & yy >= doy_start & yy <= doy_end
      } else {
        keep <- keep & (yy >= doy_start | yy <= doy_end)
      }
    }
  }

  keep
}


#' Filter a table by calendar year and day-of-year
#'
#' @param dat A data frame or tibble.
#' @param time_col Name of the timestamp column.
#' @param Year Optional numeric vector of calendar years.
#' @param DOY Optional numeric vector of day-of-year values.
#' @param label Text used in error messages.
#'
#' @return A filtered tibble.
#'
#' @importFrom tibble as_tibble
#' @keywords internal
dmea_filter_time_table <- function(dat,
                                   time_col,
                                   Year = NULL,
                                   DOY = NULL,
                                   label = "rows") {
  dat <- tibble::as_tibble(dat)

  if (!time_col %in% names(dat)) {
    stop("Column '", time_col, "' not found in ", label, ".")
  }

  keep <- dmea_time_filter_index(
    times = dat[[time_col]],
    Year = Year,
    DOY = DOY
  )

  dat[keep, , drop = FALSE]
}


#' Prepare climate data for superposed epoch analysis
#'
#' @param clim Climate data frame. First column must contain timestamps.
#' @param vars Climate variables to use, or \code{"all"}.
#' @param step Step size in units.
#' @param unit One of \code{"hours"}, \code{"days"}, or \code{"mins"}.
#' @param agg_fun Aggregation function applied to all variables when
#'   \code{var_funs} is not supplied.
#' @param var_funs Optional named vector or named list of aggregation functions
#'   for individual variables.
#'
#' @return A tibble with aggregated climate data.
#'
#' @importFrom tibble as_tibble
#' @importFrom lubridate floor_date
#' @importFrom dplyr %>% group_by summarise across all_of cur_column rename
#' @keywords internal
dmea_prepare_climate <- function(clim,
                                 vars = "all",
                                 step = 1,
                                 unit = c("hours", "days", "mins"),
                                 agg_fun = "mean",
                                 var_funs = NULL) {
  TIME_BIN <- NULL
  unit <- match.arg(unit)

  clim <- tibble::as_tibble(clim)
  names(clim)[1] <- "TIME"
  clim$TIME <- dmea_parse_time(clim$TIME)

  if (any(is.na(clim$TIME))) {
    stop("Some climate timestamps could not be parsed.")
  }

  clim <- clim[order(clim$TIME), , drop = FALSE]

  if (identical(vars, "all")) {
    vars <- names(clim)[vapply(clim, is.numeric, logical(1))]
    vars <- setdiff(vars, "TIME")
  }

  if (length(vars) == 0) {
    stop("No climate variables selected.")
  }

  miss <- setdiff(vars, names(clim))

  if (length(miss) > 0) {
    stop("These climate variables were not found: ", paste(miss, collapse = ", "))
  }

  period_unit <- dmea_unit_string(step, unit)

  clim$TIME_BIN <- lubridate::floor_date(
    clim$TIME,
    unit = period_unit
  )

  if (is.null(var_funs)) {
    var_funs <- stats::setNames(rep(agg_fun, length(vars)), vars)
  } else {
    if (is.null(names(var_funs))) {
      if (length(var_funs) == 1) {
        var_funs <- stats::setNames(rep(var_funs, length(vars)), vars)
      } else {
        stop("'var_funs' must be named by climate variable, or length 1.")
      }
    } else {
      tmp <- stats::setNames(rep(agg_fun, length(vars)), vars)
      shared <- intersect(names(var_funs), vars)
      tmp[shared] <- unlist(var_funs[shared], use.names = FALSE)
      var_funs <- tmp
    }
  }

  out <- clim %>%
    dplyr::group_by(TIME_BIN) %>%
    dplyr::summarise(
      dplyr::across(
        dplyr::all_of(vars),
        ~ dmea_apply_fun(.x, var_funs[dplyr::cur_column()][[1]])
      ),
      .groups = "drop"
    ) %>%
    dplyr::rename(TIME = TIME_BIN)

  out
}


#' Extract climate values around anchor times
#'
#' @param anchor_times Anchor times.
#' @param clim_tbl Aggregated climate table from [dmea_prepare_climate()].
#' @param lag_values Lag values in seconds.
#' @param vars Climate variables to extract.
#'
#' @return A long-format tibble.
#'
#' @importFrom tibble tibble
#' @importFrom dplyr left_join all_of
#' @importFrom tidyr pivot_longer
#' @keywords internal
dmea_extract_from_anchors <- function(anchor_times,
                                      clim_tbl,
                                      lag_values,
                                      vars) {
  if (length(anchor_times) == 0) {
    return(tibble::tibble())
  }

  tz_use <- attr(clim_tbl$TIME, "tzone")

  if (is.null(tz_use) || length(tz_use) == 0) {
    tz_use <- "UTC"
  }

  anchor_times <- as.POSIXct(
    anchor_times,
    origin = "1970-01-01",
    tz = tz_use
  )

  exp <- tibble::tibble(
    event_id = rep(seq_along(anchor_times), each = length(lag_values)),
    anchor_time = rep(anchor_times, each = length(lag_values)),
    lag = rep(lag_values, times = length(anchor_times))
  )

  exp$TIME <- as.POSIXct(
    exp$anchor_time + exp$lag,
    origin = "1970-01-01",
    tz = tz_use
  )

  join <- dplyr::left_join(
    exp,
    clim_tbl[, c("TIME", vars), drop = FALSE],
    by = "TIME"
  )

  out <- tidyr::pivot_longer(
    join,
    cols = dplyr::all_of(vars),
    names_to = "variable",
    values_to = "value"
  )

  out
}


#' Sample one null anchor time
#'
#' @param event_time Original event time.
#' @param clim_times Candidate climate timestamps.
#' @param null Null model. One of \code{"same_doy_window"},
#'   \code{"same_month"}, or \code{"random_time"}.
#' @param doy_window DOY window used for \code{null = "same_doy_window"}.
#' @param match_hour Logical. For hourly or minute data, keep the same hour
#'   when possible.
#' @param match_minute Logical. For minute data, keep the same minute when
#'   possible.
#' @param unit One of \code{"hours"}, \code{"days"}, or \code{"mins"}.
#'
#' @return One sampled \code{POSIXct} timestamp.
#'
#' @importFrom lubridate month yday hour minute
#' @keywords internal
dmea_sample_anchor_one <- function(event_time,
                                   clim_times,
                                   null = c("same_doy_window", "same_month", "random_time"),
                                   doy_window = 15,
                                   match_hour = TRUE,
                                   match_minute = TRUE,
                                   unit = c("hours", "days", "mins")) {
  null <- match.arg(null)
  unit <- match.arg(unit)

  candidates <- clim_times

  tz_use <- attr(clim_times, "tzone")

  if (is.null(tz_use) || length(tz_use) == 0) {
    tz_use <- "UTC"
  }

  if (null == "same_month") {
    candidates <- candidates[
      lubridate::month(candidates) == lubridate::month(event_time)
    ]
  }

  if (null == "same_doy_window") {
    ev_doy <- lubridate::yday(event_time)
    cdoy <- lubridate::yday(candidates)

    dd <- abs(cdoy - ev_doy)
    dd <- pmin(dd, 366 - dd)

    candidates <- candidates[dd <= doy_window]
  }

  if (unit %in% c("hours", "mins") && isTRUE(match_hour)) {
    hh <- lubridate::hour(event_time)
    candidates2 <- candidates[lubridate::hour(candidates) == hh]

    if (length(candidates2) > 0) {
      candidates <- candidates2
    }
  }

  if (unit == "mins" && isTRUE(match_minute)) {
    mm <- lubridate::minute(event_time)
    candidates2 <- candidates[lubridate::minute(candidates) == mm]

    if (length(candidates2) > 0) {
      candidates <- candidates2
    }
  }

  if (length(candidates) == 0) {
    candidates <- clim_times
  }

  out <- sample(candidates, size = 1)

  as.POSIXct(
    out,
    origin = "1970-01-01",
    tz = tz_use
  )
}


#' Composite statistic helper
#'
#' @param x Numeric vector.
#' @param statistic Statistic. One of \code{"mean"} or \code{"median"}.
#'
#' @return Numeric scalar.
#'
#' @keywords internal
dmea_stat_fun <- function(x, statistic = c("mean", "median")) {
  statistic <- match.arg(statistic)

  if (all(is.na(x))) {
    return(NA_real_)
  }

  if (statistic == "mean") {
    return(mean(x, na.rm = TRUE))
  }

  median(x, na.rm = TRUE)
}


#' Build an epoch table around event times
#'
#' @description
#' Builds a long-format epoch table by extracting climate values before and
#' after event times.
#'
#' The optional \code{Year} and \code{DOY} arguments allow the user to restrict
#' which events are included in the superposed epoch analysis. The full climate
#' table is retained for lag extraction so that values before and after selected
#' events are still available.
#'
#' @param events Output of [dm_event_times()] or a data frame containing an
#'   \code{event_time} column.
#' @param clim Climate data frame. The first column must contain timestamps.
#' @param vars Climate variables to use, or \code{"all"}.
#' @param lag_before Number of lag steps before the event.
#' @param lag_after Number of lag steps after the event.
#' @param step Step size in units.
#' @param unit One of \code{"hours"}, \code{"days"}, or \code{"mins"}.
#' @param agg_fun Aggregation function applied to all climate variables when
#'   \code{var_funs} is not supplied.
#' @param var_funs Optional named vector or named list of aggregation functions
#'   for each variable.
#' @param Year Optional numeric vector of calendar years to include in the SEA.
#'   Events whose \code{event_time} falls outside these years are removed before
#'   epoch extraction.
#' @param DOY Optional numeric vector of length one or two giving the
#'   day-of-year window to include in the SEA. If length one, only that DOY is
#'   retained. If length two, the inclusive range is retained. Cross-year DOY
#'   windows are supported, for example \code{DOY = c(300, 60)}.
#' @param restrict_null_to_window Logical. If \code{TRUE}, null anchor times in
#'   [dm_epoch_test()] are sampled only from the same \code{Year}/\code{DOY}
#'   window. The full climate table is still retained for extracting
#'   \code{lag_before} and \code{lag_after} windows. Default is \code{TRUE}.
#'
#' @return
#' An object of class \code{c("dm_epoch", "dm_epoch_extract")}.
#'
#' @examples
#' \donttest{
#' # events <- dm_event_times(zg, event = "GRO_start")
#' # ep <- dm_epoch_extract(
#' #   events = events,
#' #   clim = climate,
#' #   vars = c("Rain", "temp", "vpd"),
#' #   Year = c(2022, 2023, 2024),
#' #   DOY = c(100, 300),
#' #   lag_before = 24,
#' #   lag_after = 24,
#' #   unit = "hours"
#' # )
#' }
#'
#' @importFrom tibble as_tibble
#' @importFrom lubridate floor_date
#' @importFrom dplyr left_join all_of
#' @importFrom tidyr pivot_longer
#' @export
dm_epoch_extract <- function(events,
                             clim,
                             vars = "all",
                             lag_before = 24,
                             lag_after = 24,
                             step = 1,
                             unit = c("hours", "days", "mins"),
                             agg_fun = "mean",
                             var_funs = NULL,
                             Year = NULL,
                             DOY = NULL,
                             restrict_null_to_window = TRUE) {

  unit <- match.arg(unit)

  if (!("event_time" %in% names(events))) {
    stop("'events' must contain an 'event_time' column.")
  }

  if (!is.numeric(lag_before) ||
      !is.numeric(lag_after) ||
      length(lag_before) != 1 ||
      length(lag_after) != 1 ||
      is.na(lag_before) ||
      is.na(lag_after) ||
      lag_before < 0 ||
      lag_after < 0) {
    stop("'lag_before' and 'lag_after' must be non-negative scalars.")
  }

  if (!is.numeric(step) ||
      length(step) != 1 ||
      is.na(step) ||
      step <= 0) {
    stop("'step' must be a positive number.")
  }

  ev <- tibble::as_tibble(events)
  ev$event_time <- dmea_parse_time(ev$event_time)

  if (any(is.na(ev$event_time))) {
    stop("Some event times could not be parsed.")
  }

  n_events_before_filter <- nrow(ev)

  ev <- dmea_filter_time_table(
    dat = ev,
    time_col = "event_time",
    Year = Year,
    DOY = DOY,
    label = "events"
  )

  if (nrow(ev) == 0) {
    stop("No events remain after filtering by Year and/or DOY.")
  }

  if ("event_id" %in% names(ev)) {
    ev$original_event_id <- ev$event_id
  }

  ev$event_id <- seq_len(nrow(ev))

  clim_tbl <- dmea_prepare_climate(
    clim = clim,
    vars = vars,
    step = step,
    unit = unit,
    agg_fun = agg_fun,
    var_funs = var_funs
  )

  vars_use <- setdiff(names(clim_tbl), "TIME")

  if (length(vars_use) == 0) {
    stop("No usable climate variables found after aggregation.")
  }

  bin_unit <- dmea_unit_string(step, unit)

  ev$event_time_aligned <- lubridate::floor_date(
    ev$event_time,
    unit = bin_unit
  )

  lag_values_units <- seq.int(-lag_before, lag_after, by = 1L) * step
  lag_values_seconds <- lag_values_units * dmea_step_seconds(1, unit)

  exp <- ev[
    rep(seq_len(nrow(ev)), each = length(lag_values_units)),
    ,
    drop = FALSE
  ]

  exp$lag <- rep(lag_values_units, times = nrow(ev))
  exp$lag_seconds <- rep(lag_values_seconds, times = nrow(ev))
  exp$TIME <- exp$event_time_aligned + exp$lag_seconds

  epoch_wide <- dplyr::left_join(
    exp,
    clim_tbl,
    by = "TIME"
  )

  epoch_long <- tidyr::pivot_longer(
    epoch_wide,
    cols = dplyr::all_of(vars_use),
    names_to = "variable",
    values_to = "value"
  )

  out <- list(
    events = ev,
    epoch_data = epoch_long,
    climate_data = clim_tbl,
    settings = list(
      vars = vars_use,
      lag_before = lag_before,
      lag_after = lag_after,
      step = step,
      unit = unit,
      lag_values = lag_values_units,
      agg_fun = agg_fun,
      var_funs = var_funs,
      Year = Year,
      DOY = DOY,
      restrict_null_to_window = isTRUE(restrict_null_to_window),
      n_events_before_filter = n_events_before_filter,
      n_events_after_filter = nrow(ev)
    )
  )

  class(out) <- c("dm_epoch", "dm_epoch_extract")

  out
}


#' Test superposed epoch composites against a null distribution
#'
#' @description
#' Tests observed superposed epoch composites against empirical null
#' distributions generated by randomly sampled anchor times.
#'
#' If the input object was created with \code{Year} and/or \code{DOY} filters
#' in [dm_epoch_extract()] and \code{restrict_null_to_window = TRUE}, null
#' anchors are sampled from the same calendar-year and day-of-year window.
#'
#' @param x Object returned by [dm_epoch_extract()].
#' @param statistic Composite statistic. One of \code{"mean"} or
#'   \code{"median"}.
#' @param null Null model. One of \code{"same_doy_window"},
#'   \code{"same_month"}, or \code{"random_time"}.
#' @param n_iter Number of null iterations.
#' @param doy_window DOY window used for \code{null = "same_doy_window"}.
#' @param match_hour Logical. For hourly or minute data, keep the same hour
#'   when possible.
#' @param match_minute Logical. For minute data, keep the same minute when
#'   possible.
#' @param conf_level Confidence level for null envelopes.
#' @param seed Optional random seed.
#'
#' @return
#' An object of class \code{c("dm_epoch", "dm_epoch_test")}.
#'
#' @importFrom tibble as_tibble tibble
#' @importFrom dplyr %>% group_by summarise mutate left_join bind_rows
#' @importFrom stats sd quantile
#' @export
dm_epoch_test <- function(x,
                          statistic = c("mean", "median"),
                          null = c("same_doy_window", "same_month", "random_time"),
                          n_iter = 1000,
                          doy_window = 15,
                          match_hour = TRUE,
                          match_minute = TRUE,
                          conf_level = 0.95,
                          seed = NULL) {
  variable <- value <- null_stat <- extreme <- NULL

  if (!inherits(x, "dm_epoch_extract")) {
    stop("'x' must be output of dm_epoch_extract().")
  }

  statistic <- match.arg(statistic)
  null <- match.arg(null)

  if (!is.numeric(n_iter) ||
      length(n_iter) != 1 ||
      is.na(n_iter) ||
      n_iter < 1) {
    stop("'n_iter' must be a positive integer.")
  }

  n_iter <- as.integer(n_iter)

  if (!is.numeric(conf_level) ||
      length(conf_level) != 1 ||
      is.na(conf_level) ||
      conf_level <= 0 ||
      conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1.")
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  ep <- tibble::as_tibble(x$epoch_data)
  ev <- tibble::as_tibble(x$events)
  clim_tbl <- tibble::as_tibble(x$climate_data)

  vars_use <- x$settings$vars
  lag_values <- x$settings$lag_values
  unit <- x$settings$unit

  tz_use <- attr(clim_tbl$TIME, "tzone")

  if (is.null(tz_use) || length(tz_use) == 0) {
    tz_use <- "UTC"
  }

  observed <- ep %>%
    dplyr::group_by(variable, lag) %>%
    dplyr::summarise(
      observed = dmea_stat_fun(value, statistic = statistic),
      n = sum(!is.na(value)),
      .groups = "drop"
    )

  null_list <- vector("list", n_iter)

  clim_times_all <- as.POSIXct(
    clim_tbl$TIME,
    origin = "1970-01-01",
    tz = tz_use
  )

  use_restricted_null <- FALSE

  if (!is.null(x$settings$restrict_null_to_window)) {
    use_restricted_null <- isTRUE(x$settings$restrict_null_to_window)
  }

  if (isTRUE(use_restricted_null) &&
      (!is.null(x$settings$Year) || !is.null(x$settings$DOY))) {

    null_keep <- dmea_time_filter_index(
      times = clim_times_all,
      Year = x$settings$Year,
      DOY = x$settings$DOY
    )

    clim_times <- clim_times_all[null_keep]

    if (length(clim_times) == 0) {
      stop(
        "No climate timestamps remain for null-anchor sampling after applying ",
        "the Year/DOY filter. Set restrict_null_to_window = FALSE in ",
        "dm_epoch_extract() if you want to sample null anchors from the full ",
        "climate record."
      )
    }

  } else {
    clim_times <- clim_times_all
  }

  for (ii in seq_len(n_iter)) {
    anchors <- do.call(
      c,
      lapply(ev$event_time_aligned, function(tt) {
        dmea_sample_anchor_one(
          event_time = as.POSIXct(tt, origin = "1970-01-01", tz = tz_use),
          clim_times = clim_times,
          null = null,
          doy_window = doy_window,
          match_hour = match_hour,
          match_minute = match_minute,
          unit = unit
        )
      })
    )

    anchors <- as.POSIXct(
      anchors,
      origin = "1970-01-01",
      tz = tz_use
    )

    null_ep <- dmea_extract_from_anchors(
      anchor_times = anchors,
      clim_tbl = clim_tbl,
      lag_values = lag_values * dmea_step_seconds(1, unit),
      vars = vars_use
    )

    if (nrow(null_ep) == 0) {
      null_sum <- tibble::tibble(
        iter = ii,
        variable = character(0),
        lag = numeric(0),
        null_stat = numeric(0)
      )
    } else {
      null_sum <- null_ep %>%
        dplyr::mutate(
          lag = .data$lag / dmea_step_seconds(1, unit)
        ) %>%
        dplyr::group_by(variable, lag) %>%
        dplyr::summarise(
          null_stat = dmea_stat_fun(value, statistic = statistic),
          .groups = "drop"
        )

      null_sum$iter <- ii

      null_sum <- null_sum[
        ,
        c("iter", "variable", "lag", "null_stat"),
        drop = FALSE
      ]
    }

    null_list[[ii]] <- null_sum
  }

  null_long <- dplyr::bind_rows(null_list)

  alpha <- (1 - conf_level) / 2

  null_summary <- null_long %>%
    dplyr::group_by(variable, lag) %>%
    dplyr::summarise(
      null_mean = mean(null_stat, na.rm = TRUE),
      null_sd = stats::sd(null_stat, na.rm = TRUE),
      ci_low = stats::quantile(
        null_stat,
        probs = alpha,
        na.rm = TRUE,
        names = FALSE
      ),
      ci_high = stats::quantile(
        null_stat,
        probs = 1 - alpha,
        na.rm = TRUE,
        names = FALSE
      ),
      .groups = "drop"
    )

  summary_tbl <- dplyr::left_join(
    observed,
    null_summary,
    by = c("variable", "lag")
  )

  pvals <- null_long %>%
    dplyr::left_join(
      summary_tbl[
        ,
        c("variable", "lag", "observed", "null_mean"),
        drop = FALSE
      ],
      by = c("variable", "lag")
    ) %>%
    dplyr::mutate(
      extreme = abs(.data$null_stat - .data$null_mean) >=
        abs(.data$observed - .data$null_mean)
    ) %>%
    dplyr::group_by(variable, lag) %>%
    dplyr::summarise(
      p_value = mean(extreme, na.rm = TRUE),
      .groups = "drop"
    )

  summary_tbl <- dplyr::left_join(
    summary_tbl,
    pvals,
    by = c("variable", "lag")
  )

  summary_tbl$significant <- with(
    summary_tbl,
    observed < ci_low | observed > ci_high
  )

  out <- list(
    events = ev,
    epoch_data = ep,
    climate_data = clim_tbl,
    summary = summary_tbl,
    null_distribution = null_long,
    settings = c(
      x$settings,
      list(
        statistic = statistic,
        null = null,
        n_iter = n_iter,
        doy_window = doy_window,
        match_hour = match_hour,
        match_minute = match_minute,
        conf_level = conf_level,
        seed = seed
      )
    )
  )

  class(out) <- c("dm_epoch", "dm_epoch_test")

  out
}


#' Summarize a dm_epoch object
#'
#' @param object Object of class \code{"dm_epoch"}.
#' @param ... Unused.
#'
#' @return
#' An object of class \code{"summary.dm_epoch"}.
#'
#' @importFrom tibble as_tibble
#' @importFrom dplyr %>% group_by summarise
#' @export
summary.dm_epoch <- function(object, ...) {
  variable <- significant <- NULL

  if (!inherits(object, "dm_epoch")) {
    stop("'object' must inherit from 'dm_epoch'.")
  }

  if (inherits(object, "dm_epoch_test")) {
    tab <- tibble::as_tibble(object$summary)

    out <- list(
      n_events = nrow(object$events),
      variables = unique(tab$variable),
      lag_range = range(tab$lag, na.rm = TRUE),
      significant_counts = tab %>%
        dplyr::group_by(variable) %>%
        dplyr::summarise(
          n_significant = sum(significant %in% TRUE, na.rm = TRUE),
          .groups = "drop"
        ),
      summary_table = tab,
      settings = object$settings
    )
  } else {
    tab <- tibble::as_tibble(object$epoch_data)

    out <- list(
      n_events = nrow(object$events),
      variables = unique(tab$variable),
      lag_range = range(tab$lag, na.rm = TRUE),
      settings = object$settings
    )
  }

  class(out) <- "summary.dm_epoch"

  out
}


#' Print summary of a dm_epoch object
#'
#' @param x Object of class \code{"summary.dm_epoch"}.
#' @param ... Unused.
#'
#' @return
#' The input object, invisibly.
#'
#' @export
print.summary.dm_epoch <- function(x, ...) {
  cat("dm_epoch summary\n")
  cat("----------------\n")
  cat("Number of events:", x$n_events, "\n")
  cat("Variables:", paste(x$variables, collapse = ", "), "\n")
  cat("Lag range:", paste(x$lag_range, collapse = " to "), "\n")

  if (!is.null(x$settings$Year)) {
    cat("Year filter:", paste(x$settings$Year, collapse = ", "), "\n")
  }

  if (!is.null(x$settings$DOY)) {
    cat("DOY filter:", paste(x$settings$DOY, collapse = " to "), "\n")
  }

  if (!is.null(x$settings$n_events_before_filter) &&
      !is.null(x$settings$n_events_after_filter)) {
    cat(
      "Events before/after Year-DOY filtering:",
      x$settings$n_events_before_filter,
      "/",
      x$settings$n_events_after_filter,
      "\n"
    )
  }

  if (!is.null(x$significant_counts)) {
    cat("\nSignificant lags by variable:\n")
    print(x$significant_counts)
  }

  invisible(x)
}


#' Plot a dm_epoch object
#'
#' @param x Object of class \code{"dm_epoch"}.
#' @param y Unused.
#' @param type Plot type. One of \code{"composite"}, \code{"heatmap"}, or
#'   \code{"difference"}.
#' @param variables Optional climate variables to plot.
#' @param facet Logical. If \code{TRUE}, facet by variable.
#' @param show_ci Logical. If \code{TRUE} and \code{x} is a
#'   \code{"dm_epoch_test"} object, show the null confidence ribbon.
#' @param legend_position Legend position passed to [ggplot2::theme()].
#' @param ... Unused.
#'
#' @return
#' A \code{ggplot} object.
#'
#' @method plot dm_epoch
#' @importFrom tibble as_tibble
#' @importFrom dplyr %>% group_by summarise
#' @importFrom ggplot2 ggplot aes geom_ribbon geom_line geom_hline geom_vline
#'   geom_tile theme_bw theme element_text labs facet_wrap
#' @export
plot.dm_epoch <- function(x,
                          y = NULL,
                          type = c("composite", "heatmap", "difference"),
                          variables = NULL,
                          facet = TRUE,
                          show_ci = TRUE,
                          legend_position = "right",
                          ...) {
  variable <- value <- observed <- ci_low <- ci_high <- null_mean <- NULL
  diff_obs_null <- event_id <- NULL

  if (!inherits(x, "dm_epoch")) {
    stop("'x' must inherit from 'dm_epoch'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_epoch().")
  }

  type <- match.arg(type)

  lag_lab <- paste0("Lag (", x$settings$unit, ")")

  if (inherits(x, "dm_epoch_test")) {
    tab <- tibble::as_tibble(x$summary)
    ep <- tibble::as_tibble(x$epoch_data)
  } else {
    ep <- tibble::as_tibble(x$epoch_data)

    tab <- ep %>%
      dplyr::group_by(variable, lag) %>%
      dplyr::summarise(
        observed = mean(value, na.rm = TRUE),
        n = sum(!is.na(value)),
        .groups = "drop"
      )
  }

  if (!is.null(variables)) {
    tab <- tab[tab$variable %in% variables, , drop = FALSE]
    ep <- ep[ep$variable %in% variables, , drop = FALSE]
  }

  if (nrow(tab) == 0) {
    stop("No data remain after filtering by variable.")
  }

  if (type == "composite") {
    p <- ggplot2::ggplot(
      tab,
      ggplot2::aes(
        x = lag,
        y = observed,
        colour = variable,
        group = variable
      )
    )

    if (inherits(x, "dm_epoch_test") &&
        isTRUE(show_ci) &&
        all(c("ci_low", "ci_high") %in% names(tab))) {
      p <- p +
        ggplot2::geom_ribbon(
          data = tab,
          mapping = ggplot2::aes(
            x = lag,
            ymin = ci_low,
            ymax = ci_high,
            fill = variable,
            group = variable
          ),
          inherit.aes = FALSE,
          alpha = 0.18,
          colour = NA
        )
    }

    if (inherits(x, "dm_epoch_test") &&
        "null_mean" %in% names(tab)) {
      p <- p +
        ggplot2::geom_line(
          data = tab,
          mapping = ggplot2::aes(
            x = lag,
            y = null_mean,
            colour = variable,
            group = variable
          ),
          linetype = 2,
          linewidth = 0.6,
          alpha = 0.8,
          inherit.aes = FALSE
        )
    }

    p <- p +
      ggplot2::geom_hline(yintercept = 0, linetype = 2) +
      ggplot2::geom_vline(xintercept = 0, linetype = 2) +
      ggplot2::geom_line(linewidth = 0.9) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = lag_lab,
        y = paste0(
          "Composite ",
          if (!is.null(x$settings$statistic)) {
            x$settings$statistic
          } else {
            "mean"
          }
        ),
        colour = "Variable",
        fill = "Variable",
        title = "Superposed epoch composite"
      )

    if (isTRUE(facet)) {
      p <- p +
        ggplot2::facet_wrap(
          stats::as.formula("~ variable"),
          scales = "free_y",
          ncol = 1
        )
    }

    return(p)
  }

  if (type == "difference") {
    if (!inherits(x, "dm_epoch_test")) {
      stop("type = 'difference' requires output from dm_epoch_test().")
    }

    if (!"null_mean" %in% names(tab)) {
      stop("No null mean available in the dm_epoch_test object.")
    }

    tab$diff_obs_null <- tab$observed - tab$null_mean

    p <- ggplot2::ggplot(
      tab,
      ggplot2::aes(
        x = lag,
        y = diff_obs_null,
        colour = variable,
        group = variable
      )
    ) +
      ggplot2::geom_hline(yintercept = 0, linetype = 2) +
      ggplot2::geom_vline(xintercept = 0, linetype = 2) +
      ggplot2::geom_line(linewidth = 0.9) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = lag_lab,
        y = "Observed - null mean",
        colour = "Variable",
        title = "Difference between observed composite and null mean"
      )

    if (isTRUE(facet)) {
      p <- p +
        ggplot2::facet_wrap(
          stats::as.formula("~ variable"),
          scales = "free_y",
          ncol = 1
        )
    }

    return(p)
  }

  if (type == "heatmap") {
    ep2 <- ep
    ep2$event_id <- factor(
      ep2$event_id,
      levels = rev(sort(unique(ep2$event_id)))
    )

    p <- ggplot2::ggplot(
      ep2,
      ggplot2::aes(x = lag, y = event_id, fill = value)
    ) +
      ggplot2::geom_tile() +
      ggplot2::geom_vline(xintercept = 0, linetype = 2) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = lag_lab,
        y = "Event ID",
        fill = "Value",
        title = "Epoch heatmap"
      )

    if (isTRUE(facet)) {
      p <- p +
        ggplot2::facet_wrap(
          stats::as.formula("~ variable"),
          scales = "free_x",
          ncol = 1
        )
    }

    return(p)
  }

  stop("Unknown plot type.")
}


#' Null-coalescing helper
#'
#' @param a First object.
#' @param b Fallback object.
#'
#' @return \code{a} if not \code{NULL}, otherwise \code{b}.
#'
#' @keywords internal
#' @noRd
`%||%` <- function(a, b) {
  if (!is.null(a)) {
    a
  } else {
    b
  }
}

#' Plot detrended dendrometer series
#'
#' @description
#' S3 plotting method for objects returned by [dm.detrend.fit()].
#'
#' The default plot compares:
#' \itemize{
#'   \item original daily dendrometer series,
#'   \item fitted curve reconstructed on the original daily scale,
#'   \item residuals (`observed - fitted_original`),
#'   \item detrended standardized series.
#' }
#'
#' @param x An object of class \code{"dm_detrended"} returned by
#'   [dm.detrend.fit()].
#' @param y Unused.
#' @param type Plot type. One of:
#'   \describe{
#'     \item{`"compare"`}{Default three-panel comparison of original daily vs
#'       fitted, residuals, and detrended standardized series.}
#'     \item{`"fit"`}{Original daily and fitted original-scale values only.}
#'     \item{`"residual"`}{Residuals only.}
#'     \item{`"detrended"`}{Detrended standardized series only.}
#'     \item{`"boxplot"`}{Distribution of detrended standardized values by series
#'       or season.}
#'   }
#' @param series Optional character vector of dendrometer series to plot.
#'   Default is \code{NULL}, meaning all available series are used.
#' @param seasons Optional character vector of vegetation-season labels to plot.
#'   Default is \code{NULL}, meaning all seasons are used.
#' @param x_axis Character string controlling the x-axis. One of:
#'   \describe{
#'     \item{`"default"`}{Uses date for `"compare"` and `"fit"`, and season day
#'       for `"residual"` and `"detrended"`.}
#'     \item{`"date"`}{Use actual calendar date.}
#'     \item{`"season_day"`}{Use vegetation-season day.}
#'     \item{`"doy"`}{Use calendar day-of-year.}
#'   }
#' @param facet_by Character string controlling faceting. One of:
#'   \describe{
#'     \item{`"series"`}{Facet by dendrometer series.}
#'     \item{`"season"`}{Facet by vegetation season.}
#'     \item{`"none"`}{No faceting.}
#'   }
#' @param ncol Optional integer giving the number of columns in faceted plots
#'   where [ggplot2::facet_wrap()] is used.
#' @param box_group For \code{type = "boxplot"}, grouping variable on the x-axis.
#'   One of \code{"series"} or \code{"season"}.
#' @param show_observed Logical. If \code{TRUE}, original daily observations are
#'   shown in plot types where relevant. Default is \code{TRUE}.
#' @param show_fitted Logical. If \code{TRUE}, fitted original-scale values are
#'   shown in plot types where relevant. Default is \code{TRUE}.
#' @param point_alpha Numeric alpha level used for observed points. Default is
#'   \code{0.7}.
#' @param line_width Numeric line width used for fitted, residual, and detrended
#'   lines. Default is \code{0.8}.
#' @param legend_position Character string specifying legend position.
#'   Default is \code{"right"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return
#' A \code{ggplot2} object.
#'
#' @examples
#' \donttest{
#' fit1 <- dm.growth.fit(
#'   df = gf_nepa17,
#'   TreeNum = 1:2,
#'   method = "gompertz",
#'   year_mode = "yearly",
#'   verbose = FALSE
#' )
#'
#' det1 <- dm.detrend.fit(fit1)
#'
#' plot(det1)
#' plot(det1, type = "fit")
#' plot(det1, type = "residual")
#' plot(det1, type = "detrended")
#' plot(det1, type = "boxplot")
#' plot(det1, type = "compare", facet_by = "series")
#' plot(det1, type = "compare", facet_by = "season")
#' }
#'
#' @method plot dm_detrended
#' @importFrom dplyr filter mutate select %>%
#' @importFrom tibble as_tibble
#' @importFrom ggplot2 ggplot aes geom_point geom_line geom_hline geom_boxplot
#'   geom_jitter facet_wrap facet_grid theme_bw theme element_text labs
#' @export
plot.dm_detrended <- function(x,
                              y = NULL,
                              type = c("compare", "fit", "residual", "detrended", "boxplot"),
                              series = NULL,
                              seasons = NULL,
                              x_axis = c("default", "date", "season_day", "doy"),
                              facet_by = c("series", "season", "none"),
                              ncol = NULL,
                              box_group = c("series", "season"),
                              show_observed = TRUE,
                              show_fitted = TRUE,
                              point_alpha = 0.7,
                              line_width = 0.8,
                              legend_position = "right",
                              ...) {

  TIME <- doy <- season_day <- season_label <- series_name <- observed <- fitted_original <- NULL
  residual <- detrended_std <- x_plot <- facet_var <- colour_group <- group_id <- panel <- group_var <- NULL

  type <- match.arg(type)
  x_axis <- match.arg(x_axis)
  facet_by <- match.arg(facet_by)
  box_group <- match.arg(box_group)

  if (!inherits(x, "dm_detrended")) {
    stop("'x' must be an object of class 'dm_detrended'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_detrended().")
  }

  if (is.null(x$detrended_long) || !is.data.frame(x$detrended_long)) {
    stop("The object does not contain a valid 'detrended_long' table.")
  }

  dat <- tibble::as_tibble(x$detrended_long)

  if (!is.null(series)) {
    dat <- dat %>% dplyr::filter(.data$series %in% series)
  }
  if (!is.null(seasons)) {
    dat <- dat %>% dplyr::filter(.data$season_label %in% seasons)
  }

  if (nrow(dat) == 0) {
    stop("No rows remain after filtering by series and/or seasons.")
  }

  dat$season_label <- as.character(dat$season_label)
  dat$TIME_date <- as.Date(dat$TIME)

  x_cfg <- if (x_axis == "default") {
    if (type %in% c("compare", "fit")) {
      list(var = "TIME_date", label = "Date")
    } else {
      list(var = "season_day", label = "Season day")
    }
  } else if (x_axis == "date") {
    list(var = "TIME_date", label = "Date")
  } else if (x_axis == "doy") {
    list(var = "doy", label = "DOY")
  } else {
    list(var = "season_day", label = "Season day")
  }

  x_var <- x_cfg$var
  x_lab <- x_cfg$label

  grp_cfg <- dmdp_prepare_groups(dat, facet_by = facet_by)
  dat <- grp_cfg$data

  if (type == "compare") {
    p <- ggplot2::ggplot() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(grp_cfg$use_colour)) legend_position else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    obsfit_dat <- dat
    obsfit_dat$panel <- "Original vs fitted"

    resid_dat <- dat
    resid_dat$panel <- "Residual"

    detr_dat <- dat
    detr_dat$panel <- "Detrended standardized"

    if (isTRUE(show_observed)) {
      p <- p + ggplot2::geom_point(
        data = obsfit_dat[is.finite(obsfit_dat$observed), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$observed,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        alpha = point_alpha,
        size = 1.4
      )
    }

    if (isTRUE(show_fitted)) {
      p <- p + ggplot2::geom_line(
        data = obsfit_dat[is.finite(obsfit_dat$fitted_original), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$fitted_original,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        linewidth = line_width
      )
    }

    p <- p +
      ggplot2::geom_hline(
        data = data.frame(panel = "Residual"),
        mapping = ggplot2::aes(yintercept = 0),
        inherit.aes = FALSE,
        linetype = 2
      ) +
      ggplot2::geom_line(
        data = resid_dat[is.finite(resid_dat$residual), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$residual,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        linewidth = line_width
      ) +
      ggplot2::geom_hline(
        data = data.frame(panel = "Detrended standardized"),
        mapping = ggplot2::aes(yintercept = 1),
        inherit.aes = FALSE,
        linetype = 2
      ) +
      ggplot2::geom_line(
        data = detr_dat[is.finite(detr_dat$detrended_std), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$detrended_std,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        linewidth = line_width
      ) +
      ggplot2::labs(
        x = x_lab,
        y = NULL,
        colour = grp_cfg$legend_title,
        title = "Original daily, fitted, residual, and detrended standardized series"
      )

    if (facet_by == "none") {
      p <- p + ggplot2::facet_grid(panel ~ ., scales = "free_y")
    } else {
      p <- p + ggplot2::facet_grid(panel ~ facet_var, scales = "free_y")
    }

    return(p)
  }

  if (type == "fit") {
    p <- ggplot2::ggplot() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(grp_cfg$use_colour)) legend_position else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    if (isTRUE(show_observed)) {
      p <- p + ggplot2::geom_point(
        data = dat[is.finite(dat$observed), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$observed,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        alpha = point_alpha,
        size = 1.4
      )
    }

    if (isTRUE(show_fitted)) {
      p <- p + ggplot2::geom_line(
        data = dat[is.finite(dat$fitted_original), , drop = FALSE],
        mapping = ggplot2::aes(
          x = .data[[x_var]],
          y = .data$fitted_original,
          colour = .data$colour_group,
          group = .data$group_id
        ),
        linewidth = line_width
      )
    }

    p <- p + ggplot2::labs(
      x = x_lab,
      y = "Growth",
      colour = grp_cfg$legend_title,
      title = "Original daily and fitted original-scale growth"
    )

    if (isTRUE(grp_cfg$facet_enabled)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ facet_var"), scales = "free_y", ncol = ncol)
    }

    return(p)
  }

  if (type == "residual") {
    p <- ggplot2::ggplot(
      dat[is.finite(dat$residual), , drop = FALSE],
      ggplot2::aes(
        x = .data[[x_var]],
        y = .data$residual,
        colour = .data$colour_group,
        group = .data$group_id
      )
    ) +
      ggplot2::geom_hline(yintercept = 0, linetype = 2) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(grp_cfg$use_colour)) legend_position else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = x_lab,
        y = "Residual",
        colour = grp_cfg$legend_title,
        title = "Residuals from original daily and fitted original-scale growth"
      )

    if (isTRUE(grp_cfg$facet_enabled)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ facet_var"), scales = "free_y", ncol = ncol)
    }

    return(p)
  }

  if (type == "detrended") {
    p <- ggplot2::ggplot(
      dat[is.finite(dat$detrended_std), , drop = FALSE],
      ggplot2::aes(
        x = .data[[x_var]],
        y = .data$detrended_std,
        colour = .data$colour_group,
        group = .data$group_id
      )
    ) +
      ggplot2::geom_hline(yintercept = 1, linetype = 2) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(grp_cfg$use_colour)) legend_position else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = x_lab,
        y = "Detrended standardized",
        colour = grp_cfg$legend_title,
        title = "Detrended standardized dendrometer series"
      )

    if (isTRUE(grp_cfg$facet_enabled)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ facet_var"), scales = "free_y", ncol = ncol)
    }

    return(p)
  }

  if (type == "boxplot") {
    if (box_group == "series") {
      dat$group_var <- dat$series
      x_lab2 <- "Series"
    } else {
      dat$group_var <- dat$season_label
      x_lab2 <- "Season"
    }

    p <- ggplot2::ggplot(
      dat[is.finite(dat$detrended_std), , drop = FALSE],
      ggplot2::aes(x = .data$group_var, y = .data$detrended_std)
    ) +
      ggplot2::geom_boxplot(outlier.shape = NA) +
      ggplot2::geom_jitter(width = 0.15, alpha = point_alpha, size = 1.2) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      ) +
      ggplot2::labs(
        x = x_lab2,
        y = "Detrended standardized",
        title = "Distribution of detrended standardized values"
      )

    if (facet_by == "series" && box_group == "season") {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = ncol)
    } else if (facet_by == "season" && box_group == "series") {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ season_label"), scales = "free_y", ncol = ncol)
    }

    return(p)
  }

  stop("Unknown plot type.")
}


#' @keywords internal
dmdp_prepare_groups <- function(dat, facet_by = c("series", "season", "none")) {
  facet_by <- match.arg(facet_by)

  dat$group_id <- interaction(dat$series, dat$season_label, drop = TRUE)

  n_series <- length(unique(dat$series))
  n_seasons <- length(unique(dat$season_label))

  if (facet_by == "series") {
    dat$facet_var <- dat$series
    if (n_seasons > 1) {
      dat$colour_group <- factor(dat$season_label)
      use_colour <- TRUE
      legend_title <- "Season"
    } else {
      dat$colour_group <- factor("curve")
      use_colour <- FALSE
      legend_title <- NULL
    }
    return(list(
      data = dat,
      facet_enabled = TRUE,
      use_colour = use_colour,
      legend_title = legend_title
    ))
  }

  if (facet_by == "season") {
    dat$facet_var <- dat$season_label
    if (n_series > 1) {
      dat$colour_group <- factor(dat$series)
      use_colour <- TRUE
      legend_title <- "Series"
    } else {
      dat$colour_group <- factor("curve")
      use_colour <- FALSE
      legend_title <- NULL
    }
    return(list(
      data = dat,
      facet_enabled = TRUE,
      use_colour = use_colour,
      legend_title = legend_title
    ))
  }

  dat$facet_var <- "All"
  if (n_series > 1 && n_seasons > 1) {
    dat$colour_group <- factor(paste(dat$series, dat$season_label, sep = " | "))
    use_colour <- TRUE
    legend_title <- "Series | Season"
  } else if (n_series > 1) {
    dat$colour_group <- factor(dat$series)
    use_colour <- TRUE
    legend_title <- "Series"
  } else if (n_seasons > 1) {
    dat$colour_group <- factor(dat$season_label)
    use_colour <- TRUE
    legend_title <- "Season"
  } else {
    dat$colour_group <- factor("curve")
    use_colour <- FALSE
    legend_title <- NULL
  }

  list(
    data = dat,
    facet_enabled = FALSE,
    use_colour = use_colour,
    legend_title = legend_title
  )
}

#' @title Apply the Zero-Growth (ZG) approach to segment dendrometer data into TWD/GRO phases
#'
#' @description
#' Implements the Zero-Growth approach (Zweifel et al., 2016) on a single
#' dendrometer series to (i) classify each timestamp into \strong{tree water deficit}
#' (TWD; reversible shrinkage/expansion) or \strong{growth} (GRO; irreversible
#' expansion), (ii) summarize each contiguous phase (start/end, duration,
#' magnitude, rates, TWD statistics), and (iii) optionally smooth the raw
#' series prior to phase assignment to reduce spurious phase flips.
#'
#' The growth line is computed as the running cumulative maximum of the
#' (raw or smoothed) dendrometer series. TWD is defined as the vertical
#' distance between the growth line and the observed measurement.
#'
#' @details
#' \strong{Phase assignment.}
#' Let \eqn{y_t} be the dendrometer measurement (raw or smoothed). The growth
#' line is \eqn{g_t=\max_{s\le t}(y_s)}. Points with \eqn{y_t = g_t} are labeled
#' \strong{GRO} (phase = 2). Points with \eqn{y_t < g_t} are labeled \strong{TWD}
#' (phase = 1). Contiguous runs of identical labels are summarized into
#' individual phases.
#'
#' \strong{Summaries per phase (ZG_cycle).}
#' For each phase, the function returns:
#' \itemize{
#'   \item \code{Phases} (1 = TWD, 2 = GRO)
#'   \item \code{Start}, \code{End} (\code{POSIXct})
#'   \item \code{Duration_h} (hours)
#'   \item \code{Magnitude} (mm): for GRO only, computed from the \emph{growth line}
#'         as \eqn{\Delta g = g_{\mathrm{end}}-g_{\mathrm{start}}}; \code{NA} for TWD
#'   \item \code{rate} (\eqn{\mu}m/h): \code{Magnitude * 1000 / Duration_h} (GRO only)
#'   \item \code{max.twd} (mm), \code{Max.twd.time} (\code{POSIXct}): peak TWD and its time (TWD only)
#'   \item \code{Avg.twd} (mm), \code{STD.twd} (mm): mean and SD of TWD within the phase (TWD only)
#'   \item \code{AUC.load} (mm*h): area under the TWD curve from phase start to \code{Max.twd.time} (TWD only)
#'   \item \code{AUC.total} (mm*h): area under the full TWD curve across the phase (TWD only)
#'   \item \code{DOY}: day-of-year of the phase start
#'   \item \code{ABr.value}: \emph{Absolut Baumreaktion (ABr)} for TWD phases, defined here as
#'         \deqn{\mathrm{ABr} = \max(\mathrm{TWD}) \times \left(\frac{AUC_{load}}{\max(\mathrm{TWD})}\right)^{\beta}}
#'         returned as \code{NA} for GRO phases. This metric anchors severity on the event peak while
#'         allowing the loading duration to contribute with exponent \code{beta}.
#' }
#'
#' \strong{Point-level output (ZG_phase).}
#' The second element returns the input time series augmented with:
#' \itemize{
#'   \item \code{Phases} (1/2), \code{TWD} (= \code{GRO - dm}), and \code{GRO} (growth line).
#' }
#'
#' \strong{Smoothing (optional).}
#' If \code{smoothing} is provided (hours), the function uses
#' \code{smooth_dm(..., method = "median_mean", window_hours = smoothing)} to create a
#' smoothed series used \emph{only} for phase assignment; all magnitudes and TWD
#' statistics are still computed against the \emph{original} \code{dm}. This reduces
#' short-lived phase flips due to noise. Valid range is 1–24 hours.
#'
#' \strong{Temporal resolution.}
#' The function auto-detects the median sampling interval (in minutes) to
#' parameterize smoothing. If multiple distinct intervals are detected, a
#' warning is emitted (results may be degraded if resolution is inconsistent).
#'
#' \strong{Notes & caveats.}
#' \itemize{
#'   \item Negative \code{Magnitude} values in GRO phases indicate residual noise or
#'         insufficient smoothing; a warning is issued.
#'   \item Very short phases may occur around local extrema; consider smoothing
#'         or post-filtering minimal run lengths upstream if needed.
#'   \item Ensure timestamps are regular (or nearly regular) for best results.
#' }
#'
#' @references
#' Zweifel R, Haeni M, Buchmann N, Eugster W (2016) Are trees able to grow in periods of stem shrinkage?
#' \emph{New Phytologist}, 211:839–849. \doi{10.1111/nph.13995}
#'
#' @param df A data frame with the \strong{first column} containing date-time
#'   (character, \code{POSIXct}, or \code{Date} convertible to \code{POSIXct} using
#'   \code{lubridate::ymd_hms}) and the \strong{subsequent columns} containing one or
#'   more dendrometer series (mm) at constant temporal resolution.
#' @param TreeNum Integer index of the dendrometer series column to analyze.
#'   For example, \code{TreeNum = 1} refers to \code{df[[2]]} (the first dm column after
#'   the time column).
#' @param smoothing \code{NULL} (default) for no smoothing, or a numeric value
#'   between 1 and 24 specifying the smoothing window in \strong{hours} used by
#'   \code{smooth_dm(method = "median_mean")} for phase labeling.
#' @param beta Numeric scalar giving the exponent applied to the effective loading
#'   duration in the ABr formula. Defaults to \code{0.1}. Set \code{beta = 0} to
#'   make ABr depend only on \code{max.twd}.
#'
#' @return
#' A named list of class \code{"ZG_output"} with two tibbles:
#' \itemize{
#'   \item \code{ZG_cycle}: one row per contiguous phase with columns
#'         \code{Phases}, \code{Start}, \code{End}, \code{Duration_h}, \code{Magnitude}, \code{rate},
#'         \code{max.twd}, \code{Max.twd.time}, \code{AUC.load}, \code{AUC.total}, \code{ABr.value},
#'         \code{Avg.twd}, \code{STD.twd}, \code{DOY}.
#'   \item \code{ZG_phase}: point-level data with columns
#'         \code{TIME}, \code{dm}, \code{Phases}, \code{TWD}, \code{GRO}.
#' }
#'
#' @section Column definitions (ZG_cycle):
#' \describe{
#'   \item{Phases}{Integer; 1 = TWD (reversible shrinkage/expansion),
#'   2 = GRO (irreversible expansion).}
#'   \item{Start, End}{\code{POSIXct}; phase boundaries.}
#'   \item{Duration_h}{Numeric; phase duration in hours.}
#'   \item{Magnitude}{Numeric; GRO-only millimeter change of growth line across the phase. \code{NA} for TWD.}
#'   \item{rate}{Numeric; GRO-only rate in (\code{Magnitude*1000/Duration_h}).}
#'   \item{max.twd}{Numeric; peak TWD within the TWD phase.}
#'   \item{Max.twd.time}{\code{POSIXct}; time of \code{max.twd} within the TWD phase.}
#'   \item{AUC.load}{Numeric; pre-peak area under the TWD curve in mm*h, integrated from
#'   phase start to \code{Max.twd.time}.}
#'   \item{AUC.total}{Numeric; total area under the TWD curve in mm*h across the full TWD phase.}
#'   \item{ABr.value}{Numeric; Absolut Baumreaktion value for TWD phases,
#'   \deqn{\max(\mathrm{TWD}) \times \left(\frac{AUC_{load}}{\max(\mathrm{TWD})}\right)^{\beta}} \code{NA} for GRO.}
#'   \item{Avg.twd, STD.twd}{Numeric; mean and standard deviation of TWD in the phase.}
#'   \item{DOY}{Integer; day-of-year for the phase start.}
#' }
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(gf_nepa17)
#'
#' # Minimal example (no smoothing)
#' zg <- phase.zg(df = gf_nepa17[1:600, ], TreeNum = 1)
#' head(zg$ZG_cycle, 5)
#' head(zg$ZG_phase, 5)
#'
#' # With smoothing (e.g., 6 hours) to reduce short flips
#' zg6 <- phase.zg(df = gf_nepa17[1:600, ], TreeNum = 1, smoothing = 6, beta = 0.1)
#' subset(zg6$ZG_cycle, Phases == 1L)[1:5, c("Start","End","max.twd","ABr.value")]
#' }
#'
#' @importFrom lubridate ymd_hms ymd yday
#' @importFrom dplyr mutate rename select `%>%`
#' @importFrom tibble tibble
#' @importFrom stats median sd qnorm na.exclude
#' @importFrom moments kurtosis
#'
#' @export
phase.zg <- function(df, TreeNum, smoothing = NULL, beta = 0.1) {

  Duration_h <- NULL

  if (TreeNum > (ncol(df) - 1)) {
    stop("The tree number is not valid for the provided dataset.")
  }

  if (!is.numeric(beta) || length(beta) != 1 || is.na(beta) || beta < 0) {
    stop("beta must be a single non-negative numeric value.")
  }

  # -------------------------------------------------------
  # Helpers
  # -------------------------------------------------------
  parse_time_dm <- function(x) {
    if (inherits(x, "POSIXct")) return(as.POSIXct(x))
    if (inherits(x, "Date")) return(as.POSIXct(x))

    out <- suppressWarnings(lubridate::ymd_hms(x, quiet = TRUE))
    if (all(is.na(out))) {
      out_date <- suppressWarnings(lubridate::ymd(x, quiet = TRUE))
      if (!all(is.na(out_date))) {
        out <- as.POSIXct(out_date)
      }
    }
    if (all(is.na(out))) {
      out <- suppressWarnings(as.POSIXct(x))
    }
    out
  }

  cummax_na <- function(x) {
    out <- rep(NA_real_, length(x))
    current_max <- NA_real_

    for (i in seq_along(x)) {
      if (!is.na(x[i])) {
        if (is.na(current_max)) {
          current_max <- x[i]
        } else {
          current_max <- max(current_max, x[i])
        }
      }
      out[i] <- current_max
    }
    out
  }

  phase_cal_zg <- function(y) {
    all_max <- cummax_na(y)
    phases <- rep(NA_integer_, length(y))

    ok <- !is.na(y) & !is.na(all_max)
    phases[ok & y == all_max] <- 2L
    phases[ok & y <  all_max] <- 1L

    phases
  }

  reso_den <- function(input_time) {
    time1 <- as.POSIXct(input_time)
    dmin <- diff(as.numeric(time1)) / 60
    dmin <- dmin[is.finite(dmin) & dmin > 0]

    if (length(dmin) == 0) {
      warning("Could not infer temporal resolution from the time column.")
      return(NA_real_)
    }

    uniq <- unique(round(dmin, 6))
    if (length(uniq) > 1) {
      warning(
        "The temporal resolution of the dendrometer data is not fully consistent. ",
        "Results may be affected if the time series contains irregular spacing or missing intervals."
      )
    }

    stats::median(dmin)
  }

  trapz_time <- function(time_vec, value_vec) {
    if (length(time_vec) < 2 || length(value_vec) < 2) return(0)

    xh <- as.numeric(difftime(time_vec, time_vec[1], units = "hours"))
    keep <- is.finite(xh) & !is.na(value_vec)
    xh <- xh[keep]
    yv <- value_vec[keep]

    if (length(xh) < 2) return(0)

    sum(diff(xh) * (yv[-1] + yv[-length(yv)]) / 2)
  }

  phase_stats_zg <- function(dm_data, y_class) {

    gro_crv <- cummax_na(dm_data$dm)
    twd_crv <- gro_crv - dm_data$dm

    n <- nrow(dm_data)
    prev_y <- c(NA_integer_, y_class[-n])
    next_y <- c(y_class[-1], NA_integer_)

    start_idx <- which(!is.na(y_class) & (is.na(prev_y) | y_class != prev_y))
    end_idx   <- which(!is.na(y_class) & (is.na(next_y) | y_class != next_y))

    if (length(start_idx) == 0) {
      out_cycle <- tibble::tibble(
        Phases = integer(),
        Start = as.POSIXct(character()),
        End = as.POSIXct(character()),
        Duration_h = numeric(),
        Magnitude = numeric(),
        rate = numeric(),
        max.twd = numeric(),
        Max.twd.time = as.POSIXct(character()),
        AUC.load = numeric(),
        AUC.total = numeric(),
        ABr.value = numeric(),
        Avg.twd = numeric(),
        STD.twd = numeric(),
        DOY = integer()
      )

      out_phase <- dm_data %>%
        dplyr::mutate(
          Phases = y_class,
          TWD = twd_crv,
          GRO = gro_crv
        )

      return(list(out_cycle, tibble::as_tibble(out_phase)))
    }

    ph <- y_class[start_idx]
    strt_t <- dm_data$TIME[start_idx]

    # End is the boundary time (start of next phase), last phase ends at last timestamp
    boundary_idx <- pmin(end_idx + 1L, n)
    end_t <- dm_data$TIME[boundary_idx]

    dur_h <- as.numeric(difftime(end_t, strt_t, units = "hours"))

    # Growth magnitude over each non-overlapping phase segment
    magn <- rep(NA_real_, length(start_idx))
    gro_idx <- which(ph == 2L)
    if (length(gro_idx) > 0) {
      magn[gro_idx] <- gro_crv[end_idx[gro_idx]] - gro_crv[start_idx[gro_idx]]
    }

    rate <- rep(NA_real_, length(magn))
    ok_rate <- !is.na(magn) & !is.na(dur_h) & dur_h > 0
    rate[ok_rate] <- magn[ok_rate] * 1000 / dur_h[ok_rate]

    max_values <- rep(NA_real_, length(start_idx))
    max_time   <- rep(as.POSIXct(NA), length(start_idx))
    avg_twd    <- rep(NA_real_, length(start_idx))
    std_twd    <- rep(NA_real_, length(start_idx))
    auc_load   <- rep(NA_real_, length(start_idx))
    auc_total  <- rep(NA_real_, length(start_idx))
    abr_value  <- rep(NA_real_, length(start_idx))

    twd_idx <- which(ph == 1L)

    if (length(twd_idx) > 0) {
      for (k in twd_idx) {
        rng <- start_idx[k]:end_idx[k]
        twd_seg <- twd_crv[rng]
        time_seg <- dm_data$TIME[rng]

        if (!all(is.na(twd_seg))) {
          mx <- max(twd_seg, na.rm = TRUE)
          max_values[k] <- mx

          # last time of the maximum within phase
          wmx <- which(twd_seg == mx)
          max_time[k] <- time_seg[max(wmx)]

          avg_twd[k] <- mean(twd_seg, na.rm = TRUE)
          std_twd[k] <- stats::sd(twd_seg, na.rm = TRUE)

          peak_pos <- max(wmx)
          load_times <- c(strt_t[k], time_seg[seq_len(peak_pos)])
          load_vals  <- c(0, twd_seg[seq_len(peak_pos)])
          total_times <- c(strt_t[k], time_seg, end_t[k])
          total_vals  <- c(0, twd_seg, 0)

          auc_load[k] <- trapz_time(load_times, load_vals)
          auc_total[k] <- trapz_time(total_times, total_vals)

          if (!is.na(max_values[k]) && max_values[k] > 0 && !is.na(auc_load[k])) {
            abr_value[k] <- max_values[k] * ((auc_load[k] / max_values[k])^beta)
          }
        }
      }
    }

    dy <- lubridate::yday(strt_t)

    out_cycle <- tibble::tibble(
      Phases = ph,
      Start = strt_t,
      End = end_t,
      Duration_h = dur_h,
      Magnitude = magn,
      rate = rate,
      max.twd = max_values,
      Max.twd.time = as.POSIXct(max_time, origin = "1970-01-01", tz = attr(dm_data$TIME, "tzone")),
      AUC.load = auc_load,
      AUC.total = auc_total,
      ABr.value = abr_value,
      Avg.twd = avg_twd,
      STD.twd = std_twd,
      DOY = dy
    ) %>%
      dplyr::filter(Duration_h >= 0)

    out_phase <- dm_data %>%
      dplyr::mutate(
        Phases = y_class,
        TWD = twd_crv,
        GRO = gro_crv
      )

    list(out_cycle, tibble::as_tibble(out_phase))
  }

  # -------------------------------------------------------
  # Prepare input
  # -------------------------------------------------------
  df[[1]] <- parse_time_dm(df[[1]])
  if (all(is.na(df[[1]]))) {
    stop("Could not parse the first column as date-time.")
  }

  dm_data <- tibble::as_tibble(df) %>%
    dplyr::select(c(1, TreeNum + 1)) %>%
    dplyr::rename(
      TIME = 1,
      dm = 2
    )

  r.denro <- reso_den(dm_data$TIME)
  sf <- smoothing

  # -------------------------------------------------------
  # Optional smoothing for phase assignment
  # -------------------------------------------------------
  if (is.null(sf)) {
    y_for_phase <- dm_data$dm
  } else {
    if (!is.numeric(sf) || length(sf) != 1 || is.na(sf) || sf < 1 || sf > 24) {
      stop("smoothing must be a single numeric value between 1 and 24 hours.")
    }

    warning(
      paste0(
        "You are applying smoothing to raw dendrometer data. ",
        "The smoothing value is ", sf, " hour(s)."
      )
    )

    y_for_phase <- smooth_dm(
      time = dm_data$TIME,
      dm = dm_data$dm,
      method = "median_mean",
      window_hours = sf
    )
  }

  # -------------------------------------------------------
  # Classify phases and summarize
  # -------------------------------------------------------
  y_class <- phase_cal_zg(y_for_phase)
  ot <- phase_stats_zg(dm_data, y_class)

  out <- list(
    ZG_cycle = ot[[1]],
    ZG_phase = ot[[2]]
  )

  class(out) <- "ZG_output"
  out
}

#' @title Detection and interpolation of missing values in dendrometer data
#'
#' @description
#' Detects missing timestamps (based on provided resolution), inserts rows with \code{NA},
#' and optionally interpolates missing values using either cubic spline interpolation
#' (\code{\link[zoo:na.spline]{na.spline}}) or seasonal decomposition interpolation
#' (\code{\link[forecast:na.interp]{na.interp}}).
#'
#' Optionally, the function can test the reliability of interpolation for increasing
#' gap lengths (e.g., 6 hours to 3 days) using real data. This allows the user to assess
#' how well a given interpolation method recovers missing values over different durations.
#'
#' @param df A data frame with the first column as datetime (\code{"yyyy-mm-dd HH:MM:SS"},
#'   or POSIXct/Date), followed by dendrometer values.
#' @param resolution Integer. Temporal resolution in minutes (e.g., 60 for hourly data).
#' @param fill Logical. If \code{TRUE}, missing values will be interpolated.
#' @param method Character. Interpolation method (only used if \code{fill = TRUE}):
#'   \itemize{
#'     \item \code{"spline"} --- cubic spline (\code{zoo::na.spline}).
#'     \item \code{"seasonal"} --- seasonal decomposition (\code{forecast::na.interp}).
#'   }
#' @param assess Logical. If \code{TRUE}, run a simulation-based test to evaluate how well
#'   interpolation performs at different gap lengths (on existing data).
#' @param assess_lengths_hours Integer vector. The gap durations (in hours) to test during
#'   assessment. Default: \code{seq(6, 72, by = 6)}.
#' @param assess_samples_per_length Integer. Number of artificial gaps per gap length
#'   per series. Default: \code{10}.
#' @param assess_buffer_hours Integer. Minimum number of hours of valid data required
#'   before and after the artificial gap to allow valid assessment. Default: \code{6}.
#' @param assess_seed Integer or \code{NULL}. Random seed to ensure reproducibility
#'   of sampling windows. Default: \code{NULL}.
#' @param verbose Logical. If \code{TRUE}, prints messages during filling and testing.
#'
#' @details
#' The assessment simulates interpolation over artificial gaps of different durations.
#' For each dendrometer series and each gap length:
#' \enumerate{
#'   \item Random windows (with clean data) are selected.
#'   \item Data in the window is temporarily set to \code{NA}.
#'   \item The gap is interpolated using the selected method.
#'   \item The predicted values are compared to the original (true) values using:
#'     \itemize{
#'       \item MAPE --- Mean Absolute Percentage Error.
#'       \item MdAPE --- Median Absolute Percentage Error.
#'       \item RMSE_pct --- Root Mean Square Error (\%).
#'       \item Bias_pct --- Mean Percentage Error (\%).
#'       \item Max_diff_abs --- Mean absolute difference in daily maximum.
#'       \item Min_diff_abs --- Mean absolute difference in daily minimum.
#'       \item Time_max_diff_h --- Mean absolute difference in time of daily maximum (hours).
#'       \item Time_min_diff_h --- Mean absolute difference in time of daily minimum (hours).
#'     }
#' }
#'
#' @return
#' A list of class \code{"dm_na_interpolation"} with components:
#' \describe{
#'   \item{\code{$data}}{Data frame containing the original or gap-filled series.}
#'   \item{\code{$gap_info}}{Table describing timestamp gaps and internal NA runs.}
#'   \item{\code{$assessment}}{A tibble summarizing interpolation accuracy for each
#'   series and gap length, or \code{NULL} when \code{assess = FALSE}.}
#'   \item{\code{$filled}}{Logical indicating whether interpolation was performed.}
#'   \item{\code{$assessed}}{Logical indicating whether interpolation assessment was performed.}
#'   \item{\code{$method}}{Interpolation method used.}
#'   \item{\code{$resolution}}{Temporal resolution in minutes.}
#' }
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(nepa17)
#'
#' #res0 <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#' #plot(res0)
#' #plot(res0, type = "gaps")
#'
#' #res1 <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "spline"
#' #)
#' #plot(res1)
#' #plot(res1, type = "gaps")
#' #plot(res1, type = "interpolation", original = nepa17[1:1000, ])
#'
#' #res2 <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "seasonal",
#' #  assess = TRUE
#' # )
#' #plot(res2)
#' #plot(res2, type = "assessment", metric = "MdAPE")
#' }
#'
#' @importFrom stats ts median
#' @importFrom lubridate ymd_hms minutes
#' @importFrom dplyr mutate rename select bind_cols first lead last left_join summarise group_by arrange bind_rows relocate
#' @importFrom tibble as_tibble tibble
#' @importFrom zoo na.spline
#' @importFrom forecast na.interp
#' @importFrom utils head
#' @export
dm.na.interpolation <- function(df, resolution, fill = FALSE, method = "spline",
                                assess = FALSE,
                                assess_lengths_hours = seq(6, 72, by = 6),
                                assess_samples_per_length = 10,
                                assess_buffer_hours = 6,
                                assess_seed = NULL,
                                verbose = FALSE) {

  to_posix <- function(x) {
    if (inherits(x, "POSIXct")) return(x)
    if (inherits(x, "Date")) return(as.POSIXct(x))
    lubridate::ymd_hms(x)
  }
  TIME <- time_min <- diff_time <- start_time <- end_time <- type <- series <- prev_time <- next_time <- NULL
  start_missing <- end_missing <- n_consecutive <- duration_min <- duration_hr <- Series <- n_cases <- n_points <- MAPE <- MdAPE <- NULL
  RMSE_pct <- Bias_pct <- Max_diff_abs <- Min_diff_abs <- Time_max_diff_h <- Time_min_diff_h  <- NULL

  make_output <- function(data, gap_info, assessment = NULL,
                          filled = FALSE, assessed = FALSE) {
    out <- list(
      data = tibble::as_tibble(data),
      gap_info = tibble::as_tibble(gap_info),
      assessment = if (is.null(assessment)) NULL else tibble::as_tibble(assessment),
      filled = filled,
      assessed = assessed,
      method = method,
      resolution = resolution
    )
    class(out) <- "dm_na_interpolation"
    out
  }

  replace_na_with_spline <- function(column) {
    if (!is.numeric(column) || !anyNA(column)) return(column)
    if (sum(!is.na(column)) < 2) return(column)

    sp <- tryCatch(
      zoo::na.spline(column),
      error = function(e) column
    )

    if (length(sp) == length(column)) {
      column[is.na(column)] <- sp[is.na(column)]
    }
    column
  }

  replace_na_with_seasonality <- function(column, s) {
    if (!is.numeric(column) || !anyNA(column)) return(column)
    if (sum(!is.na(column)) < 2) return(column)

    tryCatch(
      as.numeric(forecast::na.interp(stats::ts(column, frequency = s))),
      error = function(e) column
    )
  }

  pct_errors <- function(truth, fit) {
    pe  <- 100 * (fit - truth) / truth
    ape <- abs(pe)
    c(
      MAPE     = mean(ape, na.rm = TRUE),
      MdAPE    = stats::median(ape, na.rm = TRUE),
      RMSE_pct = sqrt(mean(pe^2, na.rm = TRUE)),
      Bias_pct = mean(pe, na.rm = TRUE)
    )
  }

  quiet <- function(x) suppressWarnings(suppressMessages(x))

  mean_or_na <- function(x) {
    if (length(x) == 0 || all(is.na(x))) return(NA_real_)
    mean(x, na.rm = TRUE)
  }

  find_na_runs <- function(time_vec, x, series_name) {
    na_flag <- is.na(x)

    if (!any(na_flag)) {
      return(tibble::tibble(
        series = character(0),
        start_time = as.POSIXct(character()),
        end_time = as.POSIXct(character()),
        n_consecutive = integer(0)
      ))
    }

    r <- rle(na_flag)
    ends <- cumsum(r$lengths)
    starts <- c(1, utils::head(ends, -1) + 1)
    sel <- which(r$values)

    if (!length(sel)) {
      return(tibble::tibble(
        series = character(0),
        start_time = as.POSIXct(character()),
        end_time = as.POSIXct(character()),
        n_consecutive = integer(0)
      ))
    }

    tibble::tibble(
      series = series_name,
      start_time = time_vec[starts[sel]],
      end_time = time_vec[ends[sel]],
      n_consecutive = r$lengths[sel]
    )
  }

  if (!(method %in% c("spline", "seasonal"))) {
    stop("The method must be either 'spline' or 'seasonal'.")
  }

  df <- tibble::as_tibble(df)
  names(df)[1] <- "TIME"
  df$TIME <- to_posix(df$TIME)

  if (any(is.na(df$TIME))) {
    stop("The datetime column could not be fully converted to POSIXct.")
  }

  df2 <- df %>%
    dplyr::mutate(
      time_min = as.integer(difftime(TIME, dplyr::first(TIME), units = "mins")),
      diff_time = dplyr::lead(time_min) - time_min
    )

  gap_rows <- which(df2$diff_time > resolution)

  if (length(gap_rows) > 0L) {
    ts_gap_info <- tibble::tibble(
      type = "timestamp_gap",
      series = "ALL",
      prev_time = df2$TIME[gap_rows],
      next_time = df2$TIME[gap_rows] + lubridate::minutes(df2$diff_time[gap_rows]),
      start_missing = df2$TIME[gap_rows] + lubridate::minutes(resolution),
      end_missing = df2$TIME[gap_rows] + lubridate::minutes(df2$diff_time[gap_rows] - resolution),
      n_consecutive = as.integer(df2$diff_time[gap_rows] / resolution - 1L),
      duration_min = as.integer(df2$diff_time[gap_rows] - resolution),
      duration_hr = (df2$diff_time[gap_rows] - resolution) / 60
    )
  } else {
    ts_gap_info <- tibble::tibble(
      type = character(0),
      series = character(0),
      prev_time = as.POSIXct(character()),
      next_time = as.POSIXct(character()),
      start_missing = as.POSIXct(character()),
      end_missing = as.POSIXct(character()),
      n_consecutive = integer(0),
      duration_min = integer(0),
      duration_hr = numeric(0)
    )
  }

  df <- dplyr::select(df2, -time_min, -diff_time)

  if (nrow(ts_gap_info) > 0L) {
    x1 <- dplyr::first(df$TIME)
    x2 <- dplyr::last(df$TIME)
    full_time <- tibble::tibble(
      TIME = seq(from = x1, to = x2, by = paste(resolution, "mins"))
    )
    df_work <- dplyr::left_join(full_time, df, by = "TIME")
  } else {
    df_work <- df
  }

  na_runs_list <- lapply(names(df)[-1], function(nm) {
    find_na_runs(df$TIME, df[[nm]], nm)
  })
  na_runs_info <- dplyr::bind_rows(na_runs_list)

  if (nrow(na_runs_info)) {
    na_runs_info <- na_runs_info %>%
      dplyr::mutate(
        type = "internal_na",
        prev_time = as.POSIXct(NA),
        next_time = as.POSIXct(NA),
        duration_min = NA_integer_,
        duration_hr = NA_real_
      ) %>%
      dplyr::rename(
        start_missing = start_time,
        end_missing = end_time
      ) %>%
      dplyr::relocate(
        type, series, prev_time, next_time, start_missing, end_missing,
        n_consecutive, duration_min, duration_hr
      )
  }

  gap_info <- dplyr::bind_rows(ts_gap_info, na_runs_info)

  if (nrow(ts_gap_info) == 0L && nrow(na_runs_info) == 0L) {
    message("There is no gap in your dendrometer data.")
    return(make_output(
      data = df,
      gap_info = gap_info,
      assessment = NULL,
      filled = FALSE,
      assessed = FALSE
    ))
  }

  if (nrow(ts_gap_info) == 0L && nrow(na_runs_info) > 0L) {
    message("There is no gap in timestamp but there are internal NA values.")
    print(gap_info)

    if (!fill) {
      return(make_output(
        data = df,
        gap_info = gap_info,
        assessment = NULL,
        filled = FALSE,
        assessed = FALSE
      ))
    }
  }

  if (nrow(ts_gap_info) > 0L) {
    print(gap_info)

    if (!fill) {
      return(make_output(
        data = df_work,
        gap_info = gap_info,
        assessment = NULL,
        filled = FALSE,
        assessed = FALSE
      ))
    }
  }

  if (verbose) {
    message("Filling missing values using method = '", method, "'.")
  }

  if (nrow(ts_gap_info) > 0L) {
    if (max(ts_gap_info$n_consecutive, na.rm = TRUE) >= (1440 / resolution)) {
      warning("There is a timestamp gap lasting more than 24 h; interpolation may be unreliable.")
    }
  }

  if (method == "spline") {
    df_vals <- lapply(df_work[-1], replace_na_with_spline)
  } else {
    s <- 1440 / resolution
    df_vals <- lapply(df_work[-1], function(col) replace_na_with_seasonality(col, s))
  }

  df_filled <- dplyr::bind_cols(df_work[1], tibble::as_tibble(df_vals))

  if (!assess) {
    return(make_output(
      data = df_filled,
      gap_info = gap_info,
      assessment = NULL,
      filled = TRUE,
      assessed = FALSE
    ))
  }

  if (!is.null(assess_seed)) {
    set.seed(assess_seed)
  }

  if (verbose) {
    message(
      "Running self test on artificial gaps: ",
      paste(assess_lengths_hours, collapse = ", "),
      " hours"
    )
  }

  time_dates <- as.Date(df_filled$TIME)
  time_hours <- as.numeric(format(df_filled$TIME, "%H")) +
    as.numeric(format(df_filled$TIME, "%M")) / 60 +
    as.numeric(format(df_filled$TIME, "%S")) / 3600

  circular_hour_diff <- function(x, y) {
    d <- abs(x - y)
    pmin(d, 24 - d)
  }

  fast_daily_extrema <- function(date_vec, hour_vec, value_vec) {
    udates <- unique(date_vec)

    out <- lapply(udates, function(dd) {
      idx <- which(date_vec == dd)
      vv <- value_vec[idx]
      hh <- hour_vec[idx]

      if (length(vv) == 0 || all(is.na(vv))) {
        return(data.frame(
          DATE = dd,
          Min = NA_real_,
          Time_min_h = NA_real_,
          Max = NA_real_,
          Time_max_h = NA_real_
        ))
      }

      i_min <- which.min(replace(vv, is.na(vv), Inf))[1]
      i_max <- which.max(replace(vv, is.na(vv), -Inf))[1]

      data.frame(
        DATE = dd,
        Min = min(vv, na.rm = TRUE),
        Time_min_h = hh[i_min],
        Max = max(vv, na.rm = TRUE),
        Time_max_h = hh[i_max]
      )
    })

    do.call(rbind, out)
  }

  daily_case_differences_fast <- function(idx_days, truth, fit) {
    out_na <- c(
      Max_diff_abs = NA_real_,
      Min_diff_abs = NA_real_,
      Time_max_diff_h = NA_real_,
      Time_min_diff_h = NA_real_
    )

    if (!any(idx_days)) {
      return(out_na)
    }

    truth_daily <- fast_daily_extrema(
      date_vec = time_dates[idx_days],
      hour_vec = time_hours[idx_days],
      value_vec = truth[idx_days]
    )

    fit_daily <- fast_daily_extrema(
      date_vec = time_dates[idx_days],
      hour_vec = time_hours[idx_days],
      value_vec = fit[idx_days]
    )

    merged <- merge(
      truth_daily,
      fit_daily,
      by = "DATE",
      suffixes = c("_truth", "_fit")
    )

    if (nrow(merged) == 0) {
      return(out_na)
    }

    c(
      Max_diff_abs = mean(abs(merged$Max_fit - merged$Max_truth), na.rm = TRUE),
      Min_diff_abs = mean(abs(merged$Min_fit - merged$Min_truth), na.rm = TRUE),
      Time_max_diff_h = mean(
        circular_hour_diff(merged$Time_max_h_fit, merged$Time_max_h_truth),
        na.rm = TRUE
      ),
      Time_min_diff_h = mean(
        circular_hour_diff(merged$Time_min_h_fit, merged$Time_min_h_truth),
        na.rm = TRUE
      )
    )
  }

  impute_with_gap <- function(series, gap_idx, meth) {
    tmp <- series
    tmp[gap_idx] <- NA_real_

    if (meth == "spline") {
      tryCatch(
        as.numeric(quiet(zoo::na.spline(tmp))),
        error = function(e) tmp
      )
    } else {
      s <- 1440 / resolution
      tryCatch(
        as.numeric(quiet(forecast::na.interp(stats::ts(tmp, frequency = s)))),
        error = function(e) tmp
      )
    }
  }

  pts_per_hour <- max(1L, round(60 / resolution))
  buffer_pts <- max(1L, round(assess_buffer_hours * pts_per_hour))
  n_total <- nrow(df_filled)

  max_rows <- (ncol(df_filled) - 1L) *
    length(assess_lengths_hours) *
    assess_samples_per_length

  stat_rows <- vector("list", max_rows)
  row_id <- 0L

  for (j in seq.int(2, ncol(df_filled))) {
    series_name <- names(df_filled)[j]
    y_full <- df_filled[[j]]

    ok_full <- is.finite(y_full)
    cs_ok <- c(0L, cumsum(ok_full))

    for (gap_h in assess_lengths_hours) {
      gap_pts <- max(1L, round(gap_h * pts_per_hour))

      start_min <- buffer_pts + 1L
      start_max <- n_total - gap_pts - buffer_pts + 1L

      if (start_max < start_min) {
        if (verbose) {
          message("No valid windows for ", series_name, " at ", gap_h, " h.")
        }
        next
      }

      starts <- seq.int(start_min, start_max)

      win_start <- starts - buffer_pts
      win_end <- starts + gap_pts - 1L + buffer_pts
      required_n <- gap_pts + 2L * buffer_pts

      finite_counts <- cs_ok[win_end + 1L] - cs_ok[win_start]
      valid_starts <- starts[finite_counts == required_n]

      if (length(valid_starts) == 0L) {
        if (verbose) {
          message("No valid windows for ", series_name, " at ", gap_h, " h.")
        }
        next
      }

      starts_to_use <- if (length(valid_starts) > assess_samples_per_length) {
        sample(valid_starts, assess_samples_per_length, replace = FALSE)
      } else {
        valid_starts
      }

      for (i in starts_to_use) {
        idx_gap <- i:(i + gap_pts - 1L)

        y_imputed <- impute_with_gap(y_full, idx_gap, method)
        errs_all <- pct_errors(y_full[idx_gap], y_imputed[idx_gap])

        affected_dates <- unique(time_dates[idx_gap])
        idx_days <- time_dates %in% affected_dates

        daily_errs <- daily_case_differences_fast(
          idx_days = idx_days,
          truth = y_full,
          fit = y_imputed
        )

        row_id <- row_id + 1L
        stat_rows[[row_id]] <- data.frame(
          Series = series_name,
          gap_h = gap_h,
          n_cases = 1L,
          n_points = length(idx_gap),
          MAPE = errs_all["MAPE"],
          MdAPE = errs_all["MdAPE"],
          RMSE_pct = errs_all["RMSE_pct"],
          Bias_pct = errs_all["Bias_pct"],
          Max_diff_abs = daily_errs["Max_diff_abs"],
          Min_diff_abs = daily_errs["Min_diff_abs"],
          Time_max_diff_h = daily_errs["Time_max_diff_h"],
          Time_min_diff_h = daily_errs["Time_min_diff_h"],
          stringsAsFactors = FALSE
        )
      }
    }
  }

  if (row_id == 0L) {
    warning("Assessment produced no results (insufficient clean windows).")
    assessment <- tibble::tibble()
  } else {
    raw_stats <- do.call(rbind, stat_rows[seq_len(row_id)])

    assessment <- as.data.frame(raw_stats) %>%
      dplyr::group_by(Series, gap_h) %>%
      dplyr::summarise(
        n_cases = sum(n_cases),
        n_points = sum(n_points),
        MAPE = mean_or_na(MAPE),
        MdAPE = mean_or_na(MdAPE),
        RMSE_pct = mean_or_na(RMSE_pct),
        Bias_pct = mean_or_na(Bias_pct),
        Max_diff_abs = mean_or_na(Max_diff_abs),
        Min_diff_abs = mean_or_na(Min_diff_abs),
        Time_max_diff_h = mean_or_na(Time_max_diff_h),
        Time_min_diff_h = mean_or_na(Time_min_diff_h),
        .groups = "drop"
      ) %>%
      dplyr::arrange(Series, gap_h)
  }

  make_output(
    data = df_filled,
    gap_info = gap_info,
    assessment = assessment,
    filled = TRUE,
    assessed = TRUE
  )
}


#' @title Plot method for dendrometer NA interpolation results
#'
#' @description
#' S3 plot method for objects returned by \code{\link{dm.na.interpolation}}.
#'
#' @param x Object returned by \code{dm.na.interpolation()}.
#' @param type Character. Plot type:
#'   \code{"gaps"}, \code{"interpolation"}, or \code{"assessment"}.
#'   If omitted, the default is:
#'   \itemize{
#'     \item \code{"assessment"} when assessment exists,
#'     \item \code{"interpolation"} when filled data exist,
#'     \item \code{"gaps"} when gaps exist but data were not filled,
#'     \item otherwise \code{"interpolation"}.
#'   }
#' @param series Optional character vector of series names to plot.
#' @param metric Character assessment metric used when \code{type = "assessment"}.
#' @param original Optional original input data frame used before interpolation.
#' @param start Optional start datetime for subsetting.
#' @param end Optional end datetime for subsetting.
#' @param free_y Logical. If \code{TRUE}, facets use free y scales.
#' @param ncol Integer. Number of facet columns.
#' @param facet Logical. If \code{TRUE}, facet assessment plots by series.
#' @param empty_gaps Character. Behavior when \code{type = "gaps"} but no gaps are
#'   present: \code{"plot"} for an informative empty plot, or \code{"error"}.
#' @param ... Further arguments.
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(nepa17)
#'
#' ## No gaps: defaults to time-series plot
#' #res0 <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#' #plot(res0)
#' #plot(res0, type = "gaps")
#' #plot(res0, type = "gaps", empty_gaps = "error")
#'
#' ## Filled data
#' #res1 <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "spline"
#' #)
#' #plot(res1)
#' #plot(res1, type = "gaps")
#' #plot(res1, type = "interpolation", original = nepa17[1:1000, ])
#'
#' ## Assessed data
#' #res2 <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "seasonal",
#' #  assess = TRUE
#' #)
#' #plot(res2)
#' #plot(res2, type = "assessment", metric = "MdAPE")
#' }
#'
#' @method plot dm_na_interpolation
#' @export
plot.dm_na_interpolation <- function(x,
                                     type = NULL,
                                     series = NULL,
                                     metric = "MdAPE",
                                     original = NULL,
                                     start = NULL,
                                     end = NULL,
                                     free_y = TRUE,
                                     ncol = 1,
                                     facet = TRUE,
                                     empty_gaps = c("plot", "error"),
                                     ...) {
  if (!inherits(x, "dm_na_interpolation")) {
    stop("x must be an object returned by dm.na.interpolation().")
  }

  empty_gaps <- match.arg(empty_gaps)

  has_assessment <- isTRUE(x$assessed) &&
    !is.null(x$assessment) &&
    nrow(x$assessment) > 0

  has_gaps <- !is.null(x$gap_info) &&
    nrow(x$gap_info) > 0

  if (is.null(type)) {
    if (has_assessment) {
      type <- "assessment"
    } else if (isTRUE(x$filled)) {
      type <- "interpolation"
    } else if (has_gaps) {
      type <- "gaps"
    } else {
      type <- "interpolation"
    }
  }

  type <- match.arg(type, choices = c("gaps", "interpolation", "assessment"))

  if (type == "gaps") {
    return(plot_dm_gaps(
      x = x,
      series = series,
      empty_gaps = empty_gaps
    ))
  }

  if (type == "interpolation") {
    return(plot_dm_interpolation(
      x = x,
      original = original,
      series = series,
      start = start,
      end = end,
      free_y = free_y,
      ncol = ncol
    ))
  }

  if (type == "assessment") {
    if (!has_assessment) {
      stop("Assessment plot is only available when dm.na.interpolation(..., assess = TRUE) produced results.")
    }

    return(plot_dm_assessment(
      x = x,
      metric = metric,
      series = series,
      facet = facet
    ))
  }
}


#' @title Plot detected gaps in dendrometer data
#'
#' @param x Object returned by \code{dm.na.interpolation()}.
#' @param series Optional character vector of series names to plot.
#' @param empty_gaps Character. Behavior when no gaps are present:
#'   \code{"plot"} for an informative empty plot, or \code{"error"}.
#'
#' @return A \code{ggplot2} object.
#' @examples
#' \donttest{
#' #res <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#' #plot_dm_gaps(res)
#' }
#' @export
plot_dm_gaps <- function(x, series = NULL, empty_gaps = c("plot", "error")) {
  empty_gaps <- match.arg(empty_gaps)
  start_missing <- end_missing <- type <- NULL
  parts <- .dm_interp_extract_parts(x)
  data <- parts$data
  gap_info <- parts$gap_info

  if (is.null(gap_info) || nrow(gap_info) == 0) {
    if (empty_gaps == "error") {
      stop("No gap information available.")
    }
    return(.dm_interp_empty_plot("No gaps detected in the data."))
  }

  series_all <- names(data)[-1]

  if (is.null(series)) {
    series <- series_all
  } else {
    series <- intersect(series, series_all)
    if (!length(series)) {
      stop("None of the requested 'series' were found in the result.")
    }
  }

  gap_plot <- .dm_interp_expand_gap_info(gap_info, series_all = series_all)
  gap_plot <- gap_plot[gap_plot$series %in% series, , drop = FALSE]

  if (nrow(gap_plot) == 0) {
    if (empty_gaps == "error") {
      stop("No gaps available for the selected series.")
    }
    return(.dm_interp_empty_plot("No gaps available for the selected series."))
  }

  gap_plot$series <- factor(gap_plot$series, levels = rev(series))

  ggplot2::ggplot(gap_plot) +
    ggplot2::geom_segment(
      ggplot2::aes(
        x = start_missing,
        xend = end_missing,
        y = series,
        yend = series,
        colour = type
      ),
      linewidth = 5,
      alpha = 0.6,
      lineend = "butt"
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = start_missing, y = series, colour = type),
      size = 1.5
    ) +
    ggplot2::labs(
      x = "Time",
      y = "Series",
      colour = "Gap type",
      title = "Detected gaps in dendrometer data"
    ) +
    ggplot2::theme_bw()
}


#' @title Plot original and interpolated dendrometer series
#'
#' @param x Object returned by \code{dm.na.interpolation()}.
#' @param original Optional original input data frame used in
#'   \code{dm.na.interpolation()}.
#' @param series Optional character vector of series names to plot.
#' @param start Optional start datetime for plotting subset.
#' @param end Optional end datetime for plotting subset.
#' @param free_y Logical. If \code{TRUE}, each facet gets its own y-scale.
#' @param ncol Integer. Number of facet columns.
#'
#' @return A \code{ggplot2} object.
#' @examples
#' \donttest{
#' #res <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "spline"
#' #)
#' #plot_dm_interpolation(res, original = nepa17[1:1000, ])
#' }
#' @export
plot_dm_interpolation <- function(x, original = NULL, series = NULL,
                                  start = NULL, end = NULL,
                                  free_y = TRUE, ncol = 1) {
  TIME <- xmin <- xmax <- type <- filled <- NULL
  parts <- .dm_interp_extract_parts(x)
  plot_data <- parts$data
  gap_info <- parts$gap_info

  plot_data <- tibble::as_tibble(plot_data)
  names(plot_data)[1] <- "TIME"

  series_all <- names(plot_data)[-1]

  if (is.null(series)) {
    series <- series_all
  } else {
    series <- intersect(series, series_all)
    if (!length(series)) {
      stop("None of the requested 'series' were found in the result.")
    }
  }

  plot_data <- plot_data[, c("TIME", series), drop = FALSE]

  if (!is.null(start)) {
    plot_data <- plot_data[plot_data$TIME >= as.POSIXct(start), , drop = FALSE]
  }
  if (!is.null(end)) {
    plot_data <- plot_data[plot_data$TIME <= as.POSIXct(end), , drop = FALSE]
  }

  filled_long <- tidyr::pivot_longer(
    plot_data,
    cols = -TIME,
    names_to = "series",
    values_to = "filled"
  )

  if (!is.null(original)) {
    original <- tibble::as_tibble(original)
    names(original)[1] <- "TIME"

    miss_series <- setdiff(series, names(original))
    if (length(miss_series)) {
      stop(
        "These series are missing in 'original': ",
        paste(miss_series, collapse = ", ")
      )
    }

    original <- dplyr::left_join(
      plot_data["TIME"],
      original[, c("TIME", series), drop = FALSE],
      by = "TIME"
    )

    if (!is.null(start)) {
      original <- original[original$TIME >= as.POSIXct(start), , drop = FALSE]
    }
    if (!is.null(end)) {
      original <- original[original$TIME <= as.POSIXct(end), , drop = FALSE]
    }

    original_long <- tidyr::pivot_longer(
      original,
      cols = -TIME,
      names_to = "series",
      values_to = "original"
    )

    plot_df <- dplyr::left_join(
      filled_long,
      original_long,
      by = c("TIME", "series")
    )

    plot_df$imputed <- is.na(plot_df$original) & !is.na(plot_df$filled)

  } else {
    plot_df <- filled_long
    plot_df$original <- NA_real_
    plot_df$imputed <- FALSE

    if (!is.null(gap_info) && nrow(gap_info) > 0) {
      gap_plot <- .dm_interp_expand_gap_info(gap_info, series_all = series_all)
      gap_plot <- gap_plot[gap_plot$series %in% series, , drop = FALSE]

      if (!is.null(start)) {
        gap_plot <- gap_plot[gap_plot$end_missing >= as.POSIXct(start), , drop = FALSE]
      }
      if (!is.null(end)) {
        gap_plot <- gap_plot[gap_plot$start_missing <= as.POSIXct(end), , drop = FALSE]
      }

      for (i in seq_len(nrow(gap_plot))) {
        sel <- plot_df$series == gap_plot$series[i] &
          plot_df$TIME >= gap_plot$start_missing[i] &
          plot_df$TIME <= gap_plot$end_missing[i]
        plot_df$imputed[sel] <- TRUE
      }
    }
  }

  gap_rect <- NULL
  if (!is.null(gap_info) && nrow(gap_info) > 0) {
    gap_rect <- .dm_interp_expand_gap_info(gap_info, series_all = series_all)
    gap_rect <- gap_rect[gap_rect$series %in% series, , drop = FALSE]

    if (!is.null(start)) {
      gap_rect <- gap_rect[gap_rect$end_missing >= as.POSIXct(start), , drop = FALSE]
    }
    if (!is.null(end)) {
      gap_rect <- gap_rect[gap_rect$start_missing <= as.POSIXct(end), , drop = FALSE]
    }

    if (nrow(gap_rect) > 0) {
      step_sec <- .dm_interp_time_step_seconds(plot_data$TIME)
      gap_rect$xmin <- gap_rect$start_missing - step_sec / 2
      gap_rect$xmax <- gap_rect$end_missing + step_sec / 2
    }
  }

  title_txt <- if (isTRUE(x$filled)) {
    "Original and interpolated dendrometer series"
  } else {
    "Dendrometer series"
  }

  p <- ggplot2::ggplot(plot_df, ggplot2::aes(x = TIME)) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Time",
      y = "Dendrometer value",
      title = title_txt
    )

  if (!is.null(gap_rect) && nrow(gap_rect) > 0) {
    p <- p +
      ggplot2::geom_rect(
        data = gap_rect,
        ggplot2::aes(
          xmin = xmin, xmax = xmax,
          ymin = -Inf, ymax = Inf,
          fill = type
        ),
        inherit.aes = FALSE,
        alpha = 0.12
      )
  }

  if (any(!is.na(plot_df$original))) {
    p <- p +
      ggplot2::geom_line(
        ggplot2::aes(y = original),
        linewidth = 0.35,
        colour = "grey45",
        na.rm = TRUE
      ) +
      ggplot2::geom_line(
        ggplot2::aes(y = filled),
        linewidth = 0.45,
        colour = "#1F78B4",
        na.rm = TRUE
      ) +
      ggplot2::geom_point(
        data = plot_df[plot_df$imputed, , drop = FALSE],
        ggplot2::aes(y = filled),
        colour = "#D7301F",
        size = 1.1,
        na.rm = TRUE
      )
  } else {
    p <- p +
      ggplot2::geom_line(
        ggplot2::aes(y = filled),
        linewidth = 0.45,
        colour = "#1F78B4",
        na.rm = TRUE
      )

    if (any(plot_df$imputed)) {
      p <- p +
        ggplot2::geom_point(
          data = plot_df[plot_df$imputed, , drop = FALSE],
          ggplot2::aes(y = filled),
          colour = "#D7301F",
          size = 1.1,
          na.rm = TRUE
        )
    }
  }

  p +
    ggplot2::facet_wrap(
      ~series,
      scales = if (free_y) "free_y" else "fixed",
      ncol = ncol
    ) +
    ggplot2::labs(fill = "Gap type")
}


#' @title Plot interpolation assessment metrics
#'
#' @param x Object returned by \code{dm.na.interpolation()} with
#'   \code{assess = TRUE}.
#' @param metric Character. One of:
#'   \code{"MAPE"}, \code{"MdAPE"}, \code{"RMSE_pct"}, \code{"Bias_pct"},
#'   \code{"Max_diff_abs"}, \code{"Min_diff_abs"},
#'   \code{"Time_max_diff_h"}, \code{"Time_min_diff_h"}.
#' @param series Optional character vector of series names to plot.
#' @param facet Logical. If \code{TRUE}, create one panel per series.
#'
#' @return A \code{ggplot2} object.
#' @examples
#' \donttest{
#' #res <- dm.na.interpolation(
#' #  nepa17[1:1000, ],
#' #  resolution = 60,
#' #  fill = TRUE,
#' #  method = "seasonal",
#' #  assess = TRUE
#' #)
#' #plot_dm_assessment(res, metric = "MdAPE")
#' }
#' @export
plot_dm_assessment <- function(x, metric = "MdAPE", series = NULL, facet = TRUE) {
  gap_h <- value <- Series <- NULL
  parts <- .dm_interp_extract_parts(x)
  assessment <- parts$assessment

  if (is.null(assessment) || nrow(assessment) == 0) {
    stop("No assessment found. Run dm.na.interpolation(..., assess = TRUE).")
  }

  valid_metrics <- c(
    "MAPE", "MdAPE", "RMSE_pct", "Bias_pct",
    "Max_diff_abs", "Min_diff_abs",
    "Time_max_diff_h", "Time_min_diff_h"
  )

  if (!(metric %in% valid_metrics)) {
    stop(
      "Invalid metric. Choose one of: ",
      paste(valid_metrics, collapse = ", ")
    )
  }

  assessment <- tibble::as_tibble(assessment)

  if (!is.null(series)) {
    assessment <- assessment[assessment$Series %in% series, , drop = FALSE]
    if (nrow(assessment) == 0) {
      stop("No matching series found in assessment.")
    }
  }

  assessment$value <- assessment[[metric]]

  p <- ggplot2::ggplot(
    assessment,
    ggplot2::aes(x = gap_h, y = value, group = Series, colour = Series)
  ) +
    ggplot2::geom_line(linewidth = 0.5, na.rm = TRUE) +
    ggplot2::geom_point(size = 1.8, na.rm = TRUE) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Artificial gap length (hours)",
      y = metric,
      colour = "Series",
      title = paste("Interpolation assessment:", metric)
    )

  if (facet) {
    p + ggplot2::facet_wrap(~Series, scales = "free_y")
  } else {
    p
  }
}

# -------------------------------------------------------------------------
# Internal helpers
# ----------------------------------------------------------------
.dm_interp_extract_parts <- function(x) {
  if (!inherits(x, "dm_na_interpolation")) {
    stop("x must be an object returned by dm.na.interpolation().")
  }

  list(
    data = tibble::as_tibble(x$data),
    gap_info = x$gap_info,
    assessment = x$assessment
  )
}


.dm_interp_expand_gap_info <- function(gap_info, series_all) {
  if (is.null(gap_info) || nrow(gap_info) == 0) {
    return(tibble::tibble(
      type = character(0),
      series = character(0),
      start_missing = as.POSIXct(character()),
      end_missing = as.POSIXct(character()),
      n_consecutive = integer(0)
    ))
  }

  out <- list()
  k <- 0L

  ts_rows <- gap_info[gap_info$type == "timestamp_gap", , drop = FALSE]
  if (nrow(ts_rows) > 0) {
    for (i in seq_len(nrow(ts_rows))) {
      for (s in series_all) {
        k <- k + 1L
        out[[k]] <- tibble::tibble(
          type = ts_rows$type[i],
          series = s,
          start_missing = ts_rows$start_missing[i],
          end_missing = ts_rows$end_missing[i],
          n_consecutive = ts_rows$n_consecutive[i]
        )
      }
    }
  }

  na_rows <- gap_info[gap_info$type == "internal_na", , drop = FALSE]
  if (nrow(na_rows) > 0) {
    na_rows <- na_rows[na_rows$series %in% series_all, , drop = FALSE]
    if (nrow(na_rows) > 0) {
      for (i in seq_len(nrow(na_rows))) {
        k <- k + 1L
        out[[k]] <- tibble::tibble(
          type = na_rows$type[i],
          series = na_rows$series[i],
          start_missing = na_rows$start_missing[i],
          end_missing = na_rows$end_missing[i],
          n_consecutive = na_rows$n_consecutive[i]
        )
      }
    }
  }

  if (!length(out)) {
    return(tibble::tibble(
      type = character(0),
      series = character(0),
      start_missing = as.POSIXct(character()),
      end_missing = as.POSIXct(character()),
      n_consecutive = integer(0)
    ))
  }

  dplyr::bind_rows(out)
}


.dm_interp_time_step_seconds <- function(time_vec) {
  if (length(time_vec) < 2) {
    return(0)
  }
  stats::median(diff(as.numeric(time_vec)), na.rm = TRUE)
}


.dm_interp_empty_plot <- function(label = "No gaps detected in the data.") {
  ggplot2::ggplot() +
    ggplot2::annotate("text", x = 0, y = 0, label = label, size = 5) +
    ggplot2::xlim(-1, 1) +
    ggplot2::ylim(-1, 1) +
    ggplot2::labs(
      title = "Detected gaps in dendrometer data",
      x = NULL,
      y = NULL
    ) +
    ggplot2::theme_void()
}

#' @title Join subdaily climate features to point-level dendrometer output
#'
#' @description
#' Joins timestamp-level subdaily climate features to point-level output from
#' \code{phase.zg()} or \code{phase.sc()}.
#'
#' The climate input can be:
#' \itemize{
#'   \item the output of \code{dm_subdaily_clim()}
#'   \item the output of \code{read.climate()}
#'   \item a raw climate data frame
#'   \item a valid climate file path readable by \code{read.climate()}
#' }
#'
#' If the supplied climate input is not already a subdaily climate-feature table,
#' the function attempts to standardize it with \code{read.climate()} and then
#' joins by exact timestamp using the \code{TIME} column.
#'
#' @param x Object returned by \code{phase.zg()} or \code{phase.sc()}.
#' @param clim_sub Climate input. This can be a subdaily climate-feature table,
#'   a standardized climate object returned by \code{read.climate()}, a raw
#'   climate data frame, or a valid climate file path.
#'
#' @return
#' The same phase object with climate features appended to \code{ZG_phase} or
#' \code{SC_phase}. The returned object has class
#' \code{c("ZG_output_clim", "ZG_output", ...)} or
#' \code{c("SC_output_clim", "SC_output", ...)}.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#'
#' clim_sub <- dm_subdaily_clim(
#'   ktm_clim_hourly,
#'   mean_vars = c("temp", "VPD", "RH"),
#'   sum_vars  = c("prec"),
#'   lag_vars  = c("temp", "VPD", "RH"),
#'   roll_hours = c(1, 3, 6, 24),
#'   lag_hours  = c(1, 3, 6, 24)
#' )
#'
#' zg_point_clim <- dm_join_subdaily_clim(zg, clim_sub)
#' head(zg_point_clim$ZG_phase)
#' }
#'
#' @importFrom dplyr left_join
#' @importFrom tibble as_tibble
#' @export
dm_join_subdaily_clim <- function(x, clim_sub) {

  if (!inherits(x, "ZG_output") && !inherits(x, "SC_output")) {
    stop("'x' must be an object returned by phase.zg() or phase.sc().")
  }

  # standardize climate input
  clim <- if (inherits(clim_sub, "subdaily_clim") || inherits(clim_sub, "dm_clim")) {
    tibble::as_tibble(clim_sub)
  } else if (is.data.frame(clim_sub) || (is.character(clim_sub) && length(clim_sub) == 1 && file.exists(clim_sub))) {
    tibble::as_tibble(read.climate(clim_sub, verbose = FALSE))
  } else {
    stop(
      "'clim_sub' must be one of: output of dm_subdaily_clim(), output of read.climate(), ",
      "a raw climate data frame, or a valid climate file path."
    )
  }

  if (!"TIME" %in% names(clim)) {
    stop("Climate input must contain a 'TIME' column after standardization.")
  }

  clim$TIME <- as.POSIXct(clim$TIME)

  phase_name <- if (inherits(x, "ZG_output")) "ZG_phase" else "SC_phase"
  ph <- tibble::as_tibble(x[[phase_name]])

  if (!"TIME" %in% names(ph)) {
    names(ph)[1] <- "TIME"
  }

  ph$TIME <- as.POSIXct(ph$TIME)

  out <- x
  out[[phase_name]] <- dplyr::left_join(ph, clim, by = "TIME")

  if (inherits(x, "ZG_output")) {
    class(out) <- unique(c("ZG_output_clim", class(x)))
  } else {
    class(out) <- unique(c("SC_output_clim", class(x)))
  }

  out
}

#' @title Running correlation between dendrometer data and climate
#'
#' @description
#' Calculates running correlations between a selected daily dendrometer summary
#' and one or more climate variables. The user can select the daily dendrometer
#' statistic, correlation method, optional bootstrap confidence intervals, and
#' lagged / antecedent climate transformations.
#'
#' @details
#' The dendrometer series is first aggregated to daily resolution. The daily
#' dendrometer statistic used for correlation is controlled by \code{dm_stat}:
#' \itemize{
#'   \item \code{"mean"}: daily mean dendrometer value
#'   \item \code{"min"}: daily minimum
#'   \item \code{"max"}: daily maximum
#'   \item \code{"median"}: daily median
#'   \item \code{"amplitude"}: daily amplitude (\code{max - min})
#'   \item \code{"change"}: day-to-day change in the daily mean
#' }
#'
#' Users can choose the climate variables to analyze via \code{clim_vars}.
#' Climate transformation settings can be given as:
#' \itemize{
#'   \item a single value applied to all selected climate variables
#'   \item an unnamed vector with one value per selected climate variable
#'   \item a named vector mapping each selected climate variable to its own setting
#' }
#'
#' This applies to \code{clim_fun}, \code{lag_days}, and \code{accum_days}.
#'
#' @param df A data frame with the first column containing date-time in the format
#'   \code{yyyy-mm-dd HH:MM:SS} (or convertible to \code{POSIXct}) and dendrometer
#'   series in the following columns.
#' @param Clim A data frame with the first column containing daily date
#'   (\code{yyyy-mm-dd}, \code{Date}, or convertible) and one or more climate
#'   variables in the following columns.
#' @param TreeNum Integer indicating the dendrometer series to analyze.
#' @param win_size Integer giving the running window size in days. Minimum is 18.
#' @param cor_method Correlation method: one of \code{"pearson"},
#'   \code{"kendall"}, or \code{"spearman"}.
#' @param boot Logical. If \code{TRUE}, bootstrap confidence intervals are
#'   computed for each running correlation.
#' @param R Integer number of bootstrap iterations.
#' @param boot.ci Numeric confidence level selector: one of \code{0.01},
#'   \code{0.05} (default), or \code{0.1}.
#' @param set_seed Integer seed for reproducibility of bootstrap results.
#' @param dm_stat Daily dendrometer statistic used for correlation. One of
#'   \code{"mean"}, \code{"min"}, \code{"max"}, \code{"median"},
#'   \code{"amplitude"}, or \code{"change"}.
#' @param clim_vars Optional character vector of climate variables to analyze.
#'   If \code{NULL}, all numeric climate variables are used.
#' @param lag_days Climate lag in days. Can be:
#'   \itemize{
#'     \item one value for all selected climate variables
#'     \item one value per selected climate variable
#'     \item a named numeric vector keyed by climate variable name
#'   }
#' @param accum_days Antecedent window length in days for climate transformation.
#'   Can be scalar, per-variable, or a named numeric vector.
#' @param clim_fun Climate transformation over the antecedent window. One of
#'   \code{"raw"}, \code{"mean"}, \code{"sum"}, \code{"max"},
#'   \code{"min"}, or \code{"median"}. Can be scalar, per-variable, or a named
#'   character vector.
#' @param min_complete Minimum number of complete paired observations required in
#'   a running window to calculate correlation. If \code{NULL}, defaults to
#'   \code{max(3, ceiling(0.8 * win_size))}.
#' @param p_adjust_method Method for p-value adjustment in the non-bootstrap
#'   output. Passed to \code{p.adjust()}. Use \code{"none"} to skip adjustment.
#'
#' @return
#' A list with class \code{"mov_cor_dm"} (and \code{"mov_cor_dm_boot"} if
#' bootstrapped) containing:
#' \itemize{
#'   \item \code{results}: named list of tibbles, one per climate variable
#'   \item \code{metadata}: analysis metadata
#'   \item \code{call}: the matched function call
#' }
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(gf_nepa17)
#' data(ktm_rain17)
#'
#' # one common climate transformation for all selected variables
#' out_corr <- mov.cor.dm(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   TreeNum = 1,
#'   win_size = 21,
#'   clim_fun = "raw"
#' )
#' print(out_corr)
#' summary(out_corr)
#'
#' # variable-specific climate transformations
#' out_varfun <- mov.cor.dm(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   TreeNum = 1,
#'   win_size = 21,
#'   clim_vars = c("rainfall"),
#'   clim_fun = c(rainfall = "sum"),
#'   lag_days = c(rainfall = 1),
#'   accum_days = c(rainfall = 7)
#' )
#'
#' # bootstrap confidence intervals
#' out_boot <- mov.cor.dm(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   TreeNum = 1,
#'   win_size = 21,
#'   boot = TRUE,
#'   R = 250
#' )
#' summary(out_boot)
#' }
#'
#' @importFrom lubridate ymd_hms ymd
#' @importFrom dplyr mutate group_by summarise rename select filter %>%
#' @importFrom tibble as_tibble tibble
#' @importFrom stats cor.test cor median sd p.adjust quantile
#' @importFrom boot boot
#' @export
mov.cor.dm <- function(df,
                       Clim,
                       TreeNum,
                       win_size,
                       cor_method = c("pearson", "kendall", "spearman"),
                       boot = FALSE,
                       R = 1000,
                       boot.ci = 0.05,
                       set_seed = 1,
                       dm_stat = c("mean", "min", "max", "median", "amplitude", "change"),
                       clim_vars = NULL,
                       lag_days = 0,
                       accum_days = 1,
                       clim_fun = "raw",
                       min_complete = NULL,
                       p_adjust_method = "BH") {
  TIME <- DATE <- dm <- TIME <- NULL

  cor_method <- match.arg(cor_method)
  dm_stat <- match.arg(dm_stat)

  allowed_clim_fun <- c("raw", "mean", "sum", "max", "min", "median")

  if (!is.numeric(TreeNum) || length(TreeNum) != 1 || is.na(TreeNum)) {
    stop("'TreeNum' must be a single numeric value.")
  }
  if (TreeNum > ncol(df) - 1) {
    stop("The tree number is not valid for the provided dendrometer dataset.")
  }

  if (!is.numeric(win_size) || length(win_size) != 1 || is.na(win_size) || win_size < 18) {
    stop("'win_size' must be a single numeric value >= 18.")
  }

  if (!is.logical(boot) || length(boot) != 1) {
    stop("'boot' must be TRUE or FALSE.")
  }

  if (!boot.ci %in% c(0.01, 0.05, 0.1)) {
    stop("'boot.ci' must be one of 0.01, 0.05, or 0.1.")
  }

  if (!is.numeric(R) || length(R) != 1 || is.na(R) || R <= 0) {
    stop("'R' must be a positive number.")
  }

  if (is.null(min_complete)) {
    min_complete <- max(3L, ceiling(0.8 * win_size))
  } else {
    if (!is.numeric(min_complete) || length(min_complete) != 1 || is.na(min_complete) || min_complete < 3) {
      stop("'min_complete' must be a single numeric value >= 3.")
    }
    min_complete <- as.integer(min_complete)
  }

  if (!p_adjust_method %in% c("holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", "none")) {
    stop("'p_adjust_method' must be one of the supported p.adjust methods or 'none'.")
  }

  parse_datetime <- function(x) {
    if (inherits(x, "POSIXct")) return(as.POSIXct(x))
    if (inherits(x, "Date")) return(as.POSIXct(x))

    out <- suppressWarnings(lubridate::ymd_hms(x, quiet = TRUE))
    if (all(is.na(out))) {
      out_date <- suppressWarnings(lubridate::ymd(x, quiet = TRUE))
      if (!all(is.na(out_date))) out <- as.POSIXct(out_date)
    }
    if (all(is.na(out))) out <- suppressWarnings(as.POSIXct(x))
    out
  }

  parse_date_only <- function(x) {
    if (inherits(x, "Date")) return(as.Date(x))
    if (inherits(x, "POSIXct")) return(as.Date(x))

    out <- suppressWarnings(lubridate::ymd(x, quiet = TRUE))
    if (all(is.na(out))) {
      out_dt <- suppressWarnings(lubridate::ymd_hms(x, quiet = TRUE))
      if (!all(is.na(out_dt))) out <- as.Date(out_dt)
    }
    if (all(is.na(out))) out <- suppressWarnings(as.Date(x))
    out
  }

  resolve_per_var_arg <- function(arg, var_names, arg_name, allowed = NULL) {
    if (length(arg) == 1 && is.null(names(arg))) {
      out <- rep(arg, length(var_names))
      names(out) <- var_names
    } else if (!is.null(names(arg))) {
      miss <- setdiff(var_names, names(arg))
      if (length(miss) > 0) {
        stop(sprintf(
          "The following selected climate variables are missing in '%s': %s",
          arg_name, paste(miss, collapse = ", ")
        ))
      }
      out <- arg[var_names]
    } else if (length(arg) == length(var_names)) {
      out <- arg
      names(out) <- var_names
    } else {
      stop(sprintf(
        "'%s' must be either length 1, length equal to 'clim_vars', or a named vector.",
        arg_name
      ))
    }

    if (!is.null(allowed)) {
      bad <- setdiff(unique(as.character(out)), allowed)
      if (length(bad) > 0) {
        stop(sprintf(
          "Unsupported value(s) in '%s': %s",
          arg_name, paste(bad, collapse = ", ")
        ))
      }
    }

    out
  }

  transform_clim_series <- function(x, lag_days, accum_days, clim_fun) {
    n <- length(x)
    out <- rep(NA_real_, n)

    for (i in seq_len(n)) {
      end_idx <- i - lag_days

      if (clim_fun == "raw") {
        if (end_idx >= 1 && end_idx <= n) out[i] <- x[end_idx]
      } else {
        start_idx <- end_idx - accum_days + 1
        if (start_idx >= 1 && end_idx <= n) {
          ww <- x[start_idx:end_idx]
          if (!all(is.na(ww))) {
            out[i] <- switch(
              clim_fun,
              mean = mean(ww, na.rm = TRUE),
              sum = sum(ww, na.rm = TRUE),
              max = max(ww, na.rm = TRUE),
              min = min(ww, na.rm = TRUE),
              median = stats::median(ww, na.rm = TRUE)
            )
          }
        }
      }
    }

    out
  }

  summarize_daily_dm <- function(dm_df, stat) {
    daily <- dm_df %>%
      dplyr::mutate(DATE = as.Date(TIME)) %>%
      dplyr::group_by(DATE) %>%
      dplyr::summarise(
        dm_mean = mean(dm, na.rm = TRUE),
        dm_min = min(dm, na.rm = TRUE),
        dm_max = max(dm, na.rm = TRUE),
        dm_median = stats::median(dm, na.rm = TRUE),
        .groups = "drop"
      )

    daily$dm_amplitude <- daily$dm_max - daily$dm_min
    daily$dm_change <- c(NA_real_, diff(daily$dm_mean))

    daily_value <- switch(
      stat,
      mean = daily$dm_mean,
      min = daily$dm_min,
      max = daily$dm_max,
      median = daily$dm_median,
      amplitude = daily$dm_amplitude,
      change = daily$dm_change
    )

    tibble::tibble(TIME = as.Date(daily$DATE), dm_value = daily_value)
  }

  # dendrometer
  df[[1]] <- parse_datetime(df[[1]])
  if (all(is.na(df[[1]]))) {
    stop("Could not parse the first column of 'df' as date-time.")
  }

  dm_name <- names(df)[TreeNum + 1]

  dm_df <- tibble::as_tibble(df) %>%
    dplyr::select(1, TreeNum + 1) %>%
    dplyr::rename(
      TIME = 1,
      dm = 2
    )

  dm_daily <- summarize_daily_dm(dm_df, dm_stat)

  # climate
  Clim[[1]] <- parse_date_only(Clim[[1]])
  if (all(is.na(Clim[[1]]))) {
    stop("Could not parse the first column of 'Clim' as daily date.")
  }

  clim_tbl <- tibble::as_tibble(Clim) %>%
    dplyr::rename(TIME = 1) %>%
    dplyr::mutate(TIME = as.Date(TIME)) %>%
    dplyr::group_by(TIME) %>%
    dplyr::summarise(
      dplyr::across(
        .cols = where(is.numeric),
        .fns = ~ mean(.x, na.rm = TRUE)
      ),
      .groups = "drop"
    )

  all_clim_names <- names(clim_tbl)[-1]

  if (length(all_clim_names) == 0) {
    stop("'Clim' must contain at least one numeric climate variable after the date column.")
  }

  if (is.null(clim_vars)) {
    clim_vars <- all_clim_names
  } else {
    if (!is.character(clim_vars)) {
      stop("'clim_vars' must be a character vector.")
    }
    miss <- setdiff(clim_vars, all_clim_names)
    if (length(miss) > 0) {
      stop(sprintf(
        "The following climate variables were not found in 'Clim': %s",
        paste(miss, collapse = ", ")
      ))
    }
  }

  clim_fun_map <- resolve_per_var_arg(clim_fun, clim_vars, "clim_fun", allowed = allowed_clim_fun)
  lag_map <- resolve_per_var_arg(lag_days, clim_vars, "lag_days")
  accum_map <- resolve_per_var_arg(accum_days, clim_vars, "accum_days")

  lag_map <- as.integer(unname(lag_map))
  names(lag_map) <- clim_vars

  accum_map <- as.integer(unname(accum_map))
  names(accum_map) <- clim_vars

  if (any(is.na(lag_map)) || any(lag_map < 0)) {
    stop("All values in 'lag_days' must be non-negative integers.")
  }
  if (any(is.na(accum_map)) || any(accum_map < 1)) {
    stop("All values in 'accum_days' must be positive integers.")
  }

  raw_bad <- clim_vars[clim_fun_map == "raw" & accum_map != 1]
  if (length(raw_bad) > 0) {
    warning(
      "For variables using clim_fun = 'raw', accum_days is ignored. Affected variables: ",
      paste(raw_bad, collapse = ", ")
    )
  }

  clim_trans <- tibble::tibble(TIME = clim_tbl$TIME)
  for (nm in clim_vars) {
    clim_trans[[nm]] <- transform_clim_series(
      x = clim_tbl[[nm]],
      lag_days = unname(lag_map[nm]),
      accum_days = unname(accum_map[nm]),
      clim_fun = unname(clim_fun_map[nm])
    )
  }

  common_dates <- intersect(dm_daily$TIME, clim_trans$TIME)

  if (length(common_dates) == 0) {
    stop("'df' and 'Clim' must have time overlap after daily aggregation.")
  }

  if ((length(common_dates) != nrow(dm_daily)) || (length(common_dates) != nrow(clim_trans))) {
    warning("The temporal coverage of 'df' and 'Clim' is not the same. Output will be shortened to the common time period.")
  }

  dm_daily <- dm_daily %>% dplyr::filter(TIME %in% common_dates)
  clim_trans <- clim_trans %>% dplyr::filter(TIME %in% common_dates)

  dm_daily <- dm_daily[order(dm_daily$TIME), , drop = FALSE]
  clim_trans <- clim_trans[order(clim_trans$TIME), , drop = FALSE]

  n_row <- nrow(clim_trans)

  wl <- ifelse(win_size %% 2 == 0, win_size + 1, win_size)
  wl <- as.integer(wl)
  half_w <- (wl - 1) / 2

  if (n_row < wl) {
    stop("Not enough overlapping daily observations for the requested 'win_size'.")
  }

  if (boot) set.seed(set_seed)

  result_list <- lapply(clim_vars, function(cln) {
    cls <- clim_trans[[cln]]

    if (boot) {
      estimate <- data.frame(
        TIME = as.Date(dm_daily$TIME),
        corr = NA_real_,
        corr_lw_lim = NA_real_,
        corr_up_lim = NA_real_,
        n_complete = NA_real_
      )

      for (end_idx in seq.int(wl, n_row)) {
        start_idx <- end_idx - wl + 1
        mid_idx <- start_idx + half_w

        val_1 <- cls[start_idx:end_idx]
        val_2 <- dm_daily$dm_value[start_idx:end_idx]

        ok <- is.finite(val_1) & is.finite(val_2)
        estimate$n_complete[mid_idx] <- sum(ok)

        if (sum(ok) >= min_complete) {
          temp_data <- data.frame(x = val_1[ok], y = val_2[ok])

          b.corr <- suppressWarnings(
            boot::boot(
              temp_data,
              statistic = function(temp_data, i) {
                stats::cor(temp_data[i, "x"], temp_data[i, "y"], method = cor_method)
              },
              R = R
            )
          )

          estimate$corr[mid_idx] <- b.corr$t0

          cis <- c(0.01, 0.05, 0.1)
          loc.cis <- which(cis == boot.ci)
          cis_lw <- c(0.005, 0.025, 0.05)
          cis_up <- c(0.995, 0.975, 0.95)
          cis_app <- c(cis_lw[loc.cis], cis_up[loc.cis])

          qv <- stats::quantile(as.vector(b.corr$t), cis_app, na.rm = TRUE)
          estimate$corr_lw_lim[mid_idx] <- qv[1]
          estimate$corr_up_lim[mid_idx] <- qv[2]
        }
      }

      estimate <- tibble::as_tibble(estimate)
      estimate$significance <- with(estimate, ifelse(corr_lw_lim * corr_up_lim > 0, TRUE, FALSE))
      estimate$clim_var <- cln
      estimate$tree <- dm_name
      estimate

    } else {
      estimate <- data.frame(
        TIME = as.Date(dm_daily$TIME),
        corr = NA_real_,
        p_val = NA_real_,
        n_complete = NA_real_
      )

      for (end_idx in seq.int(wl, n_row)) {
        start_idx <- end_idx - wl + 1
        mid_idx <- start_idx + half_w

        val_1 <- cls[start_idx:end_idx]
        val_2 <- dm_daily$dm_value[start_idx:end_idx]

        ok <- is.finite(val_1) & is.finite(val_2)
        estimate$n_complete[mid_idx] <- sum(ok)

        if (sum(ok) >= min_complete) {
          a <- suppressWarnings(
            stats::cor.test(val_1[ok], val_2[ok], method = cor_method)
          )
          estimate$corr[mid_idx] <- unname(a$estimate)
          estimate$p_val[mid_idx] <- a$p.value
        }
      }

      estimate <- tibble::as_tibble(estimate)
      estimate$p_adj <- NA_real_
      okp <- !is.na(estimate$p_val)

      if (any(okp)) {
        if (p_adjust_method == "none") {
          estimate$p_adj[okp] <- estimate$p_val[okp]
        } else {
          estimate$p_adj[okp] <- stats::p.adjust(estimate$p_val[okp], method = p_adjust_method)
        }
      }

      estimate$significance <- ifelse(!is.na(estimate$p_adj) & estimate$p_adj < 0.05, TRUE, FALSE)
      estimate$clim_var <- cln
      estimate$tree <- dm_name
      estimate
    }
  })

  names(result_list) <- clim_vars

  climate_settings <- tibble::tibble(
    clim_var = clim_vars,
    clim_fun = unname(clim_fun_map[clim_vars]),
    lag_days = unname(lag_map[clim_vars]),
    accum_days = unname(accum_map[clim_vars])
  )

  out <- list(
    results = result_list,
    metadata = list(
      tree_num = TreeNum,
      tree_name = dm_name,
      clim_names = clim_vars,
      climate_settings = climate_settings,
      win_size = wl,
      cor_method = cor_method,
      boot = boot,
      R = if (boot) R else NA_integer_,
      boot_ci = if (boot) boot.ci else NA_real_,
      set_seed = if (boot) set_seed else NA_integer_,
      dm_stat = dm_stat,
      min_complete = min_complete,
      p_adjust_method = if (!boot) p_adjust_method else NA_character_,
      n_obs = n_row
    ),
    call = match.call()
  )

  class(out) <- unique(c(
    if (boot) "mov_cor_dm_boot",
    "mov_cor_dm",
    if (boot) "mov.cor.boot" else "mov.cor"
  ))

  out
}


#' @title Print method for running dendrometer-climate correlation objects
#'
#' @param x Object of class \code{"mov_cor_dm"}.
#' @param ... Unused.
#'
#' @return The object, invisibly.
#' @export
print.mov_cor_dm <- function(x, ...) {
  cat("Running dendrometer-climate correlation\n")
  cat("Tree: ", x$metadata$tree_name, "\n", sep = "")
  cat("Window size: ", x$metadata$win_size, " days\n", sep = "")
  cat("Correlation method: ", x$metadata$cor_method, "\n", sep = "")
  cat("Dendrometer statistic: ", x$metadata$dm_stat, "\n", sep = "")
  cat("Bootstrap: ", x$metadata$boot, "\n", sep = "")
  cat("Climate variables: ", paste(x$metadata$clim_names, collapse = ", "), "\n\n", sep = "")

  cat("Climate settings\n")
  print(x$metadata$climate_settings)

  invisible(x)
}


#' @title Summary method for running dendrometer-climate correlation objects
#'
#' @description
#' Summarizes running dendrometer-climate correlations for both bootstrapped and
#' non-bootstrapped outputs, and appends climate-specific settings to the
#' summary tables.
#'
#' @param object Object of class \code{"mov_cor_dm"}.
#' @param absolute Logical. If \code{TRUE} (default), the strongest correlation is
#'   identified by absolute magnitude.
#' @param top_n Integer. Number of top windows to return per climate variable.
#' @param ... Unused.
#'
#' @return A list of class \code{"summary_mov_cor_dm"} containing:
#' \itemize{
#'   \item \code{table}: summary table for each climate variable
#'   \item \code{top_windows}: top running windows per climate variable
#'   \item \code{metadata}: metadata from the original object
#'   \item \code{call}: original function call
#' }
#'
#' @export
summary.mov_cor_dm <- function(object, absolute = TRUE, top_n = 5, ...) {

  if (!inherits(object, "mov_cor_dm")) {
    stop("'object' must be of class 'mov_cor_dm'.")
  }

  if (!is.numeric(top_n) || length(top_n) != 1 || is.na(top_n) || top_n < 1) {
    stop("'top_n' must be a single positive number.")
  }
  top_n <- as.integer(top_n)

  boot_flag <- isTRUE(object$metadata$boot)
  climate_settings <- object$metadata$climate_settings

  summarize_one <- function(tab, boot = FALSE, absolute = TRUE, top_n = 5) {
    tab <- tibble::as_tibble(tab)

    ok <- !is.na(tab$corr)
    if (!any(ok)) {
      sm <- tibble::tibble(
        clim_var = unique(tab$clim_var)[1],
        tree = unique(tab$tree)[1],
        n_windows = 0L,
        n_significant = 0L,
        prop_significant = NA_real_,
        peak_time = as.Date(NA),
        peak_corr = NA_real_,
        mean_corr = NA_real_,
        median_corr = NA_real_,
        min_corr = NA_real_,
        max_corr = NA_real_
      )

      if (boot) {
        sm$peak_lw <- NA_real_
        sm$peak_up <- NA_real_
      } else {
        sm$peak_p <- NA_real_
        sm$peak_p_adj <- NA_real_
      }

      return(list(summary = sm, top = tab[0, , drop = FALSE]))
    }

    tab_ok <- tab[ok, , drop = FALSE]

    peak_idx <- if (absolute) {
      which.max(abs(tab_ok$corr))
    } else {
      which.max(tab_ok$corr)
    }

    sm <- tibble::tibble(
      clim_var = unique(tab_ok$clim_var)[1],
      tree = unique(tab_ok$tree)[1],
      n_windows = nrow(tab_ok),
      n_significant = sum(tab_ok$significance, na.rm = TRUE),
      prop_significant = mean(tab_ok$significance, na.rm = TRUE),
      peak_time = tab_ok$TIME[peak_idx],
      peak_corr = tab_ok$corr[peak_idx],
      mean_corr = mean(tab_ok$corr, na.rm = TRUE),
      median_corr = stats::median(tab_ok$corr, na.rm = TRUE),
      min_corr = min(tab_ok$corr, na.rm = TRUE),
      max_corr = max(tab_ok$corr, na.rm = TRUE)
    )

    if (boot) {
      sm$peak_lw <- tab_ok$corr_lw_lim[peak_idx]
      sm$peak_up <- tab_ok$corr_up_lim[peak_idx]

      ord_idx <- order(!tab_ok$significance, -abs(tab_ok$corr))
      top_tab <- tab_ok[ord_idx, , drop = FALSE]
    } else {
      sm$peak_p <- tab_ok$p_val[peak_idx]
      sm$peak_p_adj <- tab_ok$p_adj[peak_idx]

      p_adj_sort <- ifelse(is.na(tab_ok$p_adj), Inf, tab_ok$p_adj)
      ord_idx <- order(!tab_ok$significance, p_adj_sort, -abs(tab_ok$corr))
      top_tab <- tab_ok[ord_idx, , drop = FALSE]
    }

    top_tab <- utils::head(top_tab, top_n)

    list(summary = sm, top = top_tab)
  }

  res_list <- lapply(
    object$results,
    summarize_one,
    boot = boot_flag,
    absolute = absolute,
    top_n = top_n
  )

  summary_table <- dplyr::bind_rows(lapply(res_list, `[[`, "summary"))
  top_windows <- dplyr::bind_rows(lapply(res_list, `[[`, "top"))

  if (!is.null(climate_settings) && nrow(climate_settings) > 0) {
    idx_sum <- match(summary_table$clim_var, climate_settings$clim_var)
    summary_table$clim_fun <- climate_settings$clim_fun[idx_sum]
    summary_table$lag_days <- climate_settings$lag_days[idx_sum]
    summary_table$accum_days <- climate_settings$accum_days[idx_sum]

    idx_top <- match(top_windows$clim_var, climate_settings$clim_var)
    top_windows$clim_fun <- climate_settings$clim_fun[idx_top]
    top_windows$lag_days <- climate_settings$lag_days[idx_top]
    top_windows$accum_days <- climate_settings$accum_days[idx_top]
  }

  out <- list(
    table = summary_table,
    top_windows = top_windows,
    metadata = object$metadata,
    call = object$call
  )

  class(out) <- "summary_mov_cor_dm"
  out
}


#' @title Print method for summaries of running dendrometer-climate correlation objects
#'
#' @param x Object of class \code{"summary_mov_cor_dm"}.
#' @param ... Unused.
#'
#' @return The summary object, invisibly.
#' @export
print.summary_mov_cor_dm <- function(x, ...) {
  cat("Summary of running dendrometer-climate correlations\n")
  cat("Tree: ", x$metadata$tree_name, "\n", sep = "")
  cat("Window size: ", x$metadata$win_size, " days\n", sep = "")
  cat("Method: ", x$metadata$cor_method, "\n", sep = "")
  cat("Dendrometer statistic: ", x$metadata$dm_stat, "\n", sep = "")
  cat("Bootstrap: ", x$metadata$boot, "\n\n", sep = "")

  cat("Overall summary\n")
  print(x$table)

  cat("\nTop windows\n")
  print(x$top_windows)

  invisible(x)
}

#' Fit bimodal dendrometer growth curves by vegetation season
#'
#' @description
#' Fits bimodal cumulative growth curves to daily dendrometer series using either
#' a double-Gompertz or double-Richards model.
#'
#' Overall growing-season timing is derived from the fitted cumulative-growth
#' curve using \code{growth_fraction}. Pulse-specific timing is derived from the
#' derivative of the fitted curve using a pulse-specific relative rate threshold
#' given by \code{rate_threshold_fraction}.
#'
#' Pulse-specific timing is returned as \code{NA} when the fitted curve does not
#' show a convincing two-pulse pattern according to derivative-based criteria.
#'
#' If \code{fallback_to_single = TRUE} and no convincing two-pulse pattern is
#' found, the function refits a corresponding single growth curve and returns
#' that fit instead. In that case, overall season timing is still returned, but
#' pulse-specific timing remains \code{NA}.
#'
#' For double-Gompertz and double-Richards fits, the total asymptote can
#' optionally be fixed to the observed maximum cumulative growth of the
#' corresponding series and vegetation season. In the double-curve case, this
#' means \code{a + a2} is fixed, while the relative contribution of the first
#' and second pulse is estimated.
#'
#' @param df A data frame or tibble. The first column must be a time stamp
#'   \code{Date}, \code{POSIXct}, or parseable date-time string. Remaining
#'   selected columns must be numeric dendrometer series.
#' @param TreeNum Either \code{"all"} to use all dendrometer series, a numeric
#'   vector selecting dendrometer columns by position, or a character vector of
#'   column names.
#' @param method Double-curve fitting method. One of \code{"gompertz"} or
#'   \code{"richards"}.
#' @param years Either \code{"all"} to fit all available vegetation seasons, or
#'   a character vector of season labels to retain.
#' @param year_mode Either \code{"yearly"} to fit one curve per vegetation
#'   season, or \code{"pooled"} to fit one pooled curve across all retained
#'   seasons.
#' @param fit_GRO Logical. If \code{TRUE}, processed daily series are converted
#'   to cumulative growth using a cumulative maximum within each vegetation
#'   season.
#' @param site_mode Vegetation season definition. One of \code{"NH"},
#'   \code{"SH"}, or \code{"CS"}.
#' @param custom_veg_season Numeric vector of length two giving the start and
#'   end day-of-year for custom vegetation seasons in \code{"CS"} mode.
#' @param growth_fraction Numeric vector of length two giving the lower and
#'   upper fractions of final fitted seasonal growth used to define overall
#'   growing-season onset and cessation.
#' @param min_unique_growth Minimum number of unique non-missing cumulative
#'   growth values required before a fit is attempted.
#' @param rate_threshold_fraction Numeric scalar between 0 and 1. Pulse start
#'   and end are defined as the first and last days where fitted growth rate
#'   exceeds this fraction of the pulse-specific peak fitted growth rate.
#' @param peak_separation_min Minimum number of days separating the two fitted
#'   derivative peaks required to classify a fit as truly two-pulse.
#' @param valley_ratio_max Maximum allowed ratio between the valley rate and the
#'   weaker of the two derivative peaks. Smaller values require a deeper valley
#'   between pulses.
#' @param min_relative_peak Minimum relative height, expressed as a fraction of
#'   the global derivative maximum, for a local derivative peak to be considered.
#' @param fallback_to_single Logical. If \code{TRUE}, and the fitted double curve
#'   does not show a convincing two-pulse pattern according to derivative-based
#'   criteria, the function refits a corresponding single growth curve
#'   \code{"gompertz"} or \code{"richards"} and returns that fit instead.
#' @param fix_a_to_observed_max Logical. If \code{TRUE}, the total asymptote of
#'   the double-growth curve is fixed to the observed maximum cumulative growth
#'   for that series and vegetation season. For double-Gompertz and
#'   double-Richards models, this means \code{a + a2} is fixed, while the
#'   relative contribution of the two pulses is estimated. This is most
#'   appropriate with \code{year_mode = "yearly"}. For
#'   \code{year_mode = "pooled"}, one pooled maximum is used.
#' @param fixed_a_multiplier Numeric multiplier applied to the observed maximum
#'   when \code{fix_a_to_observed_max = TRUE}. The default is \code{1}, meaning
#'   the total asymptote is fixed exactly to the observed seasonal maximum.
#'   Values slightly above 1, for example \code{1.01} or \code{1.05}, allow the
#'   fitted asymptote to sit slightly above the observed maximum.
#' @param start_value_double_gompertz_parameters Optional named list of starting
#'   values for the double-Gompertz fit. Supported names are \code{a}, \code{k},
#'   \code{t0}, \code{a2}, \code{k2}, and \code{t02}. If
#'   \code{fix_a_to_observed_max = TRUE}, supplied \code{a} and \code{a2} are
#'   used only to initialize the relative pulse contribution.
#' @param start_value_double_richards_parameters Optional named list of starting
#'   values for the double-Richards fit. Supported names are \code{a}, \code{k},
#'   \code{t0}, \code{v}, \code{a2}, \code{k2}, \code{t02}, and \code{v2}. If
#'   \code{fix_a_to_observed_max = TRUE}, supplied \code{a} and \code{a2} are
#'   used only to initialize the relative pulse contribution.
#' @param verbose Logical. If \code{TRUE}, prints a short completion message.
#'
#' @return
#' An object of class \code{"dm_growth_fit"} with elements:
#' \describe{
#'   \item{call}{The matched function call.}
#'   \item{original_daily_data}{Raw daily dendrometer data after resampling to
#'     daily maxima and assigning vegetation seasons, but before centering and
#'     optional cumulative-growth transformation.}
#'   \item{processed_data}{Daily processed data used for fitting.}
#'   \item{fitted_data}{Daily fitted values on the full vegetation-season grid.}
#'   \item{fit_statistics}{Fit-level statistics and estimated timing.}
#'   \item{fit_parameters}{Fit-level model parameters and convergence
#'     information.}
#'   \item{season_table}{Vegetation seasons retained for fitting.}
#' }
#'
#' @details
#' The second pulse is constrained to occur after the first pulse, which improves
#' numerical stability and reduces label switching between the two components.
#'
#' For double-Gompertz and double-Richards models, the total seasonal asymptote
#' is:
#'
#' \deqn{a_{total} = a + a2}
#'
#' When \code{fix_a_to_observed_max = TRUE}, the function fits:
#'
#' \deqn{a + a2 = max(y_{obs}) \times fixed\_a\_multiplier}
#'
#' where \eqn{y_{obs}} is the observed cumulative growth of the selected
#' dendrometer series and vegetation season.
#'
#' The fitted pulse contributions are still returned as \code{a} and \code{a2},
#' and their sum should equal \code{fixed_a_value}, apart from small numerical
#' rounding.
#'
#' Non-applicable parameters are returned as \code{NA}. For example, \code{b}
#' and \code{b2} are relevant for double-Gompertz, while \code{v} and \code{v2}
#' are relevant for double-Richards.
#'
#' @examples
#' \donttest{
#' # Double-Gompertz with total seasonal asymptote fixed
#' # fit_gomp <- dm.growth.fit.double(
#' #   df = dendro_data,
#' #   TreeNum = "all",
#' #   method = "gompertz",
#' #   year_mode = "yearly",
#' #   fit_GRO = TRUE,
#' #   fix_a_to_observed_max = TRUE
#' # )
#'
#' # Double-Richards with the total asymptote set 2 percent above observed max
#' # fit_rich <- dm.growth.fit.double(
#' #   df = dendro_data,
#' #   TreeNum = "all",
#' #   method = "richards",
#' #   year_mode = "yearly",
#' #   fit_GRO = TRUE,
#' #   fix_a_to_observed_max = TRUE,
#' #   fixed_a_multiplier = 1.02
#' # )
#' }
#'
#' @seealso [dm.growth.fit()], [summary.dm_growth_fit()],
#'   [print.dm_growth_fit()]
#'
#' @importFrom dplyr %>% select all_of any_of bind_cols bind_rows filter arrange
#'   mutate across group_by ungroup distinct left_join
#' @importFrom tibble as_tibble tibble
#' @importFrom lubridate parse_date_time year yday month
#' @importFrom stats lm coef qlogis predict quantile median na.omit
#' @importFrom utils modifyList
#'
#' @export
dm.growth.fit.double <- function(
    df,
    TreeNum = "all",
    method = c("gompertz", "richards"),
    years = "all",
    year_mode = c("yearly", "pooled"),
    fit_GRO = TRUE,
    site_mode = c("NH", "SH", "CS"),
    custom_veg_season = c(275, 274),
    growth_fraction = c(0.1, 0.9),
    min_unique_growth = 10,
    rate_threshold_fraction = 0.1,
    peak_separation_min = 10,
    valley_ratio_max = 0.4,
    min_relative_peak = 0.05,
    fallback_to_single = TRUE,
    fix_a_to_observed_max = FALSE,
    fixed_a_multiplier = 1,
    start_value_double_gompertz_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_
    ),
    start_value_double_richards_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_, v2 = 1
    ),
    verbose = TRUE
) {
  cl <- match.call()

  TIME <- any_observed <- doy <- season_day <- season_end <- NULL
  season_label <- season_length <- season_start <- NULL

  method <- match.arg(method)
  year_mode <- match.arg(year_mode)
  site_mode <- match.arg(site_mode)

  dmgfd_validate_inputs(
    df = df,
    growth_fraction = growth_fraction,
    min_unique_growth = min_unique_growth,
    custom_veg_season = custom_veg_season,
    rate_threshold_fraction = rate_threshold_fraction,
    peak_separation_min = peak_separation_min,
    valley_ratio_max = valley_ratio_max,
    min_relative_peak = min_relative_peak,
    fallback_to_single = fallback_to_single,
    fix_a_to_observed_max = fix_a_to_observed_max,
    fixed_a_multiplier = fixed_a_multiplier
  )

  df <- tibble::as_tibble(df)
  names(df)[1] <- "TIME"

  if (!inherits(df$TIME, "Date") &&
      !inherits(df$TIME, "POSIXct") &&
      !inherits(df$TIME, "POSIXt")) {
    df$TIME <- suppressWarnings(
      lubridate::parse_date_time(
        df$TIME,
        orders = c(
          "ymd HMS", "ymd HM", "ymd",
          "dmy HMS", "dmy HM", "dmy",
          "mdy HMS", "mdy HM", "mdy"
        ),
        quiet = TRUE
      )
    )
  }

  if (inherits(df$TIME, "Date")) {
    df$TIME <- as.POSIXct(df$TIME)
  }

  if (any(is.na(df$TIME))) {
    stop("Some timestamps in the first column could not be parsed.")
  }

  obs_years <- sort(unique(lubridate::year(as.Date(df$TIME))))

  if (site_mode == "SH" && length(obs_years) < 2) {
    stop("For site_mode = 'SH', df must contain observations from at least 2 calendar years.")
  }

  if (site_mode == "CS" &&
      custom_veg_season[1] > custom_veg_season[2] &&
      length(obs_years) < 2) {
    stop(
      "For site_mode = 'CS' with custom_veg_season[1] > custom_veg_season[2], ",
      "df must contain observations from at least 2 calendar years."
    )
  }

  all_series <- names(df)[-1]

  if (length(all_series) == 0) {
    stop("df must contain a time column followed by at least one dendrometer column.")
  }

  if (is.character(TreeNum) &&
      length(TreeNum) == 1 &&
      tolower(TreeNum) == "all") {
    keep_cols <- all_series
  } else if (is.numeric(TreeNum)) {
    tree_idx <- as.integer(TreeNum)

    if (any(is.na(tree_idx)) ||
        any(tree_idx < 1) ||
        any(tree_idx > length(all_series))) {
      stop(
        "'TreeNum' numeric values must be between 1 and ", length(all_series),
        ". Here, 1 refers to the first dendrometer series after the TIME column."
      )
    }

    keep_cols <- all_series[tree_idx]
  } else if (is.character(TreeNum)) {
    missing_cols <- setdiff(TreeNum, all_series)

    if (length(missing_cols) > 0) {
      warning(
        "These TreeNum column names were not found and were ignored: ",
        paste(missing_cols, collapse = ", ")
      )
    }

    keep_cols <- intersect(TreeNum, all_series)

    if (length(keep_cols) == 0) {
      stop("None of the requested TreeNum column names were found in df.")
    }
  } else {
    stop("TreeNum must be 'all', a numeric vector, or a character vector of column names.")
  }

  df <- dplyr::select(df, TIME, dplyr::all_of(keep_cols))

  num_cols <- names(df)[vapply(df, is.numeric, logical(1))]
  num_cols <- setdiff(num_cols, "TIME")

  if (length(num_cols) == 0) {
    stop("No numeric dendrometer series found to fit.")
  }

  df <- dplyr::select(df, TIME, dplyr::all_of(num_cols))

  dm_daily <- dendro.resample(
    df = df,
    by = "D",
    value = "max",
    method = "aggregate"
  )

  season_tbl_obs <- dmgfd_build_season_table(
    time_vec = dm_daily$TIME,
    site_mode = site_mode,
    custom_veg_season = custom_veg_season
  )

  observed_daily <- dplyr::bind_cols(dm_daily, season_tbl_obs) %>%
    dplyr::filter(!is.na(season_label)) %>%
    dplyr::arrange(TIME)

  original_daily_data <- observed_daily %>%
    dplyr::select(
      TIME, doy, season_label, season_start, season_end, season_day,
      dplyr::all_of(num_cols)
    )

  season_table <- observed_daily %>%
    dplyr::distinct(season_label, season_start, season_end) %>%
    dplyr::arrange(season_start)

  if (!(is.character(years) &&
        length(years) == 1 &&
        tolower(years) == "all")) {
    season_table <- season_table %>%
      dplyr::filter(season_label %in% as.character(years))
  }

  if (nrow(season_table) == 0) {
    stop("No seasons remained after applying site_mode/custom_veg_season/years.")
  }

  full_grids <- lapply(seq_len(nrow(season_table)), function(i) {
    ss <- season_table$season_start[i]
    se <- season_table$season_end[i]
    sl <- season_table$season_label[i]
    tt <- seq(ss, se, by = "day")

    tibble::tibble(
      TIME = tt,
      doy = lubridate::yday(tt),
      season_label = sl,
      season_start = ss,
      season_end = se,
      season_day = seq_along(tt),
      season_length = length(tt)
    )
  })

  full_grid <- dplyr::bind_rows(full_grids)

  dat <- full_grid %>%
    dplyr::left_join(
      observed_daily %>%
        dplyr::select(TIME, season_label, dplyr::all_of(num_cols)),
      by = c("TIME", "season_label")
    ) %>%
    dplyr::arrange(season_start, TIME)

  dat_proc <- dat %>%
    dplyr::group_by(season_label) %>%
    dplyr::arrange(TIME, .by_group = TRUE) %>%
    dplyr::mutate(
      dplyr::across(
        dplyr::all_of(num_cols),
        ~ . - dmgfd_first_non_na(.)
      )
    ) %>%
    dplyr::ungroup()

  if (isTRUE(fit_GRO)) {
    dat_proc <- dat_proc %>%
      dplyr::group_by(season_label) %>%
      dplyr::arrange(TIME, .by_group = TRUE) %>%
      dplyr::mutate(
        dplyr::across(
          dplyr::all_of(num_cols),
          dmgfd_cummax_na
        )
      ) %>%
      dplyr::ungroup()
  }

  dat_proc$any_observed <- rowSums(
    !is.na(as.data.frame(dat_proc[, num_cols, drop = FALSE]))
  ) > 0

  fitted_dat <- dat_proc %>%
    dplyr::select(
      TIME, doy, season_label, season_start, season_end,
      season_day, season_length, any_observed
    )

  for (cc in num_cols) {
    fitted_dat[[cc]] <- NA_real_
  }

  param_rows <- list()
  row_id <- 1L

  if (year_mode == "yearly") {
    fit_ids <- unique(dat_proc$season_label)

    for (series_name in num_cols) {
      for (sid in fit_ids) {
        idx <- which(dat_proc$season_label == sid)

        x_full <- dat_proc$season_day[idx]
        y_full <- dat_proc[[series_name]][idx]

        ss <- unique(dat_proc$season_start[idx])[1]
        se <- unique(dat_proc$season_end[idx])[1]
        full_len <- unique(dat_proc$season_length[idx])[1]

        good <- is.finite(y_full)
        x_obs <- x_full[good]
        y_obs <- y_full[good]

        fit_res <- dmgfd_fit_one_curve(
          x_obs = x_obs,
          y_obs = y_obs,
          x_pred = x_full,
          season_start_date = ss,
          season_length = full_len,
          method = method,
          growth_fraction = growth_fraction,
          min_unique_growth = min_unique_growth,
          rate_threshold_fraction = rate_threshold_fraction,
          peak_separation_min = peak_separation_min,
          valley_ratio_max = valley_ratio_max,
          min_relative_peak = min_relative_peak,
          fallback_to_single = fallback_to_single,
          fix_a_to_observed_max = fix_a_to_observed_max,
          fixed_a_multiplier = fixed_a_multiplier,
          start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
          start_value_double_richards_parameters = start_value_double_richards_parameters
        )

        fitted_dat[[series_name]][idx] <- fit_res$pred

        param_rows[[row_id]] <- tibble::tibble(
          series = series_name,
          fit_id = sid,
          year_mode = year_mode,
          method = fit_res$params$method_used,
          site_mode = site_mode,
          season_start = ss,
          season_end = se,
          included_seasons = NA_character_,

          season_length = fit_res$params$season_length,
          n_obs = fit_res$params$n_obs,
          n_days_observed = fit_res$params$n_days_observed,
          first_obs_day = fit_res$params$first_obs_day,
          last_obs_day = fit_res$params$last_obs_day,
          n_missing_days = fit_res$params$n_missing_days,
          extrapolated = fit_res$params$extrapolated,
          anchor_added = fit_res$params$anchor_added,

          growth_start_day = fit_res$params$growth_start_day,
          growth_end_day = fit_res$params$growth_end_day,
          growth_start_season_day = fit_res$params$growth_start_season_day,
          growth_end_season_day = fit_res$params$growth_end_season_day,
          growth_start_date = fit_res$params$growth_start_date,
          growth_end_date = fit_res$params$growth_end_date,

          peak_rate = fit_res$params$peak_rate,
          rate_start_day = fit_res$params$rate_start_day,
          rate_end_day = fit_res$params$rate_end_day,
          rate_start_season_day = fit_res$params$rate_start_season_day,
          rate_end_season_day = fit_res$params$rate_end_season_day,
          rate_start_date = fit_res$params$rate_start_date,
          rate_end_date = fit_res$params$rate_end_date,

          fallback_used = fit_res$params$fallback_used,
          two_pulse_detected = fit_res$params$two_pulse_detected,

          peak1_day = fit_res$params$peak1_day,
          peak2_day = fit_res$params$peak2_day,
          peak1_season_day = fit_res$params$peak1_season_day,
          peak2_season_day = fit_res$params$peak2_season_day,
          separator_day = fit_res$params$separator_day,
          separator_season_day = fit_res$params$separator_season_day,
          separator_date = fit_res$params$separator_date,

          valley_rate = fit_res$params$valley_rate,
          pulse1_peak_rate = fit_res$params$pulse1_peak_rate,
          pulse2_peak_rate = fit_res$params$pulse2_peak_rate,

          pulse1_start_day = fit_res$params$pulse1_start_day,
          pulse1_end_day = fit_res$params$pulse1_end_day,
          pulse1_start_season_day = fit_res$params$pulse1_start_season_day,
          pulse1_end_season_day = fit_res$params$pulse1_end_season_day,
          pulse1_start_date = fit_res$params$pulse1_start_date,
          pulse1_end_date = fit_res$params$pulse1_end_date,

          pulse2_start_day = fit_res$params$pulse2_start_day,
          pulse2_end_day = fit_res$params$pulse2_end_day,
          pulse2_start_season_day = fit_res$params$pulse2_start_season_day,
          pulse2_end_season_day = fit_res$params$pulse2_end_season_day,
          pulse2_start_date = fit_res$params$pulse2_start_date,
          pulse2_end_date = fit_res$params$pulse2_end_date,

          converged = fit_res$params$converged,
          fixed_a_used = fit_res$params$fixed_a_used,
          fixed_a_value = fit_res$params$fixed_a_value,

          a = fit_res$params$a,
          b = fit_res$params$b,
          k = fit_res$params$k,
          t0 = fit_res$params$t0,
          v = fit_res$params$v,
          a2 = fit_res$params$a2,
          b2 = fit_res$params$b2,
          k2 = fit_res$params$k2,
          t02 = fit_res$params$t02,
          v2 = fit_res$params$v2,

          edf = fit_res$params$edf,
          span = fit_res$params$span,
          spline_df = fit_res$params$spline_df,
          spar = fit_res$params$spar,
          model_note = fit_res$params$model_note
        )

        row_id <- row_id + 1L
      }
    }
  }

  if (year_mode == "pooled") {
    included_seasons <- paste(unique(dat_proc$season_label), collapse = ", ")

    for (series_name in num_cols) {
      x_obs_all <- dat_proc$season_day
      y_obs_all <- dat_proc[[series_name]]

      good <- is.finite(y_obs_all)
      x_obs <- x_obs_all[good]
      y_obs <- y_obs_all[good]

      x_template <- seq_len(max(dat_proc$season_day, na.rm = TRUE))

      fit_res <- dmgfd_fit_one_curve(
        x_obs = x_obs,
        y_obs = y_obs,
        x_pred = x_template,
        season_start_date = as.Date(NA),
        season_length = max(x_template, na.rm = TRUE),
        method = method,
        growth_fraction = growth_fraction,
        min_unique_growth = min_unique_growth,
        rate_threshold_fraction = rate_threshold_fraction,
        peak_separation_min = peak_separation_min,
        valley_ratio_max = valley_ratio_max,
        min_relative_peak = min_relative_peak,
        fallback_to_single = fallback_to_single,
        fix_a_to_observed_max = fix_a_to_observed_max,
        fixed_a_multiplier = fixed_a_multiplier,
        start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
        start_value_double_richards_parameters = start_value_double_richards_parameters
      )

      fitted_dat[[series_name]] <- fit_res$pred[match(dat_proc$season_day, x_template)]

      param_rows[[row_id]] <- tibble::tibble(
        series = series_name,
        fit_id = "pooled",
        year_mode = year_mode,
        method = fit_res$params$method_used,
        site_mode = site_mode,
        season_start = as.Date(NA),
        season_end = as.Date(NA),
        included_seasons = included_seasons,

        season_length = fit_res$params$season_length,
        n_obs = fit_res$params$n_obs,
        n_days_observed = fit_res$params$n_days_observed,
        first_obs_day = fit_res$params$first_obs_day,
        last_obs_day = fit_res$params$last_obs_day,
        n_missing_days = fit_res$params$n_missing_days,
        extrapolated = fit_res$params$extrapolated,
        anchor_added = fit_res$params$anchor_added,

        growth_start_day = fit_res$params$growth_start_day,
        growth_end_day = fit_res$params$growth_end_day,
        growth_start_season_day = fit_res$params$growth_start_season_day,
        growth_end_season_day = fit_res$params$growth_end_season_day,
        growth_start_date = fit_res$params$growth_start_date,
        growth_end_date = fit_res$params$growth_end_date,

        peak_rate = fit_res$params$peak_rate,
        rate_start_day = fit_res$params$rate_start_day,
        rate_end_day = fit_res$params$rate_end_day,
        rate_start_season_day = fit_res$params$rate_start_season_day,
        rate_end_season_day = fit_res$params$rate_end_season_day,
        rate_start_date = fit_res$params$rate_start_date,
        rate_end_date = fit_res$params$rate_end_date,

        fallback_used = fit_res$params$fallback_used,
        two_pulse_detected = fit_res$params$two_pulse_detected,

        peak1_day = fit_res$params$peak1_day,
        peak2_day = fit_res$params$peak2_day,
        peak1_season_day = fit_res$params$peak1_season_day,
        peak2_season_day = fit_res$params$peak2_season_day,
        separator_day = fit_res$params$separator_day,
        separator_season_day = fit_res$params$separator_season_day,
        separator_date = fit_res$params$separator_date,

        valley_rate = fit_res$params$valley_rate,
        pulse1_peak_rate = fit_res$params$pulse1_peak_rate,
        pulse2_peak_rate = fit_res$params$pulse2_peak_rate,

        pulse1_start_day = fit_res$params$pulse1_start_day,
        pulse1_end_day = fit_res$params$pulse1_end_day,
        pulse1_start_season_day = fit_res$params$pulse1_start_season_day,
        pulse1_end_season_day = fit_res$params$pulse1_end_season_day,
        pulse1_start_date = fit_res$params$pulse1_start_date,
        pulse1_end_date = fit_res$params$pulse1_end_date,

        pulse2_start_day = fit_res$params$pulse2_start_day,
        pulse2_end_day = fit_res$params$pulse2_end_day,
        pulse2_start_season_day = fit_res$params$pulse2_start_season_day,
        pulse2_end_season_day = fit_res$params$pulse2_end_season_day,
        pulse2_start_date = fit_res$params$pulse2_start_date,
        pulse2_end_date = fit_res$params$pulse2_end_date,

        converged = fit_res$params$converged,
        fixed_a_used = fit_res$params$fixed_a_used,
        fixed_a_value = fit_res$params$fixed_a_value,

        a = fit_res$params$a,
        b = fit_res$params$b,
        k = fit_res$params$k,
        t0 = fit_res$params$t0,
        v = fit_res$params$v,
        a2 = fit_res$params$a2,
        b2 = fit_res$params$b2,
        k2 = fit_res$params$k2,
        t02 = fit_res$params$t02,
        v2 = fit_res$params$v2,

        edf = fit_res$params$edf,
        span = fit_res$params$span,
        spline_df = fit_res$params$spline_df,
        spar = fit_res$params$spar,
        model_note = fit_res$params$model_note
      )

      row_id <- row_id + 1L
    }
  }

  parameter_table <- dplyr::bind_rows(param_rows)

  stats_cols <- c(
    "series", "fit_id", "year_mode", "method", "site_mode",
    "season_start", "season_end", "included_seasons", "season_length",
    "n_obs", "n_days_observed", "first_obs_day", "last_obs_day",
    "n_missing_days", "extrapolated", "anchor_added",

    "growth_start_day", "growth_end_day",
    "growth_start_season_day", "growth_end_season_day",
    "growth_start_date", "growth_end_date",

    "peak_rate",
    "rate_start_day", "rate_end_day",
    "rate_start_season_day", "rate_end_season_day",
    "rate_start_date", "rate_end_date",

    "fallback_used",
    "two_pulse_detected",

    "peak1_day", "peak2_day",
    "peak1_season_day", "peak2_season_day",
    "separator_day", "separator_season_day", "separator_date",

    "valley_rate",
    "pulse1_peak_rate", "pulse2_peak_rate",

    "pulse1_start_day", "pulse1_end_day",
    "pulse1_start_season_day", "pulse1_end_season_day",
    "pulse1_start_date", "pulse1_end_date",

    "pulse2_start_day", "pulse2_end_day",
    "pulse2_start_season_day", "pulse2_end_season_day",
    "pulse2_start_date", "pulse2_end_date"
  )

  param_cols <- c(
    "series", "fit_id", "year_mode", "method", "site_mode",
    "fallback_used", "converged",
    "fixed_a_used", "fixed_a_value",
    "a", "b", "k", "t0", "v",
    "a2", "b2", "k2", "t02", "v2",
    "edf", "span", "spline_df", "spar", "model_note"
  )

  fit_statistics <- parameter_table %>%
    dplyr::select(dplyr::any_of(stats_cols))

  fit_parameters <- parameter_table %>%
    dplyr::select(dplyr::any_of(param_cols))

  out <- list(
    call = cl,
    original_daily_data = original_daily_data,
    processed_data = dat_proc,
    fitted_data = fitted_dat,
    fit_statistics = fit_statistics,
    fit_parameters = fit_parameters,
    season_table = season_table
  )

  class(out) <- "dm_growth_fit"

  if (isTRUE(verbose)) {
    message(
      "dm.growth.fit.double completed: ",
      nrow(fit_parameters), " fit(s) across ",
      length(unique(fit_parameters$series)), " series."
    )
  }

  out
}


# helpers ----------------------------------------------------------------------

#' Validate inputs for double dendrometer growth fitting
#'
#' @param df Input data frame.
#' @param growth_fraction Numeric vector of two growth fractions.
#' @param min_unique_growth Minimum number of unique growth values.
#' @param custom_veg_season Custom vegetation-season DOY range.
#' @param rate_threshold_fraction Fraction of peak growth rate.
#' @param peak_separation_min Minimum separation between derivative peaks.
#' @param valley_ratio_max Maximum allowed valley-to-peak ratio.
#' @param min_relative_peak Minimum relative derivative peak height.
#' @param fallback_to_single Logical fallback flag.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param fixed_a_multiplier Multiplier for fixed total asymptote.
#'
#' @return Invisibly returns \code{TRUE} if checks pass.
#'
#' @keywords internal
dmgfd_validate_inputs <- function(df,
                                  growth_fraction,
                                  min_unique_growth,
                                  custom_veg_season,
                                  rate_threshold_fraction,
                                  peak_separation_min,
                                  valley_ratio_max,
                                  min_relative_peak,
                                  fallback_to_single,
                                  fix_a_to_observed_max,
                                  fixed_a_multiplier) {
  if (!is.data.frame(df)) {
    stop("df must be a data.frame or tibble.")
  }

  if (ncol(df) < 2) {
    stop("df must contain a time column and at least one dendrometer series.")
  }

  if (!is.numeric(growth_fraction) ||
      length(growth_fraction) != 2 ||
      any(!is.finite(growth_fraction)) ||
      growth_fraction[1] < 0 ||
      growth_fraction[2] > 1 ||
      growth_fraction[1] >= growth_fraction[2]) {
    stop("growth_fraction must be a numeric vector like c(0.1, 0.9).")
  }

  if (!is.numeric(min_unique_growth) ||
      length(min_unique_growth) != 1 ||
      !is.finite(min_unique_growth) ||
      min_unique_growth < 2) {
    stop("min_unique_growth must be a single finite number >= 2.")
  }

  if (!is.numeric(custom_veg_season) ||
      length(custom_veg_season) != 2 ||
      any(!is.finite(custom_veg_season))) {
    stop("custom_veg_season must be a numeric vector of length 2.")
  }

  if (!is.numeric(rate_threshold_fraction) ||
      length(rate_threshold_fraction) != 1 ||
      !is.finite(rate_threshold_fraction) ||
      rate_threshold_fraction <= 0 ||
      rate_threshold_fraction >= 1) {
    stop("rate_threshold_fraction must be a single number between 0 and 1.")
  }

  if (!is.numeric(peak_separation_min) ||
      length(peak_separation_min) != 1 ||
      !is.finite(peak_separation_min) ||
      peak_separation_min < 1) {
    stop("peak_separation_min must be a single number >= 1.")
  }

  if (!is.numeric(valley_ratio_max) ||
      length(valley_ratio_max) != 1 ||
      !is.finite(valley_ratio_max) ||
      valley_ratio_max <= 0 ||
      valley_ratio_max >= 1) {
    stop("valley_ratio_max must be a single number between 0 and 1.")
  }

  if (!is.numeric(min_relative_peak) ||
      length(min_relative_peak) != 1 ||
      !is.finite(min_relative_peak) ||
      min_relative_peak <= 0 ||
      min_relative_peak >= 1) {
    stop("min_relative_peak must be a single number between 0 and 1.")
  }

  if (!is.logical(fallback_to_single) ||
      length(fallback_to_single) != 1 ||
      is.na(fallback_to_single)) {
    stop("fallback_to_single must be TRUE or FALSE.")
  }

  if (!is.logical(fix_a_to_observed_max) ||
      length(fix_a_to_observed_max) != 1 ||
      is.na(fix_a_to_observed_max)) {
    stop("fix_a_to_observed_max must be TRUE or FALSE.")
  }

  if (!is.numeric(fixed_a_multiplier) ||
      length(fixed_a_multiplier) != 1 ||
      !is.finite(fixed_a_multiplier) ||
      fixed_a_multiplier <= 0) {
    stop("fixed_a_multiplier must be a single positive finite number.")
  }

  invisible(TRUE)
}


#' Check whether a value is a finite numeric scalar
#'
#' @param x Object to test.
#'
#' @return Logical scalar.
#'
#' @keywords internal
dmgfd_is_scalar_finite <- function(x) {
  is.numeric(x) && length(x) == 1 && !is.na(x) && is.finite(x)
}


#' Return first non-missing value
#'
#' @param x Numeric vector.
#'
#' @return First non-missing value or \code{NA_real_}.
#'
#' @keywords internal
dmgfd_first_non_na <- function(x) {
  idx <- which(!is.na(x))[1]

  if (length(idx) == 0 || is.na(idx)) {
    return(NA_real_)
  }

  x[idx]
}


#' Cumulative maximum while preserving missing values
#'
#' @param x Numeric vector.
#'
#' @return Numeric vector.
#'
#' @keywords internal
dmgfd_cummax_na <- function(x) {
  out <- x
  idx <- which(!is.na(x))

  if (length(idx) > 0) {
    out[idx] <- cummax(x[idx])
  }

  out
}


#' Convert year and day-of-year to Date
#'
#' @param year Calendar year.
#' @param doy Day of year.
#'
#' @return Date.
#'
#' @keywords internal
dmgfd_doy_to_date <- function(year, doy) {
  as.Date(sprintf("%04d-01-01", year)) + (doy - 1)
}


#' Build vegetation-season table
#'
#' @param time_vec Time vector.
#' @param site_mode One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.
#' @param custom_veg_season Custom vegetation-season DOY range.
#'
#' @return Tibble with season labels and season timing.
#'
#' @keywords internal
dmgfd_build_season_table <- function(time_vec,
                                     site_mode = c("NH", "SH", "CS"),
                                     custom_veg_season = c(275, 274)) {
  site_mode <- match.arg(site_mode)

  dates <- as.Date(time_vec)
  yr <- lubridate::year(dates)
  doy <- lubridate::yday(dates)
  mo <- lubridate::month(dates)

  n <- length(dates)

  season_label <- rep(NA_character_, n)
  season_start <- as.Date(rep(NA_character_, n))
  season_end <- as.Date(rep(NA_character_, n))
  season_day <- rep(NA_integer_, n)

  if (site_mode == "NH") {
    season_start <- as.Date(sprintf("%04d-01-01", yr))
    season_end <- as.Date(sprintf("%04d-12-31", yr))
    season_label <- as.character(yr)
    season_day <- as.integer(dates - season_start) + 1L
  }

  if (site_mode == "SH") {
    start_year <- ifelse(mo >= 7, yr, yr - 1L)
    end_year <- start_year + 1L

    season_start <- as.Date(sprintf("%04d-07-01", start_year))
    season_end <- as.Date(sprintf("%04d-06-30", end_year))
    season_label <- sprintf("%04d/%04d", start_year, end_year)
    season_day <- as.integer(dates - season_start) + 1L
  }

  if (site_mode == "CS") {
    start_doy <- as.integer(custom_veg_season[1])
    end_doy <- as.integer(custom_veg_season[2])

    if (start_doy <= end_doy) {
      inside <- doy >= start_doy & doy <= end_doy

      season_start[inside] <- dmgfd_doy_to_date(yr[inside], start_doy)
      season_end[inside] <- dmgfd_doy_to_date(yr[inside], end_doy)
      season_label[inside] <- as.character(yr[inside])
      season_day[inside] <- as.integer(dates[inside] - season_start[inside]) + 1L
    } else {
      inside_late <- doy >= start_doy
      inside_early <- doy <= end_doy

      if (any(inside_late)) {
        sy <- yr[inside_late]
        ey <- sy + 1L

        season_start[inside_late] <- dmgfd_doy_to_date(sy, start_doy)
        season_end[inside_late] <- dmgfd_doy_to_date(ey, end_doy)
        season_label[inside_late] <- sprintf("%04d/%04d", sy, ey)
        season_day[inside_late] <- as.integer(dates[inside_late] - season_start[inside_late]) + 1L
      }

      if (any(inside_early)) {
        ey <- yr[inside_early]
        sy <- ey - 1L

        season_start[inside_early] <- dmgfd_doy_to_date(sy, start_doy)
        season_end[inside_early] <- dmgfd_doy_to_date(ey, end_doy)
        season_label[inside_early] <- sprintf("%04d/%04d", sy, ey)
        season_day[inside_early] <- as.integer(dates[inside_early] - season_start[inside_early]) + 1L
      }
    }
  }

  tibble::tibble(
    season_label = season_label,
    season_start = season_start,
    season_end = season_end,
    season_day = season_day
  )
}


#' Add zero-growth anchor point
#'
#' @param x Season-day values.
#' @param y Growth values.
#'
#' @return List with \code{x} and \code{y}.
#'
#' @keywords internal
dmgfd_add_anchor_points <- function(x, y) {
  if (length(x) == 0) {
    return(list(x = x, y = y))
  }

  if (min(x, na.rm = TRUE) > 1 && !any(x == 1)) {
    x <- c(1, x)
    y <- c(0, y)
  }

  ord <- order(x)

  list(
    x = x[ord],
    y = y[ord]
  )
}


#' Empty parameter template for double growth fitting
#'
#' @return Named list of model and timing parameters.
#'
#' @keywords internal
dmgfd_empty_model_params <- function() {
  list(
    converged = FALSE,
    fallback_used = FALSE,
    fixed_a_used = FALSE,
    fixed_a_value = NA_real_,
    method_used = NA_character_,

    a = NA_real_,
    b = NA_real_,
    k = NA_real_,
    t0 = NA_real_,
    v = NA_real_,

    a2 = NA_real_,
    b2 = NA_real_,
    k2 = NA_real_,
    t02 = NA_real_,
    v2 = NA_real_,

    edf = NA_real_,
    span = NA_real_,
    spline_df = NA_real_,
    spar = NA_real_,
    model_note = NA_character_,

    n_obs = NA_real_,
    n_days_observed = NA_real_,
    first_obs_day = NA_real_,
    last_obs_day = NA_real_,
    season_length = NA_real_,
    n_missing_days = NA_real_,
    extrapolated = NA,
    anchor_added = FALSE,

    growth_start_day = NA_real_,
    growth_end_day = NA_real_,
    growth_start_season_day = NA_real_,
    growth_end_season_day = NA_real_,
    growth_start_date = as.Date(NA),
    growth_end_date = as.Date(NA),

    peak_rate = NA_real_,
    rate_start_day = NA_real_,
    rate_end_day = NA_real_,
    rate_start_season_day = NA_real_,
    rate_end_season_day = NA_real_,
    rate_start_date = as.Date(NA),
    rate_end_date = as.Date(NA),

    two_pulse_detected = FALSE,
    peak1_day = NA_real_,
    peak2_day = NA_real_,
    peak1_season_day = NA_real_,
    peak2_season_day = NA_real_,
    separator_day = NA_real_,
    separator_season_day = NA_real_,
    separator_date = as.Date(NA),
    valley_rate = NA_real_,
    pulse1_peak_rate = NA_real_,
    pulse2_peak_rate = NA_real_,

    pulse1_start_day = NA_real_,
    pulse1_end_day = NA_real_,
    pulse1_start_season_day = NA_real_,
    pulse1_end_season_day = NA_real_,
    pulse1_start_date = as.Date(NA),
    pulse1_end_date = as.Date(NA),

    pulse2_start_day = NA_real_,
    pulse2_end_day = NA_real_,
    pulse2_start_season_day = NA_real_,
    pulse2_end_season_day = NA_real_,
    pulse2_start_date = as.Date(NA),
    pulse2_end_date = as.Date(NA)
  )
}


#' Guess two pulse locations from growth increments
#'
#' @param x Season-day values.
#' @param y Cumulative growth values.
#'
#' @return Numeric vector of two initial pulse locations.
#'
#' @keywords internal
dmgfd_guess_two_pulses <- function(x, y) {
  x <- as.numeric(x)
  y <- as.numeric(y)

  ord <- order(x)
  x <- x[ord]
  y <- y[ord]

  dy <- c(NA_real_, diff(y))
  dy[!is.finite(dy)] <- -Inf

  min_sep <- max(14, round(0.15 * diff(range(x, na.rm = TRUE))))
  o <- order(dy, decreasing = TRUE)

  if (length(o) < 2 || !is.finite(dy[o[1]])) {
    qs <- stats::quantile(x, probs = c(0.33, 0.67), na.rm = TRUE, names = FALSE)
    return(as.numeric(qs))
  }

  t1 <- x[o[1]]
  t2 <- NA_real_

  if (length(o) > 1) {
    for (j in o[-1]) {
      if (is.finite(dy[j]) && abs(x[j] - t1) >= min_sep) {
        t2 <- x[j]
        break
      }
    }
  }

  if (!is.finite(t2)) {
    qs <- stats::quantile(x, probs = c(0.33, 0.67), na.rm = TRUE, names = FALSE)
    t1 <- qs[1]
    t2 <- qs[2]
  }

  sort(c(t1, t2))
}


#' Infer single Gompertz starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial asymptote.
#'
#' @return Named list with \code{a}, \code{b}, and \code{k}.
#'
#' @keywords internal
dmgfd_infer_gompertz_starts <- function(x, y, a0) {
  frac <- pmin(pmax(y / a0, 1e-6), 1 - 1e-6)
  z <- log(-log(frac))
  ok <- is.finite(x) & is.finite(z)

  b0 <- 0.5
  k0 <- 0.01

  if (sum(ok) >= 2) {
    lm_fit <- try(stats::lm(z[ok] ~ x[ok]), silent = TRUE)

    if (!inherits(lm_fit, "try-error")) {
      cf <- stats::coef(lm_fit)

      if (length(cf) == 2 && all(is.finite(cf))) {
        b_try <- unname(cf[1])
        k_try <- -unname(cf[2])

        if (is.finite(b_try)) {
          b0 <- b_try
        }

        if (is.finite(k_try) && k_try > 1e-4) {
          k0 <- k_try
        }
      }
    }
  }

  list(
    a = a0,
    b = b0,
    k = k0
  )
}


#' Infer logistic/Richards starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial asymptote.
#'
#' @return Named list with \code{a}, \code{k}, and \code{t0}.
#'
#' @keywords internal
dmgfd_infer_logistic_starts <- function(x, y, a0) {
  frac <- pmin(pmax(y / a0, 1e-6), 1 - 1e-6)
  z <- stats::qlogis(frac)
  ok <- is.finite(x) & is.finite(z)

  k0 <- 0.03
  t00 <- stats::median(x, na.rm = TRUE)

  if (sum(ok) >= 2) {
    lm_fit <- try(stats::lm(z[ok] ~ x[ok]), silent = TRUE)

    if (!inherits(lm_fit, "try-error")) {
      cf <- stats::coef(lm_fit)

      if (length(cf) == 2 && all(is.finite(cf))) {
        k_try <- abs(unname(cf[2]))

        if (is.finite(k_try) && k_try > 1e-4) {
          k0 <- k_try

          t0_try <- -unname(cf[1]) / k0

          if (is.finite(t0_try)) {
            t00 <- t0_try
          }
        }
      }
    }
  }

  list(
    a = a0,
    k = k0,
    t0 = t00
  )
}


#' Infer double-Gompertz starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial total asymptote.
#'
#' @return Named list of starting values.
#'
#' @keywords internal
dmgfd_infer_double_gompertz_starts <- function(x, y, a0) {
  pulses <- dmgfd_guess_two_pulses(x, y)
  base <- dmgfd_infer_gompertz_starts(x, y, a0)
  k0 <- if (dmgfd_is_scalar_finite(base$k)) base$k else 0.01

  list(
    a = a0 * 0.5,
    b = k0 * pulses[1],
    k = k0,
    t0 = pulses[1],
    a2 = a0 * 0.5,
    b2 = k0 * pulses[2],
    k2 = k0,
    t02 = pulses[2]
  )
}


#' Infer double-Richards starting values
#'
#' @param x Season-day values.
#' @param y Growth values.
#' @param a0 Initial total asymptote.
#'
#' @return Named list of starting values.
#'
#' @keywords internal
dmgfd_infer_double_richards_starts <- function(x, y, a0) {
  pulses <- dmgfd_guess_two_pulses(x, y)
  base <- dmgfd_infer_logistic_starts(x, y, a0)
  k0 <- if (dmgfd_is_scalar_finite(base$k)) base$k else 0.03

  list(
    a = a0 * 0.5,
    k = k0,
    t0 = pulses[1],
    v = 1,
    a2 = a0 * 0.5,
    k2 = k0,
    t02 = pulses[2],
    v2 = 1
  )
}


#' Fit double-Gompertz model
#'
#' @param x Observed season-day values.
#' @param y Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param fixed_a_value Optional fixed total asymptote on original scale.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param start_value_double_gompertz_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgfd_fit_model_double_gompertz <- function(
    x,
    y,
    x_pred,
    fixed_a_value = NA_real_,
    fix_a_to_observed_max = FALSE,
    start_value_double_gompertz_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_
    )
) {
  params <- dmgfd_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    start_guess <- dmgfd_infer_double_gompertz_starts(x, y_adj, a0)
    start_guess <- utils::modifyList(start_guess, start_value_double_gompertz_parameters)

    if (!dmgfd_is_scalar_finite(start_guess$a)) {
      start_guess$a <- a0 * 0.5
    }
    if (!dmgfd_is_scalar_finite(start_guess$k)) {
      start_guess$k <- 0.01
    }
    if (!dmgfd_is_scalar_finite(start_guess$t0)) {
      start_guess$t0 <- stats::quantile(x, 0.33, na.rm = TRUE, names = FALSE)
    }
    if (!dmgfd_is_scalar_finite(start_guess$a2)) {
      start_guess$a2 <- a0 * 0.5
    }
    if (!dmgfd_is_scalar_finite(start_guess$k2)) {
      start_guess$k2 <- start_guess$k
    }
    if (!dmgfd_is_scalar_finite(start_guess$t02)) {
      start_guess$t02 <- stats::quantile(x, 0.67, na.rm = TRUE, names = FALSE)
    }

    dt_start <- max(5, start_guess$t02 - start_guess$t0)

    dat <- data.frame(
      x = x,
      y = y_adj
    )

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgfd_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      p_start <- 0.5

      if (dmgfd_is_scalar_finite(start_guess$a) &&
          dmgfd_is_scalar_finite(start_guess$a2) &&
          (start_guess$a + start_guess$a2) > 0) {
        p_start <- start_guess$a / (start_guess$a + start_guess$a2)
      }

      p_start <- min(max(p_start, 0.05), 0.95)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit * p * exp(-exp(-(k * (x - t0)))) +
          a_fixed_fit * (1 - p) * exp(-exp(-(k2 * (x - (t0 + dt))))),
        data = dat,
        start = list(
          p = p_start,
          k = start_guess$k,
          t0 = start_guess$t0,
          k2 = start_guess$k2,
          dt = dt_start
        ),
        lower = c(
          p = 0.001,
          k = 1e-6,
          t0 = min(x, na.rm = TRUE),
          k2 = 1e-6,
          dt = 1
        ),
        upper = c(
          p = 0.999,
          k = 5,
          t0 = max(x, na.rm = TRUE),
          k2 = 5,
          dt = diff(range(x, na.rm = TRUE))
        ),
        control = minpack.lm::nls.lm.control(maxiter = 1000)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- pmax(as.numeric(pred), 0)
      pred <- cummax(pred)

      cf <- stats::coef(mod)
      p_est <- unname(cf["p"])

      params$a <- a_fixed_original * p_est
      params$a2 <- a_fixed_original * (1 - p_est)

      params$k <- unname(cf["k"])
      params$t0 <- unname(cf["t0"])
      params$b <- params$k * params$t0

      params$k2 <- unname(cf["k2"])
      params$t02 <- unname(cf["t0"] + cf["dt"])
      params$b2 <- params$k2 * params$t02

      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$method_used <- "double_gompertz"
      params$model_note <- "Double-Gompertz total asymptote a + a2 fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    mod <- minpack.lm::nlsLM(
      y ~ a * exp(-exp(-(k * (x - t0)))) +
        a2 * exp(-exp(-(k2 * (x - (t0 + dt))))),
      data = dat,
      start = list(
        a = start_guess$a,
        k = start_guess$k,
        t0 = start_guess$t0,
        a2 = start_guess$a2,
        k2 = start_guess$k2,
        dt = dt_start
      ),
      lower = c(
        a = 0,
        k = 1e-6,
        t0 = min(x, na.rm = TRUE),
        a2 = 0,
        k2 = 1e-6,
        dt = 1
      ),
      upper = c(
        a = Inf,
        k = 5,
        t0 = max(x, na.rm = TRUE),
        a2 = Inf,
        k2 = 5,
        dt = diff(range(x, na.rm = TRUE))
      ),
      control = minpack.lm::nls.lm.control(maxiter = 1000)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- pmax(as.numeric(pred), 0)
    pred <- cummax(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$k <- unname(cf["k"])
    params$t0 <- unname(cf["t0"])
    params$b <- params$k * params$t0

    params$a2 <- unname(cf["a2"])
    params$k2 <- unname(cf["k2"])
    params$t02 <- unname(cf["t0"] + cf["dt"])
    params$b2 <- params$k2 * params$t02

    params$converged <- TRUE
    params$method_used <- "double_gompertz"

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    params$method_used <- "double_gompertz"
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit double-Richards model
#'
#' @param x Observed season-day values.
#' @param y Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param fixed_a_value Optional fixed total asymptote on original scale.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param start_value_double_richards_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgfd_fit_model_double_richards <- function(
    x,
    y,
    x_pred,
    fixed_a_value = NA_real_,
    fix_a_to_observed_max = FALSE,
    start_value_double_richards_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_, v2 = 1
    )
) {
  params <- dmgfd_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    start_guess <- dmgfd_infer_double_richards_starts(x, y_adj, a0)
    start_guess <- utils::modifyList(start_guess, start_value_double_richards_parameters)

    if (!dmgfd_is_scalar_finite(start_guess$a)) {
      start_guess$a <- a0 * 0.5
    }
    if (!dmgfd_is_scalar_finite(start_guess$k)) {
      start_guess$k <- 0.03
    }
    if (!dmgfd_is_scalar_finite(start_guess$t0)) {
      start_guess$t0 <- stats::quantile(x, 0.33, na.rm = TRUE, names = FALSE)
    }
    if (!dmgfd_is_scalar_finite(start_guess$v)) {
      start_guess$v <- 1
    }
    if (!dmgfd_is_scalar_finite(start_guess$a2)) {
      start_guess$a2 <- a0 * 0.5
    }
    if (!dmgfd_is_scalar_finite(start_guess$k2)) {
      start_guess$k2 <- start_guess$k
    }
    if (!dmgfd_is_scalar_finite(start_guess$t02)) {
      start_guess$t02 <- stats::quantile(x, 0.67, na.rm = TRUE, names = FALSE)
    }
    if (!dmgfd_is_scalar_finite(start_guess$v2)) {
      start_guess$v2 <- 1
    }

    dt_start <- max(5, start_guess$t02 - start_guess$t0)

    dat <- data.frame(
      x = x,
      y = y_adj
    )

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgfd_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      p_start <- 0.5

      if (dmgfd_is_scalar_finite(start_guess$a) &&
          dmgfd_is_scalar_finite(start_guess$a2) &&
          (start_guess$a + start_guess$a2) > 0) {
        p_start <- start_guess$a / (start_guess$a + start_guess$a2)
      }

      p_start <- min(max(p_start, 0.05), 0.95)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit * p / ((1 + v * exp(-k * (x - t0)))^(1 / v)) +
          a_fixed_fit * (1 - p) / ((1 + v2 * exp(-k2 * (x - (t0 + dt))))^(1 / v2)),
        data = dat,
        start = list(
          p = p_start,
          k = start_guess$k,
          t0 = start_guess$t0,
          v = start_guess$v,
          k2 = start_guess$k2,
          dt = dt_start,
          v2 = start_guess$v2
        ),
        lower = c(
          p = 0.001,
          k = 1e-6,
          t0 = min(x, na.rm = TRUE),
          v = 1e-3,
          k2 = 1e-6,
          dt = 1,
          v2 = 1e-3
        ),
        upper = c(
          p = 0.999,
          k = 5,
          t0 = max(x, na.rm = TRUE),
          v = 50,
          k2 = 5,
          dt = diff(range(x, na.rm = TRUE)),
          v2 = 50
        ),
        control = minpack.lm::nls.lm.control(maxiter = 1000)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- pmax(as.numeric(pred), 0)
      pred <- cummax(pred)

      cf <- stats::coef(mod)
      p_est <- unname(cf["p"])

      params$a <- a_fixed_original * p_est
      params$a2 <- a_fixed_original * (1 - p_est)

      params$k <- unname(cf["k"])
      params$t0 <- unname(cf["t0"])
      params$v <- unname(cf["v"])

      params$k2 <- unname(cf["k2"])
      params$t02 <- unname(cf["t0"] + cf["dt"])
      params$v2 <- unname(cf["v2"])

      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$method_used <- "double_richards"
      params$model_note <- "Double-Richards total asymptote a + a2 fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    mod <- minpack.lm::nlsLM(
      y ~ a / ((1 + v * exp(-k * (x - t0)))^(1 / v)) +
        a2 / ((1 + v2 * exp(-k2 * (x - (t0 + dt))))^(1 / v2)),
      data = dat,
      start = list(
        a = start_guess$a,
        k = start_guess$k,
        t0 = start_guess$t0,
        v = start_guess$v,
        a2 = start_guess$a2,
        k2 = start_guess$k2,
        dt = dt_start,
        v2 = start_guess$v2
      ),
      lower = c(
        a = 0,
        k = 1e-6,
        t0 = min(x, na.rm = TRUE),
        v = 1e-3,
        a2 = 0,
        k2 = 1e-6,
        dt = 1,
        v2 = 1e-3
      ),
      upper = c(
        a = Inf,
        k = 5,
        t0 = max(x, na.rm = TRUE),
        v = 50,
        a2 = Inf,
        k2 = 5,
        dt = diff(range(x, na.rm = TRUE)),
        v2 = 50
      ),
      control = minpack.lm::nls.lm.control(maxiter = 1000)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- pmax(as.numeric(pred), 0)
    pred <- cummax(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$k <- unname(cf["k"])
    params$t0 <- unname(cf["t0"])
    params$v <- unname(cf["v"])

    params$a2 <- unname(cf["a2"])
    params$k2 <- unname(cf["k2"])
    params$t02 <- unname(cf["t0"] + cf["dt"])
    params$v2 <- unname(cf["v2"])

    params$converged <- TRUE
    params$method_used <- "double_richards"

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    params$method_used <- "double_richards"
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit single-Gompertz fallback model
#'
#' @param x Observed season-day values.
#' @param y Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param fixed_a_value Optional fixed asymptote.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param start_value_gompertz_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgfd_fit_model_single_gompertz <- function(
    x,
    y,
    x_pred,
    fixed_a_value = NA_real_,
    fix_a_to_observed_max = FALSE,
    start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_)
) {
  params <- dmgfd_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgfd_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      start_guess <- dmgfd_infer_gompertz_starts(x, y_adj, a_fixed_fit)
      start_guess <- utils::modifyList(start_guess, start_value_gompertz_parameters)

      if (!dmgfd_is_scalar_finite(start_guess$b)) {
        start_guess$b <- 0.5
      }

      if (!dmgfd_is_scalar_finite(start_guess$k)) {
        start_guess$k <- 0.01
      }

      dat <- data.frame(x = x, y = y_adj)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit * exp(-exp(b - k * x)),
        data = dat,
        start = list(
          b = start_guess$b,
          k = start_guess$k
        ),
        control = minpack.lm::nls.lm.control(maxiter = 500)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- as.numeric(pred)

      cf <- stats::coef(mod)

      params$a <- a_fixed_original
      params$b <- unname(cf["b"])
      params$k <- unname(cf["k"])
      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$method_used <- "single_gompertz"
      params$model_note <- "Single-Gompertz fallback asymptote a fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    start_guess <- dmgfd_infer_gompertz_starts(x, y_adj, a0)
    start_guess <- utils::modifyList(start_guess, start_value_gompertz_parameters)

    if (!dmgfd_is_scalar_finite(start_guess$a)) {
      start_guess$a <- a0
    }

    if (!dmgfd_is_scalar_finite(start_guess$b)) {
      start_guess$b <- 0.5
    }

    if (!dmgfd_is_scalar_finite(start_guess$k)) {
      start_guess$k <- 0.01
    }

    dat <- data.frame(x = x, y = y_adj)

    mod <- minpack.lm::nlsLM(
      y ~ a * exp(-exp(b - k * x)),
      data = dat,
      start = start_guess,
      control = minpack.lm::nls.lm.control(maxiter = 500)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- as.numeric(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$b <- unname(cf["b"])
    params$k <- unname(cf["k"])
    params$converged <- TRUE
    params$method_used <- "single_gompertz"

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    params$method_used <- "single_gompertz"
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Fit single-Richards fallback model
#'
#' @param x Observed season-day values.
#' @param y Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param fixed_a_value Optional fixed asymptote.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param start_value_richards_parameters Starting values.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgfd_fit_model_single_richards <- function(
    x,
    y,
    x_pred,
    fixed_a_value = NA_real_,
    fix_a_to_observed_max = FALSE,
    start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1)
) {
  params <- dmgfd_empty_model_params()

  res <- tryCatch({
    cons <- ifelse(
      min(y, na.rm = TRUE) < 0,
      abs(min(y, na.rm = TRUE)) + 1e-8,
      0
    )

    y_adj <- y + cons
    a0 <- max(y_adj, na.rm = TRUE) * 1.05 + 1e-6

    use_fixed_a <- isTRUE(fix_a_to_observed_max) &&
      dmgfd_is_scalar_finite(fixed_a_value) &&
      fixed_a_value > 0

    if (isTRUE(use_fixed_a)) {
      a_fixed_original <- fixed_a_value
      a_fixed_fit <- fixed_a_value + cons

      logi_start <- dmgfd_infer_logistic_starts(x, y_adj, a_fixed_fit)

      start_vals <- list(
        k = if (dmgfd_is_scalar_finite(start_value_richards_parameters$k)) {
          start_value_richards_parameters$k
        } else {
          logi_start$k
        },
        t0 = if (dmgfd_is_scalar_finite(start_value_richards_parameters$t0)) {
          start_value_richards_parameters$t0
        } else {
          logi_start$t0
        },
        v = if (dmgfd_is_scalar_finite(start_value_richards_parameters$v)) {
          start_value_richards_parameters$v
        } else {
          1
        }
      )

      dat <- data.frame(x = x, y = y_adj)

      mod <- minpack.lm::nlsLM(
        y ~ a_fixed_fit / ((1 + v * exp(-k * (x - t0)))^(1 / v)),
        data = dat,
        start = start_vals,
        control = minpack.lm::nls.lm.control(maxiter = 500)
      )

      pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
      pred <- as.numeric(pred)

      cf <- stats::coef(mod)

      params$a <- a_fixed_original
      params$k <- unname(cf["k"])
      params$t0 <- unname(cf["t0"])
      params$v <- unname(cf["v"])
      params$converged <- TRUE
      params$fixed_a_used <- TRUE
      params$fixed_a_value <- a_fixed_original
      params$method_used <- "single_richards"
      params$model_note <- "Single-Richards fallback asymptote a fixed to observed seasonal maximum."

      return(list(pred = pred, params = params))
    }

    logi_start <- dmgfd_infer_logistic_starts(x, y_adj, a0)

    start_vals <- list(
      a = if (dmgfd_is_scalar_finite(start_value_richards_parameters$a)) {
        start_value_richards_parameters$a
      } else {
        logi_start$a
      },
      k = if (dmgfd_is_scalar_finite(start_value_richards_parameters$k)) {
        start_value_richards_parameters$k
      } else {
        logi_start$k
      },
      t0 = if (dmgfd_is_scalar_finite(start_value_richards_parameters$t0)) {
        start_value_richards_parameters$t0
      } else {
        logi_start$t0
      },
      v = if (dmgfd_is_scalar_finite(start_value_richards_parameters$v)) {
        start_value_richards_parameters$v
      } else {
        1
      }
    )

    dat <- data.frame(x = x, y = y_adj)

    mod <- minpack.lm::nlsLM(
      y ~ a / ((1 + v * exp(-k * (x - t0)))^(1 / v)),
      data = dat,
      start = start_vals,
      control = minpack.lm::nls.lm.control(maxiter = 500)
    )

    pred <- stats::predict(mod, newdata = data.frame(x = x_pred)) - cons
    pred <- as.numeric(pred)

    cf <- stats::coef(mod)

    params$a <- unname(cf["a"])
    params$k <- unname(cf["k"])
    params$t0 <- unname(cf["t0"])
    params$v <- unname(cf["v"])
    params$converged <- TRUE
    params$method_used <- "single_richards"

    list(pred = pred, params = params)
  }, error = function(e) {
    params$model_note <- conditionMessage(e)
    params$method_used <- "single_richards"
    list(pred = rep(NA_real_, length(x_pred)), params = params)
  })

  res
}


#' Add overall cumulative-growth timing
#'
#' @param params Parameter list.
#' @param pred_full Fitted cumulative growth.
#' @param x_pred Prediction season-day values.
#' @param growth_fraction Growth fractions.
#' @param season_start_date Season start date.
#'
#' @return Updated parameter list.
#'
#' @keywords internal
dmgfd_add_overall_timing <- function(params,
                                     pred_full,
                                     x_pred,
                                     growth_fraction,
                                     season_start_date = as.Date(NA)) {
  valid_pred <- is.finite(pred_full) & is.finite(x_pred)

  if (sum(valid_pred) == 0) {
    return(params)
  }

  pred_use <- pred_full[valid_pred]
  doy_use <- x_pred[valid_pred]

  final_val <- tail(stats::na.omit(pred_use), 1)

  if (length(final_val) == 1 &&
      is.finite(final_val) &&
      final_val > 0) {
    low_val <- final_val * growth_fraction[1]
    high_val <- final_val * growth_fraction[2]

    st_pos <- suppressWarnings(min(doy_use[pred_use > low_val], na.rm = TRUE))
    en_pos <- suppressWarnings(max(doy_use[pred_use < high_val], na.rm = TRUE))

    if (is.finite(st_pos)) {
      params$growth_start_season_day <- st_pos

      if (!is.na(season_start_date)) {
        params$growth_start_date <- season_start_date + (st_pos - 1)
        params$growth_start_day <- lubridate::yday(params$growth_start_date)
      }
    }

    if (is.finite(en_pos)) {
      params$growth_end_season_day <- en_pos

      if (!is.na(season_start_date)) {
        params$growth_end_date <- season_start_date + (en_pos - 1)
        params$growth_end_day <- lubridate::yday(params$growth_end_date)
      }
    }
  }

  params
}


#' Add overall rate-based timing
#'
#' @param params Parameter list.
#' @param pred_full Fitted cumulative growth.
#' @param x_pred Prediction season-day values.
#' @param rate_threshold_fraction Fraction of peak rate.
#' @param season_start_date Season start date.
#'
#' @return Updated parameter list.
#'
#' @keywords internal
dmgfd_add_overall_rate_timing <- function(params,
                                          pred_full,
                                          x_pred,
                                          rate_threshold_fraction = 0.1,
                                          season_start_date = as.Date(NA)) {
  rate <- dmgfd_growth_rate(pred_full)

  ok <- is.finite(rate) & is.finite(x_pred)

  if (sum(ok) == 0) {
    return(params)
  }

  r <- rate[ok]
  x <- x_pred[ok]

  peak_rate <- suppressWarnings(max(r, na.rm = TRUE))

  if (!is.finite(peak_rate) || peak_rate <= 0) {
    return(params)
  }

  thr <- rate_threshold_fraction * peak_rate
  above <- which(r >= thr)

  if (length(above) == 0) {
    return(params)
  }

  params$peak_rate <- peak_rate
  params$rate_start_season_day <- min(x[above], na.rm = TRUE)
  params$rate_end_season_day <- max(x[above], na.rm = TRUE)

  if (!is.na(season_start_date)) {
    params$rate_start_date <- season_start_date + (params$rate_start_season_day - 1)
    params$rate_end_date <- season_start_date + (params$rate_end_season_day - 1)

    params$rate_start_day <- lubridate::yday(params$rate_start_date)
    params$rate_end_day <- lubridate::yday(params$rate_end_date)
  }

  params
}


#' Fit single-curve fallback
#'
#' @param method Original double method.
#' @param x Observed season-day values.
#' @param y Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param fixed_a_value Optional fixed asymptote.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param start_value_double_gompertz_parameters Double-Gompertz starts.
#' @param start_value_double_richards_parameters Double-Richards starts.
#'
#' @return List with \code{pred} and \code{params}.
#'
#' @keywords internal
dmgfd_fit_single_fallback <- function(method,
                                      x,
                                      y,
                                      x_pred,
                                      fixed_a_value = NA_real_,
                                      fix_a_to_observed_max = FALSE,
                                      start_value_double_gompertz_parameters,
                                      start_value_double_richards_parameters) {
  params <- dmgfd_empty_model_params()

  if (method == "gompertz") {
    start_single <- list(
      a = if (dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$a) &&
              dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$a2)) {
        start_value_double_gompertz_parameters$a +
          start_value_double_gompertz_parameters$a2
      } else if (dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$a)) {
        start_value_double_gompertz_parameters$a
      } else {
        NA_real_
      },
      b = if (dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$k) &&
              dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$t0)) {
        start_value_double_gompertz_parameters$k *
          start_value_double_gompertz_parameters$t0
      } else {
        NA_real_
      },
      k = if (dmgfd_is_scalar_finite(start_value_double_gompertz_parameters$k)) {
        start_value_double_gompertz_parameters$k
      } else {
        NA_real_
      }
    )

    single_res <- dmgfd_fit_model_single_gompertz(
      x = x,
      y = y,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_gompertz_parameters = start_single
    )

    params$a <- single_res$params$a
    params$b <- single_res$params$b
    params$k <- single_res$params$k
    params$converged <- single_res$params$converged
    params$fixed_a_used <- single_res$params$fixed_a_used
    params$fixed_a_value <- single_res$params$fixed_a_value
    params$model_note <- single_res$params$model_note
    params$method_used <- "single_gompertz"
    params$fallback_used <- TRUE

    return(list(pred = single_res$pred, params = params))
  }

  if (method == "richards") {
    start_single <- list(
      a = if (dmgfd_is_scalar_finite(start_value_double_richards_parameters$a) &&
              dmgfd_is_scalar_finite(start_value_double_richards_parameters$a2)) {
        start_value_double_richards_parameters$a +
          start_value_double_richards_parameters$a2
      } else if (dmgfd_is_scalar_finite(start_value_double_richards_parameters$a)) {
        start_value_double_richards_parameters$a
      } else {
        NA_real_
      },
      k = if (dmgfd_is_scalar_finite(start_value_double_richards_parameters$k)) {
        start_value_double_richards_parameters$k
      } else {
        NA_real_
      },
      t0 = if (dmgfd_is_scalar_finite(start_value_double_richards_parameters$t0)) {
        start_value_double_richards_parameters$t0
      } else {
        NA_real_
      },
      v = if (dmgfd_is_scalar_finite(start_value_double_richards_parameters$v)) {
        start_value_double_richards_parameters$v
      } else {
        1
      }
    )

    single_res <- dmgfd_fit_model_single_richards(
      x = x,
      y = y,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_richards_parameters = start_single
    )

    params$a <- single_res$params$a
    params$k <- single_res$params$k
    params$t0 <- single_res$params$t0
    params$v <- single_res$params$v
    params$converged <- single_res$params$converged
    params$fixed_a_used <- single_res$params$fixed_a_used
    params$fixed_a_value <- single_res$params$fixed_a_value
    params$model_note <- single_res$params$model_note
    params$method_used <- "single_richards"
    params$fallback_used <- TRUE

    return(list(pred = single_res$pred, params = params))
  }

  stop("Unknown single fallback method: ", method)
}


#' Calculate non-negative fitted growth rate
#'
#' @param pred Fitted cumulative growth.
#'
#' @return Numeric growth-rate vector.
#'
#' @keywords internal
dmgfd_growth_rate <- function(pred) {
  pred <- as.numeric(pred)

  if (length(pred) < 2) {
    return(rep(NA_real_, length(pred)))
  }

  rate <- c(NA_real_, diff(pred))
  rate[!is.finite(rate)] <- NA_real_
  rate <- pmax(rate, 0)

  rate
}


#' Identify local maxima
#'
#' @param x Numeric vector.
#'
#' @return Integer indices of local maxima.
#'
#' @keywords internal
dmgfd_local_maxima <- function(x) {
  x <- as.numeric(x)
  n <- length(x)

  if (n < 3) {
    return(integer(0))
  }

  which(
    is.finite(x[2:(n - 1)]) &
      x[2:(n - 1)] >= x[1:(n - 2)] &
      x[2:(n - 1)] >= x[3:n]
  ) + 1L
}


#' Detect two-pulse pattern from fitted growth rate
#'
#' @param rate Fitted growth-rate vector.
#' @param x_pred Prediction season-day values.
#' @param peak_separation_min Minimum separation between peaks.
#' @param valley_ratio_max Maximum allowed valley-to-peak ratio.
#' @param min_relative_peak Minimum relative peak height.
#'
#' @return List describing detected pulse pattern.
#'
#' @keywords internal
dmgfd_detect_two_pulse_pattern <- function(rate,
                                           x_pred,
                                           peak_separation_min = 10,
                                           valley_ratio_max = 0.4,
                                           min_relative_peak = 0.05) {
  out <- list(
    detected = FALSE,
    peak1_day = NA_real_,
    peak2_day = NA_real_,
    separator_day = NA_real_,
    valley_rate = NA_real_,
    pulse1_peak_rate = NA_real_,
    pulse2_peak_rate = NA_real_
  )

  ok <- is.finite(rate) & is.finite(x_pred)

  if (sum(ok) < 5) {
    return(out)
  }

  r <- as.numeric(rate)
  x <- as.numeric(x_pred)

  peak_idx <- dmgfd_local_maxima(r)

  if (length(peak_idx) < 2) {
    return(out)
  }

  global_peak <- max(r, na.rm = TRUE)

  if (!is.finite(global_peak) || global_peak <= 0) {
    return(out)
  }

  peak_idx <- peak_idx[r[peak_idx] >= min_relative_peak * global_peak]

  if (length(peak_idx) < 2) {
    return(out)
  }

  best_score <- -Inf
  best <- NULL

  for (i in seq_len(length(peak_idx) - 1)) {
    for (j in (i + 1):length(peak_idx)) {
      p1 <- peak_idx[i]
      p2 <- peak_idx[j]

      if ((x[p2] - x[p1]) < peak_separation_min) {
        next
      }

      valley_idx <- p1:p2
      valley_rate <- suppressWarnings(min(r[valley_idx], na.rm = TRUE))

      if (!is.finite(valley_rate)) {
        next
      }

      if (valley_rate > valley_ratio_max * min(r[p1], r[p2])) {
        next
      }

      score <- r[p1] + r[p2] - valley_rate

      if (score > best_score) {
        sep_idx <- valley_idx[which.min(r[valley_idx])]

        best_score <- score
        best <- list(
          p1 = p1,
          p2 = p2,
          sep = sep_idx,
          valley_rate = valley_rate
        )
      }
    }
  }

  if (is.null(best)) {
    return(out)
  }

  out$detected <- TRUE
  out$peak1_day <- x[best$p1]
  out$peak2_day <- x[best$p2]
  out$separator_day <- x[best$sep]
  out$valley_rate <- best$valley_rate
  out$pulse1_peak_rate <- r[best$p1]
  out$pulse2_peak_rate <- r[best$p2]

  out
}


#' Estimate pulse window from fitted growth rate
#'
#' @param rate Fitted growth-rate vector.
#' @param x_pred Prediction season-day values.
#' @param threshold Pulse-specific rate threshold.
#' @param domain_idx Indices belonging to one pulse domain.
#'
#' @return List with start and end season-day.
#'
#' @keywords internal
dmgfd_pulse_window_from_rate <- function(rate,
                                         x_pred,
                                         threshold,
                                         domain_idx) {
  out <- list(
    start_day = NA_real_,
    end_day = NA_real_
  )

  if (length(domain_idx) == 0) {
    return(out)
  }

  r <- rate[domain_idx]
  x <- x_pred[domain_idx]

  ok <- is.finite(r) & is.finite(x)

  if (sum(ok) == 0) {
    return(out)
  }

  r <- r[ok]
  x <- x[ok]

  above <- which(r >= threshold)

  if (length(above) == 0) {
    return(out)
  }

  out$start_day <- min(x[above], na.rm = TRUE)
  out$end_day <- max(x[above], na.rm = TRUE)

  out
}


#' Fit one double growth curve
#'
#' @description
#' Internal helper used by [dm.growth.fit.double()] to fit one double curve for
#' one series and one season, or one pooled double curve.
#'
#' @param x_obs Observed season-day values.
#' @param y_obs Observed cumulative growth values.
#' @param x_pred Prediction season-day values.
#' @param season_start_date Season start date.
#' @param season_length Season length.
#' @param method Double-growth method.
#' @param growth_fraction Growth fractions used for cumulative timing.
#' @param min_unique_growth Minimum number of unique growth values.
#' @param rate_threshold_fraction Fraction of peak rate.
#' @param peak_separation_min Minimum separation between peaks.
#' @param valley_ratio_max Maximum allowed valley-to-peak ratio.
#' @param min_relative_peak Minimum relative derivative peak height.
#' @param fallback_to_single Logical fallback flag.
#' @param fix_a_to_observed_max Logical fixed-asymptote flag.
#' @param fixed_a_multiplier Multiplier for fixed asymptote.
#' @param start_value_double_gompertz_parameters Double-Gompertz starts.
#' @param start_value_double_richards_parameters Double-Richards starts.
#'
#' @return List with fitted predictions and parameters.
#'
#' @keywords internal
dmgfd_fit_one_curve <- function(x_obs,
                                y_obs,
                                x_pred,
                                season_start_date = as.Date(NA),
                                season_length = NA_real_,
                                method,
                                growth_fraction,
                                min_unique_growth,
                                rate_threshold_fraction = 0.1,
                                peak_separation_min = 10,
                                valley_ratio_max = 0.4,
                                min_relative_peak = 0.05,
                                fallback_to_single = TRUE,
                                fix_a_to_observed_max = FALSE,
                                fixed_a_multiplier = 1,
                                start_value_double_gompertz_parameters,
                                start_value_double_richards_parameters) {
  pred_full <- rep(NA_real_, length(x_pred))

  params <- dmgfd_empty_model_params()

  params$n_obs <- sum(is.finite(x_obs) & is.finite(y_obs))
  params$n_days_observed <- NA_integer_
  params$first_obs_day <- NA_real_
  params$last_obs_day <- NA_real_
  params$season_length <- season_length
  params$n_missing_days <- NA_real_
  params$extrapolated <- NA
  params$anchor_added <- FALSE
  params$method_used <- paste0("double_", method)

  good <- is.finite(x_obs) & is.finite(y_obs)

  if (sum(good) == 0) {
    params$model_note <- "No observed data in this vegetation year."
    return(list(pred = pred_full, params = params))
  }

  x_raw <- x_obs[good]
  y_raw <- y_obs[good]

  params$n_days_observed <- length(x_raw)
  params$first_obs_day <- min(x_raw, na.rm = TRUE)
  params$last_obs_day <- max(x_raw, na.rm = TRUE)

  if (is.finite(season_length)) {
    params$n_missing_days <- season_length - length(unique(x_raw))
    params$extrapolated <- params$first_obs_day > 1 ||
      params$last_obs_day < season_length
  }

  if (length(unique(y_raw)) < min_unique_growth) {
    params$model_note <- paste0(
      "Too few unique cumulative-growth values (",
      length(unique(y_raw)), " < ", min_unique_growth, ")."
    )

    return(list(pred = pred_full, params = params))
  }

  anchored <- dmgfd_add_anchor_points(x_raw, y_raw)

  x_fit <- anchored$x
  y_fit <- anchored$y

  params$anchor_added <- length(x_fit) > length(x_raw)

  dyn_range <- max(y_fit, na.rm = TRUE) - min(y_fit, na.rm = TRUE)

  if (!is.finite(dyn_range) || dyn_range <= 0) {
    params$model_note <- "Observed cumulative-growth range is too small."
    return(list(pred = pred_full, params = params))
  }

  fixed_a_value <- NA_real_

  if (isTRUE(fix_a_to_observed_max) &&
      method %in% c("gompertz", "richards")) {
    fixed_a_value <- suppressWarnings(max(y_raw, na.rm = TRUE)) * fixed_a_multiplier

    if (!is.finite(fixed_a_value) || fixed_a_value <= 0) {
      fixed_a_value <- NA_real_
    }
  }

  fit_res <- switch(
    method,
    gompertz = dmgfd_fit_model_double_gompertz(
      x = x_fit,
      y = y_fit,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_double_gompertz_parameters = start_value_double_gompertz_parameters
    ),
    richards = dmgfd_fit_model_double_richards(
      x = x_fit,
      y = y_fit,
      x_pred = x_pred,
      fixed_a_value = fixed_a_value,
      fix_a_to_observed_max = fix_a_to_observed_max,
      start_value_double_richards_parameters = start_value_double_richards_parameters
    )
  )

  pred_full <- fit_res$pred
  params[names(fit_res$params)] <- fit_res$params
  params$method_used <- paste0("double_", method)

  params <- dmgfd_add_overall_timing(
    params = params,
    pred_full = pred_full,
    x_pred = x_pred,
    growth_fraction = growth_fraction,
    season_start_date = season_start_date
  )

  params <- dmgfd_add_overall_rate_timing(
    params = params,
    pred_full = pred_full,
    x_pred = x_pred,
    rate_threshold_fraction = rate_threshold_fraction,
    season_start_date = season_start_date
  )

  rate <- dmgfd_growth_rate(pred_full)

  pulse_info <- dmgfd_detect_two_pulse_pattern(
    rate = rate,
    x_pred = x_pred,
    peak_separation_min = peak_separation_min,
    valley_ratio_max = valley_ratio_max,
    min_relative_peak = min_relative_peak
  )

  if (isTRUE(pulse_info$detected)) {
    params$two_pulse_detected <- TRUE

    params$peak1_season_day <- pulse_info$peak1_day
    params$peak2_season_day <- pulse_info$peak2_day
    params$separator_season_day <- pulse_info$separator_day
    params$valley_rate <- pulse_info$valley_rate
    params$pulse1_peak_rate <- pulse_info$pulse1_peak_rate
    params$pulse2_peak_rate <- pulse_info$pulse2_peak_rate

    if (!is.na(season_start_date)) {
      peak1_date <- season_start_date + (params$peak1_season_day - 1)
      peak2_date <- season_start_date + (params$peak2_season_day - 1)

      params$separator_date <- season_start_date + (params$separator_season_day - 1)

      params$peak1_day <- lubridate::yday(peak1_date)
      params$peak2_day <- lubridate::yday(peak2_date)
      params$separator_day <- lubridate::yday(params$separator_date)
    }

    thr1 <- rate_threshold_fraction * params$pulse1_peak_rate
    thr2 <- rate_threshold_fraction * params$pulse2_peak_rate

    idx1 <- which(is.finite(x_pred) & x_pred <= params$separator_season_day)
    idx2 <- which(is.finite(x_pred) & x_pred > params$separator_season_day)

    win1 <- dmgfd_pulse_window_from_rate(
      rate = rate,
      x_pred = x_pred,
      threshold = thr1,
      domain_idx = idx1
    )

    win2 <- dmgfd_pulse_window_from_rate(
      rate = rate,
      x_pred = x_pred,
      threshold = thr2,
      domain_idx = idx2
    )

    if (is.finite(win1$start_day)) {
      params$pulse1_start_season_day <- win1$start_day

      if (!is.na(season_start_date)) {
        params$pulse1_start_date <- season_start_date + (win1$start_day - 1)
        params$pulse1_start_day <- lubridate::yday(params$pulse1_start_date)
      }
    }

    if (is.finite(win1$end_day)) {
      params$pulse1_end_season_day <- win1$end_day

      if (!is.na(season_start_date)) {
        params$pulse1_end_date <- season_start_date + (win1$end_day - 1)
        params$pulse1_end_day <- lubridate::yday(params$pulse1_end_date)
      }
    }

    if (is.finite(win2$start_day)) {
      params$pulse2_start_season_day <- win2$start_day

      if (!is.na(season_start_date)) {
        params$pulse2_start_date <- season_start_date + (win2$start_day - 1)
        params$pulse2_start_day <- lubridate::yday(params$pulse2_start_date)
      }
    }

    if (is.finite(win2$end_day)) {
      params$pulse2_end_season_day <- win2$end_day

      if (!is.na(season_start_date)) {
        params$pulse2_end_date <- season_start_date + (win2$end_day - 1)
        params$pulse2_end_day <- lubridate::yday(params$pulse2_end_date)
      }
    }

    return(list(pred = pred_full, params = params))
  }

  params$two_pulse_detected <- FALSE

  if (!isTRUE(fallback_to_single)) {
    if (is.na(params$model_note) || !nzchar(params$model_note)) {
      params$model_note <- "No convincing two-pulse pattern detected."
    } else {
      params$model_note <- paste(
        params$model_note,
        "No convincing two-pulse pattern detected."
      )
    }

    return(list(pred = pred_full, params = params))
  }

  meta_fields <- c(
    "n_obs", "n_days_observed", "first_obs_day", "last_obs_day",
    "season_length", "n_missing_days", "extrapolated", "anchor_added"
  )

  double_note <- params$model_note

  fallback_res <- dmgfd_fit_single_fallback(
    method = method,
    x = x_fit,
    y = y_fit,
    x_pred = x_pred,
    fixed_a_value = fixed_a_value,
    fix_a_to_observed_max = fix_a_to_observed_max,
    start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
    start_value_double_richards_parameters = start_value_double_richards_parameters
  )

  pred_full <- fallback_res$pred

  fb_params <- dmgfd_empty_model_params()
  fb_params[meta_fields] <- params[meta_fields]
  fb_params[names(fallback_res$params)] <- fallback_res$params
  fb_params$two_pulse_detected <- FALSE
  fb_params$fallback_used <- TRUE

  if (is.na(double_note) || !nzchar(double_note)) {
    fb_params$model_note <- paste0(
      "No convincing two-pulse pattern detected; ",
      fb_params$method_used,
      " fallback used."
    )
  } else {
    fb_params$model_note <- paste(
      double_note,
      paste0(
        "No convincing two-pulse pattern detected; ",
        fb_params$method_used,
        " fallback used."
      )
    )
  }

  fb_params <- dmgfd_add_overall_timing(
    params = fb_params,
    pred_full = pred_full,
    x_pred = x_pred,
    growth_fraction = growth_fraction,
    season_start_date = season_start_date
  )

  fb_params <- dmgfd_add_overall_rate_timing(
    params = fb_params,
    pred_full = pred_full,
    x_pred = x_pred,
    rate_threshold_fraction = rate_threshold_fraction,
    season_start_date = season_start_date
  )

  list(
    pred = pred_full,
    params = fb_params
  )
}

#' @title Truncation of dendrometer data
#'
#' @description
#' Truncates dendrometer data to a user-defined calendar-year and day-of-year
#' window.
#'
#' The first column of \code{df} is treated as the date-time column and is
#' renamed internally to \code{TIME}. The date-time column may be a
#' \code{Date}, \code{POSIXct}, or character vector in the format
#' \code{"yyyy-mm-dd HH:MM:SS"}. Character dates in the format
#' \code{"yyyy-mm-dd"} are also accepted.
#'
#' This version supports leap years correctly. For example, \code{DOY = 366}
#' is valid for leap years such as 2016, 2020, or 2024, but invalid for
#' non-leap years.
#'
#' @param df A data frame where the first column contains date or date-time
#'   values. The first column is renamed to \code{TIME} in the returned data.
#' @param CalYear Numeric value or numeric vector of length two giving the
#'   calendar year or start and end calendar years.
#' @param DOY Numeric value or numeric vector of length two giving the day of
#'   year or start and end day of year.
#'
#' @details
#' The truncation rules are:
#' \itemize{
#'   \item If \code{CalYear} has length one and \code{DOY} has length one,
#'     data from that single day are returned.
#'   \item If \code{CalYear} has length one and \code{DOY} has length two,
#'     data from the first DOY to the second DOY within the same year are
#'     returned.
#'   \item If \code{CalYear} has length two and \code{DOY} has length one,
#'     data from that DOY in the first year to the same DOY in the second year
#'     are returned.
#'   \item If \code{CalYear} has length two and \code{DOY} has length two,
#'     data from the first DOY of the first year to the second DOY of the
#'     second year are returned.
#' }
#'
#' The function uses real calendar dates internally rather than row positions.
#' Therefore, leap-year DOY 366 is handled correctly.
#'
#' @return
#' A tibble containing the truncated dendrometer data.
#'
#' @importFrom lubridate ymd_hms ymd
#' @importFrom dplyr rename filter select %>%
#' @importFrom tibble as_tibble
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(nepa)
#'
#' # Extract data from DOY 20 to 50 in 2017
#' trunc1 <- dendro.truncate(df = nepa, CalYear = 2017, DOY = c(20, 50))
#' head(trunc1, 10)
#'
#' # Leap-year example, if the data contain year 2020
#' # trunc2 <- dendro.truncate(df = df2020, CalYear = 2020, DOY = c(1, 366))
#' }
#'
#' @export
dendro.truncate <- function(df, CalYear, DOY) {
  .DATE <- NULL

  if (missing(df) || is.null(df)) {
    stop("'df' is missing or NULL.")
  }

  if (missing(CalYear) || is.null(CalYear) ||
      missing(DOY) || is.null(DOY)) {
    stop('Either "CalYear" and/or "DOY" are empty.')
  }

  cy <- CalYear
  doy <- DOY

  if (length(cy) < 1 || length(cy) > 2) {
    stop("The length of input 'CalYear' must be one or two.")
  }

  if (length(doy) < 1 || length(doy) > 2) {
    stop("The length of input 'DOY' must be one or two.")
  }

  if (!is.numeric(cy)) {
    stop("'CalYear' must be numeric.")
  }

  if (!is.numeric(doy)) {
    stop("'DOY' must be numeric.")
  }

  if (any(is.na(cy)) || any(is.na(doy))) {
    stop("'CalYear' and 'DOY' must not contain NA values.")
  }

  if (any(cy != floor(cy))) {
    stop("'CalYear' must contain whole years.")
  }

  if (any(doy != floor(doy))) {
    stop("'DOY' must contain whole day-of-year values.")
  }

  cy <- as.integer(cy)
  doy <- as.integer(doy)

  if (any(doy < 1 | doy > 366)) {
    stop("'DOY' values must be between 1 and 366.")
  }

  if (ncol(df) < 1) {
    stop("'df' must contain at least one column.")
  }

  ## Parse date-time column if needed
  if (!inherits(df[[1]], "Date") &&
      !inherits(df[[1]], "POSIXct") &&
      !inherits(df[[1]], "POSIXt")) {

    parsed_time <- lubridate::ymd_hms(df[[1]], quiet = TRUE)

    if (all(is.na(parsed_time))) {
      parsed_time <- lubridate::ymd(df[[1]], quiet = TRUE)
    }

    if (all(is.na(parsed_time))) {
      stop(
        "The first column could not be parsed as date-time. ",
        "Use format 'yyyy-mm-dd HH:MM:SS' or 'yyyy-mm-dd'."
      )
    }

    df[[1]] <- parsed_time
  }

  data <- tibble::as_tibble(df) %>%
    dplyr::rename(TIME = 1)

  data$.DATE <- as.Date(data$TIME)

  if (any(is.na(data$.DATE))) {
    stop("Some values in the TIME column could not be converted to dates.")
  }

  available_years <- unique(as.integer(format(data$.DATE, "%Y")))

  for (yy in cy) {
    if (!yy %in% available_years) {
      stop("Provided year ", yy, " does not exist in dataset.")
    }
  }

  make_date_from_year_doy <- function(year, doy_value) {
    candidate <- as.Date(sprintf("%04d-01-01", year)) + doy_value - 1L

    if (as.integer(format(candidate, "%Y")) != year) {
      stop(
        "DOY ", doy_value, " is not valid for year ", year, ". ",
        "This usually means DOY 366 was requested for a non-leap year."
      )
    }

    candidate
  }

  if (length(cy) == 1 && length(doy) == 1) {
    start_date <- make_date_from_year_doy(cy[1], doy[1])
    end_date <- start_date

  } else if (length(cy) == 1 && length(doy) == 2) {
    start_date <- make_date_from_year_doy(cy[1], doy[1])
    end_date <- make_date_from_year_doy(cy[1], doy[2])

  } else if (length(cy) == 2 && length(doy) == 1) {
    start_date <- make_date_from_year_doy(cy[1], doy[1])
    end_date <- make_date_from_year_doy(cy[2], doy[1])

  } else {
    start_date <- make_date_from_year_doy(cy[1], doy[1])
    end_date <- make_date_from_year_doy(cy[2], doy[2])
  }

  if (end_date < start_date) {
    stop(
      "The requested truncation window is invalid: end date is before start date."
    )
  }

  out <- data %>%
    dplyr::filter(.DATE >= start_date, .DATE <= end_date) %>%
    dplyr::select(-.DATE)

  if (nrow(out) == 0) {
    stop("No data available for the selected CalYear/DOY window.")
  }

  out
}

# =========================================================
# Helpers
# =========================================================

.dm_prepare_event_clim <- function(clim_df) {
  if (inherits(clim_df, "dm_clim") || inherits(clim_df, "subdaily_clim")) {
    dat <- tibble::as_tibble(clim_df)
  } else if (is.data.frame(clim_df) ||
             (is.character(clim_df) && length(clim_df) == 1 && file.exists(clim_df))) {
    dat <- tibble::as_tibble(read.climate(clim_df, verbose = FALSE))
  } else {
    stop(
      "'clim_df' must be one of: output of read.climate(), output of dm_subdaily_clim(), ",
      "a raw climate data frame, or a valid climate file path."
    )
  }

  if (!"TIME" %in% names(dat)) {
    stop("Climate input must contain a 'TIME' column.")
  }

  dat$TIME <- as.POSIXct(dat$TIME)
  dat <- dat[order(dat$TIME), , drop = FALSE]

  d <- diff(as.numeric(dat$TIME)) / 3600
  d <- d[is.finite(d) & d > 0]
  if (length(d) == 0) {
    stop("Could not infer temporal resolution from climate time column.")
  }

  res_h <- stats::median(d)
  if (res_h >= 24) {
    stop(
      "Event based climate extraction requires subdaily climate data. ",
      "Use climate data with timestamps finer than 24 hours."
    )
  }

  dat
}

.dm_check_numeric_vars <- function(dat, vars, arg_name) {
  allowed <- setdiff(names(dat)[vapply(dat, is.numeric, logical(1))], "TIME")
  if (is.null(vars)) return(character(0))
  miss <- setdiff(vars, allowed)
  if (length(miss) > 0) {
    stop(sprintf(
      "Unknown or non-numeric variable(s) in %s: %s",
      arg_name, paste(miss, collapse = ", ")
    ))
  }
  vars
}

.dm_match_event_row <- function(time_vec, target_time, rule = c("nearest", "previous", "next")) {
  rule <- match.arg(rule)

  if (is.na(target_time)) return(NA_integer_)

  if (rule == "nearest") {
    return(which.min(abs(as.numeric(time_vec) - as.numeric(target_time))))
  }

  if (rule == "previous") {
    idx <- which(time_vec <= target_time)
    return(if (length(idx) == 0) NA_integer_ else max(idx))
  }

  idx <- which(time_vec >= target_time)
  if (length(idx) == 0) NA_integer_ else min(idx)
}

.dm_subset_event_window <- function(dat, event_time, hours_back) {
  start_time <- event_time - as.difftime(hours_back, units = "hours")
  dat[dat$TIME > start_time & dat$TIME <= event_time, , drop = FALSE]
}

.dm_safe_agg_event <- function(x, FUN) {
  if (length(x) == 0 || all(is.na(x))) return(NA_real_)
  FUN(x, na.rm = TRUE)
}

.dm_filter_event_table <- function(dat, time_col = "event_time", Year = NULL, DOY = NULL) {
  tt <- as.POSIXct(dat[[time_col]])

  keep <- rep(TRUE, length(tt))

  if (!is.null(Year)) {
    keep <- keep & lubridate::year(tt) %in% Year
  }

  if (!is.null(DOY)) {
    if (length(DOY) != 2) stop("'DOY' must be a numeric vector of length 2.")
    keep <- keep & lubridate::yday(tt) >= min(DOY) & lubridate::yday(tt) <= max(DOY)
  }

  out <- dat[keep, , drop = FALSE]
  if (nrow(out) == 0) stop("No event data available for the selected Year or DOY window.")
  out
}

.dm_sig_stars <- function(p) {
  if (is.na(p)) return("NA")
  if (p < 0.0001) return("****")
  if (p < 0.001) return("***")
  if (p < 0.01) return("**")
  if (p < 0.05) return("*")
  "ns"
}


# =========================================================
# 1) Extract event times and climate before events
# =========================================================

#' @title Extract climate at and before phase events
#'
#' @description
#' Extracts climate at event times and summarizes climate in windows preceding
#' those events. Event times can be phase starts, phase ends, or
#' \code{Max.twd.time} for \code{phase.zg()} output.
#'
#' @param x Output of \code{phase.zg()} or \code{phase.sc()}.
#' @param clim_df Subdaily climate input. This can be a standardized climate
#'   object from \code{read.climate()}, a raw climate data frame, or a valid
#'   climate file path.
#' @param event One of \code{"phase_start"}, \code{"phase_end"}, or
#'   \code{"max_twd"}.
#' @param phase Phase selector. For \code{ZG_output}: \code{"all"},
#'   \code{"TWD"}, or \code{"GRO"}. For \code{SC_output}: \code{"all"},
#'   \code{"Shrinkage"}, \code{"Expansion"}, or \code{"Increment"}.
#' @param windows Numeric vector of antecedent windows in hours, e.g.
#'   \code{c(0, 1, 3, 6, 12, 24, 48)}. A value of \code{0} extracts climate at
#'   the event time.
#' @param mean_vars,max_vars,min_vars,sum_vars,median_vars Climate variables to
#'   summarize in antecedent windows.
#' @param instant_vars Climate variables to extract at the exact event time.
#'   If \code{NULL}, all selected variables are used.
#' @param instant_match Matching rule for event-time extraction:
#'   \code{"nearest"}, \code{"previous"}, or \code{"next"}.
#'
#' @return A tibble with one row per event and climate summaries in the
#'   requested antecedent windows.
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#'
#' evt_zg <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "phase_start",
#'   phase = "all",
#'   windows = c(0, 1, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' head(evt_zg)
#'
#' evt_maxtwd <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "max_twd",
#'   windows = c(0, 1, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' head(evt_maxtwd)
#' }
#' @export
dm_event_climate <- function(x,
                             clim_df,
                             event = c("phase_start", "phase_end", "max_twd"),
                             phase = "all",
                             windows = c(0, 1, 3, 6, 12, 24, 48),
                             mean_vars = NULL,
                             max_vars = NULL,
                             min_vars = NULL,
                             sum_vars = NULL,
                             median_vars = NULL,
                             instant_vars = NULL,
                             instant_match = c("nearest", "previous", "next")) {

  event <- match.arg(event)
  instant_match <- match.arg(instant_match)

  if (!inherits(x, "ZG_output") && !inherits(x, "SC_output")) {
    stop("'x' must be an object returned by phase.zg() or phase.sc().")
  }

  clim <- .dm_prepare_event_clim(clim_df)
  numeric_vars <- setdiff(names(clim)[vapply(clim, is.numeric, logical(1))], "TIME")

  if (all(c(
    is.null(mean_vars), is.null(max_vars), is.null(min_vars),
    is.null(sum_vars), is.null(median_vars), is.null(instant_vars)
  ))) {
    mean_vars <- numeric_vars
  }

  mean_vars   <- .dm_check_numeric_vars(clim, mean_vars, "mean_vars")
  max_vars    <- .dm_check_numeric_vars(clim, max_vars, "max_vars")
  min_vars    <- .dm_check_numeric_vars(clim, min_vars, "min_vars")
  sum_vars    <- .dm_check_numeric_vars(clim, sum_vars, "sum_vars")
  median_vars <- .dm_check_numeric_vars(clim, median_vars, "median_vars")

  if (is.null(instant_vars)) {
    instant_vars <- unique(c(mean_vars, max_vars, min_vars, sum_vars, median_vars))
  }
  instant_vars <- .dm_check_numeric_vars(clim, instant_vars, "instant_vars")

  windows <- sort(unique(as.numeric(windows)))
  if (any(!is.finite(windows)) || any(windows < 0)) {
    stop("'windows' must contain non negative numeric values.")
  }

  # -----------------------------------------------------
  # Build event table
  # -----------------------------------------------------
  if (inherits(x, "ZG_output")) {
    cyc <- tibble::as_tibble(x$ZG_cycle)
    cyc$Phase <- factor(cyc$Phases, levels = c(1, 2), labels = c("TWD", "GRO"))

    if (event == "phase_start") {
      evt <- cyc
      evt$event_time <- as.POSIXct(evt$Start)
      evt$event_type <- paste0(as.character(evt$Phase), "_start")
    } else if (event == "phase_end") {
      evt <- cyc
      evt$event_time <- as.POSIXct(evt$End)
      evt$event_type <- paste0(as.character(evt$Phase), "_end")
    } else {
      evt <- cyc[cyc$Phases == 1L & !is.na(cyc$Max.twd.time), , drop = FALSE]
      evt$event_time <- as.POSIXct(evt$Max.twd.time)
      evt$event_type <- "MaxTWD"
      evt$Phase <- factor("TWD", levels = c("TWD", "GRO"))
    }

    if (phase != "all") {
      evt <- evt[as.character(evt$Phase) == phase, , drop = FALSE]
    }

  } else {
    if (event == "max_twd") {
      stop("'max_twd' is only available for ZG_output.")
    }

    cyc <- tibble::as_tibble(x$SC_cycle)
    cyc$Phase <- factor(cyc$Phases, levels = c(1, 2, 3),
                        labels = c("Shrinkage", "Expansion", "Increment"))

    if (event == "phase_start") {
      evt <- cyc
      evt$event_time <- as.POSIXct(evt$Start)
      evt$event_type <- paste0(as.character(evt$Phase), "_start")
    } else {
      evt <- cyc
      evt$event_time <- as.POSIXct(evt$End)
      evt$event_type <- paste0(as.character(evt$Phase), "_end")
    }

    if (phase != "all") {
      evt <- evt[as.character(evt$Phase) == phase, , drop = FALSE]
    }
  }

  if (nrow(evt) == 0) {
    stop("No events available for the selected event phase combination.")
  }

  # -----------------------------------------------------
  # Extract climate for each event
  # -----------------------------------------------------
  res <- lapply(seq_len(nrow(evt)), function(i) {
    out <- evt[i, , drop = FALSE]

    et <- as.POSIXct(out$event_time)

    # exact climate at event
    idx0 <- .dm_match_event_row(clim$TIME, et, rule = instant_match)
    out$clim_match_time <- as.POSIXct(NA)
    out$clim_match_diff_min <- NA_real_

    if (!is.na(idx0)) {
      out$clim_match_time <- clim$TIME[idx0]
      out$clim_match_diff_min <- as.numeric(difftime(clim$TIME[idx0], et, units = "mins"))

      for (v in instant_vars) {
        out[[paste0(v, "_at_event")]] <- clim[[v]][idx0]
      }
    } else {
      for (v in instant_vars) {
        out[[paste0(v, "_at_event")]] <- NA_real_
      }
    }

    # antecedent windows
    for (w in windows[windows > 0]) {
      sub <- .dm_subset_event_window(clim, et, w)
      out[[paste0("n_clim_prev_", w, "h")]] <- nrow(sub)

      for (v in mean_vars) {
        out[[paste0(v, "_mean_prev_", w, "h")]] <- .dm_safe_agg_event(sub[[v]], mean)
      }
      for (v in max_vars) {
        out[[paste0(v, "_max_prev_", w, "h")]] <- .dm_safe_agg_event(sub[[v]], max)
      }
      for (v in min_vars) {
        out[[paste0(v, "_min_prev_", w, "h")]] <- .dm_safe_agg_event(sub[[v]], min)
      }
      for (v in sum_vars) {
        out[[paste0(v, "_sum_prev_", w, "h")]] <- .dm_safe_agg_event(sub[[v]], sum)
      }
      for (v in median_vars) {
        out[[paste0(v, "_median_prev_", w, "h")]] <- .dm_safe_agg_event(sub[[v]], stats::median)
      }
    }

    out
  })

  out <- dplyr::bind_rows(res)
  out
}


# =========================================================
# 2) Summarize event climate output
# =========================================================

#' @title Summarize event-based climate results
#'
#' @description
#' Summarizes a climate variable extracted by \code{dm_event_climate()} by
#' phase, event type, month, or month-of-year.
#'
#' @param event_data Output of \code{dm_event_climate()}.
#' @param climate_var Name of the climate-derived column to summarize.
#' @param group_var Grouping variable. One of \code{"Phase"} or \code{"event_type"}.
#' @param by One of \code{"none"}, \code{"month"}, \code{"month_of_year"}, or \code{"year"}.
#' @param Year Optional numeric year or vector of years for filtering.
#' @param DOY Optional numeric vector of length 2 for filtering.
#'
#' @return A tibble of summary statistics.
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' evt_zg <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "phase_start",
#'   windows = c(0, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' smry <- dm_event_climate_summary(
#'   evt_zg,
#'   climate_var = "VPD_mean_prev_6h",
#'   group_var = "Phase",
#'   by = "month_of_year"
#' )
#'
#' head(smry)
#' }
#'
#' @export
dm_event_climate_summary <- function(event_data,
                                     climate_var,
                                     group_var = c("Phase", "event_type"),
                                     by = c("none", "month", "month_of_year", "year"),
                                     Year = NULL,
                                     DOY = NULL) {
  .by <- n <- NULL

  group_var <- match.arg(group_var)
  by <- match.arg(by)

  dat <- tibble::as_tibble(event_data)
  if (!"event_time" %in% names(dat)) stop("'event_data' must contain 'event_time'.")
  if (!climate_var %in% names(dat)) stop(sprintf("'%s' not found.", climate_var))
  if (!group_var %in% names(dat)) stop(sprintf("'%s' not found.", group_var))

  dat <- .dm_filter_event_table(dat, time_col = "event_time", Year = Year, DOY = DOY)

  tt <- as.POSIXct(dat$event_time)
  dat$.by <- "all"

  if (by == "month") {
    dat$.by <- format(lubridate::floor_date(tt, "month"), "%Y-%m")
  } else if (by == "month_of_year") {
    dat$.by <- factor(
      as.character(lubridate::month(tt, label = TRUE, abbr = TRUE)),
      levels = month.abb
    )
  } else if (by == "year") {
    dat$.by <- factor(lubridate::year(tt))
  }

  dat <- dat[!is.na(dat[[climate_var]]) & !is.na(dat[[group_var]]), , drop = FALSE]

  dat %>%
    dplyr::group_by(.by, .data[[group_var]]) %>%
    dplyr::summarise(
      n = dplyr::n(),
      mean = mean(.data[[climate_var]], na.rm = TRUE),
      median = stats::median(.data[[climate_var]], na.rm = TRUE),
      sd = stats::sd(.data[[climate_var]], na.rm = TRUE),
      se = sd / sqrt(n),
      min = min(.data[[climate_var]], na.rm = TRUE),
      max = max(.data[[climate_var]], na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::rename(group = !!group_var, by_group = .by)
}


# =========================================================
# 3) Test climate differences between event groups
# =========================================================

#' @title Test differences in event-based climate between groups
#'
#' @description
#' Tests whether an event-based climate variable differs between groups such as
#' phases or event types.
#'
#' @param event_data Output of \code{dm_event_climate()}.
#' @param climate_var Name of the climate-derived column to test.
#' @param group_var Grouping variable. One of \code{"Phase"} or \code{"event_type"}.
#' @param by One of \code{"none"}, \code{"month"}, \code{"month_of_year"}, or \code{"year"}.
#' @param Year Optional numeric year or vector of years for filtering.
#' @param DOY Optional numeric vector of length 2 for filtering.
#' @param method One of \code{"auto"}, \code{"anova"}, \code{"kruskal"},
#'   \code{"t.test"}, or \code{"wilcox"}.
#'
#' @return A list with summary statistics, overall tests, and pairwise tests.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' evt_zg <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "phase_start",
#'   windows = c(0, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' tst <- dm_event_climate_test(
#'   evt_zg,
#'   climate_var = "VPD_mean_prev_6h",
#'   group_var = "Phase",
#'   by = "month_of_year",
#'   method = "auto"
#' )
#'
#' tst$summary
#' tst$overall_tests
#' head(tst$pairwise_tests)
#' }
#'
#' @export
dm_event_climate_test <- function(event_data,
                                  climate_var,
                                  group_var = c("Phase", "event_type"),
                                  by = c("none", "month", "month_of_year", "year"),
                                  Year = NULL,
                                  DOY = NULL,
                                  method = c("auto", "anova", "kruskal", "t.test", "wilcox")) {

  group_var <- match.arg(group_var)
  by <- match.arg(by)
  method <- match.arg(method)

  dat <- tibble::as_tibble(event_data)
  if (!"event_time" %in% names(dat)) stop("'event_data' must contain 'event_time'.")
  if (!climate_var %in% names(dat)) stop(sprintf("'%s' not found.", climate_var))
  if (!group_var %in% names(dat)) stop(sprintf("'%s' not found.", group_var))

  dat <- .dm_filter_event_table(dat, time_col = "event_time", Year = Year, DOY = DOY)

  tt <- as.POSIXct(dat$event_time)
  dat$.by <- "all"

  if (by == "month") {
    dat$.by <- format(lubridate::floor_date(tt, "month"), "%Y-%m")
  } else if (by == "month_of_year") {
    dat$.by <- factor(
      as.character(lubridate::month(tt, label = TRUE, abbr = TRUE)),
      levels = month.abb
    )
  } else if (by == "year") {
    dat$.by <- factor(lubridate::year(tt))
  }

  dat <- dat[!is.na(dat[[climate_var]]) & !is.na(dat[[group_var]]), , drop = FALSE]
  if (nrow(dat) == 0) stop("No complete observations available after filtering.")

  summary_tab <- dm_event_climate_summary(
    event_data = dat,
    climate_var = climate_var,
    group_var = group_var,
    by = "none"
  )

  run_one <- function(dd) {
    g <- dd[[group_var]]
    y <- dd[[climate_var]]
    ng <- length(unique(g))

    if (ng < 2) {
      return(list(
        overall = tibble::tibble(method = NA_character_, p.value = NA_real_),
        pairwise = NULL
      ))
    }

    meth <- method
    if (meth == "auto") {
      meth <- if (ng == 2) "wilcox" else "kruskal"
    }

    overall_p <- NA_real_
    overall_method <- meth
    pairwise <- NULL

    if (meth == "anova") {
      fit <- stats::lm(y ~ g)
      overall_p <- stats::anova(fit)[["Pr(>F)"]][1]
      if (ng > 2) {
        pw <- stats::pairwise.t.test(y, g, p.adjust.method = "BH")
        pairwise <- as.data.frame(as.table(pw$p.value))
        names(pairwise) <- c("group1", "group2", "p.value")
        pairwise <- pairwise[!is.na(pairwise$p.value), , drop = FALSE]
      }
    } else if (meth == "kruskal") {
      overall_p <- stats::kruskal.test(y ~ g)$p.value
      if (ng > 2) {
        pw <- stats::pairwise.wilcox.test(y, g, p.adjust.method = "BH", exact = FALSE)
        pairwise <- as.data.frame(as.table(pw$p.value))
        names(pairwise) <- c("group1", "group2", "p.value")
        pairwise <- pairwise[!is.na(pairwise$p.value), , drop = FALSE]
      }
    } else if (meth == "t.test") {
      if (ng != 2) stop("t.test requires exactly 2 groups.")
      overall_p <- stats::t.test(y ~ g)$p.value
      lev <- unique(as.character(g))
      pairwise <- tibble::tibble(group1 = lev[1], group2 = lev[2], p.value = overall_p)
    } else if (meth == "wilcox") {
      if (ng == 2) {
        overall_p <- stats::wilcox.test(y ~ g, exact = FALSE)$p.value
        lev <- unique(as.character(g))
        pairwise <- tibble::tibble(group1 = lev[1], group2 = lev[2], p.value = overall_p)
      } else {
        overall_p <- stats::kruskal.test(y ~ g)$p.value
        pw <- stats::pairwise.wilcox.test(y, g, p.adjust.method = "BH", exact = FALSE)
        pairwise <- as.data.frame(as.table(pw$p.value))
        names(pairwise) <- c("group1", "group2", "p.value")
        pairwise <- pairwise[!is.na(pairwise$p.value), , drop = FALSE]
      }
    }

    list(
      overall = tibble::tibble(method = overall_method, p.value = overall_p),
      pairwise = pairwise
    )
  }

  sp <- split(dat, dat$.by)
  test_list <- lapply(sp, run_one)

  overall_tests <- dplyr::bind_rows(
    lapply(names(test_list), function(nm) {
      cbind(by_group = nm, test_list[[nm]]$overall)
    })
  )

  pairwise_tests <- dplyr::bind_rows(
    lapply(names(test_list), function(nm) {
      pw <- test_list[[nm]]$pairwise
      if (is.null(pw)) return(NULL)
      cbind(by_group = nm, pw)
    })
  )

  list(
    summary = summary_tab,
    overall_tests = tibble::as_tibble(overall_tests),
    pairwise_tests = tibble::as_tibble(pairwise_tests),
    climate_var = climate_var,
    group_var = group_var,
    by = by
  )
}


# =========================================================
# 4) Box violin plots with optional test annotation
# =========================================================

#' @title Plot event-based climate distributions
#'
#' @description
#' Draws boxplots or violins of event-based climate variables across phases or
#' event types, with optional significance annotation.
#'
#' @param event_data Output of \code{dm_event_climate()}.
#' @param climate_var Name of the climate-derived column to plot.
#' @param group_var Grouping variable. One of \code{"Phase"} or \code{"event_type"}.
#' @param facet_by One of \code{"none"}, \code{"month"}, \code{"month_of_year"}, or \code{"year"}.
#' @param Year Optional numeric year or vector of years for filtering.
#' @param DOY Optional numeric vector of length 2 for filtering.
#' @param geom One of \code{"boxplot"}, \code{"violin"}, or \code{"both"}.
#' @param add_test Logical. If \code{TRUE}, adds per-panel test results.
#' @param test_method Passed to \code{dm_event_climate_test()}.
#' @param point_alpha Point transparency.
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' evt_zg <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "phase_start",
#'   windows = c(0, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' if (any(!is.na(evt_zg$VPD_mean_prev_6h))) {
#'   plot_event_climate_box(
#'     evt_zg,
#'     climate_var = "VPD_mean_prev_6h",
#'     group_var = "Phase",
#'     facet_by = "month_of_year",
#'     geom = "both",
#'     add_test = TRUE
#'   )
#' }
#' }
#'
#' @export
plot_event_climate_box <- function(event_data,
                                   climate_var,
                                   group_var = c("Phase", "event_type"),
                                   facet_by = c("none", "month", "month_of_year", "year"),
                                   Year = NULL,
                                   DOY = NULL,
                                   geom = c("boxplot", "violin", "both"),
                                   add_test = TRUE,
                                   test_method = c("auto", "anova", "kruskal", "t.test", "wilcox"),
                                   point_alpha = 0.5) {
  group <- label <- value <- NULL

  group_var <- match.arg(group_var)
  facet_by <- match.arg(facet_by)
  geom <- match.arg(geom)
  test_method <- match.arg(test_method)

  dat <- tibble::as_tibble(event_data)
  if (!"event_time" %in% names(dat)) stop("'event_data' must contain 'event_time'.")
  if (!climate_var %in% names(dat)) stop(sprintf("'%s' not found.", climate_var))
  if (!group_var %in% names(dat)) stop(sprintf("'%s' not found.", group_var))

  dat <- .dm_filter_event_table(dat, time_col = "event_time", Year = Year, DOY = DOY)

  tt <- as.POSIXct(dat$event_time)
  dat$group <- dat[[group_var]]
  dat$value <- dat[[climate_var]]
  dat$.facet <- "all"

  if (facet_by == "month") {
    dat$.facet <- format(lubridate::floor_date(tt, "month"), "%Y-%m")
  } else if (facet_by == "month_of_year") {
    dat$.facet <- factor(
      as.character(lubridate::month(tt, label = TRUE, abbr = TRUE)),
      levels = month.abb
    )
  } else if (facet_by == "year") {
    dat$.facet <- factor(lubridate::year(tt))
  }

  dat <- dat[!is.na(dat$value) & !is.na(dat$group), , drop = FALSE]
  if (nrow(dat) == 0) stop("No complete observations available after filtering.")

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = group, y = value))

  if (geom %in% c("violin", "both")) {
    p <- p + ggplot2::geom_violin(fill = "grey80", alpha = 0.8, trim = FALSE)
  }

  if (geom %in% c("boxplot", "both")) {
    p <- p + ggplot2::geom_boxplot(
      width = if (geom == "both") 0.18 else 0.6,
      outlier.shape = NA,
      fill = if (geom == "both") "white" else "grey80"
    )
  }

  p <- p +
    ggplot2::geom_point(
      position = ggplot2::position_jitter(width = 0.12, height = 0),
      size = 1.2,
      alpha = point_alpha,
      color = "grey25"
    ) +
    ggplot2::labs(
      x = group_var,
      y = climate_var
    ) +
    ggplot2::theme_minimal()

  if (facet_by != "none") {
    p <- p + ggplot2::facet_wrap(~ .facet, scales = "free_y")
  }

  if (isTRUE(add_test)) {
    tst <- dm_event_climate_test(
      event_data = dat,
      climate_var = climate_var,
      group_var = group_var,
      by = facet_by,
      method = test_method
    )

    ann <- tst$overall_tests
    ann$label <- paste0(
      ann$method, ": p = ", format.pval(ann$p.value, digits = 3, eps = 1e-4),
      " (", vapply(ann$p.value, .dm_sig_stars, character(1)), ")"
    )

    if (facet_by == "none") {
      ann$.facet <- "all"
    } else {
      ann$.facet <- ann$by_group
    }

    p <- p +
      ggplot2::geom_text(
        data = ann,
        ggplot2::aes(x = Inf, y = Inf, label = label),
        inherit.aes = FALSE,
        hjust = 1.05,
        vjust = 1.1,
        size = 3.3
      )
  }

  print(p)
  invisible(p)
}


# =========================================================
# 5) Scatterplot with R2, slope, p_value
# =========================================================

#' @title Plot event-based climate-response relationships
#'
#' @description
#' Draws a scatterplot between an event-based climate variable and a response
#' variable and annotates the fitted linear relationship with R², slope,
#' and p-value.
#'
#' @param event_data Output of \code{dm_event_climate()}.
#' @param climate_var Name of the climate-derived column.
#' @param response_var Name of the response variable.
#' @param group_var Optional grouping variable for color.
#' @param Year Optional numeric year or vector of years for filtering.
#' @param DOY Optional numeric vector of length 2 for filtering.
#' @param add_smooth Logical. If \code{TRUE}, adds linear fit line.
#' @param point_alpha Point transparency.
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' evt_maxtwd <- dm_event_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   event = "max_twd",
#'   windows = c(0, 3, 6, 12, 24),
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' if (all(c("VPD_mean_prev_6h", "max.twd") %in% names(evt_maxtwd)) &&
#'     any(is.finite(evt_maxtwd$VPD_mean_prev_6h)) &&
#'     any(is.finite(evt_maxtwd$max.twd))) {
#'   plot_event_climate_relation(
#'     evt_maxtwd,
#'     climate_var = "VPD_mean_prev_6h",
#'     response_var = "max.twd",
#'     group_var = "Phase"
#'   )
#' }
#' }
#'
#' @export
plot_event_climate_relation <- function(event_data,
                                        climate_var,
                                        response_var,
                                        group_var = NULL,
                                        Year = NULL,
                                        DOY = NULL,
                                        add_smooth = TRUE,
                                        point_alpha = 0.7) {

  dat <- tibble::as_tibble(event_data)
  if (!"event_time" %in% names(dat)) stop("'event_data' must contain 'event_time'.")
  if (!climate_var %in% names(dat)) stop(sprintf("'%s' not found.", climate_var))
  if (!response_var %in% names(dat)) stop(sprintf("'%s' not found.", response_var))

  dat <- .dm_filter_event_table(dat, time_col = "event_time", Year = Year, DOY = DOY)

  ok <- is.finite(dat[[climate_var]]) & is.finite(dat[[response_var]])
  dat <- dat[ok, , drop = FALSE]
  if (nrow(dat) < 3) stop("Not enough complete observations for regression.")

  fit <- stats::lm(dat[[response_var]] ~ dat[[climate_var]])
  sm <- summary(fit)

  slope <- unname(stats::coef(fit)[2])
  pval <- unname(sm$coefficients[2, 4])
  r2 <- unname(sm$r.squared)

  stat_label <- paste0(
    "R2 = ", formatC(r2, digits = 3, format = "f"),
    "   slope = ", formatC(slope, digits = 3, format = "f"),
    "   p = ", format.pval(pval, digits = 3, eps = 1e-4)
  )

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = .data[[climate_var]], y = .data[[response_var]]))

  if (!is.null(group_var) && group_var %in% names(dat)) {
    p <- p + ggplot2::geom_point(ggplot2::aes(color = .data[[group_var]]), alpha = point_alpha)
  } else {
    p <- p + ggplot2::geom_point(alpha = point_alpha)
  }

  if (isTRUE(add_smooth)) {
    p <- p + ggplot2::geom_smooth(method = "lm", se = TRUE, color = "black")
  }

  p <- p +
    ggplot2::labs(
      x = climate_var,
      y = response_var,
      subtitle = stat_label,
      color = group_var
    ) +
    ggplot2::theme_minimal()

  print(p)
  invisible(p)
}

#' Reconstruct or remove selected cycle components from a dm_wavelet object
#'
#' @description
#' Reconstructs a selected oscillatory component, or a selected period band,
#' from a \code{dm_wavelet} object using \code{WaveletComp::reconstruct()}.
#'
#' Requested periods are supplied in \strong{hours}. They are internally
#' converted to the native wavelet period units used when \code{dm_wavelet()}
#' was computed, then passed to \code{WaveletComp::reconstruct()}.
#'
#' The function supports two modes:
#' \itemize{
#'   \item \code{mode = "extract"} returns the selected cycle or band itself.
#'   \item \code{mode = "remove"} returns the original series with the selected
#'   cycle or band removed.
#' }
#'
#' @param x An object of class \code{"dm_wavelet"} returned by
#'   \code{dm_wavelet()}.
#' @param series Optional character vector of series names to reconstruct. If
#'   \code{NULL}, all available series are used.
#' @param mode One of \code{"extract"} or \code{"remove"}.
#' @param period_hours Optional numeric vector of exact periods, in hours, to
#'   reconstruct. If supplied, \code{lower_hours} and \code{upper_hours} are
#'   ignored.
#' @param lower_hours Optional lower period bound in hours for band
#'   reconstruction.
#' @param upper_hours Optional upper period bound in hours for band
#'   reconstruction.
#' @param lvl Minimum wavelet power level to include in the reconstruction.
#' @param only_sig Logical. If \code{TRUE}, use wavelet power significance in
#'   reconstruction.
#' @param siglvl Significance level used when \code{only_sig = TRUE}.
#' @param only_coi Logical. If \code{TRUE}, restrict reconstruction to the cone
#'   of influence.
#' @param only_ridge Logical. If \code{TRUE}, reconstruct from the power ridge
#'   only.
#' @param rescale Logical. Passed to \code{WaveletComp::reconstruct()}.
#' @param verbose Logical. If \code{TRUE}, prints a completion message.
#'
#' @return
#' An object of class \code{"dm_wavelet_reconstruct"} with elements:
#' \describe{
#'   \item{call}{Matched function call.}
#'   \item{series}{Selected series names.}
#'   \item{mode}{Reconstruction mode, either \code{"extract"} or \code{"remove"}.}
#'   \item{selection}{A list describing the requested selection in hours and in
#'   native wavelet units.}
#'   \item{results}{Named list of \code{WaveletComp::reconstruct()} outputs.}
#'   \item{reconstructed_long}{Tidy table with columns \code{TIME},
#'   \code{series}, \code{original}, \code{reconstructed}, \code{difference},
#'   and \code{filtered}.}
#'   \item{used_periods}{Per-series table of periods actually used in the
#'   reconstruction, in native units and in hours.}
#'   \item{filtered_wide}{Wide table with \code{TIME} in the first column and
#'   one filtered series column per selected tree/series.}
#'   \item{parent}{Minimal metadata copied from the parent \code{dm_wavelet}
#'   object.}
#' }
#'
#' @examples
#' \donttest{
#' wv <- dm_wavelet(
#'   x = gf_nepa17,
#'   TreeNum = 1:2,
#'   source = "raw",
#'   make_pval = TRUE,
#'   verbose = FALSE
#' )
#'
#' # extract circadian component
#' rec_extract <- dm_wavelet_reconstruct(
#'   wv,
#'   mode = "extract",
#'   lower_hours = 20,
#'   upper_hours = 28,
#'   only_sig = TRUE
#' )
#'
#' # remove circadian component
#' rec_remove <- dm_wavelet_reconstruct(
#'   wv,
#'   mode = "remove",
#'   lower_hours = 20,
#'   upper_hours = 28,
#'   only_sig = TRUE
#' )
#'
#' head(rec_extract$filtered_wide)
#' head(rec_remove$filtered_wide)
#' }
#'
#' @importFrom dplyr bind_rows %>%
#' @importFrom tidyr pivot_wider
#' @importFrom tibble tibble
#' @export
dm_wavelet_reconstruct <- function(x,
                                   series = NULL,
                                   mode = c("extract", "remove"),
                                   period_hours = NULL,
                                   lower_hours = NULL,
                                   upper_hours = NULL,
                                   lvl = 0,
                                   only_sig = TRUE,
                                   siglvl = 0.05,
                                   only_coi = FALSE,
                                   only_ridge = FALSE,
                                   rescale = FALSE,
                                   verbose = TRUE) {
  TIME <- NULL

  if (!inherits(x, "dm_wavelet")) {
    stop("'x' must be an object of class 'dm_wavelet'.")
  }

  if (!requireNamespace("WaveletComp", quietly = TRUE)) {
    stop("Package 'WaveletComp' is required for dm_wavelet_reconstruct().")
  }

  mode <- match.arg(mode)

  if (is.null(series)) {
    series <- x$series
  }

  miss <- setdiff(series, x$series)
  if (length(miss) > 0) {
    stop("These requested series were not found: ", paste(miss, collapse = ", "))
  }

  if (!is.null(period_hours) && (!is.null(lower_hours) || !is.null(upper_hours))) {
    warning("'period_hours' was supplied, so 'lower_hours' and 'upper_hours' are ignored.")
    lower_hours <- NULL
    upper_hours <- NULL
  }

  if (is.null(period_hours) && xor(is.null(lower_hours), is.null(upper_hours))) {
    stop("Provide both 'lower_hours' and 'upper_hours' for a band reconstruction.")
  }

  if (is.null(period_hours) && is.null(lower_hours) && is.null(upper_hours)) {
    stop("Provide either 'period_hours' or both 'lower_hours' and 'upper_hours'.")
  }

  if (!is.null(period_hours)) {
    if (!is.numeric(period_hours) || any(!is.finite(period_hours)) || any(period_hours <= 0)) {
      stop("'period_hours' must contain positive finite numbers.")
    }
  }

  if (!is.null(lower_hours)) {
    if (!is.numeric(lower_hours) || length(lower_hours) != 1 || !is.finite(lower_hours) || lower_hours <= 0) {
      stop("'lower_hours' must be one positive finite number.")
    }
    if (!is.numeric(upper_hours) || length(upper_hours) != 1 || !is.finite(upper_hours) || upper_hours <= 0) {
      stop("'upper_hours' must be one positive finite number.")
    }
    if (lower_hours >= upper_hours) {
      stop("'lower_hours' must be smaller than 'upper_hours'.")
    }
  }

  if (!is.numeric(siglvl) || length(siglvl) != 1 || is.na(siglvl) || siglvl <= 0 || siglvl >= 1) {
    stop("'siglvl' must be a single number between 0 and 1.")
  }

  cl <- match.call()

  dwr_hours_to_native <- function(hours, time_unit) {
    if (time_unit == "days") return(hours / 24)
    if (time_unit == "hours") return(hours)
    if (time_unit == "mins") return(hours * 60)
    if (time_unit == "secs") return(hours * 3600)
    hours
  }

  dwr_native_to_hours <- function(native, time_unit) {
    if (time_unit == "days") return(native * 24)
    if (time_unit == "hours") return(native)
    if (time_unit == "mins") return(native / 60)
    if (time_unit == "secs") return(native / 3600)
    native
  }

  results_list <- vector("list", length(series))
  names(results_list) <- series

  tidy_list <- vector("list", length(series))
  names(tidy_list) <- series

  used_periods_list <- vector("list", length(series))
  names(used_periods_list) <- series

  period_native <- if (!is.null(period_hours)) dwr_hours_to_native(period_hours, x$time_unit) else NULL
  lower_native <- if (!is.null(lower_hours)) dwr_hours_to_native(lower_hours, x$time_unit) else NULL
  upper_native <- if (!is.null(upper_hours)) dwr_hours_to_native(upper_hours, x$time_unit) else NULL

  for (i in seq_along(series)) {
    ss <- series[i]
    wt <- x$results[[ss]]$wavelet
    tt <- x$results[[ss]]$time

    rec_args <- list(
      WT = wt,
      lvl = lvl,
      only.coi = only_coi,
      only.sig = only_sig,
      siglvl = siglvl,
      only.ridge = only_ridge,
      rescale = rescale,
      plot.waves = FALSE,
      plot.rec = FALSE,
      verbose = FALSE
    )

    if (!is.null(period_native)) {
      rec_args$sel.period <- period_native
    } else {
      rec_args$sel.lower <- lower_native
      rec_args$sel.upper <- upper_native
    }

    rec <- do.call(WaveletComp::reconstruct, rec_args)

    sdat <- rec$series
    sname <- names(sdat)[!grepl("\\.trend$|\\.r$", names(sdat))][1]
    rname <- paste0(sname, ".r")

    if (!(sname %in% names(sdat)) || !(rname %in% names(sdat))) {
      stop(
        "Could not identify original and reconstructed series columns in ",
        "WaveletComp::reconstruct() output for series '", ss, "'."
      )
    }

    original <- as.numeric(sdat[[sname]])
    reconstructed <- as.numeric(sdat[[rname]])
    difference <- original - reconstructed

    filtered <- if (mode == "extract") reconstructed else difference

    tidy_list[[i]] <- tibble::tibble(
      TIME = tt,
      series = ss,
      original = original,
      reconstructed = reconstructed,
      difference = difference,
      filtered = filtered
    )

    used_idx <- rec$rnum.used
    if (length(used_idx) > 0 && !is.null(wt$Period)) {
      used_native <- as.numeric(wt$Period[used_idx])
      used_hours <- dwr_native_to_hours(used_native, x$time_unit)

      used_periods_list[[i]] <- tibble::tibble(
        series = ss,
        period_native = used_native,
        period_hours = used_hours
      )
    } else {
      used_periods_list[[i]] <- tibble::tibble(
        series = ss,
        period_native = numeric(0),
        period_hours = numeric(0)
      )
    }

    results_list[[i]] <- rec
  }

  reconstructed_long <- dplyr::bind_rows(tidy_list)

  filtered_wide <- reconstructed_long %>%
    dplyr::select(TIME, series, filtered) %>%
    tidyr::pivot_wider(
      names_from = series,
      values_from = filtered
    ) %>%
    as.data.frame()

  out <- list(
    call = cl,
    series = series,
    mode = mode,
    selection = list(
      period_hours = period_hours,
      lower_hours = lower_hours,
      upper_hours = upper_hours,
      period_native = period_native,
      lower_native = lower_native,
      upper_native = upper_native,
      lvl = lvl,
      only_sig = only_sig,
      siglvl = siglvl,
      only_coi = only_coi,
      only_ridge = only_ridge,
      rescale = rescale
    ),
    results = results_list,
    reconstructed_long = reconstructed_long,
    used_periods = dplyr::bind_rows(used_periods_list),
    filtered_wide = filtered_wide,
    parent = list(
      input_type = x$input_type,
      source = x$source,
      time_unit = x$time_unit,
      dt = x$dt,
      dt_hours = x$dt_hours
    )
  )

  class(out) <- "dm_wavelet_reconstruct"

  if (isTRUE(verbose)) {
    msg <- if (!is.null(period_hours)) {
      paste0("selected period(s): ", paste(round(period_hours, 4), collapse = ", "), " hour(s)")
    } else {
      paste0("selected band: ", round(lower_hours, 4), " to ", round(upper_hours, 4), " hours")
    }

    action_msg <- if (mode == "extract") "extracted" else "removed"

    message(
      "dm_wavelet_reconstruct completed for ", length(series),
      " series (", msg, "; mode = '", mode, "', selected component ", action_msg, ")."
    )
  }

  out
}

############### plotting #####################################################
#' Plot method for dm_wavelet_reconstruct objects
#'
#' @description
#' Plots reconstructed or filtered cycle components extracted by
#' \code{dm_wavelet_reconstruct()}.
#'
#' @param x An object of class \code{"dm_wavelet_reconstruct"}.
#' @param y Unused.
#' @param series Optional character vector of series names to plot. If
#'   \code{NULL}, all available reconstructed series are used.
#' @param type One of:
#'   \describe{
#'     \item{`"compare"`}{Original and reconstructed series together.}
#'     \item{`"reconstructed"`}{Reconstructed component only.}
#'     \item{`"difference"`}{Original minus reconstructed.}
#'     \item{`"filtered"`}{Directly plot the returned filtered series. This is
#'       the extracted component for \code{mode = "extract"} and the
#'       component-removed series for \code{mode = "remove"}.}
#'   }
#' @param facet Logical. If \code{TRUE}, facet by series.
#' @param legend_position Legend position passed to ggplot2.
#' @param line_width Line width.
#' @param alpha Alpha transparency for original series in compare plots.
#' @param main Optional title.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return A \code{ggplot2} object.
#'
#' @method plot dm_wavelet_reconstruct
#' @importFrom ggplot2 ggplot aes geom_line facet_wrap theme_bw theme
#'   element_text labs
#' @export
plot.dm_wavelet_reconstruct <- function(x,
                                        y = NULL,
                                        series = NULL,
                                        type = c("compare", "reconstructed", "difference", "filtered"),
                                        facet = TRUE,
                                        legend_position = "right",
                                        line_width = 0.8,
                                        alpha = 0.7,
                                        main = NULL,
                                        ...) {
  TIME <- value <- signal <- reconstructed <- difference <- filtered <- NULL

  if (!inherits(x, "dm_wavelet_reconstruct")) {
    stop("'x' must be an object of class 'dm_wavelet_reconstruct'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_wavelet_reconstruct().")
  }

  type <- match.arg(type)

  dat <- x$reconstructed_long

  if (!is.null(series)) {
    miss <- setdiff(series, unique(dat$series))
    if (length(miss) > 0) {
      stop("These requested series were not found: ", paste(miss, collapse = ", "))
    }
    dat <- dat[dat$series %in% series, , drop = FALSE]
  }

  if (nrow(dat) == 0) {
    stop("No rows remain after filtering.")
  }

  mode_label <- if (!is.null(x$mode) && x$mode == "remove") {
    "Filtered series (selected component removed)"
  } else {
    "Filtered series (selected component extracted)"
  }

  if (type == "compare") {
    long_dat <- tibble::tibble(
      TIME = c(dat$TIME, dat$TIME),
      series = c(dat$series, dat$series),
      signal = c(rep("original", nrow(dat)), rep("reconstructed", nrow(dat))),
      value = c(dat$original, dat$reconstructed)
    )

    p <- ggplot2::ggplot(
      long_dat,
      ggplot2::aes(x = TIME, y = value, colour = signal, group = signal)
    ) +
      ggplot2::geom_line(
        data = long_dat[long_dat$signal == "original", , drop = FALSE],
        linewidth = line_width,
        alpha = alpha
      ) +
      ggplot2::geom_line(
        data = long_dat[long_dat$signal == "reconstructed", , drop = FALSE],
        linewidth = line_width
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Value",
        colour = "Series type",
        title = if (is.null(main)) "Original and reconstructed wavelet component" else main
      )

    if (isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  if (type == "reconstructed") {
    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = TIME, y = reconstructed, colour = series)
    ) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(facet)) "none" else legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Reconstructed value",
        colour = "Series",
        title = if (is.null(main)) "Reconstructed wavelet component" else main
      )

    if (isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  if (type == "difference") {
    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = TIME, y = difference, colour = series)
    ) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(facet)) "none" else legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Original - reconstructed",
        colour = "Series",
        title = if (is.null(main)) "Difference between original and reconstructed series" else main
      )

    if (isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  if (type == "filtered") {
    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = TIME, y = filtered, colour = series)
    ) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (isTRUE(facet)) "none" else legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = if (!is.null(x$mode) && x$mode == "remove") {
          "Filtered value (component removed)"
        } else {
          "Filtered value (component extracted)"
        },
        colour = "Series",
        title = if (is.null(main)) mode_label else main
      )

    if (isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ series"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  stop("Unknown plot type.")
}

#################### summary extaction #############################
#' Summarize a dm_wavelet_reconstruct object
#'
#' @description
#' Summarizes the output of \code{dm_wavelet_reconstruct()}.
#'
#' For each reconstructed series, the summary reports:
#' \itemize{
#'   \item number of observations,
#'   \item variance of the original series,
#'   \item variance of the reconstructed component,
#'   \item variance of the remaining component,
#'   \item variance of the returned filtered series,
#'   \item proportion of original variance represented by reconstructed and filtered series,
#'   \item correlation and \eqn{R^2} between original and reconstructed,
#'   \item number and range of periods used in reconstruction.
#' }
#'
#' @param object An object of class \code{"dm_wavelet_reconstruct"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return
#' An object of class \code{"summary.dm_wavelet_reconstruct"} with elements:
#' \describe{
#'   \item{overview}{One-row summary of the reconstruction object.}
#'   \item{series_summary}{Per-series summary table.}
#'   \item{used_periods}{The table of periods actually used in the reconstruction.}
#'   \item{selection}{The selection settings used for reconstruction.}
#' }
#'
#' @method summary dm_wavelet_reconstruct
#' @export
summary.dm_wavelet_reconstruct <- function(object, ...) {

  if (!inherits(object, "dm_wavelet_reconstruct")) {
    stop("'object' must be an object of class 'dm_wavelet_reconstruct'.")
  }

  dat <- object$reconstructed_long
  used <- object$used_periods

  if (is.null(dat) || nrow(dat) == 0) {
    stop("The object contains no reconstructed data.")
  }

  safe_var <- function(z) {
    z <- z[is.finite(z)]
    if (length(z) < 2) return(NA_real_)
    stats::var(z)
  }

  safe_cor <- function(x, y) {
    ok <- is.finite(x) & is.finite(y)
    if (sum(ok) < 2) return(NA_real_)
    stats::cor(x[ok], y[ok])
  }

  ser_names <- unique(dat$series)

  series_rows <- lapply(ser_names, function(ss) {
    dss <- dat[dat$series == ss, , drop = FALSE]
    uss <- used[used$series == ss, , drop = FALSE]

    v_orig <- safe_var(dss$original)
    v_rec  <- safe_var(dss$reconstructed)
    v_rem  <- safe_var(dss$difference)
    v_filt <- safe_var(dss$filtered)

    cor_or_rec <- safe_cor(dss$original, dss$reconstructed)
    r2_or_rec <- if (is.finite(cor_or_rec)) cor_or_rec^2 else NA_real_

    tibble::tibble(
      series = ss,
      n_obs = sum(is.finite(dss$original)),
      variance_original = v_orig,
      variance_reconstructed = v_rec,
      variance_remaining = v_rem,
      variance_filtered = v_filt,
      prop_variance_reconstructed = if (is.finite(v_orig) && v_orig > 0) v_rec / v_orig else NA_real_,
      prop_variance_remaining = if (is.finite(v_orig) && v_orig > 0) v_rem / v_orig else NA_real_,
      prop_variance_filtered = if (is.finite(v_orig) && v_orig > 0) v_filt / v_orig else NA_real_,
      correlation_original_reconstructed = cor_or_rec,
      r_squared_original_reconstructed = r2_or_rec,
      n_used_periods = nrow(uss),
      min_used_period_hours = if (nrow(uss) > 0) min(uss$period_hours, na.rm = TRUE) else NA_real_,
      max_used_period_hours = if (nrow(uss) > 0) max(uss$period_hours, na.rm = TRUE) else NA_real_,
      mean_used_period_hours = if (nrow(uss) > 0) mean(uss$period_hours, na.rm = TRUE) else NA_real_
    )
  })

  series_summary <- dplyr::bind_rows(series_rows)

  overview <- tibble::tibble(
    mode = object$mode,
    n_series = length(unique(dat$series)),
    input_type = object$parent$input_type,
    source = object$parent$source,
    time_unit = object$parent$time_unit,
    dt = object$parent$dt,
    dt_hours = object$parent$dt_hours
  )

  out <- list(
    overview = overview,
    series_summary = series_summary,
    used_periods = used,
    selection = object$selection
  )

  class(out) <- "summary.dm_wavelet_reconstruct"
  out
}


#' Print method for summary.dm_wavelet_reconstruct
#'
#' @param x An object of class \code{"summary.dm_wavelet_reconstruct"}.
#' @param digits Number of digits for rounded numeric printing.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#'
#' @method print summary.dm_wavelet_reconstruct
#' @export
print.summary.dm_wavelet_reconstruct <- function(x, digits = 4, ...) {

  if (!inherits(x, "summary.dm_wavelet_reconstruct")) {
    stop("'x' must be an object of class 'summary.dm_wavelet_reconstruct'.")
  }

  cat("dm_wavelet_reconstruct summary\n")
  cat("------------------------------\n")
  print(x$overview)

  cat("\nSelection settings:\n")
  print(x$selection)

  cat("\nPer-series summary:\n")
  sx <- x$series_summary
  num_cols <- vapply(sx, is.numeric, logical(1))
  sx[num_cols] <- lapply(sx[num_cols], function(z) round(z, digits = digits))
  print(sx)

  if (!is.null(x$used_periods) && nrow(x$used_periods) > 0) {
    cat("\nUsed periods (hours):\n")
    up <- x$used_periods
    num_cols2 <- vapply(up, is.numeric, logical(1))
    up[num_cols2] <- lapply(up[num_cols2], function(z) round(z, digits = digits))
    print(up)
  }

  cat("\nNote:\n")
  cat("Variance ratios are reported relative to the original series.\n")
  cat("Wavelet reconstruction is not strictly orthogonal, so reconstructed and remaining variances do not necessarily sum to the original variance.\n")

  invisible(x)
}

#' @title Join climate summaries to dendrometer phase windows
#'
#' @description
#' Summarizes climate conditions within each phase window of
#' \code{phase.zg()} or \code{phase.sc()} output and appends those summaries to
#' \code{ZG_cycle} or \code{SC_cycle}.
#'
#' The climate input can be:
#' \itemize{
#'   \item the output of \code{read.climate()}
#'   \item a raw climate data frame
#'   \item a valid climate file path readable by \code{read.climate()}
#' }
#'
#' Climate variables are summarized over each phase interval defined by the
#' \code{Start} and \code{End} columns of the phase-cycle table.
#'
#' @param x Object returned by \code{phase.zg()} or \code{phase.sc()}.
#' @param clim_df Climate input. This can be a standardized climate object
#'   returned by \code{read.climate()}, a raw climate data frame, or a valid
#'   climate file path.
#' @param mean_vars Character vector of climate variables to summarize by mean
#'   within each phase window.
#' @param min_vars Character vector of climate variables to summarize by minimum
#'   within each phase window.
#' @param max_vars Character vector of climate variables to summarize by maximum
#'   within each phase window.
#' @param sum_vars Character vector of climate variables to summarize by sum
#'   within each phase window.
#' @param median_vars Character vector of climate variables to summarize by median
#'   within each phase window.
#' @param suffix Character suffix appended to the generated climate summary
#'   columns. Default is \code{"_phase"}.
#'
#' @return
#' The same phase object with climate summaries appended to \code{ZG_cycle} or
#' \code{SC_cycle}. The returned object has class
#' \code{c("ZG_output_clim", "ZG_output", ...)} or
#' \code{c("SC_output_clim", "SC_output", ...)}.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#'
#' zg_clim <- dm_join_phase_clim(
#'   zg,
#'   ktm_clim_hourly,
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' head(zg_clim$ZG_cycle)
#' }
#'
#' @importFrom dplyr bind_cols
#' @importFrom tibble as_tibble
#' @importFrom stats median
#' @export
dm_join_phase_clim <- function(x,
                               clim_df,
                               mean_vars = NULL,
                               min_vars = NULL,
                               max_vars = NULL,
                               sum_vars = NULL,
                               median_vars = NULL,
                               suffix = "_phase") {

  if (!inherits(x, "ZG_output") && !inherits(x, "SC_output")) {
    stop("'x' must be an object returned by phase.zg() or phase.sc().")
  }

  # standardize climate input
  clim <- if (inherits(clim_df, "dm_clim")) {
    tibble::as_tibble(clim_df)
  } else if (is.data.frame(clim_df) || (is.character(clim_df) && length(clim_df) == 1 && file.exists(clim_df))) {
    tibble::as_tibble(read.climate(clim_df, verbose = FALSE))
  } else {
    stop(
      "'clim_df' must be one of: output of read.climate(), ",
      "a raw climate data frame, or a valid climate file path."
    )
  }

  if (!"TIME" %in% names(clim)) {
    stop("Climate input must contain a 'TIME' column after standardization.")
  }

  clim$TIME <- as.POSIXct(clim$TIME)

  numeric_vars <- setdiff(names(clim)[vapply(clim, is.numeric, logical(1))], "TIME")

  if (all(c(
    is.null(mean_vars), is.null(min_vars), is.null(max_vars),
    is.null(sum_vars), is.null(median_vars)
  ))) {
    mean_vars <- numeric_vars
  }

  check_vars <- function(vars, allowed, arg_name) {
    if (is.null(vars)) return(character(0))
    miss <- setdiff(vars, allowed)
    if (length(miss) > 0) {
      stop(sprintf(
        "Unknown or non-numeric variable(s) in %s: %s",
        arg_name, paste(miss, collapse = ", ")
      ))
    }
    vars
  }

  mean_vars   <- check_vars(mean_vars, numeric_vars, "mean_vars")
  min_vars    <- check_vars(min_vars, numeric_vars, "min_vars")
  max_vars    <- check_vars(max_vars, numeric_vars, "max_vars")
  sum_vars    <- check_vars(sum_vars, numeric_vars, "sum_vars")
  median_vars <- check_vars(median_vars, numeric_vars, "median_vars")

  cycle_name <- if (inherits(x, "ZG_output")) "ZG_cycle" else "SC_cycle"
  cyc <- tibble::as_tibble(x[[cycle_name]])

  if (!all(c("Start", "End") %in% names(cyc))) {
    stop(sprintf("'%s' must contain 'Start' and 'End' columns.", cycle_name))
  }

  cyc$Start <- as.POSIXct(cyc$Start)
  cyc$End   <- as.POSIXct(cyc$End)

  safe_agg <- function(z, FUN) {
    if (length(z) == 0 || all(is.na(z))) return(NA_real_)
    FUN(z, na.rm = TRUE)
  }

  summarise_window <- function(start_time, end_time) {
    idx <- clim$TIME >= start_time & clim$TIME <= end_time
    sub <- clim[idx, , drop = FALSE]

    out <- list(n_clim = sum(idx, na.rm = TRUE))

    for (v in mean_vars) {
      out[[paste0(v, "_mean", suffix)]] <- safe_agg(sub[[v]], mean)
    }
    for (v in min_vars) {
      out[[paste0(v, "_min", suffix)]] <- safe_agg(sub[[v]], min)
    }
    for (v in max_vars) {
      out[[paste0(v, "_max", suffix)]] <- safe_agg(sub[[v]], max)
    }
    for (v in sum_vars) {
      out[[paste0(v, "_sum", suffix)]] <- safe_agg(sub[[v]], sum)
    }
    for (v in median_vars) {
      out[[paste0(v, "_median", suffix)]] <- safe_agg(sub[[v]], stats::median)
    }

    tibble::as_tibble(out)
  }

  clim_stats <- dplyr::bind_rows(
    lapply(seq_len(nrow(cyc)), function(i) {
      summarise_window(cyc$Start[i], cyc$End[i])
    })
  )

  out <- x
  out[[cycle_name]] <- dplyr::bind_cols(cyc, clim_stats)

  if (inherits(x, "ZG_output")) {
    class(out) <- unique(c("ZG_output_clim", class(x)))
  } else {
    class(out) <- unique(c("SC_output_clim", class(x)))
  }

  out
}

#' Plot dendrometer growth-fit results
#'
#' @description
#' Creates \pkg{ggplot2}-based plots for objects returned by
#' [dm.growth.fit()] and [dm.growth.fit.double()].
#'
#' The plotting method supports multiple views of fitted dendrometer growth
#' curves, including observed versus fitted trajectories, residuals, timing
#' summaries, overlays of multiple curves, and distributions of fitted model
#' parameters.
#'
#' The x-axis can be displayed as vegetation-season day, calendar day-of-year
#' (DOY), or actual date, depending on the selected \code{x_axis} argument and
#' the type of plot.
#'
#' When \code{x_axis = "date"} and faceting separates individual years or
#' series-year combinations, each facet receives its own x-axis range. This
#' avoids plotting one yearly fitted curve inside the full dendrometer time
#' window when several seasons are present in the input object.
#'
#' @param x An object of class \code{"dm_growth_fit"} returned by
#'   [dm.growth.fit()] or [dm.growth.fit.double()].
#' @param type Character string specifying the plot type. One of
#'   \code{"fit"}, \code{"season"}, \code{"residuals"}, \code{"timing"},
#'   \code{"overlay"}, or \code{"parameters"}.
#' @param series Optional filter selecting one or more dendrometer series to
#'   plot. May be a character vector of series names. Default is \code{NULL},
#'   meaning all available series are used.
#' @param fit_id Optional filter selecting one or more fit identifiers to plot.
#'   May be a character or numeric vector. Numeric values are coerced internally
#'   to character. Default is \code{NULL}, meaning all fits are used.
#' @param facet_by Character string controlling faceting layout. One of
#'   \code{"default"}, \code{"tree"}, \code{"year"}, or \code{"none"}.
#' @param ncol Optional integer giving the number of columns in faceted plots.
#'   Passed to [ggplot2::facet_wrap()].
#' @param normalize Logical. If \code{TRUE}, observed and fitted values are
#'   divided by the maximum observed or fitted value within each curve.
#' @param x_axis Character string controlling the x-axis representation. One of
#'   \code{"default"}, \code{"season_day"}, \code{"doy"}, or \code{"date"}.
#' @param observed_source Character string controlling which observed daily
#'   series is plotted against the fitted curve. One of \code{"processed"} or
#'   \code{"original_daily"}.
#' @param show_observed Logical. If \code{TRUE}, observed values are shown in
#'   plot types where observed data are relevant.
#' @param show_fitted Logical. If \code{TRUE}, fitted values are shown in plot
#'   types where fitted curves are relevant.
#' @param show_timing Logical. If \code{TRUE}, timing markers are added to
#'   \code{"fit"} and \code{"season"} plots whenever timing information is
#'   available in \code{x$fit_statistics}.
#' @param point_alpha Numeric alpha level used for observed points.
#' @param line_width Numeric line width used for fitted curves.
#' @param legend_position Character string specifying legend position, passed to
#'   [ggplot2::theme()].
#' @param ... Further arguments passed to or from other methods.
#'
#' @details
#' The plotting method returns a \code{ggplot} object. The returned plot can be
#' further modified using normal \pkg{ggplot2} syntax.
#'
#' Timing markers are taken from the \code{fit_statistics} table inside the
#' \code{"dm_growth_fit"} object:
#' \itemize{
#'   \item \code{growth_start_*} and \code{growth_end_*} represent growing-season
#'     timing based on cumulative fitted growth.
#'   \item \code{rate_start_*} and \code{rate_end_*} represent active-growth
#'     timing based on the fitted growth-rate curve.
#'   \item For double-growth fits, pulse-specific timing such as
#'     \code{pulse1_start_*}, \code{pulse2_start_*}, and \code{separator_*} is
#'     used in \code{type = "timing"} when available.
#' }
#'
#' For yearly fits in southern hemisphere or cross-year custom seasons,
#' calendar timing variables such as \code{growth_start_day} and
#' \code{rate_start_day} are interpreted as true calendar DOY, while
#' \code{*_season_day} variables represent day counts relative to vegetation
#' season start.
#'
#' In pooled fits, date-based and calendar-DOY timing fields may be unavailable,
#' because pooled fits are not anchored to a single season start date.
#'
#' @return A \code{ggplot} object.
#'
#' @seealso
#' [dm.growth.fit()], [dm.growth.fit.double()], [summary.dm_growth_fit()]
#'
#' @examples
#' \donttest{
#' # fit <- dm.growth.fit(...)
#' # plot(fit, type = "fit")
#' # plot(fit, type = "fit", facet_by = "year", x_axis = "date")
#' # plot(fit, type = "fit", facet_by = "year", x_axis = "season_day")
#' # plot(fit, type = "residuals", facet_by = "year")
#' # plot(fit, type = "timing")
#' # plot(fit, type = "parameters")
#' }
#'
#' @method plot dm_growth_fit
#' @importFrom ggplot2 ggplot aes theme_bw theme labs facet_wrap geom_point
#'   geom_line geom_hline geom_vline geom_text geom_segment geom_boxplot
#'   geom_jitter element_text
#' @importFrom dplyr %>% group_by mutate if_else ungroup filter select all_of
#'   left_join transmute bind_rows
#' @importFrom tidyr pivot_longer
#' @importFrom tibble as_tibble
#' @importFrom stats as.formula na.omit
#' @importFrom rlang .data
#' @export
plot.dm_growth_fit <- function(x,
                               type = c("fit", "season", "residuals", "timing", "overlay", "parameters"),
                               series = NULL,
                               fit_id = NULL,
                               facet_by = c("default", "tree", "year", "none"),
                               ncol = NULL,
                               normalize = FALSE,
                               x_axis = c("default", "season_day", "doy", "date"),
                               observed_source = c("processed", "original_daily"),
                               show_observed = TRUE,
                               show_fitted = TRUE,
                               show_timing = TRUE,
                               point_alpha = 0.7,
                               line_width = 0.8,
                               legend_position = "right",
                               ...) {

  type <- match.arg(type)
  facet_by <- match.arg(facet_by)
  x_axis <- match.arg(x_axis)
  observed_source <- match.arg(observed_source)

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_growth_fit().")
  }

  if (!inherits(x, "dm_growth_fit")) {
    stop("Input must be an object of class 'dm_growth_fit'.")
  }

  if (type %in% c("fit", "season", "residuals", "overlay")) {
    dat <- dmgp_tidy_fit_data(x, observed_source = observed_source)

    if (!is.null(series)) {
      dat <- dat[dat$series %in% series, , drop = FALSE]
    }

    if (!is.null(fit_id)) {
      dat <- dat[dat$fit_id %in% as.character(fit_id), , drop = FALSE]
    }

    if (nrow(dat) == 0) {
      stop("No rows remain after filtering by series and/or fit_id.")
    }

    cfg <- dmgp_prepare_curve_groups(dat, facet_by = facet_by)
    dat <- cfg$data

    x_cfg <- dmgp_resolve_x_axis(type = type, x_axis = x_axis)
    x_var <- x_cfg$var
    x_lab <- x_cfg$label

    facet_scales <- dmgp_resolve_facet_scales(
      x_var = x_var,
      facet_by = facet_by
    )

    if (!x_var %in% names(dat)) {
      stop("Column '", x_var, "' not found in plotting data.")
    }

    if (isTRUE(normalize)) {
      dat <- dat %>%
        dplyr::group_by(.data$group_id) %>%
        dplyr::mutate(
          .norm = suppressWarnings(max(c(.data$observed, .data$fitted), na.rm = TRUE)),
          observed_plot = dplyr::if_else(
            is.finite(.data$.norm) & .data$.norm > 0,
            .data$observed / .data$.norm,
            NA_real_
          ),
          fitted_plot = dplyr::if_else(
            is.finite(.data$.norm) & .data$.norm > 0,
            .data$fitted / .data$.norm,
            NA_real_
          )
        ) %>%
        dplyr::ungroup()

      y_lab <- "Normalized growth"

    } else {
      dat$observed_plot <- dat$observed
      dat$fitted_plot <- dat$fitted
      y_lab <- "Growth"
    }

    plot_title_obs <- if (observed_source == "original_daily") {
      "Original daily and fitted growth"
    } else {
      "Observed and fitted growth"
    }

    if (type == "fit") {
      p <- ggplot2::ggplot() +
        ggplot2::theme_bw() +
        ggplot2::theme(
          legend.position = if (isTRUE(cfg$use_colour)) legend_position else "none"
        )

      if (isTRUE(show_observed)) {
        p <- p +
          ggplot2::geom_point(
            data = dat[is.finite(dat$observed_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$observed_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            alpha = point_alpha,
            size = 1.4
          )
      }

      if (isTRUE(show_fitted)) {
        p <- p +
          ggplot2::geom_line(
            data = dat[is.finite(dat$fitted_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$fitted_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            linewidth = line_width
          )
      }

      p <- p +
        ggplot2::labs(
          x = x_lab,
          y = y_lab,
          colour = cfg$legend_title,
          title = plot_title_obs
        )

      if (isTRUE(cfg$facet_enabled)) {
        p <- p +
          ggplot2::facet_wrap(
            stats::as.formula("~ facet_var"),
            scales = facet_scales,
            ncol = ncol
          )
      }

      if (isTRUE(show_timing) && !is.null(x$fit_statistics)) {
        timing_marks <- dmgp_prepare_boundary_marks(
          fit_statistics = x$fit_statistics,
          facet_by = facet_by,
          x_mode = x_cfg$timing_mode,
          series = series,
          fit_id = fit_id
        )

        if (nrow(timing_marks$cumulative) > 0) {
          p <- p +
            ggplot2::geom_vline(
              data = timing_marks$cumulative,
              mapping = ggplot2::aes(
                xintercept = .data$x,
                linetype = .data$boundary,
                colour = .data$curve_group
              ),
              alpha = 0.5
            )
        }

        if (nrow(timing_marks$rate) > 0) {
          p <- p +
            ggplot2::geom_text(
              data = timing_marks$rate,
              mapping = ggplot2::aes(
                x = .data$x,
                y = .data$y,
                label = .data$triangle,
                colour = .data$curve_group
              ),
              inherit.aes = FALSE,
              show.legend = FALSE,
              size = 4
            )
        }
      }

      return(p)
    }

    if (type == "season") {
      p <- ggplot2::ggplot() +
        ggplot2::theme_bw() +
        ggplot2::theme(
          legend.position = if (isTRUE(cfg$use_colour)) legend_position else "none"
        )

      if (isTRUE(show_observed)) {
        p <- p +
          ggplot2::geom_point(
            data = dat[is.finite(dat$observed_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$observed_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            alpha = point_alpha,
            size = 1.4
          )
      }

      if (isTRUE(show_fitted)) {
        p <- p +
          ggplot2::geom_line(
            data = dat[is.finite(dat$fitted_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$fitted_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            linewidth = line_width
          )
      }

      p <- p +
        ggplot2::labs(
          x = x_lab,
          y = y_lab,
          colour = cfg$legend_title,
          title = plot_title_obs
        )

      if (isTRUE(cfg$facet_enabled)) {
        p <- p +
          ggplot2::facet_wrap(
            stats::as.formula("~ facet_var"),
            scales = facet_scales,
            ncol = ncol
          )
      }

      if (isTRUE(show_timing) && !is.null(x$fit_statistics)) {
        timing_marks <- dmgp_prepare_boundary_marks(
          fit_statistics = x$fit_statistics,
          facet_by = facet_by,
          x_mode = x_cfg$timing_mode,
          series = series,
          fit_id = fit_id
        )

        if (nrow(timing_marks$cumulative) > 0) {
          p <- p +
            ggplot2::geom_vline(
              data = timing_marks$cumulative,
              mapping = ggplot2::aes(
                xintercept = .data$x,
                linetype = .data$boundary,
                colour = .data$curve_group
              ),
              alpha = 0.5
            )
        }

        if (nrow(timing_marks$rate) > 0) {
          p <- p +
            ggplot2::geom_text(
              data = timing_marks$rate,
              mapping = ggplot2::aes(
                x = .data$x,
                y = .data$y,
                label = .data$triangle,
                colour = .data$curve_group
              ),
              inherit.aes = FALSE,
              show.legend = FALSE,
              size = 4
            )
        }
      }

      return(p)
    }

    if (type == "residuals") {
      dat$residual <- dat$observed - dat$fitted
      dat <- dat[is.finite(dat$residual), , drop = FALSE]

      if (nrow(dat) == 0) {
        stop("No finite residuals available to plot.")
      }

      p <- ggplot2::ggplot(
        dat,
        ggplot2::aes(
          x = .data[[x_var]],
          y = .data$residual,
          colour = .data$colour_group,
          group = .data$group_id
        )
      ) +
        ggplot2::geom_hline(yintercept = 0, linetype = 2) +
        ggplot2::geom_point(alpha = point_alpha, size = 1.4) +
        ggplot2::theme_bw() +
        ggplot2::theme(
          legend.position = if (isTRUE(cfg$use_colour)) legend_position else "none"
        ) +
        ggplot2::labs(
          x = x_lab,
          y = "Observed - fitted",
          colour = cfg$legend_title,
          title = if (observed_source == "original_daily") {
            "Residuals of original daily data against fitted curve"
          } else {
            "Residuals"
          }
        )

      if (isTRUE(cfg$facet_enabled)) {
        p <- p +
          ggplot2::facet_wrap(
            stats::as.formula("~ facet_var"),
            scales = facet_scales,
            ncol = ncol
          )
      }

      return(p)
    }

    if (type == "overlay") {
      p <- ggplot2::ggplot() +
        ggplot2::theme_bw() +
        ggplot2::theme(legend.position = legend_position)

      if (isTRUE(show_observed)) {
        p <- p +
          ggplot2::geom_point(
            data = dat[is.finite(dat$observed_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$observed_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            alpha = 0.35,
            size = 1.1
          )
      }

      if (isTRUE(show_fitted)) {
        p <- p +
          ggplot2::geom_line(
            data = dat[is.finite(dat$fitted_plot), , drop = FALSE],
            mapping = ggplot2::aes(
              x = .data[[x_var]],
              y = .data$fitted_plot,
              colour = .data$colour_group,
              group = .data$group_id
            ),
            linewidth = line_width
          )
      }

      p <- p +
        ggplot2::labs(
          x = x_lab,
          y = y_lab,
          colour = cfg$legend_title,
          title = if (observed_source == "original_daily") {
            "Overlay of fitted curves with original daily data"
          } else {
            "Overlay of fitted growth curves"
          }
        )

      if (facet_by == "default") {
        p <- p +
          ggplot2::facet_wrap(
            stats::as.formula("~ series"),
            scales = facet_scales,
            ncol = ncol
          )
      } else if (isTRUE(cfg$facet_enabled)) {
        p <- p +
          ggplot2::facet_wrap(
            stats::as.formula("~ facet_var"),
            scales = facet_scales,
            ncol = ncol
          )
      }

      return(p)
    }
  }

  if (type == "timing") {
    x_cfg <- dmgp_resolve_x_axis(type = "timing", x_axis = x_axis)

    tdat <- dmgp_prepare_timing_plot_data(
      fit_statistics = x$fit_statistics,
      facet_by = facet_by,
      x_mode = x_cfg$timing_mode,
      series = series,
      fit_id = fit_id
    )

    if (nrow(tdat$base_rows) == 0) {
      stop("No timing information available to plot.")
    }

    timing_facet_scales <- dmgp_resolve_timing_facet_scales(
      x_mode = x_cfg$timing_mode,
      facet_by = facet_by
    )

    p <- ggplot2::ggplot() +
      ggplot2::theme_bw() +
      ggplot2::theme(legend.position = legend_position)

    if (nrow(tdat$overall_segments) > 0) {
      p <- p +
        ggplot2::geom_segment(
          data = tdat$overall_segments,
          mapping = ggplot2::aes(
            x = .data$x,
            xend = .data$xend,
            y = .data$row_label,
            yend = .data$row_label,
            colour = .data$window
          ),
          linewidth = 1
        )
    }

    if (nrow(tdat$overall_ticks) > 0) {
      p <- p +
        ggplot2::geom_point(
          data = tdat$overall_ticks,
          mapping = ggplot2::aes(
            x = .data$x,
            y = .data$row_label,
            colour = .data$window
          ),
          shape = 124,
          size = 6,
          stroke = 1.1
        )
    }

    if (nrow(tdat$rate_points) > 0) {
      p <- p +
        ggplot2::geom_text(
          data = tdat$rate_points,
          mapping = ggplot2::aes(
            x = .data$x,
            y = .data$row_label,
            label = .data$triangle,
            colour = .data$window
          ),
          size = 4.5,
          show.legend = FALSE
        )
    }

    if (nrow(tdat$pulse_segments) > 0) {
      p <- p +
        ggplot2::geom_segment(
          data = tdat$pulse_segments,
          mapping = ggplot2::aes(
            x = .data$x,
            xend = .data$xend,
            y = .data$row_label,
            yend = .data$row_label,
            colour = .data$window
          ),
          linewidth = 0.8,
          alpha = 0.9
        )
    }

    if (nrow(tdat$separator_points) > 0) {
      p <- p +
        ggplot2::geom_point(
          data = tdat$separator_points,
          mapping = ggplot2::aes(
            x = .data$x,
            y = .data$row_label,
            colour = .data$window
          ),
          size = 2.2
        )
    }

    p <- p +
      ggplot2::labs(
        x = x_cfg$label,
        y = tdat$y_label,
        colour = "Timing element",
        title = "Timing of overall season and pulses"
      )

    if (isTRUE(tdat$facet_enabled)) {
      p <- p +
        ggplot2::facet_wrap(
          stats::as.formula("~ facet_var"),
          scales = timing_facet_scales,
          ncol = ncol
        )
    }

    return(p)
  }

  if (type == "parameters") {
    if (is.null(x$fit_parameters) || nrow(x$fit_parameters) == 0) {
      stop("No fit_parameters table found in the object.")
    }

    pars <- tibble::as_tibble(x$fit_parameters)

    if (!is.null(series)) {
      pars <- pars[pars$series %in% series, , drop = FALSE]
    }

    if (!is.null(fit_id)) {
      pars <- pars[pars$fit_id %in% as.character(fit_id), , drop = FALSE]
    }

    if (nrow(pars) == 0) {
      stop("No parameter rows remain after filtering by series and/or fit_id.")
    }

    param_candidates <- c(
      "a", "b", "k", "t0", "v",
      "a2", "b2", "k2", "t02", "v2",
      "edf", "span", "spline_df", "spar"
    )

    param_cols <- intersect(param_candidates, names(pars))

    if (length(param_cols) == 0) {
      stop("No parameter columns available to plot.")
    }

    plong <- pars %>%
      dplyr::select(.data$series, .data$fit_id, dplyr::all_of(param_cols)) %>%
      tidyr::pivot_longer(
        cols = dplyr::all_of(param_cols),
        names_to = "parameter",
        values_to = "value"
      ) %>%
      dplyr::filter(is.finite(.data$value))

    if (nrow(plong) == 0) {
      stop("No finite parameter values available to plot.")
    }

    if (facet_by == "tree") {
      plong$group_label <- plong$fit_id
      x_lab <- "Fit ID"
    } else if (facet_by == "year") {
      plong$group_label <- plong$series
      x_lab <- "Series"
    } else {
      if (length(unique(plong$series)) > 1) {
        plong$group_label <- plong$series
        x_lab <- "Series"
      } else {
        plong$group_label <- plong$fit_id
        x_lab <- "Fit ID"
      }
    }

    p <- ggplot2::ggplot(
      plong,
      ggplot2::aes(x = .data$group_label, y = .data$value)
    ) +
      ggplot2::geom_boxplot(outlier.shape = NA) +
      ggplot2::geom_jitter(width = 0.15, alpha = 0.7, size = 1.6) +
      ggplot2::facet_wrap(
        stats::as.formula("~ parameter"),
        scales = "free_y",
        ncol = ncol
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "none",
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      ) +
      ggplot2::labs(
        x = x_lab,
        y = "Parameter value",
        title = "Distribution of fitted parameters"
      )

    return(p)
  }

  stop("Unknown plot type.")
}


# helpers ----------------------------------------------------------------------

#' Convert growth-fit object to tidy plotting data
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to combine fitted and observed
#' daily dendrometer growth data into a long-format plotting table.
#'
#' @param x An object of class \code{"dm_growth_fit"}.
#' @param observed_source Character string. Either \code{"processed"} or
#'   \code{"original_daily"}.
#'
#' @return A tibble containing fitted values, observed values, series labels,
#'   fit identifiers, and panel identifiers.
#'
#' @keywords internal
dmgp_tidy_fit_data <- function(x,
                               observed_source = c("processed", "original_daily")) {
  observed_source <- match.arg(observed_source)

  fitd <- tibble::as_tibble(x$fitted_data)

  fit_meta_cols <- intersect(
    c(
      "TIME", "doy", "season_label", "season_start", "season_end",
      "season_day", "season_length", "any_observed"
    ),
    names(fitd)
  )

  fit_series_cols <- setdiff(names(fitd), fit_meta_cols)

  if (length(fit_series_cols) == 0) {
    stop("No fitted series columns found in fitted_data.")
  }

  if (observed_source == "processed") {
    obsd <- tibble::as_tibble(x$processed_data)
  } else {
    if (is.null(x$original_daily_data)) {
      stop(
        "This dm_growth_fit object does not contain 'original_daily_data'. ",
        "Refit with the updated fitting function."
      )
    }

    obsd <- tibble::as_tibble(x$original_daily_data)
  }

  obs_meta_cols <- intersect(
    c(
      "TIME", "doy", "season_label", "season_start", "season_end",
      "season_day", "season_length", "any_observed"
    ),
    names(obsd)
  )

  obs_series_cols <- intersect(
    fit_series_cols,
    setdiff(names(obsd), obs_meta_cols)
  )

  if (length(obs_series_cols) == 0) {
    stop("No matching observed series columns found for plotting.")
  }

  fit_long <- fitd %>%
    dplyr::select(
      dplyr::all_of(fit_meta_cols),
      dplyr::all_of(obs_series_cols)
    ) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(obs_series_cols),
      names_to = "series",
      values_to = "fitted"
    )

  obs_long <- obsd %>%
    dplyr::select(
      dplyr::all_of(obs_meta_cols),
      dplyr::all_of(obs_series_cols)
    ) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(obs_series_cols),
      names_to = "series",
      values_to = "observed"
    )

  join_cols <- intersect(names(fit_long), names(obs_long))
  join_cols <- setdiff(join_cols, c("observed", "fitted"))

  dat <- dplyr::left_join(fit_long, obs_long, by = join_cols)

  if (
    !is.null(x$fit_statistics) &&
    "fit_id" %in% names(x$fit_statistics) &&
    length(unique(stats::na.omit(x$fit_statistics$fit_id))) == 1 &&
    unique(stats::na.omit(x$fit_statistics$fit_id)) == "pooled"
  ) {
    dat$fit_id <- "pooled"
  } else {
    dat$fit_id <- as.character(dat$season_label)
  }

  dat$panel_id <- paste(dat$series, dat$fit_id, sep = " | ")

  dat
}


#' Prepare curve grouping and faceting information
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to define panel identifiers,
#' facet variables, colour groups, and legend titles.
#'
#' @param dat Tidy plotting data returned by [dmgp_tidy_fit_data()].
#' @param facet_by Character string controlling faceting layout.
#'
#' @return A list containing modified plotting data and faceting settings.
#'
#' @keywords internal
dmgp_prepare_curve_groups <- function(dat,
                                      facet_by = c("default", "tree", "year", "none")) {
  facet_by <- match.arg(facet_by)

  n_series <- length(unique(dat$series))
  n_fit <- length(unique(dat$fit_id))

  dat$group_id <- dat$panel_id

  if (facet_by == "default") {
    dat$facet_var <- dat$panel_id
    dat$colour_group <- factor("curve")

    return(list(
      data = dat,
      facet_enabled = TRUE,
      use_colour = FALSE,
      legend_title = NULL
    ))
  }

  if (facet_by == "tree") {
    dat$facet_var <- dat$series
    dat$colour_group <- factor(dat$fit_id)

    return(list(
      data = dat,
      facet_enabled = TRUE,
      use_colour = n_fit > 1,
      legend_title = "Fit ID"
    ))
  }

  if (facet_by == "year") {
    dat$facet_var <- dat$fit_id
    dat$colour_group <- factor(dat$series)

    return(list(
      data = dat,
      facet_enabled = TRUE,
      use_colour = n_series > 1,
      legend_title = "Series"
    ))
  }

  dat$facet_var <- "All"

  if (n_series > 1 && n_fit > 1) {
    dat$colour_group <- factor(dat$panel_id)
    legend_title <- "Series | Fit"
  } else if (n_series > 1) {
    dat$colour_group <- factor(dat$series)
    legend_title <- "Series"
  } else if (n_fit > 1) {
    dat$colour_group <- factor(dat$fit_id)
    legend_title <- "Fit ID"
  } else {
    dat$colour_group <- factor("curve")
    legend_title <- NULL
  }

  list(
    data = dat,
    facet_enabled = FALSE,
    use_colour = length(unique(dat$colour_group)) > 1,
    legend_title = legend_title
  )
}


#' Resolve x-axis mode for dendrometer growth-fit plots
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to determine which column is
#' used on the x-axis and which timing coordinate system should be used.
#'
#' @param type Plot type.
#' @param x_axis Requested x-axis representation.
#'
#' @return A list with elements \code{var}, \code{label}, and
#'   \code{timing_mode}.
#'
#' @keywords internal
dmgp_resolve_x_axis <- function(type,
                                x_axis = c("default", "season_day", "doy", "date")) {
  x_axis <- match.arg(x_axis)

  if (x_axis == "default") {
    if (type %in% c("fit", "timing")) {
      return(list(
        var = "TIME",
        label = "Date",
        timing_mode = "date"
      ))
    }

    return(list(
      var = "season_day",
      label = "Season day",
      timing_mode = "season_day"
    ))
  }

  if (x_axis == "date") {
    return(list(
      var = "TIME",
      label = "Date",
      timing_mode = "date"
    ))
  }

  if (x_axis == "doy") {
    return(list(
      var = "doy",
      label = "DOY",
      timing_mode = "doy"
    ))
  }

  list(
    var = "season_day",
    label = "Season day",
    timing_mode = "season_day"
  )
}


#' Resolve facet scales for curve plots
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to choose facet scale behavior.
#'
#' For date-based plots faceted by year or by individual series-fit panels, the
#' x-axis is freed so that each year or season is shown only over its own
#' temporal window. This prevents yearly fitted curves from being plotted inside
#' the full dendrometer record window when several seasons are present.
#'
#' For season-day and DOY plots, the x-axis remains shared so curves remain
#' directly comparable across facets.
#'
#' @param x_var Name of the x-axis variable.
#' @param facet_by Faceting mode.
#'
#' @return A character string accepted by [ggplot2::facet_wrap()] argument
#'   \code{scales}.
#'
#' @keywords internal
dmgp_resolve_facet_scales <- function(x_var,
                                      facet_by = c("default", "tree", "year", "none")) {
  facet_by <- match.arg(facet_by)

  if (identical(x_var, "TIME") && facet_by %in% c("default", "year")) {
    return("free")
  }

  "free_y"
}


#' Resolve facet scales for timing plots
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to choose facet scales for
#' timing-summary plots.
#'
#' @param x_mode Timing coordinate system. One of \code{"date"}, \code{"doy"},
#'   or \code{"season_day"}.
#' @param facet_by Faceting mode.
#'
#' @return A character string accepted by [ggplot2::facet_wrap()] argument
#'   \code{scales}.
#'
#' @keywords internal
dmgp_resolve_timing_facet_scales <- function(x_mode = c("date", "doy", "season_day"),
                                             facet_by = c("default", "tree", "year", "none")) {
  x_mode <- match.arg(x_mode)
  facet_by <- match.arg(facet_by)

  if (identical(x_mode, "date") && facet_by %in% c("default", "year", "tree")) {
    return("free")
  }

  "free_y"
}


#' Prepare timing boundary markers for fitted-curve plots
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to prepare vertical timing
#' markers for \code{type = "fit"} and \code{type = "season"} plots.
#'
#' @param fit_statistics Fit-statistics table from a \code{"dm_growth_fit"}
#'   object.
#' @param facet_by Faceting mode.
#' @param x_mode Timing coordinate system. One of \code{"date"}, \code{"doy"},
#'   or \code{"season_day"}.
#' @param series Optional series filter.
#' @param fit_id Optional fit identifier filter.
#'
#' @return A list with two data frames: \code{cumulative} and \code{rate}.
#'
#' @keywords internal
dmgp_prepare_boundary_marks <- function(fit_statistics,
                                        facet_by = c("default", "tree", "year", "none"),
                                        x_mode = c("date", "doy", "season_day"),
                                        series = NULL,
                                        fit_id = NULL) {
  facet_by <- match.arg(facet_by)
  x_mode <- match.arg(x_mode)

  fs <- tibble::as_tibble(fit_statistics)

  if (!is.null(series)) {
    fs <- fs[fs$series %in% series, , drop = FALSE]
  }

  if (!is.null(fit_id)) {
    fs <- fs[fs$fit_id %in% as.character(fit_id), , drop = FALSE]
  }

  if (nrow(fs) == 0) {
    return(list(
      cumulative = data.frame(),
      rate = data.frame()
    ))
  }

  fs$panel_id <- paste(fs$series, fs$fit_id, sep = " | ")

  if (facet_by == "tree") {
    fs$facet_var <- fs$series
    fs$curve_group <- as.character(fs$fit_id)
  } else if (facet_by == "year") {
    fs$facet_var <- as.character(fs$fit_id)
    fs$curve_group <- as.character(fs$series)
  } else if (facet_by == "none") {
    fs$facet_var <- "All"
    fs$curve_group <- if (length(unique(fs$panel_id)) > 1) {
      fs$panel_id
    } else {
      "curve"
    }
  } else {
    fs$facet_var <- fs$panel_id
    fs$curve_group <- "curve"
  }

  if (x_mode == "date") {
    gstart <- "growth_start_date"
    gend <- "growth_end_date"
    rstart <- "rate_start_date"
    rend <- "rate_end_date"
  } else if (x_mode == "doy") {
    gstart <- "growth_start_day"
    gend <- "growth_end_day"
    rstart <- "rate_start_day"
    rend <- "rate_end_day"
  } else {
    gstart <- "growth_start_season_day"
    gend <- "growth_end_season_day"
    rstart <- "rate_start_season_day"
    rend <- "rate_end_season_day"
  }

  cumulative <- data.frame()

  if (all(c(gstart, gend) %in% names(fs))) {
    cumulative <- dplyr::bind_rows(
      fs %>%
        dplyr::transmute(
          facet_var = .data$facet_var,
          curve_group = .data$curve_group,
          x = .data[[gstart]],
          boundary = "cumulative_start"
        ),
      fs %>%
        dplyr::transmute(
          facet_var = .data$facet_var,
          curve_group = .data$curve_group,
          x = .data[[gend]],
          boundary = "cumulative_end"
        )
    ) %>%
      dplyr::filter(!is.na(.data$x))
  }

  rate <- data.frame()

  if (all(c(rstart, rend) %in% names(fs))) {
    rate <- dplyr::bind_rows(
      fs %>%
        dplyr::transmute(
          facet_var = .data$facet_var,
          curve_group = .data$curve_group,
          x = .data[[rstart]],
          y = Inf,
          triangle = "\u25C0"
        ),
      fs %>%
        dplyr::transmute(
          facet_var = .data$facet_var,
          curve_group = .data$curve_group,
          x = .data[[rend]],
          y = Inf,
          triangle = "\u25B6"
        )
    ) %>%
      dplyr::filter(!is.na(.data$x))
  }

  list(
    cumulative = cumulative,
    rate = rate
  )
}


#' Prepare timing-summary plot data
#'
#' @description
#' Internal helper used by [plot.dm_growth_fit()] to prepare the line segments,
#' point markers, and labels used in \code{type = "timing"} plots.
#'
#' @param fit_statistics Fit-statistics table from a \code{"dm_growth_fit"}
#'   object.
#' @param facet_by Faceting mode.
#' @param x_mode Timing coordinate system. One of \code{"date"}, \code{"doy"},
#'   or \code{"season_day"}.
#' @param series Optional series filter.
#' @param fit_id Optional fit identifier filter.
#'
#' @return A list of data frames used by the timing plot.
#'
#' @keywords internal
dmgp_prepare_timing_plot_data <- function(fit_statistics,
                                          facet_by = c("default", "tree", "year", "none"),
                                          x_mode = c("date", "doy", "season_day"),
                                          series = NULL,
                                          fit_id = NULL) {
  facet_by <- match.arg(facet_by)
  x_mode <- match.arg(x_mode)

  fs <- tibble::as_tibble(fit_statistics)

  if (!is.null(series)) {
    fs <- fs[fs$series %in% series, , drop = FALSE]
  }

  if (!is.null(fit_id)) {
    fs <- fs[fs$fit_id %in% as.character(fit_id), , drop = FALSE]
  }

  if (nrow(fs) == 0) {
    return(list(
      x_mode = "season_day",
      facet_enabled = FALSE,
      y_label = "Fit",
      base_rows = fs[0, , drop = FALSE],
      overall_segments = data.frame(),
      overall_ticks = data.frame(),
      rate_points = data.frame(),
      pulse_segments = data.frame(),
      separator_points = data.frame()
    ))
  }

  if (facet_by %in% c("default", "tree")) {
    fs$facet_var <- fs$series
    fs$row_label <- as.character(fs$fit_id)
    y_label <- "Fit ID"
    facet_enabled <- length(unique(fs$facet_var)) > 1
  } else if (facet_by == "year") {
    fs$facet_var <- as.character(fs$fit_id)
    fs$row_label <- fs$series
    y_label <- "Series"
    facet_enabled <- length(unique(fs$facet_var)) > 1
  } else {
    fs$facet_var <- "All"
    fs$row_label <- paste(fs$series, fs$fit_id, sep = " | ")
    y_label <- "Series | Fit"
    facet_enabled <- FALSE
  }

  if (x_mode == "date") {
    gstart <- "growth_start_date"
    gend <- "growth_end_date"
    rstart <- "rate_start_date"
    rend <- "rate_end_date"
    p1s <- "pulse1_start_date"
    p1e <- "pulse1_end_date"
    p2s <- "pulse2_start_date"
    p2e <- "pulse2_end_date"
    sepv <- "separator_date"
  } else if (x_mode == "doy") {
    gstart <- "growth_start_day"
    gend <- "growth_end_day"
    rstart <- "rate_start_day"
    rend <- "rate_end_day"
    p1s <- "pulse1_start_day"
    p1e <- "pulse1_end_day"
    p2s <- "pulse2_start_day"
    p2e <- "pulse2_end_day"
    sepv <- "separator_day"
  } else {
    gstart <- "growth_start_season_day"
    gend <- "growth_end_season_day"
    rstart <- "rate_start_season_day"
    rend <- "rate_end_season_day"
    p1s <- "pulse1_start_season_day"
    p1e <- "pulse1_end_season_day"
    p2s <- "pulse2_start_season_day"
    p2e <- "pulse2_end_season_day"
    sepv <- "separator_season_day"
  }

  make_seg <- function(df, x1, x2, label) {
    if (!all(c(x1, x2) %in% names(df))) {
      return(data.frame())
    }

    df %>%
      dplyr::transmute(
        facet_var = .data$facet_var,
        row_label = .data$row_label,
        x = .data[[x1]],
        xend = .data[[x2]],
        window = label
      ) %>%
      dplyr::filter(!is.na(.data$x), !is.na(.data$xend))
  }

  make_ticks <- function(df, xcol, label) {
    if (!xcol %in% names(df)) {
      return(data.frame())
    }

    df %>%
      dplyr::transmute(
        facet_var = .data$facet_var,
        row_label = .data$row_label,
        x = .data[[xcol]],
        window = label
      ) %>%
      dplyr::filter(!is.na(.data$x))
  }

  make_pts <- function(df, xcol, label, tri = NULL) {
    if (!xcol %in% names(df)) {
      return(data.frame())
    }

    out <- df %>%
      dplyr::transmute(
        facet_var = .data$facet_var,
        row_label = .data$row_label,
        x = .data[[xcol]],
        window = label
      ) %>%
      dplyr::filter(!is.na(.data$x))

    if (!is.null(tri)) {
      out$triangle <- tri
    }

    out
  }

  overall_segments <- make_seg(fs, gstart, gend, "overall season")

  overall_ticks <- dplyr::bind_rows(
    make_ticks(fs, gstart, "overall season"),
    make_ticks(fs, gend, "overall season")
  )

  rate_points <- dplyr::bind_rows(
    make_pts(fs, rstart, "rate season", "\u25C0"),
    make_pts(fs, rend, "rate season", "\u25B6")
  )

  pulse_segments <- dplyr::bind_rows(
    make_seg(fs, p1s, p1e, "pulse 1"),
    make_seg(fs, p2s, p2e, "pulse 2")
  )

  separator_points <- make_pts(fs, sepv, "separator")

  list(
    x_mode = x_mode,
    facet_enabled = facet_enabled,
    y_label = y_label,
    base_rows = fs,
    overall_segments = overall_segments,
    overall_ticks = overall_ticks,
    rate_points = rate_points,
    pulse_segments = pulse_segments,
    separator_points = separator_points
  )
}

#' @title Plot method for moving dendrometer-climate correlation
#'
#' @description
#' S3 plotting method for output of \code{mov.cor.dm()}.
#' Supports heatmaps, line plots, faceted line plots, peak-correlation summaries,
#' and comparison across multiple \code{mov.cor.dm()} objects.
#'
#' @param x Object returned by \code{mov.cor.dm()}.
#' @param y Unused.
#' @param sig.only Logical. If \code{TRUE}, only significant windows are shown in
#'   heatmaps and significance is highlighted in other plot types.
#' @param ci Numeric confidence level between 0 and 1. For non-bootstrap
#'   results, significance is based on \code{p_val < 1 - ci} or
#'   \code{p_adj < 1 - ci}. For bootstrap results, the stored logical
#'   \code{significance} column is used.
#' @param clim_vars Character vector of climate variables to plot, or
#'   \code{"all"} for all variables.
#' @param type Plot type. One of \code{"heatmap"}, \code{"line"},
#'   \code{"facet"}, \code{"peak"}, or \code{"compare"}.
#' @param x_axis X-axis style. One of \code{"time"} or \code{"doy"}.
#' @param use_adjusted Logical. For non-bootstrap objects, if \code{TRUE},
#'   significance uses \code{p_adj}; otherwise \code{p_val}.
#' @param show_na Logical. If \code{TRUE}, missing values are shown in heatmaps.
#' @param show_ci Logical. If \code{TRUE}, bootstrap confidence intervals are
#'   shown in line/facet/compare plots when available.
#' @param sig_mode One of \code{"outline"}, \code{"point"}, \code{"filter"},
#'   or \code{"none"} describing how significance is displayed in non-heatmap plots.
#' @param annotate_peak Logical. If \code{TRUE}, the strongest absolute
#'   correlation is marked for each climate variable in line/facet/compare plots.
#' @param show_window_label Logical. If \code{TRUE}, \code{type = "peak"} adds
#'   the climate transformation settings above each bar.
#' @param compare_with Optional list of additional \code{mov_cor_dm} objects for
#'   comparison. Used only when \code{type = "compare"}.
#' @param compare_labels Optional character vector of labels for the comparison
#'   objects. If \code{NULL}, defaults are generated automatically.
#' @param low_col Colour for negative correlations.
#' @param mid_col Colour for zero correlations.
#' @param high_col Colour for positive correlations.
#' @param na_col Fill colour for missing values in heatmaps.
#' @param line_size Numeric line width for line plots.
#' @param point_size Numeric point size for significance markers and peak markers.
#' @param alpha_sig Numeric transparency used for significance highlighting.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @method plot mov_cor_dm
#' @importFrom dplyr bind_rows mutate filter arrange %>% left_join
#' @importFrom tibble as_tibble
#' @importFrom lubridate yday
#' @importFrom ggplot2 ggplot geom_line geom_point geom_raster geom_tile geom_col
#'   geom_ribbon geom_text aes theme_minimal labs facet_wrap theme element_text
#'   scale_fill_gradient2 scale_x_date theme_bw
#' @export
plot.mov_cor_dm <- function(x,
                            y = NULL,
                            sig.only = TRUE,
                            ci = 0.95,
                            clim_vars = "all",
                            type = c("heatmap", "line", "facet", "peak", "compare"),
                            x_axis = c("time", "doy"),
                            use_adjusted = TRUE,
                            show_na = TRUE,
                            show_ci = FALSE,
                            sig_mode = c("outline", "point", "filter", "none"),
                            annotate_peak = FALSE,
                            show_window_label = TRUE,
                            compare_with = NULL,
                            compare_labels = NULL,
                            low_col = "red",
                            mid_col = "white",
                            high_col = "blue",
                            na_col = "grey79",
                            line_size = 0.5,
                            point_size = 1.8,
                            alpha_sig = 0.9,
                            ...) {

  TIME <- corr <- para <- clim_var <- X <- source <- corr_lw_lim <- corr_up_lim <- NULL
  clim_fun <- lag_days <- accum_days <- window_label <- y_lab <- NULL

  if (!inherits(x, "mov_cor_dm")) {
    stop("The input object is not an output of 'mov.cor.dm()'.")
  }

  type <- match.arg(type)
  x_axis <- match.arg(x_axis)
  sig_mode <- match.arg(sig_mode)

  if (!is.numeric(ci) || length(ci) != 1 || is.na(ci) || ci <= 0 || ci >= 1) {
    stop("'ci' must be a single number between 0 and 1.")
  }

  .sig_note <- function(obj, ci, use_adjusted, sig.only, sig_mode) {
    if (isTRUE(obj$metadata$boot)) {
      base <- "Significance based on bootstrap CI excluding 0."
    } else {
      alpha_level <- 1 - ci
      if (isTRUE(use_adjusted)) {
        base <- paste0("Significance based on adjusted p-values < ", signif(alpha_level, 3), ".")
      } else {
        base <- paste0("Significance based on raw p-values < ", signif(alpha_level, 3), ".")
      }
    }

    mode_note <- if (type == "heatmap") {
      if (isTRUE(sig.only)) " Heatmap displays only significant windows." else " Heatmap displays all windows."
    } else {
      paste0(" sig_mode = '", sig_mode, "'.")
    }

    paste0(base, mode_note)
  }

  .extract_plot_data <- function(obj, keep_vars, label, ci, use_adjusted) {
    if (!inherits(obj, "mov_cor_dm")) {
      stop("All comparison objects must inherit from 'mov_cor_dm'.")
    }
    if (!is.list(obj$results)) {
      stop("Comparison object does not contain valid 'results'.")
    }

    varnames <- names(obj$results)
    miss <- setdiff(keep_vars, varnames)
    if (length(miss) > 0) {
      stop(sprintf(
        "The following climate variables are missing from comparison object '%s': %s",
        label, paste(miss, collapse = ", ")
      ))
    }

    dat <- dplyr::bind_rows(obj$results[keep_vars])

    if (!"clim_var" %in% names(dat)) {
      dat$clim_var <- rep(keep_vars, lengths(obj$results[keep_vars]))
    }

    dat$TIME <- as.Date(dat$TIME)
    dat$para <- toupper(dat$clim_var)
    dat$source <- label

    if (isTRUE(obj$metadata$boot)) {
      dat$sig_plot <- dat$significance
    } else {
      alpha_level <- 1 - ci
      if (use_adjusted && "p_adj" %in% names(dat)) {
        dat$sig_plot <- !is.na(dat$p_adj) & dat$p_adj < alpha_level
      } else {
        dat$sig_plot <- !is.na(dat$p_val) & dat$p_val < alpha_level
      }
    }

    dat
  }

  .peak_df <- function(din, by_source = FALSE) {
    if (by_source) {
      split_list <- split(din, interaction(din$clim_var, din$source, drop = TRUE))
    } else {
      split_list <- split(din, din$clim_var)
    }

    pk <- lapply(split_list, function(tab) {
      tab <- tab[!is.na(tab$corr), , drop = FALSE]
      if (nrow(tab) == 0) return(NULL)
      tab[which.max(abs(tab$corr)), , drop = FALSE]
    })
    dplyr::bind_rows(pk)
  }

  varnames <- names(x$results)
  if (length(varnames) == 0) {
    stop("No climate-variable results found in object.")
  }

  if (length(clim_vars) == 1 && identical(clim_vars, "all")) {
    keep_vars <- varnames
  } else {
    if (!is.character(clim_vars)) {
      stop("'clim_vars' must be 'all' or a character vector of climate variable names.")
    }
    miss <- setdiff(clim_vars, varnames)
    if (length(miss) > 0) {
      stop(sprintf(
        "The following climate variables are not available in the object: %s",
        paste(miss, collapse = ", ")
      ))
    }
    keep_vars <- clim_vars
  }

  dat <- .extract_plot_data(x, keep_vars, label = "x1", ci = ci, use_adjusted = use_adjusted)

  if (type == "compare") {
    objs <- c(list(x), compare_with)
    if (length(objs) < 2) {
      stop("For type = 'compare', provide at least one object in 'compare_with'.")
    }
    if (is.null(compare_labels)) {
      compare_labels <- paste0("obj", seq_along(objs))
    }
    if (length(compare_labels) != length(objs)) {
      stop("'compare_labels' must have the same length as c(list(x), compare_with).")
    }

    dat <- dplyr::bind_rows(
      lapply(seq_along(objs), function(i) {
        .extract_plot_data(objs[[i]], keep_vars, label = compare_labels[i], ci = ci, use_adjusted = use_adjusted)
      })
    )
  }

  if (nrow(dat) == 0) {
    stop("No data available for plotting.")
  }

  if (x_axis == "doy") {
    dat$X <- lubridate::yday(dat$TIME)
    x_lab <- "Day of year"
  } else {
    dat$X <- dat$TIME
    x_lab <- "Time"
  }

  sig_caption <- .sig_note(x, ci = ci, use_adjusted = use_adjusted, sig.only = sig.only, sig_mode = sig_mode)

  if (type != "heatmap" && sig_mode == "filter") {
    dat <- dat[dat$sig_plot %in% TRUE, , drop = FALSE]
    if (nrow(dat) == 0) {
      stop("No significant windows available after filtering.")
    }
  }

  if (type == "heatmap") {
    dat_hm <- dat

    if (isTRUE(sig.only)) {
      dat_hm <- dat_hm[dat_hm$sig_plot %in% TRUE, , drop = FALSE]
      if (nrow(dat_hm) == 0) {
        stop("No significant windows available for the heatmap.")
      }
    }

    p <- ggplot2::ggplot(dat_hm, ggplot2::aes(x = X, y = para, fill = corr)) +
      ggplot2::geom_tile() +
      ggplot2::scale_fill_gradient2(
        low = low_col,
        mid = mid_col,
        high = high_col,
        midpoint = 0,
        limits = c(-1, 1),
        na.value = if (show_na) na_col else "transparent"
      ) +
      ggplot2::labs(
        title = "Running correlation with climate",
        x = x_lab,
        y = "Climate variable",
        fill = "r",
        caption = sig_caption
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        plot.caption = ggplot2::element_text(hjust = 0, size = 9)
      )

    if (x_axis == "time") {
      p <- p + ggplot2::scale_x_date(date_labels = "%b-%y", date_breaks = "1 month")
    }

    print(p)
    return(invisible(p))
  }

  if (type == "line") {
    if (length(unique(dat$para)) > 1) {
      warning("Multiple climate variables selected; lines are overlaid. Use type = 'facet' for separate panels.")
    }

    p <- ggplot2::ggplot(dat, ggplot2::aes(x = X, y = corr, color = para)) +
      ggplot2::geom_line(linewidth = line_size) +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Running correlation with climate",
        x = x_lab,
        y = "Correlation",
        color = "Climate variable",
        caption = sig_caption
      ) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        plot.caption = ggplot2::element_text(hjust = 0, size = 9)
      )

    if (show_ci && all(c("corr_lw_lim", "corr_up_lim") %in% names(dat))) {
      p <- p +
        ggplot2::geom_ribbon(
          ggplot2::aes(ymin = corr_lw_lim, ymax = corr_up_lim, fill = para),
          alpha = 0.15,
          color = NA,
          inherit.aes = TRUE
        )
    }

    if (sig_mode == "point" && sig.only) {
      dat2 <- dat[dat$sig_plot %in% TRUE, , drop = FALSE]
      if (nrow(dat2) > 0) {
        p <- p + ggplot2::geom_point(
          data = dat2,
          ggplot2::aes(x = X, y = corr, color = para),
          size = point_size,
          alpha = alpha_sig
        )
      }
    }

    if (annotate_peak) {
      pk <- .peak_df(dat, by_source = FALSE)
      if (nrow(pk) > 0) {
        p <- p + ggplot2::geom_point(
          data = pk,
          ggplot2::aes(x = X, y = corr, color = para),
          size = point_size + 1,
          shape = 21,
          fill = "yellow",
          stroke = 1.1
        )
      }
    }

    print(p)
    return(invisible(p))
  }

  if (type == "facet") {
    p <- ggplot2::ggplot(dat, ggplot2::aes(x = X, y = corr)) +
      ggplot2::geom_line(linewidth = line_size, color = "black") +
      ggplot2::facet_wrap(~ para, scales = "free_y", ncol = 1) +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Running correlation with climate",
        x = x_lab,
        y = "Correlation",
        caption = sig_caption
      ) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        plot.caption = ggplot2::element_text(hjust = 0, size = 9)
      )

    if (show_ci && all(c("corr_lw_lim", "corr_up_lim") %in% names(dat))) {
      p <- p +
        ggplot2::geom_ribbon(
          ggplot2::aes(ymin = corr_lw_lim, ymax = corr_up_lim),
          alpha = 0.15,
          fill = "grey70",
          color = NA
        )
    }

    if (sig_mode %in% c("point", "outline") && sig.only) {
      dat2 <- dat[dat$sig_plot %in% TRUE, , drop = FALSE]
      if (nrow(dat2) > 0) {
        p <- p + ggplot2::geom_point(
          data = dat2,
          ggplot2::aes(x = X, y = corr),
          size = point_size,
          color = "red",
          alpha = alpha_sig
        )
      }
    }

    if (annotate_peak) {
      pk <- .peak_df(dat, by_source = FALSE)
      if (nrow(pk) > 0) {
        p <- p + ggplot2::geom_point(
          data = pk,
          ggplot2::aes(x = X, y = corr),
          size = point_size + 1,
          shape = 21,
          fill = "yellow",
          color = "black",
          stroke = 1.1
        )
      }
    }

    print(p)
    return(invisible(p))
  }

  if (type == "compare") {
    p <- ggplot2::ggplot(dat, ggplot2::aes(x = X, y = corr, color = source)) +
      ggplot2::geom_line(linewidth = line_size) +
      ggplot2::facet_wrap(~ para, scales = "free_y", ncol = 1) +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Comparison of running correlations",
        x = x_lab,
        y = "Correlation",
        color = "Object",
        caption = sig_caption
      ) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        plot.caption = ggplot2::element_text(hjust = 0, size = 9)
      )

    if (show_ci && all(c("corr_lw_lim", "corr_up_lim") %in% names(dat))) {
      p <- p +
        ggplot2::geom_ribbon(
          ggplot2::aes(ymin = corr_lw_lim, ymax = corr_up_lim, fill = source),
          alpha = 0.12,
          color = NA
        )
    }

    if (sig_mode == "point" && sig.only) {
      dat2 <- dat[dat$sig_plot %in% TRUE, , drop = FALSE]
      if (nrow(dat2) > 0) {
        p <- p + ggplot2::geom_point(
          data = dat2,
          ggplot2::aes(x = X, y = corr, color = source),
          size = point_size,
          alpha = alpha_sig
        )
      }
    }

    if (annotate_peak) {
      pk <- .peak_df(dat, by_source = TRUE)
      if (nrow(pk) > 0) {
        p <- p + ggplot2::geom_point(
          data = pk,
          ggplot2::aes(x = X, y = corr, color = source),
          size = point_size + 1,
          shape = 21,
          fill = "yellow",
          stroke = 1.1
        )
      }
    }

    print(p)
    return(invisible(p))
  }

  if (type == "peak") {
    peak_df <- .peak_df(dat, by_source = FALSE)

    if (nrow(peak_df) == 0) {
      stop("No non-missing correlation values available.")
    }

    if (!is.null(x$metadata$climate_settings) && nrow(x$metadata$climate_settings) > 0) {
      peak_df <- dplyr::left_join(peak_df, x$metadata$climate_settings, by = "clim_var")
      peak_df$window_label <- paste0(
        peak_df$clim_fun,
        ", lag=", peak_df$lag_days,
        ", win=", peak_df$accum_days
      )
    } else {
      peak_df$window_label <- NA_character_
    }

    p <- ggplot2::ggplot(peak_df, ggplot2::aes(x = para, y = corr, fill = corr)) +
      ggplot2::geom_col() +
      ggplot2::scale_fill_gradient2(
        low = low_col,
        mid = mid_col,
        high = high_col,
        midpoint = 0,
        limits = c(-1, 1),
        na.value = na_col
      ) +
      ggplot2::labs(
        title = "Peak absolute running correlation",
        x = "Climate variable",
        y = "Peak correlation",
        fill = "r",
        caption = sig_caption
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        plot.caption = ggplot2::element_text(hjust = 0, size = 9)
      )

    if (isTRUE(show_window_label) && any(!is.na(peak_df$window_label))) {
      y_off <- 0.04 * max(abs(peak_df$corr), na.rm = TRUE)
      peak_df$y_lab <- ifelse(peak_df$corr >= 0, peak_df$corr + y_off, peak_df$corr - y_off)

      p <- p + ggplot2::geom_text(
        data = peak_df,
        ggplot2::aes(x = para, y = y_lab, label = window_label),
        inherit.aes = FALSE,
        size = 3
      )
    }

    print(p)
    return(invisible(p))
  }
}


#' @title Backward-compatible wrapper for plotting moving correlation
#'
#' @description
#' Wrapper around \code{plot()} for objects returned by \code{mov.cor.dm()}.
#'
#' @param mov.cor.output Output of \code{mov.cor.dm()}.
#' @param ... Passed to \code{plot.mov_cor_dm()}.
#'
#' @return A \code{ggplot2} object.
#' @export
plot_mov.cor <- function(mov.cor.output, ...) {
  plot(mov.cor.output, ...)
}


#' @method plot mov.cor
#' @export
plot.mov.cor <- function(x, y = NULL, ...) {
  if (!inherits(x, "mov_cor_dm")) {
    x <- list(
      results = x,
      metadata = list(
        boot = FALSE,
        clim_names = names(x),
        tree_name = NA_character_,
        win_size = NA_integer_,
        cor_method = NA_character_,
        dm_stat = NA_character_,
        climate_settings = NULL
      )
    )
    class(x) <- c("mov_cor_dm", "mov.cor")
  }
  plot.mov_cor_dm(x, ...)
}


#' @method plot mov.cor.boot
#' @export
plot.mov.cor.boot <- function(x, y = NULL, ...) {
  if (!inherits(x, "mov_cor_dm")) {
    x <- list(
      results = x,
      metadata = list(
        boot = TRUE,
        clim_names = names(x),
        tree_name = NA_character_,
        win_size = NA_integer_,
        cor_method = NA_character_,
        dm_stat = NA_character_,
        climate_settings = NULL
      )
    )
    class(x) <- c("mov_cor_dm_boot", "mov_cor_dm", "mov.cor.boot")
  }
  plot.mov_cor_dm(x, ...)
}


#' @title Plot method for summaries of moving dendrometer-climate correlation
#'
#' @description
#' S3 plotting method for objects returned by \code{summary.mov_cor_dm()}.
#'
#' @param x Object of class \code{"summary_mov_cor_dm"}.
#' @param y Unused.
#' @param type Plot type. One of \code{"peak_corr"}, \code{"prop_significant"},
#'   or \code{"peak_time"}.
#' @param x_axis X-axis style for \code{type = "peak_time"}. One of
#'   \code{"time"} or \code{"doy"}.
#' @param low_col Colour for negative values.
#' @param mid_col Colour for zero.
#' @param high_col Colour for positive values.
#' @param show_window_label Logical. If \code{TRUE}, climate settings are shown
#'   as labels in \code{type = "peak_corr"}.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @method plot summary_mov_cor_dm
#' @importFrom ggplot2 ggplot aes geom_col geom_point geom_text scale_fill_gradient2
#' @export
plot.summary_mov_cor_dm <- function(x,
                                    y = NULL,
                                    type = c("peak_corr", "prop_significant", "peak_time"),
                                    x_axis = c("time", "doy"),
                                    low_col = "red",
                                    mid_col = "white",
                                    high_col = "blue",
                                    show_window_label = TRUE,
                                    ...) {
  clim_var <- peak_corr <- y_lab <- window_label <- prop_significant <- X <- NULL

  if (!inherits(x, "summary_mov_cor_dm")) {
    stop("Input must be an object of class 'summary_mov_cor_dm'.")
  }

  type <- match.arg(type)
  x_axis <- match.arg(x_axis)

  tab <- tibble::as_tibble(x$table)

  if (nrow(tab) == 0) {
    stop("No summary data available for plotting.")
  }

  if (type == "peak_corr") {
    p <- ggplot2::ggplot(tab, ggplot2::aes(x = clim_var, y = peak_corr, fill = peak_corr)) +
      ggplot2::geom_col() +
      ggplot2::scale_fill_gradient2(
        low = low_col,
        mid = mid_col,
        high = high_col,
        midpoint = 0,
        limits = c(-1, 1)
      ) +
      ggplot2::labs(
        x = "Climate variable",
        y = "Peak correlation",
        fill = "r",
        title = "Peak running correlation by climate variable"
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    if (isTRUE(show_window_label) &&
        all(c("clim_fun", "lag_days", "accum_days") %in% names(tab))) {
      tab$window_label <- paste0(tab$clim_fun, ", lag=", tab$lag_days, ", win=", tab$accum_days)
      y_off <- 0.04 * max(abs(tab$peak_corr), na.rm = TRUE)
      tab$y_lab <- ifelse(tab$peak_corr >= 0, tab$peak_corr + y_off, tab$peak_corr - y_off)

      p <- p + ggplot2::geom_text(
        data = tab,
        ggplot2::aes(x = clim_var, y = y_lab, label = window_label),
        inherit.aes = FALSE,
        size = 3
      )
    }

    print(p)
    return(invisible(p))
  }

  if (type == "prop_significant") {
    p <- ggplot2::ggplot(tab, ggplot2::aes(x = clim_var, y = prop_significant)) +
      ggplot2::geom_col(fill = "steelblue") +
      ggplot2::labs(
        x = "Climate variable",
        y = "Proportion significant",
        title = "Proportion of significant windows"
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "peak_time") {
    tt <- as.Date(tab$peak_time)
    if (x_axis == "doy") {
      tab$X <- lubridate::yday(tt)
      x_lab <- "Peak time (day of year)"
    } else {
      tab$X <- tt
      x_lab <- "Peak time"
    }

    p <- ggplot2::ggplot(tab, ggplot2::aes(x = X, y = clim_var)) +
      ggplot2::geom_point(size = 3, color = "black") +
      ggplot2::labs(
        x = x_lab,
        y = "Climate variable",
        title = "Timing of strongest running correlation"
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }
}

#' @title Daily rainfall data of Kathmandu for 2017.
#' @description This file contains daily rainfall data of Kathmandu. The source of this data is 'Government of Nepal, Department of Hydrology and Meteorology'.
#' @format A data frame with 365 rows and 2 variables:
#' \describe{
#'   \item{\code{TIME}}{Date in YYYY-MM-DD format.}
#'   \item{\code{rainfall}}{double rainfall in millimeters}
#'}
#' @source \url{http://www.mfd.gov.np/city?id=31/}
"ktm_rain17"

#' @title Calculate tree-, species-, or site-level statistics from clim.twd output
#'
#' @description
#' Calculates grouped statistics from the output of \code{clim.twd()}.
#'
#' The function can summarize relative dendrometer change trajectories and
#' event-level metrics:
#' \itemize{
#'   \item by tree,
#'   \item by species,
#'   \item by site,
#'   \item or by species within site.
#' }
#'
#' It can also restrict IDs to specific months, years, or custom day-of-year
#' windows based on the adverse-phase start date of each ID.
#'
#' @param x An object of class \code{"clim_twd_output"} returned by
#'   \code{clim.twd()}.
#' @param tree_info Optional metadata table describing trees. It must contain a
#'   column named \code{tree}, and may additionally contain \code{species} and
#'   \code{site}. A named character vector is also accepted for species-only
#'   mapping, where names are trees and values are species.
#' @param response Character. Which response table from \code{clim.twd()} should
#'   be summarized:
#'   \describe{
#'     \item{`"adverse_change"`}{Only the adverse phase.}
#'     \item{`"normal_change"`}{Only the following normal phase.}
#'     \item{`"full_period_change"`}{Adverse + following normal phase, reset to 0
#'     at each adverse start.}
#'     \item{`"continuous_full_period_change"`}{Adverse + following normal phase,
#'     continuing from the previous full period instead of resetting to 0.}
#'   }
#' @param group_by Character. Grouping level for trajectory summaries. One of
#'   \code{"tree"}, \code{"species"}, \code{"site"}, or \code{"species_site"}.
#' @param center Character. Central tendency used for grouped trajectories and
#'   grouped period metrics. One of \code{"mean"} or \code{"median"}.
#' @param conf_level Numeric confidence/limit level. Default is \code{0.95}.
#'   Empirical limits are returned as the lower and upper quantiles corresponding
#'   to this level.
#' @param months Optional numeric vector of months (1--12). Only IDs whose
#'   adverse-phase start falls in these months are retained.
#' @param years Optional numeric vector of years. Only IDs whose adverse-phase
#'   start year falls in these years are retained.
#' @param doy_window Optional numeric vector of length 2 defining the allowed
#'   day-of-year window for adverse-phase start. Wrapped windows are supported,
#'   for example \code{c(300, 50)}.
#' @param year_window Optional numeric vector of length 2 defining the allowed
#'   year range for adverse-phase start.
#' @param include_tree_series Logical. If \code{TRUE}, the filtered long-format
#'   tree-level data are stored in the output.
#'
#' @return
#' A list of class \code{"clim_twd_stats"} with:
#' \describe{
#'   \item{trajectory_summary}{Grouped time-series summary for each ID and date,
#'   including central tendency, SD bands, and empirical 95\% limits.}
#'   \item{id_summary}{Grouped per-ID summary derived from
#'   \code{x$period_info}.}
#'   \item{selected_ids}{IDs retained after temporal filtering.}
#'   \item{phase_table}{Filtered phase table.}
#'   \item{tree_info}{Metadata table used for grouping.}
#'   \item{long_data}{Filtered long-format tree-level response table, if
#'   \code{include_tree_series = TRUE}.}
#'   \item{settings}{List of settings used to generate the summaries.}
#' }
#'
#' @details
#' \strong{Temporal filtering} is based on the \code{adverse_start} date in
#' \code{x$phase_table}. Multiple filters are combined, so for example users can
#' simultaneously restrict to:
#' \itemize{
#'   \item specific months,
#'   \item specific years,
#'   \item a day-of-year window,
#'   \item and a year range.
#' }
#'
#' If \code{group_by = "species"}, \code{"site"}, or \code{"species_site"},
#' \code{tree_info} must provide the required columns.
#'
#' @examples
#' \donttest{
#' rel_out <- clim.twd(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   thresholdClim = "<10",
#'   thresholdDays = ">5",
#'   showPlot = FALSE
#' )
#'
#' # tree-level statistics
#' st1 <- clim.twd.stats(
#'   rel_out,
#'   response = "full_period_change",
#'   group_by = "tree"
#' )
#'
#' summary(st1)
#' plot(st1, type = "trajectory")
#'
#' # species metadata
#' tree_info <- data.frame(
#'   tree = c("T2", "T3"),
#'   species = c("Pinus", "Pinus"),
#'   site = c("Ktm", "Ktm")
#' )
#'
#' # species-level summaries restricted to IDs starting in months 6 to 8
#' st2 <- clim.twd.stats(
#'   rel_out,
#'   tree_info = tree_info,
#'   response = "full_period_change",
#'   group_by = "species",
#'   center = "median",
#'   months = 6:8
#' )
#'
#' summary(st2)
#' plot(st2, type = "trajectory", band = "limit95")
#'
#' # species within site
#' st3 <- clim.twd.stats(
#'   rel_out,
#'   tree_info = tree_info,
#'   response = "continuous_full_period_change",
#'   group_by = "species_site",
#'   doy_window = c(150, 250),
#'   year_window = c(2017, 2018)
#' )
#' }
#'
#' @importFrom dplyr mutate group_by summarise ungroup arrange filter select
#'   left_join bind_rows all_of n_distinct %>%
#' @importFrom tidyr pivot_longer
#' @importFrom tibble tibble as_tibble
#' @importFrom lubridate month year yday
#' @export
clim.twd.stats <- function(x,
                           tree_info = NULL,
                           response = c("adverse_change", "normal_change",
                                        "full_period_change", "continuous_full_period_change"),
                           group_by = c("tree", "species", "site", "species_site"),
                           center = c("mean", "median"),
                           conf_level = 0.95,
                           months = NULL,
                           years = NULL,
                           doy_window = NULL,
                           year_window = NULL,
                           include_tree_series = TRUE) {

  IDs <- TIME <- phase_type <- tree <- species <- site <- group_label <- value <- NULL
  adverse_start <- center_value <- adverse_end <- normal_start <- normal_end <- NULL
  full_end <- no_adverse_for_tree <- lag_to_below_zero <- lag_to_max_adverse_decline <- NULL
  lag_to_return_zero <- max_adverse_decline_value <- change_end_adverse <- change_end_full_period <- NULL
  continuous_start_full_period <- continuous_end_full_period <- NULL

  if (!inherits(x, "clim_twd_output")) {
    stop("'x' must be of class 'clim_twd_output'.")
  }

  response <- match.arg(response)
  group_by <- match.arg(group_by)
  center <- match.arg(center)

  if (!is.numeric(conf_level) || length(conf_level) != 1 || is.na(conf_level) ||
      conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be a single number between 0 and 1.")
  }

  if (is.null(x[[response]]) || nrow(x[[response]]) == 0) {
    stop("Selected response table is empty: ", response)
  }

  if (is.null(x$phase_table) || nrow(x$phase_table) == 0) {
    stop("The supplied clim.twd object has no phase_table.")
  }

  # -----------------------------
  # helpers
  # -----------------------------
  agg_fun <- if (center == "mean") {
    function(z) if (all(is.na(z))) NA_real_ else mean(z, na.rm = TRUE)
  } else {
    function(z) if (all(is.na(z))) NA_real_ else stats::median(z, na.rm = TRUE)
  }

  get_limits <- function(z, conf_level = 0.95) {
    if (all(is.na(z))) {
      return(c(NA_real_, NA_real_))
    }
    alpha <- (1 - conf_level) / 2
    stats::quantile(z, probs = c(alpha, 1 - alpha), na.rm = TRUE, names = FALSE)
  }

  within_doy_window <- function(doy, win) {
    if (length(win) != 2 || any(is.na(win))) {
      stop("'doy_window' must be a numeric vector of length 2.")
    }
    if (win[1] <= win[2]) {
      doy >= win[1] & doy <= win[2]
    } else {
      doy >= win[1] | doy <= win[2]
    }
  }

  # -----------------------------
  # metadata table
  # -----------------------------
  response_df <- x[[response]]
  phase_table <- x$phase_table
  period_info <- x$period_info

  tree_cols <- setdiff(names(response_df), c("TIME", "IDs", "phase_type"))

  if (is.null(tree_info)) {
    info_tbl <- tibble::tibble(
      tree = tree_cols,
      species = NA_character_,
      site = NA_character_
    )
  } else if (is.character(tree_info) && !is.null(names(tree_info))) {
    info_tbl <- tibble::tibble(
      tree = names(tree_info),
      species = unname(tree_info),
      site = NA_character_
    )
  } else if (is.data.frame(tree_info)) {
    info_tbl <- tibble::as_tibble(tree_info)

    if (!("tree" %in% names(info_tbl))) {
      stop("'tree_info' data frame must contain a column named 'tree'.")
    }
    if (!("species" %in% names(info_tbl))) info_tbl$species <- NA_character_
    if (!("site" %in% names(info_tbl))) info_tbl$site <- NA_character_

    info_tbl <- info_tbl %>% dplyr::select(tree, species, site)
  } else {
    stop("'tree_info' must be NULL, a named character vector, or a data frame with at least a 'tree' column.")
  }

  # validate grouping requirements
  if (group_by == "species" && all(is.na(info_tbl$species))) {
    stop("group_by = 'species' requires species information in 'tree_info'.")
  }
  if (group_by == "site" && all(is.na(info_tbl$site))) {
    stop("group_by = 'site' requires site information in 'tree_info'.")
  }
  if (group_by == "species_site" && (all(is.na(info_tbl$species)) || all(is.na(info_tbl$site)))) {
    stop("group_by = 'species_site' requires both species and site information in 'tree_info'.")
  }

  # -----------------------------
  # filter IDs by time windows
  # -----------------------------
  phase_tbl2 <- phase_table %>%
    dplyr::mutate(
      start_month = lubridate::month(adverse_start),
      start_year = lubridate::year(adverse_start),
      start_doy = lubridate::yday(adverse_start)
    )

  keep <- rep(TRUE, nrow(phase_tbl2))

  if (!is.null(months)) {
    keep <- keep & phase_tbl2$start_month %in% months
  }
  if (!is.null(years)) {
    keep <- keep & phase_tbl2$start_year %in% years
  }
  if (!is.null(doy_window)) {
    keep <- keep & within_doy_window(phase_tbl2$start_doy, doy_window)
  }
  if (!is.null(year_window)) {
    if (length(year_window) != 2 || any(is.na(year_window))) {
      stop("'year_window' must be a numeric vector of length 2.")
    }
    ylo <- min(year_window)
    yhi <- max(year_window)
    keep <- keep & phase_tbl2$start_year >= ylo & phase_tbl2$start_year <= yhi
  }

  phase_tbl2 <- phase_tbl2[keep, , drop = FALSE]
  selected_ids <- phase_tbl2$IDs

  if (length(selected_ids) == 0) {
    stop("No IDs remain after applying the requested temporal filters.")
  }

  # -----------------------------
  # long-format response data
  # -----------------------------
  long_dat <- response_df %>%
    dplyr::filter(IDs %in% selected_ids) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(tree_cols),
      names_to = "tree",
      values_to = "value"
    ) %>%
    dplyr::left_join(info_tbl, by = "tree")

  long_dat$group_label <- switch(
    group_by,
    tree = long_dat$tree,
    species = long_dat$species,
    site = long_dat$site,
    species_site = paste(long_dat$species, long_dat$site, sep = " @ ")
  )

  if (all(is.na(long_dat$group_label))) {
    stop("Grouping produced only missing labels. Check 'tree_info' and 'group_by'.")
  }

  # -----------------------------
  # trajectory summary
  # -----------------------------
  trajectory_summary <- long_dat %>%
    dplyr::group_by(IDs, TIME, phase_type, group_label) %>%
    dplyr::summarise(
      n_trees = sum(!is.na(value)),
      center_value = agg_fun(value),
      sd = if (sum(!is.na(value)) <= 1) NA_real_ else stats::sd(value, na.rm = TRUE),
      sd_lower = center_value - sd,
      sd_upper = center_value + sd,
      limit95_lower = get_limits(value, conf_level)[1],
      limit95_upper = get_limits(value, conf_level)[2],
      .groups = "drop"
    ) %>%
    dplyr::arrange(IDs, TIME, group_label)

  # -----------------------------
  # grouped ID summary from period_info
  # -----------------------------
  period_info2 <- period_info %>%
    dplyr::filter(IDs %in% selected_ids) %>%
    dplyr::left_join(info_tbl, by = "tree")

  period_info2$group_label <- switch(
    group_by,
    tree = period_info2$tree,
    species = period_info2$species,
    site = period_info2$site,
    species_site = paste(period_info2$species, period_info2$site, sep = " @ ")
  )

  safe_agg <- function(z) {
    if (all(is.na(z))) return(NA_real_)
    agg_fun(z)
  }

  id_summary <- period_info2 %>%
    dplyr::group_by(IDs, group_label) %>%
    dplyr::summarise(
      n_trees = dplyr::n(),
      adverse_start = min(adverse_start, na.rm = TRUE),
      adverse_end = min(adverse_end, na.rm = TRUE),
      normal_start = suppressWarnings(min(normal_start, na.rm = TRUE)),
      normal_end = suppressWarnings(min(normal_end, na.rm = TRUE)),
      full_end = min(full_end, na.rm = TRUE),
      prop_no_adverse = mean(no_adverse_for_tree, na.rm = TRUE),
      lag_to_below_zero = safe_agg(lag_to_below_zero),
      lag_to_max_adverse_decline = safe_agg(lag_to_max_adverse_decline),
      lag_to_return_zero = safe_agg(lag_to_return_zero),
      max_adverse_decline_value = safe_agg(max_adverse_decline_value),
      change_end_adverse = safe_agg(change_end_adverse),
      change_end_full_period = safe_agg(change_end_full_period),
      continuous_start_full_period = safe_agg(continuous_start_full_period),
      continuous_end_full_period = safe_agg(continuous_end_full_period),
      .groups = "drop"
    ) %>%
    dplyr::mutate(
      normal_start = ifelse(is.infinite(normal_start), NA, normal_start),
      normal_end = ifelse(is.infinite(normal_end), NA, normal_end)
    ) %>%
    dplyr::arrange(IDs, group_label)

  out <- list(
    call = match.call(),
    response = response,
    group_by = group_by,
    center = center,
    conf_level = conf_level,
    selected_ids = selected_ids,
    phase_table = phase_tbl2,
    tree_info = info_tbl,
    trajectory_summary = trajectory_summary,
    id_summary = id_summary,
    long_data = if (isTRUE(include_tree_series)) long_dat else NULL,
    settings = list(
      months = months,
      years = years,
      doy_window = doy_window,
      year_window = year_window
    )
  )

  class(out) <- "clim_twd_stats"
  out
}


#' @title Summarize clim.twd.stats output
#'
#' @param object An object of class \code{"clim_twd_stats"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An object of class \code{"summary.clim_twd_stats"}.
#'
#' @method summary clim_twd_stats
#' @export
summary.clim_twd_stats <- function(object, ...) {
  if (!inherits(object, "clim_twd_stats")) {
    stop("'object' must be of class 'clim_twd_stats'.")
  }

  overview <- tibble::tibble(
    response = object$response,
    group_by = object$group_by,
    center = object$center,
    conf_level = object$conf_level,
    n_ids = length(unique(object$selected_ids)),
    n_groups = length(unique(object$trajectory_summary$group_label)),
    n_rows_trajectory = nrow(object$trajectory_summary),
    n_rows_id_summary = nrow(object$id_summary)
  )

  out <- list(
    overview = overview,
    phase_table = object$phase_table,
    trajectory_head = utils::head(object$trajectory_summary, 12),
    id_summary = object$id_summary
  )

  class(out) <- "summary.clim_twd_stats"
  out
}


#' @title Print summary of clim.twd.stats output
#'
#' @param x An object of class \code{"summary.clim_twd_stats"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#'
#' @method print summary.clim_twd_stats
#' @export
print.summary.clim_twd_stats <- function(x, ...) {
  if (!inherits(x, "summary.clim_twd_stats")) {
    stop("'x' must be of class 'summary.clim_twd_stats'.")
  }

  cat("Summary of clim.twd.stats output\n")
  cat("--------------------------------\n\n")

  cat("Overview:\n")
  print(x$overview)

  cat("\nFiltered phase table:\n")
  print(x$phase_table)

  cat("\nTrajectory summary preview:\n")
  print(x$trajectory_head)

  cat("\nID summary:\n")
  print(x$id_summary)

  invisible(x)
}


#' @title Plot method for clim.twd.stats output
#'
#' @description
#' Plots grouped trajectories or grouped ID-level metrics from
#' \code{clim.twd.stats()}.
#'
#' @param x An object of class \code{"clim_twd_stats"}.
#' @param y Unused.
#' @param type One of \code{"trajectory"} or \code{"id_metric"}.
#' @param ids Optional numeric vector of IDs to plot.
#' @param groups Optional character vector of group labels to plot.
#' @param band One of \code{"none"}, \code{"sd"}, \code{"limit95"}, or \code{"both"}.
#'   Used for \code{type = "trajectory"}.
#' @param metric Metric column from \code{x$id_summary} for \code{type = "id_metric"}.
#' @param facet_by One of \code{"IDs"}, \code{"group"}, or \code{"none"}.
#' @param legend_position Legend position.
#' @param main Optional title.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return A \code{ggplot2} object.
#'
#' @method plot clim_twd_stats
#' @importFrom dplyr filter bind_rows %>%
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon geom_col facet_wrap
#'   theme_minimal theme element_text labs position_dodge
#' @export
plot.clim_twd_stats <- function(x,
                                y = NULL,
                                type = c("trajectory", "id_metric"),
                                ids = NULL,
                                groups = NULL,
                                band = c("none", "sd", "limit95", "both"),
                                metric = c("lag_to_below_zero",
                                           "lag_to_max_adverse_decline",
                                           "lag_to_return_zero",
                                           "max_adverse_decline_value",
                                           "change_end_adverse",
                                           "change_end_full_period",
                                           "continuous_end_full_period"),
                                facet_by = c("IDs", "group", "none"),
                                legend_position = "bottom",
                                main = NULL,
                                ...) {

  IDs <- TIME <- group_label <- center_value <- sd_lower <- sd_upper <- NULL
  limit95_lower <- limit95_upper <- group_label <- value <- NULL

  if (!inherits(x, "clim_twd_stats")) {
    stop("'x' must be of class 'clim_twd_stats'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plotting.")
  }

  type <- match.arg(type)
  band <- match.arg(band)
  facet_by <- match.arg(facet_by)
  metric <- match.arg(metric)

  if (type == "trajectory") {
    dat <- x$trajectory_summary

    if (!is.null(ids)) dat <- dat %>% dplyr::filter(IDs %in% ids)
    if (!is.null(groups)) dat <- dat %>% dplyr::filter(group_label %in% groups)

    if (nrow(dat) == 0) stop("No trajectory rows remain after filtering.")

    p <- ggplot2::ggplot(dat, ggplot2::aes(x = TIME, y = center_value, colour = group_label, fill = group_label))

    if (band %in% c("sd", "both")) {
      p <- p + ggplot2::geom_ribbon(
        ggplot2::aes(ymin = sd_lower, ymax = sd_upper),
        alpha = 0.15,
        colour = NA
      )
    }

    if (band %in% c("limit95", "both")) {
      p <- p + ggplot2::geom_ribbon(
        ggplot2::aes(ymin = limit95_lower, ymax = limit95_upper),
        alpha = 0.10,
        colour = NA
      )
    }

    p <- p +
      ggplot2::geom_line() +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      ) +
      ggplot2::labs(
        title = if (is.null(main)) {
          paste("Grouped trajectory:", x$group_by, "(", x$center, ")")
        } else main,
        x = "Time",
        y = "Relative change",
        colour = x$group_by,
        fill = x$group_by
      )

    if (facet_by == "IDs" && length(unique(dat$IDs)) > 1) {
      p <- p + ggplot2::facet_wrap(~ IDs, scales = "free_x", ncol = 1)
    } else if (facet_by == "group" && length(unique(dat$group_label)) > 1) {
      p <- p + ggplot2::facet_wrap(~ group_label, scales = "free_x", ncol = 1)
    }

    return(p)
  }

  dat <- x$id_summary

  if (!is.null(ids)) dat <- dat %>% dplyr::filter(IDs %in% ids)
  if (!is.null(groups)) dat <- dat %>% dplyr::filter(group_label %in% groups)

  if (nrow(dat) == 0) stop("No ID summary rows remain after filtering.")

  p <- ggplot2::ggplot(
    dat,
    ggplot2::aes(x = group_label, y = .data[[metric]], fill = group_label)
  ) +
    ggplot2::geom_col() +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      legend.position = legend_position,
      axis.title = ggplot2::element_text(size = 14),
      axis.text = ggplot2::element_text(size = 12, angle = 45, hjust = 1)
    ) +
    ggplot2::labs(
      title = if (is.null(main)) paste("ID metric:", metric) else main,
      x = x$group_by,
      y = metric,
      fill = x$group_by
    )

  if (facet_by == "IDs" && length(unique(dat$IDs)) > 1) {
    p <- p + ggplot2::facet_wrap(~ IDs, scales = "free_y", ncol = 1)
  }

  p
}

#' @title Statistical testing for clim.twd output
#'
#' @description
#' Performs hypothesis tests on parameters derived from \code{clim.twd()} output.
#'
#' This function is designed for scenarios such as:
#' \itemize{
#'   \item species comparison as a whole,
#'   \item species comparison by year,
#'   \item site comparison as a whole,
#'   \item site comparison by year,
#'   \item species-within-site comparison,
#'   \item ID-wise comparisons.
#' }
#'
#' The function uses the \code{period_info} table from \code{clim.twd()} and
#' tests selected parameters across groups such as trees, species, sites, or
#' species-site combinations.
#'
#' @param x An object of class \code{"clim_twd_output"} returned by
#'   \code{clim.twd()}.
#' @param tree_info Optional metadata table describing trees. It must contain a
#'   column named \code{tree}, and may additionally contain \code{species} and
#'   \code{site}. A named character vector is also accepted for species-only
#'   mapping, where names are trees and values are species.
#' @param parameter Character. Parameter from \code{x$period_info} to test.
#'   Supported examples include:
#'   \describe{
#'     \item{`"lag_to_below_zero"`}{Lag from adverse start until relative change
#'     first drops below zero.}
#'     \item{`"lag_to_max_adverse_decline"`}{Lag from adverse start until maximum
#'     adverse decline is reached.}
#'     \item{`"lag_to_return_zero"`}{Lag from adverse start until relative change
#'     returns to zero or above.}
#'     \item{`"max_adverse_decline_value"`}{Most negative relative change during
#'     the adverse phase.}
#'     \item{`"change_end_adverse"`}{Relative change at the end of the adverse phase.}
#'     \item{`"change_end_full_period"`}{Relative change at the end of the full period.}
#'     \item{`"continuous_end_full_period"`}{Continuous end value across periods.}
#'   }
#' @param compare_by Character. Grouping variable to compare. One of
#'   \code{"tree"}, \code{"species"}, \code{"site"}, or \code{"species_site"}.
#' @param within Character. Defines the comparison scenario:
#'   \describe{
#'     \item{`"all"`}{Compare groups across all retained IDs.}
#'     \item{`"year"`}{Compare groups separately within each adverse-start year.}
#'     \item{`"ID"`}{Compare groups separately within each ID.}
#'   }
#' @param test_method Character. One of \code{"auto"}, \code{"t_test"},
#'   \code{"welch_t"}, \code{"wilcox"}, \code{"anova"}, or \code{"kruskal"}.
#'   With \code{"auto"}, the function uses:
#'   \itemize{
#'     \item Welch t-test or one-way ANOVA when group distributions look
#'     approximately normal,
#'     \item Wilcoxon or Kruskal-Wallis otherwise.
#'   }
#' @param ids Optional numeric vector of IDs to retain before testing.
#' @param years Optional numeric vector of years to retain based on
#'   \code{adverse_start}.
#' @param months Optional numeric vector of months (1--12) to retain based on
#'   \code{adverse_start}.
#' @param doy_window Optional numeric vector of length 2 defining the allowed
#'   day-of-year window of \code{adverse_start}. Wrapped windows such as
#'   \code{c(300, 50)} are supported.
#' @param year_window Optional numeric vector of length 2 defining the allowed
#'   year range for \code{adverse_start}.
#' @param pairwise Logical. If \code{TRUE}, pairwise post-hoc tests are computed
#'   for strata with more than two groups when applicable.
#' @param p_adjust_method Character. P-value adjustment method for pairwise tests.
#'   Default is \code{"BH"}.
#' @param min_n_per_group Minimum number of non-missing observations required per
#'   group before testing. Default is \code{2}.
#'
#' @return
#' A list of class \code{"clim_twd_test"} containing:
#' \describe{
#'   \item{data_used}{Filtered test data used for the analyses.}
#'   \item{test_results}{Main test results by stratum.}
#'   \item{pairwise_results}{Pairwise comparison table where available.}
#'   \item{group_summary}{Descriptive summaries by stratum and group.}
#'   \item{settings}{Settings used for the analysis.}
#' }
#'
#' @examples
#' \donttest{
#' rel_out <- clim.twd(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   thresholdClim = "<10",
#'   thresholdDays = ">5",
#'   showPlot = FALSE
#' )
#'
#' tree_info <- data.frame(
#'   tree = c("T2", "T3"),
#'   species = c("Pinus", "Pinus"),
#'   site = c("Ktm", "Ktm")
#' )
#'
#' # species comparison across all retained IDs
#' tst1 <- clim.twd.test(
#'   rel_out,
#'   tree_info = tree_info,
#'   parameter = "max_adverse_decline_value",
#'   compare_by = "species",
#'   within = "all"
#' )
#'
#' summary(tst1)
#' plot(tst1)
#'
#' # species comparison by year
#' tst2 <- clim.twd.test(
#'   rel_out,
#'   tree_info = tree_info,
#'   parameter = "lag_to_return_zero",
#'   compare_by = "species",
#'   within = "year"
#' )
#'
#' # site comparison by year
#' tst3 <- clim.twd.test(
#'   rel_out,
#'   tree_info = tree_info,
#'   parameter = "change_end_full_period",
#'   compare_by = "site",
#'   within = "year",
#'   test_method = "kruskal"
#' )
#' }
#'
#' @importFrom dplyr mutate group_by summarise ungroup arrange filter select
#'   left_join bind_rows n_distinct all_of %>%
#' @importFrom tibble tibble as_tibble
#' @importFrom lubridate year month yday
#' @importFrom ggplot2 ggplot aes geom_boxplot geom_jitter facet_wrap theme_minimal
#'   theme element_text labs
#' @importFrom stats shapiro.test t.test wilcox.test kruskal.test aov
#'   pairwise.t.test pairwise.wilcox.test
#' @export
clim.twd.test <- function(x,
                          tree_info = NULL,
                          parameter = c(
                            "lag_to_below_zero",
                            "lag_to_max_adverse_decline",
                            "lag_to_return_zero",
                            "max_adverse_decline_value",
                            "change_end_adverse",
                            "change_end_full_period",
                            "continuous_end_full_period"
                          ),
                          compare_by = c("tree", "species", "site", "species_site"),
                          within = c("all", "year", "ID"),
                          test_method = c("auto", "t_test", "welch_t", "wilcox", "anova", "kruskal"),
                          ids = NULL,
                          years = NULL,
                          months = NULL,
                          doy_window = NULL,
                          year_window = NULL,
                          pairwise = TRUE,
                          p_adjust_method = "BH",
                          min_n_per_group = 2) {

  tree <- species <- site <- group_label <- stratum <- n_group <- .n_group <- NULL
  adverse_start <- IDs <- value <- start_year <- start_month <- start_doy <- NULL

  if (!inherits(x, "clim_twd_output")) {
    stop("'x' must be of class 'clim_twd_output'.")
  }

  if (is.null(x$period_info) || nrow(x$period_info) == 0) {
    stop("The supplied object has no 'period_info' table.")
  }

  parameter <- match.arg(parameter)
  compare_by <- match.arg(compare_by)
  within <- match.arg(within)
  test_method <- match.arg(test_method)

  if (!parameter %in% names(x$period_info)) {
    stop("Parameter not found in x$period_info: ", parameter)
  }

  # -----------------------------
  # helpers
  # -----------------------------
  within_doy_window <- function(doy, win) {
    if (length(win) != 2 || any(is.na(win))) {
      stop("'doy_window' must be a numeric vector of length 2.")
    }
    if (win[1] <= win[2]) {
      doy >= win[1] & doy <= win[2]
    } else {
      doy >= win[1] | doy <= win[2]
    }
  }

  safe_shapiro <- function(z) {
    z <- z[is.finite(z)]
    if (length(z) < 3 || length(z) > 5000) return(NA_real_)
    out <- try(stats::shapiro.test(z), silent = TRUE)
    if (inherits(out, "try-error")) return(NA_real_)
    out$p.value
  }

  infer_method <- function(dat, compare_by) {
    vals_split <- split(dat$value, dat$group_label)
    vals_split <- vals_split[vapply(vals_split, function(z) sum(is.finite(z)) > 0, logical(1))]

    ng <- length(vals_split)
    if (ng < 2) return(NA_character_)

    shp <- vapply(vals_split, safe_shapiro, numeric(1))
    normal_like <- all(is.na(shp) | shp > 0.05)

    if (ng == 2) {
      if (normal_like) return("welch_t")
      return("wilcox")
    }

    if (normal_like) return("anova")
    "kruskal"
  }

  run_main_test <- function(dat, method) {
    ng <- length(unique(dat$group_label))
    if (ng < 2) {
      return(tibble::tibble(
        method = NA_character_,
        statistic = NA_real_,
        p_value = NA_real_,
        df = NA_real_,
        n_groups = ng,
        note = "Fewer than 2 groups remained."
      ))
    }

    if (method == "t_test") {
      if (ng != 2) {
        return(tibble::tibble(
          method = method,
          statistic = NA_real_,
          p_value = NA_real_,
          df = NA_real_,
          n_groups = ng,
          note = "t_test requires exactly 2 groups."
        ))
      }
      fit <- stats::t.test(value ~ group_label, data = dat, var.equal = TRUE)
      return(tibble::tibble(
        method = method,
        statistic = unname(fit$statistic),
        p_value = fit$p.value,
        df = unname(fit$parameter),
        n_groups = ng,
        note = NA_character_
      ))
    }

    if (method == "welch_t") {
      if (ng != 2) {
        return(tibble::tibble(
          method = method,
          statistic = NA_real_,
          p_value = NA_real_,
          df = NA_real_,
          n_groups = ng,
          note = "welch_t requires exactly 2 groups."
        ))
      }
      fit <- stats::t.test(value ~ group_label, data = dat, var.equal = FALSE)
      return(tibble::tibble(
        method = method,
        statistic = unname(fit$statistic),
        p_value = fit$p.value,
        df = unname(fit$parameter),
        n_groups = ng,
        note = NA_character_
      ))
    }

    if (method == "wilcox") {
      if (ng != 2) {
        return(tibble::tibble(
          method = method,
          statistic = NA_real_,
          p_value = NA_real_,
          df = NA_real_,
          n_groups = ng,
          note = "wilcox requires exactly 2 groups."
        ))
      }
      fit <- stats::wilcox.test(value ~ group_label, data = dat, exact = FALSE)
      return(tibble::tibble(
        method = method,
        statistic = unname(fit$statistic),
        p_value = fit$p.value,
        df = NA_real_,
        n_groups = ng,
        note = NA_character_
      ))
    }

    if (method == "anova") {
      fit <- stats::aov(value ~ group_label, data = dat)
      sm <- summary(fit)[[1]]
      return(tibble::tibble(
        method = method,
        statistic = sm[1, "F value"],
        p_value = sm[1, "Pr(>F)"],
        df = sm[1, "Df"],
        n_groups = ng,
        note = NA_character_
      ))
    }

    if (method == "kruskal") {
      fit <- stats::kruskal.test(value ~ group_label, data = dat)
      return(tibble::tibble(
        method = method,
        statistic = unname(fit$statistic),
        p_value = fit$p.value,
        df = unname(fit$parameter),
        n_groups = ng,
        note = NA_character_
      ))
    }

    tibble::tibble(
      method = method,
      statistic = NA_real_,
      p_value = NA_real_,
      df = NA_real_,
      n_groups = ng,
      note = "Unknown method."
    )
  }

  run_pairwise_test <- function(dat, method, p_adjust_method) {
    ng <- length(unique(dat$group_label))
    if (ng < 3) return(NULL)

    if (method %in% c("anova", "t_test", "welch_t")) {
      pw <- stats::pairwise.t.test(
        x = dat$value,
        g = dat$group_label,
        p.adjust.method = p_adjust_method,
        pool.sd = FALSE
      )
    } else if (method %in% c("kruskal", "wilcox")) {
      pw <- stats::pairwise.wilcox.test(
        x = dat$value,
        g = dat$group_label,
        p.adjust.method = p_adjust_method,
        exact = FALSE
      )
    } else {
      return(NULL)
    }

    mat <- pw$p.value
    if (is.null(mat)) return(NULL)

    out <- list()
    rn <- rownames(mat)
    cn <- colnames(mat)

    idx <- 1L
    for (i in seq_len(nrow(mat))) {
      for (j in seq_len(ncol(mat))) {
        if (!is.na(mat[i, j])) {
          out[[idx]] <- tibble::tibble(
            group1 = rn[i],
            group2 = cn[j],
            p_value = mat[i, j]
          )
          idx <- idx + 1L
        }
      }
    }

    if (length(out) == 0) return(NULL)
    dplyr::bind_rows(out)
  }

  # -----------------------------
  # metadata
  # -----------------------------
  period_info <- x$period_info

  if (is.null(tree_info)) {
    info_tbl <- tibble::tibble(
      tree = unique(period_info$tree),
      species = NA_character_,
      site = NA_character_
    )
  } else if (is.character(tree_info) && !is.null(names(tree_info))) {
    info_tbl <- tibble::tibble(
      tree = names(tree_info),
      species = unname(tree_info),
      site = NA_character_
    )
  } else if (is.data.frame(tree_info)) {
    info_tbl <- tibble::as_tibble(tree_info)
    if (!("tree" %in% names(info_tbl))) {
      stop("'tree_info' data frame must contain a column named 'tree'.")
    }
    if (!("species" %in% names(info_tbl))) info_tbl$species <- NA_character_
    if (!("site" %in% names(info_tbl))) info_tbl$site <- NA_character_
    info_tbl <- info_tbl %>% dplyr::select(tree, species, site)
  } else {
    stop("'tree_info' must be NULL, a named character vector, or a data frame with at least a 'tree' column.")
  }

  if (compare_by == "species" && all(is.na(info_tbl$species))) {
    stop("compare_by = 'species' requires species information in 'tree_info'.")
  }
  if (compare_by == "site" && all(is.na(info_tbl$site))) {
    stop("compare_by = 'site' requires site information in 'tree_info'.")
  }
  if (compare_by == "species_site" && (all(is.na(info_tbl$species)) || all(is.na(info_tbl$site)))) {
    stop("compare_by = 'species_site' requires both species and site information in 'tree_info'.")
  }

  dat <- period_info %>%
    dplyr::left_join(info_tbl, by = "tree") %>%
    dplyr::mutate(
      start_year = lubridate::year(adverse_start),
      start_month = lubridate::month(adverse_start),
      start_doy = lubridate::yday(adverse_start),
      value = .data[[parameter]]
    )

  if (!is.null(ids)) {
    dat <- dat %>% dplyr::filter(IDs %in% ids)
  }
  if (!is.null(years)) {
    dat <- dat %>% dplyr::filter(start_year %in% years)
  }
  if (!is.null(months)) {
    dat <- dat %>% dplyr::filter(start_month %in% months)
  }
  if (!is.null(doy_window)) {
    dat <- dat %>% dplyr::filter(within_doy_window(start_doy, doy_window))
  }
  if (!is.null(year_window)) {
    if (length(year_window) != 2 || any(is.na(year_window))) {
      stop("'year_window' must be a numeric vector of length 2.")
    }
    ylo <- min(year_window)
    yhi <- max(year_window)
    dat <- dat %>% dplyr::filter(start_year >= ylo, start_year <= yhi)
  }

  if (nrow(dat) == 0) {
    stop("No rows remain after filtering.")
  }

  dat$group_label <- switch(
    compare_by,
    tree = dat$tree,
    species = dat$species,
    site = dat$site,
    species_site = paste(dat$species, dat$site, sep = " @ ")
  )

  dat <- dat %>% dplyr::filter(!is.na(group_label), !is.na(value))

  if (nrow(dat) == 0) {
    stop("No rows remain after removing missing groups or missing parameter values.")
  }

  dat$stratum <- switch(
    within,
    all = "all",
    year = as.character(dat$start_year),
    ID = paste0("ID_", dat$IDs)
  )

  # drop groups with too few observations within each stratum
  dat <- dat %>%
    dplyr::group_by(stratum, group_label) %>%
    dplyr::mutate(.n_group = sum(is.finite(value))) %>%
    dplyr::ungroup() %>%
    dplyr::filter(.n_group >= min_n_per_group) %>%
    dplyr::select(-.n_group)

  if (nrow(dat) == 0) {
    stop("No rows remain after applying 'min_n_per_group'.")
  }

  # descriptive group summary
  group_summary <- dat %>%
    dplyr::group_by(stratum, group_label) %>%
    dplyr::summarise(
      n = sum(is.finite(value)),
      mean = if (all(is.na(value))) NA_real_ else mean(value, na.rm = TRUE),
      median = if (all(is.na(value))) NA_real_ else stats::median(value, na.rm = TRUE),
      sd = if (sum(is.finite(value)) <= 1) NA_real_ else stats::sd(value, na.rm = TRUE),
      min = if (all(is.na(value))) NA_real_ else min(value, na.rm = TRUE),
      max = if (all(is.na(value))) NA_real_ else max(value, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::arrange(stratum, group_label)

  # main tests and pairwise tests
  strata <- unique(dat$stratum)
  main_list <- list()
  pair_list <- list()
  idx_main <- 1L
  idx_pair <- 1L

  for (ss in strata) {
    dss <- dat %>% dplyr::filter(stratum == ss)

    # groups still valid
    ng <- length(unique(dss$group_label))
    if (ng < 2) {
      main_list[[idx_main]] <- tibble::tibble(
        stratum = ss,
        parameter = parameter,
        compare_by = compare_by,
        method = NA_character_,
        statistic = NA_real_,
        p_value = NA_real_,
        df = NA_real_,
        n_groups = ng,
        note = "Fewer than 2 valid groups."
      )
      idx_main <- idx_main + 1L
      next
    }

    mth <- if (test_method == "auto") infer_method(dss, compare_by) else test_method

    res_main <- run_main_test(dss, mth)
    res_main <- cbind(
      tibble::tibble(
        stratum = ss,
        parameter = parameter,
        compare_by = compare_by
      ),
      res_main
    )
    main_list[[idx_main]] <- res_main
    idx_main <- idx_main + 1L

    if (isTRUE(pairwise)) {
      pw <- run_pairwise_test(dss, mth, p_adjust_method = p_adjust_method)
      if (!is.null(pw) && nrow(pw) > 0) {
        pw <- cbind(
          tibble::tibble(
            stratum = ss,
            parameter = parameter,
            compare_by = compare_by,
            method = mth
          ),
          pw
        )
        pair_list[[idx_pair]] <- pw
        idx_pair <- idx_pair + 1L
      }
    }
  }

  test_results <- dplyr::bind_rows(main_list)
  pairwise_results <- dplyr::bind_rows(pair_list)

  out <- list(
    call = match.call(),
    data_used = dat,
    test_results = test_results,
    pairwise_results = pairwise_results,
    group_summary = group_summary,
    settings = list(
      parameter = parameter,
      compare_by = compare_by,
      within = within,
      test_method = test_method,
      ids = ids,
      years = years,
      months = months,
      doy_window = doy_window,
      year_window = year_window,
      pairwise = pairwise,
      p_adjust_method = p_adjust_method,
      min_n_per_group = min_n_per_group
    )
  )

  class(out) <- "clim_twd_test"
  out
}


#' @title Summarize clim.twd.test output
#'
#' @param object An object of class \code{"clim_twd_test"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An object of class \code{"summary.clim_twd_test"}.
#'
#' @method summary clim_twd_test
#' @export
summary.clim_twd_test <- function(object, ...) {
  if (!inherits(object, "clim_twd_test")) {
    stop("'object' must be of class 'clim_twd_test'.")
  }

  overview <- tibble::tibble(
    parameter = object$settings$parameter,
    compare_by = object$settings$compare_by,
    within = object$settings$within,
    test_method = object$settings$test_method,
    n_rows_data = nrow(object$data_used),
    n_strata = length(unique(object$data_used$stratum)),
    n_groups = length(unique(object$data_used$group_label)),
    n_main_tests = nrow(object$test_results),
    n_pairwise_tests = if (is.null(object$pairwise_results)) 0L else nrow(object$pairwise_results)
  )

  out <- list(
    overview = overview,
    group_summary = object$group_summary,
    test_results = object$test_results,
    pairwise_head = utils::head(object$pairwise_results, 20)
  )

  class(out) <- "summary.clim_twd_test"
  out
}


#' @title Print summary of clim.twd.test output
#'
#' @param x An object of class \code{"summary.clim_twd_test"}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#'
#' @method print summary.clim_twd_test
#' @export
print.summary.clim_twd_test <- function(x, ...) {
  if (!inherits(x, "summary.clim_twd_test")) {
    stop("'x' must be of class 'summary.clim_twd_test'.")
  }

  cat("Summary of clim.twd.test output\n")
  cat("--------------------------------\n\n")

  cat("Overview:\n")
  print(x$overview)

  cat("\nGroup summary:\n")
  print(x$group_summary)

  cat("\nMain test results:\n")
  print(x$test_results)

  if (!is.null(x$pairwise_head) && nrow(x$pairwise_head) > 0) {
    cat("\nPairwise comparisons (preview):\n")
    print(x$pairwise_head)
  }

  invisible(x)
}


#' @title Plot method for clim.twd.test output
#'
#' @description
#' Plots the data used in \code{clim.twd.test()} as grouped boxplots with jittered
#' observations, faceted by stratum when applicable.
#'
#' @param x An object of class \code{"clim_twd_test"}.
#' @param y Unused.
#' @param show_points Logical. If \code{TRUE}, add jittered points.
#' @param facet Logical. If \code{TRUE}, facet by stratum when multiple strata exist.
#' @param legend_position Legend position.
#' @param main Optional plot title.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return A \code{ggplot2} object.
#'
#' @method plot clim_twd_test
#' @export
plot.clim_twd_test <- function(x,
                               y = NULL,
                               show_points = TRUE,
                               facet = TRUE,
                               legend_position = "none",
                               main = NULL,
                               ...) {
  group_label <- value <- NULL

  if (!inherits(x, "clim_twd_test")) {
    stop("'x' must be of class 'clim_twd_test'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plotting.")
  }

  dat <- x$data_used
  if (is.null(dat) || nrow(dat) == 0) {
    stop("No data available for plotting.")
  }

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = group_label, y = value, fill = group_label)) +
    ggplot2::geom_boxplot(outlier.shape = NA)

  if (isTRUE(show_points)) {
    p <- p + ggplot2::geom_jitter(width = 0.15, alpha = 0.7, size = 1.5)
  }

  p <- p +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      legend.position = legend_position,
      axis.title = ggplot2::element_text(size = 14),
      axis.text = ggplot2::element_text(size = 12, angle = 45, hjust = 1)
    ) +
    ggplot2::labs(
      title = if (is.null(main)) {
        paste0(
          "Parameter: ", x$settings$parameter,
          " | Compare by: ", x$settings$compare_by,
          " | Within: ", x$settings$within
        )
      } else main,
      x = x$settings$compare_by,
      y = x$settings$parameter
    )

  if (isTRUE(facet) && length(unique(dat$stratum)) > 1) {
    p <- p + ggplot2::facet_wrap(~ stratum, scales = "free_y", ncol = 1)
  }

  p
}

#' @title Calculate relative dendrometer change during adverse climate periods and the following normal phase
#'
#' @description
#' Identifies adverse climate periods from a climate time series and calculates
#' relative dendrometer change from the start of each adverse phase.
#'
#' In addition to the adverse phase itself, the function also defines a
#' \emph{complete period} for each event ID:
#' \itemize{
#'   \item the adverse climate phase, and
#'   \item the following normal climate phase until the next adverse phase starts
#'   (or until the end of the common series).
#' }
#'
#' Relative dendrometer change is always calculated from the first day of the
#' adverse phase for that ID.
#'
#' The function returns:
#' \itemize{
#'   \item \code{adverse_change}: relative change only during adverse phases,
#'   \item \code{normal_change}: relative change only during the following normal phases,
#'   \item \code{full_period_change}: relative change across adverse + following normal phase,
#'   \item \code{continuous_full_period_change}: same as \code{full_period_change},
#'   but values do not reset to 0 at the next adverse phase and instead continue
#'   from the end of the previous full period,
#'   \item \code{period_info}: per-ID and per-tree summary metrics.
#' }
#'
#' @details
#' The algorithm works as follows:
#' \enumerate{
#'   \item Climate data are thresholded to identify adverse days.
#'   \item Consecutive adverse days are grouped into candidate adverse runs.
#'   \item Only runs satisfying \code{thresholdDays} are retained as true adverse events.
#'   \item For each retained adverse event, the following normal period is defined
#'   as all non-adverse days until the next adverse event starts (or the end of
#'   the series).
#'   \item Dendrometer data are resampled to daily resolution using
#'   \code{\link[dendRoAnalyst:dendro.resample]{dendro.resample}}.
#'   \item Relative dendrometer change is calculated from the first day of the
#'   adverse phase for each ID.
#' }
#'
#' \strong{Threshold syntax:}
#' \itemize{
#'   \item single-threshold forms such as \code{"<5"}, \code{"<=10"},
#'   \code{">3"}, \code{">=0"}, \code{"==0"},
#'   \item range forms for \code{thresholdClim} such as \code{"between(5,10)"}
#'   or \code{"5:10"}, both interpreted inclusively.
#' }
#'
#' \code{thresholdDays} must use a single-threshold form such as \code{">5"}.
#'
#' @references
#' Raffelsbauer V, Spannl S, Peña K, Pucha-Cofrep D, Steppe K, Bräuning A (2019).
#' Tree Circumference Changes and Species-Specific Growth Recovery After Extreme Dry Events
#' in a Montane Rainforest in Southern Ecuador.
#' \emph{Frontiers in Plant Science}, 10, 342.
#' \doi{10.3389/fpls.2019.00342}
#'
#' @param df A data frame with the first column containing date-time stamps
#'   (\code{yyyy-mm-dd HH:MM:SS}, convertible to \code{POSIXct}) and subsequent
#'   columns containing dendrometer series.
#' @param Clim A data frame with the first column containing dates and the second
#'   column containing the climate variable of interest.
#' @param dailyValue Character. Daily aggregation statistic used when resampling
#'   dendrometer data. One of \code{"max"}, \code{"min"}, \code{"mean"}, or
#'   \code{"sum"}. Default is \code{"max"}.
#' @param thresholdClim Character string defining the climate threshold.
#'   Supported forms are:
#'   \itemize{
#'     \item \code{"<5"}, \code{"<=5"}, \code{">5"}, \code{">=5"}, \code{"==5"}
#'     \item \code{"between(5,10)"}
#'     \item \code{"5:10"}
#'   }
#' @param thresholdDays Character string defining the minimum duration threshold
#'   for consecutive adverse days. Must use a single-threshold form such as
#'   \code{">5"} or \code{">=3"}.
#' @param showPlot Logical. If \code{TRUE}, diagnostic plots are produced and the
#'   user can interactively choose IDs and phase type (\code{"adverse"},
#'   \code{"normal"}, or \code{"both"}). Default is \code{TRUE}.
#'
#' @return
#' A list of class \code{"clim_twd_output"} containing:
#' \describe{
#'   \item{adverse_change}{A data frame of relative dendrometer change during
#'   adverse phases only.}
#'   \item{normal_change}{A data frame of relative dendrometer change during the
#'   following normal phases only.}
#'   \item{full_period_change}{A data frame of relative dendrometer change across
#'   adverse + following normal phase, reset to 0 at each adverse start.}
#'   \item{continuous_full_period_change}{A data frame similar to
#'   \code{full_period_change}, but values continue from the end of the previous
#'   full period instead of resetting to 0.}
#'   \item{period_info}{A data frame with one row per ID and tree, containing
#'   adverse and normal phase boundaries, the date and magnitude of maximum
#'   adverse decline during the adverse phase, lags to first drop below zero, lags
#'   to maximum adverse decline, and lags to return to zero.}
#'   \item{phase_table}{A data frame with adverse and normal phase boundaries for
#'   each ID.}
#' }
#'
#' @note
#' If the dendrometer and climate datasets have different time spans, analysis is
#' automatically restricted to their common overlap.
#'
#' @seealso
#' \code{\link[dendRoAnalyst:dendro.resample]{dendro.resample}},
#' \code{\link{phase.zg}},
#' \code{\link{phase.sc}}
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_rain17)
#'
#' rel_out <- clim.twd(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   dailyValue = "max",
#'   thresholdClim = "<10",
#'   thresholdDays = ">5",
#'   showPlot = FALSE
#' )
#'
#' head(rel_out$adverse_change)
#' head(rel_out$normal_change)
#' head(rel_out$full_period_change)
#' head(rel_out$continuous_full_period_change)
#' head(rel_out$period_info)
#'
#' rel_out2 <- clim.twd(
#'   df = gf_nepa17,
#'   Clim = ktm_rain17,
#'   thresholdClim = "between(5,10)",
#'   thresholdDays = ">3",
#'   showPlot = FALSE
#' )
#' }
#'
#' @importFrom lubridate ymd_hms ymd
#' @importFrom dplyr mutate group_by summarise ungroup rename across select
#'   filter arrange bind_rows all_of left_join first transmute %>%
#' @importFrom tibble as_tibble tibble
#' @importFrom ggplot2 ggplot geom_area geom_rect geom_text aes theme_minimal
#'   labs geom_line geom_point facet_wrap theme element_text scale_fill_manual
#'
#' @export
clim.twd <- function(df, Clim, dailyValue = "max",
                     thresholdClim = "<10", thresholdDays = ">5",
                     showPlot = TRUE) {

  TIME <- DATE <- IDs <- climate <- phase_type <- Variable <- Value <- NULL
  adverse_ID <- adverse_start <- adverse_end <- has_normal_phase <- normal_start <- NULL
  normal_end <- xmin <- xmax <- NULL

  parse_threshold <- function(x, arg_name) {
    x <- trimws(x)

    if (grepl("^between\\s*\\(\\s*-?[0-9.]+\\s*,\\s*-?[0-9.]+\\s*\\)$",
              x, ignore.case = TRUE)) {
      nums <- gsub("^between\\s*\\(|\\)$", "", x, ignore.case = TRUE)
      nums <- strsplit(nums, ",")[[1]]
      nums <- as.numeric(trimws(nums))
      if (length(nums) != 2 || any(is.na(nums))) {
        stop("Invalid '", arg_name, "' range. Use for example 'between(5,10)'.")
      }
      lo <- min(nums)
      hi <- max(nums)
      return(list(type = "between", lower = lo, upper = hi))
    }

    if (grepl("^\\s*-?[0-9.]+\\s*:\\s*-?[0-9.]+\\s*$", x)) {
      nums <- strsplit(x, ":")[[1]]
      nums <- as.numeric(trimws(nums))
      if (length(nums) != 2 || any(is.na(nums))) {
        stop("Invalid '", arg_name, "' range. Use for example '5:10'.")
      }
      lo <- min(nums)
      hi <- max(nums)
      return(list(type = "between", lower = lo, upper = hi))
    }

    mm <- regexec("^\\s*(<=|>=|==|<|>)\\s*(-?[0-9.]+)\\s*$", x)
    rr <- regmatches(x, mm)[[1]]

    if (length(rr) == 3) {
      return(list(
        type = "single",
        op = rr[2],
        value = as.numeric(rr[3])
      ))
    }

    stop(
      "'", arg_name, "' must look like '<5', '<=5', '>5', '>=5', '==5', ",
      "'between(5,10)', or '5:10'."
    )
  }

  apply_threshold <- function(z, thr) {
    if (thr$type == "between") {
      return(z >= thr$lower & z <= thr$upper)
    }

    switch(
      thr$op,
      "<"  = z < thr$value,
      "<=" = z <= thr$value,
      ">"  = z > thr$value,
      ">=" = z >= thr$value,
      "==" = z == thr$value
    )
  }

  find_first_below_zero <- function(x) {
    ii <- which(x < 0)
    if (length(ii) == 0) return(NA_integer_)
    ii[1]
  }

  find_return_to_zero <- function(x, start_idx) {
    if (is.na(start_idx)) return(NA_integer_)
    ii <- which(seq_along(x) > start_idx & x >= 0)
    if (length(ii) == 0) return(NA_integer_)
    ii[1]
  }

  if (!inherits(df[[1]], "Date") && !inherits(df[[1]], "POSIXct")) {
    df[[1]] <- lubridate::ymd_hms(df[[1]])
  }
  if (!inherits(Clim[[1]], "Date") && !inherits(Clim[[1]], "POSIXct")) {
    Clim[[1]] <- lubridate::ymd(Clim[[1]])
  }

  df <- dendro.resample(df = df, by = "D", value = dailyValue)
  df <- df %>% dplyr::mutate(TIME = as.Date(TIME))

  clim <- tibble::as_tibble(Clim) %>%
    dplyr::rename(DATE = 1, climate = 2)

  ctime <- intersect(df$TIME, clim$DATE)
  if (length(ctime) == 0) {
    stop("The dendrometer and climate data do not have a common time period.")
  } else {
    if (!((length(ctime) == nrow(df)) && (length(ctime) == nrow(clim)))) {
      message("The analyses period is reduced to common time period.")
      common_t <- as.Date(ctime, origin = "1970-01-01", tz = "UTC")
      df   <- df   %>% dplyr::filter(TIME %in% common_t)
      clim <- clim %>% dplyr::filter(DATE %in% common_t)
    }
  }

  tree_cols <- names(df)[names(df) != "TIME"]

  thr_clim <- parse_threshold(thresholdClim, "thresholdClim")
  thr_days <- parse_threshold(thresholdDays, "thresholdDays")

  if (thr_days$type != "single") {
    stop("'thresholdDays' must use a single-threshold form such as '>5' or '>=3'.")
  }

  adverse_flag <- apply_threshold(clim$climate, thr_clim)

  rr <- rle(adverse_flag)
  adverse_ids <- integer(length(adverse_flag))
  adverse_start_flag <- rep(FALSE, length(adverse_flag))

  pos <- 1L
  id_counter <- 0L

  for (i in seq_along(rr$lengths)) {
    run_len <- rr$lengths[i]
    run_val <- rr$values[i]
    idx <- pos:(pos + run_len - 1L)

    keep_run <- isTRUE(run_val) && isTRUE(apply_threshold(run_len, thr_days))

    if (keep_run) {
      id_counter <- id_counter + 1L
      adverse_ids[idx] <- id_counter
      adverse_start_flag[idx[1]] <- TRUE
    }

    pos <- pos + run_len
  }

  clim <- clim %>%
    dplyr::mutate(
      adverse_start = adverse_start_flag,
      adverse_ID = adverse_ids
    )

  if (max(adverse_ids, na.rm = TRUE) == 0) {
    empty_out <- list(
      adverse_change = tibble::tibble(),
      normal_change = tibble::tibble(),
      full_period_change = tibble::tibble(),
      continuous_full_period_change = tibble::tibble(),
      period_info = tibble::tibble(),
      phase_table = tibble::tibble()
    )
    class(empty_out) <- "clim_twd_output"

    if (showPlot) {
      message("No adverse periods found for the given thresholds.")
      p1 <- ggplot2::ggplot(clim, ggplot2::aes(x = DATE, y = climate)) +
        ggplot2::geom_area(fill = "lightblue") +
        ggplot2::theme_minimal() +
        ggplot2::labs(
          title = "Climate plot",
          x = "Date",
          y = "Climate value"
        )
      print(p1)
    }

    return(empty_out)
  }

  phase_table <- clim %>%
    dplyr::filter(adverse_ID > 0) %>%
    dplyr::group_by(adverse_ID) %>%
    dplyr::summarise(
      adverse_start = min(DATE),
      adverse_end = max(DATE),
      n_days_adverse = dplyr::n(),
      .groups = "drop"
    ) %>%
    dplyr::arrange(adverse_ID) %>%
    dplyr::rename(IDs = adverse_ID)

  next_start <- c(phase_table$adverse_start[-1], as.Date(NA))
  series_last_date <- max(df$TIME, na.rm = TRUE)

  phase_table$normal_start <- phase_table$adverse_end + 1
  phase_table$full_end <- as.Date(ifelse(
    is.na(next_start),
    as.character(series_last_date),
    as.character(as.Date(next_start) - 1)
  ))
  phase_table$has_normal_phase <- phase_table$normal_start <= phase_table$full_end
  phase_table$normal_end <- ifelse(
    phase_table$has_normal_phase,
    as.character(phase_table$full_end),
    NA_character_
  )
  phase_table$normal_end <- as.Date(phase_table$normal_end)

  full_id <- integer(nrow(clim))
  phase_type_vec <- rep(NA_character_, nrow(clim))

  for (i in seq_len(nrow(phase_table))) {
    id_i <- phase_table$IDs[i]
    a_start <- phase_table$adverse_start[i]
    a_end <- phase_table$adverse_end[i]
    f_end <- phase_table$full_end[i]

    idx_full <- which(clim$DATE >= a_start & clim$DATE <= f_end)
    idx_adv  <- which(clim$DATE >= a_start & clim$DATE <= a_end)
    idx_norm <- which(clim$DATE > a_end & clim$DATE <= f_end)

    full_id[idx_full] <- id_i
    phase_type_vec[idx_adv] <- "adverse"
    if (length(idx_norm) > 0) phase_type_vec[idx_norm] <- "normal"
  }

  clim <- clim %>%
    dplyr::mutate(
      IDs = full_id,
      phase_type = phase_type_vec
    )

  df <- dplyr::left_join(
    df,
    clim %>% dplyr::select(DATE, IDs, phase_type),
    by = c("TIME" = "DATE")
  )

  full_period_change <- df %>%
    dplyr::filter(IDs > 0) %>%
    dplyr::arrange(IDs, TIME) %>%
    dplyr::group_by(IDs) %>%
    dplyr::mutate(
      dplyr::across(
        dplyr::all_of(tree_cols),
        ~ . - dplyr::first(.)
      )
    ) %>%
    dplyr::ungroup()

  adverse_change <- full_period_change %>%
    dplyr::filter(phase_type == "adverse")

  normal_change <- full_period_change %>%
    dplyr::filter(phase_type == "normal")

  continuous_list <- list()
  offset_vals <- stats::setNames(rep(0, length(tree_cols)), tree_cols)

  for (i in seq_len(nrow(phase_table))) {
    id_i <- phase_table$IDs[i]

    seg_df <- full_period_change %>%
      dplyr::filter(IDs == id_i) %>%
      dplyr::arrange(TIME)

    if (nrow(seg_df) == 0) next

    cont_df <- seg_df
    for (cc in tree_cols) {
      cont_df[[cc]] <- seg_df[[cc]] + offset_vals[[cc]]
      offset_vals[[cc]] <- cont_df[[cc]][nrow(cont_df)]
    }

    continuous_list[[i]] <- cont_df
  }

  continuous_full_period_change <- dplyr::bind_rows(continuous_list)

  period_info_list <- list()

  for (i in seq_len(nrow(phase_table))) {
    id_i <- phase_table$IDs[i]
    a_start <- phase_table$adverse_start[i]
    a_end <- phase_table$adverse_end[i]
    n_start <- phase_table$normal_start[i]
    n_end <- phase_table$normal_end[i]
    f_end <- phase_table$full_end[i]

    seg_df <- full_period_change %>%
      dplyr::filter(IDs == id_i) %>%
      dplyr::arrange(TIME)

    adv_df <- adverse_change %>%
      dplyr::filter(IDs == id_i) %>%
      dplyr::arrange(TIME)

    cont_df <- continuous_full_period_change %>%
      dplyr::filter(IDs == id_i) %>%
      dplyr::arrange(TIME)

    for (cc in tree_cols) {
      x_full <- seg_df[[cc]]
      t_full <- seg_df$TIME

      x_adv <- adv_df[[cc]]
      t_adv <- adv_df$TIME

      x_cont <- cont_df[[cc]]

      below_idx <- find_first_below_zero(x_full)
      return_idx <- find_return_to_zero(x_full, below_idx)

      if (length(x_adv) == 0 || all(is.na(x_adv))) {
        min_adv_idx <- NA_integer_
        min_adv_val <- NA_real_
      } else {
        min_adv_idx <- which.min(x_adv)
        min_adv_val <- x_adv[min_adv_idx]
      }

      no_adverse_tree <- is.na(below_idx)

      period_info_list[[length(period_info_list) + 1L]] <- tibble::tibble(
        IDs = id_i,
        tree = cc,
        adverse_start = a_start,
        adverse_end = a_end,
        normal_start = if (phase_table$has_normal_phase[i]) n_start else as.Date(NA),
        normal_end = if (phase_table$has_normal_phase[i]) n_end else as.Date(NA),
        full_end = f_end,
        no_adverse_for_tree = no_adverse_tree,
        first_below_zero_date = if (is.na(below_idx)) as.Date(NA) else as.Date(t_full[below_idx]),
        lag_to_below_zero = if (is.na(below_idx)) NA_integer_ else as.integer(t_full[below_idx] - a_start),
        max_adverse_decline_value = min_adv_val,
        max_adverse_decline_date = if (is.na(min_adv_idx)) as.Date(NA) else as.Date(t_adv[min_adv_idx]),
        lag_to_max_adverse_decline = if (is.na(min_adv_idx)) NA_integer_ else as.integer(t_adv[min_adv_idx] - a_start),
        return_to_zero_date = if (is.na(return_idx)) as.Date(NA) else as.Date(t_full[return_idx]),
        lag_to_return_zero = if (is.na(return_idx)) NA_integer_ else as.integer(t_full[return_idx] - a_start),
        change_end_adverse = if (length(x_adv) == 0) NA_real_ else x_adv[length(x_adv)],
        change_end_full_period = if (length(x_full) == 0) NA_real_ else x_full[length(x_full)],
        continuous_start_full_period = if (length(x_cont) == 0) NA_real_ else x_cont[1],
        continuous_end_full_period = if (length(x_cont) == 0) NA_real_ else x_cont[length(x_cont)]
      )
    }
  }

  period_info <- dplyr::bind_rows(period_info_list)

  if (showPlot) {
    phase_plot_df <- dplyr::bind_rows(
      phase_table %>%
        dplyr::transmute(
          IDs,
          xmin = adverse_start,
          xmax = adverse_end,
          phase_type = "adverse"
        ),
      phase_table %>%
        dplyr::filter(has_normal_phase) %>%
        dplyr::transmute(
          IDs,
          xmin = normal_start,
          xmax = normal_end,
          phase_type = "normal"
        )
    )

    p1 <- ggplot2::ggplot(clim, ggplot2::aes(x = DATE, y = climate, group = 1)) +
      ggplot2::geom_area(fill = "lightblue") +
      ggplot2::geom_rect(
        data = phase_plot_df,
        ggplot2::aes(xmin = xmin, xmax = xmax, ymin = -Inf, ymax = Inf, fill = phase_type),
        inherit.aes = FALSE,
        alpha = 0.18
      ) +
      ggplot2::geom_text(
        data = phase_table,
        ggplot2::aes(x = adverse_start, y = max(clim$climate, na.rm = TRUE), label = IDs),
        inherit.aes = FALSE
      ) +
      ggplot2::scale_fill_manual(values = c(adverse = "firebrick2", normal = "darkseagreen3")) +
      ggplot2::theme_minimal() +
      ggplot2::labs(
        title = "Climate plot with adverse and following normal phases",
        x = "Date",
        y = "Climate value",
        fill = "Phase"
      )
    print(p1)

    print("The period IDs are listed below:")
    print(unique(full_period_change$IDs))
    print("Please choose period IDs from the list. Press Enter twice when done.")
    perd <- scan()

    if (length(perd) == 0) {
      perd <- phase_table$IDs[1]
    }

    ptype <- readline("Choose phase type: 'adverse', 'normal', or 'both' [both]: ")
    if (ptype == "") ptype <- "both"
    ptype <- tolower(trimws(ptype))

    if (!(ptype %in% c("adverse", "normal", "both"))) {
      warning("Invalid phase type. Using 'both'.")
      ptype <- "both"
    }

    if (ptype == "adverse") {
      fdf <- adverse_change %>% dplyr::filter(IDs %in% perd)
    } else if (ptype == "normal") {
      fdf <- normal_change %>% dplyr::filter(IDs %in% perd)
    } else {
      fdf <- full_period_change %>% dplyr::filter(IDs %in% perd)
    }

    if (nrow(fdf) == 0) {
      message("No rows available for the selected IDs and phase type.")
    } else {
      data_long <- tidyr::pivot_longer(
        fdf,
        cols = dplyr::all_of(tree_cols),
        names_to = "Variable",
        values_to = "Value"
      )

      p2 <- ggplot2::ggplot(data_long, ggplot2::aes(x = TIME, y = Value, color = Variable))

      if (ptype == "both") {
        phase_plot_df2 <- dplyr::bind_rows(
          phase_table %>%
            dplyr::filter(IDs %in% perd) %>%
            dplyr::transmute(
              IDs,
              xmin = adverse_start,
              xmax = adverse_end,
              phase_type = "adverse"
            ),
          phase_table %>%
            dplyr::filter(IDs %in% perd, has_normal_phase) %>%
            dplyr::transmute(
              IDs,
              xmin = normal_start,
              xmax = normal_end,
              phase_type = "normal"
            )
        )

        p2 <- p2 +
          ggplot2::geom_rect(
            data = phase_plot_df2,
            ggplot2::aes(xmin = xmin, xmax = xmax, ymin = -Inf, ymax = Inf, fill = phase_type),
            inherit.aes = FALSE,
            alpha = 0.12
          ) +
          ggplot2::scale_fill_manual(values = c(adverse = "firebrick2", normal = "darkseagreen3"))
      }

      p2 <- p2 +
        ggplot2::geom_line() +
        ggplot2::geom_point() +
        ggplot2::labs(
          title = paste("Relative dendrometer change:", ptype, "phase"),
          x = "Time",
          y = "Relative change from adverse-phase start",
          color = "Trees",
          fill = "Phase"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          legend.position = "bottom",
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12)
        )

      if (length(perd) > 1) {
        p2 <- p2 + ggplot2::facet_wrap(~ IDs, scales = "free_x", ncol = 1)
      }

      print(p2)
    }
  }

  out <- list(
    adverse_change = adverse_change,
    normal_change = normal_change,
    full_period_change = full_period_change,
    continuous_full_period_change = continuous_full_period_change,
    period_info = period_info,
    phase_table = phase_table
  )

  class(out) <- "clim_twd_output"
  out
}

#' @title Interpolate missing dendrometer values using a site network
#'
#' @description
#' Fills \code{NA} gaps in one or more focal dendrometer series by borrowing
#' information from a reference network recorded at the same site and on the
#' same time grid.
#'
#' The focal dataset \code{df} is treated as the master time axis. The
#' reference network is aligned to that axis, so missing timestamps already
#' inserted into \code{df} are preserved during interpolation.
#'
#' Two interpolation modes are available:
#' \itemize{
#'   \item \code{niMethod = "proportional"} uses the average relative change in
#'   the reference network between consecutive timestamps and propagates that
#'   change to the focal series.
#'   \item \code{niMethod = "linear"} fits a cross-sensor regression between
#'   reference values at time \eqn{t-1} and \eqn{t}, and predicts the focal
#'   value at time \eqn{t} from the focal value at time \eqn{t-1}.
#' }
#'
#' Optional bootstrap-based uncertainty limits can be attached, synthetic-gap
#' validation can be used to assess recovery performance, and an optional
#' post-processing step can detect and correct abrupt post-gap jumps in the
#' interpolated series.
#'
#' @details
#' The function assumes that missing timestamps have already been inserted into
#' \code{df} and corresponding measurements set to \code{NA}, for example with
#' \code{\link{dm.na.interpolation}}.
#'
#' For a focal series \eqn{a}, let \eqn{a_{t-1}} be the last available focal
#' value before the current timestamp. Let \eqn{b_{t-1}} and \eqn{b_t} be the
#' vectors of reference values at the previous and current timestamps.
#'
#' In proportional mode, the relative reference change is
#' \deqn{
#'   \delta = \mathrm{mean}\left(\frac{b_t - b_{t-1}}{\max(b_{t-1}, \epsilon)}\right)
#' }
#' and the focal estimate is
#' \deqn{
#'   \hat{a}_t = a_{t-1}(1 + \delta).
#' }
#' Bootstrap resampling of reference sensors is used to estimate a 95\%
#' interval.
#'
#' In linear mode, a simple regression of \eqn{b_t} on \eqn{b_{t-1}} is fit
#' across reference sensors at each step, and the focal value is predicted from
#' \eqn{a_{t-1}} with a 95\% prediction interval.
#'
#' If fewer than two valid reference sensors are available at a timestamp pair,
#' or if the focal previous value is missing, the focal value is left unchanged.
#'
#' If \code{correct_gap_jumps = TRUE}, the function inspects the first observed
#' point immediately after each imputed run. If a jump larger than
#' \code{jump_threshold} is detected, the estimated offset is removed from that
#' point onward. This is useful when a dendrometer resumes from a shifted
#' baseline after a gap.
#'
#' If \code{assess = TRUE}, the function inserts artificial gaps into observed
#' sections of each focal series, interpolates them with the chosen method, and
#' compares predictions with the true values. This produces both per-gap
#' summaries and seasonal diagnostics of interpolation error.
#'
#' The returned object is a data frame with class
#' \code{"network_interpolation"}.
#'
#' @param df Data frame containing focal dendrometer data. The first column must
#'   be datetime (\code{POSIXct}, \code{Date}, or character convertible by
#'   \code{lubridate::ymd_hms}). Remaining columns are focal series.
#' @param referenceDF Data frame containing reference dendrometer data. The
#'   first column must be datetime, and remaining columns are reference series.
#' @param niMethod Character. Interpolation method. One of
#'   \code{"proportional"} or \code{"linear"}.
#' @param n_boot Integer. Number of bootstrap resamples used in proportional
#'   mode.
#' @param return_flags Logical. If \code{TRUE}, add a logical column
#'   \code{<series>_interp} indicating which focal values were imputed.
#' @param return_pi Logical. If \code{TRUE}, add 95\% prediction-limit columns
#'   \code{<series>_pi_lo} and \code{<series>_pi_hi}.
#' @param pi_for_all Logical. If \code{TRUE}, prediction limits are also
#'   attached for non-imputed rows where the model could be formed. If
#'   \code{FALSE}, limits are returned only for imputed rows.
#' @param return_fit Logical. If \code{TRUE}, add \code{<series>_fit} columns
#'   containing the model-implied fitted value.
#' @param assess Logical. If \code{TRUE}, run synthetic-gap validation.
#' @param assess_lengths_steps Integer vector. Artificial gap lengths,
#'   expressed in number of rows/time steps, to test during validation.
#' @param assess_samples_per_length Integer. Number of sampled artificial gaps
#'   per gap length and per focal series.
#' @param assess_buffer_steps Integer. Number of observed steps required before
#'   and after each artificial gap when
#'   \code{assess_use_only_observed = TRUE}.
#' @param assess_seed Integer or \code{NULL}. Random seed for reproducible
#'   validation sampling.
#' @param assess_use_only_observed Logical. If \code{TRUE}, synthetic gaps are
#'   sampled only from windows fully observed in the focal series, including
#'   the requested buffer.
#' @param progress Logical. If \code{TRUE}, display a single-line progress
#'   indicator.
#' @param correct_gap_jumps Logical. If \code{TRUE}, inspect the first observed
#'   point after each imputed run and optionally remove a detected jump.
#' @param jump_threshold Numeric. Minimum absolute jump size required before a
#'   post-gap correction is applied. Required if
#'   \code{correct_gap_jumps = TRUE}.
#' @param jump_adjustment_method Character. Method used to estimate the
#'   post-gap offset. One of \code{"window_median"} or \code{"point_diff"}.
#' @param jump_adjust_window Integer. Window size used by
#'   \code{jump_adjustment_method = "window_median"}.
#' @param jump_set_point_na Logical. If \code{TRUE}, the jump point itself is
#'   set to \code{NA} after correction.
#'
#' @return
#' A data frame with class \code{"network_interpolation"}.
#'
#' The data frame contains the original \code{TIME} column, focal series
#' columns, and optionally:
#' \describe{
#'   \item{\code{<series>_interp}}{Logical flag for imputed rows.}
#'   \item{\code{<series>_pi_lo}, \code{<series>_pi_hi}}{95\% prediction
#'   limits.}
#'   \item{\code{<series>_fit}}{Model-implied fitted values.}
#'   \item{\code{<series>_jump_correction}}{Numeric jump offset removed at the
#'   first observed point after an imputed run. \code{NA} means no correction
#'   was applied.}
#' }
#'
#' In addition, the following attributes are attached:
#' \describe{
#'   \item{\code{network_original}}{Original focal data on the aligned time
#'   grid.}
#'   \item{\code{network_reference}}{Reference data aligned to the focal time
#'   grid.}
#'   \item{\code{network_diagnostics}}{Long-format diagnostics for plotting.}
#'   \item{\code{network_summary}}{Per-series interpolation summary table.}
#'   \item{\code{network_jump_corrections}}{Table of inspected and corrected
#'   post-gap jumps.}
#'   \item{\code{network_validation_raw}}{Per-gap validation summary table, or
#'   \code{NULL}.}
#'   \item{\code{network_validation_points}}{Point-level validation results, or
#'   \code{NULL}.}
#'   \item{\code{network_validation_summary}}{Summarized validation metrics by
#'   series and gap length, or \code{NULL}.}
#'   \item{\code{network_validation_seasonal}}{Seasonal validation diagnostics
#'   by day of year, or \code{NULL}.}
#'   \item{\code{network_method}}{Interpolation method used.}
#'   \item{\code{network_n_boot}}{Number of bootstrap resamples used.}
#'   \item{\code{network_assessed}}{Logical indicating whether validation was
#'   run.}
#'   \item{\code{network_jump_settings}}{List of jump-correction settings.}
#' }
#'
#' @section Interpretation:
#' Smaller validation metrics such as \code{MAE}, \code{RMSE}, \code{MAPE}, and
#' \code{MdAPE} indicate better gap recovery. \code{PI_coverage_95} indicates
#' how well the nominal 95\% uncertainty interval captures the true values
#' under synthetic-gap validation. Seasonal diagnostics help identify periods
#' of the year when network interpolation is more or less reliable.
#'
#' If many post-gap corrections are applied or \code{max_abs_gap_jump} is
#' large, it may indicate re-zeroing, sensor reset, or baseline
#' discontinuities after gaps.
#'
#' @section Notes:
#' \itemize{
#'   \item The focal dataset \code{df} defines the output time grid.
#'   \item Reference series are aligned to that grid using the datetime column.
#'   \item At least two valid reference sensors are required at consecutive
#'   timestamps for interpolation.
#'   \item Jump correction is a post-processing step and does not alter the
#'   core interpolation logic.
#' }
#'
#' @examples
#' \donttest{
#' #library(dendRoAnalyst)
#' #data("gf_nepa17")
#'
#' ## Create an artificial focal gap
#' #df1 <- gf_nepa17
#' #df1[40:50, "T2"] <- NA
#'
#' ## Build a small reference network
#' #ref <- cbind(gf_nepa17, gf_nepa17[, 2:3], gf_nepa17[, 2:3])
#' #colnames(ref) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")
#'
#' ## Interpolate with uncertainty limits
#' #out <- network.interpolation(
#' #  df1, ref,
#' #  niMethod = "proportional",
#' #  n_boot = 100,
#' #  return_flags = TRUE,
#' #  return_pi = TRUE
#' #)
#'
#' #head(out, 10)
#'
#' # Interpolate with jump correction and validation
#' #out2 <- network.interpolation(
#' #  df1, ref,
#' #  niMethod = "proportional",
#' #  n_boot = 100,
#' #  return_flags = TRUE,
#' #  return_pi = TRUE,
#' #  assess = TRUE,
#' #  assess_lengths_steps = c(1, 2, 4),
#' #  correct_gap_jumps = TRUE,
#' #  jump_threshold = 0.05,
#' #  jump_adjustment_method = "window_median",
#' #  jump_adjust_window = 5
#' #)
#' # Plotting
#' #plot(out2)
#' #plot(out2, type = "compare")
#' #plot(out2, type = "seasonal_error")
#'
#' # Extracting the information using attr()
#' #attr(out, "network_validation_summary")
#' #attr(out, "network_validation_points")
#' #attr(out, "network_validation_seasonal")
#' }
#'
#' @seealso
#' \code{\link{plot.network_interpolation}},
#' \code{\link{dm.na.interpolation}}
#'
#' @export
network.interpolation <- function(df, referenceDF,
                                  niMethod = c("proportional", "linear"),
                                  n_boot = 1000,
                                  return_flags = TRUE,
                                  return_pi = TRUE,
                                  pi_for_all = FALSE,
                                  return_fit = FALSE,
                                  assess = FALSE,
                                  assess_lengths_steps = c(1, 2, 3, 6, 12, 24),
                                  assess_samples_per_length = 20,
                                  assess_buffer_steps = 1,
                                  assess_seed = NULL,
                                  assess_use_only_observed = TRUE,
                                  progress = interactive(),
                                  correct_gap_jumps = FALSE,
                                  jump_threshold = NULL,
                                  jump_adjustment_method = c("window_median", "point_diff"),
                                  jump_adjust_window = 5,
                                  jump_set_point_na = FALSE) {

  niMethod <- match.arg(niMethod)
  jump_adjustment_method <- match.arg(jump_adjustment_method)

  if (isTRUE(correct_gap_jumps) &&
      (is.null(jump_threshold) || !is.finite(jump_threshold) || jump_threshold <= 0)) {
    stop("When correct_gap_jumps = TRUE, provide a positive numeric 'jump_threshold'.")
  }

  Bias <- End_value_abs_diff <- MAE <- MAPE <- Max_abs_diff_within_gap <- MdAPE <- Mean_PI_width <- Mean_ref_n <- PI_coverage_95 <- RMSE <- Success_rate <- TIME <- abs_pct_error <- applied_offset <- case_id <- covered <- doy <- error <- filled <- gap_steps <- jump_correction <- jump_removed <- jump_time <- max_abs_gap_jump <- mean_abs_error <- mean_abs_pct_error <- mean_error <- mean_pred <- mean_truth <- n_gap_jumps_removed <- pred <- raw_jump <- ref_n <- rmse <- series <- start_time <- end_time <- truth <- NULL

  to_posixct <- function(x) {
    if (inherits(x, "POSIXct")) return(x)
    if (inherits(x, "Date")) return(as.POSIXct(x))
    lubridate::ymd_hms(x)
  }

  mean_or_na <- function(x) {
    if (!length(x) || all(is.na(x))) return(NA_real_)
    mean(x, na.rm = TRUE)
  }

  median_or_na <- function(x) {
    if (!length(x) || all(is.na(x))) return(NA_real_)
    stats::median(x, na.rm = TRUE)
  }

  boot_mean_delta_fast <- function(delta, n_boot) {
    delta <- delta[is.finite(delta)]
    n <- length(delta)

    if (n < 2L) {
      return(list(
        mean = if (n) mean(delta) else NA_real_,
        lo = NA_real_,
        hi = NA_real_
      ))
    }

    idx <- matrix(sample.int(n, size = n * n_boot, replace = TRUE), nrow = n)
    means <- colMeans(matrix(delta[idx], nrow = n))

    list(
      mean = mean(means, na.rm = TRUE),
      lo = stats::quantile(means, 0.025, na.rm = TRUE, names = FALSE),
      hi = stats::quantile(means, 0.975, na.rm = TRUE, names = FALSE)
    )
  }

  fast_linear_pred <- function(x, y, x0, level = 0.95) {
    ok <- is.finite(x) & is.finite(y)
    x <- x[ok]
    y <- y[ok]

    n <- length(x)
    if (n < 2L || !is.finite(x0)) {
      return(c(fit = NA_real_, lwr = NA_real_, upr = NA_real_))
    }

    xbar <- mean(x)
    ybar <- mean(y)
    sxx <- sum((x - xbar)^2)

    if (!is.finite(sxx) || sxx <= 0) {
      return(c(fit = NA_real_, lwr = NA_real_, upr = NA_real_))
    }

    beta <- sum((x - xbar) * (y - ybar)) / sxx
    alpha <- ybar - beta * xbar
    fit <- alpha + beta * x0

    if (n <= 2L) {
      return(c(fit = fit, lwr = NA_real_, upr = NA_real_))
    }

    resid <- y - (alpha + beta * x)
    mse <- sum(resid^2) / (n - 2L)

    if (!is.finite(mse)) {
      return(c(fit = fit, lwr = NA_real_, upr = NA_real_))
    }

    se_pred <- sqrt(mse * (1 + 1 / n + ((x0 - xbar)^2 / sxx)))
    tval <- stats::qt((1 + level) / 2, df = n - 2L)

    c(
      fit = fit,
      lwr = fit - tval * se_pred,
      upr = fit + tval * se_pred
    )
  }

  estimate_jump_offset_gap <- function(x, j,
                                       adjustment_method = c("window_median", "point_diff"),
                                       adjust_window = 5) {
    adjustment_method <- match.arg(adjustment_method)

    if (length(x) == 0 || j <= 1 || j > length(x)) {
      return(NA_real_)
    }

    if (adjustment_method == "point_diff") {
      if (is.na(x[j]) || is.na(x[j - 1])) {
        return(NA_real_)
      }
      return(x[j] - x[j - 1])
    }

    n <- length(x)

    pre_idx <- seq.int(max(1, j - adjust_window), j - 1)
    post_idx <- seq.int(j, min(n, j + adjust_window - 1))

    pre_vals <- x[pre_idx]
    post_vals <- x[post_idx]

    pre_vals <- pre_vals[!is.na(pre_vals)]
    post_vals <- post_vals[!is.na(post_vals)]

    if (length(pre_vals) < 2 || length(post_vals) < 2) {
      if (is.na(x[j]) || is.na(x[j - 1])) {
        return(NA_real_)
      }
      return(x[j] - x[j - 1])
    }

    stats::median(post_vals) - stats::median(pre_vals)
  }

  apply_jump_correction_gap <- function(x, j, offset, set_jump_point_na = FALSE) {
    if (length(x) == 0 || is.na(offset) || j > length(x)) {
      return(x)
    }

    idx <- j:length(x)
    idx <- idx[!is.na(x[idx])]

    if (length(idx) > 0) {
      x[idx] <- x[idx] - offset
    }

    if (isTRUE(set_jump_point_na) && j <= length(x)) {
      x[j] <- NA_real_
    }

    x
  }

  correct_gap_end_jumps <- function(value, fit, pi_lo, pi_hi,
                                    interp_flag, original_values, time_vec,
                                    jump_threshold,
                                    adjustment_method = c("window_median", "point_diff"),
                                    adjust_window = 5,
                                    set_jump_point_na = FALSE) {
    adjustment_method <- match.arg(adjustment_method)

    n <- length(value)
    jump_correction <- rep(NA_real_, n)

    if (!any(interp_flag, na.rm = TRUE)) {
      return(list(
        value = value,
        fit = fit,
        pi_lo = pi_lo,
        pi_hi = pi_hi,
        jump_correction = jump_correction,
        table = tibble::tibble()
      ))
    }

    r <- rle(interp_flag)
    ends <- cumsum(r$lengths)
    gap_end_idx <- ends[r$values]
    candidate_idx <- gap_end_idx + 1L
    candidate_idx <- candidate_idx[candidate_idx <= n]

    if (!length(candidate_idx)) {
      return(list(
        value = value,
        fit = fit,
        pi_lo = pi_lo,
        pi_hi = pi_hi,
        jump_correction = jump_correction,
        table = tibble::tibble()
      ))
    }

    rows <- vector("list", length(candidate_idx))
    rr <- 0L

    for (j in candidate_idx) {
      if (!is.finite(original_values[j])) next

      raw_jump <- if (j > 1 && is.finite(value[j]) && is.finite(value[j - 1])) {
        value[j] - value[j - 1]
      } else {
        NA_real_
      }

      offset <- estimate_jump_offset_gap(
        x = value,
        j = j,
        adjustment_method = adjustment_method,
        adjust_window = adjust_window
      )

      removed <- is.finite(offset) && abs(offset) >= jump_threshold

      if (removed) {
        value <- apply_jump_correction_gap(value, j, offset, set_jump_point_na = set_jump_point_na)
        fit   <- apply_jump_correction_gap(fit,   j, offset, set_jump_point_na = FALSE)
        pi_lo <- apply_jump_correction_gap(pi_lo, j, offset, set_jump_point_na = FALSE)
        pi_hi <- apply_jump_correction_gap(pi_hi, j, offset, set_jump_point_na = FALSE)
        jump_correction[j] <- offset
      }

      rr <- rr + 1L
      rows[[rr]] <- tibble::tibble(
        jump_index = j,
        jump_time = time_vec[j],
        raw_jump = raw_jump,
        estimated_offset = offset,
        applied_offset = if (removed) offset else NA_real_,
        jump_removed = removed
      )
    }

    jump_table <- if (rr > 0L) dplyr::bind_rows(rows[seq_len(rr)]) else tibble::tibble()

    list(
      value = value,
      fit = fit,
      pi_lo = pi_lo,
      pi_hi = pi_hi,
      jump_correction = jump_correction,
      table = jump_table
    )
  }

  interpolate_one_series <- function(a_raw, ref_mat, ref_ok, niMethod, n_boot, tiny = 1e-6) {
    n <- length(a_raw)

    a <- as.numeric(a_raw)
    interp_flag <- rep(FALSE, n)
    ref_n <- rep(NA_integer_, n)
    fit <- rep(NA_real_, n)
    pi_lo <- rep(NA_real_, n)
    pi_hi <- rep(NA_real_, n)

    for (i in 2:n) {
      a_prev <- a[i - 1L]
      if (!is.finite(a_prev)) next

      ok <- ref_ok[i - 1L, ] & ref_ok[i, ]
      n_ok <- sum(ok)
      ref_n[i] <- n_ok

      if (n_ok < 2L) next

      b_prev <- ref_mat[i - 1L, ok]
      b_cur  <- ref_mat[i, ok]

      if (niMethod == "proportional") {
        rel <- (b_cur - b_prev) / pmax(b_prev, tiny)
        bm <- boot_mean_delta_fast(rel, n_boot)

        if (!is.finite(bm$mean)) next

        fit_i <- a_prev * (1 + bm$mean)
        lo_i  <- if (is.na(bm$lo)) NA_real_ else a_prev * (1 + bm$lo)
        hi_i  <- if (is.na(bm$hi)) NA_real_ else a_prev * (1 + bm$hi)

      } else {
        pred <- fast_linear_pred(b_prev, b_cur, a_prev, level = 0.95)
        fit_i <- unname(pred["fit"])
        lo_i  <- unname(pred["lwr"])
        hi_i  <- unname(pred["upr"])

        if (!is.finite(fit_i)) next
      }

      fit[i] <- fit_i
      pi_lo[i] <- lo_i
      pi_hi[i] <- hi_i

      if (is.na(a[i]) && is.finite(fit_i)) {
        a[i] <- fit_i
        interp_flag[i] <- TRUE
      }
    }

    list(
      value = a,
      interp_flag = interp_flag,
      ref_n = ref_n,
      fit = fit,
      pi_lo = pi_lo,
      pi_hi = pi_hi
    )
  }

  df <- tibble::as_tibble(df)
  referenceDF <- tibble::as_tibble(referenceDF)

  names(df)[1] <- "TIME"
  names(referenceDF)[1] <- "TIME"

  df$TIME <- to_posixct(df$TIME)
  referenceDF$TIME <- to_posixct(referenceDF$TIME)

  if (any(is.na(df$TIME))) {
    stop("Could not convert the first column of 'df' to POSIXct.")
  }
  if (any(is.na(referenceDF$TIME))) {
    stop("Could not convert the first column of 'referenceDF' to POSIXct.")
  }

  df <- df[order(df$TIME), , drop = FALSE]
  referenceDF <- referenceDF[order(referenceDF$TIME), , drop = FALSE]

  time_min <- min(df$TIME, na.rm = TRUE)
  time_max <- max(df$TIME, na.rm = TRUE)

  referenceDF <- referenceDF[
    referenceDF$TIME >= time_min & referenceDF$TIME <= time_max,
    ,
    drop = FALSE
  ]

  if (nrow(referenceDF) == 0) {
    stop("'referenceDF' has no overlap with the time span of 'df'.")
  }

  referenceDF <- dplyr::left_join(df["TIME"], referenceDF, by = "TIME")

  if (ncol(referenceDF) < 2L) {
    stop("'referenceDF' must contain at least one reference series.")
  }

  ref_mat <- as.matrix(referenceDF[, -1, drop = FALSE])
  suppressWarnings(storage.mode(ref_mat) <- "double")
  ref_ok <- is.finite(ref_mat)

  if (!any(ref_ok)) {
    stop("No usable reference values remain after aligning 'referenceDF' to the time grid of 'df'.")
  }

  original_df <- df
  out <- df

  add_cols <- list()
  diag_list <- vector("list", max(1L, ncol(df) - 1L))
  sum_list <- vector("list", max(1L, ncol(df) - 1L))
  jump_tables <- vector("list", max(1L, ncol(df) - 1L))

  n_series <- max(1L, ncol(df) - 1L)
  progress_total <- n_series + if (assess) n_series * length(assess_lengths_steps) else 0L
  progress_value <- 0L
  .progress_last_width <- 0L

  render_progress_line <- function(note = "") {
    if (!isTRUE(progress) || progress_total <= 0L) return(invisible(NULL))

    width_bar <- 30L
    frac <- progress_value / progress_total
    n_done <- floor(width_bar * frac)
    bar <- paste0(
      "[",
      paste(rep("=", n_done), collapse = ""),
      paste(rep("-", width_bar - n_done), collapse = ""),
      "]"
    )

    txt <- sprintf(
      "\r%s %3d%% (%d/%d) %s",
      bar,
      round(100 * frac),
      progress_value,
      progress_total,
      note
    )

    pad <- max(0L, .progress_last_width - nchar(txt, type = "width"))
    cat(txt, paste(rep(" ", pad), collapse = ""))
    utils::flush.console()

    .progress_last_width <<- nchar(txt, type = "width")
    invisible(NULL)
  }

  update_progress <- function(step = 1L, note = "") {
    progress_value <<- min(progress_total, progress_value + step)
    render_progress_line(note)
    invisible(NULL)
  }

  progress_note <- function(note = "") {
    render_progress_line(note)
    invisible(NULL)
  }

  finish_progress <- function() {
    if (!isTRUE(progress)) return(invisible(NULL))
    cat("\n")
    utils::flush.console()
    invisible(NULL)
  }

  if (isTRUE(progress) && progress_total > 0L) {
    on.exit(finish_progress(), add = TRUE)
    render_progress_line("Starting...")
  }

  for (j in seq.int(2, ncol(df))) {
    colname <- names(df)[j]
    progress_note(paste0("Interpolating series ", colname, "..."))

    raw_a <- df[[j]]

    res <- interpolate_one_series(
      a_raw = raw_a,
      ref_mat = ref_mat,
      ref_ok = ref_ok,
      niMethod = niMethod,
      n_boot = n_boot
    )

    jump_correction <- rep(NA_real_, length(raw_a))
    jump_tbl <- tibble::tibble()

    if (isTRUE(correct_gap_jumps)) {
      jc <- correct_gap_end_jumps(
        value = res$value,
        fit = res$fit,
        pi_lo = res$pi_lo,
        pi_hi = res$pi_hi,
        interp_flag = res$interp_flag,
        original_values = raw_a,
        time_vec = df$TIME,
        jump_threshold = jump_threshold,
        adjustment_method = jump_adjustment_method,
        adjust_window = jump_adjust_window,
        set_jump_point_na = jump_set_point_na
      )

      res$value <- jc$value
      res$fit <- jc$fit
      res$pi_lo <- jc$pi_lo
      res$pi_hi <- jc$pi_hi
      jump_correction <- jc$jump_correction

      if (nrow(jc$table) > 0) {
        jump_tbl <- jc$table
        jump_tbl$series <- colname
      }
    }

    jump_tables[[j - 1L]] <- jump_tbl

    out[[j]] <- res$value

    if (return_flags) {
      add_cols[[paste0(colname, "_interp")]] <- res$interp_flag
    }

    if (return_pi) {
      out_pi_lo <- ifelse(res$interp_flag | pi_for_all, res$pi_lo, NA_real_)
      out_pi_hi <- ifelse(res$interp_flag | pi_for_all, res$pi_hi, NA_real_)
      add_cols[[paste0(colname, "_pi_lo")]] <- out_pi_lo
      add_cols[[paste0(colname, "_pi_hi")]] <- out_pi_hi
    }

    if (return_fit) {
      out_fit <- ifelse(res$interp_flag | pi_for_all, res$fit, NA_real_)
      add_cols[[paste0(colname, "_fit")]] <- out_fit
    }

    if (isTRUE(correct_gap_jumps)) {
      add_cols[[paste0(colname, "_jump_correction")]] <- jump_correction
    }

    diag_list[[j - 1L]] <- tibble::tibble(
      TIME = df$TIME,
      series = colname,
      original = raw_a,
      value = res$value,
      observed = is.finite(raw_a),
      interp = res$interp_flag,
      remaining_na = is.na(res$value),
      ref_n = res$ref_n,
      fit = res$fit,
      pi_lo = res$pi_lo,
      pi_hi = res$pi_hi,
      pi_width = res$pi_hi - res$pi_lo,
      jump_correction = jump_correction
    )

    sum_list[[j - 1L]] <- tibble::tibble(
      series = colname,
      n_rows = length(raw_a),
      n_missing_input = sum(is.na(raw_a)),
      n_imputed = sum(res$interp_flag),
      n_remaining_na = sum(is.na(res$value)),
      mean_ref_n = mean_or_na(res$ref_n),
      median_ref_n = median_or_na(res$ref_n),
      mean_pi_width = mean_or_na((res$pi_hi - res$pi_lo)[res$interp_flag]),
      median_pi_width = median_or_na((res$pi_hi - res$pi_lo)[res$interp_flag]),
      n_gap_jumps_removed = sum(!is.na(jump_correction)),
      max_abs_gap_jump = if (all(is.na(jump_correction))) NA_real_ else max(abs(jump_correction), na.rm = TRUE)
    )

    update_progress(note = paste0("Finished series ", colname))
  }

  if (length(add_cols)) {
    out <- dplyr::bind_cols(out, tibble::as_tibble(add_cols))
  }

  diagnostics <- dplyr::bind_rows(diag_list)
  summary_tbl <- dplyr::bind_rows(sum_list)
  jump_corrections <- dplyr::bind_rows(jump_tables)

  validation_raw <- NULL
  validation_points <- NULL
  validation_summary <- NULL
  validation_seasonal <- NULL

  if (assess) {
    if (!is.null(assess_seed)) {
      set.seed(assess_seed)
    }

    max_cases <- n_series * length(assess_lengths_steps) * assess_samples_per_length
    max_points <- max_cases * max(assess_lengths_steps)

    val_rows <- vector("list", max(1L, max_cases))
    point_rows <- vector("list", max(1L, max_points))
    val_id <- 0L
    point_id <- 0L

    push_val_row <- function(row) {
      val_id <<- val_id + 1L
      if (val_id > length(val_rows)) {
        length(val_rows) <<- max(length(val_rows) * 2L, val_id)
      }
      val_rows[[val_id]] <<- row
    }

    push_point_row <- function(row) {
      point_id <<- point_id + 1L
      if (point_id > length(point_rows)) {
        length(point_rows) <<- max(length(point_rows) * 2L, point_id)
      }
      point_rows[[point_id]] <<- row
    }

    for (j in seq.int(2, ncol(df))) {
      colname <- names(df)[j]
      raw_a <- as.numeric(df[[j]])
      n <- length(raw_a)

      if (n < 2L) {
        update_progress(length(assess_lengths_steps), note = paste0("Skipping validation for ", colname))
        next
      }

      finite_raw <- is.finite(raw_a)
      cs_finite <- c(0L, cumsum(finite_raw))

      for (gap_steps in as.integer(assess_lengths_steps)) {
        progress_note(paste0("Validating ", colname, " | gap=", gap_steps, " step(s)"))

        if (!is.finite(gap_steps) || gap_steps < 1L) {
          update_progress(note = paste0("Skipped ", colname, " | gap=", gap_steps))
          next
        }

        min_start <- 2L + assess_buffer_steps
        max_start <- n - gap_steps - assess_buffer_steps + 1L

        if (max_start >= min_start) {
          starts <- seq.int(min_start, max_start)

          gap_counts <- cs_finite[starts + gap_steps] - cs_finite[starts]
          prev_ok <- finite_raw[starts - 1L]

          valid <- (gap_counts == gap_steps) & prev_ok

          if (assess_use_only_observed) {
            win_left <- starts - assess_buffer_steps
            win_right <- starts + gap_steps - 1L + assess_buffer_steps
            win_counts <- cs_finite[win_right + 1L] - cs_finite[win_left]
            valid <- valid & (win_counts == (gap_steps + 2L * assess_buffer_steps))
          }

          valid_starts <- starts[valid]

          if (length(valid_starts)) {
            starts_to_use <- if (length(valid_starts) > assess_samples_per_length) {
              sample(valid_starts, assess_samples_per_length, replace = FALSE)
            } else {
              valid_starts
            }

            for (s in starts_to_use) {
              idx_gap <- s:(s + gap_steps - 1L)
              case_id <- paste(colname, gap_steps, s, sep = "_")

              a_mask <- raw_a
              a_mask[idx_gap] <- NA_real_

              res_case <- interpolate_one_series(
                a_raw = a_mask,
                ref_mat = ref_mat,
                ref_ok = ref_ok,
                niMethod = niMethod,
                n_boot = n_boot
              )

              if (isTRUE(correct_gap_jumps)) {
                jc_case <- correct_gap_end_jumps(
                  value = res_case$value,
                  fit = res_case$fit,
                  pi_lo = res_case$pi_lo,
                  pi_hi = res_case$pi_hi,
                  interp_flag = res_case$interp_flag,
                  original_values = a_mask,
                  time_vec = df$TIME,
                  jump_threshold = jump_threshold,
                  adjustment_method = jump_adjustment_method,
                  adjust_window = jump_adjust_window,
                  set_jump_point_na = jump_set_point_na
                )

                res_case$value <- jc_case$value
                res_case$fit <- jc_case$fit
                res_case$pi_lo <- jc_case$pi_lo
                res_case$pi_hi <- jc_case$pi_hi
              }

              truth <- raw_a[idx_gap]
              pred <- res_case$value[idx_gap]
              pi_lo_gap <- res_case$pi_lo[idx_gap]
              pi_hi_gap <- res_case$pi_hi[idx_gap]
              ref_n_gap <- res_case$ref_n[idx_gap]
              filled <- res_case$interp_flag[idx_gap] & is.finite(pred)

              n_points <- length(idx_gap)
              n_filled <- sum(filled)
              success_rate <- n_filled / n_points

              for (k in seq_along(idx_gap)) {
                err_k <- if (is.finite(pred[k]) && is.finite(truth[k])) pred[k] - truth[k] else NA_real_
                ape_k <- if (is.finite(err_k) && is.finite(truth[k])) {
                  100 * abs(err_k) / pmax(abs(truth[k]), 1e-6)
                } else {
                  NA_real_
                }
                cov_k <- if (is.finite(truth[k]) && is.finite(pi_lo_gap[k]) && is.finite(pi_hi_gap[k])) {
                  truth[k] >= pi_lo_gap[k] && truth[k] <= pi_hi_gap[k]
                } else {
                  NA
                }

                push_point_row(tibble::tibble(
                  series = colname,
                  gap_steps = gap_steps,
                  case_id = case_id,
                  TIME = df$TIME[idx_gap[k]],
                  truth = truth[k],
                  pred = pred[k],
                  filled = filled[k],
                  error = err_k,
                  abs_pct_error = ape_k,
                  pi_lo = pi_lo_gap[k],
                  pi_hi = pi_hi_gap[k],
                  covered = cov_k,
                  ref_n = ref_n_gap[k]
                ))
              }

              if (n_filled > 0L) {
                err <- pred[filled] - truth[filled]
                denom <- pmax(abs(truth[filled]), 1e-6)
                pe <- 100 * err / denom

                pi_ok <- filled & is.finite(pi_lo_gap) & is.finite(pi_hi_gap)

                PI_coverage_95 <- if (any(pi_ok)) {
                  mean(truth[pi_ok] >= pi_lo_gap[pi_ok] & truth[pi_ok] <= pi_hi_gap[pi_ok], na.rm = TRUE)
                } else {
                  NA_real_
                }

                Mean_PI_width <- if (any(pi_ok)) {
                  mean(pi_hi_gap[pi_ok] - pi_lo_gap[pi_ok], na.rm = TRUE)
                } else {
                  NA_real_
                }

                End_value_abs_diff <- if (is.finite(pred[length(pred)])) {
                  abs(pred[length(pred)] - truth[length(truth)])
                } else {
                  NA_real_
                }

                Max_abs_diff_within_gap <- max(abs(err), na.rm = TRUE)

                row <- tibble::tibble(
                  series = colname,
                  gap_steps = gap_steps,
                  case_id = case_id,
                  start_time = df$TIME[min(idx_gap)],
                  end_time = df$TIME[max(idx_gap)],
                  n_points = n_points,
                  n_filled = n_filled,
                  Success_rate = success_rate,
                  MAE = mean(abs(err), na.rm = TRUE),
                  MAPE = mean(abs(pe), na.rm = TRUE),
                  MdAPE = stats::median(abs(pe), na.rm = TRUE),
                  RMSE = sqrt(mean(err^2, na.rm = TRUE)),
                  Bias = mean(err, na.rm = TRUE),
                  PI_coverage_95 = PI_coverage_95,
                  Mean_PI_width = Mean_PI_width,
                  Mean_ref_n = mean(ref_n_gap, na.rm = TRUE),
                  End_value_abs_diff = End_value_abs_diff,
                  Max_abs_diff_within_gap = Max_abs_diff_within_gap
                )
              } else {
                row <- tibble::tibble(
                  series = colname,
                  gap_steps = gap_steps,
                  case_id = case_id,
                  start_time = df$TIME[min(idx_gap)],
                  end_time = df$TIME[max(idx_gap)],
                  n_points = n_points,
                  n_filled = n_filled,
                  Success_rate = success_rate,
                  MAE = NA_real_,
                  MAPE = NA_real_,
                  MdAPE = NA_real_,
                  RMSE = NA_real_,
                  Bias = NA_real_,
                  PI_coverage_95 = NA_real_,
                  Mean_PI_width = NA_real_,
                  Mean_ref_n = mean(ref_n_gap, na.rm = TRUE),
                  End_value_abs_diff = NA_real_,
                  Max_abs_diff_within_gap = NA_real_
                )
              }

              push_val_row(row)
            }
          }
        }

        update_progress(note = paste0("Validated ", colname, " | gap=", gap_steps))
      }
    }

    if (val_id > 0L) {
      validation_raw <- dplyr::bind_rows(val_rows[seq_len(val_id)])
      validation_points <- dplyr::bind_rows(point_rows[seq_len(point_id)])

      validation_summary <- validation_raw %>%
        dplyr::group_by(series, gap_steps) %>%
        dplyr::summarise(
          n_cases = dplyr::n(),
          n_points = sum(n_points, na.rm = TRUE),
          n_filled = sum(n_filled, na.rm = TRUE),
          Success_rate = mean_or_na(Success_rate),
          MAE = mean_or_na(MAE),
          MAPE = mean_or_na(MAPE),
          MdAPE = mean_or_na(MdAPE),
          RMSE = mean_or_na(RMSE),
          Bias = mean_or_na(Bias),
          PI_coverage_95 = mean_or_na(PI_coverage_95),
          Mean_PI_width = mean_or_na(Mean_PI_width),
          Mean_ref_n = mean_or_na(Mean_ref_n),
          End_value_abs_diff = mean_or_na(End_value_abs_diff),
          Max_abs_diff_within_gap = mean_or_na(Max_abs_diff_within_gap),
          .groups = "drop"
        ) %>%
        dplyr::arrange(series, gap_steps)

      validation_seasonal <- validation_points %>%
        dplyr::filter(filled, is.finite(truth), is.finite(pred)) %>%
        dplyr::mutate(doy = as.integer(format(TIME, "%j"))) %>%
        dplyr::group_by(series, doy) %>%
        dplyr::summarise(
          n_points = dplyr::n(),
          mean_error = mean(error, na.rm = TRUE),
          mean_abs_error = mean(abs(error), na.rm = TRUE),
          mean_abs_pct_error = mean(abs_pct_error, na.rm = TRUE),
          rmse = sqrt(mean(error^2, na.rm = TRUE)),
          mean_truth = mean(truth, na.rm = TRUE),
          mean_pred = mean(pred, na.rm = TRUE),
          coverage = mean(covered, na.rm = TRUE),
          .groups = "drop"
        ) %>%
        dplyr::arrange(series, doy)
    } else {
      validation_raw <- tibble::tibble()
      validation_points <- tibble::tibble()
      validation_summary <- tibble::tibble()
      validation_seasonal <- tibble::tibble()
      warning("Assessment produced no results (no valid synthetic windows found).")
    }
  }

  attr(out, "network_original") <- original_df
  attr(out, "network_reference") <- referenceDF
  attr(out, "network_diagnostics") <- diagnostics
  attr(out, "network_summary") <- summary_tbl
  attr(out, "network_jump_corrections") <- jump_corrections
  attr(out, "network_validation_raw") <- validation_raw
  attr(out, "network_validation_points") <- validation_points
  attr(out, "network_validation_summary") <- validation_summary
  attr(out, "network_validation_seasonal") <- validation_seasonal
  attr(out, "network_method") <- niMethod
  attr(out, "network_n_boot") <- n_boot
  attr(out, "network_return_flags") <- return_flags
  attr(out, "network_return_pi") <- return_pi
  attr(out, "network_pi_for_all") <- pi_for_all
  attr(out, "network_return_fit") <- return_fit
  attr(out, "network_assessed") <- assess
  attr(out, "network_jump_settings") <- list(
    correct_gap_jumps = correct_gap_jumps,
    jump_threshold = jump_threshold,
    jump_adjustment_method = jump_adjustment_method,
    jump_adjust_window = jump_adjust_window,
    jump_set_point_na = jump_set_point_na
  )

  class(out) <- c("network_interpolation", class(out))
  out
}


#' @title Plot method for network interpolation output
#'
#' @description
#' S3 plot method for objects returned by
#' \code{\link{network.interpolation}}.
#'
#' Depending on \code{type}, the method can display:
#' \itemize{
#'   \item interpolated series through time,
#'   \item uncertainty width,
#'   \item reference-network availability,
#'   \item summary diagnostics,
#'   \item synthetic-gap validation metrics,
#'   \item interval coverage,
#'   \item actual versus interpolated validation points,
#'   \item seasonal validation diagnostics.
#' }
#'
#' Jump-corrected post-gap points are marked in the interpolation plot when
#' jump correction was enabled in \code{\link{network.interpolation}}.
#'
#' @param x Object returned by \code{\link{network.interpolation}}.
#' @param type Character. Plot type. One of \code{"interpolation"},
#'   \code{"uncertainty"}, \code{"availability"}, \code{"summary"},
#'   \code{"validation"}, \code{"coverage"}, \code{"compare"}, or
#'   \code{"seasonal_error"}.
#' @param series Optional character vector of focal series names to plot.
#' @param start Optional start datetime for time-based plots.
#' @param end Optional end datetime for time-based plots.
#' @param show_pi Logical. If \code{TRUE}, show 95\% prediction intervals in
#'   the interpolation plot where available.
#' @param show_fit Logical. If \code{TRUE}, show model-implied fitted values in
#'   the interpolation plot where available.
#' @param free_y Logical. If \code{TRUE}, facets use free y scales where
#'   applicable.
#' @param ncol Integer. Number of facet columns.
#' @param summary_metric Character metric used when
#'   \code{type = "summary"}. One of \code{"n_imputed"},
#'   \code{"n_remaining_na"}, \code{"n_missing_input"},
#'   \code{"mean_pi_width"}, \code{"median_pi_width"}, \code{"mean_ref_n"},
#'   \code{"median_ref_n"}, \code{"n_gap_jumps_removed"}, or
#'   \code{"max_abs_gap_jump"}.
#' @param validation_metric Character metric used when
#'   \code{type = "validation"}. One of \code{"MAE"}, \code{"MAPE"},
#'   \code{"MdAPE"}, \code{"RMSE"}, \code{"Bias"},
#'   \code{"Success_rate"}, \code{"Mean_PI_width"}, \code{"Mean_ref_n"},
#'   \code{"End_value_abs_diff"}, or \code{"Max_abs_diff_within_gap"}.
#' @param seasonal_metric Character metric used when
#'   \code{type = "seasonal_error"}. One of \code{"mean_error"},
#'   \code{"mean_abs_error"}, \code{"mean_abs_pct_error"}, \code{"rmse"},
#'   or \code{"coverage"}.
#' @param compare_colour Character. Colouring used when
#'   \code{type = "compare"}. One of \code{"series"} or
#'   \code{"gap_steps"}.
#' @param facet Logical. If \code{TRUE}, use faceting where supported.
#' @param empty Character. Behavior when no suitable data are available:
#'   \code{"plot"} returns an informative empty plot and \code{"error"} throws
#'   an error.
#' @param ... Further arguments passed through the generic.
#'
#' @return A \code{ggplot2} object.
#'
#' @details
#' Plot types:
#' \describe{
#'   \item{\code{"interpolation"}}{Shows original and interpolated focal series
#'   through time. Imputed points are highlighted. If jump correction was
#'   applied, corrected post-gap points are marked in green.}
#'   \item{\code{"uncertainty"}}{Shows the width of the prediction interval
#'   through time.}
#'   \item{\code{"availability"}}{Shows the number of valid reference sensors
#'   available at each step.}
#'   \item{\code{"summary"}}{Shows per-series summary diagnostics produced by
#'   the interpolation run.}
#'   \item{\code{"validation"}}{Shows synthetic-gap recovery metrics across
#'   tested gap lengths.}
#'   \item{\code{"coverage"}}{Shows how often true values fall inside the
#'   nominal 95\% prediction interval across gap lengths.}
#'   \item{\code{"compare"}}{Scatter plot of actual versus interpolated values
#'   from synthetic-gap validation.}
#'   \item{\code{"seasonal_error"}}{Shows how validation error varies through
#'   the season, summarized by day of year.}
#' }
#'
#' Validation-based plots require that
#' \code{network.interpolation(..., assess = TRUE)} was used to create
#' \code{x}.
#'
#' @examples
#' \donttest{
#' #library(dendRoAnalyst)
#' #data("gf_nepa17")
#'
#' #df1 <- gf_nepa17
#' #df1[40:50, "T2"] <- NA
#'
#' #ref <- cbind(gf_nepa17, gf_nepa17[, 2:3], gf_nepa17[, 2:3])
#' #colnames(ref) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")
#'
#' #out <- network.interpolation(
#' #  df1, ref,
#' #  niMethod = "proportional",
#' #  n_boot = 100,
#' #  assess = TRUE,
#' #  assess_lengths_steps = c(1, 2, 4),
#' #  correct_gap_jumps = TRUE,
#' #  jump_threshold = 0.05
#' #)
#'
#' #plot(out)
#' #plot(out, type = "uncertainty")
#' #plot(out, type = "availability")
#' #plot(out, type = "summary", summary_metric = "n_gap_jumps_removed")
#' #plot(out, type = "validation", validation_metric = "MAPE")
#' #plot(out, type = "coverage")
#' #plot(out, type = "compare")
#' #plot(out, type = "seasonal_error", seasonal_metric = "mean_abs_error")
#' }
#'
#' @seealso
#' \code{\link{network.interpolation}}
#'
#' @method plot network_interpolation
#' @export
plot.network_interpolation <- function(x,
                                       type = c("interpolation", "uncertainty", "availability", "summary", "validation", "coverage", "compare", "seasonal_error"),
                                       series = NULL,
                                       start = NULL,
                                       end = NULL,
                                       show_pi = TRUE,
                                       show_fit = FALSE,
                                       free_y = TRUE,
                                       ncol = 1,
                                       summary_metric = c("n_imputed", "n_remaining_na", "n_missing_input", "mean_pi_width", "median_pi_width", "mean_ref_n", "median_ref_n", "n_gap_jumps_removed", "max_abs_gap_jump"),
                                       validation_metric = c("MAE", "MAPE", "MdAPE", "RMSE", "Bias", "Success_rate", "Mean_PI_width", "Mean_ref_n", "End_value_abs_diff", "Max_abs_diff_within_gap"),
                                       seasonal_metric = c("mean_error", "mean_abs_error", "mean_abs_pct_error", "rmse", "coverage"),
                                       compare_colour = c("series", "gap_steps"),
                                       facet = TRUE,
                                       empty = c("plot", "error"),
                                       ...) {
  type <- match.arg(type)
  summary_metric <- match.arg(summary_metric)
  validation_metric <- match.arg(validation_metric)
  seasonal_metric <- match.arg(seasonal_metric)
  compare_colour <- match.arg(compare_colour)
  empty <- match.arg(empty)

  if (!inherits(x, "network_interpolation")) {
    stop("x must be an object returned by network.interpolation().")
  }

  if (type == "interpolation") {
    return(.plot_network_interpolation(
      x = x,
      series = series,
      start = start,
      end = end,
      show_pi = show_pi,
      show_fit = show_fit,
      free_y = free_y,
      ncol = ncol,
      empty = empty
    ))
  }

  if (type == "uncertainty") {
    return(.plot_network_uncertainty(
      x = x,
      series = series,
      start = start,
      end = end,
      free_y = free_y,
      ncol = ncol,
      empty = empty
    ))
  }

  if (type == "availability") {
    return(.plot_network_availability(
      x = x,
      series = series,
      start = start,
      end = end,
      free_y = free_y,
      ncol = ncol,
      empty = empty
    ))
  }

  if (type == "summary") {
    return(.plot_network_summary(
      x = x,
      metric = summary_metric,
      empty = empty
    ))
  }

  if (type == "validation") {
    return(.plot_network_validation(
      x = x,
      metric = validation_metric,
      series = series,
      facet = facet,
      empty = empty
    ))
  }

  if (type == "coverage") {
    return(.plot_network_coverage(
      x = x,
      series = series,
      facet = facet,
      empty = empty
    ))
  }

  if (type == "seasonal_error") {
    return(.plot_network_seasonal_error(
      x = x,
      metric = seasonal_metric,
      series = series,
      facet = facet,
      empty = empty
    ))
  }

  .plot_network_compare(
    x = x,
    series = series,
    facet = facet,
    colour_by = compare_colour,
    empty = empty
  )
}


#' @keywords internal
#' @noRd
.plot_network_interpolation <- function(x, series = NULL, start = NULL, end = NULL,
                                        show_pi = TRUE, show_fit = FALSE,
                                        free_y = TRUE, ncol = 1,
                                        empty = c("plot", "error")) {
  empty <- match.arg(empty)
  TIME <- fit <- original <- pi_hi <- pi_lo <- value <- jump_correction <- NULL

  d <- .network_interp_diag_subset(x, series = series, start = start, end = end)

  if (nrow(d) == 0) {
    if (empty == "error") stop("No rows available for plotting.")
    return(.network_interp_empty_plot("No rows available for plotting."))
  }

  method <- attr(x, "network_method", exact = TRUE)

  p <- ggplot2::ggplot(d, ggplot2::aes(x = TIME)) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Time",
      y = "Dendrometer value",
      title = paste("Network interpolation:", method)
    )

  if (show_pi && any(is.finite(d$pi_lo) & is.finite(d$pi_hi))) {
    p <- p +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = pi_lo, ymax = pi_hi),
        fill = "grey70",
        alpha = 0.30,
        na.rm = TRUE
      )
  }

  p <- p +
    ggplot2::geom_line(
      ggplot2::aes(y = original),
      linewidth = 0.35,
      colour = "grey45",
      na.rm = TRUE
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = value),
      linewidth = 0.45,
      colour = "#1F78B4",
      na.rm = TRUE
    )

  if (show_fit && any(is.finite(d$fit))) {
    p <- p +
      ggplot2::geom_line(
        ggplot2::aes(y = fit),
        linewidth = 0.35,
        linetype = 2,
        colour = "black",
        na.rm = TRUE
      )
  }

  if (any(d$interp, na.rm = TRUE)) {
    p <- p +
      ggplot2::geom_point(
        data = d[d$interp, , drop = FALSE],
        ggplot2::aes(x = TIME, y = value),
        colour = "#D7301F",
        size = 1.2,
        inherit.aes = FALSE,
        na.rm = TRUE
      )
  }

  jump_df <- d[!is.na(d$jump_correction), , drop = FALSE]

  if (nrow(jump_df) > 0) {
    p <- p +
      ggplot2::geom_point(
        data = jump_df,
        ggplot2::aes(x = TIME, y = value),
        colour = "darkgreen",
        size = 2,
        inherit.aes = FALSE,
        na.rm = TRUE
      )
  }

  p +
    ggplot2::facet_wrap(
      ~series,
      scales = if (free_y) "free_y" else "fixed",
      ncol = ncol
    )
}


#' @keywords internal
#' @noRd
.plot_network_uncertainty <- function(x, series = NULL, start = NULL, end = NULL,
                                      free_y = TRUE, ncol = 1,
                                      empty = c("plot", "error")) {
  empty <- match.arg(empty)
  TIME <- pi_width <- NULL

  d <- .network_interp_diag_subset(x, series = series, start = start, end = end)
  d <- d[is.finite(d$pi_width), , drop = FALSE]

  if (nrow(d) == 0) {
    if (empty == "error") stop("No prediction intervals available for plotting.")
    return(.network_interp_empty_plot("No prediction intervals available for plotting."))
  }

  ggplot2::ggplot(d, ggplot2::aes(x = TIME, y = pi_width)) +
    ggplot2::geom_line(linewidth = 0.45, na.rm = TRUE) +
    ggplot2::geom_point(
      data = d[d$interp, , drop = FALSE],
      size = 1.1,
      na.rm = TRUE
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Time",
      y = "95% interval width",
      title = "Interpolation uncertainty over time"
    ) +
    ggplot2::facet_wrap(
      ~series,
      scales = if (free_y) "free_y" else "fixed",
      ncol = ncol
    )
}


#' @keywords internal
#' @noRd
.plot_network_availability <- function(x, series = NULL, start = NULL, end = NULL,
                                       free_y = TRUE, ncol = 1,
                                       empty = c("plot", "error")) {
  TIME <- ref_n <- NULL
  empty <- match.arg(empty)

  d <- .network_interp_diag_subset(x, series = series, start = start, end = end)

  if (nrow(d) == 0 || !any(is.finite(d$ref_n))) {
    if (empty == "error") stop("No reference availability data available for plotting.")
    return(.network_interp_empty_plot("No reference availability data available for plotting."))
  }

  ggplot2::ggplot(d, ggplot2::aes(x = TIME, y = ref_n)) +
    ggplot2::geom_line(linewidth = 0.45, na.rm = TRUE) +
    ggplot2::geom_point(
      data = d[d$interp, , drop = FALSE],
      colour = "#D7301F",
      size = 1.2,
      na.rm = TRUE
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Time",
      y = "Available reference sensors",
      title = "Reference network availability"
    ) +
    ggplot2::facet_wrap(
      ~series,
      scales = if (free_y) "free_y" else "fixed",
      ncol = ncol
    )
}


#' @keywords internal
#' @noRd
.plot_network_summary <- function(x,
                                  metric = c("n_imputed", "n_remaining_na", "n_missing_input", "mean_pi_width", "median_pi_width", "mean_ref_n", "median_ref_n", "n_gap_jumps_removed", "max_abs_gap_jump"),
                                  empty = c("plot", "error")) {
  metric <- match.arg(metric)
  empty <- match.arg(empty)
  series <- value <- NULL

  s <- attr(x, "network_summary", exact = TRUE)

  if (is.null(s) || nrow(s) == 0) {
    if (empty == "error") stop("No summary information available for plotting.")
    return(.network_interp_empty_plot("No summary information available for plotting."))
  }

  s <- tibble::as_tibble(s)
  s$value <- s[[metric]]

  ggplot2::ggplot(s, ggplot2::aes(x = series, y = value)) +
    ggplot2::geom_col() +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Series",
      y = metric,
      title = paste("Network interpolation summary:", metric)
    )
}


#' @keywords internal
#' @noRd
.plot_network_validation <- function(x,
                                     metric = c("MAE", "MAPE", "MdAPE", "RMSE", "Bias", "Success_rate", "Mean_PI_width", "Mean_ref_n", "End_value_abs_diff", "Max_abs_diff_within_gap"),
                                     series = NULL,
                                     facet = TRUE,
                                     empty = c("plot", "error")) {
  metric <- match.arg(metric)
  empty <- match.arg(empty)
  gap_steps <- value <- NULL

  s <- .network_interp_validation_subset(x, series = series)

  if (nrow(s) == 0) {
    if (empty == "error") stop("No validation results available for plotting.")
    return(.network_interp_empty_plot("No validation results available for plotting."))
  }

  s$value <- s[[metric]]

  p <- ggplot2::ggplot(
    s,
    ggplot2::aes(x = gap_steps, y = value, group = series, colour = series)
  ) +
    ggplot2::geom_line(linewidth = 0.5, na.rm = TRUE) +
    ggplot2::geom_point(size = 1.8, na.rm = TRUE) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Artificial gap length (steps)",
      y = metric,
      colour = "Series",
      title = paste("Validation:", metric)
    )

  if (facet) {
    p + ggplot2::facet_wrap(~series, scales = "free_y")
  } else {
    p
  }
}


#' @keywords internal
#' @noRd
.plot_network_coverage <- function(x,
                                   series = NULL,
                                   facet = TRUE,
                                   empty = c("plot", "error")) {
  gap_steps <- PI_coverage_95 <- NULL
  empty <- match.arg(empty)

  s <- .network_interp_validation_subset(x, series = series)

  if (nrow(s) == 0 || !any(is.finite(s$PI_coverage_95))) {
    if (empty == "error") stop("No validation coverage results available for plotting.")
    return(.network_interp_empty_plot("No validation coverage results available for plotting."))
  }

  p <- ggplot2::ggplot(
    s,
    ggplot2::aes(x = gap_steps, y = PI_coverage_95, group = series, colour = series)
  ) +
    ggplot2::geom_hline(yintercept = 0.95, linetype = 2) +
    ggplot2::geom_line(linewidth = 0.5, na.rm = TRUE) +
    ggplot2::geom_point(size = 1.8, na.rm = TRUE) +
    ggplot2::coord_cartesian(ylim = c(0, 1)) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Artificial gap length (steps)",
      y = "95% PI coverage",
      colour = "Series",
      title = "Validation of interval coverage"
    )

  if (facet) {
    p + ggplot2::facet_wrap(~series, scales = "fixed")
  } else {
    p
  }
}


#' @keywords internal
#' @noRd
.plot_network_compare <- function(x,
                                  series = NULL,
                                  facet = TRUE,
                                  colour_by = c("series", "gap_steps"),
                                  empty = c("plot", "error")) {
  truth <- pred <- gap_steps <- gap_steps_f <- NULL
  empty <- match.arg(empty)
  colour_by <- match.arg(colour_by)

  pdat <- .network_interp_validation_points_subset(x, series = series)
  pdat <- pdat[pdat$filled & is.finite(pdat$truth) & is.finite(pdat$pred), , drop = FALSE]

  if (nrow(pdat) == 0) {
    if (empty == "error") stop("No point-level validation comparison data available for plotting.")
    return(.network_interp_empty_plot("No point-level validation comparison data available for plotting."))
  }

  pdat$gap_steps_f <- factor(pdat$gap_steps)

  aes_colour <- if (colour_by == "series") {
    ggplot2::aes(colour = series)
  } else {
    ggplot2::aes(colour = gap_steps_f)
  }

  p <- ggplot2::ggplot(pdat, ggplot2::aes(x = truth, y = pred)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = 2) +
    ggplot2::geom_point(aes_colour, alpha = 0.75, size = 1.8, na.rm = TRUE) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Actual value",
      y = "Interpolated value",
      colour = if (colour_by == "series") "Series" else "Gap steps",
      title = "Actual vs interpolated values from validation"
    )

  if (facet) {
    p + ggplot2::facet_wrap(~series, scales = "free")
  } else {
    p
  }
}


#' @keywords internal
#' @noRd
.plot_network_seasonal_error <- function(x,
                                         metric = c("mean_error", "mean_abs_error", "mean_abs_pct_error", "rmse", "coverage"),
                                         series = NULL,
                                         facet = TRUE,
                                         empty = c("plot", "error")) {
  metric <- match.arg(metric)
  empty <- match.arg(empty)
  doy <- value <- NULL

  s <- .network_interp_validation_seasonal_subset(x, series = series)

  if (nrow(s) == 0) {
    if (empty == "error") stop("No seasonal validation results available for plotting.")
    return(.network_interp_empty_plot("No seasonal validation results available for plotting."))
  }

  s$value <- s[[metric]]

  p <- ggplot2::ggplot(
    s,
    ggplot2::aes(x = doy, y = value, colour = series, group = series)
  ) +
    ggplot2::geom_line(linewidth = 0.45, na.rm = TRUE) +
    ggplot2::geom_point(size = 1.2, na.rm = TRUE) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      x = "Day of year",
      y = metric,
      colour = "Series",
      title = paste("Seasonal validation diagnostic:", metric)
    )

  if (facet) {
    p + ggplot2::facet_wrap(~series, scales = "free_y")
  } else {
    p
  }
}


#' @keywords internal
#' @noRd
.network_interp_diag_subset <- function(x, series = NULL, start = NULL, end = NULL) {
  d <- attr(x, "network_diagnostics", exact = TRUE)
  if (is.null(d)) {
    stop("No diagnostics found on this object.")
  }

  d <- tibble::as_tibble(d)

  if (!is.null(series)) {
    d <- d[d$series %in% series, , drop = FALSE]
  }
  if (!is.null(start)) {
    d <- d[d$TIME >= as.POSIXct(start), , drop = FALSE]
  }
  if (!is.null(end)) {
    d <- d[d$TIME <= as.POSIXct(end), , drop = FALSE]
  }

  d
}


#' @keywords internal
#' @noRd
.network_interp_validation_subset <- function(x, series = NULL) {
  s <- attr(x, "network_validation_summary", exact = TRUE)
  if (is.null(s)) {
    return(tibble::tibble())
  }

  s <- tibble::as_tibble(s)

  if (!is.null(series)) {
    s <- s[s$series %in% series, , drop = FALSE]
  }

  s
}


#' @keywords internal
#' @noRd
.network_interp_validation_points_subset <- function(x, series = NULL) {
  s <- attr(x, "network_validation_points", exact = TRUE)
  if (is.null(s)) {
    return(tibble::tibble())
  }

  s <- tibble::as_tibble(s)

  if (!is.null(series)) {
    s <- s[s$series %in% series, , drop = FALSE]
  }

  s
}


#' @keywords internal
#' @noRd
.network_interp_validation_seasonal_subset <- function(x, series = NULL) {
  s <- attr(x, "network_validation_seasonal", exact = TRUE)
  if (is.null(s)) {
    return(tibble::tibble())
  }

  s <- tibble::as_tibble(s)

  if (!is.null(series)) {
    s <- s[s$series %in% series, , drop = FALSE]
  }

  s
}


#' @keywords internal
#' @noRd
.network_interp_empty_plot <- function(label) {
  ggplot2::ggplot() +
    ggplot2::annotate("text", x = 0, y = 0, label = label, size = 5) +
    ggplot2::xlim(-1, 1) +
    ggplot2::ylim(-1, 1) +
    ggplot2::labs(x = NULL, y = NULL) +
    ggplot2::theme_void()
}

#' Detrend and standardize dendrometer series from growth-fit residuals
#'
#' @description
#' Creates detrended standardized dendrometer series from objects returned by
#' [dm.growth.fit()] or [dm.growth.fit.double()].
#'
#' The function compares the original daily dendrometer series
#' (`x$original_daily_data`) with the fitted growth curve from `x$fitted_data`.
#' Because fitted values are stored on the processed scale used for modelling,
#' the function first reconstructs fitted values on the original daily scale by
#' adding back the seasonal baseline (the first non-missing original daily value
#' of each series within each vegetation season).
#'
#' Residuals are then calculated as:
#' \deqn{residual = observed_{daily} - fitted_{original\ scale}}
#'
#' To obtain a detrended standardized series with mean equal to 1 and no negative
#' values, residuals are transformed within each \code{series × season_label} as:
#' \deqn{z = \frac{residual - \min(residual) + \epsilon}{\mathrm{mean}(residual - \min(residual) + \epsilon)}}
#'
#' where \eqn{\epsilon > 0} is a small constant ensuring strictly positive values.
#' If all residuals within a season are identical, the standardized series
#' becomes a constant vector of 1.
#'
#' @param x An object of class \code{"dm_growth_fit"} returned by
#'   [dm.growth.fit()] or [dm.growth.fit.double()].
#' @param series Optional character vector of dendrometer series to retain.
#'   Default is \code{NULL}, meaning all available series are used.
#' @param seasons Optional character vector of vegetation-season labels to
#'   retain. Default is \code{NULL}, meaning all available seasons are used.
#' @param epsilon Small positive constant added after shifting residuals by their
#'   seasonal minimum. Default is \code{1e-6}. Use \code{0} to allow exact zeros.
#' @param keep_na_rows Logical. If \code{TRUE}, rows are retained even when the
#'   observed or fitted value is missing; detrended values remain \code{NA}.
#'   If \code{FALSE} (default), rows with missing observed or fitted values are
#'   removed from the long output before standardization.
#'
#' @return
#' A list of class \code{"dm_detrended"} with elements:
#' \describe{
#'   \item{call}{The matched function call.}
#'   \item{detrended_data}{Wide-format tibble with metadata columns and one
#'     detrended standardized series column per dendrometer.}
#'   \item{detrended_long}{Long-format tibble containing original daily values,
#'     reconstructed fitted values on the original scale, residuals, shifted
#'     residuals, and detrended standardized values.}
#'   \item{parameters}{Per-series and per-season summary table containing
#'     baseline values and residual standardization parameters.}
#'   \item{source_fit_statistics}{The original \code{x$fit_statistics} table.}
#' }
#'
#' @details
#' The function uses \code{x$original_daily_data}, which must be present in the
#' input object. If the object was created by an older version of
#' [dm.growth.fit()] or [dm.growth.fit.double()] that did not store
#' \code{original_daily_data}, the function will stop with an informative error.
#'
#' Standardization is carried out separately within each vegetation season. This
#' means the mean of the detrended standardized series is 1 for each
#' \code{series × season_label}, not necessarily across the entire dataset.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#'
#' fit1 <- dm.growth.fit(
#'   df = gf_nepa17,
#'   TreeNum = 1:2,
#'   method = "gompertz",
#'   year_mode = "yearly",
#'   verbose = FALSE
#' )
#'
#' det1 <- dm.detrend.fit(fit1)
#' head(det1$detrended_data)
#' head(det1$parameters)
#'
#' fit2 <- dm.growth.fit.double(
#'   df = gf_nepa17,
#'   TreeNum = 1,
#'   method = "gompertz",
#'   year_mode = "yearly",
#'   verbose = FALSE
#' )
#'
#' det2 <- dm.detrend.fit(fit2)
#' head(det2$detrended_long)
#' }
#'
#' @importFrom dplyr mutate filter select rename left_join distinct group_by summarise ungroup arrange across all_of bind_rows %>%
#' @importFrom tidyr pivot_longer pivot_wider
#' @importFrom tibble as_tibble tibble
#' @export
dm.detrend.fit <- function(x,
                           series = NULL,
                           seasons = NULL,
                           epsilon = 1e-6,
                           keep_na_rows = FALSE) {
  TIME <- doy <- season_label <- season_start <- season_end <- season_day <- NULL
  series_name <- observed <- fitted_processed <- fitted_original <- residual <- NULL
  residual_shifted <- detrended_std <- baseline <- fit_id <- n_obs <- residual_min <- NULL
  shift_added <- shifted_mean <- NULL

  cl <- match.call()

  if (!inherits(x, "dm_growth_fit")) {
    stop("'x' must be an object of class 'dm_growth_fit'.")
  }

  if (is.null(x$original_daily_data)) {
    stop(
      "The input object does not contain 'original_daily_data'. ",
      "Please refit using the updated dm.growth.fit() or dm.growth.fit.double()."
    )
  }

  if (!is.numeric(epsilon) || length(epsilon) != 1 || is.na(epsilon) || epsilon < 0) {
    stop("'epsilon' must be a single non-negative numeric value.")
  }

  if (!is.logical(keep_na_rows) || length(keep_na_rows) != 1 || is.na(keep_na_rows)) {
    stop("'keep_na_rows' must be TRUE or FALSE.")
  }

  orig <- tibble::as_tibble(x$original_daily_data)
  fitd <- tibble::as_tibble(x$fitted_data)

  meta_cols_orig <- intersect(
    c("TIME", "doy", "season_label", "season_start", "season_end", "season_day"),
    names(orig)
  )

  meta_cols_fit <- intersect(
    c("TIME", "doy", "season_label", "season_start", "season_end", "season_day", "season_length", "any_observed"),
    names(fitd)
  )

  series_cols <- intersect(
    setdiff(names(orig), meta_cols_orig),
    setdiff(names(fitd), meta_cols_fit)
  )

  if (length(series_cols) == 0) {
    stop("No matching dendrometer series were found in original_daily_data and fitted_data.")
  }

  if (!is.null(series)) {
    miss <- setdiff(series, series_cols)
    if (length(miss) > 0) {
      warning("These series were not found and were ignored: ", paste(miss, collapse = ", "))
    }
    series_cols <- intersect(series_cols, series)
    if (length(series_cols) == 0) {
      stop("No requested series remained after filtering.")
    }
  }

  orig_long <- orig %>%
    dplyr::select(dplyr::all_of(meta_cols_orig), dplyr::all_of(series_cols)) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(series_cols),
      names_to = "series_name",
      values_to = "observed"
    )

  fit_long <- fitd %>%
    dplyr::select(dplyr::all_of(meta_cols_fit), dplyr::all_of(series_cols)) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(series_cols),
      names_to = "series_name",
      values_to = "fitted_processed"
    )

  dat <- orig_long %>%
    dplyr::left_join(
      fit_long %>%
        dplyr::select(TIME, season_label, season_day, series_name, fitted_processed),
      by = c("TIME", "season_label", "season_day", "series_name")
    ) %>%
    dplyr::arrange(series_name, season_label, TIME)

  if (!is.null(seasons)) {
    dat <- dat %>% dplyr::filter(.data$season_label %in% seasons)
  }

  if (nrow(dat) == 0) {
    stop("No rows remain after filtering by series and/or seasons.")
  }

  # Baseline = first non-missing original daily value within each series × season
  baseline_tbl <- dat %>%
    dplyr::group_by(.data$series_name, .data$season_label) %>%
    dplyr::summarise(
      baseline = {
        idx <- which(!is.na(.data$observed))[1]
        if (length(idx) == 0 || is.na(idx)) NA_real_ else .data$observed[idx]
      },
      .groups = "drop"
    )

  dat <- dat %>%
    dplyr::left_join(baseline_tbl, by = c("series_name", "season_label")) %>%
    dplyr::mutate(
      fitted_original = .data$fitted_processed + .data$baseline,
      residual = .data$observed - .data$fitted_original
    )

  # Determine fit_id
  if (!is.null(x$fit_parameters) &&
      "fit_id" %in% names(x$fit_parameters) &&
      length(unique(stats::na.omit(x$fit_parameters$fit_id))) == 1 &&
      unique(stats::na.omit(x$fit_parameters$fit_id)) == "pooled") {
    dat$fit_id <- "pooled"
  } else {
    dat$fit_id <- as.character(dat$season_label)
  }

  if (!isTRUE(keep_na_rows)) {
    dat <- dat %>%
      dplyr::filter(!is.na(.data$observed), !is.na(.data$fitted_original))
  }

  if (nrow(dat) == 0) {
    stop("No rows available after removing missing observed/fitted values.")
  }

  # Standardize residuals within each series × season
  dat <- dat %>%
    dplyr::group_by(.data$series_name, .data$season_label) %>%
    dplyr::mutate(
      residual_min = if (all(is.na(.data$residual))) NA_real_ else min(.data$residual, na.rm = TRUE),
      shift_added = if (all(is.na(.data$residual))) NA_real_ else -residual_min + epsilon,
      residual_shifted = ifelse(is.na(.data$residual), NA_real_, .data$residual + .data$shift_added),
      shifted_mean = if (all(is.na(.data$residual_shifted))) NA_real_ else mean(.data$residual_shifted, na.rm = TRUE),
      detrended_std = dplyr::if_else(
        is.finite(.data$shifted_mean) & .data$shifted_mean > 0,
        .data$residual_shifted / .data$shifted_mean,
        NA_real_
      )
    ) %>%
    dplyr::ungroup()

  # If residuals are constant, detrended_std becomes exactly 1
  dat <- dat %>%
    dplyr::mutate(
      detrended_std = ifelse(is.nan(.data$detrended_std) | is.infinite(.data$detrended_std), NA_real_, .data$detrended_std)
    )

  params <- dat %>%
    dplyr::group_by(.data$series_name, .data$season_label, .data$fit_id) %>%
    dplyr::summarise(
      n_obs = sum(!is.na(.data$observed) & !is.na(.data$fitted_original)),
      baseline = dplyr::first(.data$baseline),
      residual_min = dplyr::first(.data$residual_min),
      shift_added = dplyr::first(.data$shift_added),
      shifted_mean = dplyr::first(.data$shifted_mean),
      mean_detrended = if (all(is.na(.data$detrended_std))) NA_real_ else mean(.data$detrended_std, na.rm = TRUE),
      min_detrended = if (all(is.na(.data$detrended_std))) NA_real_ else min(.data$detrended_std, na.rm = TRUE),
      max_detrended = if (all(is.na(.data$detrended_std))) NA_real_ else max(.data$detrended_std, na.rm = TRUE),
      .groups = "drop"
    ) %>%
    dplyr::rename(series = .data$series_name)

  # Wide output
  meta_keep <- intersect(
    c("TIME", "doy", "season_label", "season_start", "season_end", "season_day"),
    names(dat)
  )

  detrended_data <- dat %>%
    dplyr::select(dplyr::all_of(meta_keep), .data$series_name, .data$detrended_std) %>%
    dplyr::distinct() %>%
    tidyr::pivot_wider(
      names_from = .data$series_name,
      values_from = .data$detrended_std
    ) %>%
    dplyr::arrange(.data$season_label, .data$TIME)

  detrended_long <- dat %>%
    dplyr::select(
      TIME, doy, season_label, season_start, season_end, season_day,
      fit_id, series_name, observed, baseline, fitted_processed, fitted_original,
      residual, residual_shifted, detrended_std
    ) %>%
    dplyr::rename(series = .data$series_name) %>%
    dplyr::arrange(.data$series, .data$season_label, .data$TIME)

  out <- list(
    call = cl,
    detrended_data = detrended_data,
    detrended_long = detrended_long,
    parameters = params,
    source_fit_statistics = x$fit_statistics
  )

  class(out) <- "dm_detrended"
  out
}

#' @title Hourly climate data for Kathmandu derived from ERA5-Land
#'
#' @description
#' Hourly near-surface climate data for Kathmandu, Nepal, extracted from the
#' Copernicus Climate Change Service (C3S) ERA5-Land reanalysis. The dataset is
#' included to demonstrate climate–dendrometer workflows in
#' \pkg{dendRoAnalyst}, including joining hourly climate data with dendrometer
#' observations, event-based analyses, and wavelet-based analyses.
#'
#' @format A data frame with hourly observations and 5 variables:
#' \describe{
#'   \item{\code{TIME}}{Date-time stamp of the hourly observation.}
#'   \item{\code{temp}}{Air temperature in degree Celsius.}
#'   \item{\code{prec}}{Precipitation in millimetres.}
#'   \item{\code{VPD}}{Vapour pressure deficit in kPa.}
#'   \item{\code{RH}}{Relative humidity in percent.}
#' }
#'
#' @details
#' \code{ktm_clim_hourly} contains a single-location hourly climate time series
#' for Kathmandu derived from ERA5-Land. ERA5-Land is a global land-surface
#' reanalysis produced by replaying the land component of ERA5 at enhanced
#' spatial resolution and is widely used for land-surface and ecohydrological
#' applications.
#'
#' This dataset is intended as an example climate input for
#' \pkg{dendRoAnalyst}. It can be used, for example, with functions that join
#' dendrometer and climate data, calculate phase-climate relations, run event
#' analyses, or perform wavelet and wavelet-coherence analyses.
#'
#' Users should check the exact temporal coverage of the object in their local
#' installation, for example with:
#' \preformatted{
#' range(ktm_clim_hourly$TIME)
#' }
#'
#' @source
#' Extracted from the Copernicus Climate Change Service (C3S) Climate Data Store
#' ERA5-Land reanalysis product:
#'
#' Muñoz-Sabater, J. (2019). ERA5-Land hourly data from 1950 to present.
#' Copernicus Climate Change Service (C3S) Climate Data Store (CDS).
#'
#' @references
#' Muñoz-Sabater, J., Dutra, E., Agustí-Panareda, A., Albergel, C.,
#' Arduini, G., Balsamo, G., Boussetta, S., Choulga, M., Harrigan, S.,
#' Hersbach, H., Martens, B., Miralles, D. G., Piles, M.,
#' Rodríguez-Fernández, N. J., Zsoter, E., Buontempo, C., and Thépaut, J.-N.
#' (2021). ERA5-Land: a state-of-the-art global reanalysis dataset for land
#' applications. \emph{Earth System Science Data}, 13, 4349--4383.
#'
"ktm_clim_hourly"

#' @title Application of the stem-cycle approach to classify dendrometer phases
#'
#' @description
#' Implements the \strong{stem-cycle approach} (Downes et al., 1999; Deslauriers et al., 2011)
#' to divide a dendrometer time series into three biologically meaningful phases:
#' \enumerate{
#'   \item \strong{Shrinkage} (phase = 1): the dendrometer reading decreases
#'         compared to the previous reading.
#'   \item \strong{Expansion} (phase = 2): the dendrometer reading increases
#'         compared to the previous reading, but remains below the previous maximum.
#'   \item \strong{Increment} (phase = 3): the dendrometer reading exceeds
#'         the previous maximum (irreversible stem growth).
#' }
#'
#' For each contiguous phase, the function calculates duration, magnitude, rate,
#' and assigns day-of-year information. Optionally, the dendrometer series may be
#' smoothed before phase calculation to reduce noise and spurious phase changes.
#'
#' @details
#' Classification uses the cumulative maximum of the dendrometer series:
#' \itemize{
#'   \item If the cumulative maximum increases, the phase is labeled \emph{Increment} (3).
#'   \item If the cumulative maximum is constant and the first difference is positive,
#'         the phase is \emph{Expansion} (2).
#'   \item If the cumulative maximum is constant and the first difference is negative,
#'         the phase is \emph{Shrinkage} (1).
#' }
#'
#' The function returns both phase-level summaries (\code{SC_cycle}) and point-level
#' labels (\code{SC_phase}). Optional smoothing uses \code{smooth_dm} with
#' \code{method = "median_mean"} and a window length between 1–24 hours.
#'
#' @references
#' Deslauriers A, Rossi S, Turcotte A, Morin H, Krause C (2011)
#' A three-step procedure in SAS to analyze the time series from automatic dendrometers.
#' \emph{Dendrochronologia} 29:151–161. \doi{10.1016/j.dendro.2011.01.008}
#'
#' Downes G, Beadle C, Worledge D (1999)
#' Daily stem growth patterns in irrigated \emph{Eucalyptus globulus} and \emph{E. nitens} in relation to climate.
#' \emph{Trees} 14:102–111. \doi{10.1007/PL00009752}
#'
#' @param df A data frame with the first column containing date-time in the format
#'   \code{"yyyy-mm-dd HH:MM:SS"} (or convertible to \code{POSIXct}), followed by one or more
#'   dendrometer measurement columns (mm).
#' @param TreeNum Integer. The index of the dendrometer column to analyze.
#'   For example, \code{TreeNum = 1} selects the first dendrometer series after the time column.
#' @param smoothing Numeric or \code{NULL}. Length of the smoothing window in hours
#'   (1–24). If \code{NULL} (default), no smoothing is applied. If provided, the
#'   dendrometer series is smoothed using \code{smooth_dm(method = "median_mean")}
#'   prior to phase classification.
#'
#' @return
#' A list of class \code{"SC_output"} containing:
#' \describe{
#'   \item{SC_cycle}{A tibble with one row per contiguous phase, including:
#'     \itemize{
#'       \item \code{Phases} – Phase type (1 = Shrinkage, 2 = Expansion, 3 = Increment)
#'       \item \code{Start}, \code{End} – \code{POSIXct} start and end time of the phase
#'       \item \code{Duration_h}, \code{Duration_m} – Phase duration (hours, minutes)
#'       \item \code{Magnitude} – Change in dendrometer value during the phase (measurement unit)
#'       \item \code{rate} – Rate of change expressed in (\code{Magnitude*1000/Duration_h}) (eg. \eqn{\mu}m/hour)
#'       \item \code{DOY} – Day-of-year at phase start
#'     }}
#'   \item{SC_phase}{A tibble of point-level values including:
#'     \itemize{
#'       \item \code{TIME} – timestamp
#'       \item \code{dm} – dendrometer measurement
#'       \item \code{Phases} – phase assignment for each timestamp
#'     }}
#' }
#'
#' @seealso
#' \code{\link{phase.zg}} for the zero-growth approach;
#' \code{\link{smooth_dm}} for smoothing dendrometer series.
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(gf_nepa17)
#'
#' # Apply stem-cycle approach without smoothing
#' sc1 <- phase.sc(df = gf_nepa17, TreeNum = 1)
#' head(sc1$SC_cycle, 5)
#' head(sc1$SC_phase, 5)
#'
#' # Apply with 12-hour smoothing to reduce noise
#' sc2 <- phase.sc(df = gf_nepa17, TreeNum = 1, smoothing = 12)
#' head(sc2$SC_cycle, 5)
#' }
#'
#' @importFrom lubridate ymd_hms yday
#' @importFrom dplyr mutate select rename
#' @importFrom tibble tibble as_tibble
#' @importFrom tidyr pivot_longer
#' @importFrom pspline smooth.Pspline
#'
#' @export

phase.sc<-function(df, TreeNum, smoothing=NULL){
  ##################### function to calculate phases################
  phase_cal <- function(y){
    all_max <- cummax(y)
    all_max_diff <-diff(all_max)
    #all_max_diff <-all_max_diff[2:length(all_max_diff)]
    phases<- vector('integer', length(y)-1)
    # phases[all_max_diff > 0] <- 3
    y_diff <- diff(y)
    # phases[all_max_diff = 0 & y_diff>=0] <- 2
    # phases[all_max_diff = 0 & y_diff<0] <- 1
    phases<-sapply(seq_along(all_max_diff), function(i){
      if(all_max_diff[i]>0){
        ph <- 3
      }else{
        if(y_diff[i]>=0){
          ph <- 2
        }else{
          ph <- 1
        }
      }
      #print(length(phases))
      return(ph)
    })
    return(phases)
  }
  #####################function to cal phase stats #################
  phase_sats<-function(dm_data, y){
    ph_diff<-c(0,diff(y))
    dy<-yday(c(dm_data$TIME[1], dm_data$TIME[ph_diff!=0]))
    ph<-c(y[1],y[ph_diff!=0])
    magn<-c(dm_data$dm[1], dm_data$dm[ph_diff!=0], dm_data$dm[nrow(dm_data)])
    magn<-diff(magn)
    strt_t<-c(dm_data$TIME[1], dm_data$TIME[ph_diff!=0])
    #strt_t<-strt_t[1:length(strt_t)-1]
    end_t <- c(strt_t[2:length(strt_t)], dm_data$TIME[nrow(dm_data)])
    dur_h<-as.numeric(difftime(end_t, strt_t), units='hours')
    dur_m<-as.numeric(difftime(end_t, strt_t), units='mins')
    out <- tibble(
      'Phases' = ph,
      'Start' = strt_t,
      'End' = end_t,
      'Duration_h' = dur_h,
      'Duration_m' = dur_m,
      'Magnitude' = magn,
      'rate' = magn*1000/dur_h,
      'DOY' = dy
    )
    return(out)
  }
  #####################function to calculate resolution ############
  reso_den<-function(input_time){
    time1<-input_time
    reference<-time1[1]
    time_min<-as.integer(difftime(time1,reference, units = "mins"))
    diff_time<-diff(time_min)
    diff2_time<-unique(diff_time)
    reso<-mean(diff2_time)
    if (length(diff2_time) > 1) {
      print(diff2_time)
      warning('Warning: The temporal resolution of dendrometer data is not consistent, For better result, please use dendrometer data with consistent resolution. There may be NA values in the dataset.')
      #cat(paste('Mean temporal resolution is :', round(reso),' minutes.'))
    }else{
      reso<-mean(diff2_time)
      #cat(paste('Temporal resolution is :', round(reso),' minutes.'))
      return(round(reso))
    }
  }
  ##################################################################
  if (!inherits(df[[1]], 'Date') && !inherits(df[[1]], 'POSIXct')) {
    df[[1]] <- ymd_hms(df[[1]])
  }
  dm_data <- as_tibble(df)%>%
    dplyr::select(c(1,TreeNum+1))
  sf<-smoothing
  dm_data <- dm_data %>%
    dplyr::rename(TIME = 1,
           dm = 2)
  r.denro<-reso_den(dm_data$TIME)
  if(is.null(sf)==T){
    y_sm <- dm_data$dm
  }else{
    if(sf<1|sf>24){
      stop('smoothing must be between 1 and 24.')
    }else{
      warning(paste('You are applying smoothing to raw DM data! The smoothing value is ',sf,' hour(s).'))
      y_sm<-smooth_dm(time = dm_data$TIME, dm = dm_data$dm, method = 'median_mean', window_hours = sf)
    }
  }
  y<-phase_cal(y_sm)
  ph_st<-phase_sats(dm_data, y)
  dm_data <- dm_data%>%
    dplyr::mutate('Phases' = c(NA,y))
  out<-list(SC_cycle = ph_st, SC_phase = tibble(dm_data))
  class(out) <- "SC_output"
  return(out)
}

#' @title Dendrometer data of Kathmandu for 2017
#' @description Dendrometer data from three Chir pine tree collected in hourly resolution for 2017.
#' @format A data frame with 8753 rows and 3 variables:
#' \describe{
#'   \item{\code{Time}}{datetime time of data recording}
#'   \item{\code{T2}}{double reading for first tree}
#'   \item{\code{T3}}{double reading for second tree}
#'}
"nepa17"

#' @title Plot method for daily dendrometer statistics
#'
#' @description
#' Unified S3 plotting method for objects returned by \code{daily.data()}.
#' Missing grouping labels are removed before grouped boxplots are drawn so
#' \code{NA} does not appear on the x-axis.
#'
#' @param x Object of class \code{"daily_output"}.
#' @param y Unused.
#' @param Year Optional numeric year or vector of years for subsetting.
#' @param DOY Optional numeric vector of length 2 giving the start and end
#'   day-of-year for subsetting.
#' @param type Plot type. One of
#'   \code{"summary"}, \code{"amplitude"}, \code{"minmax_ribbon"},
#'   \code{"timing"}, \code{"timing_violin"}, \code{"lag"}, \code{"change"},
#'   \code{"boxplot"}, or \code{"heatmap"}.
#' @param stat Statistic used for \code{type = "boxplot"} or
#'   \code{type = "heatmap"}. One of
#'   \code{"amplitude"}, \code{"Max_diff"}, \code{"mean"},
#'   \code{"median"}, \code{"Max"}, \code{"Min"},
#'   \code{"lag_h"}, \code{"Time_min_h"}, \code{"Time_max_h"}.
#' @param by Grouping for \code{type = "boxplot"}:
#'   \code{"month"}, \code{"month_of_year"}, or \code{"year"}.
#'   \code{"month"} uses a continuous calendar-date axis spanning the selected
#'   data window and supports multi-year plotting, for example 2022--2024.
#'   Empty calendar months may appear as gaps, but months with available data
#'   are not clipped at the start or end of the axis.
#' @param box_style Style for \code{type = "boxplot"}:
#'   \code{"boxplot"}, \code{"violin"}, or \code{"both"}.
#' @param status_cols Colors for \code{Day_status}.
#' @param timing_segment_cols Named vector of two colors for the timing connector:
#'   one for \code{"max after min"} and one for \code{"max before/equal min"}.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @details
#' Plot types:
#' \itemize{
#'   \item \code{"summary"}: daily Min, mean, median, and Max.
#'   \item \code{"amplitude"}: daily amplitude through time.
#'   \item \code{"minmax_ribbon"}: ribbon between daily Min and Max.
#'   \item \code{"timing"}: per-day line connecting daily minimum and maximum timing.
#'   \item \code{"timing_violin"}: violin distribution of \code{Time_min_h}
#'         and \code{Time_max_h} for the selected window.
#'   \item \code{"lag"}: signed lag between daily maximum and minimum time.
#'   \item \code{"change"}: day-to-day change in daily maximum
#'         \code{Max_diff}.
#'   \item \code{"boxplot"}: grouped distributions of a chosen daily statistic.
#'         For \code{by = "month"}, monthly groups are plotted on a continuous
#'         date axis. The axis limits are extended beyond the first and last
#'         plotted month so edge months are not clipped.
#'   \item \code{"heatmap"}: year x day-of-year heatmap of a chosen daily statistic.
#' }
#'
#' For \code{type = "timing"}, the red and blue points are not connected across
#' days. Instead, each day has its own segment connecting minimum and maximum
#' time.
#'
#' @examples
#' \donttest{
#' data(nepa17)
#' dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
#'
#' plot(dd)
#' plot(dd, type = "timing")
#' plot(dd, type = "timing", Year = 2017, DOY = c(1, 6))
#' plot(dd, type = "timing_violin", Year = 2017, DOY = c(1, 6))
#' plot(dd, type = "boxplot", stat = "amplitude", by = "month")
#' plot(dd, type = "boxplot", stat = "amplitude", by = "month_of_year",
#'      box_style = "both")
#' plot(dd, type = "change")
#' }
#'
#' @method plot daily_output
#' @importFrom dplyr mutate filter %>%
#' @importFrom tidyr pivot_longer
#' @importFrom lubridate yday year month floor_date
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon geom_col geom_point
#'   geom_segment geom_boxplot geom_violin geom_tile geom_hline
#'   theme_minimal theme element_text labs scale_color_manual
#'   scale_fill_manual scale_y_continuous scale_x_continuous scale_x_discrete
#'   scale_x_date position_jitter expansion
#' @export
plot.daily_output <- function(x,
                              y = NULL,
                              Year = NULL,
                              DOY = NULL,
                              type = c("summary", "amplitude", "minmax_ribbon",
                                       "timing", "timing_violin", "lag", "change",
                                       "boxplot", "heatmap"),
                              stat = c("amplitude", "Max_diff", "mean",
                                       "median", "Max", "Min",
                                       "lag_h", "Time_min_h", "Time_max_h"),
                              by = c("month", "month_of_year", "year"),
                              box_style = c("boxplot", "violin", "both"),
                              status_cols = c(
                                growing = "forestgreen",
                                shrinking = "firebrick",
                                stable = "grey60"
                              ),
                              timing_segment_cols = c(
                                "max after min" = "grey65",
                                "max before/equal min" = "black"
                              ),
                              ...) {

  DATE <- value <- Statistic <- Hour <- group_label <- Day_status <- DOYv <- Yearv <- NULL
  Time_min_h <- Time_max_h <- order_flag <- NULL
  Min <- Max <- amplitude <- Remarks <- lag_h <- Max_diff <- NULL
  group_date <- month_index <- NULL

  if (!inherits(x, "daily_output")) {
    stop("Input must be an object of class 'daily_output'.")
  }

  type <- match.arg(type)
  stat <- match.arg(stat)
  by <- match.arg(by)
  box_style <- match.arg(box_style)

  .subset_daily <- function(dat, Year = NULL, DOY = NULL) {
    out <- dat

    if (!is.null(Year)) {
      out <- out[lubridate::year(out$DATE) %in% Year, , drop = FALSE]
    }

    if (!is.null(DOY)) {
      if (length(DOY) != 2) {
        stop("'DOY' must be a numeric vector of length 2.")
      }

      doy_min <- min(DOY)
      doy_max <- max(DOY)

      out <- out[
        lubridate::yday(out$DATE) >= doy_min &
          lubridate::yday(out$DATE) <= doy_max,
        ,
        drop = FALSE
      ]
    }

    out <- out[order(out$DATE), , drop = FALSE]

    if (nrow(out) == 0) {
      stop("No data available for the selected Year/DOY window.")
    }

    out
  }

  dat <- .subset_daily(x, Year = Year, DOY = DOY)

  if (type == "summary") {
    pdat <- tidyr::pivot_longer(
      dat,
      cols = c(Min, mean, median, Max),
      names_to = "Statistic",
      values_to = "value"
    )

    p <- ggplot2::ggplot(
      pdat,
      ggplot2::aes(x = DATE, y = value, color = Statistic)
    ) +
      ggplot2::geom_line(linewidth = 0.6) +
      ggplot2::scale_color_manual(
        values = c(
          Min = "firebrick",
          mean = "black",
          median = "grey40",
          Max = "steelblue"
        )
      ) +
      ggplot2::labs(
        x = "Date",
        y = "Daily statistic",
        color = "Statistic"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "amplitude") {
    p <- ggplot2::ggplot(dat, ggplot2::aes(x = DATE, y = amplitude)) +
      ggplot2::geom_col(fill = "steelblue", alpha = 0.8) +
      ggplot2::geom_line(color = "black", linewidth = 0.35) +
      ggplot2::labs(
        x = "Date",
        y = "Daily amplitude"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "minmax_ribbon") {
    p <- ggplot2::ggplot(dat, ggplot2::aes(x = DATE)) +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = Min, ymax = Max),
        fill = "grey80",
        alpha = 0.8
      ) +
      ggplot2::geom_line(
        ggplot2::aes(y = Min),
        color = "firebrick",
        linewidth = 0.5
      ) +
      ggplot2::geom_line(
        ggplot2::aes(y = Max),
        color = "steelblue",
        linewidth = 0.5
      ) +
      ggplot2::geom_line(
        ggplot2::aes(y = mean),
        color = "black",
        linewidth = 0.6
      ) +
      ggplot2::labs(
        x = "Date",
        y = "Daily Min-Max envelope"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "timing") {
    tdat <- dat %>%
      dplyr::mutate(
        order_flag = ifelse(
          Remarks == "*",
          "max after min",
          "max before/equal min"
        )
      )

    p <- ggplot2::ggplot(tdat, ggplot2::aes(x = DATE)) +
      ggplot2::geom_segment(
        ggplot2::aes(
          xend = DATE,
          y = Time_min_h,
          yend = Time_max_h,
          color = order_flag
        ),
        linewidth = 0.55
      ) +
      ggplot2::geom_point(
        ggplot2::aes(y = Time_min_h),
        color = "firebrick",
        size = 1.9
      ) +
      ggplot2::geom_point(
        ggplot2::aes(y = Time_max_h),
        color = "steelblue",
        size = 1.9
      ) +
      ggplot2::scale_color_manual(
        values = timing_segment_cols,
        name = "Order"
      ) +
      ggplot2::scale_y_continuous(
        limits = c(0, 24),
        breaks = seq(0, 24, 4)
      ) +
      ggplot2::labs(
        x = "Date",
        y = "Hour of day"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        legend.position = "top",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "timing_violin") {
    pdat <- tidyr::pivot_longer(
      dat,
      cols = c(Time_min_h, Time_max_h),
      names_to = "Statistic",
      values_to = "Hour"
    ) %>%
      dplyr::filter(!is.na(Hour))

    if (nrow(pdat) == 0) {
      stop("No timing values available for the selected Year/DOY window.")
    }

    p <- ggplot2::ggplot(
      pdat,
      ggplot2::aes(x = Statistic, y = Hour, fill = Statistic)
    ) +
      ggplot2::geom_violin(alpha = 0.7, trim = FALSE) +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = 0.08, height = 0),
        size = 1,
        alpha = 0.35
      ) +
      ggplot2::scale_fill_manual(
        values = c(
          Time_min_h = "firebrick",
          Time_max_h = "steelblue"
        ),
        labels = c(
          Time_min_h = "Time of minimum",
          Time_max_h = "Time of maximum"
        )
      ) +
      ggplot2::scale_y_continuous(
        limits = c(0, 24),
        breaks = seq(0, 24, 4)
      ) +
      ggplot2::labs(
        x = NULL,
        y = "Hour of day",
        fill = "Timing"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        legend.position = "top",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "lag") {
    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = DATE, y = lag_h, fill = lag_h >= 0)
    ) +
      ggplot2::geom_col(show.legend = FALSE) +
      ggplot2::geom_hline(yintercept = 0, linetype = 2) +
      ggplot2::scale_fill_manual(
        values = c(`TRUE` = "steelblue", `FALSE` = "firebrick")
      ) +
      ggplot2::labs(
        x = "Date",
        y = "Lag (hours): Time_max - Time_min"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "change") {
    dat$Day_status <- factor(
      dat$Day_status,
      levels = c("shrinking", "stable", "growing")
    )

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = DATE, y = Max_diff, fill = Day_status)
    ) +
      ggplot2::geom_col(na.rm = TRUE) +
      ggplot2::geom_hline(yintercept = 0, linetype = 2) +
      ggplot2::scale_fill_manual(
        values = status_cols,
        na.value = "grey80",
        drop = FALSE
      ) +
      ggplot2::labs(
        x = "Date",
        y = "Change in daily maximum vs previous day",
        fill = "Day status"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "boxplot") {
    bdat <- dat
    use_continuous_time <- FALSE

    if (by == "month") {
      use_continuous_time <- TRUE

      bdat$group_date <- as.Date(lubridate::floor_date(bdat$DATE, "month"))

      month_seq <- seq(
        min(bdat$group_date, na.rm = TRUE),
        max(bdat$group_date, na.rm = TRUE),
        by = "month"
      )

      n_months <- length(month_seq)
      n_years <- length(unique(lubridate::year(month_seq)))

      x_lab <- "Date"

      box_width <- 20
      point_jitter_width <- 5

      date_breaks <- if (n_months <= 24) {
        "1 month"
      } else if (n_months <= 60) {
        "3 months"
      } else if (n_months <= 120) {
        "6 months"
      } else {
        "1 year"
      }

      date_labels <- if (n_years > 1) {
        "%Y-%m"
      } else {
        "%b"
      }

      x_limits <- c(
        min(month_seq, na.rm = TRUE) - 15,
        max(month_seq, na.rm = TRUE) + 35
      )

    } else if (by == "month_of_year") {
      bdat$group_label <- factor(
        as.character(lubridate::month(bdat$DATE, label = TRUE, abbr = TRUE)),
        levels = month.abb
      )

      x_lab <- "Month of year"

    } else {
      full_levels <- as.character(sort(unique(lubridate::year(bdat$DATE))))

      bdat$group_label <- factor(
        as.character(lubridate::year(bdat$DATE)),
        levels = full_levels
      )

      x_lab <- "Year"
    }

    bdat$value <- bdat[[stat]]

    if (use_continuous_time) {
      bdat <- bdat %>%
        dplyr::filter(!is.na(value), !is.na(group_date))
    } else {
      bdat <- bdat %>%
        dplyr::filter(!is.na(value), !is.na(group_label))
    }

    if (nrow(bdat) == 0) {
      stop("No non-missing values available for the selected statistic.")
    }

    if (use_continuous_time) {
      counts <- table(format(bdat$group_date, "%Y-%m"))
    } else {
      counts <- table(bdat$group_label)
    }

    if (all(counts <= 1)) {
      warning("Each group has only one value; distributions collapse to lines/points.")
    }

    if (use_continuous_time) {
      p <- ggplot2::ggplot(
        bdat,
        ggplot2::aes(x = group_date, y = value, group = group_date)
      )

      if (box_style %in% c("violin", "both")) {
        p <- p +
          ggplot2::geom_violin(
            fill = "grey80",
            alpha = 0.8,
            trim = FALSE
          )
      }

      if (box_style %in% c("boxplot", "both")) {
        p <- p +
          ggplot2::geom_boxplot(
            width = if (box_style == "both") 8 else box_width,
            outlier.shape = NA,
            fill = if (box_style == "both") "white" else "grey80"
          )
      }

      p <- p +
        ggplot2::geom_point(
          position = ggplot2::position_jitter(
            width = point_jitter_width,
            height = 0
          ),
          size = 1.2,
          alpha = 0.45,
          color = "grey25"
        ) +
        ggplot2::scale_x_date(
          limits = x_limits,
          date_breaks = date_breaks,
          date_labels = date_labels,
          expand = ggplot2::expansion(mult = c(0, 0))
        ) +
        ggplot2::labs(
          x = x_lab,
          y = stat
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(
            angle = 90,
            hjust = 1,
            vjust = 0.5
          ),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        )

    } else {
      p <- ggplot2::ggplot(
        bdat,
        ggplot2::aes(x = group_label, y = value)
      )

      if (box_style %in% c("violin", "both")) {
        p <- p +
          ggplot2::geom_violin(
            fill = "grey80",
            alpha = 0.8,
            trim = FALSE
          )
      }

      if (box_style %in% c("boxplot", "both")) {
        p <- p +
          ggplot2::geom_boxplot(
            width = if (box_style == "both") 0.18 else 0.65,
            outlier.shape = NA,
            fill = if (box_style == "both") "white" else "grey80"
          )
      }

      p <- p +
        ggplot2::geom_point(
          position = ggplot2::position_jitter(width = 0.12, height = 0),
          size = 1.2,
          alpha = 0.45,
          color = "grey25"
        ) +
        ggplot2::scale_x_discrete(drop = FALSE) +
        ggplot2::labs(
          x = x_lab,
          y = stat
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(
            angle = 90,
            hjust = 1,
            vjust = 0.5
          ),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        )
    }

    print(p)
    return(invisible(p))
  }

  if (type == "heatmap") {
    hdat <- dat

    hdat$Yearv <- lubridate::year(hdat$DATE)
    hdat$DOYv <- lubridate::yday(hdat$DATE)
    hdat$value <- hdat[[stat]]

    hdat <- hdat %>%
      dplyr::filter(!is.na(value), !is.na(Yearv), !is.na(DOYv))

    if (nrow(hdat) == 0) {
      stop("No non-missing values available for the selected statistic.")
    }

    p <- ggplot2::ggplot(
      hdat,
      ggplot2::aes(x = DOYv, y = factor(Yearv), fill = value)
    ) +
      ggplot2::geom_tile() +
      ggplot2::labs(
        x = "Day of year",
        y = "Year",
        fill = stat
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }
}

#' @title Plot method for stem-cycle output
#'
#' @description
#' Unified S3 plotting method for objects returned by \code{phase.sc()}.
#' Supports raw phase timelines, phase ribbons, transition diagnostics,
#' daily/monthly phase balance, cumulative increment, event frequency,
#' phase heatmaps, and daily/monthly boxplots of phase statistics.
#'
#' For \code{type = "balance"}, two plotting modes are available. With
#' \code{balance_mode = "time_of_day"}, phases are drawn as continuous
#' within-day intervals, and the y-axis represents hour of day from 0 to 24.
#' This makes the timing of shrinkage, expansion, and increment visible. For
#' example, if shrinkage occurs from 00:00 to 05:00, expansion from 05:00 to
#' 15:00, and shrinkage again from 15:00 to 24:00, these intervals are shown
#' as continuous coloured blocks along the y-axis.
#'
#' With \code{balance_mode = "duration"}, the older stacked-bar behaviour is
#' used. In that case, the y-axis represents the amount of time spent in each
#' phase. With \code{temporal = "daily"}, the y-axis is hours per day. With
#' \code{temporal = "monthly"}, the y-axis is hours per month.
#'
#' @param x Object of class \code{"SC_output"} returned by \code{phase.sc()}.
#' @param y Unused.
#' @param DOY Optional numeric vector of length 2 giving start and end
#'   day-of-year.
#' @param Year Optional numeric year used together with \code{DOY}.
#' @param type Plot type. One of
#'   \code{"points"}, \code{"ribbon"}, \code{"transition"},
#'   \code{"balance"}, \code{"increment"}, \code{"frequency"},
#'   \code{"heatmap"}, or \code{"boxplot"}.
#' @param temporal Temporal scale: \code{"raw"}, \code{"daily"}, or
#'   \code{"monthly"}. For \code{type = "boxplot"}, use \code{"daily"} or
#'   \code{"monthly"}. For \code{type = "balance"}, \code{"frequency"}, and
#'   \code{"heatmap"}, \code{"raw"} is internally treated as daily summary and
#'   a warning is emitted.
#' @param x_axis X-axis style for time-based plots. One of \code{"time"} or
#'   \code{"doy"}. When \code{"doy"} is used, timestamps are converted to
#'   day-of-year on the x-axis. For \code{temporal = "monthly"}, \code{"doy"}
#'   is not meaningful and time is used instead.
#' @param balance_mode For \code{type = "balance"}, controls how phase balance
#'   is drawn. One of \code{"time_of_day"} or \code{"duration"}.
#'   \code{"time_of_day"} draws continuous phase intervals along the y-axis
#'   from 0 to 24 h, preserving when phases occurred within each day.
#'   \code{"duration"} draws stacked bars showing total time spent in each
#'   phase per day or month.
#' @param stat For \code{type = "boxplot"}, one of
#'   \code{"Duration_h"}, \code{"Duration_m"}, \code{"Magnitude"}, or
#'   \code{"rate"}.
#' @param phase For \code{type = "boxplot"}, one of
#'   \code{"all"}, \code{"Shrinkage"}, \code{"Expansion"}, or
#'   \code{"Increment"}.
#' @param cols Vector of three colours for Shrinkage, Expansion, and Increment.
#' @param phNames Vector of three labels for the three phase names.
#' @param transition_linetype Line type used for transition markers in
#'   \code{type = "transition"}.
#' @param transition_alpha Transparency of transition markers.
#' @param singleton_as_points Logical. If \code{TRUE}, the boxplot panel falls
#'   back to a point plot when every group contains only one value.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @details
#' Plot types:
#' \itemize{
#'   \item \code{"points"}: dendrometer series with points coloured by phase.
#'   \item \code{"ribbon"}: dendrometer series with background ribbons for
#'         contiguous phases.
#'   \item \code{"transition"}: dendrometer series with vertical lines at phase
#'         changes.
#'   \item \code{"balance"}: phase balance through time. With
#'         \code{balance_mode = "time_of_day"}, phase intervals are drawn
#'         continuously along the y-axis from 0 to 24 h, so the timing of
#'         shrinkage, expansion, and increment within each day is visible.
#'         With \code{balance_mode = "duration"}, stacked bars show total time
#'         spent in each phase per day or month.
#'   \item \code{"increment"}: cumulative increment, phase 3, over time.
#'   \item \code{"frequency"}: number of phase events starting per day or month.
#'   \item \code{"heatmap"}: dominant phase by hour-of-day across dates or months.
#'   \item \code{"boxplot"}: distribution of selected cycle statistics after
#'         assembling cycle summaries to the chosen daily or monthly time scale.
#' }
#'
#' The \code{"frequency"} plot uses \code{SC_cycle}.
#' The \code{"heatmap"} plot uses \code{SC_phase}.
#' The \code{"transition"} plot is especially useful for checking whether
#' smoothing reduced spurious phase switching.
#'
#' @examples
#' \donttest{
#' # data(gf_nepa17)
#' # sc <- phase.sc(df = gf_nepa17, TreeNum = 1, smoothing = 12)
#'
#' # plot(sc)
#' # plot(sc, type = "ribbon")
#' # plot(sc, type = "transition")
#'
#' # Daily within-day phase timing
#' # plot(sc, type = "balance", temporal = "daily")
#'
#' # Daily within-day phase timing for selected DOY range
#' # plot(sc, type = "balance", temporal = "daily",
#' #      DOY = c(150, 160), Year = 2023)
#'
#' # Old stacked-duration balance plot
#' # plot(sc, type = "balance", temporal = "daily",
#' #      balance_mode = "duration")
#'
#' # Monthly stacked duration
#' # plot(sc, type = "balance", temporal = "monthly",
#' #      balance_mode = "duration")
#'
#' # Boxplots
#' # plot(sc, type = "boxplot", stat = "Magnitude", temporal = "monthly")
#' # plot(sc, type = "boxplot", stat = "Duration_h",
#' #      temporal = "daily", phase = "Shrinkage")
#' }
#'
#' @method plot SC_output
#' @importFrom dplyr mutate rename select group_by summarise arrange filter
#'   slice_max ungroup n bind_rows %>%
#' @importFrom tibble tibble
#' @importFrom lubridate floor_date hour minute second yday month year
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_rect geom_col
#'   geom_boxplot geom_tile geom_vline facet_wrap theme_minimal theme_bw theme
#'   element_text labs scale_color_manual scale_fill_manual scale_x_date
#'   scale_x_discrete scale_y_continuous position_jitter
#' @export
plot.SC_output <- function(x,
                           y = NULL,
                           DOY = NULL,
                           Year = NULL,
                           type = c("points", "ribbon", "transition",
                                    "balance", "increment", "frequency",
                                    "heatmap", "boxplot"),
                           temporal = c("raw", "daily", "monthly"),
                           x_axis = c("time", "doy"),
                           balance_mode = c("time_of_day", "duration"),
                           stat = c("Duration_h", "Duration_m", "Magnitude", "rate"),
                           phase = c("all", "Shrinkage", "Expansion", "Increment"),
                           cols = c("#fee8c8", "#fdbb84", "#e34a33"),
                           phNames = c("Shrinkage", "Expansion", "Increment"),
                           transition_linetype = "dashed",
                           transition_alpha = 0.55,
                           singleton_as_points = TRUE,
                           ...) {

  TIME <- dm <- Phases <- Phase <- period <- hours <- value <- group_label <- n <- NULL
  Start <- End <- Duration_h <- Duration_m <- Magnitude <- rate <- hour <- xval <- NULL
  xmin <- xmax <- cum_increment <- X <- group_date <- ymin <- ymax <- NULL

  if (!inherits(x, "SC_output")) {
    stop("Input must be an object of class 'SC_output'.")
  }

  if (!is.character(phNames) || length(phNames) != 3) {
    stop("'phNames' must be a character vector of length 3.")
  }

  if (length(cols) != 3) {
    stop("'cols' must be a vector of length 3.")
  }

  type <- match.arg(type)
  temporal <- match.arg(temporal)
  x_axis <- match.arg(x_axis)
  balance_mode <- match.arg(balance_mode)
  stat <- match.arg(stat)
  phase <- match.arg(phase)

  if (type == "boxplot" && temporal == "raw") {
    stop("For type = 'boxplot', temporal must be 'daily' or 'monthly'.")
  }

  .time_unit <- function(x) {
    switch(
      x,
      raw = NA_character_,
      daily = "day",
      monthly = "month"
    )
  }

  .ensure_posixct <- function(tt) {
    if (inherits(tt, "POSIXct")) {
      return(tt)
    }

    if (inherits(tt, "Date")) {
      return(as.POSIXct(tt))
    }

    out <- suppressWarnings(lubridate::ymd_hms(tt, quiet = TRUE))

    if (all(is.na(out))) {
      out <- suppressWarnings(
        lubridate::parse_date_time(
          tt,
          orders = c(
            "ymd HMS", "ymd HM", "ymd",
            "dmy HMS", "dmy HM", "dmy",
            "mdy HMS", "mdy HM", "mdy"
          ),
          quiet = TRUE
        )
      )
    }

    out
  }

  .resolution_hours <- function(tt) {
    tt <- tt[!is.na(tt)]
    tt <- sort(unique(tt))

    if (length(tt) < 2) {
      stop("Could not determine time resolution because fewer than two timestamps are available.")
    }

    res_h <- stats::median(
      as.numeric(diff(tt), units = "hours"),
      na.rm = TRUE
    )

    if (!is.finite(res_h) || res_h <= 0) {
      stop("Could not determine a valid positive time resolution from the stem-cycle data.")
    }

    res_h
  }

  .hour_decimal <- function(tt) {
    lubridate::hour(tt) +
      lubridate::minute(tt) / 60 +
      lubridate::second(tt) / 3600
  }

  .split_phase_interval_by_day <- function(t0, t1, ph) {
    if (is.na(t0) || is.na(t1) || is.na(ph) || t1 <= t0) {
      return(NULL)
    }

    tz_use <- attr(t0, "tzone")

    if (is.null(tz_use) || length(tz_use) == 0 || is.na(tz_use)) {
      tz_use <- "UTC"
    }

    pieces <- list()
    ii <- 1L
    cur <- t0

    while (cur < t1) {
      cur_date <- as.Date(cur, tz = tz_use)
      next_midnight <- as.POSIXct(cur_date + 1, tz = tz_use)

      seg_end <- if (t1 < next_midnight) {
        t1
      } else {
        next_midnight
      }

      ymin <- .hour_decimal(cur)

      ymax <- if (as.Date(seg_end, tz = tz_use) > cur_date &&
                  abs(.hour_decimal(seg_end)) < 1e-8) {
        24
      } else {
        .hour_decimal(seg_end)
      }

      if (is.finite(ymin) && is.finite(ymax) && ymax > ymin) {
        pieces[[ii]] <- data.frame(
          period = cur_date,
          ymin = ymin,
          ymax = ymax,
          Phases = ph
        )
        ii <- ii + 1L
      }

      cur <- seg_end
    }

    if (length(pieces) == 0) {
      return(NULL)
    }

    dplyr::bind_rows(pieces)
  }

  .build_phase_segments <- function(td_raw) {
    td2 <- td_raw %>%
      dplyr::filter(!is.na(TIME), !is.na(Phases)) %>%
      dplyr::arrange(TIME)

    if (nrow(td2) < 1) {
      stop("No non-missing phase records available for the balance plot.")
    }

    res_h <- .resolution_hours(td2$TIME)
    res_s <- res_h * 3600

    next_time <- c(
      td2$TIME[-1],
      td2$TIME[nrow(td2)] + res_s
    )

    gap_s <- as.numeric(next_time - td2$TIME, units = "secs")

    bad_gap <- !is.finite(gap_s) |
      gap_s <= 0 |
      gap_s > 1.5 * res_s

    if (any(bad_gap)) {
      next_time[bad_gap] <- td2$TIME[bad_gap] + res_s
    }

    seg_list <- vector("list", nrow(td2))

    for (ii in seq_len(nrow(td2))) {
      seg_list[[ii]] <- .split_phase_interval_by_day(
        t0 = td2$TIME[ii],
        t1 = next_time[ii],
        ph = td2$Phases[ii]
      )
    }

    seg <- dplyr::bind_rows(seg_list)

    if (nrow(seg) == 0) {
      stop("Could not construct phase intervals for the balance plot.")
    }

    seg$Phase <- .phase_factor(seg$Phases)

    seg
  }

  .x_values <- function(tt, temporal, x_axis) {
    if (x_axis == "time") {
      return(list(x = tt, xlab = "Time"))
    }

    if (temporal == "monthly") {
      warning("x_axis = 'doy' is not meaningful for temporal = 'monthly'; using time instead.")
      return(list(x = tt, xlab = "Time"))
    }

    list(
      x = lubridate::yday(tt),
      xlab = "Day of year"
    )
  }

  .phase_factor <- function(x) {
    factor(x, levels = c(1, 2, 3), labels = phNames)
  }

  .safe_sum <- function(z) {
    if (all(is.na(z))) {
      NA_real_
    } else {
      sum(z, na.rm = TRUE)
    }
  }

  .aggregate_cycles <- function(cyc, temporal) {
    if (temporal == "raw") {
      return(cyc)
    }

    unit <- .time_unit(temporal)

    cyc %>%
      dplyr::filter(!is.na(Start), !is.na(Phases)) %>%
      dplyr::mutate(period = lubridate::floor_date(Start, unit = unit)) %>%
      dplyr::group_by(period, Phases) %>%
      dplyr::summarise(
        Start = min(Start, na.rm = TRUE),
        End = max(End, na.rm = TRUE),
        Duration_h = .safe_sum(Duration_h),
        Duration_m = .safe_sum(Duration_m),
        Magnitude = .safe_sum(Magnitude),
        rate = {
          mag <- .safe_sum(Magnitude)
          dur <- .safe_sum(Duration_h)

          if (is.na(mag) || is.na(dur) || dur == 0) {
            NA_real_
          } else {
            mag * 1000 / dur
          }
        },
        .groups = "drop"
      )
  }

  .truncate_sc <- function(obj, DOY = NULL, Year = NULL) {
    if (is.null(DOY) && is.null(Year)) {
      td <- obj$SC_phase %>%
        dplyr::rename(TIME = 1, dm = 2, Phases = 3)
    } else {
      if (is.null(DOY) || is.null(Year) || length(DOY) != 2) {
        stop("Provide both DOY = c(start, end) and Year, or neither.")
      }

      td <- dendro.truncate(obj$SC_phase, CalYear = Year, DOY = DOY) %>%
        dplyr::rename(TIME = 1, dm = 2, Phases = 3)
    }

    td$TIME <- .ensure_posixct(td$TIME)

    if (any(is.na(td$TIME))) {
      stop("Some timestamps in SC_phase could not be parsed.")
    }

    if (nrow(td) == 0) {
      stop("No data available for the requested plotting period.")
    }

    cyc <- obj$SC_cycle
    cyc$Start <- .ensure_posixct(cyc$Start)
    cyc$End <- .ensure_posixct(cyc$End)

    cyc <- cyc %>%
      dplyr::filter(
        End >= min(td$TIME, na.rm = TRUE),
        Start <= max(td$TIME, na.rm = TRUE)
      )

    list(td = td, cyc = cyc)
  }

  .aggregate_points <- function(td, temporal) {
    if (temporal == "raw") {
      td$Phase <- .phase_factor(td$Phases)
      return(td)
    }

    unit <- .time_unit(temporal)

    td %>%
      dplyr::filter(!is.na(Phases)) %>%
      dplyr::mutate(period = lubridate::floor_date(TIME, unit = unit)) %>%
      dplyr::group_by(period) %>%
      dplyr::summarise(
        dm = mean(dm, na.rm = TRUE),
        Phases = {
          ptab <- table(Phases)
          as.integer(names(ptab)[which.max(ptab)])
        },
        .groups = "drop"
      ) %>%
      dplyr::rename(TIME = period) %>%
      dplyr::mutate(Phase = .phase_factor(Phases))
  }

  out <- .truncate_sc(x, DOY = DOY, Year = Year)

  td_raw <- out$td
  cyc_raw <- out$cyc

  cyc <- .aggregate_cycles(cyc_raw, temporal = temporal)
  td <- .aggregate_points(td_raw, temporal = temporal)

  if (type %in% c("points", "ribbon", "transition")) {
    if (temporal == "raw") {
      td_plot <- td_raw %>%
        dplyr::mutate(Phase = .phase_factor(Phases))
    } else {
      td_plot <- td
    }
  }

  if (type == "points") {
    xv <- .x_values(td_plot$TIME, temporal, x_axis)
    td_plot$X <- xv$x
    td_plot <- dplyr::filter(td_plot, !is.na(X))

    p <- suppressWarnings(
      ggplot2::ggplot(td_plot, ggplot2::aes(x = X, y = dm)) +
        ggplot2::geom_line(color = "grey50") +
        ggplot2::geom_point(ggplot2::aes(color = Phase), size = 2.4) +
        ggplot2::scale_color_manual(
          values = cols,
          na.value = "grey70",
          name = "Phases",
          limits = phNames
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme_bw() +
        ggplot2::labs(
          title = "Stem-cycle phases over time",
          x = xv$xlab,
          y = if (temporal == "raw") {
            "Stem size variation"
          } else {
            paste0("Stem size variation (", temporal, ")")
          }
        ) +
        ggplot2::theme(
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12)
        )
    )

    print(p)
    return(invisible(p))
  }

  if (type == "ribbon") {
    td_r <- td_plot %>%
      dplyr::filter(!is.na(Phases))

    xv <- .x_values(td_r$TIME, temporal, x_axis)

    td_r$X <- xv$x
    td_r <- dplyr::filter(td_r, !is.na(X))

    rr <- rle(td_r$Phases)
    idx_end <- cumsum(rr$lengths)
    idx_start <- c(1, head(idx_end + 1, -1))

    ribbons <- tibble::tibble(
      xmin = td_r$X[idx_start],
      xmax = td_r$X[idx_end],
      Phase = factor(rr$values, levels = c(1, 2, 3), labels = phNames)
    )

    p <- ggplot2::ggplot(td_r, ggplot2::aes(x = X, y = dm)) +
      ggplot2::geom_rect(
        data = ribbons,
        ggplot2::aes(
          xmin = xmin,
          xmax = xmax,
          ymin = -Inf,
          ymax = Inf,
          fill = Phase
        ),
        inherit.aes = FALSE,
        alpha = 0.18
      ) +
      ggplot2::geom_line(color = "black", linewidth = 0.45) +
      ggplot2::scale_fill_manual(
        values = cols,
        name = "Phases",
        limits = phNames
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Stem-cycle phases as ribbons",
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Stem size variation"
        } else {
          paste0("Stem size variation (", temporal, ")")
        }
      ) +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "transition") {
    td_tr <- td_plot %>%
      dplyr::filter(!is.na(Phases)) %>%
      dplyr::arrange(TIME)

    xv <- .x_values(td_tr$TIME, temporal, x_axis)

    td_tr$X <- xv$x
    td_tr <- dplyr::filter(td_tr, !is.na(X))

    trans_idx <- which(diff(td_tr$Phases) != 0) + 1L

    transitions <- if (length(trans_idx) > 0) {
      td_tr[trans_idx, , drop = FALSE]
    } else {
      td_tr[0, , drop = FALSE]
    }

    p <- ggplot2::ggplot(td_tr, ggplot2::aes(x = X, y = dm)) +
      ggplot2::geom_line(color = "grey35", linewidth = 0.5) +
      ggplot2::geom_point(ggplot2::aes(color = Phase), size = 1.8) +
      ggplot2::geom_vline(
        data = transitions,
        ggplot2::aes(xintercept = X, color = Phase),
        linetype = transition_linetype,
        alpha = transition_alpha,
        show.legend = FALSE
      ) +
      ggplot2::geom_point(
        data = transitions,
        ggplot2::aes(x = X, y = dm, color = Phase),
        size = 3,
        inherit.aes = FALSE
      ) +
      ggplot2::scale_color_manual(
        values = cols,
        na.value = "grey70",
        name = "New phase",
        limits = phNames
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Stem-cycle transitions",
        x = xv$xlab,
        y = if (temporal == "raw") {
          "Stem size variation"
        } else {
          paste0("Stem size variation (", temporal, ")")
        }
      ) +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "balance") {
    unit <- if (temporal == "monthly") {
      "month"
    } else {
      "day"
    }

    if (temporal == "raw") {
      warning("For type = 'balance', raw data are summarised to daily balance.")
      unit <- "day"
    }

    if (balance_mode == "duration") {
      res_h <- .resolution_hours(td_raw$TIME)

      bal <- td_raw %>%
        dplyr::filter(!is.na(Phases)) %>%
        dplyr::mutate(
          period = lubridate::floor_date(TIME, unit = unit),
          Phase = .phase_factor(Phases)
        ) %>%
        dplyr::group_by(period, Phase) %>%
        dplyr::summarise(
          hours = dplyr::n() * res_h,
          .groups = "drop"
        )

      if (x_axis == "doy" && unit == "day") {
        bal$X <- lubridate::yday(bal$period)
        x_lab <- "Day of year"
      } else {
        if (x_axis == "doy" && unit == "month") {
          warning("x_axis = 'doy' is not meaningful for monthly balance; using time instead.")
        }

        bal$X <- bal$period
        x_lab <- if (unit == "month") {
          "Month"
        } else {
          "Date"
        }
      }

      bal <- dplyr::filter(bal, !is.na(X))

      p <- ggplot2::ggplot(
        bal,
        ggplot2::aes(x = X, y = hours, fill = Phase)
      ) +
        ggplot2::geom_col() +
        ggplot2::scale_fill_manual(
          values = cols,
          name = "Phases",
          limits = phNames
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme_bw() +
        ggplot2::labs(
          title = "Phase balance",
          x = x_lab,
          y = if (unit == "month") {
            "Hours per month"
          } else {
            "Hours per day"
          }
        ) +
        ggplot2::theme(
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12)
        )

      print(p)
      return(invisible(p))
    }

    if (balance_mode == "time_of_day") {
      if (unit == "month") {
        warning(
          "balance_mode = 'time_of_day' is designed for daily phase timing. ",
          "Using daily phase timing even though temporal = 'monthly'."
        )
        unit <- "day"
      }

      seg <- .build_phase_segments(td_raw)

      if (x_axis == "doy") {
        seg$X <- lubridate::yday(seg$period)
        seg$xmin <- seg$X - 0.45
        seg$xmax <- seg$X + 0.45
        x_lab <- "Day of year"

        p <- ggplot2::ggplot(
          seg,
          ggplot2::aes(
            xmin = xmin,
            xmax = xmax,
            ymin = ymin,
            ymax = ymax,
            fill = Phase
          )
        ) +
          ggplot2::geom_rect(color = NA)
      } else {
        seg$X <- as.Date(seg$period)
        seg$xmin <- seg$X - 0.45
        seg$xmax <- seg$X + 0.45
        x_lab <- "Date"

        p <- ggplot2::ggplot(
          seg,
          ggplot2::aes(
            xmin = xmin,
            xmax = xmax,
            ymin = ymin,
            ymax = ymax,
            fill = Phase
          )
        ) +
          ggplot2::geom_rect(color = NA) +
          ggplot2::scale_x_date()
      }

      p <- p +
        ggplot2::scale_fill_manual(
          values = cols,
          name = "Phases",
          limits = phNames
        ) +
        ggplot2::scale_y_continuous(
          limits = c(0, 24),
          breaks = seq(0, 24, by = 4),
          expand = c(0, 0)
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme_bw() +
        ggplot2::labs(
          title = "Daily phase timing",
          x = x_lab,
          y = "Hour of day"
        ) +
        ggplot2::theme(
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 12),
          axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5)
        )

      print(p)
      return(invisible(p))
    }
  }

  if (type == "increment") {
    inc <- cyc %>%
      dplyr::filter(Phases == 3L) %>%
      dplyr::arrange(Start)

    if (nrow(inc) == 0) {
      stop("No increment phases available in the selected period.")
    }

    if (temporal == "daily") {
      inc <- inc %>%
        dplyr::mutate(period = lubridate::floor_date(Start, "day")) %>%
        dplyr::group_by(period) %>%
        dplyr::summarise(
          Magnitude = sum(Magnitude, na.rm = TRUE),
          .groups = "drop"
        ) %>%
        dplyr::mutate(
          xval = period,
          cum_increment = cumsum(Magnitude)
        )
    } else if (temporal == "monthly") {
      inc <- inc %>%
        dplyr::mutate(period = lubridate::floor_date(Start, "month")) %>%
        dplyr::group_by(period) %>%
        dplyr::summarise(
          Magnitude = sum(Magnitude, na.rm = TRUE),
          .groups = "drop"
        ) %>%
        dplyr::mutate(
          xval = period,
          cum_increment = cumsum(Magnitude)
        )
    } else {
      inc <- inc %>%
        dplyr::mutate(
          xval = Start,
          cum_increment = cumsum(Magnitude)
        )
    }

    xv <- .x_values(inc$xval, temporal, x_axis)

    inc$X <- xv$x
    inc <- dplyr::filter(inc, !is.na(X))

    p <- ggplot2::ggplot(inc, ggplot2::aes(x = X, y = cum_increment)) +
      ggplot2::geom_line(color = cols[3], linewidth = 0.7) +
      ggplot2::geom_point(color = cols[3], size = 2) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Cumulative increment",
        x = xv$xlab,
        y = "Cumulative increment"
      ) +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "frequency") {
    unit <- if (temporal == "monthly") {
      "month"
    } else {
      "day"
    }

    if (temporal == "raw") {
      warning("For type = 'frequency', raw data are summarised to daily event counts.")
    }

    freq <- cyc %>%
      dplyr::filter(!is.na(Phases)) %>%
      dplyr::mutate(
        period = lubridate::floor_date(Start, unit = unit),
        Phase = .phase_factor(Phases)
      ) %>%
      dplyr::group_by(period, Phase) %>%
      dplyr::summarise(n = dplyr::n(), .groups = "drop")

    if (x_axis == "doy" && unit == "day") {
      freq$X <- lubridate::yday(freq$period)
      x_lab <- "Day of year"
    } else {
      if (x_axis == "doy" && unit == "month") {
        warning("x_axis = 'doy' is not meaningful for monthly frequency; using time instead.")
      }

      freq$X <- freq$period
      x_lab <- if (unit == "month") {
        "Month"
      } else {
        "Date"
      }
    }

    freq <- dplyr::filter(freq, !is.na(X))

    p <- ggplot2::ggplot(freq, ggplot2::aes(x = X, y = n, fill = Phase)) +
      ggplot2::geom_col() +
      ggplot2::scale_fill_manual(
        values = cols,
        name = "Phases",
        limits = phNames
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Frequency of stem-cycle events",
        x = x_lab,
        y = if (unit == "month") {
          "Number of events per month"
        } else {
          "Number of events per day"
        }
      ) +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "heatmap") {
    unit <- if (temporal == "monthly") {
      "month"
    } else {
      "day"
    }

    if (temporal == "raw") {
      warning("For type = 'heatmap', raw data are summarised to daily dominant phase by hour.")
    }

    hm <- td_raw %>%
      dplyr::filter(!is.na(Phases)) %>%
      dplyr::mutate(
        period = if (unit == "month") {
          format(lubridate::floor_date(TIME, unit = "month"), "%Y-%m")
        } else {
          as.Date(TIME)
        },
        hour = factor(lubridate::hour(TIME), levels = 23:0),
        Phase = .phase_factor(Phases)
      ) %>%
      dplyr::group_by(period, hour, Phase) %>%
      dplyr::summarise(n = dplyr::n(), .groups = "drop") %>%
      dplyr::group_by(period, hour) %>%
      dplyr::slice_max(order_by = n, n = 1, with_ties = FALSE) %>%
      dplyr::ungroup()

    if (unit == "day" && x_axis == "doy") {
      hm$X <- lubridate::yday(hm$period)
      x_lab <- "Day of year"
    } else {
      if (unit == "month" && x_axis == "doy") {
        warning("x_axis = 'doy' is not meaningful for monthly heatmap; using time instead.")
      }

      hm$X <- if (unit == "month") {
        format(hm$period, "%Y-%m")
      } else {
        format(hm$period, "%Y-%m-%d")
      }

      x_lab <- if (unit == "month") {
        "Month"
      } else {
        "Date"
      }
    }

    p <- ggplot2::ggplot(hm, ggplot2::aes(x = X, y = hour, fill = Phase)) +
      ggplot2::geom_tile(color = "white", linewidth = 0.15) +
      ggplot2::scale_fill_manual(
        values = cols,
        name = "Phases",
        limits = phNames
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme_bw() +
      ggplot2::labs(
        title = "Dominant phase heatmap",
        x = x_lab,
        y = "Hour of day"
      ) +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "boxplot") {
    cyc_box <- cyc %>%
      dplyr::mutate(Phase = .phase_factor(Phases))

    if (phase != "all") {
      cyc_box <- cyc_box %>%
        dplyr::filter(Phase == phase)
    }

    use_continuous_time <- FALSE

    if (temporal == "daily") {
      if (x_axis == "doy") {
        full_levels <- as.character(1:366)

        cyc_box$group_label <- factor(
          as.character(lubridate::yday(cyc_box$Start)),
          levels = full_levels
        )

        x_lab <- "Day of year"
      } else {
        use_continuous_time <- TRUE

        full_dates <- seq(
          as.Date(min(td_raw$TIME, na.rm = TRUE)),
          as.Date(max(td_raw$TIME, na.rm = TRUE)),
          by = "day"
        )

        cyc_box$group_date <- as.Date(cyc_box$Start)

        x_lab <- "Date"
        x_limits <- range(full_dates)
        box_width <- 0.8
        point_jitter_width <- 0.2
        date_breaks <- if (length(full_dates) > 120) "3 months" else "1 month"
        date_labels <- if (length(unique(format(full_dates, "%Y"))) > 1) "%Y-%m" else "%m-%d"
      }
    } else {
      if (x_axis == "doy") {
        warning("x_axis = 'doy' is not meaningful for monthly boxplots; using month instead.")
      }

      use_continuous_time <- TRUE

      full_dates <- seq(
        as.Date(lubridate::floor_date(min(td_raw$TIME, na.rm = TRUE), "month")),
        as.Date(lubridate::floor_date(max(td_raw$TIME, na.rm = TRUE), "month")),
        by = "month"
      )

      cyc_box$group_date <- as.Date(lubridate::floor_date(cyc_box$Start, "month"))

      x_lab <- "Date"
      x_limits <- range(full_dates)
      box_width <- 20
      point_jitter_width <- 5
      date_breaks <- if (length(full_dates) > 24) "6 months" else "1 month"
      date_labels <- if (length(unique(format(full_dates, "%Y"))) > 1) "%Y-%m" else "%b"
    }

    cyc_box$value <- cyc_box[[stat]]

    if (use_continuous_time) {
      cyc_box <- cyc_box %>%
        dplyr::filter(!is.na(value), !is.na(group_date))
    } else {
      cyc_box <- cyc_box %>%
        dplyr::filter(!is.na(value), !is.na(group_label))
    }

    if (nrow(cyc_box) == 0) {
      stop("No non-missing values available for the selected statistic/phase.")
    }

    if (use_continuous_time) {
      count_key <- format(cyc_box$group_date, "%Y-%m-%d")

      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        count_key <- paste(count_key, cyc_box$Phase, sep = "_")
      }
    } else {
      count_key <- as.character(cyc_box$group_label)

      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        count_key <- paste(count_key, cyc_box$Phase, sep = "_")
      }
    }

    counts <- table(count_key)
    singleton_groups <- all(counts <= 1)

    if (singleton_groups) {
      warning("Each group has only one value; boxplots collapse to lines.")
    }

    if (singleton_groups && isTRUE(singleton_as_points)) {
      if (use_continuous_time) {
        p <- ggplot2::ggplot(
          cyc_box,
          ggplot2::aes(x = group_date, y = value, color = Phase)
        ) +
          ggplot2::geom_point(
            position = ggplot2::position_jitter(width = point_jitter_width, height = 0),
            size = 2,
            alpha = 0.8
          ) +
          ggplot2::scale_color_manual(
            values = cols,
            name = "Phases",
            limits = phNames
          ) +
          ggplot2::scale_x_date(
            limits = x_limits,
            date_breaks = date_breaks,
            date_labels = date_labels
          ) +
          ggplot2::theme_minimal() +
          ggplot2::theme_bw() +
          ggplot2::labs(
            title = paste("Values of", stat),
            x = x_lab,
            y = stat
          ) +
          ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
            axis.title = ggplot2::element_text(size = 14),
            axis.text = ggplot2::element_text(size = 11)
          )
      } else {
        p <- ggplot2::ggplot(
          cyc_box,
          ggplot2::aes(x = group_label, y = value, color = Phase)
        ) +
          ggplot2::geom_point(
            position = ggplot2::position_jitter(width = 0.12, height = 0),
            size = 2,
            alpha = 0.8
          ) +
          ggplot2::scale_color_manual(
            values = cols,
            name = "Phases",
            limits = phNames
          ) +
          ggplot2::theme_minimal() +
          ggplot2::theme_bw() +
          ggplot2::labs(
            title = paste("Values of", stat),
            x = x_lab,
            y = stat
          ) +
          ggplot2::theme(
            axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
            axis.title = ggplot2::element_text(size = 14),
            axis.text = ggplot2::element_text(size = 11)
          ) +
          ggplot2::scale_x_discrete(drop = FALSE)
      }

      if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
        p <- p + ggplot2::facet_wrap(~ Phase, scales = "free_y")
      }

      print(p)
      return(invisible(p))
    }

    if (use_continuous_time) {
      p <- ggplot2::ggplot(
        cyc_box,
        ggplot2::aes(
          x = group_date,
          y = value,
          fill = Phase,
          group = interaction(group_date, Phase, drop = TRUE)
        )
      ) +
        ggplot2::geom_boxplot(width = box_width) +
        ggplot2::scale_fill_manual(
          values = cols,
          name = "Phases",
          limits = phNames
        ) +
        ggplot2::scale_x_date(
          limits = x_limits,
          date_breaks = date_breaks,
          date_labels = date_labels
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme_bw() +
        ggplot2::labs(
          title = paste("Boxplot of", stat),
          x = x_lab,
          y = stat
        ) +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        )
    } else {
      p <- ggplot2::ggplot(
        cyc_box,
        ggplot2::aes(x = group_label, y = value, fill = Phase)
      ) +
        ggplot2::geom_boxplot() +
        ggplot2::scale_fill_manual(
          values = cols,
          name = "Phases",
          limits = phNames
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme_bw() +
        ggplot2::labs(
          title = paste("Boxplot of", stat),
          x = x_lab,
          y = stat
        ) +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
          axis.title = ggplot2::element_text(size = 14),
          axis.text = ggplot2::element_text(size = 11)
        ) +
        ggplot2::scale_x_discrete(drop = FALSE)
    }

    if (phase == "all" && length(unique(cyc_box$Phase)) > 1) {
      p <- p + ggplot2::facet_wrap(~ Phase, scales = "free_y")
    }

    print(p)
    return(invisible(p))
  }

  stop("Unknown plot type.")
}

#' @title Locating the maxima of TWD periods
#'
#' @description This function detects the TWD phases, including their beginning (TWDb), using the phase.zg function. Then it calculates the number, time of occurance (Tm) and value of every local maximum within each TWD phase. In addition it calculates the time difference between 'TWDb' and each 'Tm' within each TWD phase.
#'
#' @param df data frame with first column containing date and time in the format \code{yyyy-mm-dd HH:MM:SS}. It should contain data with constant temporal resolution for best results.
#'
#' @param TreeNum numerical value indicating the tree to be analysed. E.g. '1' refers to the first dendrometer data column in \emph{df}.
#'
#' @param smoothing numerical value from 1 to 12 which indicates the length of the smoothing spline, i.e. 1 = 1 hour and 12 = 12 hours. Default is 5.
#'
#' @return A data frame with statistics of maxima in each TWD phase.
#'
#' @importFrom lubridate ymd_hms ymd
#' @importFrom tidyverse tidyverse_packages
#' @importFrom dplyr mutate group_by summarise ungroup %>% rename across select where case_when bind_rows
#' @importFrom tibble as_tibble tibble
#' @importFrom ggplot2 ggplot geom_area geom_rect geom_text aes theme_minimal labs geom_line geom_point facet_wrap theme element_text
#' @importFrom tidyr pivot_longer
#'
#' @examples
#' \donttest{library(dendRoAnalyst)
#' data(gf_nepa17)
#' df1=gf_nepa17[2500:3500,]
#' twd_max<-twd.maxima(df=df1, TreeNum=2)
#' head(twd_max,10)}
#'
#' @export
twd.maxima<-function(df, TreeNum, smoothing=5){
  TIME <- Phases <- mtwd <- NULL
  n<-TreeNum
  sc<-phase.sc(df=df, TreeNum=n, smoothing=smoothing)
  zg<-phase.zg(df=df, TreeNum=n)
  sc1<-sc[[1]]
  zg1<-zg[[1]]
  zg2<-zg[[2]]
  sc1.1<-sc1 %>%
    dplyr::filter(Phases == 1)
  zg1.1<- zg1 %>%
    dplyr::filter(Phases == 1)
  strt.sh<-sc1.1$Start
  end.p <- sapply(sc1.1$Start, function(i){
    return(sc1$End[which(sc1$Start==i)+1])
  })
  #######################################
  res <- sapply(seq_along(strt.sh), function(loc){
    cr.twd<-zg2 %>%
      dplyr::slice(which(TIME==strt.sh[loc]):which(TIME==end.p[loc]))
    mx <- first(max(cr.twd$TWD, na.rm = T))
    loc.mx <- first(which(zg2$TWD==mx))
    return(loc.mx)
  })
  ################################################
  zg2$mtwd<-0
  zg2$mtwd[res]<-1
  mins3<-lapply(seq_along(zg1.1[[1]]), function(twd){
    twd.st<-which(zg2$TIME==zg1.1$Start[twd])
    twd.end<-which(zg2$TIME==zg1.1$End[twd])
    temp1<-zg2%>%
      dplyr::slice(twd.st:twd.end)%>%
      dplyr::filter(mtwd == 1) %>%
      dplyr::mutate(Duration_from_start = difftime(TIME, zg1.1$Start[twd], units = 'mins'),
             Start.time = zg2$TIME[twd.st],
             End.time = zg2$TIME[twd.end],
             twd.number = twd
             )
    temp2 <- temp1%>%
      dplyr::select(c(8,9,1,3,4,7,ncol(temp1)))
    return(temp2)
  })
  return(bind_rows(mins3))
}

#' @title Fitting gompertz function on annual dendrometer data
#'
#' @description This function modells the annual growth of dendrometer data using gompertz function.
#'
#' @param df dataframe with first column containing date and time in the format \code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.
#'
#' @param CalYear numeric for year of calculation. If df has more than one year, assigning CalYear truncates the data of only that year.
#'
#' @param TreeNum numerical value indicating the tree to be analysed. E.g. '1' refers to the first dendrometer data column in \emph{df}.
#'
#' @param f_derivative logical if yes returns first derivative of gompertz curve.
#'
#' @param start A named list of start values for fitting
#'   the Gompertz curve (passed to \code{\link[minpack.lm]{nlsLM}}). Default is
#'   \code{list(b = 0.5, k = 0.005)}.
#' @param verbose logical if TRUE also returns the optimized parameters. Default is
#'  FALSE.
#'
#' @return A data frame with the modelled dendrometer series. If \code{verbose = TRUE}, returns a list
#'  with two data frames. The fitted curve and parameters.
#'
#' @importFrom stats approx median na.exclude na.omit sd predict
#' @importFrom lubridate ymd_hms ymd
#' @importFrom dplyr mutate group_by summarise ungroup %>% rename across select where case_when
#' @importFrom tibble as_tibble tibble
#' @importFrom tidyr pivot_longer
#' @importFrom minpack.lm nlsLM
#'
#' @examples
#' \donttest{library(dendRoAnalyst)
#' data(gf_nepa17)
#' gomp_fitted<-dm.fit.gompertz(df=gf_nepa17, TreeNum = 1, CalYear=2017)
#' head(gomp_fitted,10)}
#'
#' @export
dm.fit.gompertz<- function(df, CalYear, TreeNum, f_derivative=F, start = list(b=0.5, k=0.005), verbose = FALSE){
  #################functions#############################
  gompertz_fit_params<-function(x,y,start){
    a <- max(y)
    gom_mod <- minpack.lm::nlsLM(y~a*exp(-exp(b-k*x_val)),start = start)
    para <- summary(gom_mod)
    b<- para$parameters[1,1]
    k<-para$parameters[2,1]
    return(data.frame(a = a, b = b, k = k))
  }
  gompertz_fit<-function(x,y,f.d, start){
    a <- max(y)
    gom_mod <- minpack.lm::nlsLM(y~a*exp(-exp(b-k*x_val)),start = start)
    para <- summary(gom_mod)
    b<- para$parameters[1,1]
    k<-para$parameters[2,1]
    if(f.d){
      y_pred <- der_gompertz(x,a,b,k)
    }else{
      y_pred <- predict(gom_mod)
    }
    #if (isTRUE(verbose)){
    #  y_pred <- list(pred = y_pred, parameters = data.frame(a=a, b=b, k=k))
    #}
    return(y_pred)
  }
  der_gompertz<-function(doy,a,b,k){
    y.d<-exp(b-k*doy-exp(b-k*doy))
    return(y.d)
  }
  #######################################################
  if(!(length(CalYear)==1)){
    stop('CalYear must be a single value.')
  }
  if (!inherits(df[[1]], "Date") && !inherits(df[[1]], "POSIXct")) {
    df[[1]] <- ymd_hms(df[[1]])
  }
  TIME <- NULL
  df <- as_tibble(df)%>%
    dplyr::rename(TIME = 1)
  if(!(CalYear%in%unique(year(df$TIME)))){
    stop('Provided CalYear does not exist in df')
  }
  df <- df%>%
    dplyr::filter(year(TIME) == CalYear)
  df <- dendro.resample(df = df, by='D', value='mean')
  df <- df %>% dplyr::mutate(across(where(is.numeric), ~ . - first(.)))
  if(TreeNum =='all'){
    df <- df
  }else{
    df <- df%>%
      dplyr::select(c(1,TreeNum+1))
  }
  x_val <- yday(df$TIME)

  res <- apply(df[-1], 2, function(col) gompertz_fit(x_val,col,f_derivative, start = start))
  df <- dplyr::bind_cols(df[1],res)
  if (isTRUE(verbose)){
    para_gomp <- apply(df[-1], 2, function(col) gompertz_fit_params(x_val,col, start = start))
    pms <- dplyr::bind_rows(para_gomp)
    parameters <- cbind(data.frame(Trees = colnames(df)[2:ncol(df)]), pms)
    df <- list(fitted = df, parameters = parameters)
  }
  return(df)
}

# -------------------------------------------------------------------------
# Internal helpers
# -------------------------------------------------------------------------

#' Safe difference for vectors with missing values
#'
#' Returns a vector of the same length as \code{x}. The first value is \code{0}.
#' Differences are only computed where both consecutive values are non-missing;
#' otherwise \code{NA} is returned for that position.
#'
#' @param x Numeric vector.
#'
#' @return Numeric vector of same length as \code{x}.
#' @keywords internal
#' @noRd
safe_diff <- function(x) {
  n <- length(x)
  out <- rep(NA_real_, n)

  if (n == 0) {
    return(out)
  }

  out[1] <- 0

  if (n == 1) {
    return(out)
  }

  prev <- x[-n]
  curr <- x[-1]
  ok <- !is.na(prev) & !is.na(curr)

  out[which(ok) + 1] <- curr[ok] - prev[ok]
  out
}

#' Estimate jump size
#'
#' Estimates the magnitude of a jump at index \code{j}. Two methods are supported:
#' \code{"point_diff"} uses the raw consecutive difference, while
#' \code{"window_median"} uses the difference between the median before and after
#' the jump and is more robust to noise.
#'
#' @param x Numeric vector.
#' @param j Integer index of the jump.
#' @param adjustment_method Either \code{"window_median"} or \code{"point_diff"}.
#' @param adjust_window Integer window size used for \code{"window_median"}.
#'
#' @return Numeric jump offset or \code{NA_real_}.
#' @keywords internal
#' @noRd
estimate_jump_offset <- function(x, j,
                                 adjustment_method = c("window_median", "point_diff"),
                                 adjust_window = 5) {
  adjustment_method <- match.arg(adjustment_method)

  if (length(x) == 0 || j <= 1 || j > length(x)) {
    return(NA_real_)
  }

  if (adjustment_method == "point_diff") {
    if (is.na(x[j]) || is.na(x[j - 1])) {
      return(NA_real_)
    }
    return(x[j] - x[j - 1])
  }

  n <- length(x)

  pre_idx <- seq.int(max(1, j - adjust_window), j - 1)
  post_idx <- seq.int(j, min(n, j + adjust_window - 1))

  pre_vals <- x[pre_idx]
  post_vals <- x[post_idx]

  pre_vals <- pre_vals[!is.na(pre_vals)]
  post_vals <- post_vals[!is.na(post_vals)]

  if (length(pre_vals) < 2 || length(post_vals) < 2) {
    if (is.na(x[j]) || is.na(x[j - 1])) {
      return(NA_real_)
    }
    return(x[j] - x[j - 1])
  }

  stats::median(post_vals) - stats::median(pre_vals)
}

#' Apply jump correction
#'
#' Subtracts the estimated jump offset from all non-missing values from index
#' \code{j} to the end of the series.
#'
#' @param x Numeric vector.
#' @param j Integer index of the jump.
#' @param offset Numeric jump offset.
#' @param set_jump_point_na Logical. If \code{TRUE}, the exact jump point is set to
#' \code{NA} after correction.
#'
#' @return Corrected numeric vector.
#' @keywords internal
#' @noRd
apply_jump_correction <- function(x, j, offset, set_jump_point_na = FALSE) {
  if (length(x) == 0 || is.na(offset) || j > length(x)) {
    return(x)
  }

  idx <- j:length(x)
  idx <- idx[!is.na(x[idx])]

  if (length(idx) > 0) {
    x[idx] <- x[idx] - offset
  }

  if (isTRUE(set_jump_point_na) && j <= length(x)) {
    x[j] <- NA_real_
  }

  x
}

#' Detect jump indices
#'
#' Detects jump locations either manually using a threshold on the differenced
#' series or automatically using \code{changepoint::cpt.mean()} with
#' \code{penalty = "Manual"}.
#'
#' @param diff_dm Numeric differenced series.
#' @param detection_method Either \code{"manual"} or \code{"auto"}.
#' @param manual_threshold Numeric threshold used in manual mode.
#' @param auto_method_penalty Numeric manual penalty value used in automatic mode.
#' @param auto_every_second Logical. If \code{TRUE}, keeps every second detected
#' changepoint to mimic the legacy behavior.
#'
#' @return Integer vector of jump indices.
#' @keywords internal
#' @noRd
detect_jump_indices <- function(diff_dm,
                                detection_method = c("manual", "auto"),
                                manual_threshold = NULL,
                                auto_method_penalty = 10,
                                auto_every_second = TRUE) {
  detection_method <- match.arg(detection_method)

  if (detection_method == "manual") {
    if (is.null(manual_threshold)) {
      stop("'manual_threshold' must be provided when detection_method = 'manual'")
    }
    return(which(!is.na(diff_dm) & abs(diff_dm) >= manual_threshold))
  }

  valid_idx <- which(!is.na(diff_dm))

  if (length(valid_idx) < 2) {
    return(integer(0))
  }

  cp <- tryCatch(
    changepoint::cpt.mean(
      diff_dm[valid_idx],
      method = "PELT",
      penalty = "Manual",
      pen.value = auto_method_penalty
    ),
    error = function(e) NULL
  )

  if (is.null(cp)) {
    return(integer(0))
  }

  locs <- changepoint::cpts(cp)

  if (length(locs) == 0 || all(is.na(locs))) {
    return(integer(0))
  }

  if (isTRUE(auto_every_second) && length(locs) >= 2) {
    locs <- locs[seq(2, length(locs), by = 2)]
  }

  if (length(locs) == 0) {
    return(integer(0))
  }

  valid_idx[locs]
}

#' Plot jumps in a dendrometer series
#'
#' Internal plotting helper for interactive inspection of jumps.
#'
#' @param time_vec POSIXct vector.
#' @param value_vec Numeric vector.
#' @param jump_times Optional POSIXct vector of jump times to mark with vertical lines.
#' @param title Character plot title.
#' @param line_color Character color for vertical lines.
#'
#' @return Invisibly prints a ggplot object.
#' @keywords internal
#' @noRd
plot_jump_series <- function(time_vec, value_vec,
                             jump_times = NULL,
                             title = "",
                             line_color = "red") {
  TIME <- dm <- NULL
  df_plot <- data.frame(TIME = time_vec, dm = value_vec)

  p <- ggplot2::ggplot(df_plot, ggplot2::aes(TIME, dm)) +
    ggplot2::geom_line(color = "black") +
    ggplot2::theme_minimal() +
    ggplot2::xlab("Time") +
    ggplot2::ylab("Stem size variation") +
    ggplot2::ggtitle(title)

  if (!is.null(jump_times) && length(jump_times) > 0) {
    p <- p + ggplot2::geom_vline(
      xintercept = as.numeric(jump_times),
      color = line_color,
      linewidth = 1,
      alpha = 0.6
    )
  }

  print(p)
  invisible(p)
}

# -------------------------------------------------------------------------
# Exported functions
# -------------------------------------------------------------------------

#' @title Removing artefacts due to manual adjustments of dendrometers interactively
#'
#' @description Dendrometers generally have limited memory capacity beyond which
#' they stop recording. To keep the measurement ongoing, they should be adjusted
#' periodically, which can cause positive or negative jumps in the data. This
#' function locates these artefacts and interactively adjusts them one by one.
#'
#' @return A dataframe containing jump-free dendrometer data.
#'
#' @param df Data frame with first column containing date and time in the format
#' \code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.
#' @param TreeNum Numerical value indicating the tree to be analysed. E.g. \code{1}
#' refers to the first dendrometer data column in \code{df}.
#' @param detection_method Either \code{"manual"} for threshold-based jump
#' detection or \code{"auto"} for automatic detection using
#' \code{changepoint::cpt.mean()} with \code{penalty = "Manual"}.
#' @param manual_threshold Numeric threshold considered as artefact when
#' \code{detection_method = "manual"}.
#' @param auto_method_penalty Numeric manual penalty value used in
#' \code{changepoint::cpt.mean()} when \code{detection_method = "auto"}.
#' Larger values generally lead to fewer detected jumps.
#' @param adjustment_method Either \code{"window_median"} (recommended) or
#' \code{"point_diff"}. The former estimates the jump offset from local medians
#' before and after the jump, while the latter uses the raw consecutive
#' difference at the jump.
#' @param adjust_window Integer window size used for robust jump-size estimation
#' when \code{adjustment_method = "window_median"}.
#' @param plot_window Integer number of points shown before and after each jump in
#' the interactive plot.
#' @param auto_every_second Logical. If \code{TRUE}, keeps every second detected
#' changepoint in automatic mode to mimic the legacy behavior.
#' @param set_jump_point_na Logical. If \code{TRUE}, sets the exact jump point to
#' \code{NA} after correction.
#'
#' @importFrom lubridate ymd_hms
#' @importFrom changepoint cpt.mean cpts
#' @importFrom ggplot2 ggplot aes geom_line geom_vline theme_minimal xlab ylab ggtitle
#' @importFrom stats median
#'
#' @export
i.jump.locator <- function(df,
                           TreeNum,
                           detection_method = c("manual", "auto"),
                           manual_threshold = NULL,
                           auto_method_penalty = 10,
                           adjustment_method = c("window_median", "point_diff"),
                           adjust_window = 5,
                           plot_window = 20,
                           auto_every_second = TRUE,
                           set_jump_point_na = FALSE) {
  detection_method <- match.arg(detection_method)
  adjustment_method <- match.arg(adjustment_method)

  if (TreeNum > ncol(df) - 1) {
    stop("Tree number does not exist in the provided dataset")
  }

  if (!inherits(df[[1]], "Date") && !inherits(df[[1]], "POSIXct")) {
    df[[1]] <- lubridate::ymd_hms(df[[1]])
  }

  series_name <- names(df)[TreeNum + 1]

  df1 <- data.frame(
    TIME = df[[1]],
    dm = df[[TreeNum + 1]]
  )

  diff_dm <- safe_diff(df1$dm)

  jl <- detect_jump_indices(
    diff_dm = diff_dm,
    detection_method = detection_method,
    manual_threshold = manual_threshold,
    auto_method_penalty = auto_method_penalty,
    auto_every_second = auto_every_second
  )

  if (length(jl) == 0) {
    message("There is no jump in the dendrometer.")
    names(df1)[2] <- series_name
    return(df1)
  }

  jump.info <- data.frame(
    Index = jl,
    Jump_at = df1$TIME[jl],
    Raw_jump = diff_dm[jl]
  )

  print(jump.info)

  suppressWarnings(
    plot_jump_series(
      time_vec = df1$TIME,
      value_vec = df1$dm,
      jump_times = jump.info$Jump_at,
      title = "Overview of jump(s)",
      line_color = "red"
    )
  )

  invisible(readline(prompt = "Overview of detected jumps shown. Press [enter] to continue..."))
  data <- df1$dm

  for (i in seq_along(jl)) {
    j <- jl[i]
    fl <- max(1, j - plot_window)
    fm <- min(length(data), j + plot_window)

    suppressWarnings(
      plot_jump_series(
        time_vec = df1$TIME[fl:fm],
        value_vec = data[fl:fm],
        jump_times = df1$TIME[j],
        title = paste(df1$TIME[j], ", before adjustment"),
        line_color = "red"
      )
    )

    user_input <- ""
    while (!user_input %in% c("yes", "no")) {
      user_input <- tolower(
        readline(prompt = paste(
          "Do you want to adjust the jump at",
          df1$TIME[j],
          "(yes/no): "
        ))
      )

      if (!user_input %in% c("yes", "no")) {
        print("Invalid input. Please enter 'yes' or 'no'.")
      }
    }

    if (user_input == "no") {
      print(paste("Skipping jump at", df1$TIME[j]))
      next
    }

    offset <- estimate_jump_offset(
      x = data,
      j = j,
      adjustment_method = adjustment_method,
      adjust_window = adjust_window
    )

    print(paste("Estimated jump at", df1$TIME[j], "=", round(offset, 4)))

    data <- apply_jump_correction(
      x = data,
      j = j,
      offset = offset,
      set_jump_point_na = set_jump_point_na
    )

    suppressWarnings(
      plot_jump_series(
        time_vec = df1$TIME[fl:fm],
        value_vec = data[fl:fm],
        jump_times = df1$TIME[j],
        title = paste(df1$TIME[j], ", after adjustment"),
        line_color = "green"
      )
    )
  }

  df1$dm <- data

  suppressWarnings(
    plot_jump_series(
      time_vec = df1$TIME,
      value_vec = df1$dm,
      jump_times = jump.info$Jump_at,
      title = "Overview after adjustment of jump(s)",
      line_color = "green"
    )
  )

  names(df1)[2] <- series_name
  df1
}

#' @title Removing artefacts due to manual adjustments of dendrometers automatically for more than one dendrometer
#'
#' @description Dendrometers generally have limited memory capacity beyond which
#' they stop recording. To keep the measurement ongoing, they should be adjusted
#' periodically, which can cause positive or negative jumps in the data. This
#' function locates these artefacts and adjusts them automatically. Unlike
#' \code{\link{ i.jump.locator }}, it can handle datasets with more than one
#' dendrometer.
#'
#' @return A dataframe containing jump-free dendrometer data.
#'
#' @param df Data frame with first column containing date and time in the format
#' \code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.
#' @param detection_method Either \code{"manual"} for threshold-based jump
#' detection or \code{"auto"} for automatic detection using
#' \code{changepoint::cpt.mean()} with \code{penalty = "Manual"}.
#' @param manual_threshold Numeric threshold considered as artefact when
#' \code{detection_method = "manual"}.
#' @param auto_method_penalty Numeric manual penalty value used in
#' \code{changepoint::cpt.mean()} when \code{detection_method = "auto"}.
#' Larger values generally lead to fewer detected jumps.
#' @param adjustment_method Either \code{"window_median"} (recommended) or
#' \code{"point_diff"}.
#' @param adjust_window Integer window size used for robust jump-size estimation
#' when \code{adjustment_method = "window_median"}.
#' @param auto_every_second Logical. If \code{TRUE}, keeps every second detected
#' changepoint in automatic mode to mimic the legacy behavior.
#' @param set_jump_point_na Logical. If \code{TRUE}, sets the exact jump point to
#' \code{NA} after correction.
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(nepa)
#'
#' # Manual detection
#' jump_free_nepa <- jump.locator(
#'   df = nepa,
#'   detection_method = "manual",
#'   manual_threshold = 1
#' )
#'
#' # Automatic detection with cpt.mean() and penalty = "Manual"
#' jump_free_nepa2 <- jump.locator(
#'   df = nepa,
#'   detection_method = "auto",
#'   auto_method_penalty = 10
#' )
#' }
#'
#' @importFrom lubridate ymd_hms
#' @importFrom changepoint cpt.mean cpts
#' @importFrom stats median
#'
#' @export
jump.locator <- function(df,
                         detection_method = c("manual", "auto"),
                         manual_threshold = NULL,
                         auto_method_penalty = 10,
                         adjustment_method = c("window_median", "point_diff"),
                         adjust_window = 5,
                         auto_every_second = TRUE,
                         set_jump_point_na = FALSE) {
  detection_method <- match.arg(detection_method)
  adjustment_method <- match.arg(adjustment_method)

  if (!inherits(df[[1]], "Date") && !inherits(df[[1]], "POSIXct")) {
    df[[1]] <- lubridate::ymd_hms(df[[1]])
  }

  out <- as.data.frame(df, stringsAsFactors = FALSE)
  names(out)[1] <- "TIME"

  dendro_names <- names(out)[-1]

  for (cl in seq_along(dendro_names)) {
    nm <- dendro_names[cl]
    column <- out[[nm]]

    diff_dm <- safe_diff(column)

    jl <- detect_jump_indices(
      diff_dm = diff_dm,
      detection_method = detection_method,
      manual_threshold = manual_threshold,
      auto_method_penalty = auto_method_penalty,
      auto_every_second = auto_every_second
    )

    if (length(jl) == 0) {
      message(paste("There is no jump in", nm))
      next
    }

    offsets <- rep(NA_real_, length(jl))

    for (i in seq_along(jl)) {
      j <- jl[i]

      offset <- estimate_jump_offset(
        x = column,
        j = j,
        adjustment_method = adjustment_method,
        adjust_window = adjust_window
      )

      offsets[i] <- offset

      column <- apply_jump_correction(
        x = column,
        j = j,
        offset = offset,
        set_jump_point_na = set_jump_point_na
      )
    }

    jump_table <- data.frame(
      Index = jl,
      Jump_time = out$TIME[jl],
      Raw_jump = diff_dm[jl],
      Applied_offset = offsets
    )

    message(paste("Jump information in", nm))
    print(jump_table)

    out[[nm]] <- column
  }

  out
}

#' @title Calculate the mean detrended dendrometer series
#'
#' @description
#' Computes a mean detrended dendrometer series across multiple trees from the
#' output of \code{dm.detrend.fit()}.
#'
#' This is useful for creating one representative detrended series for a species,
#' site, or treatment group after detrending individual dendrometer series.
#'
#' Optionally, the function can:
#' \itemize{
#'   \item calculate a \strong{robust mean} using a trimmed mean across trees,
#'   \item remove temporal autocorrelation from the mean detrended series using
#'   \code{forecast::auto.arima()},
#'   \item rescale the autocorrelation-removed series so it stays non-negative
#'   and has mean = 1 within each vegetation season.
#' }
#'
#' @param detrended_dm An object of class \code{"dm_detrended"} returned by
#'   \code{dm.detrend.fit()}, or a data frame in the same wide format as
#'   \code{$detrended_data}.
#' @param series Optional character vector of tree/series names to include.
#'   Default is \code{NULL}, meaning all available detrended series are used.
#' @param ac1.remove Logical. If \code{TRUE}, removes temporal autocorrelation
#'   from the mean detrended series using \code{forecast::auto.arima()} applied
#'   separately within each season. Default is \code{TRUE}.
#' @param robust.mean Logical. If \code{TRUE}, calculates a trimmed mean across
#'   trees at each time step. Default is \code{TRUE}.
#' @param trim Proportion to trim from each tail when \code{robust.mean = TRUE}.
#'   Default is \code{0.15}.
#' @param seasonal_rescale Logical. If \code{TRUE}, the autocorrelation-removed
#'   series is shifted to non-negative values and rescaled to mean = 1 within
#'   each season. Default is \code{TRUE}.
#'
#' @return
#' A tibble of class \code{"mean_dm_detrended"} containing:
#' \itemize{
#'   \item metadata columns copied from the detrended input,
#'   \item \code{STD_DDM}: the mean detrended series,
#'   \item \code{RES_DDM}: the autocorrelation-removed mean detrended series
#'         (returned only when \code{ac1.remove = TRUE}).
#' }
#'
#' @examples
#' \donttest{
#' fit1 <- dm.growth.fit(
#'   df = gf_nepa17,
#'   TreeNum = 1:2,
#'   method = "gompertz",
#'   year_mode = "yearly",
#'   verbose = FALSE
#' )
#'
#' det1 <- dm.detrend.fit(fit1)
#'
#' m_det <- mean_detrended.dm(det1)
#' head(m_det, 10)
#' }
#'
#' @importFrom dplyr mutate group_by ungroup arrange select all_of %>%
#' @importFrom tibble as_tibble
#' @importFrom stats residuals
#' @export
mean_detrended.dm <- function(detrended_dm,
                              series = NULL,
                              ac1.remove = TRUE,
                              robust.mean = TRUE,
                              trim = 0.15,
                              seasonal_rescale = TRUE) {

  # -----------------------------
  # helpers
  # -----------------------------
  row_trimmed_mean <- function(x, trim = 0.15) {
    if (all(is.na(x))) return(NA_real_)
    mean(x, trim = trim, na.rm = TRUE)
  }

  row_simple_mean <- function(x) {
    if (all(is.na(x))) return(NA_real_)
    mean(x, na.rm = TRUE)
  }

  ac1_remove_one <- function(x) {
    if (all(is.na(x))) return(rep(NA_real_, length(x)))

    ok <- which(!is.na(x))
    if (length(ok) < 3) return(rep(NA_real_, length(x)))

    x_ok <- x[ok]

    fit <- tryCatch(
      forecast::auto.arima(x_ok),
      error = function(e) NULL
    )

    if (is.null(fit)) {
      return(rep(NA_real_, length(x)))
    }

    res <- rep(NA_real_, length(x))
    res[ok] <- as.numeric(stats::residuals(fit))
    res
  }

  rescale_positive_mean1 <- function(x, eps = 1e-6) {
    out <- rep(NA_real_, length(x))
    ok <- !is.na(x)

    if (!any(ok)) return(out)

    x_ok <- x[ok]
    xmin <- min(x_ok, na.rm = TRUE)
    shifted <- x_ok - xmin + eps
    m <- mean(shifted, na.rm = TRUE)

    if (!is.finite(m) || m <= 0) {
      out[ok] <- NA_real_
      return(out)
    }

    out[ok] <- shifted / m
    out
  }

  # -----------------------------
  # input
  # -----------------------------
  if (inherits(detrended_dm, "dm_detrended")) {
    dat <- tibble::as_tibble(detrended_dm$detrended_data)
  } else if (is.data.frame(detrended_dm)) {
    dat <- tibble::as_tibble(detrended_dm)
  } else {
    stop(
      "'detrended_dm' must be either:\n",
      "  1. an object of class 'dm_detrended', or\n",
      "  2. a data frame in the same format as $detrended_data."
    )
  }

  meta_cols <- intersect(
    c("TIME", "doy", "season_label", "season_start", "season_end", "season_day"),
    names(dat)
  )

  value_cols <- setdiff(names(dat), meta_cols)

  if (length(value_cols) == 0) {
    stop("No detrended series columns were found.")
  }

  if (!is.null(series)) {
    miss <- setdiff(series, value_cols)
    if (length(miss) > 0) {
      warning("These requested series were not found and were ignored: ",
              paste(miss, collapse = ", "))
    }
    value_cols <- intersect(value_cols, series)
    if (length(value_cols) == 0) {
      stop("None of the requested series were found.")
    }
  }

  if (!requireNamespace("forecast", quietly = TRUE) && isTRUE(ac1.remove)) {
    stop("Package 'forecast' is required when ac1.remove = TRUE.")
  }

  # -----------------------------
  # mean detrended series
  # -----------------------------
  mat <- as.matrix(dat[, value_cols, drop = FALSE])

  if (isTRUE(robust.mean)) {
    mean_series <- apply(mat, 1, row_trimmed_mean, trim = trim)
  } else {
    mean_series <- apply(mat, 1, row_simple_mean)
  }

  out <- dat[, meta_cols, drop = FALSE]
  out$STD_DDM <- as.numeric(mean_series)

  # -----------------------------
  # remove autocorrelation
  # -----------------------------
  if (isTRUE(ac1.remove)) {

    if ("season_label" %in% names(out)) {
      out <- out %>%
        dplyr::group_by(.data$season_label) %>%
        dplyr::arrange(.data$TIME, .by_group = TRUE) %>%
        dplyr::mutate(
          RES_DDM = ac1_remove_one(.data$STD_DDM)
        ) %>%
        dplyr::ungroup()
    } else {
      out$RES_DDM <- ac1_remove_one(out$STD_DDM)
    }

    if (isTRUE(seasonal_rescale)) {
      if ("season_label" %in% names(out)) {
        out <- out %>%
          dplyr::group_by(.data$season_label) %>%
          dplyr::mutate(
            RES_DDM = rescale_positive_mean1(.data$RES_DDM)
          ) %>%
          dplyr::ungroup()
      } else {
        out$RES_DDM <- rescale_positive_mean1(out$RES_DDM)
      }
    }
  }

  class(out) <- c("mean_dm_detrended", class(out))
  out
}
###----------------------------------------------------------------------------------------------

#' @title Plot mean detrended dendrometer series
#'
#' @description
#' S3 plotting method for objects returned by \code{mean_detrended.dm()}.
#'
#' It can plot:
#' \itemize{
#'   \item the mean detrended series (\code{STD_DDM}),
#'   \item the autocorrelation-removed mean detrended series (\code{RES_DDM}),
#'   \item or both together.
#' }
#'
#' @param x An object of class \code{"mean_dm_detrended"} returned by
#'   \code{mean_detrended.dm()}.
#' @param y Unused.
#' @param type Plot type. One of:
#'   \describe{
#'     \item{`"series"`}{Plot the time series over calendar time, DOY, or season day.}
#'     \item{`"seasonal"`}{Overlay seasons on a season-day scale.}
#'     \item{`"boxplot"`}{Show boxplots of the detrended values by season or metric.}
#'   }
#' @param value Which variable to plot. One of:
#'   \describe{
#'     \item{`"both"`}{Plot both \code{STD_DDM} and \code{RES_DDM} when available.}
#'     \item{`"STD_DDM"`}{Plot only the mean detrended series.}
#'     \item{`"RES_DDM"`}{Plot only the autocorrelation-removed mean detrended series.}
#'   }
#' @param seasons Optional character vector of \code{season_label} values to retain.
#' @param x_axis Character string controlling the x-axis. One of:
#'   \describe{
#'     \item{`"default"`}{Uses calendar date for \code{type = "series"} and
#'       season day for \code{type = "seasonal"}.}
#'     \item{`"date"`}{Use actual calendar date.}
#'     \item{`"doy"`}{Use calendar day-of-year.}
#'     \item{`"season_day"`}{Use vegetation season day.}
#'   }
#' @param facet_by Character string controlling faceting. One of:
#'   \describe{
#'     \item{`"none"`}{No faceting.}
#'     \item{`"season"`}{Facet by \code{season_label}.}
#'     \item{`"metric"`}{Facet by metric (\code{STD_DDM}, \code{RES_DDM}).}
#'   }
#' @param ncol Optional integer giving the number of facet columns.
#' @param box_group For \code{type = "boxplot"}, grouping variable on the x-axis.
#'   One of \code{"metric"} or \code{"season"}.
#' @param alpha Numeric alpha transparency for lines/points.
#' @param line_width Numeric line width.
#' @param point_size Numeric point size.
#' @param legend_position Character legend position passed to ggplot2.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return
#' A \code{ggplot2} object.
#'
#' @method plot mean_dm_detrended
#' @importFrom dplyr filter mutate %>%
#' @importFrom tidyr pivot_longer
#' @importFrom tibble as_tibble
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_hline geom_boxplot
#'   geom_jitter facet_wrap theme_bw theme element_text labs
#' @export
plot.mean_dm_detrended <- function(x,
                                   y = NULL,
                                   type = c("series", "seasonal", "boxplot"),
                                   value = c("both", "STD_DDM", "RES_DDM"),
                                   seasons = NULL,
                                   x_axis = c("default", "date", "doy", "season_day"),
                                   facet_by = c("none", "season", "metric"),
                                   ncol = NULL,
                                   box_group = c("metric", "season"),
                                   alpha = 0.8,
                                   line_width = 0.8,
                                   point_size = 1.4,
                                   legend_position = "right",
                                   ...) {

  type <- match.arg(type)
  value <- match.arg(value)
  x_axis <- match.arg(x_axis)
  facet_by <- match.arg(facet_by)
  box_group <- match.arg(box_group)

  if (!inherits(x, "mean_dm_detrended")) {
    stop("'x' must be an object of class 'mean_dm_detrended'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.mean_dm_detrended().")
  }

  dat <- tibble::as_tibble(x)

  if (!"TIME" %in% names(dat)) {
    stop("The input object must contain a 'TIME' column.")
  }

  if (!is.null(seasons)) {
    if (!"season_label" %in% names(dat)) {
      stop("The object has no 'season_label' column for seasonal filtering.")
    }
    dat <- dat %>% dplyr::filter(.data$season_label %in% seasons)
  }

  metric_cols <- intersect(c("STD_DDM", "RES_DDM"), names(dat))

  if (length(metric_cols) == 0) {
    stop("No plottable columns found. Expected 'STD_DDM' and/or 'RES_DDM'.")
  }

  if (value == "STD_DDM") {
    if (!"STD_DDM" %in% metric_cols) {
      stop("Column 'STD_DDM' is not available.")
    }
    metric_cols <- "STD_DDM"
  }

  if (value == "RES_DDM") {
    if (!"RES_DDM" %in% metric_cols) {
      stop("Column 'RES_DDM' is not available.")
    }
    metric_cols <- "RES_DDM"
  }

  dat$TIME_DATE <- as.Date(dat$TIME)

  if (!"doy" %in% names(dat)) {
    dat$doy <- lubridate::yday(dat$TIME_DATE)
  }
  if (!"season_day" %in% names(dat)) {
    dat$season_day <- seq_len(nrow(dat))
  }

  long_dat <- dat %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(metric_cols),
      names_to = "metric",
      values_to = "value"
    )

  if (nrow(long_dat) == 0) {
    stop("No data available to plot.")
  }

  if (x_axis == "default") {
    if (type == "seasonal") {
      x_var <- "season_day"
      x_lab <- "Season day"
    } else {
      x_var <- "TIME_DATE"
      x_lab <- "Date"
    }
  } else if (x_axis == "date") {
    x_var <- "TIME_DATE"
    x_lab <- "Date"
  } else if (x_axis == "doy") {
    x_var <- "doy"
    x_lab <- "DOY"
  } else {
    x_var <- "season_day"
    x_lab <- "Season day"
  }

  if (type == "series") {
    if (facet_by == "none") {
      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data[[x_var]],
          y = .data$value,
          colour = .data$metric,
          group = .data$metric
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::geom_point(alpha = alpha, size = point_size)
    } else if (facet_by == "season") {
      if (!"season_label" %in% names(long_dat)) {
        stop("The object has no 'season_label' column for faceting by season.")
      }
      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data[[x_var]],
          y = .data$value,
          colour = .data$metric,
          group = .data$metric
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::geom_point(alpha = alpha, size = point_size) +
        ggplot2::facet_wrap(stats::as.formula("~ season_label"), scales = "free_x", ncol = ncol)
    } else {
      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data[[x_var]],
          y = .data$value,
          colour = .data$metric,
          group = .data$metric
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::geom_point(alpha = alpha, size = point_size) +
        ggplot2::facet_wrap(stats::as.formula("~ metric"), scales = "free_y", ncol = ncol)
    }

    p <- p +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = x_lab,
        y = "Mean detrended value",
        colour = "Metric",
        title = "Mean detrended dendrometer series"
      )

    return(p)
  }

  if (type == "seasonal") {
    if (!"season_label" %in% names(long_dat)) {
      stop("The object has no 'season_label' column required for type = 'seasonal'.")
    }

    if (facet_by == "metric") {
      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data$season_day,
          y = .data$value,
          colour = .data$season_label,
          group = .data$season_label
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::facet_wrap(stats::as.formula("~ metric"), scales = "free_y", ncol = ncol) +
        ggplot2::labs(colour = "Season")
    } else if (facet_by == "season") {
      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data$season_day,
          y = .data$value,
          colour = .data$metric,
          group = .data$metric
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::facet_wrap(stats::as.formula("~ season_label"), scales = "free_y", ncol = ncol) +
        ggplot2::labs(colour = "Metric")
    } else {
      long_dat$colour_group <- interaction(long_dat$metric, long_dat$season_label, drop = TRUE)

      p <- ggplot2::ggplot(
        long_dat,
        ggplot2::aes(
          x = .data$season_day,
          y = .data$value,
          colour = .data$colour_group,
          group = .data$colour_group
        )
      ) +
        ggplot2::geom_hline(yintercept = 1, linetype = 2) +
        ggplot2::geom_line(alpha = alpha, linewidth = line_width) +
        ggplot2::labs(colour = "Metric | Season")
    }

    p <- p +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Season day",
        y = "Mean detrended value",
        title = "Seasonal overlay of mean detrended series"
      )

    return(p)
  }

  if (type == "boxplot") {
    if (box_group == "metric") {
      long_dat$group_var <- long_dat$metric
      x_lab2 <- "Metric"
    } else {
      if (!"season_label" %in% names(long_dat)) {
        stop("The object has no 'season_label' column required for boxplots by season.")
      }
      long_dat$group_var <- long_dat$season_label
      x_lab2 <- "Season"
    }

    p <- ggplot2::ggplot(
      long_dat[is.finite(long_dat$value), , drop = FALSE],
      ggplot2::aes(x = .data$group_var, y = .data$value, fill = .data$metric)
    ) +
      ggplot2::geom_boxplot(outlier.shape = NA) +
      ggplot2::geom_jitter(width = 0.15, alpha = alpha, size = point_size) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = legend_position,
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11),
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
      ) +
      ggplot2::labs(
        x = x_lab2,
        y = "Mean detrended value",
        fill = "Metric",
        title = "Distribution of mean detrended series"
      )

    if (facet_by == "metric") {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ metric"), scales = "free_y", ncol = ncol)
    } else if (facet_by == "season" && box_group == "metric" && "season_label" %in% names(long_dat)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ season_label"), scales = "free_y", ncol = ncol)
    }

    return(p)
  }

  stop("Unknown plot type.")
}

#' Compare dendrometer growth-fitting methods using fit statistics
#'
#' @description
#' Fits one or more growth models to dendrometer series and returns a compact
#' table of evaluation statistics only.
#'
#' This function is intended for method comparison rather than data extraction.
#' It runs the selected fitting methods, aligns observed and fitted values on
#' all non-missing observation days, and calculates goodness-of-fit measures for
#' each fitted series and vegetation-season fit.
#'
#' Supported single-curve methods are `"gam"`, `"gompertz"`, `"logistic"`,
#' `"richards"`, `"loess"`, and `"spline"`, which are fitted using
#' [dm.growth.fit()]. Supported bimodal methods are `"double_gompertz"` and
#' `"double_richards"`, which are fitted using [dm.growth.fit.double()].
#'
#' The function returns only an evaluation table and does not return fitted
#' curves, processed data, or parameter objects. It is therefore useful for
#' comparing alternative fitting methods across series, years, or sites before
#' selecting a final modeling approach.
#'
#' @details
#' Evaluation statistics are calculated by comparing observed daily values
#' against fitted daily values on all days where observed values are available.
#'
#' The returned metrics have the following interpretation:
#' \itemize{
#'   \item `rmse`: root mean squared error. Smaller values indicate closer fit.
#'   \item `mae`: mean absolute error. Smaller values indicate closer fit and are
#'     less sensitive to large deviations than RMSE.
#'   \item `bias`: mean fitted minus observed value. Positive values indicate
#'     overestimation and negative values indicate underestimation.
#'   \item `r2`: coefficient of determination, computed from residual and total
#'     sums of squares. Larger values indicate better fit.
#'   \item `correlation`: Pearson correlation between observed and fitted values.
#'     Larger values indicate stronger agreement in shape.
#'   \item `nrmse`: normalized RMSE, calculated as RMSE divided by the observed
#'     value range. This allows comparison across series with different
#'     magnitudes.
#'   \item `rss`: residual sum of squares.
#'   \item `aic_approx` and `bic_approx`: approximate information criteria based
#'     on RSS and an effective parameter count rather than the exact likelihood
#'     returned by each modeling function.
#' }
#'
#' The columns `aic_approx` and `bic_approx` are labeled as approximate because
#' they are not extracted from the original model objects through a unified
#' likelihood interface. Instead, they are computed from the residual sum of
#' squares and an effective number of fitted parameters. They are therefore most
#' useful for relative comparison among methods fitted to the same dataset.
#'
#' Negative values of `aic_approx` or `bic_approx` are not an error. They occur
#' naturally when the average residual variance is smaller than 1, because the
#' logarithmic term in the information-criterion formula becomes negative. In
#' practice, the absolute sign is not important; only differences among methods
#' fitted to the same data should be interpreted.
#'
#' For methods with flexible smoothness, such as GAM or spline, the effective
#' parameter count is approximate. Consequently, `aic_approx` and `bic_approx`
#' should be treated as heuristic ranking measures rather than exact
#' likelihood-based criteria.
#'
#' @param df A data frame or tibble. The first column must be a time stamp and
#'   the remaining selected columns must be numeric dendrometer series.
#' @param TreeNum Either `"all"`, a numeric vector selecting dendrometer series
#'   by position, or a character vector of series names.
#' @param methods Character vector of fitting methods to evaluate.
#' @param years Either `"all"` or a character vector of vegetation-season labels.
#' @param year_mode Either `"yearly"` or `"pooled"`.
#' @param fit_GRO Logical. If `TRUE`, convert processed daily series to
#'   cumulative growth before fitting.
#' @param site_mode Vegetation season definition. One of `"NH"`, `"SH"`, or
#'   `"CS"`.
#' @param custom_veg_season Numeric vector of length 2 defining a custom season
#'   in day-of-year coordinates when `site_mode = "CS"`.
#' @param growth_fraction Numeric vector of length 2 used by the fitting
#'   functions to estimate cumulative-growth timing.
#' @param min_unique_growth Minimum number of unique cumulative-growth values
#'   required before a fit is attempted.
#' @param rate_threshold_fraction Numeric scalar between 0 and 1. Passed to the
#'   fitting functions where supported.
#' @param peak_separation_min Minimum peak separation in days for
#'   [dm.growth.fit.double()].
#' @param valley_ratio_max Maximum allowed valley-to-peak ratio for
#'   [dm.growth.fit.double()].
#' @param min_relative_peak Minimum relative peak height for
#'   [dm.growth.fit.double()].
#' @param start_value_gompertz_parameters Optional starting values for Gompertz
#'   fits.
#' @param start_value_richards_parameters Optional starting values for logistic
#'   and Richards fits.
#' @param start_value_double_gompertz_parameters Optional starting values for
#'   double-Gompertz fits.
#' @param start_value_double_richards_parameters Optional starting values for
#'   double-Richards fits.
#' @param loess_span Span used when `method = "loess"`.
#' @param spline_df Degrees of freedom used when `method = "spline"`.
#' @param verbose Logical. If `TRUE`, prints progress messages.
#'
#' @return
#' A tibble with one row per fitted series and fit, containing:
#' \describe{
#'   \item{method}{Fitting method used for the row.}
#'   \item{series}{Series name.}
#'   \item{fit_id}{Vegetation-season label or `"pooled"`.}
#'   \item{n_compare}{Number of observed days used for comparison.}
#'   \item{rmse}{Root mean squared error.}
#'   \item{mae}{Mean absolute error.}
#'   \item{bias}{Mean fitted minus observed value.}
#'   \item{r2}{Coefficient of determination.}
#'   \item{correlation}{Pearson correlation between observed and fitted values.}
#'   \item{nrmse}{RMSE divided by the observed range.}
#'   \item{rss}{Residual sum of squares.}
#'   \item{aic_approx}{Approximate AIC based on RSS and effective parameter count.}
#'   \item{bic_approx}{Approximate BIC based on RSS and effective parameter count.}
#'   \item{k_effective}{Effective parameter count used in the approximate
#'     information criteria.}
#'   \item{converged}{Logical convergence flag returned by the fitting function.}
#'   \item{model_note}{Model note, warning, or error message returned by the
#'     fitting function.}
#' }
#'
#' @seealso [dm.growth.fit()], [dm.growth.fit.double()]
#'
#' @export
dm.growth.evaluate <- function(
    df,
    TreeNum = "all",
    methods = c(
      "gam", "gompertz", "logistic", "richards", "loess", "spline",
      "double_gompertz", "double_richards"
    ),
    years = "all",
    year_mode = c("yearly", "pooled"),
    fit_GRO = TRUE,
    site_mode = c("NH", "SH", "CS"),
    custom_veg_season = c(275, 274),
    growth_fraction = c(0.1, 0.9),
    min_unique_growth = 10,
    rate_threshold_fraction = 0.1,
    peak_separation_min = 10,
    valley_ratio_max = 0.4,
    min_relative_peak = 0.05,
    start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_),
    start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1),
    start_value_double_gompertz_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_
    ),
    start_value_double_richards_parameters = list(
      a = NA_real_, k = NA_real_, t0 = NA_real_, v = 1,
      a2 = NA_real_, k2 = NA_real_, t02 = NA_real_, v2 = 1
    ),
    loess_span = 0.2,
    spline_df = 10,
    verbose = TRUE
) {
  year_mode <- match.arg(year_mode)
  site_mode <- match.arg(site_mode)


  supported_methods <- c(
    "gam", "gompertz", "logistic", "richards", "loess", "spline",
    "double_gompertz", "double_richards"
  )

  methods <- unique(as.character(methods))
  bad_methods <- setdiff(methods, supported_methods)
  if (length(bad_methods) > 0) {
    stop(
      "Unsupported methods: ", paste(bad_methods, collapse = ", "),
      ". Supported methods are: ", paste(supported_methods, collapse = ", "), "."
    )
  }

  out_list <- vector("list", length(methods))
  names(out_list) <- methods

  for (i in seq_along(methods)) {
    mm <- methods[i]

    if (isTRUE(verbose)) {
      message("Evaluating method: ", mm)
    }

    fit_obj <- dmgv_run_method(
      method = mm,
      df = df,
      TreeNum = TreeNum,
      years = years,
      year_mode = year_mode,
      fit_GRO = fit_GRO,
      site_mode = site_mode,
      custom_veg_season = custom_veg_season,
      growth_fraction = growth_fraction,
      min_unique_growth = min_unique_growth,
      rate_threshold_fraction = rate_threshold_fraction,
      peak_separation_min = peak_separation_min,
      valley_ratio_max = valley_ratio_max,
      min_relative_peak = min_relative_peak,
      start_value_gompertz_parameters = start_value_gompertz_parameters,
      start_value_richards_parameters = start_value_richards_parameters,
      start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
      start_value_double_richards_parameters = start_value_double_richards_parameters,
      loess_span = loess_span,
      spline_df = spline_df
    )

    out_list[[i]] <- dmgv_extract_evaluation_table(fit_obj, method_name = mm)
  }

  dplyr::bind_rows(out_list)
}


# Internal helpers -------------------------------------------------------------

#' @keywords internal
dmgv_run_method <- function(
    method,
    df,
    TreeNum,
    years,
    year_mode,
    fit_GRO,
    site_mode,
    custom_veg_season,
    growth_fraction,
    min_unique_growth,
    rate_threshold_fraction,
    peak_separation_min,
    valley_ratio_max,
    min_relative_peak,
    start_value_gompertz_parameters,
    start_value_richards_parameters,
    start_value_double_gompertz_parameters,
    start_value_double_richards_parameters,
    loess_span,
    spline_df
) {
  if (method %in% c("gam", "gompertz", "logistic", "richards", "loess", "spline")) {
    return(
      dm.growth.fit(
        df = df,
        TreeNum = TreeNum,
        method = method,
        years = years,
        year_mode = year_mode,
        fit_GRO = fit_GRO,
        site_mode = site_mode,
        custom_veg_season = custom_veg_season,
        growth_fraction = growth_fraction,
        min_unique_growth = min_unique_growth,
        rate_threshold_fraction = rate_threshold_fraction,
        start_value_gompertz_parameters = start_value_gompertz_parameters,
        start_value_richards_parameters = start_value_richards_parameters,
        loess_span = loess_span,
        spline_df = spline_df,
        verbose = FALSE
      )
    )
  }

  if (method == "double_gompertz") {
    return(
      dm.growth.fit.double(
        df = df,
        TreeNum = TreeNum,
        method = "gompertz",
        years = years,
        year_mode = year_mode,
        fit_GRO = fit_GRO,
        site_mode = site_mode,
        custom_veg_season = custom_veg_season,
        growth_fraction = growth_fraction,
        min_unique_growth = min_unique_growth,
        rate_threshold_fraction = rate_threshold_fraction,
        peak_separation_min = peak_separation_min,
        valley_ratio_max = valley_ratio_max,
        min_relative_peak = min_relative_peak,
        start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
        start_value_double_richards_parameters = start_value_double_richards_parameters,
        verbose = FALSE
      )
    )
  }

  if (method == "double_richards") {
    return(
      dm.growth.fit.double(
        df = df,
        TreeNum = TreeNum,
        method = "richards",
        years = years,
        year_mode = year_mode,
        fit_GRO = fit_GRO,
        site_mode = site_mode,
        custom_veg_season = custom_veg_season,
        growth_fraction = growth_fraction,
        min_unique_growth = min_unique_growth,
        rate_threshold_fraction = rate_threshold_fraction,
        peak_separation_min = peak_separation_min,
        valley_ratio_max = valley_ratio_max,
        min_relative_peak = min_relative_peak,
        start_value_double_gompertz_parameters = start_value_double_gompertz_parameters,
        start_value_double_richards_parameters = start_value_double_richards_parameters,
        verbose = FALSE
      )
    )
  }

  stop("Unknown method: ", method)
}

#' @keywords internal
dmgv_effective_parameters <- function(method, edf = NA_real_, spline_df = NA_real_) {
  if (method == "gam") {
    if (is.finite(edf)) return(as.numeric(edf) + 1)
    return(NA_real_)
  }
  if (method == "gompertz") return(3)
  if (method == "logistic") return(3)
  if (method == "richards") return(4)
  if (method == "loess") {
    if (is.finite(edf)) return(as.numeric(edf))
    return(NA_real_)
  }
  if (method == "spline") {
    if (is.finite(spline_df)) return(as.numeric(spline_df))
    return(NA_real_)
  }
  if (method == "double_gompertz") return(6)
  if (method == "double_richards") return(8)
  NA_real_
}

#' @keywords internal
dmgv_safe_r2 <- function(obs, fit) {
  ok <- is.finite(obs) & is.finite(fit)
  if (sum(ok) < 2) return(NA_real_)

  obs <- obs[ok]
  fit <- fit[ok]

  sse <- sum((obs - fit)^2)
  sst <- sum((obs - mean(obs))^2)

  if (!is.finite(sst) || sst <= 0) return(NA_real_)
  1 - sse / sst
}

#' @keywords internal
dmgv_safe_cor <- function(obs, fit) {
  ok <- is.finite(obs) & is.finite(fit)
  if (sum(ok) < 2) return(NA_real_)
  suppressWarnings(stats::cor(obs[ok], fit[ok]))
}

#' @keywords internal
dmgv_extract_evaluation_table <- function(fit_obj, method_name) {
  TIME  <- season_label  <- season_day  <- observed  <- series  <- NULL
  fit_id  <- fitted  <- obs_range  <- rmse  <- method_requested  <- edf  <- spline_df <- NULL
  converged <- model_note <- k_effective <- rss <- n_compare <- method <- mae <- NULL
  bias <- r2 <- correlation <- nrmse <- aic_approx <- bic_approx <- NULL
  meta_cols <- c(
    "TIME", "season_label", "season_start", "season_end",
    "season_day", "season_length", "any_observed"
  )

  proc <- fit_obj$processed_data
  fitd <- fit_obj$fitted_data
  fit_params <- fit_obj$fit_parameters

  series_cols <- intersect(
    setdiff(names(proc), meta_cols),
    setdiff(names(fitd), meta_cols)
  )

  obs_long <- proc %>%
    dplyr::select(TIME, season_label, season_day, dplyr::all_of(series_cols)) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(series_cols),
      names_to = "series",
      values_to = "observed"
    )

  fit_long <- fitd %>%
    dplyr::select(TIME, season_label, season_day, dplyr::all_of(series_cols)) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(series_cols),
      names_to = "series",
      values_to = "fitted"
    )

  comp_long <- obs_long %>%
    dplyr::left_join(fit_long, by = c("TIME", "season_label", "season_day", "series")) %>%
    dplyr::filter(is.finite(observed))

  if (all(fit_params$fit_id == "pooled")) {
    comp_long$fit_id <- "pooled"
  } else {
    comp_long$fit_id <- as.character(comp_long$season_label)
  }

  metric_tbl <- comp_long %>%
    dplyr::group_by(series, fit_id) %>%
    dplyr::summarise(
      n_compare = sum(is.finite(observed) & is.finite(fitted)),
      rss = sum((observed - fitted)^2, na.rm = TRUE),
      rmse = sqrt(mean((observed - fitted)^2, na.rm = TRUE)),
      mae = mean(abs(observed - fitted), na.rm = TRUE),
      bias = mean(fitted - observed, na.rm = TRUE),
      r2 = dmgv_safe_r2(observed, fitted),
      correlation = dmgv_safe_cor(observed, fitted),
      obs_range = diff(range(observed, na.rm = TRUE)),
      .groups = "drop"
    ) %>%
    dplyr::mutate(
      nrmse = dplyr::if_else(is.finite(obs_range) & obs_range > 0, rmse / obs_range, NA_real_)
    ) %>%
    dplyr::select(-obs_range)

  eval_tbl <- fit_params %>%
    dplyr::mutate(
      method_requested = method_name,
      k_effective = mapply(
        FUN = dmgv_effective_parameters,
        method = method_requested,
        edf = dplyr::coalesce(edf, NA_real_),
        spline_df = dplyr::coalesce(spline_df, NA_real_)
      )
    ) %>%
    dplyr::select(series, fit_id, converged, model_note, k_effective, method_requested) %>%
    dplyr::left_join(metric_tbl, by = c("series", "fit_id")) %>%
    dplyr::mutate(
      aic_approx = dplyr::if_else(
        is.finite(rss) & rss > 0 & is.finite(k_effective) & n_compare > 0,
        n_compare * log(rss / n_compare) + 2 * k_effective,
        NA_real_
      ),
      bic_approx = dplyr::if_else(
        is.finite(rss) & rss > 0 & is.finite(k_effective) & n_compare > 0,
        n_compare * log(rss / n_compare) + log(n_compare) * k_effective,
        NA_real_
      )
    ) %>%
    dplyr::rename(method = method_requested) %>%
    dplyr::select(
      method, series, fit_id,
      n_compare,
      rmse, mae, bias, r2, correlation, nrmse, rss,
      aic_approx, bic_approx, k_effective,
      converged, model_note
    )

  class(eval_tbl) <- c("dm_growth_evaluation", class(eval_tbl))
  attr(eval_tbl, "call") <- match.call()
  eval_tbl
}

#' @title Smoothing of Dendrometer Time Series
#'
#' @description
#' Applies various smoothing techniques to dendrometer (dm) time series data
#' using a user-defined or automatically detected temporal resolution. The function
#' supports several smoothing methods: robust median+mean, penalized spline,
#' Savitzky-Golay filter, exponential moving average (EMA), and LOESS.
#'
#' @details
#' The function is designed to smooth dendrometer time series (e.g., 10–60 min resolution)
#' while preserving key features such as diurnal fluctuations or long-term growth,
#' depending on the window size. It auto-detects the resolution (in minutes) if not provided.
#'
#' @param time A POSIXct vector representing the time column.
#' @param dm A numeric vector of dendrometer values corresponding to \code{time}.
#' @param resolution_min Integer. The resolution of the time series in minutes. If \code{NULL},
#'   the resolution is auto-detected based on median time difference.
#' @param method Smoothing method. One of: \code{"median_mean"}, \code{"pspline"},
#'   \code{"sg"} (Savitzky-Golay), \code{"ema"} (exponential moving average), or \code{"loess"}.
#' @param window_hours Numeric. Smoothing window length in hours. Converted to points using resolution.
#' @param sg_order Integer. Polynomial order for Savitzky-Golay smoothing. Ignored unless \code{method = "sg"}.
#' @param ema_alpha Numeric. Smoothing factor (0–1) for exponential moving average. If \code{NULL},
#'   it is automatically calculated from the window size.
#'
#' @return A numeric vector of the same length as \code{dm} containing the smoothed values.
#'
#' @examples
#' \dontrun{
#' # Example: Create synthetic dendrometer time series (30-min resolution)
#' time_seq <- seq.POSIXt(from = as.POSIXct("2023-06-01 00:00:00"),
#'                        to   = as.POSIXct("2023-06-03 00:00:00"),
#'                        by   = "30 mins")
#' set.seed(123)
#' dm_raw <- cumsum(rnorm(length(time_seq), mean = 0.005, sd = 0.01)) +
#'           0.1 * sin(2 * pi * as.numeric(difftime(time_seq, min(time_seq), units = "hours")) / 24)
#'
#' # Median plus moving mean smoothing (default robust filter)
#' dm_medmean <- smooth_dm(time = time_seq, dm = dm_raw,
#'                         method = "median_mean", window_hours = 3)
#'
#' # Penalized spline smoothing
#' dm_pspline <- smooth_dm(time = time_seq, dm = dm_raw,
#'                         method = "pspline", window_hours = 6)
#'
#' # Savitzky-Golay filter (requires 'signal' package)
#' if (requireNamespace("signal", quietly = TRUE)) {
#'   dm_sg <- smooth_dm(time = time_seq, dm = dm_raw,
#'                      method = "sg", window_hours = 2, sg_order = 2)
#' }
#'
#' # Exponential moving average smoothing
#' dm_ema <- smooth_dm(time = time_seq, dm = dm_raw,
#'                     method = "ema", window_hours = 4)
#'
#' # LOESS smoothing
#' dm_loess <- smooth_dm(time = time_seq, dm = dm_raw,
#'                       method = "loess", window_hours = 3)
#'
#' # Plot raw and smoothed series
#' plot(time_seq, dm_raw, type = "l", col = "gray", lwd = 1, main = "Smoothed Dendrometer Series",
#'      ylab = "DM", xlab = "Time")
#' lines(time_seq, dm_medmean, col = "blue", lwd = 2)
#' lines(time_seq, dm_pspline, col = "green", lwd = 2)
#' lines(time_seq, dm_ema, col = "orange", lwd = 2)
#' lines(time_seq, dm_loess, col = "purple", lwd = 2)
#' legend("topright", legend = c("Raw", "Median+Mean", "P-spline", "EMA", "LOESS"),
#'        col = c("gray", "blue", "green", "orange", "purple"), lwd = 2)
#' }
#'
#' @importFrom zoo rollapply
#' @importFrom stats runmed loess
#' @importFrom utils tail
#' @export

smooth_dm <- function(time, dm,
                      resolution_min = NULL,
                      method = c("median_mean","pspline","sg","ema","loess"),
                      window_hours = 3,
                      sg_order = 2,
                      ema_alpha = NULL) {
  stopifnot(length(time) == length(dm))
  method <- match.arg(method)

  # auto-detect resolution if needed
  if (is.null(resolution_min)) {
    dt <- as.numeric(diff(as.POSIXct(time)), units = "mins")
    dt <- dt[is.finite(dt) & dt > 0]
    if (!length(dt)) stop("Cannot detect resolution")
    resolution_min <- as.integer(round(stats::median(dt)))
  }
  # window length in points
  k <- max(3L, as.integer(round(window_hours * 60 / resolution_min)))
  if (k %% 2 == 0) k <- k + 1L  # odd window works better for centered filters

  x <- dm

  if (method == "median_mean") {
    x_med <- stats::runmed(x, k = k, endrule = "median")
    y_sm  <- zoo::rollapply(x_med, width = k, align = "center",
                            FUN = function(z) mean(z, na.rm = TRUE),
                            partial = TRUE, fill = NA)
    na_idx <- which(is.na(y_sm))
    if (length(na_idx)) y_sm[na_idx] <- x_med[na_idx]
    return(y_sm)
  }

  if (method == "pspline") {
    spar_pts <- k
    fit <- pspline::smooth.Pspline(x = seq_along(x), y = x, norder = 2, spar = spar_pts)
    return(as.numeric(fit$ysmth))
  }

  if (method == "sg") {
    if (!requireNamespace("signal", quietly = TRUE))
      stop("Install 'signal' for Savitzky Golay")
    if (k <= sg_order) k <- sg_order + 1 + (sg_order %% 2 == 0)
    if (k %% 2 == 0) k <- k + 1
    y_sm <- signal::sgolayfilt(x, p = sg_order, n = k)
    return(as.numeric(y_sm))
  }

  if (method == "ema") {
    if (is.null(ema_alpha)) {
      ema_alpha <- 1 - exp(-log(2) / (k/2))
    }
    y_fwd <- stats::filter(x, ema_alpha, method = "recursive", init = x[1])
    y_bwd <- rev(stats::filter(rev(x), ema_alpha, method = "recursive", init = tail(x,1)))
    y_sm  <- (as.numeric(y_fwd) + as.numeric(y_bwd)) / 2
    return(y_sm)
  }

  if (method == "loess") {
    span <- min(0.99, max(0.05, k / length(x)))
    y_sm <- stats::loess(x ~ seq_along(x), span = span, degree = 1)$fitted
    return(as.numeric(y_sm))
  }
}

#' @title Reading dendrometer data
#' @description
#' Reads dendrometer data from \code{.csv}, \code{.txt}, \code{.tsv}, or \code{.xlsx} files,
#' automatically parsing the datetime information and converting it to
#' \code{\%Y-\%m-\%d \%H:\%M:\%S} in the requested timezone.
#'
#' Supports:
#' \itemize{
#'   \item automatic delimiter detection for text files,
#'   \item real decimal-mark auto-detection for text files,
#'   \item Excel serial date support (\code{1900} / \code{1904} systems),
#'   \item separate \code{date_col} + \code{time_col},
#'   \item optional import report output,
#'   \item optional time-resolution diagnostics.
#' }
#'
#' @param file Path to file (\code{.csv}, \code{.txt}, \code{.tsv}, \code{.xlsx}).
#' @param sep Optional delimiter for text files. If \code{NULL}, auto-detect
#'   among comma, semicolon, tab, and pipe.
#' @param dec Optional decimal mark for text files. If \code{NULL}, auto-detect
#'   between \code{"."} and \code{","}.
#' @param datetime_col Integer or name of the datetime column (default \code{1}).
#'   Ignored if \code{date_col} is provided.
#' @param date_col Optional integer or name of a date column.
#' @param time_col Optional integer or name of a time column. Used together with
#'   \code{date_col}. If \code{NULL}, only the date column is parsed and
#'   \code{"00:00:00"} can be appended if \code{assume_midnight = TRUE}.
#' @param tz Time zone for parsed datetimes (default \code{"UTC"}).
#' @param sheet Excel sheet name or index (for \code{.xlsx}; default \code{NULL} = first sheet).
#' @param range Excel cell range (optional).
#' @param na Character vector of strings to treat as NA.
#' @param assume_midnight Logical; if \code{TRUE}, rows with only a date get \code{"00:00:00"}.
#' @param orders Optional vector of \pkg{lubridate} orders to try. If \code{NULL},
#'   a comprehensive default set is used.
#' @param excel_dates Character. One of \code{"auto"}, \code{"none"}, \code{"1900"},
#'   or \code{"1904"}. Controls handling of numeric Excel serial dates.
#' @param drop_dup_times Logical; if \code{TRUE}, drop duplicated timestamps
#'   (keep first) with a warning/message.
#' @param detect_resolution Logical; if \code{TRUE}, compute basic time-resolution diagnostics.
#' @param return_report Logical; if \code{TRUE}, return a list with
#'   \code{$data} and \code{$report}. If \code{FALSE}, return the tibble only.
#'   In both cases, the report is attached as \code{attr(x, "import_report")}.
#' @param quiet Logical; if \code{TRUE}, suppress informational messages.
#'
#' @return
#' If \code{return_report = FALSE}, a tibble with a POSIXct first column and the
#' remaining data columns unchanged.
#'
#' If \code{return_report = TRUE}, a list with:
#' \describe{
#'   \item{\code{$data}}{The imported tibble.}
#'   \item{\code{$report}}{A structured import report.}
#' }
#'
#' @importFrom lubridate parse_date_time ymd_hms
#' @importFrom readxl read_excel
#' @importFrom tibble as_tibble tibble
#' @importFrom utils read.csv read.table head
#' @importFrom tools file_ext
#' @export
read.dendrometer <- function(
    file,
    sep = NULL,
    dec = NULL,
    datetime_col = 1,
    date_col = NULL,
    time_col = NULL,
    tz = "UTC",
    sheet = NULL,
    range = NULL,
    na = c("", "NA", "NaN", "nan", "null", "NULL", "-9999"),
    assume_midnight = TRUE,
    orders = NULL,
    excel_dates = c("auto", "none", "1900", "1904"),
    drop_dup_times = TRUE,
    detect_resolution = FALSE,
    return_report = FALSE,
    quiet = TRUE
) {

  excel_dates <- match.arg(excel_dates)

  # ---------------------------------------------------------------------------
  # helpers
  # ---------------------------------------------------------------------------

  default_orders <- function() {
    c(
      "Y-m-d H:M:S", "Y-m-d H:M", "Y-m-d",
      "Y/m/d H:M:S", "Y/m/d H:M", "Y/m/d",
      "Y.m.d H:M:S", "Y.m.d H:M", "Y.m.d",
      "d.m.Y H:M:S", "d.m.Y H:M", "d.m.Y",
      "d-m-Y H:M:S", "d-m-Y H:M", "d-m-Y",
      "d/m/Y H:M:S", "d/m/Y H:M", "d/m/Y",
      "m/d/Y H:M:S", "m/d/Y H:M", "m/d/Y",
      "Y-m-d I:M p", "Y-m-d I:M:S p",
      "m/d/Y I:M p", "m/d/Y I:M:S p",
      "d/m/Y I:M p", "d/m/Y I:M:S p",
      "Ymd HMS", "Ymd HM", "Ymd",
      "dmY HMS", "dmY HM", "dmY",
      "mdY HMS", "mdY HM", "mdY"
    )
  }

  get_col_index <- function(data, col, arg_name) {
    if (is.null(col)) return(NULL)

    if (is.character(col)) {
      idx <- match(col, names(data))
      if (is.na(idx)) stop(arg_name, " '", col, "' not found in columns.")
      return(idx)
    }

    idx <- as.integer(col)
    if (!is.finite(idx) || idx < 1 || idx > ncol(data)) {
      stop(arg_name, " index out of bounds.")
    }
    idx
  }

  sniff_sep <- function(line) {
    cands <- c("," = ",", ";" = ";", "\t" = "\t", "|" = "|")
    counts <- vapply(
      cands,
      function(s) length(strsplit(line, s, fixed = TRUE)[[1]]) - 1L,
      integer(1)
    )
    if (max(counts) == 0) "," else names(which.max(counts))
  }

  sniff_dec <- function(lines, sep) {
    if (length(lines) == 0) return(".")

    # If comma is delimiter, decimal comma is highly unlikely in plain delimited files.
    if (identical(sep, ",")) return(".")

    split_lines <- strsplit(lines, sep, fixed = TRUE)
    tokens <- unlist(split_lines, use.names = FALSE)
    tokens <- trimws(tokens)
    tokens <- tokens[nzchar(tokens)]

    # Simple numeric token patterns
    comma_hits <- sum(grepl("^-?[0-9]+,[0-9]+$", tokens))
    dot_hits   <- sum(grepl("^-?[0-9]+\\.[0-9]+$", tokens))

    if (comma_hits > dot_hits) "," else "."
  }

  detect_format_string <- function(x_chr) {
    x_chr <- trimws(as.character(x_chr))
    samp <- x_chr[!is.na(x_chr) & nzchar(x_chr)]
    samp <- utils::head(samp, 50)
    if (!length(samp)) return(NA_character_)

    pats <- list(
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%m/%d/%Y %H:%M:%S",
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2}$" = "%m/%d/%Y %H:%M",
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%Y-%m-%d %H:%M:%S",
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2}$" = "%Y-%m-%d %H:%M",
      "^\\d{4}-\\d{1,2}-\\d{1,2}$" = "%Y-%m-%d",
      "^\\d{4}/\\d{1,2}/\\d{1,2} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%Y/%m/%d %H:%M:%S",
      "^\\d{4}/\\d{1,2}/\\d{1,2} \\d{1,2}:\\d{1,2}$" = "%Y/%m/%d %H:%M",
      "^\\d{4}/\\d{1,2}/\\d{1,2}$" = "%Y/%m/%d",
      "^\\d{4}\\.\\d{1,2}\\.\\d{1,2} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%Y.%m.%d %H:%M:%S",
      "^\\d{4}\\.\\d{1,2}\\.\\d{1,2} \\d{1,2}:\\d{1,2}$" = "%Y.%m.%d %H:%M",
      "^\\d{4}\\.\\d{1,2}\\.\\d{1,2}$" = "%Y.%m.%d",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%d.%m.%Y %H:%M:%S",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4} \\d{1,2}:\\d{1,2}$" = "%d.%m.%Y %H:%M",
      "^\\d{1,2}\\.\\d{1,2}\\.\\d{4}$" = "%d.%m.%Y",
      "^\\d{1,2}-\\d{1,2}-\\d{4} \\d{1,2}:\\d{1,2}:\\d{1,2}$" = "%d-%m-%Y %H:%M:%S",
      "^\\d{1,2}-\\d{1,2}-\\d{4} \\d{1,2}:\\d{1,2}$" = "%d-%m-%Y %H:%M",
      "^\\d{1,2}-\\d{1,2}-\\d{4}$" = "%d-%m-%Y",
      "^\\d{1,2}/\\d{1,2}/\\d{4} \\d{1,2}:\\d{1,2} [APap][Mm]$" = "%m/%d/%Y %I:%M %p",
      "^\\d{4}-\\d{1,2}-\\d{1,2} \\d{1,2}:\\d{1,2} [APap][Mm]$" = "%Y-%m-%d %I:%M %p"
    )

    hits <- vapply(names(pats), function(rx) mean(grepl(rx, samp)), numeric(1))
    if (max(hits) > 0.9) pats[[which.max(hits)]] else NA_character_
  }

  append_midnight_if_needed <- function(x_chr, assume_midnight) {
    if (!assume_midnight) return(x_chr)

    x_chr <- trimws(x_chr)
    only_date <- grepl("^\\d{4}[-/.]\\d{1,2}[-/.]\\d{1,2}$", x_chr) |
      grepl("^\\d{1,2}[-/.]\\d{1,2}[-/.]\\d{4}$", x_chr) |
      grepl("^\\d{8}$", x_chr)

    x_chr[only_date] <- paste0(x_chr[only_date], " 00:00:00")
    x_chr
  }

  excel_serial_to_posix <- function(x_num, tz, system = c("1900", "1904")) {
    system <- match.arg(system)
    origin <- if (system == "1900") "1899-12-30" else "1904-01-01"
    out <- as.POSIXct(x_num * 86400, origin = origin, tz = tz)
    out
  }

  choose_excel_system <- function(x_num, tz, excel_dates) {
    systems <- if (excel_dates == "auto") c("1900", "1904") else excel_dates

    best <- list(
      system = NA_character_,
      dt = rep(as.POSIXct(NA, tz = tz), length(x_num)),
      score = -Inf
    )

    for (sys in systems) {
      dt <- suppressWarnings(excel_serial_to_posix(x_num, tz = tz, system = sys))
      yrs <- suppressWarnings(as.integer(format(dt, "%Y")))
      score <- sum(!is.na(dt) & yrs >= 1980 & yrs <= 2100)

      if (score > best$score) {
        best$system <- sys
        best$dt <- dt
        best$score <- score
      }
    }

    best
  }

  parse_datetime_vector <- function(x, tz, assume_midnight, orders, excel_dates) {
    if (is.null(orders)) orders <- default_orders()

    # Already parsed classes
    if (inherits(x, "POSIXct")) {
      return(list(
        dt = as.POSIXct(x, tz = tz),
        parse_method = "POSIXct",
        detected_format = "%Y-%m-%d %H:%M:%S",
        excel_system = NA_character_
      ))
    }

    if (inherits(x, "Date")) {
      dt <- as.POSIXct(x, tz = tz)
      return(list(
        dt = dt,
        parse_method = "Date",
        detected_format = "%Y-%m-%d",
        excel_system = NA_character_
      ))
    }

    detected_format <- NA_character_
    excel_system <- NA_character_

    # Text parse attempt
    x_chr <- trimws(as.character(x))
    x_chr <- append_midnight_if_needed(x_chr, assume_midnight = assume_midnight)
    detected_format <- detect_format_string(x_chr)

    dt_text <- suppressWarnings(
      lubridate::parse_date_time(
        x_chr,
        orders = orders,
        tz = tz,
        quiet = TRUE
      )
    )
    score_text <- sum(!is.na(dt_text))

    # Excel serial attempt
    dt_excel <- rep(as.POSIXct(NA, tz = tz), length(x_chr))
    score_excel <- -Inf

    if (excel_dates != "none") {
      numeric_like <- is.numeric(x) || all(
        is.na(x_chr) | !nzchar(x_chr) | grepl("^-?[0-9]+([.][0-9]+)?$", x_chr)
      )

      if (numeric_like) {
        x_num <- suppressWarnings(as.numeric(x_chr))
        chosen <- choose_excel_system(x_num, tz = tz, excel_dates = excel_dates)
        dt_excel <- chosen$dt
        excel_system <- chosen$system
        score_excel <- chosen$score
      }
    }

    # Decide
    use_excel <- FALSE
    if (score_excel > score_text && score_excel > 0) {
      use_excel <- TRUE
    } else if (score_text == 0 && score_excel > 0) {
      use_excel <- TRUE
    }

    if (use_excel) {
      list(
        dt = dt_excel,
        parse_method = "excel_serial",
        detected_format = paste0("Excel serial date (", excel_system, " system)"),
        excel_system = excel_system
      )
    } else {
      list(
        dt = dt_text,
        parse_method = "text",
        detected_format = detected_format,
        excel_system = NA_character_
      )
    }
  }

  compute_resolution_diag <- function(dt) {
    dt_ok <- dt[!is.na(dt)]
    dt_ok <- sort(unique(dt_ok))

    if (length(dt_ok) < 2) {
      return(list(
        n_timestamps = length(dt_ok),
        start_time = if (length(dt_ok)) min(dt_ok) else as.POSIXct(NA),
        end_time = if (length(dt_ok)) max(dt_ok) else as.POSIXct(NA),
        median_step_seconds = NA_real_,
        median_step_minutes = NA_real_,
        dominant_step_seconds = NA_real_,
        dominant_step_minutes = NA_real_,
        irregular_intervals = NA_integer_,
        estimated_missing_steps = NA_integer_,
        step_table = tibble::tibble(
          step_seconds = numeric(0),
          n = integer(0),
          proportion = numeric(0)
        )
      ))
    }

    diffs_sec <- as.numeric(diff(dt_ok), units = "secs")
    diffs_sec <- round(diffs_sec)

    step_tab <- as.data.frame(table(diffs_sec), stringsAsFactors = FALSE)
    names(step_tab) <- c("step_seconds", "n")
    step_tab$step_seconds <- as.numeric(as.character(step_tab$step_seconds))
    step_tab$n <- as.integer(step_tab$n)
    step_tab <- step_tab[order(step_tab$n, decreasing = TRUE), , drop = FALSE]
    step_tab$proportion <- step_tab$n / sum(step_tab$n)
    step_tab <- tibble::as_tibble(step_tab)

    dominant_step <- if (nrow(step_tab)) step_tab$step_seconds[1] else NA_real_
    median_step <- stats::median(diffs_sec, na.rm = TRUE)

    irregular_intervals <- if (is.finite(dominant_step)) {
      sum(diffs_sec != dominant_step, na.rm = TRUE)
    } else {
      NA_integer_
    }

    estimated_missing_steps <- if (is.finite(dominant_step) && dominant_step > 0) {
      sum(pmax(round(diffs_sec / dominant_step) - 1L, 0L), na.rm = TRUE)
    } else {
      NA_integer_
    }

    list(
      n_timestamps = length(dt_ok),
      start_time = min(dt_ok),
      end_time = max(dt_ok),
      median_step_seconds = median_step,
      median_step_minutes = median_step / 60,
      dominant_step_seconds = dominant_step,
      dominant_step_minutes = dominant_step / 60,
      irregular_intervals = irregular_intervals,
      estimated_missing_steps = estimated_missing_steps,
      step_table = step_tab
    )
  }

  make_report <- function(
    file, ext, sep_used, dec_used, datetime_mode, datetime_name,
    parse_method, detected_format, excel_system, tz,
    n_rows_input, n_rows_output, n_cols_output,
    n_bad_datetime, bad_rows, n_dup, dup_dropped,
    resolution_diag
  ) {
    list(
      file = normalizePath(file, winslash = "/", mustWork = FALSE),
      file_type = ext,
      separator = sep_used,
      decimal_mark = dec_used,
      datetime_source = datetime_mode,
      datetime_column_name = datetime_name,
      datetime_parse_method = parse_method,
      datetime_format_detected = detected_format,
      excel_date_system = excel_system,
      target_format = "%Y-%m-%d %H:%M:%S",
      tz = tz,
      n_rows_input = n_rows_input,
      n_rows_output = n_rows_output,
      n_cols_output = n_cols_output,
      n_bad_datetime = n_bad_datetime,
      bad_datetime_rows = bad_rows,
      n_duplicated_timestamps = n_dup,
      duplicated_timestamps_dropped = dup_dropped,
      resolution_diagnostics = resolution_diag
    )
  }

  # ---------------------------------------------------------------------------
  # read file
  # ---------------------------------------------------------------------------

  ext <- tolower(tools::file_ext(file))
  if (!ext %in% c("csv", "txt", "tsv", "xlsx")) {
    stop("Unsupported file extension: ", ext, " (use csv/txt/tsv/xlsx)")
  }

  sep_used <- NA_character_
  dec_used <- NA_character_

  if (ext == "xlsx") {
    df <- if (is.null(sheet) && is.null(range)) {
      readxl::read_excel(file, na = na)
    } else if (is.null(range)) {
      readxl::read_excel(file, sheet = sheet, na = na)
    } else {
      readxl::read_excel(file, sheet = sheet, range = range, na = na)
    }
    df <- tibble::as_tibble(df)
  } else {
    if (ext == "tsv" && is.null(sep)) sep <- "\t"

    lines <- readLines(file, n = 100L, warn = FALSE)
    lines <- lines[nzchar(trimws(lines))]

    if (!length(lines)) stop("File appears to be empty.")

    first_line <- lines[1]
    data_lines <- if (length(lines) > 1) lines[-1] else character(0)

    if (is.null(sep)) sep <- sniff_sep(first_line)
    if (is.null(dec)) dec <- sniff_dec(data_lines, sep = sep)

    sep_used <- sep
    dec_used <- dec

    if (requireNamespace("readr", quietly = TRUE)) {
      df <- readr::read_delim(
        file = file,
        delim = sep,
        na = na,
        locale = readr::locale(decimal_mark = dec),
        show_col_types = FALSE,
        progress = FALSE
      )
      df <- tibble::as_tibble(df)
    } else {
      if (sep %in% c(",", ";", "|")) {
        df <- utils::read.csv(
          file,
          header = TRUE,
          sep = sep,
          dec = dec,
          na.strings = na,
          check.names = FALSE,
          stringsAsFactors = FALSE
        )
      } else {
        df <- utils::read.table(
          file,
          header = TRUE,
          sep = sep,
          dec = dec,
          na.strings = na,
          check.names = FALSE,
          stringsAsFactors = FALSE
        )
      }
      df <- tibble::as_tibble(df)
    }
  }

  if (ncol(df) < 1) stop("No columns found in file.")

  n_rows_input <- nrow(df)

  # ---------------------------------------------------------------------------
  # build datetime
  # ---------------------------------------------------------------------------

  use_separate_cols <- !is.null(date_col) || !is.null(time_col)

  if (use_separate_cols) {
    if (is.null(date_col)) {
      stop("If 'time_col' is supplied, 'date_col' must also be supplied.")
    }

    date_idx <- get_col_index(df, date_col, "date_col")
    time_idx <- get_col_index(df, time_col, "time_col")

    date_name <- names(df)[date_idx]
    out_name <- if (is.null(time_idx)) date_name else "DateTime"

    raw_date <- trimws(as.character(df[[date_idx]]))
    raw_datetime <- if (is.null(time_idx)) {
      raw_date
    } else {
      raw_time <- trimws(as.character(df[[time_idx]]))
      paste(raw_date, raw_time)
    }

    parsed <- parse_datetime_vector(
      raw_datetime,
      tz = tz,
      assume_midnight = assume_midnight,
      orders = orders,
      excel_dates = excel_dates
    )

    dt <- parsed$dt

    drop_idx <- sort(unique(c(date_idx, time_idx)))
    other_df <- df[, setdiff(seq_len(ncol(df)), drop_idx), drop = FALSE]

    time_df <- tibble::tibble(..dt.. = dt)
    names(time_df)[1] <- out_name
    df <- tibble::as_tibble(cbind(time_df, other_df))

    datetime_mode <- if (is.null(time_idx)) "date_col" else "date_col + time_col"
    datetime_name <- out_name

  } else {
    dt_idx <- get_col_index(df, datetime_col, "datetime_col")
    old_name <- names(df)[dt_idx]

    parsed <- parse_datetime_vector(
      df[[dt_idx]],
      tz = tz,
      assume_midnight = assume_midnight,
      orders = orders,
      excel_dates = excel_dates
    )

    dt <- parsed$dt
    df[[dt_idx]] <- dt

    if (dt_idx != 1L) {
      df <- df[, c(dt_idx, setdiff(seq_len(ncol(df)), dt_idx)), drop = FALSE]
    }

    datetime_mode <- "datetime_col"
    datetime_name <- old_name
  }

  # ---------------------------------------------------------------------------
  # reporting messages
  # ---------------------------------------------------------------------------

  n_bad_datetime <- sum(is.na(df[[1]]))
  bad_rows <- which(is.na(df[[1]]))
  bad_rows <- utils::head(bad_rows, 20L)

  if (!quiet) {
    if (!is.na(parsed$detected_format)) {
      message(
        "Detected datetime format: '", parsed$detected_format,
        "'. Converted to '%Y-%m-%d %H:%M:%S'."
      )
    } else {
      message("Datetime parsed and converted to '%Y-%m-%d %H:%M:%S'.")
    }

    if (!is.na(sep_used)) {
      message("Detected separator: '", sep_used, "'.")
    }
    if (!is.na(dec_used)) {
      message("Detected decimal mark: '", dec_used, "'.")
    }

    if (parsed$parse_method == "excel_serial") {
      message("Datetime interpreted as Excel serial date (", parsed$excel_system, " system).")
    }

    if (n_bad_datetime > 0) {
      message(n_bad_datetime, " rows could not be parsed as datetime and remain NA.")
    }
  }

  # ---------------------------------------------------------------------------
  # sort + duplicates
  # ---------------------------------------------------------------------------

  ord <- order(df[[1]], na.last = TRUE)
  df <- df[ord, , drop = FALSE]

  dup <- duplicated(df[[1]]) & !is.na(df[[1]])
  n_dup <- sum(dup)

  if (n_dup > 0) {
    if (drop_dup_times) {
      if (!quiet) {
        message(
          "Found ", n_dup,
          " duplicated timestamps; keeping first occurrence and dropping the rest."
        )
      }
      df <- df[!dup, , drop = FALSE]
    } else {
      if (!quiet) {
        message(
          "Found ", n_dup,
          " duplicated timestamps; leaving as-is (set drop_dup_times = TRUE to drop)."
        )
      }
    }
  }

  # ---------------------------------------------------------------------------
  # resolution diagnostics
  # ---------------------------------------------------------------------------

  resolution_diag <- NULL
  if (detect_resolution) {
    resolution_diag <- compute_resolution_diag(df[[1]])

    if (!quiet && !is.null(resolution_diag)) {
      if (is.finite(resolution_diag$dominant_step_minutes)) {
        message(
          "Detected dominant time step: ",
          format(round(resolution_diag$dominant_step_minutes, 4), trim = TRUE),
          " minutes."
        )
      }
    }
  }

  # ---------------------------------------------------------------------------
  # import report
  # ---------------------------------------------------------------------------

  report <- make_report(
    file = file,
    ext = ext,
    sep_used = sep_used,
    dec_used = dec_used,
    datetime_mode = datetime_mode,
    datetime_name = names(df)[1],
    parse_method = parsed$parse_method,
    detected_format = parsed$detected_format,
    excel_system = parsed$excel_system,
    tz = tz,
    n_rows_input = n_rows_input,
    n_rows_output = nrow(df),
    n_cols_output = ncol(df),
    n_bad_datetime = n_bad_datetime,
    bad_rows = bad_rows,
    n_dup = n_dup,
    dup_dropped = drop_dup_times,
    resolution_diag = resolution_diag
  )

  attr(df, "import_report") <- report

  if (return_report) {
    return(list(
      data = df,
      report = report
    ))
  }

  df
}

# helpers ----------------------------------------------------------------------

#' @keywords internal
dmwc_parse_time <- function(tt) {
  if (inherits(tt, "POSIXct") || inherits(tt, "POSIXt")) return(as.POSIXct(tt))
  if (inherits(tt, "Date")) return(as.POSIXct(tt))

  out <- suppressWarnings(lubridate::ymd_hms(tt, quiet = TRUE))
  if (all(is.na(out))) {
    out <- suppressWarnings(
      lubridate::parse_date_time(
        tt,
        orders = c(
          "ymd HMS", "ymd HM", "ymd",
          "dmy HMS", "dmy HM", "dmy",
          "mdy HMS", "mdy HM", "mdy"
        ),
        quiet = TRUE
      )
    )
  }
  as.POSIXct(out)
}

#' @keywords internal
dmwc_infer_dt <- function(time_vec) {
  dsec <- as.numeric(diff(time_vec), units = "secs")
  dsec <- dsec[is.finite(dsec) & dsec > 0]

  if (length(dsec) == 0) {
    stop("Could not infer time step from TIME column.")
  }

  med <- stats::median(dsec, na.rm = TRUE)

  if (med >= 86400 && abs(med / 86400 - round(med / 86400)) < 1e-6) {
    return(list(dt = med / 86400, unit = "days", sec = med, hours = med / 3600))
  }
  if (med >= 3600 && abs(med / 3600 - round(med / 3600)) < 1e-6) {
    return(list(dt = med / 3600, unit = "hours", sec = med, hours = med / 3600))
  }
  if (med >= 60 && abs(med / 60 - round(med / 60)) < 1e-6) {
    return(list(dt = med / 60, unit = "mins", sec = med, hours = med / 3600))
  }

  list(dt = med, unit = "secs", sec = med, hours = med / 3600)
}

#' @keywords internal
dmwc_period_to_hours <- function(period, time_unit) {
  if (time_unit == "days") return(period * 24)
  if (time_unit == "hours") return(period)
  if (time_unit == "mins") return(period / 60)
  if (time_unit == "secs") return(period / 3600)
  period
}

#' @keywords internal
dmwc_fun_match <- function(fun_name) {
  fun_name <- match.arg(fun_name, c("mean", "sum", "max", "min"))
  switch(
    fun_name,
    mean = function(z) if (all(is.na(z))) NA_real_ else mean(z, na.rm = TRUE),
    sum  = function(z) if (all(is.na(z))) NA_real_ else sum(z, na.rm = TRUE),
    max  = function(z) if (all(is.na(z))) NA_real_ else max(z, na.rm = TRUE),
    min  = function(z) if (all(is.na(z))) NA_real_ else min(z, na.rm = TRUE)
  )
}

#' @keywords internal
dmwc_resample_regular <- function(dat, target_sec, fun_map) {
  tz_use <- attr(dat$TIME, "tzone")
  if (is.null(tz_use) || length(tz_use) == 0) tz_use <- "UTC"

  bucket_num <- floor(as.numeric(dat$TIME) / target_sec) * target_sec
  dat$TIME_BIN <- as.POSIXct(bucket_num, origin = "1970-01-01", tz = tz_use)

  val_cols <- setdiff(names(dat), c("TIME", "TIME_BIN"))

  agg_list <- lapply(val_cols, function(cc) {
    ff <- fun_map[[cc]]
    stats::aggregate(dat[[cc]], by = list(TIME = dat$TIME_BIN), FUN = ff)
  })

  out <- agg_list[[1]]
  names(out)[2] <- val_cols[1]

  if (length(agg_list) > 1) {
    for (i in 2:length(agg_list)) {
      tmp <- agg_list[[i]]
      names(tmp)[2] <- val_cols[i]
      out <- merge(out, tmp, by = "TIME", all = TRUE)
    }
  }

  full_time <- seq(min(out$TIME, na.rm = TRUE), max(out$TIME, na.rm = TRUE), by = target_sec)
  out <- merge(
    data.frame(TIME = as.POSIXct(full_time, origin = "1970-01-01", tz = tz_use)),
    out,
    by = "TIME",
    all.x = TRUE
  )
  out <- out[order(out$TIME), , drop = FALSE]
  tibble::as_tibble(out)
}

#' @keywords internal
dmwc_fill_na <- function(z) {
  z2 <- zoo::na.approx(z, na.rm = FALSE)
  z2 <- zoo::na.locf(z2, na.rm = FALSE)
  z2 <- zoo::na.locf(z2, fromLast = TRUE, na.rm = FALSE)
  z2
}

#' @keywords internal
dmwc_select_cols <- function(dat, selector, exclude = character(0)) {
  cols <- setdiff(names(dat), exclude)
  cols <- cols[vapply(dat[cols], is.numeric, logical(1))]

  if (length(cols) == 0) stop("No numeric series available.")

  if (is.character(selector) && length(selector) == 1 && tolower(selector) == "all") {
    return(cols)
  }

  if (is.numeric(selector)) {
    idx <- as.integer(selector)
    if (any(is.na(idx)) || any(idx < 1) || any(idx > length(cols))) {
      stop("Numeric selector is out of range.")
    }
    return(cols[idx])
  }

  if (is.character(selector)) {
    miss <- setdiff(selector, cols)
    if (length(miss) > 0) {
      warning("These requested columns were not found and were ignored: ",
              paste(miss, collapse = ", "))
    }
    keep <- intersect(selector, cols)
    if (length(keep) == 0) stop("None of the requested columns were found.")
    return(keep)
  }

  stop("Selector must be 'all', numeric indices, or character names.")
}


# main function ----------------------------------------------------------------

#' Wavelet coherence between dendrometer and climate series
#'
#' @description
#' Computes cross-wavelet power and wavelet coherence between dendrometer series
#' and climate variables using \code{WaveletComp::analyze.coherency()}.
#'
#' The function:
#' \itemize{
#'   \item crops dendrometer and climate inputs to their common time window,
#'   \item detects their temporal resolutions,
#'   \item resamples both to the coarser detected resolution,
#'   \item optionally interpolates missing values,
#'   \item optionally uses first differences of the dendrometer series,
#'   \item computes pairwise coherence for all selected tree/climate combinations.
#' }
#'
#' @param dm Dendrometer input. Either a data frame with time in the first column
#'   or an object of class \code{"dm_detrended"}.
#' @param clim Climate data frame. The first column must be time; remaining
#'   selected columns must be numeric climate variables.
#' @param TreeNum Dendrometer series selector: \code{"all"}, numeric indices,
#'   or character names.
#' @param clim_vars Climate variable selector: \code{"all"}, numeric indices,
#'   or character names.
#' @param source One of \code{"auto"}, \code{"raw"}, \code{"detrended"},
#'   or \code{"first_diff"}.
#' @param detrended_col For \code{dm_detrended} input, which table to use.
#' @param dm_fun Aggregation function for dendrometer resampling: one of
#'   \code{"mean"}, \code{"max"}, \code{"min"}, \code{"sum"}.
#' @param clim_funs Either a single aggregation function applied to all climate
#'   variables, or a named character vector such as
#'   \code{c(temp = "mean", VPD = "mean", prec = "sum")}.
#' @param na_action One of \code{"interpolate"} or \code{"fail"}.
#' @param loess_span Detrending span passed to \code{WaveletComp::analyze.coherency()}.
#'   Use \code{0} to disable internal loess detrending.
#' @param dj Frequency resolution passed to \code{WaveletComp::analyze.coherency()}.
#' @param lowerPeriod Optional lower period in native analysis units.
#' @param upperPeriod Optional upper period in native analysis units.
#' @param window.type.t Time-direction smoothing window type.
#' @param window.type.s Scale-direction smoothing window type.
#' @param window.size.t Time-direction smoothing window size.
#' @param window.size.s Scale-direction smoothing window size.
#' @param make.pval Logical. If \code{TRUE}, compute p-values.
#' @param make_pval Optional alias for \code{make.pval}.
#' @param method Surrogate generation method for p-values.
#' @param n.sim Number of simulations.
#' @param n_sim Optional alias for \code{n.sim}.
#' @param verbose Logical.
#'
#' @return
#' An object of class \code{"dm_wavelet_coherence"}.
#'
#' @details
#' \code{WaveletComp::analyze.coherency()} expects two aligned series in one data
#' frame plus a \code{dt} value in the analysis time units, and returns
#' cross-wavelet power, coherence, phase angles, p-values, Fourier periods, and
#' cone-of-influence fields.
#'
#' @examples
#' \donttest{
#' wc <- dm_wavelet_coherence(
#'   dm = gf_nepa17,
#'   clim = ktm_clim_hourly,
#'   TreeNum = 1,
#'   clim_vars = c("temp", "VPD"),
#'   source = "raw",
#'   dm_fun = "mean",
#'   clim_funs = c(temp = "mean", VPD = "mean"),
#'   loess_span = 0,
#'   make.pval = TRUE,
#'   n.sim = 10,
#'   verbose = FALSE
#' )
#'
#' plot(wc)
#' plot(wc, type = "cross_power")
#' plot(wc, type = "average_coherence")
#' plot(wc, type = "phase")
#' plot(wc, type = "series")
#' }
#'
#' @importFrom dplyr bind_rows arrange %>%
#' @importFrom tibble tibble as_tibble
#' @importFrom lubridate ymd_hms parse_date_time
#' @importFrom zoo na.approx na.locf
#' @importFrom stats setNames
#' @export
dm_wavelet_coherence <- function(dm,
                                 clim,
                                 TreeNum = "all",
                                 clim_vars = "all",
                                 source = c("auto", "raw", "detrended", "first_diff"),
                                 detrended_col = c("detrended_data"),
                                 dm_fun = c("mean", "max", "min", "sum"),
                                 clim_funs = "mean",
                                 na_action = c("interpolate", "fail"),
                                 loess_span = 0,
                                 dj = 1 / 20,
                                 lowerPeriod = NULL,
                                 upperPeriod = NULL,
                                 window.type.t = 1,
                                 window.type.s = 1,
                                 window.size.t = 5,
                                 window.size.s = 1 / 4,
                                 make.pval = TRUE,
                                 make_pval = NULL,
                                 method = "white.noise",
                                 n.sim = 10,
                                 n_sim = NULL,
                                 verbose = TRUE) {

  TIME <- NULL

  if (!requireNamespace("WaveletComp", quietly = TRUE)) {
    stop("Package 'WaveletComp' is required for dm_wavelet_coherence().")
  }

  source <- match.arg(source)
  detrended_col <- match.arg(detrended_col)
  dm_fun <- match.arg(dm_fun)
  na_action <- match.arg(na_action)

  # support snake_case aliases too
  if (!is.null(make_pval)) {
    make.pval <- make_pval
  }
  if (!is.null(n_sim)) {
    n.sim <- n_sim
  }

  cl <- match.call()

  # dendrometer input
  if (inherits(dm, "dm_detrended")) {
    dm_type <- "dm_detrended"
    if (source == "raw") {
      stop("source = 'raw' is not valid for a dm_detrended object.")
    }
    dm0 <- tibble::as_tibble(dm[[detrended_col]])
    dm_meta <- intersect(
      c("TIME", "doy", "season_label", "season_start", "season_end", "season_day"),
      names(dm0)
    )
    dm_cols <- dmwc_select_cols(dm0, TreeNum, exclude = dm_meta)
    dm_dat <- dm0[, c("TIME", dm_cols), drop = FALSE]
    source_used <- if (source == "auto") "detrended" else source
  } else if (is.data.frame(dm)) {
    dm_type <- "raw"
    if (source == "detrended") {
      stop("source = 'detrended' requires a dm_detrended object.")
    }
    dm0 <- tibble::as_tibble(dm)
    names(dm0)[1] <- "TIME"
    dm_cols <- dmwc_select_cols(dm0, TreeNum, exclude = "TIME")
    dm_dat <- dm0[, c("TIME", dm_cols), drop = FALSE]
    source_used <- if (source == "auto") "raw" else source
  } else {
    stop("'dm' must be a data.frame or an object of class 'dm_detrended'.")
  }

  # climate input
  if (!is.data.frame(clim)) {
    stop("'clim' must be a data.frame.")
  }
  clim0 <- tibble::as_tibble(clim)
  names(clim0)[1] <- "TIME"
  clim_cols <- dmwc_select_cols(clim0, clim_vars, exclude = "TIME")
  clim_dat <- clim0[, c("TIME", clim_cols), drop = FALSE]

  # parse times
  dm_dat$TIME <- dmwc_parse_time(dm_dat$TIME)
  clim_dat$TIME <- dmwc_parse_time(clim_dat$TIME)

  if (any(is.na(dm_dat$TIME))) {
    stop("Some dendrometer timestamps could not be parsed.")
  }
  if (any(is.na(clim_dat$TIME))) {
    stop("Some climate timestamps could not be parsed.")
  }

  dm_dat <- dm_dat[order(dm_dat$TIME), , drop = FALSE]
  clim_dat <- clim_dat[order(clim_dat$TIME), , drop = FALSE]

  # crop to common overlap
  t_start <- max(min(dm_dat$TIME, na.rm = TRUE), min(clim_dat$TIME, na.rm = TRUE))
  t_end   <- min(max(dm_dat$TIME, na.rm = TRUE), max(clim_dat$TIME, na.rm = TRUE))

  if (!is.finite(as.numeric(t_start)) || !is.finite(as.numeric(t_end)) || t_start >= t_end) {
    stop("No overlapping time window between dendrometer and climate data.")
  }

  dm_dat <- dm_dat[dm_dat$TIME >= t_start & dm_dat$TIME <= t_end, , drop = FALSE]
  clim_dat <- clim_dat[clim_dat$TIME >= t_start & clim_dat$TIME <= t_end, , drop = FALSE]

  if (nrow(dm_dat) < 4 || nrow(clim_dat) < 4) {
    stop("Too few overlapping observations after cropping.")
  }

  # detect resolution and choose coarser
  dm_dt <- dmwc_infer_dt(dm_dat$TIME)
  clim_dt <- dmwc_infer_dt(clim_dat$TIME)

  target_sec <- max(dm_dt$sec, clim_dt$sec)
  target_info <- dmwc_infer_dt(
    seq(as.POSIXct("2000-01-01", tz = "UTC"), by = target_sec, length.out = 5)
  )

  # function map
  dm_fun_map <- setNames(
    replicate(length(dm_cols), dmwc_fun_match(dm_fun), simplify = FALSE),
    dm_cols
  )

  if (length(clim_funs) == 1) {
    clim_fun_map <- setNames(
      replicate(length(clim_cols), dmwc_fun_match(clim_funs), simplify = FALSE),
      clim_cols
    )
  } else {
    if (is.null(names(clim_funs))) {
      stop("'clim_funs' must be length 1 or a named vector/list matching climate variables.")
    }
    missing_fun <- setdiff(clim_cols, names(clim_funs))
    if (length(missing_fun) > 0) {
      stop("Missing aggregation functions for climate variables: ",
           paste(missing_fun, collapse = ", "))
    }
    clim_fun_map <- lapply(clim_cols, function(cc) dmwc_fun_match(clim_funs[[cc]]))
    names(clim_fun_map) <- clim_cols
  }

  # resample both to coarser resolution
  dm_res <- dmwc_resample_regular(dm_dat, target_sec = target_sec, fun_map = dm_fun_map)
  clim_res <- dmwc_resample_regular(clim_dat, target_sec = target_sec, fun_map = clim_fun_map)

  # align exactly on same time grid
  common_time <- intersect(dm_res$TIME, clim_res$TIME)
  if (length(common_time) < 8) {
    stop("Too few aligned observations after resampling.")
  }

  dm_res <- dm_res[dm_res$TIME %in% common_time, , drop = FALSE]
  clim_res <- clim_res[clim_res$TIME %in% common_time, , drop = FALSE]

  dm_res <- dm_res[order(dm_res$TIME), , drop = FALSE]
  clim_res <- clim_res[order(clim_res$TIME), , drop = FALSE]

  # NA handling
  for (cc in setdiff(names(dm_res), "TIME")) {
    if (na_action == "interpolate") {
      dm_res[[cc]] <- dmwc_fill_na(dm_res[[cc]])
    } else if (any(is.na(dm_res[[cc]]))) {
      stop("Missing values found in dendrometer series after resampling: ", cc)
    }
  }

  for (cc in setdiff(names(clim_res), "TIME")) {
    if (na_action == "interpolate") {
      clim_res[[cc]] <- dmwc_fill_na(clim_res[[cc]])
    } else if (any(is.na(clim_res[[cc]]))) {
      stop("Missing values found in climate series after resampling: ", cc)
    }
  }

  # first differences on dendrometer if requested
  if (source_used == "first_diff") {
    for (cc in dm_cols) {
      dm_res[[cc]] <- c(NA_real_, diff(dm_res[[cc]]))
    }
    dm_res <- dm_res[-1, , drop = FALSE]
    clim_res <- clim_res[-1, , drop = FALSE]
  }

  # final NA check
  for (cc in dm_cols) {
    if (any(is.na(dm_res[[cc]]))) {
      stop("Dendrometer series still contains missing values: ", cc)
    }
  }
  for (cc in clim_cols) {
    if (any(is.na(clim_res[[cc]]))) {
      stop("Climate series still contains missing values: ", cc)
    }
  }

  # pairwise coherence
  pair_results <- list()
  aligned_list <- list()
  summary_rows <- list()
  idx <- 1L

  for (dm_col in dm_cols) {
    for (clim_col in clim_cols) {
      pair_name <- paste(dm_col, clim_col, sep = "__")

      pair_df <- data.frame(
        date = dm_res$TIME,
        dm = as.numeric(dm_res[[dm_col]]),
        clim = as.numeric(clim_res[[clim_col]])
      )

      coh_args <- list(
        my.data = pair_df,
        my.pair = c("dm", "clim"),
        loess.span = loess_span,
        dt = target_info$dt,
        dj = dj,
        window.type.t = window.type.t,
        window.type.s = window.type.s,
        window.size.t = window.size.t,
        window.size.s = window.size.s,
        make.pval = make.pval,
        method = method,
        n.sim = n.sim,
        verbose = FALSE
      )

      if (!is.null(lowerPeriod)) coh_args$lowerPeriod <- lowerPeriod
      if (!is.null(upperPeriod)) coh_args$upperPeriod <- upperPeriod

      wc <- do.call(WaveletComp::analyze.coherency, coh_args)

      period_hours <- dmwc_period_to_hours(wc$Period, target_info$unit)

      pair_results[[pair_name]] <- list(
        wavelet = wc,
        time = pair_df$date,
        dm_series = pair_df$dm,
        clim_series = pair_df$clim,
        dm_name = dm_col,
        clim_name = clim_col,
        period_native = wc$Period,
        period_hours = period_hours,
        avg_coherence = if (!is.null(wc$Coherence.avg)) as.numeric(wc$Coherence.avg) else NA_real_,
        avg_cross_power = if (!is.null(wc$Power.xy.avg)) as.numeric(wc$Power.xy.avg) else NA_real_
      )

      aligned_list[[idx]] <- tibble::tibble(
        TIME = pair_df$date,
        pair = pair_name,
        dm_series = pair_df$dm,
        clim_series = pair_df$clim,
        dm_name = dm_col,
        clim_name = clim_col
      )

      avg_coh <- pair_results[[pair_name]]$avg_coherence
      if (all(is.na(avg_coh))) {
        dom_period <- NA_real_
        max_avg_coh <- NA_real_
      } else {
        jj <- which.max(avg_coh)
        dom_period <- period_hours[jj]
        max_avg_coh <- avg_coh[jj]
      }

      summary_rows[[idx]] <- tibble::tibble(
        pair = pair_name,
        dm_name = dm_col,
        clim_name = clim_col,
        n_obs = nrow(pair_df),
        dominant_coherence_period_hours = dom_period,
        max_average_coherence = max_avg_coh
      )

      idx <- idx + 1L
    }
  }

  out <- list(
    call = cl,
    dm_type = dm_type,
    source = source_used,
    dm_series = dm_cols,
    clim_vars = clim_cols,
    time_unit = target_info$unit,
    dt = target_info$dt,
    dt_hours = target_info$hours,
    resolution_seconds = target_sec,
    overlap = c(start = as.character(t_start), end = as.character(t_end)),
    aligned_data = dplyr::bind_rows(aligned_list),
    results = pair_results,
    pair_summary = dplyr::bind_rows(summary_rows),
    settings = list(
      dm_fun = dm_fun,
      clim_funs = clim_funs,
      loess_span = loess_span,
      dj = dj,
      lowerPeriod = lowerPeriod,
      upperPeriod = upperPeriod,
      window.type.t = window.type.t,
      window.type.s = window.type.s,
      window.size.t = window.size.t,
      window.size.s = window.size.s,
      make.pval = make.pval,
      method = method,
      n.sim = n.sim,
      na_action = na_action
    )
  )

  class(out) <- "dm_wavelet_coherence"

  if (isTRUE(verbose)) {
    message(
      "dm_wavelet_coherence completed for ", length(pair_results),
      " pair(s). Coarser detected resolution: ",
      out$dt, " ", out$time_unit, " (", round(out$dt_hours, 4), " hours)."
    )
  }

  out
}


# plot method ------------------------------------------------------------------

#' Plot method for dm_wavelet_coherence objects
#'
#' @param x An object of class \code{"dm_wavelet_coherence"}.
#' @param y Unused.
#' @param pair Optional pair name(s) to plot. If \code{NULL}, the first pair is used.
#' @param type One of \code{"coherence"}, \code{"cross_power"},
#'   \code{"average_coherence"}, \code{"average_cross_power"}, \code{"phase"},
#'   or \code{"series"}.
#' @param facet Logical. If \code{TRUE} and multiple pairs are selected, facet them.
#' @param log_period Logical.
#' @param log_power Logical for raster intensity.
#' @param clip_quantile Optional clipping quantiles for raster intensity.
#' @param show_sig Logical.
#' @param show_coi Logical.
#' @param siglvl Significance level.
#' @param sig_color Significance contour color.
#' @param sig_size Significance contour linewidth.
#' @param coi_fill COI fill color.
#' @param coi_alpha COI alpha.
#' @param main Optional title.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object.
#'
#' @method plot dm_wavelet_coherence
#' @importFrom dplyr bind_rows
#' @importFrom tibble tibble
#' @importFrom ggplot2 ggplot aes geom_raster geom_line geom_contour geom_segment
#'   geom_polygon facet_wrap theme_bw theme element_text labs scale_y_log10
#'   scale_x_log10 scale_fill_viridis_c coord_cartesian
#' @export
plot.dm_wavelet_coherence <- function(x,
                                      y = NULL,
                                      pair = NULL,
                                      type = c("coherence", "cross_power", "average_coherence",
                                               "average_cross_power", "phase", "series"),
                                      facet = TRUE,
                                      log_period = TRUE,
                                      log_power = TRUE,
                                      clip_quantile = c(0.01, 0.99),
                                      show_sig = TRUE,
                                      show_coi = TRUE,
                                      siglvl = 0.05,
                                      sig_color = "black",
                                      sig_size = 0.4,
                                      coi_fill = "white",
                                      coi_alpha = 0.45,
                                      main = NULL,
                                      ...) {
  TIME <- value <- signal <- period_hours <- pval <- TIME2 <- period2 <- NULL


  if (!inherits(x, "dm_wavelet_coherence")) {
    stop("'x' must be an object of class 'dm_wavelet_coherence'.")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for plot.dm_wavelet_coherence().")
  }

  type <- match.arg(type)

  if (is.null(pair)) {
    pair <- names(x$results)[1]
  }

  miss <- setdiff(pair, names(x$results))
  if (length(miss) > 0) {
    stop("These requested pairs were not found: ", paste(miss, collapse = ", "))
  }

  # aligned series
  if (type == "series") {
    dat <- x$aligned_data
    dat <- dat[dat$pair %in% pair, , drop = FALSE]

    long_dat <- tibble::tibble(
      TIME = c(dat$TIME, dat$TIME),
      pair = c(dat$pair, dat$pair),
      value = c(dat$dm_series, dat$clim_series),
      signal = c(rep("dendrometer", nrow(dat)), rep("climate", nrow(dat)))
    )

    p <- ggplot2::ggplot(long_dat, ggplot2::aes(x = TIME, y = value, colour = signal)) +
      ggplot2::geom_line() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "right",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Time",
        y = "Value",
        colour = "Series",
        title = if (is.null(main)) "Aligned dendrometer and climate series" else main
      )

    if (length(pair) > 1 && isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ pair"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  # average lines
  if (type %in% c("average_coherence", "average_cross_power")) {
    df_list <- lapply(pair, function(pp) {
      res <- x$results[[pp]]
      tibble::tibble(
        pair = pp,
        period_hours = res$period_hours,
        value = if (type == "average_coherence") res$avg_coherence else res$avg_cross_power
      )
    })
    dat <- dplyr::bind_rows(df_list)

    p <- ggplot2::ggplot(dat, ggplot2::aes(x = period_hours, y = value, colour = pair)) +
      ggplot2::geom_line() +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = if (length(pair) > 1) "right" else "none",
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      ) +
      ggplot2::labs(
        x = "Period (hours)",
        y = if (type == "average_coherence") "Average coherence" else "Average cross-wavelet power",
        colour = "Pair",
        title = if (is.null(main)) {
          if (type == "average_coherence") "Average wavelet coherence" else "Average cross-wavelet power"
        } else main
      )

    if (isTRUE(log_period) && all(dat$period_hours > 0, na.rm = TRUE)) {
      p <- p + ggplot2::scale_x_log10()
    }

    if (length(pair) > 1 && isTRUE(facet)) {
      p <- p + ggplot2::facet_wrap(stats::as.formula("~ pair"), scales = "free_y", ncol = 1)
    }

    return(p)
  }

  # raster-like plots
  df_list <- lapply(pair, function(pp) {
    res <- x$results[[pp]]
    wc <- res$wavelet

    if (type == "coherence") {
      mat <- as.matrix(wc$Coherence)
      pmat <- if (!is.null(wc$Coherence.pval)) as.matrix(wc$Coherence.pval) else NULL
    } else if (type == "cross_power") {
      mat <- as.matrix(wc$Power.xy)
      pmat <- if (!is.null(wc$Power.xy.pval)) as.matrix(wc$Power.xy.pval) else NULL
    } else { # phase
      mat <- as.matrix(wc$Angle)
      pmat <- if (!is.null(wc$Coherence.pval)) as.matrix(wc$Coherence.pval) else NULL
    }

    n_p <- min(nrow(mat), length(res$period_hours))
    n_t <- min(ncol(mat), length(res$time))
    mat <- mat[seq_len(n_p), seq_len(n_t), drop = FALSE]
    per <- res$period_hours[seq_len(n_p)]
    tt <- res$time[seq_len(n_t)]

    df <- expand.grid(
      period_hours = per,
      TIME = tt,
      KEEP.OUT.ATTRS = FALSE,
      stringsAsFactors = FALSE
    )
    df$value <- c(mat)
    df$pair <- pp
    if (!is.null(pmat)) {
      pmat <- pmat[seq_len(n_p), seq_len(n_t), drop = FALSE]
      df$pval <- c(pmat)
    } else {
      df$pval <- NA_real_
    }
    df
  })

  dat <- dplyr::bind_rows(df_list)
  dat <- dat[is.finite(dat$period_hours) & dat$period_hours > 0, , drop = FALSE]
  y_rng <- range(dat$period_hours, na.rm = TRUE)

  plot_val <- dat$value
  fill_lab <- if (type == "coherence") "Coherence" else if (type == "cross_power") "Cross-power" else "Phase (rad)"

  if (type == "cross_power") {
    if (!is.null(clip_quantile)) {
      qv <- stats::quantile(plot_val, probs = clip_quantile, na.rm = TRUE, names = FALSE)
      plot_val <- pmin(pmax(plot_val, qv[1]), qv[2])
    }
    if (isTRUE(log_power)) {
      eps <- max(min(plot_val[plot_val > 0], na.rm = TRUE) / 10, 1e-12)
      if (!is.finite(eps)) eps <- 1e-12
      plot_val <- log10(plot_val + eps)
      fill_lab <- "log10(Cross-power)"
    }
  }

  dat$plot_val <- plot_val

  p <- ggplot2::ggplot(dat, ggplot2::aes(x = TIME, y = period_hours, fill = plot_val)) +
    ggplot2::geom_raster() +
    ggplot2::scale_fill_viridis_c(option = "D", na.value = "grey85") +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "right",
      axis.title = ggplot2::element_text(size = 14),
      axis.text = ggplot2::element_text(size = 11)
    ) +
    ggplot2::labs(
      x = "Time",
      y = "Period (hours)",
      fill = fill_lab,
      title = if (is.null(main)) {
        switch(type,
               coherence = "Wavelet coherence",
               cross_power = "Cross-wavelet power",
               phase = "Wavelet phase difference")
      } else main
    )

  # COI overlay
  if (isTRUE(show_coi)) {
    coi_list <- lapply(pair, function(pp) {
      res <- x$results[[pp]]
      wc <- res$wavelet
      if (is.null(wc$coi.1) || is.null(wc$coi.2) || is.null(wc$axis.1) || is.null(wc$axis.2)) return(NULL)

      tz_use <- attr(res$time, "tzone")
      if (is.null(tz_use) || length(tz_use) == 0) tz_use <- "UTC"

      x_num <- stats::approx(
        x = as.numeric(wc$axis.1),
        y = as.numeric(res$time),
        xout = as.numeric(wc$coi.1),
        rule = 2
      )$y

      period_native <- 2^as.numeric(wc$coi.2)
      period_hours <- dmwc_period_to_hours(period_native, x$time_unit)
      period_hours <- pmin(pmax(period_hours, y_rng[1]), y_rng[2])

      tibble::tibble(
        TIME = as.POSIXct(x_num, origin = "1970-01-01", tz = tz_use),
        period_hours = period_hours,
        pair = pp
      )
    })

    coi_dat <- dplyr::bind_rows(coi_list)
    if (nrow(coi_dat) > 0) {
      p <- p + ggplot2::geom_polygon(
        data = coi_dat,
        mapping = ggplot2::aes(x = TIME, y = period_hours, group = pair),
        inherit.aes = FALSE,
        fill = coi_fill,
        alpha = coi_alpha,
        colour = NA
      )
    }
  }

  # significance contour
  if (isTRUE(show_sig) && any(is.finite(dat$pval))) {
    sig_dat <- dat[is.finite(dat$pval), , drop = FALSE]
    p <- p + ggplot2::geom_contour(
      data = sig_dat,
      mapping = ggplot2::aes(x = TIME, y = period_hours, z = pval),
      breaks = siglvl,
      colour = sig_color,
      linewidth = sig_size,
      inherit.aes = FALSE
    )
  }

  # phase arrows
  if (type == "phase") {
    phase_list <- lapply(pair, function(pp) {
      res <- x$results[[pp]]
      wc <- res$wavelet
      ang <- as.matrix(wc$Angle)
      coh <- if (!is.null(wc$Coherence)) as.matrix(wc$Coherence) else NULL

      n_p <- min(nrow(ang), length(res$period_hours))
      n_t <- min(ncol(ang), length(res$time))

      # thin grid for plotting arrows
      p_idx <- unique(round(seq(1, n_p, length.out = min(20, n_p))))
      t_idx <- unique(round(seq(1, n_t, length.out = min(30, n_t))))

      grd <- expand.grid(pi = p_idx, ti = t_idx, KEEP.OUT.ATTRS = FALSE)

      ph <- ang[cbind(grd$pi, grd$ti)]
      if (!is.null(coh)) {
        cv <- coh[cbind(grd$pi, grd$ti)]
      } else {
        cv <- rep(1, length(ph))
      }

      tibble::tibble(
        TIME = res$time[grd$ti],
        period_hours = res$period_hours[grd$pi],
        angle = ph,
        coherence = cv,
        pair = pp
      )
    })

    phase_dat <- dplyr::bind_rows(phase_list)
    phase_dat <- phase_dat[is.finite(phase_dat$angle) &
                             is.finite(phase_dat$period_hours) &
                             phase_dat$period_hours > 0 &
                             is.finite(phase_dat$coherence) &
                             phase_dat$coherence >= 0.5, , drop = FALSE]

    if (nrow(phase_dat) > 0) {
      # arrow size in data units
      xspan <- as.numeric(diff(range(dat$TIME, na.rm = TRUE)), units = "secs")
      yspan <- diff(y_rng)
      dx <- (xspan / 40) * cos(phase_dat$angle)
      dy <- (yspan / 40) * sin(phase_dat$angle)

      phase_dat$TIME2 <- phase_dat$TIME + dx
      phase_dat$period2 <- phase_dat$period_hours + dy

      p <- p + ggplot2::geom_segment(
        data = phase_dat,
        mapping = ggplot2::aes(x = TIME, y = period_hours, xend = TIME2, yend = period2),
        inherit.aes = FALSE,
        colour = "black",
        linewidth = 0.25,
        arrow = grid::arrow(length = grid::unit(0.08, "cm"))
      )
    }
  }

  if (isTRUE(log_period) && all(y_rng > 0)) {
    p <- p + ggplot2::scale_y_log10(limits = y_rng)
  } else {
    p <- p + ggplot2::coord_cartesian(ylim = y_rng, expand = FALSE)
  }

  if (length(pair) > 1 && isTRUE(facet)) {
    p <- p + ggplot2::facet_wrap(stats::as.formula("~ pair"), scales = "free_x", ncol = 1)
  }

  p
}

# =========================================================
# Internal helpers for climate attachment and plotting
# =========================================================

.dm_subset_window <- function(dat, time_col, Year = NULL, DOY = NULL) {

  if (!time_col %in% names(dat)) {
    stop(sprintf("Time column '%s' was not found.", time_col))
  }

  tt <- dat[[time_col]]

  if (!inherits(tt, "Date") && !inherits(tt, "POSIXct")) {
    tt <- as.POSIXct(tt)
  }

  keep <- rep(TRUE, length(tt))

  if (!is.null(Year)) {
    keep <- keep & lubridate::year(tt) %in% Year
  }

  if (!is.null(DOY)) {
    if (length(DOY) != 2) {
      stop("'DOY' must be a numeric vector of length 2.")
    }

    doy_min <- min(DOY, na.rm = TRUE)
    doy_max <- max(DOY, na.rm = TRUE)

    keep <- keep &
      lubridate::yday(tt) >= doy_min &
      lubridate::yday(tt) <= doy_max
  }

  out <- dat[keep, , drop = FALSE]

  if (nrow(out) == 0) {
    stop("No data available for the selected Year/DOY window.")
  }

  out
}


.dm_label_phase <- function(ph, obj_class = c("ZG", "SC")) {
  obj_class <- match.arg(obj_class)

  if (obj_class == "ZG") {
    factor(ph, levels = c(1, 2), labels = c("TWD", "GRO"))
  } else {
    factor(ph, levels = c(1, 2, 3),
           labels = c("Shrinkage", "Expansion", "Increment"))
  }
}


.dm_detect_cycle_climate_cols <- function(dat, kind = c("ZG", "SC")) {
  kind <- match.arg(kind)

  core <- if (kind == "ZG") {
    c(
      "Phases", "Start", "End", "Duration_h", "Magnitude", "rate",
      "max.twd", "Max.twd.time", "ABr.value", "Avg.twd", "STD.twd",
      "AUC_load", "AUC_total", "DOY"
    )
  } else {
    c(
      "Phases", "Start", "End", "Duration_h", "Duration_m",
      "Magnitude", "rate", "DOY"
    )
  }

  setdiff(names(dat), core)
}


.dm_detect_point_climate_cols <- function(dat, kind = c("ZG", "SC")) {
  kind <- match.arg(kind)

  core <- if (kind == "ZG") {
    c("TIME", "dm", "Phases", "TWD", "GRO")
  } else {
    c("TIME", "dm", "Phases")
  }

  setdiff(names(dat), core)
}


# =========================================================
# Plot one climate variable
# =========================================================

#' Plot climate attached to dendrometer outputs
#'
#' @description
#' General plotting function for dendrometer outputs with attached climate
#' information. It works with objects of class \code{daily_output_clim},
#' \code{ZG_output_clim}, and \code{SC_output_clim}, usually produced by
#' \code{dm_add_climate()}, \code{dm_join_daily_clim()},
#' \code{dm_join_phase_clim()}, or \code{dm_join_subdaily_clim()}.
#'
#' The function can display one selected climate variable through time,
#' by phase or day status, as boxplots or violin plots, or as a
#' climate-response relationship.
#'
#' @param x A climate-augmented dendrometer object.
#' @param climate_var Character. Name of the climate variable to plot.
#' @param metric_var Optional character. Biological response variable used when
#'   \code{type = "relation"}. If \code{NULL}, a suitable default is selected
#'   when possible.
#' @param scale Character. Data level to use. One of \code{"auto"},
#'   \code{"daily"}, \code{"cycle"}, or \code{"point"}.
#' @param type Character. Plot type. One of \code{"timeseries"},
#'   \code{"boxplot"}, \code{"violin"}, \code{"both"}, or \code{"relation"}.
#' @param group_var Optional character. Grouping variable. If \code{NULL},
#'   \code{Day_status} is used for daily outputs when available, and
#'   \code{Phases} is used for cycle- or point-level phase outputs.
#' @param Year Optional numeric year or vector of years for subsetting.
#' @param DOY Optional numeric vector of length 2 giving the day-of-year range.
#' @param point_alpha Numeric. Alpha value for points.
#' @param add_smooth Logical. If \code{TRUE} and \code{type = "relation"},
#'   a linear trend line is added.
#' @param temporal Deprecated argument kept for compatibility with older
#'   examples. It is ignored.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' zg_clim <- dm_add_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   scale = "phase",
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' dm_plot_climate(
#'   zg_clim,
#'   climate_var = "VPD_mean_phase",
#'   scale = "cycle",
#'   type = "boxplot"
#' )
#'
#' plot(
#'   zg_clim,
#'   climate_var = "VPD_mean_phase",
#'   scale = "cycle",
#'   type = "boxplot"
#' )
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_boxplot geom_violin
#'   geom_smooth theme_minimal theme element_text labs position_jitter
#' @importFrom rlang .data
#' @importFrom tibble as_tibble
#' @importFrom lubridate year yday
#'
#' @export
dm_plot_climate <- function(x,
                            climate_var,
                            metric_var = NULL,
                            scale = c("auto", "daily", "cycle", "point"),
                            type = c("timeseries", "boxplot", "violin", "both", "relation"),
                            group_var = NULL,
                            Year = NULL,
                            DOY = NULL,
                            point_alpha = 0.6,
                            add_smooth = FALSE,
                            temporal = NULL) {

  scale <- match.arg(scale)
  type <- match.arg(type)

  dat <- NULL
  time_col <- NULL
  default_group <- NULL
  obj_kind <- NULL

  # -----------------------------------------------------
  # Choose data source
  # -----------------------------------------------------
  if (inherits(x, "daily_output_clim")) {

    dat <- tibble::as_tibble(x)
    time_col <- "DATE"
    obj_kind <- "daily"
    default_group <- if ("Day_status" %in% names(dat)) "Day_status" else NULL

  } else if (inherits(x, "ZG_output_clim") || inherits(x, "SC_output_clim")) {

    kind <- if (inherits(x, "ZG_output_clim") || inherits(x, "ZG_output")) {
      "ZG"
    } else {
      "SC"
    }

    cyc_name <- if (kind == "ZG") "ZG_cycle" else "SC_cycle"
    ph_name  <- if (kind == "ZG") "ZG_phase" else "SC_phase"

    cyc_dat <- tibble::as_tibble(x[[cyc_name]])
    ph_dat  <- tibble::as_tibble(x[[ph_name]])

    cyc_clim <- .dm_detect_cycle_climate_cols(cyc_dat, kind = kind)
    ph_clim  <- .dm_detect_point_climate_cols(ph_dat, kind = kind)

    if (scale == "auto") {
      if (climate_var %in% cyc_clim) {
        scale <- "cycle"
      } else if (climate_var %in% ph_clim) {
        scale <- "point"
      } else {
        stop(
          sprintf(
            "Climate variable '%s' was not found in cycle-level or point-level data.",
            climate_var
          )
        )
      }
    }

    if (scale == "daily") {
      stop("For ZG/SC outputs, use scale = 'cycle' or scale = 'point'.")
    }

    if (scale == "cycle") {
      dat <- cyc_dat
      time_col <- "Start"
    } else {
      dat <- ph_dat
      time_col <- "TIME"
    }

    obj_kind <- kind
    default_group <- "Phases"

    if ("Phases" %in% names(dat)) {
      dat$Phases <- .dm_label_phase(dat$Phases, obj_class = kind)
    }

  } else {
    stop(
      "'x' must be a climate-augmented object returned by dm_add_climate(), ",
      "dm_join_daily_clim(), dm_join_phase_clim(), or dm_join_subdaily_clim()."
    )
  }

  if (!climate_var %in% names(dat)) {
    stop(sprintf("Climate variable '%s' was not found in the selected data.", climate_var))
  }

  if (is.null(group_var)) {
    group_var <- default_group
  }

  if (!is.null(group_var) && !group_var %in% names(dat)) {
    stop(sprintf("Grouping variable '%s' was not found in the selected data.", group_var))
  }

  dat <- .dm_subset_window(dat, time_col = time_col, Year = Year, DOY = DOY)

  # -----------------------------------------------------
  # Time series
  # -----------------------------------------------------
  if (type == "timeseries") {

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = .data[[time_col]], y = .data[[climate_var]])
    )

    if (!is.null(group_var)) {
      p <- p +
        ggplot2::geom_line(
          ggplot2::aes(color = .data[[group_var]]),
          linewidth = 0.5
        ) +
        ggplot2::geom_point(
          ggplot2::aes(color = .data[[group_var]]),
          size = 1.5,
          alpha = point_alpha
        ) +
        ggplot2::labs(color = group_var)
    } else {
      p <- p +
        ggplot2::geom_line(linewidth = 0.5) +
        ggplot2::geom_point(size = 1.5, alpha = point_alpha)
    }

    p <- p +
      ggplot2::labs(x = time_col, y = climate_var) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text  = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  # -----------------------------------------------------
  # Boxplot / violin / both
  # -----------------------------------------------------
  if (type %in% c("boxplot", "violin", "both")) {

    if (is.null(group_var)) {
      stop("A grouping variable is required for boxplot/violin style plots.")
    }

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = .data[[group_var]], y = .data[[climate_var]])
    )

    if (type %in% c("violin", "both")) {
      p <- p +
        ggplot2::geom_violin(fill = "grey80", alpha = 0.8, trim = FALSE)
    }

    if (type %in% c("boxplot", "both")) {
      p <- p +
        ggplot2::geom_boxplot(
          width = if (type == "both") 0.18 else 0.6,
          outlier.shape = NA,
          fill = if (type == "both") "white" else "grey80"
        )
    }

    p <- p +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = 0.12, height = 0),
        size = 1.2,
        alpha = point_alpha,
        color = "grey25"
      ) +
      ggplot2::labs(x = group_var, y = climate_var) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text  = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }

  # -----------------------------------------------------
  # Climate-response relationship
  # -----------------------------------------------------
  if (type == "relation") {

    if (is.null(metric_var)) {
      if (obj_kind == "daily" && "amplitude" %in% names(dat)) {
        metric_var <- "amplitude"
      } else if (obj_kind == "ZG") {
        if ("ABr.value" %in% names(dat) && any(!is.na(dat$ABr.value))) {
          metric_var <- "ABr.value"
        } else if ("Magnitude" %in% names(dat)) {
          metric_var <- "Magnitude"
        } else {
          metric_var <- names(dat)[2]
        }
      } else if (obj_kind == "SC") {
        if ("Magnitude" %in% names(dat)) {
          metric_var <- "Magnitude"
        } else {
          metric_var <- names(dat)[2]
        }
      } else {
        stop("Please provide 'metric_var' for type = 'relation'.")
      }
    }

    if (!metric_var %in% names(dat)) {
      stop(sprintf("Metric variable '%s' was not found in the selected data.", metric_var))
    }

    p <- ggplot2::ggplot(
      dat,
      ggplot2::aes(x = .data[[climate_var]], y = .data[[metric_var]])
    )

    if (!is.null(group_var)) {
      p <- p +
        ggplot2::geom_point(
          ggplot2::aes(color = .data[[group_var]]),
          alpha = point_alpha
        )
    } else {
      p <- p +
        ggplot2::geom_point(alpha = point_alpha)
    }

    if (isTRUE(add_smooth)) {
      p <- p +
        ggplot2::geom_smooth(method = "lm", se = TRUE, color = "black")
    }

    p <- p +
      ggplot2::labs(x = climate_var, y = metric_var, color = group_var) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text  = ggplot2::element_text(size = 12)
      )

    print(p)
    return(invisible(p))
  }
}


# =========================================================
# Compare multiple climate variables
# =========================================================

#' Compare multiple climate variables attached to dendrometer outputs
#'
#' @description
#' Plots and compares multiple climate variables attached to climate-augmented
#' dendrometer outputs. It works with objects of class \code{daily_output_clim},
#' \code{ZG_output_clim}, and \code{SC_output_clim}.
#'
#' Supported plot types include faceted time-series, grouped boxplots,
#' violin plots, combined violin-boxplots, correlation heatmaps, and
#' regression heatmaps.
#'
#' @param x A climate-augmented dendrometer object.
#' @param climate_vars Character vector of climate variables to compare. If
#'   \code{NULL}, all detected climate variables at the selected scale are used.
#' @param numeric_vars Optional character vector of numeric variables used when
#'   \code{type = "cor_heatmap"} or \code{type = "regression_heatmap"}. If
#'   \code{NULL}, all numeric variables in the selected data are used.
#' @param scale Character. Data level to use. One of \code{"auto"},
#'   \code{"daily"}, \code{"cycle"}, or \code{"point"}.
#' @param type Character. Plot type. One of \code{"timeseries"},
#'   \code{"boxplot"}, \code{"violin"}, \code{"both"},
#'   \code{"cor_heatmap"}, or \code{"regression_heatmap"}.
#' @param group_var Optional character. Grouping variable. If \code{NULL},
#'   \code{Day_status} is used for daily outputs when available, and
#'   \code{Phases} is used for phase outputs.
#' @param Year Optional numeric year or vector of years for subsetting.
#' @param DOY Optional numeric vector of length 2 giving the day-of-year range.
#' @param box_scales Character. Facet scale behavior for grouped plots. One of
#'   \code{"free_y"} or \code{"fixed"}.
#' @param cor_method Character. Correlation method. One of \code{"pearson"},
#'   \code{"spearman"}, or \code{"kendall"}.
#' @param use_pairwise Logical. If \code{TRUE}, pairwise complete observations
#'   are used in correlations.
#' @param regression_stat Character. Statistic shown in regression heatmaps.
#'   One of \code{"r.squared"}, \code{"slope"}, or \code{"p.value"}.
#' @param point_alpha Numeric. Alpha value for points.
#' @param show_values Logical. If \code{TRUE}, numeric values are printed inside
#'   heatmap tiles.
#' @param temporal Deprecated argument kept for compatibility with older
#'   examples. It is ignored.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @details
#' For \code{type = "regression_heatmap"}, pairwise simple linear regressions
#' are fitted as \code{y ~ x} for every variable combination. The heatmap can
#' display \eqn{R^2}, slope, or p-value.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' zg_clim <- dm_add_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   scale = "phase",
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#'
#' dm_plot_climate_compare(
#'   zg_clim,
#'   climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
#'   scale = "cycle",
#'   type = "boxplot"
#' )
#'
#' plot(
#'   zg_clim,
#'   climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
#'   scale = "cycle",
#'   type = "boxplot"
#' )
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_boxplot geom_violin
#'   geom_tile geom_text facet_wrap theme_minimal theme element_text labs
#'   position_jitter scale_fill_gradient2 scale_fill_gradient
#' @importFrom tidyr pivot_longer
#' @importFrom tibble as_tibble
#' @importFrom rlang .data
#' @importFrom stats cor lm var coef
#'
#' @export
dm_plot_climate_compare <- function(x,
                                    climate_vars = NULL,
                                    numeric_vars = NULL,
                                    scale = c("auto", "daily", "cycle", "point"),
                                    type = c("timeseries", "boxplot", "violin", "both",
                                             "cor_heatmap", "regression_heatmap"),
                                    group_var = NULL,
                                    Year = NULL,
                                    DOY = NULL,
                                    box_scales = c("free_y", "fixed"),
                                    cor_method = c("pearson", "spearman", "kendall"),
                                    use_pairwise = TRUE,
                                    regression_stat = c("r.squared", "slope", "p.value"),
                                    point_alpha = 0.5,
                                    show_values = FALSE,
                                    temporal = NULL) {

  scale <- match.arg(scale)
  type <- match.arg(type)
  box_scales <- match.arg(box_scales)
  cor_method <- match.arg(cor_method)
  regression_stat <- match.arg(regression_stat)

  dat <- NULL
  time_col <- NULL
  default_group <- NULL
  clim_candidates <- NULL

  value <- NULL
  Var1 <- NULL
  Var2 <- NULL

  # -----------------------------------------------------
  # Choose data source
  # -----------------------------------------------------
  if (inherits(x, "daily_output_clim")) {

    dat <- tibble::as_tibble(x)
    time_col <- "DATE"
    default_group <- if ("Day_status" %in% names(dat)) "Day_status" else NULL

    clim_candidates <- setdiff(
      names(dat),
      c(
        "DATE", "Min", "Time_min", "Max", "Time_max", "mean", "median",
        "amplitude", "Time_min_h", "Time_max_h", "lag_h", "Remarks",
        "Max_diff", "Day_status"
      )
    )

  } else if (inherits(x, "ZG_output_clim") || inherits(x, "SC_output_clim")) {

    kind <- if (inherits(x, "ZG_output_clim") || inherits(x, "ZG_output")) {
      "ZG"
    } else {
      "SC"
    }

    cyc_name <- if (kind == "ZG") "ZG_cycle" else "SC_cycle"
    ph_name  <- if (kind == "ZG") "ZG_phase" else "SC_phase"

    cyc_dat <- tibble::as_tibble(x[[cyc_name]])
    ph_dat  <- tibble::as_tibble(x[[ph_name]])

    cyc_clim <- .dm_detect_cycle_climate_cols(cyc_dat, kind = kind)
    ph_clim  <- .dm_detect_point_climate_cols(ph_dat, kind = kind)

    if (scale == "auto") {
      if (!is.null(climate_vars)) {
        if (all(climate_vars %in% cyc_clim)) {
          scale <- "cycle"
        } else if (all(climate_vars %in% ph_clim)) {
          scale <- "point"
        } else {
          stop(
            "Selected climate_vars were not found consistently in ",
            "cycle-level or point-level data."
          )
        }
      } else if (type %in% c("cor_heatmap", "regression_heatmap")) {
        scale <- "cycle"
      } else if (length(cyc_clim) > 0) {
        scale <- "cycle"
      } else {
        scale <- "point"
      }
    }

    if (scale == "daily") {
      stop("For ZG/SC outputs, use scale = 'cycle' or scale = 'point'.")
    }

    if (scale == "cycle") {
      dat <- cyc_dat
      time_col <- "Start"
      clim_candidates <- cyc_clim
    } else {
      dat <- ph_dat
      time_col <- "TIME"
      clim_candidates <- ph_clim
    }

    default_group <- "Phases"

    if ("Phases" %in% names(dat)) {
      dat$Phases <- .dm_label_phase(dat$Phases, obj_class = kind)
    }

  } else {
    stop(
      "'x' must be a climate-augmented object returned by dm_add_climate(), ",
      "dm_join_daily_clim(), dm_join_phase_clim(), or dm_join_subdaily_clim()."
    )
  }

  if (is.null(group_var)) {
    group_var <- default_group
  }

  if (!is.null(group_var) && !group_var %in% names(dat)) {
    stop(sprintf("Grouping variable '%s' was not found in the selected data.", group_var))
  }

  dat <- .dm_subset_window(dat, time_col = time_col, Year = Year, DOY = DOY)

  # -----------------------------------------------------
  # Select climate variables for faceted plots
  # -----------------------------------------------------
  if (type %in% c("timeseries", "boxplot", "violin", "both")) {

    if (is.null(climate_vars)) {
      climate_vars <- clim_candidates
    }

    if (length(climate_vars) == 0) {
      stop("No climate variables were detected for plotting.")
    }

    miss <- setdiff(climate_vars, names(dat))
    if (length(miss) > 0) {
      stop(sprintf(
        "The following climate_vars were not found: %s",
        paste(miss, collapse = ", ")
      ))
    }

    pdat <- tidyr::pivot_longer(
      dat,
      cols = climate_vars,
      names_to = "Climate_var",
      values_to = "value"
    )

    pdat <- pdat[!is.na(pdat$value), , drop = FALSE]
  }

  # -----------------------------------------------------
  # Time series
  # -----------------------------------------------------
  if (type == "timeseries") {

    p <- ggplot2::ggplot(
      pdat,
      ggplot2::aes(x = .data[[time_col]], y = value)
    )

    if (!is.null(group_var)) {
      p <- p +
        ggplot2::geom_line(
          ggplot2::aes(color = .data[[group_var]]),
          linewidth = 0.45
        ) +
        ggplot2::geom_point(
          ggplot2::aes(color = .data[[group_var]]),
          size = 1.2,
          alpha = point_alpha
        ) +
        ggplot2::labs(color = group_var)
    } else {
      p <- p +
        ggplot2::geom_line(linewidth = 0.45) +
        ggplot2::geom_point(size = 1.2, alpha = point_alpha)
    }

    p <- p +
      ggplot2::facet_wrap(~ Climate_var, scales = "free_y", ncol = 1) +
      ggplot2::labs(x = time_col, y = "Value") +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text  = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }

  # -----------------------------------------------------
  # Boxplot / violin / both
  # -----------------------------------------------------
  if (type %in% c("boxplot", "violin", "both")) {

    if (is.null(group_var)) {
      stop("A grouping variable is required for grouped comparison plots.")
    }

    p <- ggplot2::ggplot(
      pdat,
      ggplot2::aes(x = .data[[group_var]], y = value)
    )

    if (type %in% c("violin", "both")) {
      p <- p +
        ggplot2::geom_violin(fill = "grey80", alpha = 0.8, trim = FALSE)
    }

    if (type %in% c("boxplot", "both")) {
      p <- p +
        ggplot2::geom_boxplot(
          width = if (type == "both") 0.18 else 0.6,
          outlier.shape = NA,
          fill = if (type == "both") "white" else "grey80"
        )
    }

    p <- p +
      ggplot2::geom_point(
        position = ggplot2::position_jitter(width = 0.12, height = 0),
        size = 1.1,
        alpha = point_alpha,
        color = "grey25"
      ) +
      ggplot2::facet_wrap(~ Climate_var, scales = box_scales) +
      ggplot2::labs(x = group_var, y = "Value") +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text  = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }

  # -----------------------------------------------------
  # Heatmap inputs
  # -----------------------------------------------------
  if (type %in% c("cor_heatmap", "regression_heatmap")) {

    if (is.null(numeric_vars)) {
      numeric_vars <- names(dat)[vapply(dat, is.numeric, logical(1))]
    }

    miss <- setdiff(numeric_vars, names(dat))
    if (length(miss) > 0) {
      stop(sprintf(
        "The following numeric_vars were not found: %s",
        paste(miss, collapse = ", ")
      ))
    }

    numeric_vars <- numeric_vars[vapply(dat[numeric_vars], is.numeric, logical(1))]

    if (length(numeric_vars) < 2) {
      stop("At least two numeric variables are required for heatmap plotting.")
    }

    mdat <- dat[, numeric_vars, drop = FALSE]
  }

  # -----------------------------------------------------
  # Correlation heatmap
  # -----------------------------------------------------
  if (type == "cor_heatmap") {

    cm <- stats::cor(
      mdat,
      use = if (isTRUE(use_pairwise)) "pairwise.complete.obs" else "complete.obs",
      method = cor_method
    )

    cdat <- as.data.frame(as.table(cm))
    names(cdat) <- c("Var1", "Var2", "value")

    p <- ggplot2::ggplot(
      cdat,
      ggplot2::aes(x = Var1, y = Var2, fill = value)
    ) +
      ggplot2::geom_tile() +
      ggplot2::scale_fill_gradient2(
        low = "firebrick",
        mid = "white",
        high = "steelblue",
        midpoint = 0,
        limits = c(-1, 1)
      ) +
      ggplot2::labs(
        x = NULL,
        y = NULL,
        fill = paste0("Correlation\n(", cor_method, ")")
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
        axis.title  = ggplot2::element_text(size = 14),
        axis.text   = ggplot2::element_text(size = 11)
      )

    if (isTRUE(show_values)) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = sprintf("%.2f", value)),
          size = 3
        )
    }

    print(p)
    return(invisible(p))
  }

  # -----------------------------------------------------
  # Regression heatmap
  # -----------------------------------------------------
  if (type == "regression_heatmap") {

    pair_reg_stat <- function(xv, yv, what = c("r.squared", "slope", "p.value")) {

      what <- match.arg(what)
      ok <- is.finite(xv) & is.finite(yv)

      if (sum(ok) < 3) {
        return(NA_real_)
      }

      if (stats::var(xv[ok]) == 0 || stats::var(yv[ok]) == 0) {
        return(NA_real_)
      }

      fit <- try(stats::lm(yv[ok] ~ xv[ok]), silent = TRUE)

      if (inherits(fit, "try-error")) {
        return(NA_real_)
      }

      sm <- summary(fit)

      if (what == "r.squared") {
        return(unname(sm$r.squared))
      }

      if (what == "slope") {
        return(unname(stats::coef(fit)[2]))
      }

      if (what == "p.value") {
        return(unname(sm$coefficients[2, 4]))
      }

      NA_real_
    }

    res <- expand.grid(
      Var1 = numeric_vars,
      Var2 = numeric_vars,
      stringsAsFactors = FALSE
    )

    res$value <- mapply(
      FUN = function(v1, v2) {
        pair_reg_stat(mdat[[v1]], mdat[[v2]], what = regression_stat)
      },
      v1 = res$Var1,
      v2 = res$Var2
    )

    p <- ggplot2::ggplot(
      res,
      ggplot2::aes(x = Var1, y = Var2, fill = value)
    ) +
      ggplot2::geom_tile()

    if (regression_stat == "r.squared") {
      p <- p +
        ggplot2::scale_fill_gradient(
          low = "white",
          high = "steelblue",
          limits = c(0, 1)
        )
    } else if (regression_stat == "p.value") {
      p <- p +
        ggplot2::scale_fill_gradient(
          low = "firebrick",
          high = "white"
        )
    } else {
      p <- p +
        ggplot2::scale_fill_gradient2(
          low = "firebrick",
          mid = "white",
          high = "steelblue",
          midpoint = 0
        )
    }

    p <- p +
      ggplot2::labs(x = NULL, y = NULL, fill = regression_stat) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5),
        axis.title  = ggplot2::element_text(size = 14),
        axis.text   = ggplot2::element_text(size = 11)
      )

    if (isTRUE(show_values)) {
      lab_fmt <- if (regression_stat == "p.value") "%.3g" else "%.2f"
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = sprintf(lab_fmt, value)),
          size = 3
        )
    }

    print(p)
    return(invisible(p))
  }
}


# =========================================================
# S3 plot methods
# =========================================================

.plot_climate_s3 <- function(x,
                             y = NULL,
                             ...,
                             climate_var = NULL,
                             climate_vars = NULL,
                             numeric_vars = NULL,
                             compare = FALSE,
                             temporal = NULL) {

  if (!is.null(y)) {
    if (isTRUE(compare)) {
      if (is.null(climate_vars)) {
        climate_vars <- y
      }
    } else {
      if (is.null(climate_var)) {
        climate_var <- y
      }
    }
  }

  if (!isTRUE(compare) && (!is.null(climate_vars) || !is.null(numeric_vars))) {
    compare <- TRUE
  }

  if (isTRUE(compare)) {
    return(
      dm_plot_climate_compare(
        x = x,
        climate_vars = climate_vars,
        numeric_vars = numeric_vars,
        ...
      )
    )
  }

  if (is.null(climate_var)) {
    stop(
      "Please provide 'climate_var', for example: ",
      "plot(x, climate_var = 'VPD_mean_phase'). ",
      "For multiple variables, use compare = TRUE or provide 'climate_vars'."
    )
  }

  dm_plot_climate(
    x = x,
    climate_var = climate_var,
    ...
  )
}


#' Plot climate-augmented daily dendrometer output
#'
#' @description
#' S3 plotting method for objects of class \code{daily_output_clim}. This allows
#' climate-augmented daily dendrometer outputs to be visualized directly with
#' the generic \code{plot()} function.
#'
#' @param x An object of class \code{daily_output_clim}.
#' @param y Optional climate variable name passed as the second argument.
#' @param ... Additional arguments passed to \code{dm_plot_climate()} or
#'   \code{dm_plot_climate_compare()}.
#' @param climate_var Character. Name of one climate variable to plot.
#' @param climate_vars Character vector of climate variables for comparison
#'   plots.
#' @param numeric_vars Character vector of numeric variables used for correlation
#'   or regression heatmaps.
#' @param compare Logical. If \code{TRUE}, \code{dm_plot_climate_compare()} is
#'   called.
#' @param temporal Deprecated argument kept for compatibility with older
#'   examples. It is ignored.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' # plot(daily_clim_output, climate_var = "temp_mean", type = "timeseries")
#' }
#'
#' @method plot daily_output_clim
#' @export
plot.daily_output_clim <- function(x,
                                   y = NULL,
                                   ...,
                                   climate_var = NULL,
                                   climate_vars = NULL,
                                   numeric_vars = NULL,
                                   compare = FALSE,
                                   temporal = NULL) {

  .plot_climate_s3(
    x = x,
    y = y,
    ...,
    climate_var = climate_var,
    climate_vars = climate_vars,
    numeric_vars = numeric_vars,
    compare = compare,
    temporal = temporal
  )
}


#' Plot climate-augmented zero-growth output
#'
#' @description
#' S3 plotting method for objects of class \code{ZG_output_clim}. This allows
#' climate-augmented zero-growth outputs to be visualized directly with the
#' generic \code{plot()} function.
#'
#' @param x An object of class \code{ZG_output_clim}.
#' @param y Optional climate variable name passed as the second argument.
#' @param ... Additional arguments passed to \code{dm_plot_climate()} or
#'   \code{dm_plot_climate_compare()}.
#' @param climate_var Character. Name of one climate variable to plot.
#' @param climate_vars Character vector of climate variables for comparison
#'   plots.
#' @param numeric_vars Character vector of numeric variables used for correlation
#'   or regression heatmaps.
#' @param compare Logical. If \code{TRUE}, \code{dm_plot_climate_compare()} is
#'   called.
#' @param temporal Deprecated argument kept for compatibility with older
#'   examples. It is ignored.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' # plot(zg_clim, climate_var = "VPD_mean_phase", scale = "cycle",
#' #      type = "boxplot")
#' }
#'
#' @method plot ZG_output_clim
#' @export
plot.ZG_output_clim <- function(x,
                                y = NULL,
                                ...,
                                climate_var = NULL,
                                climate_vars = NULL,
                                numeric_vars = NULL,
                                compare = FALSE,
                                temporal = NULL) {

  .plot_climate_s3(
    x = x,
    y = y,
    ...,
    climate_var = climate_var,
    climate_vars = climate_vars,
    numeric_vars = numeric_vars,
    compare = compare,
    temporal = temporal
  )
}


#' Plot climate-augmented stem-cycle output
#'
#' @description
#' S3 plotting method for objects of class \code{SC_output_clim}. This allows
#' climate-augmented stem-cycle outputs to be visualized directly with the
#' generic \code{plot()} function.
#'
#' @param x An object of class \code{SC_output_clim}.
#' @param y Optional climate variable name passed as the second argument.
#' @param ... Additional arguments passed to \code{dm_plot_climate()} or
#'   \code{dm_plot_climate_compare()}.
#' @param climate_var Character. Name of one climate variable to plot.
#' @param climate_vars Character vector of climate variables for comparison
#'   plots.
#' @param numeric_vars Character vector of numeric variables used for correlation
#'   or regression heatmaps.
#' @param compare Logical. If \code{TRUE}, \code{dm_plot_climate_compare()} is
#'   called.
#' @param temporal Deprecated argument kept for compatibility with older
#'   examples. It is ignored.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' # plot(sc_clim, climate_var = "temp_mean_phase", scale = "cycle",
#' #      type = "violin")
#' }
#'
#' @method plot SC_output_clim
#' @export
plot.SC_output_clim <- function(x,
                                y = NULL,
                                ...,
                                climate_var = NULL,
                                climate_vars = NULL,
                                numeric_vars = NULL,
                                compare = FALSE,
                                temporal = NULL) {

  .plot_climate_s3(
    x = x,
    y = y,
    ...,
    climate_var = climate_var,
    climate_vars = climate_vars,
    numeric_vars = numeric_vars,
    compare = compare,
    temporal = temporal
  )
}

#' @title Resampling temporal resolution of dendrometer and climate data
#'
#' @description
#' This function changes the temporal resolution of dendrometer and climate
#' time series. It supports both aggregation to coarser resolutions and
#' interpolation to finer resolutions. The target resolution can be defined in
#' minutes, hours, days, weeks, or month-end frequency. Aggregation can be
#' performed using \code{mean}, \code{max}, \code{min}, \code{sum}, or
#' \code{count} (number of non-missing values). When a finer temporal
#' resolution than the original data is requested, the function can interpolate
#' the series to the new time step using linear interpolation.
#'
#' @param df A dataframe with the first column containing date-time values in
#'   the format \code{yyyy-mm-dd HH:MM:SS}.
#'
#' @param by A character string defining the target temporal resolution.
#'   Supported values are:
#'   \itemize{
#'     \item \code{"M"} for minutes
#'     \item \code{"H"} for hours
#'     \item \code{"D"} for days
#'     \item \code{"W"} for weeks
#'     \item \code{"ME"} for month-end
#'     \item values such as \code{"10M"}, \code{"30M"}, \code{"5H"},
#'     \code{"2D"}, or \code{"3W"} for custom multiples
#'   }
#'
#' @param value A character string defining the aggregation function.
#'   Supported values are \code{"mean"}, \code{"max"}, \code{"min"},
#'   \code{"sum"}, and \code{"count"}.
#'   The \code{"count"} option returns the number of non-\code{NA} values
#'   within each aggregation interval. It is only valid when
#'   \code{method = "aggregate"}.
#'
#' @param method A character string defining how resampling should be performed.
#'   Options are:
#'   \itemize{
#'     \item \code{"auto"}: automatically aggregates when the requested
#'     resolution is coarser than the input data, and interpolates when the
#'     requested resolution is finer
#'     \item \code{"aggregate"}: always aggregates values within each target
#'     interval
#'     \item \code{"interpolate"}: always interpolates to the requested regular
#'     time grid
#'   }
#'
#' @return A dataframe with resampled data. The first column contains the new
#'   time stamps and the remaining columns contain the resampled variables.
#'
#' @examples
#' \donttest{
#' library(dendRoAnalyst)
#' data(nepa17)
#'
#' # Monthly resampling using maximum values
#' resample_ME <- dendro.resample(df = gf_nepa17, by = "ME", value = "max")
#' head(resample_ME, 10)
#'
#' # Daily resampling using mean values
#' resample_D <- dendro.resample(df = gf_nepa17, by = "D", value = "mean")
#'
#' # Count non-missing values per day
#' resample_count <- dendro.resample(df = gf_nepa17, by = "D", value = "count",
#'                                   method = "aggregate")
#'
#' # Interpolate to 30-minute resolution
#' resample_30M <- dendro.resample(df = gf_nepa17, by = "30M", value = "mean",
#'                                 method = "auto")
#' }
#'
#' @importFrom stats approx median na.exclude na.omit sd
#' @importFrom lubridate ymd_hms ymd floor_date minutes seconds minute second
#' @importFrom dplyr mutate group_by summarise ungroup %>% rename across select where case_when everything all_of
#' @importFrom tibble as_tibble tibble
#' @importFrom ggplot2 ggplot geom_area geom_rect geom_text aes theme_minimal labs geom_line geom_point facet_wrap theme element_text
#' @importFrom tidyr pivot_longer
#'
#' @export

dendro.resample <- function(df, by, value = "mean",
                            method = c("auto", "aggregate", "interpolate")) {
  TIME <-period <- NULL
  method <- match.arg(method)

  if (!requireNamespace("dplyr", quietly = TRUE) ||
      !requireNamespace("tibble", quietly = TRUE) ||
      !requireNamespace("lubridate", quietly = TRUE)) {
    stop("Packages 'dplyr', 'tibble', and 'lubridate' are required.")
  }

  # -----------------------------
  # Parse target frequency
  # M = minutes, H = hours, D = days, W = weeks, ME = month-end
  # Examples: M, 10M, 30M, H, 5H, D, 2D, W, 3W, ME
  # -----------------------------
  parse_by <- function(by) {
    by <- toupper(trimws(by))

    if (by == "ME") {
      return(list(
        unit = "month",
        n = 1L,
        label = "ME",
        seconds = NA_real_
      ))
    }

    m <- regexec("^([0-9]*)(M|H|D|W)$", by)
    parts <- regmatches(by, m)[[1]]

    if (length(parts) != 3) {
      stop("Invalid 'by'. Use 'M', 'H', 'D', 'W', 'ME', or values like '10M', '30M', '5H', '2D', '3W'.")
    }

    n <- ifelse(parts[2] == "", 1L, as.integer(parts[2]))
    unit_code <- parts[3]

    unit <- switch(
      unit_code,
      "M" = "minute",
      "H" = "hour",
      "D" = "day",
      "W" = "week"
    )

    seconds <- switch(
      unit_code,
      "M" = n * 60,
      "H" = n * 3600,
      "D" = n * 86400,
      "W" = n * 7 * 86400
    )

    list(
      unit = unit,
      n = n,
      label = by,
      seconds = seconds
    )
  }

  # -----------------------------
  # Infer source temporal resolution
  # -----------------------------
  infer_resolution_seconds <- function(time_vec) {
    time_vec <- sort(time_vec)
    d <- diff(time_vec)
    d_sec <- as.numeric(d, units = "secs")
    d_sec <- d_sec[is.finite(d_sec) & d_sec > 0]

    if (length(d_sec) == 0) return(NA_real_)
    stats::median(d_sec)
  }

  # -----------------------------
  # Floor timestamps to custom bins
  # -----------------------------
  floor_to_custom <- function(x, step_seconds) {
    tzx <- attr(x, "tzone")
    secs <- as.numeric(x)
    floored <- floor(secs / step_seconds) * step_seconds
    as.POSIXct(floored, origin = "1970-01-01", tz = tzx)
  }

  # -----------------------------
  # Build interpolation grid using numeric seconds
  # -----------------------------
  make_interp_grid <- function(time_vec, target_info) {
    if (is.na(target_info$seconds)) {
      stop("Interpolation is not supported for 'ME'. Use method = 'aggregate'.")
    }

    tzx <- attr(time_vec, "tzone")
    x_min <- min(as.numeric(time_vec), na.rm = TRUE)
    x_max <- max(as.numeric(time_vec), na.rm = TRUE)

    xout_num <- seq(from = x_min, to = x_max, by = target_info$seconds)
    as.POSIXct(xout_num, origin = "1970-01-01", tz = tzx)
  }

  # -----------------------------
  # Interpolate numeric columns
  # -----------------------------
  interpolate_to_grid <- function(df, target_info) {
    xout <- make_interp_grid(df$TIME, target_info)
    out <- tibble::tibble(TIME = xout)

    value_cols <- names(df)[vapply(df, is.numeric, logical(1))]
    value_cols <- setdiff(value_cols, "TIME")

    for (col in value_cols) {
      y <- df[[col]]
      good <- !is.na(df$TIME) & !is.na(y)

      if (sum(good) < 2) {
        out[[col]] <- NA_real_
      } else {
        interp <- stats::approx(
          x = as.numeric(df$TIME[good]),
          y = y[good],
          xout = as.numeric(xout),
          method = "linear",
          rule = 1
        )
        out[[col]] <- interp$y
      }
    }

    out
  }

  # -----------------------------
  # Aggregate helper
  # -----------------------------
  aggregate_data <- function(df, target_info, value, by_original) {
    allowed_funs <- c("mean", "max", "min", "sum", "count")
    if (!value %in% allowed_funs) {
      stop("value must be one of: 'mean', 'max', 'min', 'sum', 'count'")
    }

    summarization_fun <- switch(
      value,
      mean  = function(x) if (all(is.na(x))) NA_real_ else mean(x, na.rm = TRUE),
      max   = function(x) if (all(is.na(x))) NA_real_ else max(x, na.rm = TRUE),
      min   = function(x) if (all(is.na(x))) NA_real_ else min(x, na.rm = TRUE),
      sum   = function(x) if (all(is.na(x))) NA_real_ else sum(x, na.rm = TRUE),
      count = function(x) sum(!is.na(x))
    )

    value_cols <- names(df)[vapply(df, is.numeric, logical(1))]
    value_cols <- setdiff(value_cols, "TIME")

    if (length(value_cols) == 0) {
      stop("No numeric columns found to resample.")
    }

    if (target_info$label == "ME") {
      res <- df %>%
        dplyr::mutate(period = lubridate::ceiling_date(TIME, "month") - lubridate::days(1)) %>%
        dplyr::group_by(period) %>%
        dplyr::summarise(
          dplyr::across(dplyr::all_of(value_cols), summarization_fun),
          .groups = "drop"
        ) %>%
        dplyr::rename(TIME = period)

      return(res)
    }

    period_col <- floor_to_custom(df$TIME, step_seconds = target_info$seconds)

    res <- df %>%
      dplyr::mutate(period = period_col) %>%
      dplyr::group_by(period) %>%
      dplyr::summarise(
        dplyr::across(dplyr::all_of(value_cols), summarization_fun),
        .groups = "drop"
      ) %>%
      dplyr::rename(TIME = period)

    if (toupper(trimws(by_original)) %in% c("D", "1D")) {
      res$TIME <- as.Date(res$TIME)
    }

    res
  }

  # -----------------------------
  # Convert first column to datetime
  # -----------------------------
  if (!inherits(df[[1]], "POSIXt") && !inherits(df[[1]], "Date")) {
    parsed_time <- suppressWarnings(
      lubridate::parse_date_time(
        df[[1]],
        orders = c(
          "ymd HMS", "ymd HM", "ymd",
          "dmy HMS", "dmy HM", "dmy",
          "mdy HMS", "mdy HM", "mdy"
        ),
        quiet = TRUE
      )
    )
    df[[1]] <- parsed_time
  }

  if (inherits(df[[1]], "Date")) {
    df[[1]] <- as.POSIXct(df[[1]])
  }

  df <- tibble::as_tibble(df) %>%
    dplyr::rename(TIME = 1) %>%
    dplyr::arrange(TIME)

  if (any(is.na(df$TIME))) {
    stop("Some timestamps could not be parsed.")
  }

  target_info <- parse_by(by)
  source_sec <- infer_resolution_seconds(df$TIME)

  # -----------------------------
  # Decide operation
  # -----------------------------
  op <- method

  if (method == "auto") {
    if (target_info$label == "ME") {
      op <- "aggregate"
    } else if (is.na(source_sec)) {
      op <- "aggregate"
    } else if (target_info$seconds < source_sec) {
      op <- "interpolate"
    } else {
      op <- "aggregate"
    }
  }

  # -----------------------------
  # Run operation
  # -----------------------------
  if (op == "interpolate") {
    if (value == "count") {
      stop("'count' is not valid with method = 'interpolate'.")
    }
    return(interpolate_to_grid(df, target_info))
  }

  if (op == "aggregate") {
    return(aggregate_data(df, target_info, value, by))
  }

  stop("Unknown method.")
}

#' @title Standardize dendrometer series within seasonal years
#'
#' @description
#' Standardizes one or more dendrometer series within seasonal years so that
#' multiple trees can be brought to a comparable scale while preserving their
#' within-season temporal pattern.
#'
#' Seasonal years are defined as:
#' \itemize{
#'   \item \code{"NH"}: Northern Hemisphere year, from \strong{01 Jan} to \strong{31 Dec}
#'   \item \code{"SH"}: Southern Hemisphere year, from \strong{01 Jul} to \strong{30 Jun} of the next year
#'   \item \code{"CS"}: Custom season defined by \code{CS_doys = c(doy1, doy2)}
#' }
#'
#' For \code{season_type = "CS"}:
#' \itemize{
#'   \item if \code{CS_doys[1] <= CS_doys[2]}, the season stays within one calendar year
#'   \item if \code{CS_doys[1] > CS_doys[2]}, the season wraps across years
#' }
#'
#' Standardization is applied separately for each dendrometer series and each
#' seasonal year.
#'
#' @param df A data frame whose first column contains date-time
#'   (\code{yyyy-mm-dd HH:MM:SS}, \code{POSIXct}, or \code{Date}) and all
#'   remaining columns are dendrometer series.
#' @param season_type One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.
#' @param CS_doys Optional numeric vector of length 2 defining the start and end
#'   DOY for \code{season_type = "CS"}.
#' @param method Standardization method. One of:
#'   \itemize{
#'     \item \code{"center"}: subtract seasonal reference value
#'     \item \code{"amplitude"}: subtract reference and divide by seasonal range
#'     \item \code{"robust_amplitude"}: subtract reference and divide by
#'           \code{Q_high - Q_low}
#'     \item \code{"minmax"}: rescale to \code{(x - min)/(max - min)}
#'     \item \code{"zscore"}: seasonal z-score
#'     \item \code{"robust_zscore"}: robust seasonal z-score using median and MAD
#'     \item \code{"percentile"}: convert seasonal values to percentiles in \eqn{[0,1]}
#'   }
#' @param ref_type Reference-value definition for methods that need a reference
#'   (\code{"center"}, \code{"amplitude"}, \code{"robust_amplitude"}). One of:
#'   \itemize{
#'     \item \code{"first_value"}: first non-missing value in the seasonal year
#'     \item \code{"first_n_days"}: mean of the first \code{ref_n_days} unique days
#'     \item \code{"ref_window"}: mean of values within \code{ref_doys}
#'   }
#' @param ref_n_days Number of initial days used when \code{ref_type = "first_n_days"}.
#' @param ref_doys Optional numeric vector of length 2 defining a DOY reference
#'   window when \code{ref_type = "ref_window"}.
#' @param q_low Lower quantile used by \code{method = "robust_amplitude"}.
#' @param q_high Upper quantile used by \code{method = "robust_amplitude"}.
#'
#' @return
#' A list of class \code{"dm_standardized"} containing:
#' \itemize{
#'   \item \code{data}: wide data frame with columns
#'     \code{TIME}, \code{season_year}, \code{in_season}, and standardized dendrometer series
#'   \item \code{parameters}: tibble with one row per tree and seasonal year,
#'     summarizing the reference value and scaling denominator used
#'   \item \code{metadata}: method and seasonal-year settings
#' }
#'
#' For \code{season_type = "CS"}, observations outside the custom season are
#' retained in \code{data}, but their standardized values are set to \code{NA}
#' and \code{in_season = FALSE}.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#'
#' # Northern Hemisphere seasonal-year standardization
#' out_nh <- dm_standardize(
#'   df = gf_nepa17,
#'   season_type = "NH",
#'   method = "robust_amplitude"
#' )
#'
#' # Southern Hemisphere seasonal-year standardization
#' out_sh <- dm_standardize(
#'   df = gf_nepa17,
#'   season_type = "SH",
#'   method = "center"
#' )
#'
#' # Custom season within one year
#' out_cs1 <- dm_standardize(
#'   df = gf_nepa17,
#'   season_type = "CS",
#'   CS_doys = c(100, 280),
#'   method = "robust_amplitude"
#' )
#'
#' # Custom season wrapping across years
#' out_cs2 <- dm_standardize(
#'   df = gf_nepa17,
#'   season_type = "CS",
#'   CS_doys = c(250, 120),
#'   method = "percentile"
#' )
#'
#' head(out_nh$data)
#' head(out_nh$parameters)
#' }
#'
#' @importFrom lubridate ymd_hms ymd year yday month
#' @importFrom dplyr bind_rows
#' @importFrom tibble tibble as_tibble
#' @importFrom stats quantile median mad
#' @export
dm_standardize <- function(df,
                           season_type = c("NH", "SH", "CS"),
                           CS_doys = NULL,
                           method = c("center", "amplitude", "robust_amplitude",
                                      "minmax", "zscore", "robust_zscore", "percentile"),
                           ref_type = c("first_value", "first_n_days", "ref_window"),
                           ref_n_days = 7,
                           ref_doys = NULL,
                           q_low = 0.05,
                           q_high = 0.95) {

  season_type <- match.arg(season_type)
  method <- match.arg(method)
  ref_type <- match.arg(ref_type)

  # ----------------------------
  # checks
  # ----------------------------
  if (!is.data.frame(df) || ncol(df) < 2) {
    stop("'df' must be a data frame with a time column and at least one dendrometer series.")
  }

  if (season_type == "CS") {
    if (is.null(CS_doys) || length(CS_doys) != 2 || any(is.na(CS_doys))) {
      stop("For season_type = 'CS', 'CS_doys' must be a numeric vector of length 2.")
    }
    if (any(CS_doys < 1 | CS_doys > 366)) {
      stop("'CS_doys' values must be between 1 and 366.")
    }
    CS_doys <- as.integer(CS_doys)
  } else {
    if (!is.null(CS_doys)) {
      warning("'CS_doys' is ignored unless season_type = 'CS'.")
    }
  }

  if (!is.numeric(ref_n_days) || length(ref_n_days) != 1 || is.na(ref_n_days) || ref_n_days < 1) {
    stop("'ref_n_days' must be a positive number.")
  }
  ref_n_days <- as.integer(ref_n_days)

  if (ref_type == "ref_window") {
    if (is.null(ref_doys) || length(ref_doys) != 2 || any(is.na(ref_doys))) {
      stop("For ref_type = 'ref_window', 'ref_doys' must be a numeric vector of length 2.")
    }
    if (any(ref_doys < 1 | ref_doys > 366)) {
      stop("'ref_doys' values must be between 1 and 366.")
    }
    ref_doys <- as.integer(ref_doys)
  } else {
    if (!is.null(ref_doys)) {
      warning("'ref_doys' is ignored unless ref_type = 'ref_window'.")
    }
  }

  if (!is.numeric(q_low) || !is.numeric(q_high) || length(q_low) != 1 || length(q_high) != 1 ||
      is.na(q_low) || is.na(q_high) || q_low < 0 || q_high > 1 || q_low >= q_high) {
    stop("'q_low' and 'q_high' must satisfy 0 <= q_low < q_high <= 1.")
  }

  # ----------------------------
  # parse time
  # ----------------------------
  parse_time <- function(x) {
    if (inherits(x, "POSIXct")) return(as.POSIXct(x))
    if (inherits(x, "Date")) return(as.POSIXct(x))
    out <- suppressWarnings(lubridate::ymd_hms(x, quiet = TRUE))
    if (all(is.na(out))) {
      out_date <- suppressWarnings(lubridate::ymd(x, quiet = TRUE))
      if (!all(is.na(out_date))) out <- as.POSIXct(out_date)
    }
    if (all(is.na(out))) out <- suppressWarnings(as.POSIXct(x))
    out
  }

  TIME <- parse_time(df[[1]])
  if (all(is.na(TIME))) {
    stop("Could not parse the first column of 'df' as date-time.")
  }

  dat <- tibble::as_tibble(df)
  names(dat)[1] <- "TIME"
  dat$TIME <- TIME
  dat <- dat[order(dat$TIME), , drop = FALSE]

  tree_names <- names(dat)[-1]

  # ----------------------------
  # helper functions
  # ----------------------------
  in_doy_window <- function(doy, doy_window) {
    start <- doy_window[1]
    end <- doy_window[2]
    if (start <= end) {
      doy >= start & doy <= end
    } else {
      doy >= start | doy <= end
    }
  }

  assign_season_year <- function(time, season_type, CS_doys = NULL) {
    yy <- lubridate::year(time)
    dd <- lubridate::yday(time)
    mm <- lubridate::month(time)

    if (season_type == "NH") {
      return(list(
        season_year = yy,
        in_season = rep(TRUE, length(time))
      ))
    }

    if (season_type == "SH") {
      sy <- ifelse(mm >= 7, yy, yy - 1L)
      return(list(
        season_year = sy,
        in_season = rep(TRUE, length(time))
      ))
    }

    # custom season
    d1 <- CS_doys[1]
    d2 <- CS_doys[2]

    if (d1 <= d2) {
      in_season <- dd >= d1 & dd <= d2
      sy <- yy
    } else {
      in_season <- dd >= d1 | dd <= d2
      sy <- ifelse(dd >= d1, yy, yy - 1L)
    }

    list(
      season_year = sy,
      in_season = in_season
    )
  }

  get_reference_value <- function(vals, time, doy, ref_type, ref_n_days, ref_doys) {
    ok <- !is.na(vals)
    if (!any(ok)) return(NA_real_)

    if (ref_type == "first_value") {
      return(vals[which(ok)[1]])
    }

    if (ref_type == "first_n_days") {
      dts <- as.Date(time)
      unique_days <- sort(unique(dts[ok]))
      if (length(unique_days) == 0) return(NA_real_)
      keep_days <- unique_days[seq_len(min(ref_n_days, length(unique_days)))]
      return(mean(vals[dts %in% keep_days], na.rm = TRUE))
    }

    # ref_window
    keep <- in_doy_window(doy, ref_doys) & ok
    if (!any(keep)) return(NA_real_)
    mean(vals[keep], na.rm = TRUE)
  }

  safe_divide <- function(num, denom) {
    out <- rep(NA_real_, length(num))
    ok <- !is.na(num)
    if (!is.finite(denom) || is.na(denom)) return(out)
    if (denom == 0) {
      out[ok] <- 0
      return(out)
    }
    out[ok] <- num[ok] / denom
    out
  }

  percentile_scale <- function(x) {
    out <- rep(NA_real_, length(x))
    ok <- !is.na(x)
    n <- sum(ok)
    if (n == 0) return(out)
    if (n == 1) {
      out[ok] <- 0.5
      return(out)
    }
    r <- rank(x[ok], ties.method = "average")
    out[ok] <- (r - 1) / (n - 1)
    out
  }

  # ----------------------------
  # season-year assignment
  # ----------------------------
  season_info <- assign_season_year(dat$TIME, season_type = season_type, CS_doys = CS_doys)
  dat$season_year <- season_info$season_year
  dat$in_season <- season_info$in_season
  dat$DOY <- lubridate::yday(dat$TIME)

  # prepare output
  out_data <- tibble::tibble(
    TIME = dat$TIME,
    season_year = dat$season_year,
    in_season = dat$in_season
  )

  param_list <- list()

  # ----------------------------
  # standardize each tree by season-year
  # ----------------------------
  for (tr in tree_names) {
    x <- dat[[tr]]
    std_x <- rep(NA_real_, length(x))

    season_levels <- sort(unique(dat$season_year[dat$in_season]))

    for (sy in season_levels) {
      idx <- which(dat$season_year == sy & dat$in_season)

      vals <- x[idx]
      tsub <- dat$TIME[idx]
      dsub <- dat$DOY[idx]

      n_total <- length(vals)
      n_nonmiss <- sum(!is.na(vals))

      ref_val <- NA_real_
      denom <- NA_real_
      center_val <- NA_real_

      if (n_nonmiss > 0) {
        if (method %in% c("center", "amplitude", "robust_amplitude")) {
          ref_val <- get_reference_value(
            vals = vals,
            time = tsub,
            doy = dsub,
            ref_type = ref_type,
            ref_n_days = ref_n_days,
            ref_doys = ref_doys
          )
          center_val <- ref_val
        }

        if (method == "center") {
          std_x[idx] <- vals - ref_val
          denom <- NA_real_

        } else if (method == "amplitude") {
          denom <- max(vals, na.rm = TRUE) - min(vals, na.rm = TRUE)
          std_x[idx] <- safe_divide(vals - ref_val, denom)

        } else if (method == "robust_amplitude") {
          qq <- stats::quantile(vals, probs = c(q_low, q_high), na.rm = TRUE, names = FALSE)
          denom <- qq[2] - qq[1]
          std_x[idx] <- safe_divide(vals - ref_val, denom)

        } else if (method == "minmax") {
          vmin <- min(vals, na.rm = TRUE)
          vmax <- max(vals, na.rm = TRUE)
          denom <- vmax - vmin
          std_x[idx] <- safe_divide(vals - vmin, denom)
          center_val <- vmin

        } else if (method == "zscore") {
          mu <- mean(vals, na.rm = TRUE)
          denom <- stats::sd(vals, na.rm = TRUE)
          center_val <- mu
          std_x[idx] <- safe_divide(vals - mu, denom)

        } else if (method == "robust_zscore") {
          med <- stats::median(vals, na.rm = TRUE)
          denom <- stats::mad(vals, center = med, constant = 1.4826, na.rm = TRUE)
          center_val <- med
          std_x[idx] <- safe_divide(vals - med, denom)

        } else if (method == "percentile") {
          std_x[idx] <- percentile_scale(vals)
          center_val <- NA_real_
          denom <- NA_real_
        }
      }

      param_list[[length(param_list) + 1L]] <- tibble::tibble(
        tree = tr,
        season_year = sy,
        season_type = season_type,
        method = method,
        ref_type = if (method %in% c("center", "amplitude", "robust_amplitude")) ref_type else NA_character_,
        ref_n_days = if (method %in% c("center", "amplitude", "robust_amplitude") &&
                         ref_type == "first_n_days") ref_n_days else NA_integer_,
        ref_doys_start = if (method %in% c("center", "amplitude", "robust_amplitude") &&
                             ref_type == "ref_window") ref_doys[1] else NA_integer_,
        ref_doys_end = if (method %in% c("center", "amplitude", "robust_amplitude") &&
                           ref_type == "ref_window") ref_doys[2] else NA_integer_,
        q_low = if (method == "robust_amplitude") q_low else NA_real_,
        q_high = if (method == "robust_amplitude") q_high else NA_real_,
        n_total = n_total,
        n_nonmiss = n_nonmiss,
        season_start = min(tsub),
        season_end = max(tsub),
        reference_value = ref_val,
        center_value = center_val,
        denominator = denom
      )
    }

    out_data[[tr]] <- std_x
  }

  out <- list(
    data = out_data,
    parameters = dplyr::bind_rows(param_list),
    metadata = list(
      season_type = season_type,
      CS_doys = if (season_type == "CS") CS_doys else NULL,
      method = method,
      ref_type = ref_type,
      ref_n_days = ref_n_days,
      ref_doys = ref_doys,
      q_low = q_low,
      q_high = q_high,
      trees = tree_names,
      time_range = range(dat$TIME, na.rm = TRUE)
    )
  )

  class(out) <- "dm_standardized"
  out
}

#' @title Plot method for standardized dendrometer output
#'
#' @description
#' S3 plotting method for objects returned by \code{dm_standardize()}.
#'
#' @param x Object of class \code{"dm_standardized"} returned by
#'   \code{dm_standardize()}.
#' @param y Unused.
#' @param trees Character vector of dendrometer series to plot, or \code{"all"}
#'   for all series.
#' @param type Plot type. One of:
#'   \itemize{
#'     \item \code{"series"}: standardized time series over calendar time
#'     \item \code{"seasonal"}: standardized seasonal trajectories by seasonal day
#'     \item \code{"heatmap"}: heatmap of standardized values by seasonal day and seasonal year
#'     \item \code{"boxplot"}: distribution of standardized values
#'   }
#' @param x_axis For \code{type = "series"}, x-axis style:
#'   \code{"time"}, \code{"doy"}, or \code{"season_doy"}.
#' @param facet_by Faceting option. One of \code{"tree"}, \code{"season_year"},
#'   or \code{"none"}.
#' @param legend_by Legend grouping. One of \code{"tree"}, \code{"season_year"},
#'   or \code{"none"}.
#' @param in_season_only Logical. If \code{TRUE}, only observations inside the
#'   defined season are plotted.
#' @param box_group For \code{type = "boxplot"}, grouping variable on the x-axis:
#'   \code{"tree"} or \code{"season_year"}.
#' @param alpha Line or point transparency.
#' @param line_size Line width.
#' @param point_size Point size.
#' @param ... Unused.
#'
#' @return A \code{ggplot2} object, returned invisibly.
#'
#' @examples
#' \donttest{
#' data(gf_nepa17)
#'
#' out_std <- dm_standardize(
#'   df = gf_nepa17,
#'   season_type = "NH",
#'   method = "robust_amplitude"
#' )
#'
#' plot(out_std)
#' plot(out_std, type = "seasonal")
#' plot(out_std, type = "seasonal", facet_by = "tree", legend_by = "season_year")
#' plot(out_std, type = "seasonal", facet_by = "season_year", legend_by = "tree")
#' plot(out_std, type = "heatmap", facet_by = "tree")
#' plot(out_std, type = "boxplot", facet_by = "tree", legend_by = "season_year")
#' }
#'
#' @method plot dm_standardized
#' @importFrom dplyr mutate filter %>%
#' @importFrom tidyr pivot_longer
#' @importFrom tibble as_tibble
#' @importFrom lubridate yday
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_tile geom_boxplot
#'   facet_wrap theme_bw theme element_text labs scale_x_date
#' @export
plot.dm_standardized <- function(x,
                                 y = NULL,
                                 trees = "all",
                                 type = c("series", "seasonal", "heatmap", "boxplot"),
                                 x_axis = c("time", "doy", "season_doy"),
                                 facet_by = c("tree", "season_year", "none"),
                                 legend_by = c("season_year", "tree", "none"),
                                 in_season_only = TRUE,
                                 box_group = c("tree", "season_year"),
                                 alpha = 0.8,
                                 line_size = 0.45,
                                 point_size = 1.6,
                                 ...) {

  TIME <- value <- tree <- season_year <- season_doy <- DOY <- X <- group_var <- legend_var <- in_season <- NULL

  if (!inherits(x, "dm_standardized")) {
    stop("Input must be an object of class 'dm_standardized'.")
  }

  type <- match.arg(type)
  x_axis <- match.arg(x_axis)
  facet_by <- match.arg(facet_by)
  legend_by <- match.arg(legend_by)
  box_group <- match.arg(box_group)

  if (!is.list(x) || is.null(x$data) || !is.data.frame(x$data)) {
    stop("The object does not contain a valid 'data' element.")
  }

  dat <- tibble::as_tibble(x$data)

  tree_names <- setdiff(names(dat), c("TIME", "season_year", "in_season"))
  if (length(tree_names) == 0) {
    stop("No dendrometer series found in 'x$data'.")
  }

  if (length(trees) == 1 && identical(trees, "all")) {
    keep_trees <- tree_names
  } else {
    if (!is.character(trees)) {
      stop("'trees' must be 'all' or a character vector of dendrometer column names.")
    }
    miss <- setdiff(trees, tree_names)
    if (length(miss) > 0) {
      stop(sprintf(
        "The following trees are not available in the standardized object: %s",
        paste(miss, collapse = ", ")
      ))
    }
    keep_trees <- trees
  }

  dat_long <- tidyr::pivot_longer(
    dat,
    cols = all_of(keep_trees),
    names_to = "tree",
    values_to = "value"
  )

  if (isTRUE(in_season_only)) {
    dat_long <- dat_long %>% dplyr::filter(in_season %in% TRUE)
  }

  if (nrow(dat_long) == 0) {
    stop("No observations available for plotting after filtering.")
  }

  make_season_start <- function(season_year, season_type, CS_doys = NULL) {
    season_year <- as.integer(season_year)

    if (season_type == "NH") {
      return(as.Date(paste0(season_year, "-01-01")))
    }

    if (season_type == "SH") {
      return(as.Date(paste0(season_year, "-07-01")))
    }

    as.Date(paste0(season_year, "-01-01")) + (CS_doys[1] - 1L)
  }

  season_type <- x$metadata$season_type
  CS_doys <- x$metadata$CS_doys

  dat_long$DOY <- lubridate::yday(dat_long$TIME)
  season_start <- make_season_start(
    season_year = dat_long$season_year,
    season_type = season_type,
    CS_doys = CS_doys
  )
  dat_long$season_doy <- as.integer(as.Date(dat_long$TIME) - season_start) + 1L

  if (x_axis == "time") {
    dat_long$X <- as.Date(dat_long$TIME)
    x_lab <- "Time"
  } else if (x_axis == "doy") {
    dat_long$X <- dat_long$DOY
    x_lab <- "Day of year"
  } else {
    dat_long$X <- dat_long$season_doy
    x_lab <- "Seasonal day"
  }

  dat_long$season_year <- as.factor(dat_long$season_year)

  if (legend_by == "tree") {
    dat_long$legend_var <- dat_long$tree
    legend_title <- "Tree"
  } else if (legend_by == "season_year") {
    dat_long$legend_var <- dat_long$season_year
    legend_title <- "Season year"
  } else {
    dat_long$legend_var <- "All"
    legend_title <- NULL
  }

  title_base <- paste0(
    "Standardized dendrometer data (",
    x$metadata$method, ", ",
    x$metadata$season_type, ")"
  )

  if (type == "series") {
    p <- ggplot2::ggplot(
      dat_long,
      ggplot2::aes(
        x = X,
        y = value,
        group = interaction(tree, season_year),
        color = legend_var
      )
    ) +
      ggplot2::geom_line(linewidth = line_size, alpha = alpha)

    if (facet_by == "tree") {
      p <- p + ggplot2::facet_wrap(~ tree, scales = "free_y", ncol = 1)
    } else if (facet_by == "season_year") {
      p <- p + ggplot2::facet_wrap(~ season_year, scales = "free_y", ncol = 1)
    }

    p <- p +
      ggplot2::labs(
        title = title_base,
        x = x_lab,
        y = "Standardized value",
        color = legend_title
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    if (legend_by == "none") {
      p <- p + ggplot2::theme(legend.position = "none")
    }

    if (x_axis == "time") {
      p <- p + ggplot2::scale_x_date(date_labels = "%b-%Y", date_breaks = "2 months")
    }

    print(p)
    return(invisible(p))
  }

  if (type == "seasonal") {
    p <- ggplot2::ggplot(
      dat_long,
      ggplot2::aes(
        x = season_doy,
        y = value,
        group = interaction(tree, season_year),
        color = legend_var
      )
    ) +
      ggplot2::geom_line(linewidth = line_size, alpha = alpha)

    if (facet_by == "tree") {
      p <- p + ggplot2::facet_wrap(~ tree, scales = "free_y", ncol = 1)
    } else if (facet_by == "season_year") {
      p <- p + ggplot2::facet_wrap(~ season_year, scales = "free_y", ncol = 1)
    }

    p <- p +
      ggplot2::labs(
        title = title_base,
        x = "Seasonal day",
        y = "Standardized value",
        color = legend_title
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    if (legend_by == "none") {
      p <- p + ggplot2::theme(legend.position = "none")
    }

    print(p)
    return(invisible(p))
  }

  if (type == "heatmap") {
    p <- ggplot2::ggplot(
      dat_long,
      ggplot2::aes(x = season_doy, y = season_year, fill = value)
    ) +
      ggplot2::geom_tile()

    if (facet_by == "tree") {
      p <- p + ggplot2::facet_wrap(~ tree, ncol = 1)
    } else if (facet_by == "season_year") {
      # not meaningful to facet by season_year when y is season_year
      warning("For type = 'heatmap', facet_by = 'season_year' is not very informative because season_year is already on the y-axis.")
    }

    p <- p +
      ggplot2::labs(
        title = title_base,
        x = "Seasonal day",
        y = "Season year",
        fill = "Std. value"
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    print(p)
    return(invisible(p))
  }

  if (type == "boxplot") {
    if (box_group == "tree") {
      dat_long$group_var <- dat_long$tree
      x_lab2 <- "Tree"
    } else {
      dat_long$group_var <- dat_long$season_year
      x_lab2 <- "Season year"
    }

    p <- ggplot2::ggplot(
      dat_long,
      ggplot2::aes(x = group_var, y = value, fill = legend_var)
    ) +
      ggplot2::geom_boxplot()

    if (facet_by == "tree" && box_group == "season_year") {
      p <- p + ggplot2::facet_wrap(~ tree, scales = "free_y", ncol = 1)
    } else if (facet_by == "season_year" && box_group == "tree") {
      p <- p + ggplot2::facet_wrap(~ season_year, scales = "free_y", ncol = 1)
    }

    p <- p +
      ggplot2::labs(
        title = title_base,
        x = x_lab2,
        y = "Standardized value",
        fill = legend_title
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        axis.title = ggplot2::element_text(size = 14),
        axis.text = ggplot2::element_text(size = 11)
      )

    if (legend_by == "none") {
      p <- p + ggplot2::theme(legend.position = "none")
    }

    print(p)
    return(invisible(p))
  }
}

# =========================================================
# Internal helpers for climate attachment and plotting
# =========================================================

.dm_subset_window <- function(dat, time_col, Year = NULL, DOY = NULL) {
  tt <- dat[[time_col]]

  if (!inherits(tt, "Date") && !inherits(tt, "POSIXct")) {
    tt <- as.POSIXct(tt)
  }

  keep <- rep(TRUE, length(tt))

  if (!is.null(Year)) {
    keep <- keep & lubridate::year(tt) %in% Year
  }

  if (!is.null(DOY)) {
    if (length(DOY) != 2) {
      stop("'DOY' must be a numeric vector of length 2.")
    }
    doy_min <- min(DOY)
    doy_max <- max(DOY)
    keep <- keep & lubridate::yday(tt) >= doy_min & lubridate::yday(tt) <= doy_max
  }

  out <- dat[keep, , drop = FALSE]

  if (nrow(out) == 0) {
    stop("No data available for the selected Year/DOY window.")
  }

  out
}

.dm_label_phase <- function(ph, obj_class = c("ZG", "SC")) {
  obj_class <- match.arg(obj_class)

  if (obj_class == "ZG") {
    factor(ph, levels = c(1, 2), labels = c("TWD", "GRO"))
  } else {
    factor(ph, levels = c(1, 2, 3), labels = c("Shrinkage", "Expansion", "Increment"))
  }
}

.dm_detect_cycle_climate_cols <- function(dat, kind = c("ZG", "SC")) {
  kind <- match.arg(kind)

  core <- if (kind == "ZG") {
    c("Phases", "Start", "End", "Duration_h", "Magnitude", "rate",
      "max.twd", "Max.twd.time", "ABr.value", "Avg.twd", "STD.twd", "DOY")
  } else {
    c("Phases", "Start", "End", "Duration_h", "Duration_m", "Magnitude", "rate", "DOY")
  }

  setdiff(names(dat), core)
}

.dm_detect_point_climate_cols <- function(dat, kind = c("ZG", "SC")) {
  kind <- match.arg(kind)

  core <- if (kind == "ZG") {
    c("TIME", "dm", "Phases", "TWD", "GRO")
  } else {
    c("TIME", "dm", "Phases")
  }

  setdiff(names(dat), core)
}


# =========================================================
# 1) Single helper to add climate
# =========================================================

#' @title Add climate information to dendrometer outputs
#'
#' @description
#' A single helper function that adds climate information to outputs from
#' \code{daily.data()}, \code{phase.zg()}, or \code{phase.sc()}.
#'
#' Depending on the class of \code{x} and the selected \code{scale}, the function:
#' \itemize{
#'   \item joins daily climate to \code{daily.data()} output
#'   \item joins climate summaries to phase windows in \code{ZG_cycle} or \code{SC_cycle}
#'   \item joins subdaily climate features to point-level \code{ZG_phase} or \code{SC_phase}
#' }
#'
#' @param x Object returned by \code{daily.data()}, \code{phase.zg()}, or \code{phase.sc()}.
#' @param clim Climate input. This can be:
#'   \itemize{
#'     \item a standardized object returned by \code{read.climate()}
#'     \item a raw climate data frame
#'     \item a valid climate file path readable by \code{read.climate()}
#'     \item a daily climate table returned by \code{dm_daily_clim()}
#'     \item a subdaily climate-feature table returned by \code{dm_subdaily_clim()}
#'   }
#' @param scale Character string controlling how climate is attached.
#'   One of \code{"auto"}, \code{"daily"}, \code{"phase"}, or \code{"subdaily"}.
#'   \itemize{
#'     \item For \code{daily_output}, only \code{"daily"} is supported.
#'     \item For \code{ZG_output} and \code{SC_output}, \code{"phase"} adds climate
#'           summaries to \code{ZG_cycle}/\code{SC_cycle}, while \code{"subdaily"}
#'           adds timestamp-level climate to \code{ZG_phase}/\code{SC_phase}.
#'   }
#' @param mean_vars,min_vars,max_vars,sum_vars,median_vars Climate variables to
#'   summarize by mean, minimum, maximum, sum, or median.
#'   These are used by \code{dm_daily_clim()} and \code{dm_join_phase_clim()}.
#' @param lag_vars,lagmean_vars,lagsum_vars Variables used by \code{dm_daily_clim()}
#'   to build lagged and antecedent daily climate features.
#' @param lag_days Integer vector of lag/antecedent windows in days for
#'   \code{dm_daily_clim()}.
#' @param sub_mean_vars,sub_sum_vars,sub_lag_vars Variables used by
#'   \code{dm_subdaily_clim()} to build rolling and lagged subdaily features.
#' @param roll_hours,lag_hours Numeric vectors of rolling-window and lag sizes
#'   in hours for \code{dm_subdaily_clim()}.
#' @param suffix Suffix appended to climate summaries added by
#'   \code{dm_join_phase_clim()}.
#'
#' @return
#' The same biological object with climate information added.
#'
#' @examples
#' \donttest{
#' data(nepa17)
#' data(gf_nepa17)
#' data(ktm_clim_hourly)
#'
#' # daily.data() output + daily climate
#' dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
#' dd_clim <- dm_add_climate(
#'   dd,
#'   ktm_clim_hourly,
#'   scale = "daily",
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#' head(dd_clim)
#'
#' # phase.zg() output + phase-window climate
#' zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
#' zg_clim <- dm_add_climate(
#'   zg,
#'   ktm_clim_hourly,
#'   scale = "phase",
#'   mean_vars = c("temp", "VPD", "RH"),
#'   max_vars  = c("temp", "VPD"),
#'   sum_vars  = c("prec")
#' )
#' head(zg_clim$ZG_cycle)
#'
#' # phase.sc() output + point-level subdaily climate
#' sc <- phase.sc(df = gf_nepa17[1:800, ], TreeNum = 1, smoothing = 12)
#' sc_point <- dm_add_climate(
#'   sc,
#'   ktm_clim_hourly,
#'   scale = "subdaily",
#'   sub_mean_vars = c("temp", "VPD", "RH"),
#'   sub_sum_vars  = c("prec"),
#'   sub_lag_vars  = c("temp", "VPD", "RH"),
#'   roll_hours = c(1, 3, 6, 24),
#'   lag_hours  = c(1, 3, 6, 24)
#' )
#' head(sc_point$SC_phase)
#' }
#'
#' @export
dm_add_climate <- function(x,
                           clim,
                           scale = c("auto", "daily", "phase", "subdaily"),
                           mean_vars = NULL,
                           min_vars = NULL,
                           max_vars = NULL,
                           sum_vars = NULL,
                           median_vars = NULL,
                           lag_vars = NULL,
                           lagmean_vars = NULL,
                           lagsum_vars = NULL,
                           lag_days = c(1, 3, 7),
                           sub_mean_vars = NULL,
                           sub_sum_vars = NULL,
                           sub_lag_vars = NULL,
                           roll_hours = c(3, 6, 24),
                           lag_hours = c(1, 3, 6, 24),
                           suffix = "_phase") {

  scale <- match.arg(scale)

  # -----------------------------------------------------
  # daily.data() output
  # -----------------------------------------------------
  if (inherits(x, "daily_output")) {
    if (scale %in% c("phase", "subdaily")) {
      stop("For objects from daily.data(), only scale = 'daily' or 'auto' is supported.")
    }

    if (inherits(clim, "daily_clim")) {
      return(dm_join_daily_clim(x, clim))
    }

    clim_day <- dm_daily_clim(
      clim,
      mean_vars = mean_vars,
      min_vars = min_vars,
      max_vars = max_vars,
      sum_vars = sum_vars,
      median_vars = median_vars,
      lag_vars = lag_vars,
      lagmean_vars = lagmean_vars,
      lagsum_vars = lagsum_vars,
      lag_days = lag_days
    )

    return(dm_join_daily_clim(x, clim_day))
  }

  # -----------------------------------------------------
  # phase.zg() / phase.sc() output
  # -----------------------------------------------------
  if (inherits(x, "ZG_output") || inherits(x, "SC_output")) {

    if (scale == "auto") {
      if (inherits(clim, "subdaily_clim")) {
        scale <- "subdaily"
      } else {
        scale <- "phase"
      }
    }

    if (scale == "daily") {
      stop("For phase.zg() and phase.sc() outputs, use scale = 'phase' or scale = 'subdaily'.")
    }

    if (scale == "phase") {
      return(
        dm_join_phase_clim(
          x,
          clim,
          mean_vars = mean_vars,
          min_vars = min_vars,
          max_vars = max_vars,
          sum_vars = sum_vars,
          median_vars = median_vars,
          suffix = suffix
        )
      )
    }

    if (scale == "subdaily") {
      if (inherits(clim, "subdaily_clim")) {
        return(dm_join_subdaily_clim(x, clim))
      }

      clim_sub <- dm_subdaily_clim(
        clim,
        mean_vars = sub_mean_vars,
        sum_vars = sub_sum_vars,
        lag_vars = sub_lag_vars,
        roll_hours = roll_hours,
        lag_hours = lag_hours
      )

      return(dm_join_subdaily_clim(x, clim_sub))
    }
  }

  stop(
    "'x' must be an object returned by daily.data(), phase.zg(), or phase.sc()."
  )
}

---
title: "dendRoAnalyst: end-to-end workflow and function tour"
author: "Package vignette"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{dendRoAnalyst: end-to-end workflow and function tour}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5,
  warning = FALSE,
  message = FALSE
)

library(dendRoAnalyst)

RUN_HEAVY <- FALSE
RUN_INTERACTIVE <- FALSE
```

## Overview

This vignette presents a complete workflow for **dendRoAnalyst** and also acts
as a compact function tour for the user-facing functions covered by the uploaded
source files. The sequence follows a typical dendrometer analysis pipeline:

1. read dendrometer and climate data,
2. check time resolution, jumps, and gaps,
3. resample and truncate,
4. smooth the signal,
5. compute daily summaries,
6. classify stem-cycle phases,
7. classify zero-growth phases,
8. perform superposed epoch analysis,
9. fit growth curves and compare methods,
10. detrend and standardize,
11. compute moving climate correlations,
12. analyze harsh-climate episodes,
13. interpolate missing values from a network, and
14. inspect periodicity with wavelet tools.

The vignette is written in a CRAN-friendly style. Lightweight sections are
executed; heavier, interactive, or computationally expensive sections are shown
as code examples with `eval = FALSE`. All the summary and plot functions can also
be executed by simply applying `summary()` and `plot()` functions. 

## Example datasets

```{r data}
data(gf_nepa17)
data(nepa17)
data(ktm_clim_hourly)
data(ktm_rain17)

head(gf_nepa17[, 1:3])
head(ktm_clim_hourly)
```

Throughout this vignette we use:

- `gf_nepa17` for subdaily dendrometer workflows,
- `nepa17[1:1000, ]` for compact daily examples,
- `ktm_clim_hourly` for climate-feature examples, and
- `ktm_rain17` for harsh-climate examples.

```{r objects}
dm_hourly <- gf_nepa17
dm_daily_demo <- nepa17[1:1000, ]
dm_phase_demo <- gf_nepa17[1:800, ]
clim_hourly <- ktm_clim_hourly
clim_rain <- ktm_rain17
```

## 1. Reading dendrometer and climate data

### `read.dendrometer()`

When users start from raw files, `read.dendrometer()` is the recommended entry
point.

```{r read-dendrometer, eval = FALSE}
raw_dm <- read.dendrometer(
  file = "inst/extdata/my_dendrometer_file.csv",
  detect_resolution = TRUE,
  return_report = TRUE,
  quiet = FALSE
)
```

This call returns an imported dendrometer table and, optionally, an import
report describing parsing and time-resolution diagnostics.

### `read.climate()`

```{r read-climate}
clim_std <- read.climate(
  x = clim_hourly,
  time_col = "TIME",
  vars = c("temp", "prec", "VPD", "RH"),
  verbose = FALSE
)
head(clim_std)
```

The standardized climate object can later be aggregated to daily scale,
converted to subdaily climate features, or joined directly to dendrometer
outputs.

## 2. Looking for jumps or gaps in dendrometer data

### `reso_dm()`

```{r reso}
reso_dm(dm_hourly[[1]])
```

This returns the dominant sampling interval in minutes and helps diagnose
irregular logging intervals before subsequent processing.

### `jump.locator()` and `i.jump.locator()`

```{r jump-manual}
jump_free_manual <- jump.locator(
  df = dm_daily_demo[, 1:3],
  detection_method = "manual",
  manual_threshold = 1,
  adjustment_method = "window_median"
)
head(jump_free_manual)
```

```{r jump-auto, eval = FALSE}
jump_free_auto <- jump.locator(
  df = dm_daily_demo[, 1:3],
  detection_method = "auto",
  auto_method_penalty = 10,
  adjustment_method = "window_median"
)
```

```{r jump-interactive, eval = FALSE}
# Interactive graphical workflow
if (RUN_INTERACTIVE) {
  i.jump.locator(df = dm_daily_demo[, 1:3])
}
```

### `dm.na.interpolation()` and its plot methods

```{r gap-detect}
gap_demo <- dm_daily_demo[, 1:3]
gap_demo[150:170, 2] <- NA

gap_info <- dm.na.interpolation(
  df = gap_demo,
  resolution = 60,
  fill = FALSE,
  assess = FALSE
)
head(gap_info$gap_info)
```

```{r gap-fill}
gap_fill <- dm.na.interpolation(
  df = gap_demo,
  resolution = 60,
  fill = TRUE,
  method = "spline",
  assess = TRUE,
  assess_lengths_hours = c(6, 12)
)
head(gap_fill$data)
```

```{r gap-plots, fig.height=5}
plot(gap_fill)
plot(gap_fill)
plot(gap_fill, original = gap_demo)
plot(gap_fill)
```

These functions provide both the filled series and visual diagnostics of where
and how the gaps were handled.

## 3. Resample and truncate the dataset

### `dendro.resample()`

```{r resample}
resampled_hourly <- dendro.resample(
  df = dm_hourly,
  by = "H",
  value = "mean",
  method = "auto"
)
head(resampled_hourly)
```

### `dendro.truncate()`

```{r truncate}
truncated_demo <- dendro.truncate(dm_hourly, CalYear = 2017, DOY = c(1, 120))
head(truncated_demo)
```

Resampling harmonizes time steps; truncation helps focus on one year or one
season.

## 4. Smoothing

### `smooth_dm()`

```{r smooth}
res_minutes <- reso_dm(dm_phase_demo[[1]])
smoothed_medmean <- smooth_dm(
  time = dm_phase_demo[[1]],
  dm = dm_phase_demo[[2]],
  resolution_min = res_minutes,
  method = "median_mean",
  window_hours = 3
)
head(smoothed_medmean)
```

```{r smooth-other, eval = FALSE}
smoothed_pspline <- smooth_dm(
  time = dm_phase_demo[[1]],
  dm = dm_phase_demo[[2]],
  resolution_min = res_minutes,
  method = "pspline",
  window_hours = 6
)
```

Smoothing is often applied before phase classification to suppress short spikes.

## 5. Daily approach and daily climate functions

### `daily.data()` and `plot.daily_output()`

```{r daily-data}
daily_out <- daily.data(df = dm_daily_demo, TreeNum = 1)
head(daily_out)
```

```{r daily-plots}
plot(daily_out)
plot(daily_out, type = "timing")
plot(daily_out, type = "change")
```

```{r daily-more-plots, eval = FALSE}
plot(daily_out, type = "timing_violin", Year = 2017, DOY = c(1, 6))
plot(daily_out, type = "boxplot", stat = "amplitude", by = "month_of_year")
plot(daily_out, type = "heatmap")
```

### `dm_daily_clim()` and `dm_join_daily_clim()`

```{r daily-clim}
clim_day <- dm_daily_clim(
  clim_std,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec"),
  lag_vars = c("VPD_max", "temp_mean"),
  lagmean_vars = c("temp_mean", "VPD_mean"),
  lagsum_vars = c("prec_sum"),
  lag_days = c(1, 3, 7)
)
head(clim_day)
```

```{r daily-join}
daily_clim_joined <- dm_join_daily_clim(daily_out, clim_day)
head(daily_clim_joined)
```

### `dm_add_climate()` for daily outputs

```{r add-climate-daily}
daily_clim_added <- dm_add_climate(
  daily_out,
  clim_hourly,
  scale = "daily",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(daily_clim_added)
```

## 6. Stem-cycle approach and associated functions

### `phase.sc()` and `plot.SC_output()`

```{r sc-out}
sc_out <- phase.sc(df = dm_phase_demo, TreeNum = 1)
sc_out_sm <- phase.sc(df = dm_phase_demo, TreeNum = 1, smoothing = 12)
head(sc_out$SC_phase)
head(sc_out$SC_cycle)
```

```{r sc-plots}
plot(sc_out)
plot(sc_out, type = "ribbon")
plot(sc_out, type = "increment")
```

```{r sc-more-plots, eval = FALSE}
plot(sc_out, type = "transition", x_axis = "doy", Year = 2017, DOY = c(50, 100))
plot(sc_out, type = "balance", temporal = "daily")
plot(sc_out, type = "frequency", temporal = "monthly")
plot(sc_out, type = "heatmap", temporal = "daily")
plot(sc_out, type = "boxplot", stat = "Magnitude", temporal = "monthly")
```

### `dm_subdaily_clim()`, `dm_join_subdaily_clim()`, `dm_join_phase_clim()`, and `dm_add_climate()`

```{r subdaily-clim}
clim_sub <- dm_subdaily_clim(
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  sum_vars  = c("prec"),
  lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)
head(clim_sub)
```

```{r sc-joins}
sc_phase_clim <- dm_join_phase_clim(
  sc_out,
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(sc_phase_clim$SC_cycle)

sc_point_clim <- dm_join_subdaily_clim(sc_out_sm, clim_sub)
head(sc_point_clim$SC_phase)

sc_added_clim <- dm_add_climate(
  sc_out_sm,
  clim_hourly,
  scale = "subdaily",
  sub_mean_vars = c("temp", "VPD", "RH"),
  sub_sum_vars  = c("prec"),
  sub_lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)
head(sc_added_clim$SC_phase)
```

## 7. Zero-growth approach and associated functions

### `phase.zg()`, `twd.maxima()`, and `plot.ZG_output()`

```{r zg-out}
zg_out <- phase.zg(df = dm_phase_demo, TreeNum = 1)
zg_out_sm <- phase.zg(df = dm_phase_demo, TreeNum = 1, smoothing = 6)
head(zg_out$ZG_phase)
head(zg_out$ZG_cycle)
```

```{r twd-maxima}
max_twd_tbl <- twd.maxima(dm_phase_demo, TreeNum = 1, smoothing = 5)
head(max_twd_tbl)
```

```{r zg-plots}
plot(zg_out)
plot(zg_out, temporal = "daily")
plot(zg_out, temporal = "monthly", type = "twd")
```

```{r zg-more-plots, eval = FALSE}
plot(zg_out, temporal = "monthly", type = "abr")
plot(zg_out, type = "transition", x_axis = "doy", Year = 2017, DOY = c(50, 100))
plot(zg_out, type = "boxplot", stat = "max.twd", temporal = "daily")
```

### Climate joins and climate-phase plots

```{r zg-clim-joins}
zg_phase_clim <- dm_join_phase_clim(
  zg_out,
  clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(zg_phase_clim$ZG_cycle)

zg_point_clim <- dm_join_subdaily_clim(zg_out_sm, clim_sub)
head(zg_point_clim$ZG_phase)

zg_phase_added <- dm_add_climate(
  zg_out,
  clim_hourly,
  scale = "phase",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(zg_phase_added$ZG_cycle)
```

```{r climate-phase-plots}
plot(zg_phase_added, climate_var = "VPD_mean_phase", scale = "cycle", type = "boxplot", temporal = 'daily')
```

```{r climate-phase-compare, eval = FALSE}
plot(zg_phase_added, climate_var = "temp_mean_phase", scale = "cycle", type = "timeseries")

plot(
  zg_phase_added,
  climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
  scale = "cycle",
  type = "boxplot"
)

plot(
  zg_phase_added,
  scale = "cycle",
  type = "cor_heatmap",
  numeric_vars = c("Duration_h", "max.twd", "ABr.value",
                   "temp_mean_phase", "VPD_mean_phase", "RH_mean_phase")
)
```

## 8. Superposed epoch and associated functions

### `dm_event_times()`, `dm_epoch_extract()`, `dm_epoch_test()`, and method dispatch

```{r epoch}
event_tbl <- dm_event_times(zg_out, event = "all")
head(event_tbl)
```

```{r epoch-extract, eval = FALSE}
epoch_extract <- dm_epoch_extract(
  x = zg_out,
  clim = clim_hourly,
  event = "phase_start",
  phase = "all",
  response_var = c("temp", "VPD"),
  before = 24,
  after = 24,
  unit = "hours"
)
head(epoch_extract)
```

```{r epoch-test, eval = FALSE}
epoch_test <- dm_epoch_test(
  x = zg_out,
  clim = clim_hourly,
  event = "phase_start",
  phase = "all",
  response_var = c("temp", "VPD"),
  before = 24,
  after = 24,
  unit = "hours",
  n_boot = 99
)

epoch_summary <- summary(epoch_test)   # summary.dm_epoch()
print(epoch_summary)                   # print.summary.dm_epoch()
plot(epoch_test)
```

## 9. Event-based climate functions

```{r event-climate, eval = FALSE}
evt_zg <- dm_event_climate(
  zg_out,
  clim_hourly,
  event = "phase_start",
  phase = "all",
  windows = c(0, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)
head(evt_zg)
```

```{r event-climate-summary, eval = FALSE}
evt_summary <- dm_event_climate_summary(
  evt_zg,
  climate_var = "VPD_mean_prev_6h",
  group_var = "Phase",
  by = "month_of_year"
)
head(evt_summary)
```

```{r event-climate-test, eval = FALSE}
evt_test <- dm_event_climate_test(
  evt_zg,
  climate_var = "VPD_mean_prev_6h",
  group_var = "Phase",
  by = "month_of_year",
  method = "auto"
)
head(evt_test$summary)
head(evt_test$overall_tests)
head(evt_test$pairwise_tests)
```

```{r event-climate-plots, eval = FALSE}
plot(
  evt_zg,
  climate_var = "VPD_mean_prev_6h",
  group_var = "Phase",
  facet_by = "month_of_year",
  geom = "both",
  add_test = TRUE
)

evt_maxtwd <- dm_event_climate(
  zg_out,
  clim_hourly,
  event = "max_twd",
  windows = c(0, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)

plot(
  evt_maxtwd,
  climate_var = "VPD_mean_prev_6h",
  response_var = "max.twd",
  group_var = "Phase"
)
```

## 10. Growth fit and evaluation

### `dm.fit.gompertz()`

```{r legacy-gompertz, eval = FALSE}
gomp_legacy <- dm.fit.gompertz(df = dm_hourly, TreeNum = 1, CalYear = 2017, verbose = FALSE)
head(gomp_legacy)
```

### `dm.growth.fit()`, `plot.dm_growth_fit()`, and print/summary methods

```{r growth-fit, eval = FALSE}
fit_single <- dm.growth.fit(
  df = dm_hourly,
  TreeNum = 1:2,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)

print(fit_single)                 # print.dm_growth_fit()
fit_single_summary <- summary(fit_single)   # summary.dm_growth_fit()
print(fit_single_summary)         # print.summary.dm_growth_fit()
```

```{r growth-fit-plots, eval = FALSE}
plot(fit_single, type = "fit")
plot(fit_single, type = "season")
plot(fit_single, type = "residuals")
plot(fit_single, type = "timing")
plot(fit_single, type = "overlay")
plot(fit_single, type = "parameters")
```

### `dm.growth.fit.double()` and `dm.growth.evaluate()`

```{r growth-fit-double, eval = FALSE}
fit_double <- dm.growth.fit.double(
  df = dm_hourly,
  TreeNum = 1:2,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)
print(fit_double)
```

```{r growth-evaluate, eval = FALSE}
growth_eval <- dm.growth.evaluate(
  df = dm_hourly,
  TreeNum = 1:2,
  methods = c("gompertz", "loess", "spline", "double_gompertz"),
  year_mode = "yearly",
  verbose = FALSE
)
head(growth_eval)
```

```{r growth-evaluate-plots, eval = FALSE}
plot(growth_eval, metric = "rmse", type = "boxplot")
plot(growth_eval, metric = "r2", type = "heatmap")
```

## 11. Detrending and associated functions

```{r detrend, eval = FALSE}
detrended_fit <- dm.detrend.fit(fit_single)
plot(detrended_fit, type = "compare")
plot(detrended_fit, type = "fit")
plot(detrended_fit, type = "residual")
plot(detrended_fit, type = "detrended")
plot(detrended_fit, type = "boxplot")
```

```{r mean-detrended, eval = FALSE}
mean_det <- mean_detrended.dm(detrended_fit)
head(mean_det)
plot(mean_det)
```

### `dm_standardize()` and `plot.dm_standardized()`

```{r standardize}
std_nh <- dm_standardize(
  df = dm_hourly,
  season_type = "NH",
  method = "robust_amplitude"
)
head(std_nh$data)
head(std_nh$parameters)
```

```{r standardize-plot}
plot(std_nh)
```

## 12. Moving correlations and associated functions

```{r mov-cor, eval = FALSE}
mov_out <- mov.cor.dm(
  df = dm_hourly,
  Clim = clim_hourly,
  TreeNum = 1,
  win_size = 15,
  cor_method = "pearson",
  boot = FALSE
)

print(mov_out)                # print.mov_cor_dm()
mov_summary <- summary(mov_out)   # summary.mov_cor_dm()
print(mov_summary)            # print.summary_mov_cor_dm()
```

```{r mov-cor-plots, eval = FALSE}
plot(mov_out, type = "heatmap")
plot(mov_out, type = "line", annotate_peak = TRUE)
plot(mov_out, type = "facet")
plot(mov_out, type = "peak")
plot(mov_out, type = "line")
plot(mov_summary, type = "peak_corr")
```

```{r mov-cor-boot, eval = FALSE}
mov_boot <- mov.cor.dm(
  df = dm_hourly,
  Clim = clim_hourly,
  TreeNum = 1,
  win_size = 15,
  cor_method = "pearson",
  boot = TRUE,
  R = 99,
  set_seed = 1
)
plot(mov_boot, type = "line", show_ci = TRUE)
```

## 13. Harsh climate using `clim.twd()` and associated functions

```{r clim-twd, eval = FALSE}
rel_out <- clim.twd(
  df = dm_hourly,
  Clim = clim_rain,
  thresholdClim = "<10",
  thresholdDays = ">5",
  showPlot = FALSE
)
```

```{r clim-twd-stats, eval = FALSE}
clim_stats <- clim.twd.stats(
  rel_out,
  response = "full_period_change",
  group_by = "tree"
)

clim_stats_summary <- summary(clim_stats)   # summary.clim_twd_stats()
print(clim_stats_summary)                   # print.summary.clim_twd_stats()
plot(clim_stats, type = "trajectory")
```

```{r clim-twd-test, eval = FALSE}
tree_info <- data.frame(
  tree = c("T2", "T3"),
  species = c("Pinus", "Pinus"),
  site = c("Ktm", "Ktm")
)

clim_test <- clim.twd.test(
  rel_out,
  tree_info = tree_info,
  parameter = "lag_to_return_zero",
  compare_by = "species",
  within = "year"
)

clim_test_summary <- summary(clim_test)   # summary.clim_twd_test()
print(clim_test_summary)                  # print.summary.clim_twd_test()
plot(clim_test)
```

## 14. Network interpolation

### `network.interpolation()` and `plot.network_interpolation()`

```{r network-interp, eval = FALSE}
df1 <- dm_hourly
df1[40:50, "T2"] <- NA

ref_net <- cbind(dm_hourly, dm_hourly[, 2:3], dm_hourly[, 2:3])
colnames(ref_net) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")

net_interp <- network.interpolation(
  df1,
  ref_net,
  niMethod = "proportional",
  n_boot = 100,
  return_flags = TRUE,
  return_pi = TRUE,
  assess = TRUE,
  assess_lengths_steps = c(1, 2, 4),
  correct_gap_jumps = TRUE,
  jump_threshold = 0.05
)
head(net_interp)
# to get the assessment stats we can use attr() function
attr(out, "network_validation_summary")
attr(out, "network_validation_points")
attr(out, "network_validation_seasonal")
```

```{r network-interp-plots, eval = FALSE}
plot(net_interp)
plot(net_interp, type = "uncertainty")
plot(net_interp, type = "availability")
plot(net_interp, type = "summary", summary_metric = "n_gap_jumps_removed")
plot(net_interp, type = "validation", validation_metric = "MAPE")
plot(net_interp, type = "coverage")
plot(net_interp, type = "compare")
plot(net_interp, type = "seasonal_error", seasonal_metric = "mean_abs_error")
```

## 15. Wavelet analysis and associated functions

### `dm_wavelet()`, `summary.dm_wavelet()`, and `plot.dm_wavelet()`

```{r wavelet, eval = FALSE}
wv <- dm_wavelet(
  x = dm_hourly,
  TreeNum = 1:2,
  source = "raw",
  make_pval = TRUE,
  verbose = FALSE
)

wv_summary <- summary(wv)   # summary.dm_wavelet()
print(wv_summary)           # print.summary.dm_wavelet()
```

```{r wavelet-plots, eval = FALSE}
plot(wv, type = "series")
plot(wv, type = "average")
plot(wv, type = "power")
plot(wv, series = names(wv$results)[1], type = "power")
```

### `dm_wavelet_reconstruct()` and its methods

```{r wavelet-reconstruct, eval = FALSE}
rec_extract <- dm_wavelet_reconstruct(
  wv,
  mode = "extract",
  lower_hours = 20,
  upper_hours = 28,
  only_sig = TRUE
)

rec_remove <- dm_wavelet_reconstruct(
  wv,
  mode = "remove",
  lower_hours = 20,
  upper_hours = 28,
  only_sig = TRUE
)

rec_extract_summary <- summary(rec_extract)   # summary.dm_wavelet_reconstruct()
print(rec_extract_summary)                    # print.summary.dm_wavelet_reconstruct()
```

```{r wavelet-reconstruct-plots, eval = FALSE}
plot(rec_extract)
plot(rec_remove)
```

### `dm_wavelet_coherence()` and `plot.dm_wavelet_coherence()`

```{r wavelet-coherence, eval = FALSE}
wc <- dm_wavelet_coherence(
  dm = dm_hourly,
  clim = clim_hourly,
  TreeNum = 1,
  clim_vars = c("temp", "VPD"),
  source = "raw",
  dm_fun = "mean",
  clim_funs = c(temp = "mean", VPD = "mean"),
  loess_span = 0,
  make.pval = TRUE,
  n.sim = 10,
  verbose = FALSE
)
```

```{r wavelet-coherence-plots, eval = FALSE}
plot(wc)
plot(wc, type = "cross_power")
plot(wc, type = "average_coherence")
plot(wc, type = "phase")
plot(wc, type = "series")
```

## Closing note

This vignette is intended to be both a workflow guide and a compact reference
for the main user-facing functions in the current **dendRoAnalyst** 
analysis pipeline. For classroom teaching, the companion demonstration script
can be run section by section; for package documentation, this vignette keeps
only the lighter sections evaluated and leaves heavy or interactive examples in
place for users to run locally.

## For suggestions, comments and questions please contact : <sugam.aryal@fau.de>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_evaluation.R
\name{plot.dm_growth_evaluation}
\alias{plot.dm_growth_evaluation}
\title{Plot growth-fitting evaluation statistics}
\usage{
\method{plot}{dm_growth_evaluation}(
  x,
  metric = c("rmse", "mae", "bias", "abs_bias", "r2", "correlation", "nrmse", "rss",
    "aic_approx", "bic_approx"),
  type = c("boxplot", "mean", "heatmap"),
  order_methods = TRUE,
  decreasing = NULL,
  show_points = TRUE,
  show_errorbar = TRUE,
  heatmap_label = c("series_fit", "series", "fit_id"),
  na.rm = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_growth_evaluation"}.}

\item{metric}{Evaluation metric to plot. One of \code{"rmse"},
\code{"mae"}, \code{"bias"}, \code{"abs_bias"}, \code{"r2"},
\code{"correlation"}, \code{"nrmse"}, \code{"rss"},
\code{"aic_approx"}, or \code{"bic_approx"}.}

\item{type}{Plot type. One of \code{"boxplot"}, \code{"mean"}, or
\code{"heatmap"}.}

\item{order_methods}{Logical. If \code{TRUE}, methods are ordered by their
average value for the chosen metric.}

\item{decreasing}{Logical or \code{NULL}. Controls method ordering. If
\code{NULL}, metrics where smaller values are better are ordered ascending,
and metrics where larger values are better are ordered descending.}

\item{show_points}{Logical. If \code{TRUE}, add individual fit points to
\code{"boxplot"} and \code{"mean"} displays.}

\item{show_errorbar}{Logical. If \code{TRUE}, show standard-error bars when
\code{type = "mean"}.}

\item{heatmap_label}{Character string used to label rows in the heatmap. One
of \code{"series_fit"}, \code{"series"}, or \code{"fit_id"}.}

\item{na.rm}{Logical. If \code{TRUE}, remove missing metric values before
plotting.}

\item{...}{Further arguments passed to plotting methods.}
}
\value{
A \code{ggplot} object.
}
\description{
Creates ggplot2-based comparison plots for objects returned by
[dm.growth.evaluate()].
}
\seealso{
[dm.growth.evaluate()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{plot_dm_gaps}
\alias{plot_dm_gaps}
\title{Plot detected gaps in dendrometer data}
\usage{
plot_dm_gaps(x, series = NULL, empty_gaps = c("plot", "error"))
}
\arguments{
\item{x}{Object returned by \code{dm.na.interpolation()}.}

\item{series}{Optional character vector of series names to plot.}

\item{empty_gaps}{Character. Behavior when no gaps are present:
\code{"plot"} for an informative empty plot, or \code{"error"}.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plot detected gaps in dendrometer data
}
\examples{
\donttest{
#res <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#plot_dm_gaps(res)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.daily_output.R
\name{plot.daily_output}
\alias{plot.daily_output}
\title{Plot method for daily dendrometer statistics}
\usage{
\method{plot}{daily_output}(
  x,
  y = NULL,
  Year = NULL,
  DOY = NULL,
  type = c("summary", "amplitude", "minmax_ribbon", "timing", "timing_violin", "lag",
    "change", "boxplot", "heatmap"),
  stat = c("amplitude", "Max_diff", "mean", "median", "Max", "Min", "lag_h",
    "Time_min_h", "Time_max_h"),
  by = c("month", "month_of_year", "year"),
  box_style = c("boxplot", "violin", "both"),
  status_cols = c(growing = "forestgreen", shrinking = "firebrick", stable = "grey60"),
  timing_segment_cols = c(`max after min` = "grey65", `max before/equal min` = "black"),
  ...
)
}
\arguments{
\item{x}{Object of class \code{"daily_output"}.}

\item{y}{Unused.}

\item{Year}{Optional numeric year or vector of years for subsetting.}

\item{DOY}{Optional numeric vector of length 2 giving the start and end
day-of-year for subsetting.}

\item{type}{Plot type. One of
\code{"summary"}, \code{"amplitude"}, \code{"minmax_ribbon"},
\code{"timing"}, \code{"timing_violin"}, \code{"lag"}, \code{"change"},
\code{"boxplot"}, or \code{"heatmap"}.}

\item{stat}{Statistic used for \code{type = "boxplot"} or
\code{type = "heatmap"}. One of
\code{"amplitude"}, \code{"Max_diff"}, \code{"mean"},
\code{"median"}, \code{"Max"}, \code{"Min"},
\code{"lag_h"}, \code{"Time_min_h"}, \code{"Time_max_h"}.}

\item{by}{Grouping for \code{type = "boxplot"}:
\code{"month"}, \code{"month_of_year"}, or \code{"year"}.
\code{"month"} uses a continuous calendar-date axis spanning the selected
data window and supports multi-year plotting, for example 2022--2024.
Empty calendar months may appear as gaps, but months with available data
are not clipped at the start or end of the axis.}

\item{box_style}{Style for \code{type = "boxplot"}:
\code{"boxplot"}, \code{"violin"}, or \code{"both"}.}

\item{status_cols}{Colors for \code{Day_status}.}

\item{timing_segment_cols}{Named vector of two colors for the timing connector:
one for \code{"max after min"} and one for \code{"max before/equal min"}.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
Unified S3 plotting method for objects returned by \code{daily.data()}.
Missing grouping labels are removed before grouped boxplots are drawn so
\code{NA} does not appear on the x-axis.
}
\details{
Plot types:
\itemize{
  \item \code{"summary"}: daily Min, mean, median, and Max.
  \item \code{"amplitude"}: daily amplitude through time.
  \item \code{"minmax_ribbon"}: ribbon between daily Min and Max.
  \item \code{"timing"}: per-day line connecting daily minimum and maximum timing.
  \item \code{"timing_violin"}: violin distribution of \code{Time_min_h}
        and \code{Time_max_h} for the selected window.
  \item \code{"lag"}: signed lag between daily maximum and minimum time.
  \item \code{"change"}: day-to-day change in daily maximum
        \code{Max_diff}.
  \item \code{"boxplot"}: grouped distributions of a chosen daily statistic.
        For \code{by = "month"}, monthly groups are plotted on a continuous
        date axis. The axis limits are extended beyond the first and last
        plotted month so edge months are not clipped.
  \item \code{"heatmap"}: year x day-of-year heatmap of a chosen daily statistic.
}

For \code{type = "timing"}, the red and blue points are not connected across
days. Instead, each day has its own segment connecting minimum and maximum
time.
}
\examples{
\donttest{
data(nepa17)
dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)

plot(dd)
plot(dd, type = "timing")
plot(dd, type = "timing", Year = 2017, DOY = c(1, 6))
plot(dd, type = "timing_violin", Year = 2017, DOY = c(1, 6))
plot(dd, type = "boxplot", stat = "amplitude", by = "month")
plot(dd, type = "boxplot", stat = "amplitude", by = "month_of_year",
     box_style = "both")
plot(dd, type = "change")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_coherence.R
\name{dm_wavelet_coherence}
\alias{dm_wavelet_coherence}
\title{Wavelet coherence between dendrometer and climate series}
\usage{
dm_wavelet_coherence(
  dm,
  clim,
  TreeNum = "all",
  clim_vars = "all",
  source = c("auto", "raw", "detrended", "first_diff"),
  detrended_col = c("detrended_data"),
  dm_fun = c("mean", "max", "min", "sum"),
  clim_funs = "mean",
  na_action = c("interpolate", "fail"),
  loess_span = 0,
  dj = 1/20,
  lowerPeriod = NULL,
  upperPeriod = NULL,
  window.type.t = 1,
  window.type.s = 1,
  window.size.t = 5,
  window.size.s = 1/4,
  make.pval = TRUE,
  make_pval = NULL,
  method = "white.noise",
  n.sim = 10,
  n_sim = NULL,
  verbose = TRUE
)
}
\arguments{
\item{dm}{Dendrometer input. Either a data frame with time in the first column
or an object of class \code{"dm_detrended"}.}

\item{clim}{Climate data frame. The first column must be time; remaining
selected columns must be numeric climate variables.}

\item{TreeNum}{Dendrometer series selector: \code{"all"}, numeric indices,
or character names.}

\item{clim_vars}{Climate variable selector: \code{"all"}, numeric indices,
or character names.}

\item{source}{One of \code{"auto"}, \code{"raw"}, \code{"detrended"},
or \code{"first_diff"}.}

\item{detrended_col}{For \code{dm_detrended} input, which table to use.}

\item{dm_fun}{Aggregation function for dendrometer resampling: one of
\code{"mean"}, \code{"max"}, \code{"min"}, \code{"sum"}.}

\item{clim_funs}{Either a single aggregation function applied to all climate
variables, or a named character vector such as
\code{c(temp = "mean", VPD = "mean", prec = "sum")}.}

\item{na_action}{One of \code{"interpolate"} or \code{"fail"}.}

\item{loess_span}{Detrending span passed to \code{WaveletComp::analyze.coherency()}.
Use \code{0} to disable internal loess detrending.}

\item{dj}{Frequency resolution passed to \code{WaveletComp::analyze.coherency()}.}

\item{lowerPeriod}{Optional lower period in native analysis units.}

\item{upperPeriod}{Optional upper period in native analysis units.}

\item{window.type.t}{Time-direction smoothing window type.}

\item{window.type.s}{Scale-direction smoothing window type.}

\item{window.size.t}{Time-direction smoothing window size.}

\item{window.size.s}{Scale-direction smoothing window size.}

\item{make.pval}{Logical. If \code{TRUE}, compute p-values.}

\item{make_pval}{Optional alias for \code{make.pval}.}

\item{method}{Surrogate generation method for p-values.}

\item{n.sim}{Number of simulations.}

\item{n_sim}{Optional alias for \code{n.sim}.}

\item{verbose}{Logical.}
}
\value{
An object of class \code{"dm_wavelet_coherence"}.
}
\description{
Computes cross-wavelet power and wavelet coherence between dendrometer series
and climate variables using \code{WaveletComp::analyze.coherency()}.

The function:
\itemize{
  \item crops dendrometer and climate inputs to their common time window,
  \item detects their temporal resolutions,
  \item resamples both to the coarser detected resolution,
  \item optionally interpolates missing values,
  \item optionally uses first differences of the dendrometer series,
  \item computes pairwise coherence for all selected tree/climate combinations.
}
}
\details{
\code{WaveletComp::analyze.coherency()} expects two aligned series in one data
frame plus a \code{dt} value in the analysis time units, and returns
cross-wavelet power, coherence, phase angles, p-values, Fourier periods, and
cone-of-influence fields.
}
\examples{
\donttest{
wc <- dm_wavelet_coherence(
  dm = gf_nepa17,
  clim = ktm_clim_hourly,
  TreeNum = 1,
  clim_vars = c("temp", "VPD"),
  source = "raw",
  dm_fun = "mean",
  clim_funs = c(temp = "mean", VPD = "mean"),
  loess_span = 0,
  make.pval = TRUE,
  n.sim = 10,
  verbose = FALSE
)

plot(wc)
plot(wc, type = "cross_power")
plot(wc, type = "average_coherence")
plot(wc, type = "phase")
plot(wc, type = "series")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_infer_double_richards_starts}
\alias{dmgfd_infer_double_richards_starts}
\title{Infer double-Richards starting values}
\usage{
dmgfd_infer_double_richards_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial total asymptote.}
}
\value{
Named list of starting values.
}
\description{
Infer double-Richards starting values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_join_daily_clim.R
\name{dm_join_daily_clim}
\alias{dm_join_daily_clim}
\title{Join daily climate summaries to daily dendrometer statistics}
\usage{
dm_join_daily_clim(daily_obj, clim_daily)
}
\arguments{
\item{daily_obj}{Output of \code{daily.data()} with class \code{"daily_output"}.}

\item{clim_daily}{Climate input. This can be a daily climate table, a
standardized climate object returned by \code{read.climate()}, a raw
climate data frame, or a valid climate file path.}
}
\value{
The \code{daily.data()} output with climate columns appended. The returned
object has class \code{c("daily_output_clim", "daily_output", ...)}.
}
\description{
Joins daily climate summaries to the output of \code{daily.data()} by
calendar date.

The climate input can be:
\itemize{
  \item the output of \code{dm_daily_clim()}
  \item the output of \code{read.climate()}
  \item a raw climate data frame
  \item a valid climate file path readable by \code{read.climate()}
}

If the supplied climate input is not already a daily climate table, it will
first be converted to daily summaries with \code{dm_daily_clim()} using
default daily means for numeric variables.
}
\examples{
\donttest{
data(nepa17)
data(ktm_clim_hourly)

dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)

clim_std <- read.climate(
  ktm_clim_hourly,
  time_col = "TIME",
  vars = c("temp", "prec", "VPD", "RH"),
  verbose = FALSE
)

clim_day <- dm_daily_clim(
  clim_std,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)

dd_clim <- dm_join_daily_clim(dd, clim_day)
head(dd_clim)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_extract_from_anchors}
\alias{dmea_extract_from_anchors}
\title{Extract climate values around anchor times}
\usage{
dmea_extract_from_anchors(anchor_times, clim_tbl, lag_values, vars)
}
\arguments{
\item{anchor_times}{Anchor times.}

\item{clim_tbl}{Aggregated climate table from [dmea_prepare_climate()].}

\item{lag_values}{Lag values in seconds.}

\item{vars}{Climate variables to extract.}
}
\value{
A long-format tibble.
}
\description{
Extract climate values around anchor times
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{plot_dm_assessment}
\alias{plot_dm_assessment}
\title{Plot interpolation assessment metrics}
\usage{
plot_dm_assessment(x, metric = "MdAPE", series = NULL, facet = TRUE)
}
\arguments{
\item{x}{Object returned by \code{dm.na.interpolation()} with
\code{assess = TRUE}.}

\item{metric}{Character. One of:
\code{"MAPE"}, \code{"MdAPE"}, \code{"RMSE_pct"}, \code{"Bias_pct"},
\code{"Max_diff_abs"}, \code{"Min_diff_abs"},
\code{"Time_max_diff_h"}, \code{"Time_min_diff_h"}.}

\item{series}{Optional character vector of series names to plot.}

\item{facet}{Logical. If \code{TRUE}, create one panel per series.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plot interpolation assessment metrics
}
\examples{
\donttest{
#res <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "seasonal",
#  assess = TRUE
#)
#plot_dm_assessment(res, metric = "MdAPE")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_cummax_na}
\alias{dmgfd_cummax_na}
\title{Cumulative maximum while preserving missing values}
\usage{
dmgfd_cummax_na(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector.
}
\description{
Cumulative maximum while preserving missing values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_rate_window}
\alias{dmgf_rate_window}
\title{Estimate active-growth window from growth rate}
\usage{
dmgf_rate_window(rate, x_pred, threshold_fraction = 0.1)
}
\arguments{
\item{rate}{Fitted growth-rate vector.}

\item{x_pred}{Prediction x values.}

\item{threshold_fraction}{Fraction of peak growth rate.}
}
\value{
List with peak rate, start day, and end day.
}
\description{
Estimate active-growth window from growth rate
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_fit_one_curve}
\alias{dmgf_fit_one_curve}
\title{Fit one growth curve}
\usage{
dmgf_fit_one_curve(
  x_obs,
  y_obs,
  x_pred,
  season_start_date = as.Date(NA),
  season_length = NA_real_,
  season_id = NULL,
  method,
  loess_span,
  spline_df,
  growth_fraction,
  min_unique_growth,
  rate_threshold_fraction = 0.1,
  fix_a_to_observed_max = FALSE,
  fixed_a_multiplier = 1,
  start_value_gompertz_parameters,
  start_value_richards_parameters
)
}
\arguments{
\item{x_obs}{Observed x values.}

\item{y_obs}{Observed growth values.}

\item{x_pred}{Prediction x values.}

\item{season_start_date}{Season start date.}

\item{season_length}{Season length.}

\item{season_id}{Optional season ID for pooled fits.}

\item{method}{Fitting method.}

\item{loess_span}{Loess span.}

\item{spline_df}{Spline degrees of freedom.}

\item{growth_fraction}{Growth fractions used for cumulative timing.}

\item{min_unique_growth}{Minimum number of unique growth values.}

\item{rate_threshold_fraction}{Fraction of peak rate used for rate timing.}

\item{fix_a_to_observed_max}{Logical.}

\item{fixed_a_multiplier}{Multiplier for fixed asymptote.}

\item{start_value_gompertz_parameters}{Starting values for Gompertz.}

\item{start_value_richards_parameters}{Starting values for logistic/Richards.}
}
\value{
List with fitted predictions and parameters.
}
\description{
Internal helper used by [dm.growth.fit()] to fit one curve for one series and
one season, or one pooled curve.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_resolve_timing_facet_scales}
\alias{dmgp_resolve_timing_facet_scales}
\title{Resolve facet scales for timing plots}
\usage{
dmgp_resolve_timing_facet_scales(
  x_mode = c("date", "doy", "season_day"),
  facet_by = c("default", "tree", "year", "none")
)
}
\arguments{
\item{x_mode}{Timing coordinate system. One of \code{"date"}, \code{"doy"},
or \code{"season_day"}.}

\item{facet_by}{Faceting mode.}
}
\value{
A character string accepted by [ggplot2::facet_wrap()] argument
  \code{scales}.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to choose facet scales for
timing-summary plots.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dm_epoch_test}
\alias{dm_epoch_test}
\title{Test superposed epoch composites against a null distribution}
\usage{
dm_epoch_test(
  x,
  statistic = c("mean", "median"),
  null = c("same_doy_window", "same_month", "random_time"),
  n_iter = 1000,
  doy_window = 15,
  match_hour = TRUE,
  match_minute = TRUE,
  conf_level = 0.95,
  seed = NULL
)
}
\arguments{
\item{x}{Object returned by [dm_epoch_extract()].}

\item{statistic}{Composite statistic. One of \code{"mean"} or
\code{"median"}.}

\item{null}{Null model. One of \code{"same_doy_window"},
\code{"same_month"}, or \code{"random_time"}.}

\item{n_iter}{Number of null iterations.}

\item{doy_window}{DOY window used for \code{null = "same_doy_window"}.}

\item{match_hour}{Logical. For hourly or minute data, keep the same hour
when possible.}

\item{match_minute}{Logical. For minute data, keep the same minute when
possible.}

\item{conf_level}{Confidence level for null envelopes.}

\item{seed}{Optional random seed.}
}
\value{
An object of class \code{c("dm_epoch", "dm_epoch_test")}.
}
\description{
Tests observed superposed epoch composites against empirical null
distributions generated by randomly sampled anchor times.

If the input object was created with \code{Year} and/or \code{DOY} filters
in [dm_epoch_extract()] and \code{restrict_null_to_window = TRUE}, null
anchors are sampled from the same calendar-year and day-of-year window.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_mov.cor.R
\name{plot.mov_cor_dm}
\alias{plot.mov_cor_dm}
\title{Plot method for moving dendrometer-climate correlation}
\usage{
\method{plot}{mov_cor_dm}(
  x,
  y = NULL,
  sig.only = TRUE,
  ci = 0.95,
  clim_vars = "all",
  type = c("heatmap", "line", "facet", "peak", "compare"),
  x_axis = c("time", "doy"),
  use_adjusted = TRUE,
  show_na = TRUE,
  show_ci = FALSE,
  sig_mode = c("outline", "point", "filter", "none"),
  annotate_peak = FALSE,
  show_window_label = TRUE,
  compare_with = NULL,
  compare_labels = NULL,
  low_col = "red",
  mid_col = "white",
  high_col = "blue",
  na_col = "grey79",
  line_size = 0.5,
  point_size = 1.8,
  alpha_sig = 0.9,
  ...
)
}
\arguments{
\item{x}{Object returned by \code{mov.cor.dm()}.}

\item{y}{Unused.}

\item{sig.only}{Logical. If \code{TRUE}, only significant windows are shown in
heatmaps and significance is highlighted in other plot types.}

\item{ci}{Numeric confidence level between 0 and 1. For non-bootstrap
results, significance is based on \code{p_val < 1 - ci} or
\code{p_adj < 1 - ci}. For bootstrap results, the stored logical
\code{significance} column is used.}

\item{clim_vars}{Character vector of climate variables to plot, or
\code{"all"} for all variables.}

\item{type}{Plot type. One of \code{"heatmap"}, \code{"line"},
\code{"facet"}, \code{"peak"}, or \code{"compare"}.}

\item{x_axis}{X-axis style. One of \code{"time"} or \code{"doy"}.}

\item{use_adjusted}{Logical. For non-bootstrap objects, if \code{TRUE},
significance uses \code{p_adj}; otherwise \code{p_val}.}

\item{show_na}{Logical. If \code{TRUE}, missing values are shown in heatmaps.}

\item{show_ci}{Logical. If \code{TRUE}, bootstrap confidence intervals are
shown in line/facet/compare plots when available.}

\item{sig_mode}{One of \code{"outline"}, \code{"point"}, \code{"filter"},
or \code{"none"} describing how significance is displayed in non-heatmap plots.}

\item{annotate_peak}{Logical. If \code{TRUE}, the strongest absolute
correlation is marked for each climate variable in line/facet/compare plots.}

\item{show_window_label}{Logical. If \code{TRUE}, \code{type = "peak"} adds
the climate transformation settings above each bar.}

\item{compare_with}{Optional list of additional \code{mov_cor_dm} objects for
comparison. Used only when \code{type = "compare"}.}

\item{compare_labels}{Optional character vector of labels for the comparison
objects. If \code{NULL}, defaults are generated automatically.}

\item{low_col}{Colour for negative correlations.}

\item{mid_col}{Colour for zero correlations.}

\item{high_col}{Colour for positive correlations.}

\item{na_col}{Fill colour for missing values in heatmaps.}

\item{line_size}{Numeric line width for line plots.}

\item{point_size}{Numeric point size for significance markers and peak markers.}

\item{alpha_sig}{Numeric transparency used for significance highlighting.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
S3 plotting method for output of \code{mov.cor.dm()}.
Supports heatmaps, line plots, faceted line plots, peak-correlation summaries,
and comparison across multiple \code{mov.cor.dm()} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean.detrended.dm.R
\name{plot.mean_dm_detrended}
\alias{plot.mean_dm_detrended}
\title{Plot mean detrended dendrometer series}
\usage{
\method{plot}{mean_dm_detrended}(
  x,
  y = NULL,
  type = c("series", "seasonal", "boxplot"),
  value = c("both", "STD_DDM", "RES_DDM"),
  seasons = NULL,
  x_axis = c("default", "date", "doy", "season_day"),
  facet_by = c("none", "season", "metric"),
  ncol = NULL,
  box_group = c("metric", "season"),
  alpha = 0.8,
  line_width = 0.8,
  point_size = 1.4,
  legend_position = "right",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"mean_dm_detrended"} returned by
\code{mean_detrended.dm()}.}

\item{y}{Unused.}

\item{type}{Plot type. One of:
\describe{
  \item{`"series"`}{Plot the time series over calendar time, DOY, or season day.}
  \item{`"seasonal"`}{Overlay seasons on a season-day scale.}
  \item{`"boxplot"`}{Show boxplots of the detrended values by season or metric.}
}}

\item{value}{Which variable to plot. One of:
\describe{
  \item{`"both"`}{Plot both \code{STD_DDM} and \code{RES_DDM} when available.}
  \item{`"STD_DDM"`}{Plot only the mean detrended series.}
  \item{`"RES_DDM"`}{Plot only the autocorrelation-removed mean detrended series.}
}}

\item{seasons}{Optional character vector of \code{season_label} values to retain.}

\item{x_axis}{Character string controlling the x-axis. One of:
\describe{
  \item{`"default"`}{Uses calendar date for \code{type = "series"} and
    season day for \code{type = "seasonal"}.}
  \item{`"date"`}{Use actual calendar date.}
  \item{`"doy"`}{Use calendar day-of-year.}
  \item{`"season_day"`}{Use vegetation season day.}
}}

\item{facet_by}{Character string controlling faceting. One of:
\describe{
  \item{`"none"`}{No faceting.}
  \item{`"season"`}{Facet by \code{season_label}.}
  \item{`"metric"`}{Facet by metric (\code{STD_DDM}, \code{RES_DDM}).}
}}

\item{ncol}{Optional integer giving the number of facet columns.}

\item{box_group}{For \code{type = "boxplot"}, grouping variable on the x-axis.
One of \code{"metric"} or \code{"season"}.}

\item{alpha}{Numeric alpha transparency for lines/points.}

\item{line_width}{Numeric line width.}

\item{point_size}{Numeric point size.}

\item{legend_position}{Character legend position passed to ggplot2.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
S3 plotting method for objects returned by \code{mean_detrended.dm()}.

It can plot:
\itemize{
  \item the mean detrended series (\code{STD_DDM}),
  \item the autocorrelation-removed mean detrended series (\code{RES_DDM}),
  \item or both together.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jump.locator.R
\name{jump.locator}
\alias{jump.locator}
\title{Removing artefacts due to manual adjustments of dendrometers automatically for more than one dendrometer}
\usage{
jump.locator(
  df,
  detection_method = c("manual", "auto"),
  manual_threshold = NULL,
  auto_method_penalty = 10,
  adjustment_method = c("window_median", "point_diff"),
  adjust_window = 5,
  auto_every_second = TRUE,
  set_jump_point_na = FALSE
)
}
\arguments{
\item{df}{Data frame with first column containing date and time in the format
\code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.}

\item{detection_method}{Either \code{"manual"} for threshold-based jump
detection or \code{"auto"} for automatic detection using
\code{changepoint::cpt.mean()} with \code{penalty = "Manual"}.}

\item{manual_threshold}{Numeric threshold considered as artefact when
\code{detection_method = "manual"}.}

\item{auto_method_penalty}{Numeric manual penalty value used in
\code{changepoint::cpt.mean()} when \code{detection_method = "auto"}.
Larger values generally lead to fewer detected jumps.}

\item{adjustment_method}{Either \code{"window_median"} (recommended) or
\code{"point_diff"}.}

\item{adjust_window}{Integer window size used for robust jump-size estimation
when \code{adjustment_method = "window_median"}.}

\item{auto_every_second}{Logical. If \code{TRUE}, keeps every second detected
changepoint in automatic mode to mimic the legacy behavior.}

\item{set_jump_point_na}{Logical. If \code{TRUE}, sets the exact jump point to
\code{NA} after correction.}
}
\value{
A dataframe containing jump-free dendrometer data.
}
\description{
Dendrometers generally have limited memory capacity beyond which
they stop recording. To keep the measurement ongoing, they should be adjusted
periodically, which can cause positive or negative jumps in the data. This
function locates these artefacts and adjusts them automatically. Unlike
\code{\link{ i.jump.locator }}, it can handle datasets with more than one
dendrometer.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(nepa)

# Manual detection
jump_free_nepa <- jump.locator(
  df = nepa,
  detection_method = "manual",
  manual_threshold = 1
)

# Automatic detection with cpt.mean() and penalty = "Manual"
jump_free_nepa2 <- jump.locator(
  df = nepa,
  detection_method = "auto",
  auto_method_penalty = 10
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase.sc.R
\name{phase.sc}
\alias{phase.sc}
\title{Application of the stem-cycle approach to classify dendrometer phases}
\usage{
phase.sc(df, TreeNum, smoothing = NULL)
}
\arguments{
\item{df}{A data frame with the first column containing date-time in the format
\code{"yyyy-mm-dd HH:MM:SS"} (or convertible to \code{POSIXct}), followed by one or more
dendrometer measurement columns (mm).}

\item{TreeNum}{Integer. The index of the dendrometer column to analyze.
For example, \code{TreeNum = 1} selects the first dendrometer series after the time column.}

\item{smoothing}{Numeric or \code{NULL}. Length of the smoothing window in hours
(1–24). If \code{NULL} (default), no smoothing is applied. If provided, the
dendrometer series is smoothed using \code{smooth_dm(method = "median_mean")}
prior to phase classification.}
}
\value{
A list of class \code{"SC_output"} containing:
\describe{
  \item{SC_cycle}{A tibble with one row per contiguous phase, including:
    \itemize{
      \item \code{Phases} – Phase type (1 = Shrinkage, 2 = Expansion, 3 = Increment)
      \item \code{Start}, \code{End} – \code{POSIXct} start and end time of the phase
      \item \code{Duration_h}, \code{Duration_m} – Phase duration (hours, minutes)
      \item \code{Magnitude} – Change in dendrometer value during the phase (measurement unit)
      \item \code{rate} – Rate of change expressed in (\code{Magnitude*1000/Duration_h}) (eg. \eqn{\mu}m/hour)
      \item \code{DOY} – Day-of-year at phase start
    }}
  \item{SC_phase}{A tibble of point-level values including:
    \itemize{
      \item \code{TIME} – timestamp
      \item \code{dm} – dendrometer measurement
      \item \code{Phases} – phase assignment for each timestamp
    }}
}
}
\description{
Implements the \strong{stem-cycle approach} (Downes et al., 1999; Deslauriers et al., 2011)
to divide a dendrometer time series into three biologically meaningful phases:
\enumerate{
  \item \strong{Shrinkage} (phase = 1): the dendrometer reading decreases
        compared to the previous reading.
  \item \strong{Expansion} (phase = 2): the dendrometer reading increases
        compared to the previous reading, but remains below the previous maximum.
  \item \strong{Increment} (phase = 3): the dendrometer reading exceeds
        the previous maximum (irreversible stem growth).
}

For each contiguous phase, the function calculates duration, magnitude, rate,
and assigns day-of-year information. Optionally, the dendrometer series may be
smoothed before phase calculation to reduce noise and spurious phase changes.
}
\details{
Classification uses the cumulative maximum of the dendrometer series:
\itemize{
  \item If the cumulative maximum increases, the phase is labeled \emph{Increment} (3).
  \item If the cumulative maximum is constant and the first difference is positive,
        the phase is \emph{Expansion} (2).
  \item If the cumulative maximum is constant and the first difference is negative,
        the phase is \emph{Shrinkage} (1).
}

The function returns both phase-level summaries (\code{SC_cycle}) and point-level
labels (\code{SC_phase}). Optional smoothing uses \code{smooth_dm} with
\code{method = "median_mean"} and a window length between 1–24 hours.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(gf_nepa17)

# Apply stem-cycle approach without smoothing
sc1 <- phase.sc(df = gf_nepa17, TreeNum = 1)
head(sc1$SC_cycle, 5)
head(sc1$SC_phase, 5)

# Apply with 12-hour smoothing to reduce noise
sc2 <- phase.sc(df = gf_nepa17, TreeNum = 1, smoothing = 12)
head(sc2$SC_cycle, 5)
}

}
\references{
Deslauriers A, Rossi S, Turcotte A, Morin H, Krause C (2011)
A three-step procedure in SAS to analyze the time series from automatic dendrometers.
\emph{Dendrochronologia} 29:151–161. \doi{10.1016/j.dendro.2011.01.008}

Downes G, Beadle C, Worledge D (1999)
Daily stem growth patterns in irrigated \emph{Eucalyptus globulus} and \emph{E. nitens} in relation to climate.
\emph{Trees} 14:102–111. \doi{10.1007/PL00009752}
}
\seealso{
\code{\link{phase.zg}} for the zero-growth approach;
\code{\link{smooth_dm}} for smoothing dendrometer series.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_build_season_table}
\alias{dmgf_build_season_table}
\title{Build vegetation-season table}
\usage{
dmgf_build_season_table(
  time_vec,
  site_mode = c("NH", "SH", "CS"),
  custom_veg_season = c(275, 274)
)
}
\arguments{
\item{time_vec}{Time vector.}

\item{site_mode}{One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.}

\item{custom_veg_season}{Custom vegetation season DOY range.}
}
\value{
Tibble with season labels and season timing.
}
\description{
Build vegetation-season table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_empty_model_params}
\alias{dmgf_empty_model_params}
\title{Empty model-parameter template}
\usage{
dmgf_empty_model_params()
}
\value{
Named list of empty model parameters.
}
\description{
Empty model-parameter template
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.fit.gompertz.R
\name{dm.fit.gompertz}
\alias{dm.fit.gompertz}
\title{Fitting gompertz function on annual dendrometer data}
\usage{
dm.fit.gompertz(
  df,
  CalYear,
  TreeNum,
  f_derivative = F,
  start = list(b = 0.5, k = 0.005),
  verbose = FALSE
)
}
\arguments{
\item{df}{dataframe with first column containing date and time in the format \code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.}

\item{CalYear}{numeric for year of calculation. If df has more than one year, assigning CalYear truncates the data of only that year.}

\item{TreeNum}{numerical value indicating the tree to be analysed. E.g. '1' refers to the first dendrometer data column in \emph{df}.}

\item{f_derivative}{logical if yes returns first derivative of gompertz curve.}

\item{start}{A named list of start values for fitting
the Gompertz curve (passed to \code{\link[minpack.lm]{nlsLM}}). Default is
\code{list(b = 0.5, k = 0.005)}.}

\item{verbose}{logical if TRUE also returns the optimized parameters. Default is
FALSE.}
}
\value{
A data frame with the modelled dendrometer series. If \code{verbose = TRUE}, returns a list
 with two data frames. The fitted curve and parameters.
}
\description{
This function modells the annual growth of dendrometer data using gompertz function.
}
\examples{
\donttest{library(dendRoAnalyst)
data(gf_nepa17)
gomp_fitted<-dm.fit.gompertz(df=gf_nepa17, TreeNum = 1, CalYear=2017)
head(gomp_fitted,10)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet.R
\name{plot.dm_wavelet}
\alias{plot.dm_wavelet}
\title{Plot method for wavelet analysis output}
\usage{
\method{plot}{dm_wavelet}(
  x,
  y = NULL,
  series = NULL,
  type = c("power", "average", "series"),
  facet = TRUE,
  log_period = TRUE,
  log_power = TRUE,
  clip_quantile = c(0.01, 0.99),
  show_sig = TRUE,
  show_coi = TRUE,
  coi_fill = "white",
  coi_alpha = 0.45,
  siglvl = 0.05,
  sig_color = "black",
  sig_size = 0.4,
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_wavelet"} returned by
\code{dm_wavelet()}.}

\item{y}{Unused.}

\item{series}{Optional character vector giving one or more series names to
plot. If \code{NULL}, the first available series is used.}

\item{type}{Character string specifying the plot type. One of:
\describe{
  \item{`"power"`}{Wavelet power spectrum as a raster image.}
  \item{`"average"`}{Average wavelet power spectrum across the full time series.}
  \item{`"series"`}{Original analyzed series over time.}
}}

\item{facet}{Logical. If \code{TRUE} and more than one series is selected,
panels are faceted by series.}

\item{log_period}{Logical. If \code{TRUE}, the period axis is shown on a
log10 scale where applicable.}

\item{log_power}{Logical. If \code{TRUE} and \code{type = "power"}, the
plotted wavelet power is transformed using \code{log10(power + eps)} to
improve contrast.}

\item{clip_quantile}{Optional numeric vector of length 2 giving lower and
upper quantiles used to clip wavelet power before plotting, for example
\code{c(0.01, 0.99)}. This is useful when a few extreme values dominate the
colour scale. Use \code{NULL} to disable clipping.}

\item{show_sig}{Logical. If \code{TRUE}, overlays significance information
when available. For \code{type = "average"}, significant periods are marked
as points. For \code{type = "power"}, significant regions are shown as
contour lines.}

\item{show_coi}{Logical. If \code{TRUE} and \code{type = "power"}, the cone of
influence (COI) is added as a shaded overlay.}

\item{coi_fill}{Fill colour used for the cone of influence shading.}

\item{coi_alpha}{Numeric transparency of the cone of influence shading.}

\item{siglvl}{Numeric significance threshold between 0 and 1. Default is
\code{0.05}.}

\item{sig_color}{Colour used for significance overlays.}

\item{sig_size}{Line width or point size used for significance overlays.}

\item{main}{Optional plot title. If \code{NULL}, a default title is used.}

\item{...}{Further arguments passed to or from other methods. Currently unused.}
}
\value{
A \code{ggplot2} object.
}
\description{
S3 plotting method for objects returned by \code{dm_wavelet()}.

The function provides three main visualization types:
\itemize{
  \item \code{"series"}: plots the analyzed input dendrometer series over time,
  \item \code{"average"}: plots the average wavelet power spectrum against period,
  \item \code{"power"}: plots the full wavelet power spectrum as a time-period image.
}

Periods are shown in \strong{hours}, regardless of the original temporal
resolution of the input series. For power plots, the x-axis shows the actual
time series timestamps and the y-axis shows the wavelet period in hours.
}
\details{
For \code{type = "power"}, the function uses the wavelet power matrix stored
in the \code{"dm_wavelet"} object and converts the Fourier periods to hours.
The power plot may also show:
\itemize{
  \item significance contours, when p-values are available,
  \item the cone of influence (COI), when COI information is available.
}

For \code{type = "average"}, the function plots the average wavelet power
spectrum and may optionally indicate significant periods when average-spectrum
p-values are available.

For \code{type = "series"}, the original analyzed series are plotted without
any wavelet transformation.
}
\examples{
\donttest{
wv <- dm_wavelet(
  x = gf_nepa17,
  TreeNum = 1:2,
  source = "raw",
  make_pval = TRUE,
  verbose = FALSE
)

# original series
plot(wv, type = "series")

# average wavelet power
plot(wv, type = "average")

# full power spectrum
plot(wv, type = "power")

# one selected series
plot(wv, series = names(wv$results)[1], type = "power")

# stronger contrast in the power plot
plot(
  wv,
  type = "power",
  log_power = TRUE,
  clip_quantile = c(0.05, 0.95)
)
}

}
\seealso{
\code{\link{dm_wavelet}}, \code{\link[ggplot2]{ggplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_resolve_facet_scales}
\alias{dmgp_resolve_facet_scales}
\title{Resolve facet scales for curve plots}
\usage{
dmgp_resolve_facet_scales(
  x_var,
  facet_by = c("default", "tree", "year", "none")
)
}
\arguments{
\item{x_var}{Name of the x-axis variable.}

\item{facet_by}{Faceting mode.}
}
\value{
A character string accepted by [ggplot2::facet_wrap()] argument
  \code{scales}.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to choose facet scale behavior.

For date-based plots faceted by year or by individual series-fit panels, the
x-axis is freed so that each year or season is shown only over its own
temporal window. This prevents yearly fitted curves from being plotted inside
the full dendrometer record window when several seasons are present.

For season-day and DOY plots, the x-axis remains shared so curves remain
directly comparable across facets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events_phase_plots.R
\name{dm_event_climate_summary}
\alias{dm_event_climate_summary}
\title{Summarize event-based climate results}
\usage{
dm_event_climate_summary(
  event_data,
  climate_var,
  group_var = c("Phase", "event_type"),
  by = c("none", "month", "month_of_year", "year"),
  Year = NULL,
  DOY = NULL
)
}
\arguments{
\item{event_data}{Output of \code{dm_event_climate()}.}

\item{climate_var}{Name of the climate-derived column to summarize.}

\item{group_var}{Grouping variable. One of \code{"Phase"} or \code{"event_type"}.}

\item{by}{One of \code{"none"}, \code{"month"}, \code{"month_of_year"}, or \code{"year"}.}

\item{Year}{Optional numeric year or vector of years for filtering.}

\item{DOY}{Optional numeric vector of length 2 for filtering.}
}
\value{
A tibble of summary statistics.
}
\description{
Summarizes a climate variable extracted by \code{dm_event_climate()} by
phase, event type, month, or month-of-year.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
evt_zg <- dm_event_climate(
  zg,
  ktm_clim_hourly,
  event = "phase_start",
  windows = c(0, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)

smry <- dm_event_climate_summary(
  evt_zg,
  climate_var = "VPD_mean_prev_6h",
  group_var = "Phase",
  by = "month_of_year"
)

head(smry)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{plot.dm_na_interpolation}
\alias{plot.dm_na_interpolation}
\title{Plot method for dendrometer NA interpolation results}
\usage{
\method{plot}{dm_na_interpolation}(
  x,
  type = NULL,
  series = NULL,
  metric = "MdAPE",
  original = NULL,
  start = NULL,
  end = NULL,
  free_y = TRUE,
  ncol = 1,
  facet = TRUE,
  empty_gaps = c("plot", "error"),
  ...
)
}
\arguments{
\item{x}{Object returned by \code{dm.na.interpolation()}.}

\item{type}{Character. Plot type:
\code{"gaps"}, \code{"interpolation"}, or \code{"assessment"}.
If omitted, the default is:
\itemize{
  \item \code{"assessment"} when assessment exists,
  \item \code{"interpolation"} when filled data exist,
  \item \code{"gaps"} when gaps exist but data were not filled,
  \item otherwise \code{"interpolation"}.
}}

\item{series}{Optional character vector of series names to plot.}

\item{metric}{Character assessment metric used when \code{type = "assessment"}.}

\item{original}{Optional original input data frame used before interpolation.}

\item{start}{Optional start datetime for subsetting.}

\item{end}{Optional end datetime for subsetting.}

\item{free_y}{Logical. If \code{TRUE}, facets use free y scales.}

\item{ncol}{Integer. Number of facet columns.}

\item{facet}{Logical. If \code{TRUE}, facet assessment plots by series.}

\item{empty_gaps}{Character. Behavior when \code{type = "gaps"} but no gaps are
present: \code{"plot"} for an informative empty plot, or \code{"error"}.}

\item{...}{Further arguments.}
}
\value{
A \code{ggplot2} object.
}
\description{
S3 plot method for objects returned by \code{\link{dm.na.interpolation}}.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(nepa17)

## No gaps: defaults to time-series plot
#res0 <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#plot(res0)
#plot(res0, type = "gaps")
#plot(res0, type = "gaps", empty_gaps = "error")

## Filled data
#res1 <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "spline"
#)
#plot(res1)
#plot(res1, type = "gaps")
#plot(res1, type = "interpolation", original = nepa17[1:1000, ])

## Assessed data
#res2 <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "seasonal",
#  assess = TRUE
#)
#plot(res2)
#plot(res2, type = "assessment", metric = "MdAPE")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{clim.twd.stats}
\alias{clim.twd.stats}
\title{Calculate tree-, species-, or site-level statistics from clim.twd output}
\usage{
clim.twd.stats(
  x,
  tree_info = NULL,
  response = c("adverse_change", "normal_change", "full_period_change",
    "continuous_full_period_change"),
  group_by = c("tree", "species", "site", "species_site"),
  center = c("mean", "median"),
  conf_level = 0.95,
  months = NULL,
  years = NULL,
  doy_window = NULL,
  year_window = NULL,
  include_tree_series = TRUE
)
}
\arguments{
\item{x}{An object of class \code{"clim_twd_output"} returned by
\code{clim.twd()}.}

\item{tree_info}{Optional metadata table describing trees. It must contain a
column named \code{tree}, and may additionally contain \code{species} and
\code{site}. A named character vector is also accepted for species-only
mapping, where names are trees and values are species.}

\item{response}{Character. Which response table from \code{clim.twd()} should
be summarized:
\describe{
  \item{`"adverse_change"`}{Only the adverse phase.}
  \item{`"normal_change"`}{Only the following normal phase.}
  \item{`"full_period_change"`}{Adverse + following normal phase, reset to 0
  at each adverse start.}
  \item{`"continuous_full_period_change"`}{Adverse + following normal phase,
  continuing from the previous full period instead of resetting to 0.}
}}

\item{group_by}{Character. Grouping level for trajectory summaries. One of
\code{"tree"}, \code{"species"}, \code{"site"}, or \code{"species_site"}.}

\item{center}{Character. Central tendency used for grouped trajectories and
grouped period metrics. One of \code{"mean"} or \code{"median"}.}

\item{conf_level}{Numeric confidence/limit level. Default is \code{0.95}.
Empirical limits are returned as the lower and upper quantiles corresponding
to this level.}

\item{months}{Optional numeric vector of months (1--12). Only IDs whose
adverse-phase start falls in these months are retained.}

\item{years}{Optional numeric vector of years. Only IDs whose adverse-phase
start year falls in these years are retained.}

\item{doy_window}{Optional numeric vector of length 2 defining the allowed
day-of-year window for adverse-phase start. Wrapped windows are supported,
for example \code{c(300, 50)}.}

\item{year_window}{Optional numeric vector of length 2 defining the allowed
year range for adverse-phase start.}

\item{include_tree_series}{Logical. If \code{TRUE}, the filtered long-format
tree-level data are stored in the output.}
}
\value{
A list of class \code{"clim_twd_stats"} with:
\describe{
  \item{trajectory_summary}{Grouped time-series summary for each ID and date,
  including central tendency, SD bands, and empirical 95\% limits.}
  \item{id_summary}{Grouped per-ID summary derived from
  \code{x$period_info}.}
  \item{selected_ids}{IDs retained after temporal filtering.}
  \item{phase_table}{Filtered phase table.}
  \item{tree_info}{Metadata table used for grouping.}
  \item{long_data}{Filtered long-format tree-level response table, if
  \code{include_tree_series = TRUE}.}
  \item{settings}{List of settings used to generate the summaries.}
}
}
\description{
Calculates grouped statistics from the output of \code{clim.twd()}.

The function can summarize relative dendrometer change trajectories and
event-level metrics:
\itemize{
  \item by tree,
  \item by species,
  \item by site,
  \item or by species within site.
}

It can also restrict IDs to specific months, years, or custom day-of-year
windows based on the adverse-phase start date of each ID.
}
\details{
\strong{Temporal filtering} is based on the \code{adverse_start} date in
\code{x$phase_table}. Multiple filters are combined, so for example users can
simultaneously restrict to:
\itemize{
  \item specific months,
  \item specific years,
  \item a day-of-year window,
  \item and a year range.
}

If \code{group_by = "species"}, \code{"site"}, or \code{"species_site"},
\code{tree_info} must provide the required columns.
}
\examples{
\donttest{
rel_out <- clim.twd(
  df = gf_nepa17,
  Clim = ktm_rain17,
  thresholdClim = "<10",
  thresholdDays = ">5",
  showPlot = FALSE
)

# tree-level statistics
st1 <- clim.twd.stats(
  rel_out,
  response = "full_period_change",
  group_by = "tree"
)

summary(st1)
plot(st1, type = "trajectory")

# species metadata
tree_info <- data.frame(
  tree = c("T2", "T3"),
  species = c("Pinus", "Pinus"),
  site = c("Ktm", "Ktm")
)

# species-level summaries restricted to IDs starting in months 6 to 8
st2 <- clim.twd.stats(
  rel_out,
  tree_info = tree_info,
  response = "full_period_change",
  group_by = "species",
  center = "median",
  months = 6:8
)

summary(st2)
plot(st2, type = "trajectory", band = "limit95")

# species within site
st3 <- clim.twd.stats(
  rel_out,
  tree_info = tree_info,
  response = "continuous_full_period_change",
  group_by = "species_site",
  doy_window = c(150, 250),
  year_window = c(2017, 2018)
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mov.cor.dm.R
\name{mov.cor.dm}
\alias{mov.cor.dm}
\title{Running correlation between dendrometer data and climate}
\usage{
mov.cor.dm(
  df,
  Clim,
  TreeNum,
  win_size,
  cor_method = c("pearson", "kendall", "spearman"),
  boot = FALSE,
  R = 1000,
  boot.ci = 0.05,
  set_seed = 1,
  dm_stat = c("mean", "min", "max", "median", "amplitude", "change"),
  clim_vars = NULL,
  lag_days = 0,
  accum_days = 1,
  clim_fun = "raw",
  min_complete = NULL,
  p_adjust_method = "BH"
)
}
\arguments{
\item{df}{A data frame with the first column containing date-time in the format
\code{yyyy-mm-dd HH:MM:SS} (or convertible to \code{POSIXct}) and dendrometer
series in the following columns.}

\item{Clim}{A data frame with the first column containing daily date
(\code{yyyy-mm-dd}, \code{Date}, or convertible) and one or more climate
variables in the following columns.}

\item{TreeNum}{Integer indicating the dendrometer series to analyze.}

\item{win_size}{Integer giving the running window size in days. Minimum is 18.}

\item{cor_method}{Correlation method: one of \code{"pearson"},
\code{"kendall"}, or \code{"spearman"}.}

\item{boot}{Logical. If \code{TRUE}, bootstrap confidence intervals are
computed for each running correlation.}

\item{R}{Integer number of bootstrap iterations.}

\item{boot.ci}{Numeric confidence level selector: one of \code{0.01},
\code{0.05} (default), or \code{0.1}.}

\item{set_seed}{Integer seed for reproducibility of bootstrap results.}

\item{dm_stat}{Daily dendrometer statistic used for correlation. One of
\code{"mean"}, \code{"min"}, \code{"max"}, \code{"median"},
\code{"amplitude"}, or \code{"change"}.}

\item{clim_vars}{Optional character vector of climate variables to analyze.
If \code{NULL}, all numeric climate variables are used.}

\item{lag_days}{Climate lag in days. Can be:
\itemize{
  \item one value for all selected climate variables
  \item one value per selected climate variable
  \item a named numeric vector keyed by climate variable name
}}

\item{accum_days}{Antecedent window length in days for climate transformation.
Can be scalar, per-variable, or a named numeric vector.}

\item{clim_fun}{Climate transformation over the antecedent window. One of
\code{"raw"}, \code{"mean"}, \code{"sum"}, \code{"max"},
\code{"min"}, or \code{"median"}. Can be scalar, per-variable, or a named
character vector.}

\item{min_complete}{Minimum number of complete paired observations required in
a running window to calculate correlation. If \code{NULL}, defaults to
\code{max(3, ceiling(0.8 * win_size))}.}

\item{p_adjust_method}{Method for p-value adjustment in the non-bootstrap
output. Passed to \code{p.adjust()}. Use \code{"none"} to skip adjustment.}
}
\value{
A list with class \code{"mov_cor_dm"} (and \code{"mov_cor_dm_boot"} if
bootstrapped) containing:
\itemize{
  \item \code{results}: named list of tibbles, one per climate variable
  \item \code{metadata}: analysis metadata
  \item \code{call}: the matched function call
}
}
\description{
Calculates running correlations between a selected daily dendrometer summary
and one or more climate variables. The user can select the daily dendrometer
statistic, correlation method, optional bootstrap confidence intervals, and
lagged / antecedent climate transformations.
}
\details{
The dendrometer series is first aggregated to daily resolution. The daily
dendrometer statistic used for correlation is controlled by \code{dm_stat}:
\itemize{
  \item \code{"mean"}: daily mean dendrometer value
  \item \code{"min"}: daily minimum
  \item \code{"max"}: daily maximum
  \item \code{"median"}: daily median
  \item \code{"amplitude"}: daily amplitude (\code{max - min})
  \item \code{"change"}: day-to-day change in the daily mean
}

Users can choose the climate variables to analyze via \code{clim_vars}.
Climate transformation settings can be given as:
\itemize{
  \item a single value applied to all selected climate variables
  \item an unnamed vector with one value per selected climate variable
  \item a named vector mapping each selected climate variable to its own setting
}

This applies to \code{clim_fun}, \code{lag_days}, and \code{accum_days}.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(gf_nepa17)
data(ktm_rain17)

# one common climate transformation for all selected variables
out_corr <- mov.cor.dm(
  df = gf_nepa17,
  Clim = ktm_rain17,
  TreeNum = 1,
  win_size = 21,
  clim_fun = "raw"
)
print(out_corr)
summary(out_corr)

# variable-specific climate transformations
out_varfun <- mov.cor.dm(
  df = gf_nepa17,
  Clim = ktm_rain17,
  TreeNum = 1,
  win_size = 21,
  clim_vars = c("rainfall"),
  clim_fun = c(rainfall = "sum"),
  lag_days = c(rainfall = 1),
  accum_days = c(rainfall = 7)
)

# bootstrap confidence intervals
out_boot <- mov.cor.dm(
  df = gf_nepa17,
  Clim = ktm_rain17,
  TreeNum = 1,
  win_size = 21,
  boot = TRUE,
  R = 250
)
summary(out_boot)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_guess_two_pulses}
\alias{dmgfd_guess_two_pulses}
\title{Guess two pulse locations from growth increments}
\usage{
dmgfd_guess_two_pulses(x, y)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Cumulative growth values.}
}
\value{
Numeric vector of two initial pulse locations.
}
\description{
Guess two pulse locations from growth increments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_filter_time_table}
\alias{dmea_filter_time_table}
\title{Filter a table by calendar year and day-of-year}
\usage{
dmea_filter_time_table(dat, time_col, Year = NULL, DOY = NULL, label = "rows")
}
\arguments{
\item{dat}{A data frame or tibble.}

\item{time_col}{Name of the timestamp column.}

\item{Year}{Optional numeric vector of calendar years.}

\item{DOY}{Optional numeric vector of day-of-year values.}

\item{label}{Text used in error messages.}
}
\value{
A filtered tibble.
}
\description{
Filter a table by calendar year and day-of-year
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jump.locator.R
\name{i.jump.locator}
\alias{i.jump.locator}
\title{Removing artefacts due to manual adjustments of dendrometers interactively}
\usage{
i.jump.locator(
  df,
  TreeNum,
  detection_method = c("manual", "auto"),
  manual_threshold = NULL,
  auto_method_penalty = 10,
  adjustment_method = c("window_median", "point_diff"),
  adjust_window = 5,
  plot_window = 20,
  auto_every_second = TRUE,
  set_jump_point_na = FALSE
)
}
\arguments{
\item{df}{Data frame with first column containing date and time in the format
\code{yyyy-mm-dd HH:MM:SS} and the dendrometer data in following columns.}

\item{TreeNum}{Numerical value indicating the tree to be analysed. E.g. \code{1}
refers to the first dendrometer data column in \code{df}.}

\item{detection_method}{Either \code{"manual"} for threshold-based jump
detection or \code{"auto"} for automatic detection using
\code{changepoint::cpt.mean()} with \code{penalty = "Manual"}.}

\item{manual_threshold}{Numeric threshold considered as artefact when
\code{detection_method = "manual"}.}

\item{auto_method_penalty}{Numeric manual penalty value used in
\code{changepoint::cpt.mean()} when \code{detection_method = "auto"}.
Larger values generally lead to fewer detected jumps.}

\item{adjustment_method}{Either \code{"window_median"} (recommended) or
\code{"point_diff"}. The former estimates the jump offset from local medians
before and after the jump, while the latter uses the raw consecutive
difference at the jump.}

\item{adjust_window}{Integer window size used for robust jump-size estimation
when \code{adjustment_method = "window_median"}.}

\item{plot_window}{Integer number of points shown before and after each jump in
the interactive plot.}

\item{auto_every_second}{Logical. If \code{TRUE}, keeps every second detected
changepoint in automatic mode to mimic the legacy behavior.}

\item{set_jump_point_na}{Logical. If \code{TRUE}, sets the exact jump point to
\code{NA} after correction.}
}
\value{
A dataframe containing jump-free dendrometer data.
}
\description{
Dendrometers generally have limited memory capacity beyond which
they stop recording. To keep the measurement ongoing, they should be adjusted
periodically, which can cause positive or negative jumps in the data. This
function locates these artefacts and interactively adjusts them one by one.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{print.summary.dm_growth_fit}
\alias{print.summary.dm_growth_fit}
\title{Print a summary.dm_growth_fit object}
\usage{
\method{print}{summary.dm_growth_fit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.dm_growth_fit"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Prints a formatted summary of a \code{"summary.dm_growth_fit"} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_pulse_window_from_rate}
\alias{dmgfd_pulse_window_from_rate}
\title{Estimate pulse window from fitted growth rate}
\usage{
dmgfd_pulse_window_from_rate(rate, x_pred, threshold, domain_idx)
}
\arguments{
\item{rate}{Fitted growth-rate vector.}

\item{x_pred}{Prediction season-day values.}

\item{threshold}{Pulse-specific rate threshold.}

\item{domain_idx}{Indices belonging to one pulse domain.}
}
\value{
List with start and end season-day.
}
\description{
Estimate pulse window from fitted growth rate
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_growth_rate}
\alias{dmgfd_growth_rate}
\title{Calculate non-negative fitted growth rate}
\usage{
dmgfd_growth_rate(pred)
}
\arguments{
\item{pred}{Fitted cumulative growth.}
}
\value{
Numeric growth-rate vector.
}
\description{
Calculate non-negative fitted growth rate
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_time_filter_index}
\alias{dmea_time_filter_index}
\title{Filter timestamps by calendar year and day-of-year}
\usage{
dmea_time_filter_index(times, Year = NULL, DOY = NULL)
}
\arguments{
\item{times}{A vector of date-time values.}

\item{Year}{Optional numeric vector of calendar years to retain.}

\item{DOY}{Optional numeric vector of length one or two. If length one, only
that day-of-year is retained. If length two, the inclusive DOY range is
retained. If the first value is larger than the second, the range is treated
as a cross-year DOY window, for example \code{DOY = c(300, 60)} keeps
\code{DOY >= 300} or \code{DOY <= 60}.}
}
\value{
A logical vector with the same length as \code{times}.
}
\description{
Internal helper used by the SEA functions to select timestamps by calendar
year and day-of-year.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dm_epoch_extract}
\alias{dm_epoch_extract}
\title{Build an epoch table around event times}
\usage{
dm_epoch_extract(
  events,
  clim,
  vars = "all",
  lag_before = 24,
  lag_after = 24,
  step = 1,
  unit = c("hours", "days", "mins"),
  agg_fun = "mean",
  var_funs = NULL,
  Year = NULL,
  DOY = NULL,
  restrict_null_to_window = TRUE
)
}
\arguments{
\item{events}{Output of [dm_event_times()] or a data frame containing an
\code{event_time} column.}

\item{clim}{Climate data frame. The first column must contain timestamps.}

\item{vars}{Climate variables to use, or \code{"all"}.}

\item{lag_before}{Number of lag steps before the event.}

\item{lag_after}{Number of lag steps after the event.}

\item{step}{Step size in units.}

\item{unit}{One of \code{"hours"}, \code{"days"}, or \code{"mins"}.}

\item{agg_fun}{Aggregation function applied to all climate variables when
\code{var_funs} is not supplied.}

\item{var_funs}{Optional named vector or named list of aggregation functions
for each variable.}

\item{Year}{Optional numeric vector of calendar years to include in the SEA.
Events whose \code{event_time} falls outside these years are removed before
epoch extraction.}

\item{DOY}{Optional numeric vector of length one or two giving the
day-of-year window to include in the SEA. If length one, only that DOY is
retained. If length two, the inclusive range is retained. Cross-year DOY
windows are supported, for example \code{DOY = c(300, 60)}.}

\item{restrict_null_to_window}{Logical. If \code{TRUE}, null anchor times in
[dm_epoch_test()] are sampled only from the same \code{Year}/\code{DOY}
window. The full climate table is still retained for extracting
\code{lag_before} and \code{lag_after} windows. Default is \code{TRUE}.}
}
\value{
An object of class \code{c("dm_epoch", "dm_epoch_extract")}.
}
\description{
Builds a long-format epoch table by extracting climate values before and
after event times.

The optional \code{Year} and \code{DOY} arguments allow the user to restrict
which events are included in the superposed epoch analysis. The full climate
table is retained for lag extraction so that values before and after selected
events are still available.
}
\examples{
\donttest{
# events <- dm_event_times(zg, event = "GRO_start")
# ep <- dm_epoch_extract(
#   events = events,
#   clim = climate,
#   vars = c("Rain", "temp", "vpd"),
#   Year = c(2022, 2023, 2024),
#   DOY = c(100, 300),
#   lag_before = 24,
#   lag_after = 24,
#   unit = "hours"
# )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{plot.dm_growth_fit}
\alias{plot.dm_growth_fit}
\title{Plot dendrometer growth-fit results}
\usage{
\method{plot}{dm_growth_fit}(
  x,
  type = c("fit", "season", "residuals", "timing", "overlay", "parameters"),
  series = NULL,
  fit_id = NULL,
  facet_by = c("default", "tree", "year", "none"),
  ncol = NULL,
  normalize = FALSE,
  x_axis = c("default", "season_day", "doy", "date"),
  observed_source = c("processed", "original_daily"),
  show_observed = TRUE,
  show_fitted = TRUE,
  show_timing = TRUE,
  point_alpha = 0.7,
  line_width = 0.8,
  legend_position = "right",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_growth_fit"} returned by
[dm.growth.fit()] or [dm.growth.fit.double()].}

\item{type}{Character string specifying the plot type. One of
\code{"fit"}, \code{"season"}, \code{"residuals"}, \code{"timing"},
\code{"overlay"}, or \code{"parameters"}.}

\item{series}{Optional filter selecting one or more dendrometer series to
plot. May be a character vector of series names. Default is \code{NULL},
meaning all available series are used.}

\item{fit_id}{Optional filter selecting one or more fit identifiers to plot.
May be a character or numeric vector. Numeric values are coerced internally
to character. Default is \code{NULL}, meaning all fits are used.}

\item{facet_by}{Character string controlling faceting layout. One of
\code{"default"}, \code{"tree"}, \code{"year"}, or \code{"none"}.}

\item{ncol}{Optional integer giving the number of columns in faceted plots.
Passed to [ggplot2::facet_wrap()].}

\item{normalize}{Logical. If \code{TRUE}, observed and fitted values are
divided by the maximum observed or fitted value within each curve.}

\item{x_axis}{Character string controlling the x-axis representation. One of
\code{"default"}, \code{"season_day"}, \code{"doy"}, or \code{"date"}.}

\item{observed_source}{Character string controlling which observed daily
series is plotted against the fitted curve. One of \code{"processed"} or
\code{"original_daily"}.}

\item{show_observed}{Logical. If \code{TRUE}, observed values are shown in
plot types where observed data are relevant.}

\item{show_fitted}{Logical. If \code{TRUE}, fitted values are shown in plot
types where fitted curves are relevant.}

\item{show_timing}{Logical. If \code{TRUE}, timing markers are added to
\code{"fit"} and \code{"season"} plots whenever timing information is
available in \code{x$fit_statistics}.}

\item{point_alpha}{Numeric alpha level used for observed points.}

\item{line_width}{Numeric line width used for fitted curves.}

\item{legend_position}{Character string specifying legend position, passed to
[ggplot2::theme()].}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot} object.
}
\description{
Creates \pkg{ggplot2}-based plots for objects returned by
[dm.growth.fit()] and [dm.growth.fit.double()].

The plotting method supports multiple views of fitted dendrometer growth
curves, including observed versus fitted trajectories, residuals, timing
summaries, overlays of multiple curves, and distributions of fitted model
parameters.

The x-axis can be displayed as vegetation-season day, calendar day-of-year
(DOY), or actual date, depending on the selected \code{x_axis} argument and
the type of plot.

When \code{x_axis = "date"} and faceting separates individual years or
series-year combinations, each facet receives its own x-axis range. This
avoids plotting one yearly fitted curve inside the full dendrometer time
window when several seasons are present in the input object.
}
\details{
The plotting method returns a \code{ggplot} object. The returned plot can be
further modified using normal \pkg{ggplot2} syntax.

Timing markers are taken from the \code{fit_statistics} table inside the
\code{"dm_growth_fit"} object:
\itemize{
  \item \code{growth_start_*} and \code{growth_end_*} represent growing-season
    timing based on cumulative fitted growth.
  \item \code{rate_start_*} and \code{rate_end_*} represent active-growth
    timing based on the fitted growth-rate curve.
  \item For double-growth fits, pulse-specific timing such as
    \code{pulse1_start_*}, \code{pulse2_start_*}, and \code{separator_*} is
    used in \code{type = "timing"} when available.
}

For yearly fits in southern hemisphere or cross-year custom seasons,
calendar timing variables such as \code{growth_start_day} and
\code{rate_start_day} are interpreted as true calendar DOY, while
\code{*_season_day} variables represent day counts relative to vegetation
season start.

In pooled fits, date-based and calendar-DOY timing fields may be unavailable,
because pooled fits are not anchored to a single season start date.
}
\examples{
\donttest{
# fit <- dm.growth.fit(...)
# plot(fit, type = "fit")
# plot(fit, type = "fit", facet_by = "year", x_axis = "date")
# plot(fit, type = "fit", facet_by = "year", x_axis = "season_day")
# plot(fit, type = "residuals", facet_by = "year")
# plot(fit, type = "timing")
# plot(fit, type = "parameters")
}

}
\seealso{
[dm.growth.fit()], [dm.growth.fit.double()], [summary.dm_growth_fit()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro.resample.R
\name{dendro.resample}
\alias{dendro.resample}
\title{Resampling temporal resolution of dendrometer and climate data}
\usage{
dendro.resample(
  df,
  by,
  value = "mean",
  method = c("auto", "aggregate", "interpolate")
)
}
\arguments{
\item{df}{A dataframe with the first column containing date-time values in
the format \code{yyyy-mm-dd HH:MM:SS}.}

\item{by}{A character string defining the target temporal resolution.
Supported values are:
\itemize{
  \item \code{"M"} for minutes
  \item \code{"H"} for hours
  \item \code{"D"} for days
  \item \code{"W"} for weeks
  \item \code{"ME"} for month-end
  \item values such as \code{"10M"}, \code{"30M"}, \code{"5H"},
  \code{"2D"}, or \code{"3W"} for custom multiples
}}

\item{value}{A character string defining the aggregation function.
Supported values are \code{"mean"}, \code{"max"}, \code{"min"},
\code{"sum"}, and \code{"count"}.
The \code{"count"} option returns the number of non-\code{NA} values
within each aggregation interval. It is only valid when
\code{method = "aggregate"}.}

\item{method}{A character string defining how resampling should be performed.
Options are:
\itemize{
  \item \code{"auto"}: automatically aggregates when the requested
  resolution is coarser than the input data, and interpolates when the
  requested resolution is finer
  \item \code{"aggregate"}: always aggregates values within each target
  interval
  \item \code{"interpolate"}: always interpolates to the requested regular
  time grid
}}
}
\value{
A dataframe with resampled data. The first column contains the new
  time stamps and the remaining columns contain the resampled variables.
}
\description{
This function changes the temporal resolution of dendrometer and climate
time series. It supports both aggregation to coarser resolutions and
interpolation to finer resolutions. The target resolution can be defined in
minutes, hours, days, weeks, or month-end frequency. Aggregation can be
performed using \code{mean}, \code{max}, \code{min}, \code{sum}, or
\code{count} (number of non-missing values). When a finer temporal
resolution than the original data is requested, the function can interpolate
the series to the new time step using linear interpolation.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(nepa17)

# Monthly resampling using maximum values
resample_ME <- dendro.resample(df = gf_nepa17, by = "ME", value = "max")
head(resample_ME, 10)

# Daily resampling using mean values
resample_D <- dendro.resample(df = gf_nepa17, by = "D", value = "mean")

# Count non-missing values per day
resample_count <- dendro.resample(df = gf_nepa17, by = "D", value = "count",
                                  method = "aggregate")

# Interpolate to 30-minute resolution
resample_30M <- dendro.resample(df = gf_nepa17, by = "30M", value = "mean",
                                method = "auto")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.dendrometer.R
\name{read.dendrometer}
\alias{read.dendrometer}
\title{Reading dendrometer data}
\usage{
read.dendrometer(
  file,
  sep = NULL,
  dec = NULL,
  datetime_col = 1,
  date_col = NULL,
  time_col = NULL,
  tz = "UTC",
  sheet = NULL,
  range = NULL,
  na = c("", "NA", "NaN", "nan", "null", "NULL", "-9999"),
  assume_midnight = TRUE,
  orders = NULL,
  excel_dates = c("auto", "none", "1900", "1904"),
  drop_dup_times = TRUE,
  detect_resolution = FALSE,
  return_report = FALSE,
  quiet = TRUE
)
}
\arguments{
\item{file}{Path to file (\code{.csv}, \code{.txt}, \code{.tsv}, \code{.xlsx}).}

\item{sep}{Optional delimiter for text files. If \code{NULL}, auto-detect
among comma, semicolon, tab, and pipe.}

\item{dec}{Optional decimal mark for text files. If \code{NULL}, auto-detect
between \code{"."} and \code{","}.}

\item{datetime_col}{Integer or name of the datetime column (default \code{1}).
Ignored if \code{date_col} is provided.}

\item{date_col}{Optional integer or name of a date column.}

\item{time_col}{Optional integer or name of a time column. Used together with
\code{date_col}. If \code{NULL}, only the date column is parsed and
\code{"00:00:00"} can be appended if \code{assume_midnight = TRUE}.}

\item{tz}{Time zone for parsed datetimes (default \code{"UTC"}).}

\item{sheet}{Excel sheet name or index (for \code{.xlsx}; default \code{NULL} = first sheet).}

\item{range}{Excel cell range (optional).}

\item{na}{Character vector of strings to treat as NA.}

\item{assume_midnight}{Logical; if \code{TRUE}, rows with only a date get \code{"00:00:00"}.}

\item{orders}{Optional vector of \pkg{lubridate} orders to try. If \code{NULL},
a comprehensive default set is used.}

\item{excel_dates}{Character. One of \code{"auto"}, \code{"none"}, \code{"1900"},
or \code{"1904"}. Controls handling of numeric Excel serial dates.}

\item{drop_dup_times}{Logical; if \code{TRUE}, drop duplicated timestamps
(keep first) with a warning/message.}

\item{detect_resolution}{Logical; if \code{TRUE}, compute basic time-resolution diagnostics.}

\item{return_report}{Logical; if \code{TRUE}, return a list with
\code{$data} and \code{$report}. If \code{FALSE}, return the tibble only.
In both cases, the report is attached as \code{attr(x, "import_report")}.}

\item{quiet}{Logical; if \code{TRUE}, suppress informational messages.}
}
\value{
If \code{return_report = FALSE}, a tibble with a POSIXct first column and the
remaining data columns unchanged.

If \code{return_report = TRUE}, a list with:
\describe{
  \item{\code{$data}}{The imported tibble.}
  \item{\code{$report}}{A structured import report.}
}
}
\description{
Reads dendrometer data from \code{.csv}, \code{.txt}, \code{.tsv}, or \code{.xlsx} files,
automatically parsing the datetime information and converting it to
\code{\%Y-\%m-\%d \%H:\%M:\%S} in the requested timezone.

Supports:
\itemize{
  \item automatic delimiter detection for text files,
  \item real decimal-mark auto-detection for text files,
  \item Excel serial date support (\code{1900} / \code{1904} systems),
  \item separate \code{date_col} + \code{time_col},
  \item optional import report output,
  \item optional time-resolution diagnostics.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events_phase_plots.R
\name{plot_event_climate_box}
\alias{plot_event_climate_box}
\title{Plot event-based climate distributions}
\usage{
plot_event_climate_box(
  event_data,
  climate_var,
  group_var = c("Phase", "event_type"),
  facet_by = c("none", "month", "month_of_year", "year"),
  Year = NULL,
  DOY = NULL,
  geom = c("boxplot", "violin", "both"),
  add_test = TRUE,
  test_method = c("auto", "anova", "kruskal", "t.test", "wilcox"),
  point_alpha = 0.5
)
}
\arguments{
\item{event_data}{Output of \code{dm_event_climate()}.}

\item{climate_var}{Name of the climate-derived column to plot.}

\item{group_var}{Grouping variable. One of \code{"Phase"} or \code{"event_type"}.}

\item{facet_by}{One of \code{"none"}, \code{"month"}, \code{"month_of_year"}, or \code{"year"}.}

\item{Year}{Optional numeric year or vector of years for filtering.}

\item{DOY}{Optional numeric vector of length 2 for filtering.}

\item{geom}{One of \code{"boxplot"}, \code{"violin"}, or \code{"both"}.}

\item{add_test}{Logical. If \code{TRUE}, adds per-panel test results.}

\item{test_method}{Passed to \code{dm_event_climate_test()}.}

\item{point_alpha}{Point transparency.}
}
\value{
A \code{ggplot2} object.
}
\description{
Draws boxplots or violins of event-based climate variables across phases or
event types, with optional significance annotation.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
evt_zg <- dm_event_climate(
  zg,
  ktm_clim_hourly,
  event = "phase_start",
  windows = c(0, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)

if (any(!is.na(evt_zg$VPD_mean_prev_6h))) {
  plot_event_climate_box(
    evt_zg,
    climate_var = "VPD_mean_prev_6h",
    group_var = "Phase",
    facet_by = "month_of_year",
    geom = "both",
    add_test = TRUE
  )
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_climate_phase.R
\name{dm_plot_climate_compare}
\alias{dm_plot_climate_compare}
\title{Compare multiple climate variables attached to dendrometer outputs}
\usage{
dm_plot_climate_compare(
  x,
  climate_vars = NULL,
  numeric_vars = NULL,
  scale = c("auto", "daily", "cycle", "point"),
  type = c("timeseries", "boxplot", "violin", "both", "cor_heatmap",
    "regression_heatmap"),
  group_var = NULL,
  Year = NULL,
  DOY = NULL,
  box_scales = c("free_y", "fixed"),
  cor_method = c("pearson", "spearman", "kendall"),
  use_pairwise = TRUE,
  regression_stat = c("r.squared", "slope", "p.value"),
  point_alpha = 0.5,
  show_values = FALSE,
  temporal = NULL
)
}
\arguments{
\item{x}{A climate-augmented dendrometer object.}

\item{climate_vars}{Character vector of climate variables to compare. If
\code{NULL}, all detected climate variables at the selected scale are used.}

\item{numeric_vars}{Optional character vector of numeric variables used when
\code{type = "cor_heatmap"} or \code{type = "regression_heatmap"}. If
\code{NULL}, all numeric variables in the selected data are used.}

\item{scale}{Character. Data level to use. One of \code{"auto"},
\code{"daily"}, \code{"cycle"}, or \code{"point"}.}

\item{type}{Character. Plot type. One of \code{"timeseries"},
\code{"boxplot"}, \code{"violin"}, \code{"both"},
\code{"cor_heatmap"}, or \code{"regression_heatmap"}.}

\item{group_var}{Optional character. Grouping variable. If \code{NULL},
\code{Day_status} is used for daily outputs when available, and
\code{Phases} is used for phase outputs.}

\item{Year}{Optional numeric year or vector of years for subsetting.}

\item{DOY}{Optional numeric vector of length 2 giving the day-of-year range.}

\item{box_scales}{Character. Facet scale behavior for grouped plots. One of
\code{"free_y"} or \code{"fixed"}.}

\item{cor_method}{Character. Correlation method. One of \code{"pearson"},
\code{"spearman"}, or \code{"kendall"}.}

\item{use_pairwise}{Logical. If \code{TRUE}, pairwise complete observations
are used in correlations.}

\item{regression_stat}{Character. Statistic shown in regression heatmaps.
One of \code{"r.squared"}, \code{"slope"}, or \code{"p.value"}.}

\item{point_alpha}{Numeric. Alpha value for points.}

\item{show_values}{Logical. If \code{TRUE}, numeric values are printed inside
heatmap tiles.}

\item{temporal}{Deprecated argument kept for compatibility with older
examples. It is ignored.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
Plots and compares multiple climate variables attached to climate-augmented
dendrometer outputs. It works with objects of class \code{daily_output_clim},
\code{ZG_output_clim}, and \code{SC_output_clim}.

Supported plot types include faceted time-series, grouped boxplots,
violin plots, combined violin-boxplots, correlation heatmaps, and
regression heatmaps.
}
\details{
For \code{type = "regression_heatmap"}, pairwise simple linear regressions
are fitted as \code{y ~ x} for every variable combination. The heatmap can
display \eqn{R^2}, slope, or p-value.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
zg_clim <- dm_add_climate(
  zg,
  ktm_clim_hourly,
  scale = "phase",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)

dm_plot_climate_compare(
  zg_clim,
  climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
  scale = "cycle",
  type = "boxplot"
)

plot(
  zg_clim,
  climate_vars = c("temp_mean_phase", "VPD_mean_phase", "RH_mean_phase"),
  scale = "cycle",
  type = "boxplot"
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_mov.cor.R
\name{plot_mov.cor}
\alias{plot_mov.cor}
\title{Backward-compatible wrapper for plotting moving correlation}
\usage{
plot_mov.cor(mov.cor.output, ...)
}
\arguments{
\item{mov.cor.output}{Output of \code{mov.cor.dm()}.}

\item{...}{Passed to \code{plot.mov_cor_dm()}.}
}
\value{
A \code{ggplot2} object.
}
\description{
Wrapper around \code{plot()} for objects returned by \code{mov.cor.dm()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_apply_fun}
\alias{dmea_apply_fun}
\title{Apply an aggregation function}
\usage{
dmea_apply_fun(x, fun)
}
\arguments{
\item{x}{Numeric vector.}

\item{fun}{Function or character string. Supported character values are
\code{"mean"}, \code{"sum"}, \code{"max"}, \code{"min"}, and
\code{"median"}.}
}
\value{
Numeric scalar.
}
\description{
Apply an aggregation function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dm.growth.fit.double}
\alias{dm.growth.fit.double}
\title{Fit bimodal dendrometer growth curves by vegetation season}
\usage{
dm.growth.fit.double(
  df,
  TreeNum = "all",
  method = c("gompertz", "richards"),
  years = "all",
  year_mode = c("yearly", "pooled"),
  fit_GRO = TRUE,
  site_mode = c("NH", "SH", "CS"),
  custom_veg_season = c(275, 274),
  growth_fraction = c(0.1, 0.9),
  min_unique_growth = 10,
  rate_threshold_fraction = 0.1,
  peak_separation_min = 10,
  valley_ratio_max = 0.4,
  min_relative_peak = 0.05,
  fallback_to_single = TRUE,
  fix_a_to_observed_max = FALSE,
  fixed_a_multiplier = 1,
  start_value_double_gompertz_parameters = list(a = NA_real_, k = NA_real_, t0 =
    NA_real_, a2 = NA_real_, k2 = NA_real_, t02 = NA_real_),
  start_value_double_richards_parameters = list(a = NA_real_, k = NA_real_, t0 =
    NA_real_, v = 1, a2 = NA_real_, k2 = NA_real_, t02 = NA_real_, v2 = 1),
  verbose = TRUE
)
}
\arguments{
\item{df}{A data frame or tibble. The first column must be a time stamp
\code{Date}, \code{POSIXct}, or parseable date-time string. Remaining
selected columns must be numeric dendrometer series.}

\item{TreeNum}{Either \code{"all"} to use all dendrometer series, a numeric
vector selecting dendrometer columns by position, or a character vector of
column names.}

\item{method}{Double-curve fitting method. One of \code{"gompertz"} or
\code{"richards"}.}

\item{years}{Either \code{"all"} to fit all available vegetation seasons, or
a character vector of season labels to retain.}

\item{year_mode}{Either \code{"yearly"} to fit one curve per vegetation
season, or \code{"pooled"} to fit one pooled curve across all retained
seasons.}

\item{fit_GRO}{Logical. If \code{TRUE}, processed daily series are converted
to cumulative growth using a cumulative maximum within each vegetation
season.}

\item{site_mode}{Vegetation season definition. One of \code{"NH"},
\code{"SH"}, or \code{"CS"}.}

\item{custom_veg_season}{Numeric vector of length two giving the start and
end day-of-year for custom vegetation seasons in \code{"CS"} mode.}

\item{growth_fraction}{Numeric vector of length two giving the lower and
upper fractions of final fitted seasonal growth used to define overall
growing-season onset and cessation.}

\item{min_unique_growth}{Minimum number of unique non-missing cumulative
growth values required before a fit is attempted.}

\item{rate_threshold_fraction}{Numeric scalar between 0 and 1. Pulse start
and end are defined as the first and last days where fitted growth rate
exceeds this fraction of the pulse-specific peak fitted growth rate.}

\item{peak_separation_min}{Minimum number of days separating the two fitted
derivative peaks required to classify a fit as truly two-pulse.}

\item{valley_ratio_max}{Maximum allowed ratio between the valley rate and the
weaker of the two derivative peaks. Smaller values require a deeper valley
between pulses.}

\item{min_relative_peak}{Minimum relative height, expressed as a fraction of
the global derivative maximum, for a local derivative peak to be considered.}

\item{fallback_to_single}{Logical. If \code{TRUE}, and the fitted double curve
does not show a convincing two-pulse pattern according to derivative-based
criteria, the function refits a corresponding single growth curve
\code{"gompertz"} or \code{"richards"} and returns that fit instead.}

\item{fix_a_to_observed_max}{Logical. If \code{TRUE}, the total asymptote of
the double-growth curve is fixed to the observed maximum cumulative growth
for that series and vegetation season. For double-Gompertz and
double-Richards models, this means \code{a + a2} is fixed, while the
relative contribution of the two pulses is estimated. This is most
appropriate with \code{year_mode = "yearly"}. For
\code{year_mode = "pooled"}, one pooled maximum is used.}

\item{fixed_a_multiplier}{Numeric multiplier applied to the observed maximum
when \code{fix_a_to_observed_max = TRUE}. The default is \code{1}, meaning
the total asymptote is fixed exactly to the observed seasonal maximum.
Values slightly above 1, for example \code{1.01} or \code{1.05}, allow the
fitted asymptote to sit slightly above the observed maximum.}

\item{start_value_double_gompertz_parameters}{Optional named list of starting
values for the double-Gompertz fit. Supported names are \code{a}, \code{k},
\code{t0}, \code{a2}, \code{k2}, and \code{t02}. If
\code{fix_a_to_observed_max = TRUE}, supplied \code{a} and \code{a2} are
used only to initialize the relative pulse contribution.}

\item{start_value_double_richards_parameters}{Optional named list of starting
values for the double-Richards fit. Supported names are \code{a}, \code{k},
\code{t0}, \code{v}, \code{a2}, \code{k2}, \code{t02}, and \code{v2}. If
\code{fix_a_to_observed_max = TRUE}, supplied \code{a} and \code{a2} are
used only to initialize the relative pulse contribution.}

\item{verbose}{Logical. If \code{TRUE}, prints a short completion message.}
}
\value{
An object of class \code{"dm_growth_fit"} with elements:
\describe{
  \item{call}{The matched function call.}
  \item{original_daily_data}{Raw daily dendrometer data after resampling to
    daily maxima and assigning vegetation seasons, but before centering and
    optional cumulative-growth transformation.}
  \item{processed_data}{Daily processed data used for fitting.}
  \item{fitted_data}{Daily fitted values on the full vegetation-season grid.}
  \item{fit_statistics}{Fit-level statistics and estimated timing.}
  \item{fit_parameters}{Fit-level model parameters and convergence
    information.}
  \item{season_table}{Vegetation seasons retained for fitting.}
}
}
\description{
Fits bimodal cumulative growth curves to daily dendrometer series using either
a double-Gompertz or double-Richards model.

Overall growing-season timing is derived from the fitted cumulative-growth
curve using \code{growth_fraction}. Pulse-specific timing is derived from the
derivative of the fitted curve using a pulse-specific relative rate threshold
given by \code{rate_threshold_fraction}.

Pulse-specific timing is returned as \code{NA} when the fitted curve does not
show a convincing two-pulse pattern according to derivative-based criteria.

If \code{fallback_to_single = TRUE} and no convincing two-pulse pattern is
found, the function refits a corresponding single growth curve and returns
that fit instead. In that case, overall season timing is still returned, but
pulse-specific timing remains \code{NA}.

For double-Gompertz and double-Richards fits, the total asymptote can
optionally be fixed to the observed maximum cumulative growth of the
corresponding series and vegetation season. In the double-curve case, this
means \code{a + a2} is fixed, while the relative contribution of the first
and second pulse is estimated.
}
\details{
The second pulse is constrained to occur after the first pulse, which improves
numerical stability and reduces label switching between the two components.

For double-Gompertz and double-Richards models, the total seasonal asymptote
is:

\deqn{a_{total} = a + a2}

When \code{fix_a_to_observed_max = TRUE}, the function fits:

\deqn{a + a2 = max(y_{obs}) \times fixed\_a\_multiplier}

where \eqn{y_{obs}} is the observed cumulative growth of the selected
dendrometer series and vegetation season.

The fitted pulse contributions are still returned as \code{a} and \code{a2},
and their sum should equal \code{fixed_a_value}, apart from small numerical
rounding.

Non-applicable parameters are returned as \code{NA}. For example, \code{b}
and \code{b2} are relevant for double-Gompertz, while \code{v} and \code{v2}
are relevant for double-Richards.
}
\examples{
\donttest{
# Double-Gompertz with total seasonal asymptote fixed
# fit_gomp <- dm.growth.fit.double(
#   df = dendro_data,
#   TreeNum = "all",
#   method = "gompertz",
#   year_mode = "yearly",
#   fit_GRO = TRUE,
#   fix_a_to_observed_max = TRUE
# )

# Double-Richards with the total asymptote set 2 percent above observed max
# fit_rich <- dm.growth.fit.double(
#   df = dendro_data,
#   TreeNum = "all",
#   method = "richards",
#   year_mode = "yearly",
#   fit_GRO = TRUE,
#   fix_a_to_observed_max = TRUE,
#   fixed_a_multiplier = 1.02
# )
}

}
\seealso{
[dm.growth.fit()], [summary.dm_growth_fit()],
  [print.dm_growth_fit()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.interpolation.R
\name{plot.network_interpolation}
\alias{plot.network_interpolation}
\title{Plot method for network interpolation output}
\usage{
\method{plot}{network_interpolation}(
  x,
  type = c("interpolation", "uncertainty", "availability", "summary", "validation",
    "coverage", "compare", "seasonal_error"),
  series = NULL,
  start = NULL,
  end = NULL,
  show_pi = TRUE,
  show_fit = FALSE,
  free_y = TRUE,
  ncol = 1,
  summary_metric = c("n_imputed", "n_remaining_na", "n_missing_input", "mean_pi_width",
    "median_pi_width", "mean_ref_n", "median_ref_n", "n_gap_jumps_removed",
    "max_abs_gap_jump"),
  validation_metric = c("MAE", "MAPE", "MdAPE", "RMSE", "Bias", "Success_rate",
    "Mean_PI_width", "Mean_ref_n", "End_value_abs_diff", "Max_abs_diff_within_gap"),
  seasonal_metric = c("mean_error", "mean_abs_error", "mean_abs_pct_error", "rmse",
    "coverage"),
  compare_colour = c("series", "gap_steps"),
  facet = TRUE,
  empty = c("plot", "error"),
  ...
)
}
\arguments{
\item{x}{Object returned by \code{\link{network.interpolation}}.}

\item{type}{Character. Plot type. One of \code{"interpolation"},
\code{"uncertainty"}, \code{"availability"}, \code{"summary"},
\code{"validation"}, \code{"coverage"}, \code{"compare"}, or
\code{"seasonal_error"}.}

\item{series}{Optional character vector of focal series names to plot.}

\item{start}{Optional start datetime for time-based plots.}

\item{end}{Optional end datetime for time-based plots.}

\item{show_pi}{Logical. If \code{TRUE}, show 95\% prediction intervals in
the interpolation plot where available.}

\item{show_fit}{Logical. If \code{TRUE}, show model-implied fitted values in
the interpolation plot where available.}

\item{free_y}{Logical. If \code{TRUE}, facets use free y scales where
applicable.}

\item{ncol}{Integer. Number of facet columns.}

\item{summary_metric}{Character metric used when
\code{type = "summary"}. One of \code{"n_imputed"},
\code{"n_remaining_na"}, \code{"n_missing_input"},
\code{"mean_pi_width"}, \code{"median_pi_width"}, \code{"mean_ref_n"},
\code{"median_ref_n"}, \code{"n_gap_jumps_removed"}, or
\code{"max_abs_gap_jump"}.}

\item{validation_metric}{Character metric used when
\code{type = "validation"}. One of \code{"MAE"}, \code{"MAPE"},
\code{"MdAPE"}, \code{"RMSE"}, \code{"Bias"},
\code{"Success_rate"}, \code{"Mean_PI_width"}, \code{"Mean_ref_n"},
\code{"End_value_abs_diff"}, or \code{"Max_abs_diff_within_gap"}.}

\item{seasonal_metric}{Character metric used when
\code{type = "seasonal_error"}. One of \code{"mean_error"},
\code{"mean_abs_error"}, \code{"mean_abs_pct_error"}, \code{"rmse"},
or \code{"coverage"}.}

\item{compare_colour}{Character. Colouring used when
\code{type = "compare"}. One of \code{"series"} or
\code{"gap_steps"}.}

\item{facet}{Logical. If \code{TRUE}, use faceting where supported.}

\item{empty}{Character. Behavior when no suitable data are available:
\code{"plot"} returns an informative empty plot and \code{"error"} throws
an error.}

\item{...}{Further arguments passed through the generic.}
}
\value{
A \code{ggplot2} object.
}
\description{
S3 plot method for objects returned by
\code{\link{network.interpolation}}.

Depending on \code{type}, the method can display:
\itemize{
  \item interpolated series through time,
  \item uncertainty width,
  \item reference-network availability,
  \item summary diagnostics,
  \item synthetic-gap validation metrics,
  \item interval coverage,
  \item actual versus interpolated validation points,
  \item seasonal validation diagnostics.
}

Jump-corrected post-gap points are marked in the interpolation plot when
jump correction was enabled in \code{\link{network.interpolation}}.
}
\details{
Plot types:
\describe{
  \item{\code{"interpolation"}}{Shows original and interpolated focal series
  through time. Imputed points are highlighted. If jump correction was
  applied, corrected post-gap points are marked in green.}
  \item{\code{"uncertainty"}}{Shows the width of the prediction interval
  through time.}
  \item{\code{"availability"}}{Shows the number of valid reference sensors
  available at each step.}
  \item{\code{"summary"}}{Shows per-series summary diagnostics produced by
  the interpolation run.}
  \item{\code{"validation"}}{Shows synthetic-gap recovery metrics across
  tested gap lengths.}
  \item{\code{"coverage"}}{Shows how often true values fall inside the
  nominal 95\% prediction interval across gap lengths.}
  \item{\code{"compare"}}{Scatter plot of actual versus interpolated values
  from synthetic-gap validation.}
  \item{\code{"seasonal_error"}}{Shows how validation error varies through
  the season, summarized by day of year.}
}

Validation-based plots require that
\code{network.interpolation(..., assess = TRUE)} was used to create
\code{x}.
}
\examples{
\donttest{
#library(dendRoAnalyst)
#data("gf_nepa17")

#df1 <- gf_nepa17
#df1[40:50, "T2"] <- NA

#ref <- cbind(gf_nepa17, gf_nepa17[, 2:3], gf_nepa17[, 2:3])
#colnames(ref) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")

#out <- network.interpolation(
#  df1, ref,
#  niMethod = "proportional",
#  n_boot = 100,
#  assess = TRUE,
#  assess_lengths_steps = c(1, 2, 4),
#  correct_gap_jumps = TRUE,
#  jump_threshold = 0.05
#)

#plot(out)
#plot(out, type = "uncertainty")
#plot(out, type = "availability")
#plot(out, type = "summary", summary_metric = "n_gap_jumps_removed")
#plot(out, type = "validation", validation_metric = "MAPE")
#plot(out, type = "coverage")
#plot(out, type = "compare")
#plot(out, type = "seasonal_error", seasonal_metric = "mean_abs_error")
}

}
\seealso{
\code{\link{network.interpolation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events_phase_plots.R
\name{dm_event_climate}
\alias{dm_event_climate}
\title{Extract climate at and before phase events}
\usage{
dm_event_climate(
  x,
  clim_df,
  event = c("phase_start", "phase_end", "max_twd"),
  phase = "all",
  windows = c(0, 1, 3, 6, 12, 24, 48),
  mean_vars = NULL,
  max_vars = NULL,
  min_vars = NULL,
  sum_vars = NULL,
  median_vars = NULL,
  instant_vars = NULL,
  instant_match = c("nearest", "previous", "next")
)
}
\arguments{
\item{x}{Output of \code{phase.zg()} or \code{phase.sc()}.}

\item{clim_df}{Subdaily climate input. This can be a standardized climate
object from \code{read.climate()}, a raw climate data frame, or a valid
climate file path.}

\item{event}{One of \code{"phase_start"}, \code{"phase_end"}, or
\code{"max_twd"}.}

\item{phase}{Phase selector. For \code{ZG_output}: \code{"all"},
\code{"TWD"}, or \code{"GRO"}. For \code{SC_output}: \code{"all"},
\code{"Shrinkage"}, \code{"Expansion"}, or \code{"Increment"}.}

\item{windows}{Numeric vector of antecedent windows in hours, e.g.
\code{c(0, 1, 3, 6, 12, 24, 48)}. A value of \code{0} extracts climate at
the event time.}

\item{mean_vars, max_vars, min_vars, sum_vars, median_vars}{Climate variables to
summarize in antecedent windows.}

\item{instant_vars}{Climate variables to extract at the exact event time.
If \code{NULL}, all selected variables are used.}

\item{instant_match}{Matching rule for event-time extraction:
\code{"nearest"}, \code{"previous"}, or \code{"next"}.}
}
\value{
A tibble with one row per event and climate summaries in the
  requested antecedent windows.
}
\description{
Extracts climate at event times and summarizes climate in windows preceding
those events. Event times can be phase starts, phase ends, or
\code{Max.twd.time} for \code{phase.zg()} output.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)

evt_zg <- dm_event_climate(
  zg,
  ktm_clim_hourly,
  event = "phase_start",
  phase = "all",
  windows = c(0, 1, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)

head(evt_zg)

evt_maxtwd <- dm_event_climate(
  zg,
  ktm_clim_hourly,
  event = "max_twd",
  windows = c(0, 1, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)

head(evt_maxtwd)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dendro.truncate.R
\name{dendro.truncate}
\alias{dendro.truncate}
\title{Truncation of dendrometer data}
\usage{
dendro.truncate(df, CalYear, DOY)
}
\arguments{
\item{df}{A data frame where the first column contains date or date-time
values. The first column is renamed to \code{TIME} in the returned data.}

\item{CalYear}{Numeric value or numeric vector of length two giving the
calendar year or start and end calendar years.}

\item{DOY}{Numeric value or numeric vector of length two giving the day of
year or start and end day of year.}
}
\value{
A tibble containing the truncated dendrometer data.
}
\description{
Truncates dendrometer data to a user-defined calendar-year and day-of-year
window.

The first column of \code{df} is treated as the date-time column and is
renamed internally to \code{TIME}. The date-time column may be a
\code{Date}, \code{POSIXct}, or character vector in the format
\code{"yyyy-mm-dd HH:MM:SS"}. Character dates in the format
\code{"yyyy-mm-dd"} are also accepted.

This version supports leap years correctly. For example, \code{DOY = 366}
is valid for leap years such as 2016, 2020, or 2024, but invalid for
non-leap years.
}
\details{
The truncation rules are:
\itemize{
  \item If \code{CalYear} has length one and \code{DOY} has length one,
    data from that single day are returned.
  \item If \code{CalYear} has length one and \code{DOY} has length two,
    data from the first DOY to the second DOY within the same year are
    returned.
  \item If \code{CalYear} has length two and \code{DOY} has length one,
    data from that DOY in the first year to the same DOY in the second year
    are returned.
  \item If \code{CalYear} has length two and \code{DOY} has length two,
    data from the first DOY of the first year to the second DOY of the
    second year are returned.
}

The function uses real calendar dates internally rather than row positions.
Therefore, leap-year DOY 366 is handled correctly.
}
\examples{
\donttest{
library(dendRoAnalyst)
data(nepa)

# Extract data from DOY 20 to 50 in 2017
trunc1 <- dendro.truncate(df = nepa, CalYear = 2017, DOY = c(20, 50))
head(trunc1, 10)

# Leap-year example, if the data contain year 2020
# trunc2 <- dendro.truncate(df = df2020, CalYear = 2020, DOY = c(1, 366))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_empty_model_params}
\alias{dmgfd_empty_model_params}
\title{Empty parameter template for double growth fitting}
\usage{
dmgfd_empty_model_params()
}
\value{
Named list of model and timing parameters.
}
\description{
Empty parameter template for double growth fitting
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_local_maxima}
\alias{dmgfd_local_maxima}
\title{Identify local maxima}
\usage{
dmgfd_local_maxima(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Integer indices of local maxima.
}
\description{
Identify local maxima
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_climate_phase.R
\name{dm_plot_climate}
\alias{dm_plot_climate}
\title{Plot climate attached to dendrometer outputs}
\usage{
dm_plot_climate(
  x,
  climate_var,
  metric_var = NULL,
  scale = c("auto", "daily", "cycle", "point"),
  type = c("timeseries", "boxplot", "violin", "both", "relation"),
  group_var = NULL,
  Year = NULL,
  DOY = NULL,
  point_alpha = 0.6,
  add_smooth = FALSE,
  temporal = NULL
)
}
\arguments{
\item{x}{A climate-augmented dendrometer object.}

\item{climate_var}{Character. Name of the climate variable to plot.}

\item{metric_var}{Optional character. Biological response variable used when
\code{type = "relation"}. If \code{NULL}, a suitable default is selected
when possible.}

\item{scale}{Character. Data level to use. One of \code{"auto"},
\code{"daily"}, \code{"cycle"}, or \code{"point"}.}

\item{type}{Character. Plot type. One of \code{"timeseries"},
\code{"boxplot"}, \code{"violin"}, \code{"both"}, or \code{"relation"}.}

\item{group_var}{Optional character. Grouping variable. If \code{NULL},
\code{Day_status} is used for daily outputs when available, and
\code{Phases} is used for cycle- or point-level phase outputs.}

\item{Year}{Optional numeric year or vector of years for subsetting.}

\item{DOY}{Optional numeric vector of length 2 giving the day-of-year range.}

\item{point_alpha}{Numeric. Alpha value for points.}

\item{add_smooth}{Logical. If \code{TRUE} and \code{type = "relation"},
a linear trend line is added.}

\item{temporal}{Deprecated argument kept for compatibility with older
examples. It is ignored.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
General plotting function for dendrometer outputs with attached climate
information. It works with objects of class \code{daily_output_clim},
\code{ZG_output_clim}, and \code{SC_output_clim}, usually produced by
\code{dm_add_climate()}, \code{dm_join_daily_clim()},
\code{dm_join_phase_clim()}, or \code{dm_join_subdaily_clim()}.

The function can display one selected climate variable through time,
by phase or day status, as boxplots or violin plots, or as a
climate-response relationship.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
zg_clim <- dm_add_climate(
  zg,
  ktm_clim_hourly,
  scale = "phase",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)

dm_plot_climate(
  zg_clim,
  climate_var = "VPD_mean_phase",
  scale = "cycle",
  type = "boxplot"
)

plot(
  zg_clim,
  climate_var = "VPD_mean_phase",
  scale = "cycle",
  type = "boxplot"
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{summary.dm_epoch}
\alias{summary.dm_epoch}
\title{Summarize a dm_epoch object}
\usage{
\method{summary}{dm_epoch}(object, ...)
}
\arguments{
\item{object}{Object of class \code{"dm_epoch"}.}

\item{...}{Unused.}
}
\value{
An object of class \code{"summary.dm_epoch"}.
}
\description{
Summarize a dm_epoch object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twd.maxima.R
\name{twd.maxima}
\alias{twd.maxima}
\title{Locating the maxima of TWD periods}
\usage{
twd.maxima(df, TreeNum, smoothing = 5)
}
\arguments{
\item{df}{data frame with first column containing date and time in the format \code{yyyy-mm-dd HH:MM:SS}. It should contain data with constant temporal resolution for best results.}

\item{TreeNum}{numerical value indicating the tree to be analysed. E.g. '1' refers to the first dendrometer data column in \emph{df}.}

\item{smoothing}{numerical value from 1 to 12 which indicates the length of the smoothing spline, i.e. 1 = 1 hour and 12 = 12 hours. Default is 5.}
}
\value{
A data frame with statistics of maxima in each TWD phase.
}
\description{
This function detects the TWD phases, including their beginning (TWDb), using the phase.zg function. Then it calculates the number, time of occurance (Tm) and value of every local maximum within each TWD phase. In addition it calculates the time difference between 'TWDb' and each 'Tm' within each TWD phase.
}
\examples{
\donttest{library(dendRoAnalyst)
data(gf_nepa17)
df1=gf_nepa17[2500:3500,]
twd_max<-twd.maxima(df=df1, TreeNum=2)
head(twd_max,10)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nepa.R
\docType{data}
\name{nepa}
\alias{nepa}
\title{Dendrometer data from Kathmandu}
\format{
A data frame with 14534 rows and 3 variables:
\describe{
  \item{\code{Time}}{datetime time of data recording}
  \item{\code{T2}}{double reading for first tree}
  \item{\code{T3}}{double reading for second tree}
}
}
\usage{
nepa
}
\description{
Dendrometer data from three Chir pine trees collected in hourly resolution for 2 years.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{plot.dm_epoch}
\alias{plot.dm_epoch}
\title{Plot a dm_epoch object}
\usage{
\method{plot}{dm_epoch}(
  x,
  y = NULL,
  type = c("composite", "heatmap", "difference"),
  variables = NULL,
  facet = TRUE,
  show_ci = TRUE,
  legend_position = "right",
  ...
)
}
\arguments{
\item{x}{Object of class \code{"dm_epoch"}.}

\item{y}{Unused.}

\item{type}{Plot type. One of \code{"composite"}, \code{"heatmap"}, or
\code{"difference"}.}

\item{variables}{Optional climate variables to plot.}

\item{facet}{Logical. If \code{TRUE}, facet by variable.}

\item{show_ci}{Logical. If \code{TRUE} and \code{x} is a
\code{"dm_epoch_test"} object, show the null confidence ribbon.}

\item{legend_position}{Legend position passed to [ggplot2::theme()].}

\item{...}{Unused.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot a dm_epoch object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_infer_logistic_starts}
\alias{dmgf_infer_logistic_starts}
\title{Infer logistic starting values}
\usage{
dmgf_infer_logistic_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial asymptote.}
}
\value{
Named list with \code{a}, \code{k}, and \code{t0}.
}
\description{
Infer logistic starting values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_join_phase_clim.R
\name{dm_join_phase_clim}
\alias{dm_join_phase_clim}
\title{Join climate summaries to dendrometer phase windows}
\usage{
dm_join_phase_clim(
  x,
  clim_df,
  mean_vars = NULL,
  min_vars = NULL,
  max_vars = NULL,
  sum_vars = NULL,
  median_vars = NULL,
  suffix = "_phase"
)
}
\arguments{
\item{x}{Object returned by \code{phase.zg()} or \code{phase.sc()}.}

\item{clim_df}{Climate input. This can be a standardized climate object
returned by \code{read.climate()}, a raw climate data frame, or a valid
climate file path.}

\item{mean_vars}{Character vector of climate variables to summarize by mean
within each phase window.}

\item{min_vars}{Character vector of climate variables to summarize by minimum
within each phase window.}

\item{max_vars}{Character vector of climate variables to summarize by maximum
within each phase window.}

\item{sum_vars}{Character vector of climate variables to summarize by sum
within each phase window.}

\item{median_vars}{Character vector of climate variables to summarize by median
within each phase window.}

\item{suffix}{Character suffix appended to the generated climate summary
columns. Default is \code{"_phase"}.}
}
\value{
The same phase object with climate summaries appended to \code{ZG_cycle} or
\code{SC_cycle}. The returned object has class
\code{c("ZG_output_clim", "ZG_output", ...)} or
\code{c("SC_output_clim", "SC_output", ...)}.
}
\description{
Summarizes climate conditions within each phase window of
\code{phase.zg()} or \code{phase.sc()} output and appends those summaries to
\code{ZG_cycle} or \code{SC_cycle}.

The climate input can be:
\itemize{
  \item the output of \code{read.climate()}
  \item a raw climate data frame
  \item a valid climate file path readable by \code{read.climate()}
}

Climate variables are summarized over each phase interval defined by the
\code{Start} and \code{End} columns of the phase-cycle table.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)

zg_clim <- dm_join_phase_clim(
  zg,
  ktm_clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)

head(zg_clim$ZG_cycle)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_stat_fun}
\alias{dmea_stat_fun}
\title{Composite statistic helper}
\usage{
dmea_stat_fun(x, statistic = c("mean", "median"))
}
\arguments{
\item{x}{Numeric vector.}

\item{statistic}{Statistic. One of \code{"mean"} or \code{"median"}.}
}
\value{
Numeric scalar.
}
\description{
Composite statistic helper
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mov.cor.dm.R
\name{summary.mov_cor_dm}
\alias{summary.mov_cor_dm}
\title{Summary method for running dendrometer-climate correlation objects}
\usage{
\method{summary}{mov_cor_dm}(object, absolute = TRUE, top_n = 5, ...)
}
\arguments{
\item{object}{Object of class \code{"mov_cor_dm"}.}

\item{absolute}{Logical. If \code{TRUE} (default), the strongest correlation is
identified by absolute magnitude.}

\item{top_n}{Integer. Number of top windows to return per climate variable.}

\item{...}{Unused.}
}
\value{
A list of class \code{"summary_mov_cor_dm"} containing:
\itemize{
  \item \code{table}: summary table for each climate variable
  \item \code{top_windows}: top running windows per climate variable
  \item \code{metadata}: metadata from the original object
  \item \code{call}: original function call
}
}
\description{
Summarizes running dendrometer-climate correlations for both bootstrapped and
non-bootstrapped outputs, and appends climate-specific settings to the
summary tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clim.twd.R
\name{clim.twd}
\alias{clim.twd}
\title{Calculate relative dendrometer change during adverse climate periods and the following normal phase}
\usage{
clim.twd(
  df,
  Clim,
  dailyValue = "max",
  thresholdClim = "<10",
  thresholdDays = ">5",
  showPlot = TRUE
)
}
\arguments{
\item{df}{A data frame with the first column containing date-time stamps
(\code{yyyy-mm-dd HH:MM:SS}, convertible to \code{POSIXct}) and subsequent
columns containing dendrometer series.}

\item{Clim}{A data frame with the first column containing dates and the second
column containing the climate variable of interest.}

\item{dailyValue}{Character. Daily aggregation statistic used when resampling
dendrometer data. One of \code{"max"}, \code{"min"}, \code{"mean"}, or
\code{"sum"}. Default is \code{"max"}.}

\item{thresholdClim}{Character string defining the climate threshold.
Supported forms are:
\itemize{
  \item \code{"<5"}, \code{"<=5"}, \code{">5"}, \code{">=5"}, \code{"==5"}
  \item \code{"between(5,10)"}
  \item \code{"5:10"}
}}

\item{thresholdDays}{Character string defining the minimum duration threshold
for consecutive adverse days. Must use a single-threshold form such as
\code{">5"} or \code{">=3"}.}

\item{showPlot}{Logical. If \code{TRUE}, diagnostic plots are produced and the
user can interactively choose IDs and phase type (\code{"adverse"},
\code{"normal"}, or \code{"both"}). Default is \code{TRUE}.}
}
\value{
A list of class \code{"clim_twd_output"} containing:
\describe{
  \item{adverse_change}{A data frame of relative dendrometer change during
  adverse phases only.}
  \item{normal_change}{A data frame of relative dendrometer change during the
  following normal phases only.}
  \item{full_period_change}{A data frame of relative dendrometer change across
  adverse + following normal phase, reset to 0 at each adverse start.}
  \item{continuous_full_period_change}{A data frame similar to
  \code{full_period_change}, but values continue from the end of the previous
  full period instead of resetting to 0.}
  \item{period_info}{A data frame with one row per ID and tree, containing
  adverse and normal phase boundaries, the date and magnitude of maximum
  adverse decline during the adverse phase, lags to first drop below zero, lags
  to maximum adverse decline, and lags to return to zero.}
  \item{phase_table}{A data frame with adverse and normal phase boundaries for
  each ID.}
}
}
\description{
Identifies adverse climate periods from a climate time series and calculates
relative dendrometer change from the start of each adverse phase.

In addition to the adverse phase itself, the function also defines a
\emph{complete period} for each event ID:
\itemize{
  \item the adverse climate phase, and
  \item the following normal climate phase until the next adverse phase starts
  (or until the end of the common series).
}

Relative dendrometer change is always calculated from the first day of the
adverse phase for that ID.

The function returns:
\itemize{
  \item \code{adverse_change}: relative change only during adverse phases,
  \item \code{normal_change}: relative change only during the following normal phases,
  \item \code{full_period_change}: relative change across adverse + following normal phase,
  \item \code{continuous_full_period_change}: same as \code{full_period_change},
  but values do not reset to 0 at the next adverse phase and instead continue
  from the end of the previous full period,
  \item \code{period_info}: per-ID and per-tree summary metrics.
}
}
\details{
The algorithm works as follows:
\enumerate{
  \item Climate data are thresholded to identify adverse days.
  \item Consecutive adverse days are grouped into candidate adverse runs.
  \item Only runs satisfying \code{thresholdDays} are retained as true adverse events.
  \item For each retained adverse event, the following normal period is defined
  as all non-adverse days until the next adverse event starts (or the end of
  the series).
  \item Dendrometer data are resampled to daily resolution using
  \code{\link[dendRoAnalyst:dendro.resample]{dendro.resample}}.
  \item Relative dendrometer change is calculated from the first day of the
  adverse phase for each ID.
}

\strong{Threshold syntax:}
\itemize{
  \item single-threshold forms such as \code{"<5"}, \code{"<=10"},
  \code{">3"}, \code{">=0"}, \code{"==0"},
  \item range forms for \code{thresholdClim} such as \code{"between(5,10)"}
  or \code{"5:10"}, both interpreted inclusively.
}

\code{thresholdDays} must use a single-threshold form such as \code{">5"}.
}
\note{
If the dendrometer and climate datasets have different time spans, analysis is
automatically restricted to their common overlap.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_rain17)

rel_out <- clim.twd(
  df = gf_nepa17,
  Clim = ktm_rain17,
  dailyValue = "max",
  thresholdClim = "<10",
  thresholdDays = ">5",
  showPlot = FALSE
)

head(rel_out$adverse_change)
head(rel_out$normal_change)
head(rel_out$full_period_change)
head(rel_out$continuous_full_period_change)
head(rel_out$period_info)

rel_out2 <- clim.twd(
  df = gf_nepa17,
  Clim = ktm_rain17,
  thresholdClim = "between(5,10)",
  thresholdDays = ">3",
  showPlot = FALSE
)
}

}
\references{
Raffelsbauer V, Spannl S, Peña K, Pucha-Cofrep D, Steppe K, Bräuning A (2019).
Tree Circumference Changes and Species-Specific Growth Recovery After Extreme Dry Events
in a Montane Rainforest in Southern Ecuador.
\emph{Frontiers in Plant Science}, 10, 342.
\doi{10.3389/fpls.2019.00342}
}
\seealso{
\code{\link[dendRoAnalyst:dendro.resample]{dendro.resample}},
\code{\link{phase.zg}},
\code{\link{phase.sc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/climate_functions.R
\name{read.climate}
\alias{read.climate}
\title{Read and standardize climate data for dendrometer analyses}
\usage{
read.climate(
  x,
  time_col = NULL,
  vars = NULL,
  sep = NULL,
  dec = NULL,
  header = TRUE,
  sheet = 1,
  tz = "UTC",
  drop_duplicate_time = TRUE,
  min_time_success = 0.6,
  verbose = TRUE,
  datetime_col = NULL,
  date_col = NULL,
  range = NULL,
  na = c("", "NA", "NaN", "nan", "null", "NULL", "-9999", "-999", "N/A"),
  assume_midnight = TRUE,
  orders = NULL,
  excel_dates = c("auto", "none", "1900", "1904"),
  drop_empty_cols = TRUE,
  trim_names = TRUE,
  detect_resolution = FALSE,
  return_report = FALSE,
  quiet = !verbose
)
}
\arguments{
\item{x}{A data frame or path to a file. Supported file extensions are
csv, txt, tsv, tab, dat, xls, xlsx, rds, rda, and RData.}

\item{time_col}{Backward-compatible explicit time/datetime column name or index.
If date_col is also supplied, this is treated as the time-of-day column.}

\item{vars}{Optional character vector of climate variables to keep.}

\item{sep}{Optional field separator for text files. If NULL, it is auto-detected.}

\item{dec}{Optional decimal mark for text files. If NULL, it is auto-detected.}

\item{header}{Logical; passed to text-file readers.}

\item{sheet}{Sheet name or index for Excel files.}

\item{tz}{Time zone for parsed timestamps.}

\item{drop_duplicate_time}{Logical; if TRUE, duplicated timestamps are removed.}

\item{min_time_success}{Minimum parsing success proportion for automatic time detection.}

\item{verbose}{Logical; print an import summary.}

\item{datetime_col}{Optional explicit datetime column name or index. Prefer this
when the file has one combined timestamp column.}

\item{date_col}{Optional explicit date column name or index. Can be combined with
time_col, or parsed alone with midnight appended when assume_midnight = TRUE.}

\item{range}{Optional Excel cell range.}

\item{na}{Strings to treat as missing values.}

\item{assume_midnight}{Logical; if TRUE, date-only values are assigned 00:00:00.}

\item{orders}{Optional lubridate parse_date_time() orders.}

\item{excel_dates}{One of auto, none, 1900, or 1904.}

\item{drop_empty_cols}{Logical; if TRUE, columns that are completely empty are removed.}

\item{trim_names}{Logical; if TRUE, trim whitespace from column names.}

\item{detect_resolution}{Logical; if TRUE, attach simple time-resolution diagnostics.}

\item{return_report}{Logical; if TRUE, return list(data = ..., report = ...).}

\item{quiet}{Logical; suppress messages. By default this is the inverse of verbose.}
}
\value{
A tibble of class dm_clim with TIME in the first column. An import
  report is attached as attr(x, "import_report").
}
\description{
A robust climate-data reader designed to be as flexible as read.dendrometer().
It accepts data frames and common file formats, auto-detects separators and
decimal marks, parses many datetime formats, supports Excel serial dates,
supports separate date + time columns, detects the time column automatically,
sorts timestamps, removes duplicates, converts numeric-like climate variables,
and returns a standardized tibble with a POSIXct TIME column.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/climate_functions.R
\name{dm_subdaily_clim}
\alias{dm_subdaily_clim}
\title{Subdaily climate features for dendrometer analyses}
\usage{
dm_subdaily_clim(
  clim_df,
  mean_vars = NULL,
  sum_vars = NULL,
  lag_vars = NULL,
  roll_hours = c(3, 6, 24),
  lag_hours = c(1, 3, 6, 24)
)
}
\arguments{
\item{clim_df}{Climate input. This can be:
\itemize{
  \item a standardized object returned by \code{read.climate()}
  \item a raw data frame with a time column in the first column or in a
        column named \code{TIME}
  \item a valid file path readable by \code{read.climate()}
}}

\item{mean_vars}{Variables for rolling means.}

\item{sum_vars}{Variables for rolling sums.}

\item{lag_vars}{Variables for lagged features.}

\item{roll_hours}{Numeric vector of rolling-window sizes in hours.
Fractional values are allowed, e.g. \code{0.5} for 30 minutes.}

\item{lag_hours}{Numeric vector of lag sizes in hours.
Fractional values are allowed, e.g. \code{0.25} for 15 minutes.}
}
\value{
A tibble of class \code{"subdaily_clim"} with timestamp-level climate
features added. The inferred temporal resolution in hours is stored in
\code{attr(x, "resolution_hours")}.
}
\description{
Computes rolling-window and lagged climate features at subdaily resolution
for direct linkage with point-level dendrometer outputs such as
\code{ZG_phase} and \code{SC_phase}.

The input can be a standardized climate object returned by
\code{read.climate()}, a raw data frame, or a valid file path accepted by
\code{read.climate()}.
}
\details{
The function learns the temporal resolution automatically from the median
time step in the \code{TIME} column. It works with hourly as well as
minute-resolution data (for example 60-, 30-, 15-, 10-, or 5-minute data).

Rolling windows and lags are provided in hours and may be fractional:
\itemize{
  \item \code{0.25} = 15 minutes
  \item \code{0.5} = 30 minutes
  \item \code{1} = 1 hour
  \item \code{3} = 3 hours
}

If the user requests a rolling window or lag that is smaller than the
inferred climate resolution, the function stops with an error.

If a requested window is not an exact multiple of the inferred resolution,
it is rounded to the nearest number of time steps and a warning is issued.
}
\examples{
\donttest{
data(ktm_clim_hourly)

clim_sub <- dm_subdaily_clim(
  ktm_clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  sum_vars  = c("prec"),
  lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)

head(clim_sub)
attr(clim_sub, "resolution_hours")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_normalize.R
\name{plot.dm_standardized}
\alias{plot.dm_standardized}
\title{Plot method for standardized dendrometer output}
\usage{
\method{plot}{dm_standardized}(
  x,
  y = NULL,
  trees = "all",
  type = c("series", "seasonal", "heatmap", "boxplot"),
  x_axis = c("time", "doy", "season_doy"),
  facet_by = c("tree", "season_year", "none"),
  legend_by = c("season_year", "tree", "none"),
  in_season_only = TRUE,
  box_group = c("tree", "season_year"),
  alpha = 0.8,
  line_size = 0.45,
  point_size = 1.6,
  ...
)
}
\arguments{
\item{x}{Object of class \code{"dm_standardized"} returned by
\code{dm_standardize()}.}

\item{y}{Unused.}

\item{trees}{Character vector of dendrometer series to plot, or \code{"all"}
for all series.}

\item{type}{Plot type. One of:
\itemize{
  \item \code{"series"}: standardized time series over calendar time
  \item \code{"seasonal"}: standardized seasonal trajectories by seasonal day
  \item \code{"heatmap"}: heatmap of standardized values by seasonal day and seasonal year
  \item \code{"boxplot"}: distribution of standardized values
}}

\item{x_axis}{For \code{type = "series"}, x-axis style:
\code{"time"}, \code{"doy"}, or \code{"season_doy"}.}

\item{facet_by}{Faceting option. One of \code{"tree"}, \code{"season_year"},
or \code{"none"}.}

\item{legend_by}{Legend grouping. One of \code{"tree"}, \code{"season_year"},
or \code{"none"}.}

\item{in_season_only}{Logical. If \code{TRUE}, only observations inside the
defined season are plotted.}

\item{box_group}{For \code{type = "boxplot"}, grouping variable on the x-axis:
\code{"tree"} or \code{"season_year"}.}

\item{alpha}{Line or point transparency.}

\item{line_size}{Line width.}

\item{point_size}{Point size.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
S3 plotting method for objects returned by \code{dm_standardize()}.
}
\examples{
\donttest{
data(gf_nepa17)

out_std <- dm_standardize(
  df = gf_nepa17,
  season_type = "NH",
  method = "robust_amplitude"
)

plot(out_std)
plot(out_std, type = "seasonal")
plot(out_std, type = "seasonal", facet_by = "tree", legend_by = "season_year")
plot(out_std, type = "seasonal", facet_by = "season_year", legend_by = "tree")
plot(out_std, type = "heatmap", facet_by = "tree")
plot(out_std, type = "boxplot", facet_by = "tree", legend_by = "season_year")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_smooth.R
\name{smooth_dm}
\alias{smooth_dm}
\title{Smoothing of Dendrometer Time Series}
\usage{
smooth_dm(
  time,
  dm,
  resolution_min = NULL,
  method = c("median_mean", "pspline", "sg", "ema", "loess"),
  window_hours = 3,
  sg_order = 2,
  ema_alpha = NULL
)
}
\arguments{
\item{time}{A POSIXct vector representing the time column.}

\item{dm}{A numeric vector of dendrometer values corresponding to \code{time}.}

\item{resolution_min}{Integer. The resolution of the time series in minutes. If \code{NULL},
the resolution is auto-detected based on median time difference.}

\item{method}{Smoothing method. One of: \code{"median_mean"}, \code{"pspline"},
\code{"sg"} (Savitzky-Golay), \code{"ema"} (exponential moving average), or \code{"loess"}.}

\item{window_hours}{Numeric. Smoothing window length in hours. Converted to points using resolution.}

\item{sg_order}{Integer. Polynomial order for Savitzky-Golay smoothing. Ignored unless \code{method = "sg"}.}

\item{ema_alpha}{Numeric. Smoothing factor (0–1) for exponential moving average. If \code{NULL},
it is automatically calculated from the window size.}
}
\value{
A numeric vector of the same length as \code{dm} containing the smoothed values.
}
\description{
Applies various smoothing techniques to dendrometer (dm) time series data
using a user-defined or automatically detected temporal resolution. The function
supports several smoothing methods: robust median+mean, penalized spline,
Savitzky-Golay filter, exponential moving average (EMA), and LOESS.
}
\details{
The function is designed to smooth dendrometer time series (e.g., 10–60 min resolution)
while preserving key features such as diurnal fluctuations or long-term growth,
depending on the window size. It auto-detects the resolution (in minutes) if not provided.
}
\examples{
\dontrun{
# Example: Create synthetic dendrometer time series (30-min resolution)
time_seq <- seq.POSIXt(from = as.POSIXct("2023-06-01 00:00:00"),
                       to   = as.POSIXct("2023-06-03 00:00:00"),
                       by   = "30 mins")
set.seed(123)
dm_raw <- cumsum(rnorm(length(time_seq), mean = 0.005, sd = 0.01)) +
          0.1 * sin(2 * pi * as.numeric(difftime(time_seq, min(time_seq), units = "hours")) / 24)

# Median plus moving mean smoothing (default robust filter)
dm_medmean <- smooth_dm(time = time_seq, dm = dm_raw,
                        method = "median_mean", window_hours = 3)

# Penalized spline smoothing
dm_pspline <- smooth_dm(time = time_seq, dm = dm_raw,
                        method = "pspline", window_hours = 6)

# Savitzky-Golay filter (requires 'signal' package)
if (requireNamespace("signal", quietly = TRUE)) {
  dm_sg <- smooth_dm(time = time_seq, dm = dm_raw,
                     method = "sg", window_hours = 2, sg_order = 2)
}

# Exponential moving average smoothing
dm_ema <- smooth_dm(time = time_seq, dm = dm_raw,
                    method = "ema", window_hours = 4)

# LOESS smoothing
dm_loess <- smooth_dm(time = time_seq, dm = dm_raw,
                      method = "loess", window_hours = 3)

# Plot raw and smoothed series
plot(time_seq, dm_raw, type = "l", col = "gray", lwd = 1, main = "Smoothed Dendrometer Series",
     ylab = "DM", xlab = "Time")
lines(time_seq, dm_medmean, col = "blue", lwd = 2)
lines(time_seq, dm_pspline, col = "green", lwd = 2)
lines(time_seq, dm_ema, col = "orange", lwd = 2)
lines(time_seq, dm_loess, col = "purple", lwd = 2)
legend("topright", legend = c("Raw", "Median+Mean", "P-spline", "EMA", "LOESS"),
       col = c("gray", "blue", "green", "orange", "purple"), lwd = 2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{summary.clim_twd_test}
\alias{summary.clim_twd_test}
\title{Summarize clim.twd.test output}
\usage{
\method{summary}{clim_twd_test}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"clim_twd_test"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{"summary.clim_twd_test"}.
}
\description{
Summarize clim.twd.test output
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_fit_one_curve}
\alias{dmgfd_fit_one_curve}
\title{Fit one double growth curve}
\usage{
dmgfd_fit_one_curve(
  x_obs,
  y_obs,
  x_pred,
  season_start_date = as.Date(NA),
  season_length = NA_real_,
  method,
  growth_fraction,
  min_unique_growth,
  rate_threshold_fraction = 0.1,
  peak_separation_min = 10,
  valley_ratio_max = 0.4,
  min_relative_peak = 0.05,
  fallback_to_single = TRUE,
  fix_a_to_observed_max = FALSE,
  fixed_a_multiplier = 1,
  start_value_double_gompertz_parameters,
  start_value_double_richards_parameters
)
}
\arguments{
\item{x_obs}{Observed season-day values.}

\item{y_obs}{Observed cumulative growth values.}

\item{x_pred}{Prediction season-day values.}

\item{season_start_date}{Season start date.}

\item{season_length}{Season length.}

\item{method}{Double-growth method.}

\item{growth_fraction}{Growth fractions used for cumulative timing.}

\item{min_unique_growth}{Minimum number of unique growth values.}

\item{rate_threshold_fraction}{Fraction of peak rate.}

\item{peak_separation_min}{Minimum separation between peaks.}

\item{valley_ratio_max}{Maximum allowed valley-to-peak ratio.}

\item{min_relative_peak}{Minimum relative derivative peak height.}

\item{fallback_to_single}{Logical fallback flag.}

\item{fix_a_to_observed_max}{Logical fixed-asymptote flag.}

\item{fixed_a_multiplier}{Multiplier for fixed asymptote.}

\item{start_value_double_gompertz_parameters}{Double-Gompertz starts.}

\item{start_value_double_richards_parameters}{Double-Richards starts.}
}
\value{
List with fitted predictions and parameters.
}
\description{
Internal helper used by [dm.growth.fit.double()] to fit one double curve for
one series and one season, or one pooled double curve.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_unique_prediction_curve}
\alias{dmgf_unique_prediction_curve}
\title{Reduce prediction curve to unique x values}
\usage{
dmgf_unique_prediction_curve(x_pred, pred)
}
\arguments{
\item{x_pred}{Prediction x values.}

\item{pred}{Predicted y values.}
}
\value{
List with unique \code{x} and mean \code{pred}.
}
\description{
Reduce prediction curve to unique x values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet.R
\name{summary.dm_wavelet}
\alias{summary.dm_wavelet}
\title{Summarize a dm_wavelet object}
\usage{
\method{summary}{dm_wavelet}(object, top_n = 3, ...)
}
\arguments{
\item{object}{An object of class \code{"dm_wavelet"}.}

\item{top_n}{Integer. Number of strongest periods to report per series.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{"summary.dm_wavelet"}.
}
\description{
Summarizes wavelet-analysis results produced by \code{dm_wavelet()}.

For each analyzed series, the summary reports:
\itemize{
  \item the dominant period in hours,
  \item the corresponding maximum average wavelet power,
  \item the analyzed period range in hours,
  \item the detected temporal resolution.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_prepare_curve_groups}
\alias{dmgp_prepare_curve_groups}
\title{Prepare curve grouping and faceting information}
\usage{
dmgp_prepare_curve_groups(dat, facet_by = c("default", "tree", "year", "none"))
}
\arguments{
\item{dat}{Tidy plotting data returned by [dmgp_tidy_fit_data()].}

\item{facet_by}{Character string controlling faceting layout.}
}
\value{
A list containing modified plotting data and faceting settings.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to define panel identifiers,
facet variables, colour groups, and legend titles.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dm.growth.fit}
\alias{dm.growth.fit}
\title{Fit dendrometer growth curves by vegetation season}
\usage{
dm.growth.fit(
  df,
  TreeNum = "all",
  method = c("gam", "gompertz", "logistic", "richards", "loess", "spline"),
  years = "all",
  year_mode = c("yearly", "pooled"),
  fit_GRO = TRUE,
  site_mode = c("NH", "SH", "CS"),
  custom_veg_season = c(275, 274),
  growth_fraction = c(0.1, 0.9),
  min_unique_growth = 10,
  rate_threshold_fraction = 0.1,
  fix_a_to_observed_max = FALSE,
  fixed_a_multiplier = 1,
  start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_),
  start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v =
    1),
  loess_span = 0.2,
  spline_df = 10,
  verbose = TRUE
)
}
\arguments{
\item{df}{A data frame or tibble. The first column must be a timestamp
\code{Date}, \code{POSIXct}, or parseable date-time string. All selected
remaining columns must be numeric dendrometer series.}

\item{TreeNum}{Either \code{"all"} to use all dendrometer series, a numeric
vector selecting dendrometer columns by position, or a character vector
with series names.}

\item{method}{Fitting method. One of \code{"gam"}, \code{"gompertz"},
\code{"logistic"}, \code{"richards"}, \code{"loess"}, or \code{"spline"}.}

\item{years}{Either \code{"all"} to fit all available vegetation seasons, or
a character vector of season labels to retain.}

\item{year_mode}{Either \code{"yearly"} to fit one curve per vegetation
season, or \code{"pooled"} to fit one pooled curve across all retained
seasons.}

\item{fit_GRO}{Logical. If \code{TRUE}, processed daily series are converted
to cumulative growth using a cumulative maximum within each vegetation
season.}

\item{site_mode}{Vegetation season definition. One of \code{"NH"},
\code{"SH"}, or \code{"CS"}.}

\item{custom_veg_season}{Numeric vector of length two giving the start and
end day-of-year for custom vegetation seasons in \code{"CS"} mode.
Cross-year seasons are supported, for example \code{c(275, 274)}.}

\item{growth_fraction}{Numeric vector of length two giving the lower and
upper fractions of final fitted seasonal growth used to define cumulative
growth onset and cessation, for example \code{c(0.1, 0.9)}.}

\item{min_unique_growth}{Minimum number of unique non-missing cumulative
growth values required before a fit is attempted.}

\item{rate_threshold_fraction}{Numeric scalar between 0 and 1. Active-growth
dates are derived from the fitted growth-rate curve as the first and last
days where fitted growth rate exceeds this fraction of the peak fitted
growth rate.}

\item{fix_a_to_observed_max}{Logical. If \code{TRUE} and
\code{method = "gompertz"}, \code{"logistic"}, or \code{"richards"}, the
asymptote parameter \code{a}, representing maximum seasonal growth, is fixed
to the observed maximum cumulative growth of the corresponding series and
vegetation season. This is most appropriate with
\code{year_mode = "yearly"}. For \code{year_mode = "pooled"}, one pooled
maximum is used.}

\item{fixed_a_multiplier}{Numeric multiplier applied to the observed maximum
when \code{fix_a_to_observed_max = TRUE}. The default is \code{1}, meaning
\code{a} is fixed exactly to the observed seasonal maximum. Values slightly
above 1, for example \code{1.01} or \code{1.05}, allow the fitted asymptote
to sit just above the observed maximum.}

\item{start_value_gompertz_parameters}{Optional named list with starting
values for Gompertz fits. Supported names are \code{a}, \code{b}, and
\code{k}. If \code{fix_a_to_observed_max = TRUE}, supplied \code{a} is
ignored.}

\item{start_value_richards_parameters}{Optional named list with starting
values for Richards and logistic fits. Supported names are \code{a},
\code{k}, \code{t0}, and \code{v}. If
\code{fix_a_to_observed_max = TRUE}, supplied \code{a} is ignored.}

\item{loess_span}{Span used when \code{method = "loess"}.}

\item{spline_df}{Degrees of freedom used when \code{method = "spline"}.}

\item{verbose}{Logical. If \code{TRUE}, prints a short completion message.}
}
\value{
An object of class \code{"dm_growth_fit"} with elements:
\describe{
  \item{call}{The matched function call.}
  \item{original_daily_data}{Raw daily dendrometer data after resampling to
    daily maxima and assigning vegetation seasons, but before centering and
    optional cumulative-growth transformation.}
  \item{processed_data}{Daily processed data used for fitting.}
  \item{fitted_data}{Daily fitted values on the full vegetation-season grid.}
  \item{fit_statistics}{Table with fit-level statistics and estimated timing.}
  \item{fit_parameters}{Table with fit-level model parameters and convergence
    information.}
  \item{season_table}{Vegetation seasons retained for fitting.}
}
}
\description{
Fits cumulative growth curves to daily dendrometer series using one of
several supported methods: generalized additive model \code{"gam"},
Gompertz \code{"gompertz"}, logistic \code{"logistic"}, Richards
\code{"richards"}, local regression \code{"loess"}, or smoothing spline
\code{"spline"}.

The function:
\itemize{
  \item resamples dendrometer data to daily maxima using [dendro.resample()],
  \item assigns each daily observation to a vegetation season,
  \item optionally converts daily stem-size values to cumulative growth
    with \code{fit_GRO = TRUE},
  \item fits one curve per series and season with
    \code{year_mode = "yearly"} or one pooled curve across retained seasons
    with \code{year_mode = "pooled"},
  \item estimates growing-season timing from the fitted cumulative-growth
    curve,
  \item additionally identifies active-growth timing from the fitted
    growth-rate curve.
}

For Gompertz, logistic, and Richards fits, the asymptote parameter
\code{a}, representing maximum seasonal growth, can optionally be fixed to
the observed maximum cumulative growth for that series and vegetation season.
This is controlled by \code{fix_a_to_observed_max}.
}
\details{
The first column of \code{df} is treated as the time column and renamed to
\code{"TIME"} internally. If it is not already a date-time object, the
function attempts to parse it using [lubridate::parse_date_time()].

\code{growth_start_*} and \code{growth_end_*} are based on cumulative fitted
growth. \code{rate_start_*} and \code{rate_end_*} are based on the fitted
growth-rate curve.

For Gompertz, logistic, and Richards models, \code{a} is the upper asymptote.
When \code{fix_a_to_observed_max = TRUE}, this parameter is not estimated by
nonlinear least squares. Instead, it is fixed to:

\deqn{a = max(y_{obs}) \times fixed\_a\_multiplier}

where \eqn{y_{obs}} is the observed cumulative growth for that series and
vegetation season.
}
\examples{
\donttest{
# fit <- dm.growth.fit(
#   df = dendro_data,
#   TreeNum = "all",
#   method = "gompertz",
#   year_mode = "yearly",
#   fit_GRO = TRUE,
#   fix_a_to_observed_max = TRUE
# )

# fit2 <- dm.growth.fit(
#   df = dendro_data,
#   TreeNum = "all",
#   method = "richards",
#   year_mode = "yearly",
#   fit_GRO = TRUE,
#   fix_a_to_observed_max = TRUE,
#   fixed_a_multiplier = 1.02
# )
}

}
\seealso{
[summary.dm_growth_fit()], [print.dm_growth_fit()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet.R
\name{dm_wavelet}
\alias{dm_wavelet}
\title{Wavelet analysis of dendrometer series}
\usage{
dm_wavelet(
  x,
  TreeNum = "all",
  source = c("auto", "raw", "detrended", "first_diff"),
  detrended_col = c("detrended_data"),
  na_action = c("interpolate", "fail"),
  loess_span = 0.75,
  dj = 1/20,
  lowerPeriod = NULL,
  upperPeriod = NULL,
  make_pval = TRUE,
  n_sim = 10,
  verbose = TRUE
)
}
\arguments{
\item{x}{Input data. Either:
\describe{
  \item{data.frame}{First column must be time, remaining selected numeric
    columns are dendrometer series.}
  \item{dm_detrended object}{Uses \code{x$detrended_data} as input.}
}}

\item{TreeNum}{Either \code{"all"} to use all available dendrometer series,
a numeric vector selecting series by position, or a character vector of
series names.}

\item{source}{Which series to analyze. One of:
\describe{
  \item{`"auto"`}{Uses raw series for a data frame input and detrended
    series for a \code{dm_detrended} object.}
  \item{`"raw"`}{Use raw dendrometer series from a data frame input.}
  \item{`"detrended"`}{Use detrended series from a \code{dm_detrended} object.}
  \item{`"first_diff"`}{Use first differences of the available input series.}
}}

\item{detrended_col}{For \code{dm_detrended} input, which table to use:
\code{"detrended_data"}.}

\item{na_action}{How to handle missing values after completing the regular time
grid. One of:
\describe{
  \item{`"interpolate"`}{Linearly interpolate internal gaps and carry ends
    forward/backward.}
  \item{`"fail"`}{Stop if missing values are found.}
}}

\item{loess_span}{Smoothing span passed to \code{WaveletComp::analyze.wavelet()}.}

\item{dj}{Frequency resolution parameter passed to
\code{WaveletComp::analyze.wavelet()}.}

\item{lowerPeriod}{Optional lower period bound in native time units of the
detected input resolution. If \code{NULL}, the default from
\code{WaveletComp::analyze.wavelet()} is used.}

\item{upperPeriod}{Optional upper period bound in native time units of the
detected input resolution. If \code{NULL}, the default from
\code{WaveletComp::analyze.wavelet()} is used.}

\item{make_pval}{Logical; if \code{TRUE}, compute Monte Carlo significance.}

\item{n_sim}{Number of simulations used when \code{make_pval = TRUE}.}

\item{verbose}{Logical; if \code{TRUE}, prints a completion message.}
}
\value{
An object of class \code{"dm_wavelet"} with elements:
\describe{
  \item{call}{The matched function call.}
  \item{input_type}{Either \code{"raw"} or \code{"dm_detrended"}.}
  \item{source}{Series source actually used.}
  \item{series}{Character vector of analyzed series names.}
  \item{time_unit}{Detected time unit of the input series.}
  \item{dt}{Detected time step in native units.}
  \item{dt_hours}{Detected time step expressed in hours.}
  \item{resolution_seconds}{Detected time step in seconds.}
  \item{data_used}{Regularized time series used for analysis.}
  \item{results}{Named list of wavelet results, one per series. Each entry
    contains the original \code{WaveletComp} object plus time and
    period-in-hours information.}
  \item{settings}{Analysis settings.}
}
}
\description{
Performs continuous wavelet analysis on dendrometer time series.

The function can work with:
\itemize{
  \item raw dendrometer series from a data frame,
  \item detrended dendrometer series from \code{dm.detrend.fit()} output,
  \item first differences of either raw or detrended series.
}

It automatically:
\itemize{
  \item identifies the temporal resolution of the input series,
  \item regularizes the series to a complete time grid,
  \item optionally interpolates missing values,
  \item performs wavelet analysis separately for each selected tree/series,
  \item converts wavelet periods to hours for summary and plotting.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_doy_to_date}
\alias{dmgfd_doy_to_date}
\title{Convert year and day-of-year to Date}
\usage{
dmgfd_doy_to_date(year, doy)
}
\arguments{
\item{year}{Calendar year.}

\item{doy}{Day of year.}
}
\value{
Date.
}
\description{
Convert year and day-of-year to Date
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_prepare_timing_plot_data}
\alias{dmgp_prepare_timing_plot_data}
\title{Prepare timing-summary plot data}
\usage{
dmgp_prepare_timing_plot_data(
  fit_statistics,
  facet_by = c("default", "tree", "year", "none"),
  x_mode = c("date", "doy", "season_day"),
  series = NULL,
  fit_id = NULL
)
}
\arguments{
\item{fit_statistics}{Fit-statistics table from a \code{"dm_growth_fit"}
object.}

\item{facet_by}{Faceting mode.}

\item{x_mode}{Timing coordinate system. One of \code{"date"}, \code{"doy"},
or \code{"season_day"}.}

\item{series}{Optional series filter.}

\item{fit_id}{Optional fit identifier filter.}
}
\value{
A list of data frames used by the timing plot.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to prepare the line segments,
point markers, and labels used in \code{type = "timing"} plots.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{print.summary.dm_epoch}
\alias{print.summary.dm_epoch}
\title{Print summary of a dm_epoch object}
\usage{
\method{print}{summary.dm_epoch}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"summary.dm_epoch"}.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print summary of a dm_epoch object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_reconstruct.R
\name{summary.dm_wavelet_reconstruct}
\alias{summary.dm_wavelet_reconstruct}
\title{Summarize a dm_wavelet_reconstruct object}
\usage{
\method{summary}{dm_wavelet_reconstruct}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"dm_wavelet_reconstruct"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{"summary.dm_wavelet_reconstruct"} with elements:
\describe{
  \item{overview}{One-row summary of the reconstruction object.}
  \item{series_summary}{Per-series summary table.}
  \item{used_periods}{The table of periods actually used in the reconstruction.}
  \item{selection}{The selection settings used for reconstruction.}
}
}
\description{
Summarizes the output of \code{dm_wavelet_reconstruct()}.

For each reconstructed series, the summary reports:
\itemize{
  \item number of observations,
  \item variance of the original series,
  \item variance of the reconstructed component,
  \item variance of the remaining component,
  \item variance of the returned filtered series,
  \item proportion of original variance represented by reconstructed and filtered series,
  \item correlation and \eqn{R^2} between original and reconstructed,
  \item number and range of periods used in reconstruction.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phase.zg.R
\name{phase.zg}
\alias{phase.zg}
\title{Apply the Zero-Growth (ZG) approach to segment dendrometer data into TWD/GRO phases}
\usage{
phase.zg(df, TreeNum, smoothing = NULL, beta = 0.1)
}
\arguments{
\item{df}{A data frame with the \strong{first column} containing date-time
(character, \code{POSIXct}, or \code{Date} convertible to \code{POSIXct} using
\code{lubridate::ymd_hms}) and the \strong{subsequent columns} containing one or
more dendrometer series (mm) at constant temporal resolution.}

\item{TreeNum}{Integer index of the dendrometer series column to analyze.
For example, \code{TreeNum = 1} refers to \code{df[[2]]} (the first dm column after
the time column).}

\item{smoothing}{\code{NULL} (default) for no smoothing, or a numeric value
between 1 and 24 specifying the smoothing window in \strong{hours} used by
\code{smooth_dm(method = "median_mean")} for phase labeling.}

\item{beta}{Numeric scalar giving the exponent applied to the effective loading
duration in the ABr formula. Defaults to \code{0.1}. Set \code{beta = 0} to
make ABr depend only on \code{max.twd}.}
}
\value{
A named list of class \code{"ZG_output"} with two tibbles:
\itemize{
  \item \code{ZG_cycle}: one row per contiguous phase with columns
        \code{Phases}, \code{Start}, \code{End}, \code{Duration_h}, \code{Magnitude}, \code{rate},
        \code{max.twd}, \code{Max.twd.time}, \code{AUC.load}, \code{AUC.total}, \code{ABr.value},
        \code{Avg.twd}, \code{STD.twd}, \code{DOY}.
  \item \code{ZG_phase}: point-level data with columns
        \code{TIME}, \code{dm}, \code{Phases}, \code{TWD}, \code{GRO}.
}
}
\description{
Implements the Zero-Growth approach (Zweifel et al., 2016) on a single
dendrometer series to (i) classify each timestamp into \strong{tree water deficit}
(TWD; reversible shrinkage/expansion) or \strong{growth} (GRO; irreversible
expansion), (ii) summarize each contiguous phase (start/end, duration,
magnitude, rates, TWD statistics), and (iii) optionally smooth the raw
series prior to phase assignment to reduce spurious phase flips.

The growth line is computed as the running cumulative maximum of the
(raw or smoothed) dendrometer series. TWD is defined as the vertical
distance between the growth line and the observed measurement.
}
\details{
\strong{Phase assignment.}
Let \eqn{y_t} be the dendrometer measurement (raw or smoothed). The growth
line is \eqn{g_t=\max_{s\le t}(y_s)}. Points with \eqn{y_t = g_t} are labeled
\strong{GRO} (phase = 2). Points with \eqn{y_t < g_t} are labeled \strong{TWD}
(phase = 1). Contiguous runs of identical labels are summarized into
individual phases.

\strong{Summaries per phase (ZG_cycle).}
For each phase, the function returns:
\itemize{
  \item \code{Phases} (1 = TWD, 2 = GRO)
  \item \code{Start}, \code{End} (\code{POSIXct})
  \item \code{Duration_h} (hours)
  \item \code{Magnitude} (mm): for GRO only, computed from the \emph{growth line}
        as \eqn{\Delta g = g_{\mathrm{end}}-g_{\mathrm{start}}}; \code{NA} for TWD
  \item \code{rate} (\eqn{\mu}m/h): \code{Magnitude * 1000 / Duration_h} (GRO only)
  \item \code{max.twd} (mm), \code{Max.twd.time} (\code{POSIXct}): peak TWD and its time (TWD only)
  \item \code{Avg.twd} (mm), \code{STD.twd} (mm): mean and SD of TWD within the phase (TWD only)
  \item \code{AUC.load} (mm*h): area under the TWD curve from phase start to \code{Max.twd.time} (TWD only)
  \item \code{AUC.total} (mm*h): area under the full TWD curve across the phase (TWD only)
  \item \code{DOY}: day-of-year of the phase start
  \item \code{ABr.value}: \emph{Absolut Baumreaktion (ABr)} for TWD phases, defined here as
        \deqn{\mathrm{ABr} = \max(\mathrm{TWD}) \times \left(\frac{AUC_{load}}{\max(\mathrm{TWD})}\right)^{\beta}}
        returned as \code{NA} for GRO phases. This metric anchors severity on the event peak while
        allowing the loading duration to contribute with exponent \code{beta}.
}

\strong{Point-level output (ZG_phase).}
The second element returns the input time series augmented with:
\itemize{
  \item \code{Phases} (1/2), \code{TWD} (= \code{GRO - dm}), and \code{GRO} (growth line).
}

\strong{Smoothing (optional).}
If \code{smoothing} is provided (hours), the function uses
\code{smooth_dm(..., method = "median_mean", window_hours = smoothing)} to create a
smoothed series used \emph{only} for phase assignment; all magnitudes and TWD
statistics are still computed against the \emph{original} \code{dm}. This reduces
short-lived phase flips due to noise. Valid range is 1–24 hours.

\strong{Temporal resolution.}
The function auto-detects the median sampling interval (in minutes) to
parameterize smoothing. If multiple distinct intervals are detected, a
warning is emitted (results may be degraded if resolution is inconsistent).

\strong{Notes & caveats.}
\itemize{
  \item Negative \code{Magnitude} values in GRO phases indicate residual noise or
        insufficient smoothing; a warning is issued.
  \item Very short phases may occur around local extrema; consider smoothing
        or post-filtering minimal run lengths upstream if needed.
  \item Ensure timestamps are regular (or nearly regular) for best results.
}
}
\section{Column definitions (ZG_cycle)}{

\describe{
  \item{Phases}{Integer; 1 = TWD (reversible shrinkage/expansion),
  2 = GRO (irreversible expansion).}
  \item{Start, End}{\code{POSIXct}; phase boundaries.}
  \item{Duration_h}{Numeric; phase duration in hours.}
  \item{Magnitude}{Numeric; GRO-only millimeter change of growth line across the phase. \code{NA} for TWD.}
  \item{rate}{Numeric; GRO-only rate in (\code{Magnitude*1000/Duration_h}).}
  \item{max.twd}{Numeric; peak TWD within the TWD phase.}
  \item{Max.twd.time}{\code{POSIXct}; time of \code{max.twd} within the TWD phase.}
  \item{AUC.load}{Numeric; pre-peak area under the TWD curve in mm*h, integrated from
  phase start to \code{Max.twd.time}.}
  \item{AUC.total}{Numeric; total area under the TWD curve in mm*h across the full TWD phase.}
  \item{ABr.value}{Numeric; Absolut Baumreaktion value for TWD phases,
  \deqn{\max(\mathrm{TWD}) \times \left(\frac{AUC_{load}}{\max(\mathrm{TWD})}\right)^{\beta}} \code{NA} for GRO.}
  \item{Avg.twd, STD.twd}{Numeric; mean and standard deviation of TWD in the phase.}
  \item{DOY}{Integer; day-of-year for the phase start.}
}
}

\examples{
\donttest{
library(dendRoAnalyst)
data(gf_nepa17)

# Minimal example (no smoothing)
zg <- phase.zg(df = gf_nepa17[1:600, ], TreeNum = 1)
head(zg$ZG_cycle, 5)
head(zg$ZG_phase, 5)

# With smoothing (e.g., 6 hours) to reduce short flips
zg6 <- phase.zg(df = gf_nepa17[1:600, ], TreeNum = 1, smoothing = 6, beta = 0.1)
subset(zg6$ZG_cycle, Phases == 1L)[1:5, c("Start","End","max.twd","ABr.value")]
}

}
\references{
Zweifel R, Haeni M, Buchmann N, Eugster W (2016) Are trees able to grow in periods of stem shrinkage?
\emph{New Phytologist}, 211:839–849. \doi{10.1111/nph.13995}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nepa2.R
\docType{data}
\name{nepa2}
\alias{nepa2}
\title{Dendrometer data from Kathmandu version 2}
\format{
A data frame with 14534 rows and 8 variables:
\describe{
  \item{\code{year}}{numeric year of data recording}
  \item{\code{month}}{numeric months of data recording}
  \item{\code{day}}{numeric days of data recording}
  \item{\code{hours}}{numeric hours of data recording}
  \item{\code{minutes}}{numeric minutes of data recording}
  \item{\code{seconds}}{numeric seconds of data recording}
  \item{\code{T2}}{double reading for first tree}
  \item{\code{T3}}{double reading for second tree}
}
}
\usage{
nepa2
}
\description{
Dendrometer data from three Chir pine trees collected in hourly resolution for 2 years with separated time.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_SC_output.R
\name{plot.SC_output}
\alias{plot.SC_output}
\title{Plot method for stem-cycle output}
\usage{
\method{plot}{SC_output}(
  x,
  y = NULL,
  DOY = NULL,
  Year = NULL,
  type = c("points", "ribbon", "transition", "balance", "increment", "frequency",
    "heatmap", "boxplot"),
  temporal = c("raw", "daily", "monthly"),
  x_axis = c("time", "doy"),
  balance_mode = c("time_of_day", "duration"),
  stat = c("Duration_h", "Duration_m", "Magnitude", "rate"),
  phase = c("all", "Shrinkage", "Expansion", "Increment"),
  cols = c("#fee8c8", "#fdbb84", "#e34a33"),
  phNames = c("Shrinkage", "Expansion", "Increment"),
  transition_linetype = "dashed",
  transition_alpha = 0.55,
  singleton_as_points = TRUE,
  ...
)
}
\arguments{
\item{x}{Object of class \code{"SC_output"} returned by \code{phase.sc()}.}

\item{y}{Unused.}

\item{DOY}{Optional numeric vector of length 2 giving start and end
day-of-year.}

\item{Year}{Optional numeric year used together with \code{DOY}.}

\item{type}{Plot type. One of
\code{"points"}, \code{"ribbon"}, \code{"transition"},
\code{"balance"}, \code{"increment"}, \code{"frequency"},
\code{"heatmap"}, or \code{"boxplot"}.}

\item{temporal}{Temporal scale: \code{"raw"}, \code{"daily"}, or
\code{"monthly"}. For \code{type = "boxplot"}, use \code{"daily"} or
\code{"monthly"}. For \code{type = "balance"}, \code{"frequency"}, and
\code{"heatmap"}, \code{"raw"} is internally treated as daily summary and
a warning is emitted.}

\item{x_axis}{X-axis style for time-based plots. One of \code{"time"} or
\code{"doy"}. When \code{"doy"} is used, timestamps are converted to
day-of-year on the x-axis. For \code{temporal = "monthly"}, \code{"doy"}
is not meaningful and time is used instead.}

\item{balance_mode}{For \code{type = "balance"}, controls how phase balance
is drawn. One of \code{"time_of_day"} or \code{"duration"}.
\code{"time_of_day"} draws continuous phase intervals along the y-axis
from 0 to 24 h, preserving when phases occurred within each day.
\code{"duration"} draws stacked bars showing total time spent in each
phase per day or month.}

\item{stat}{For \code{type = "boxplot"}, one of
\code{"Duration_h"}, \code{"Duration_m"}, \code{"Magnitude"}, or
\code{"rate"}.}

\item{phase}{For \code{type = "boxplot"}, one of
\code{"all"}, \code{"Shrinkage"}, \code{"Expansion"}, or
\code{"Increment"}.}

\item{cols}{Vector of three colours for Shrinkage, Expansion, and Increment.}

\item{phNames}{Vector of three labels for the three phase names.}

\item{transition_linetype}{Line type used for transition markers in
\code{type = "transition"}.}

\item{transition_alpha}{Transparency of transition markers.}

\item{singleton_as_points}{Logical. If \code{TRUE}, the boxplot panel falls
back to a point plot when every group contains only one value.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
Unified S3 plotting method for objects returned by \code{phase.sc()}.
Supports raw phase timelines, phase ribbons, transition diagnostics,
daily/monthly phase balance, cumulative increment, event frequency,
phase heatmaps, and daily/monthly boxplots of phase statistics.

For \code{type = "balance"}, two plotting modes are available. With
\code{balance_mode = "time_of_day"}, phases are drawn as continuous
within-day intervals, and the y-axis represents hour of day from 0 to 24.
This makes the timing of shrinkage, expansion, and increment visible. For
example, if shrinkage occurs from 00:00 to 05:00, expansion from 05:00 to
15:00, and shrinkage again from 15:00 to 24:00, these intervals are shown
as continuous coloured blocks along the y-axis.

With \code{balance_mode = "duration"}, the older stacked-bar behaviour is
used. In that case, the y-axis represents the amount of time spent in each
phase. With \code{temporal = "daily"}, the y-axis is hours per day. With
\code{temporal = "monthly"}, the y-axis is hours per month.
}
\details{
Plot types:
\itemize{
  \item \code{"points"}: dendrometer series with points coloured by phase.
  \item \code{"ribbon"}: dendrometer series with background ribbons for
        contiguous phases.
  \item \code{"transition"}: dendrometer series with vertical lines at phase
        changes.
  \item \code{"balance"}: phase balance through time. With
        \code{balance_mode = "time_of_day"}, phase intervals are drawn
        continuously along the y-axis from 0 to 24 h, so the timing of
        shrinkage, expansion, and increment within each day is visible.
        With \code{balance_mode = "duration"}, stacked bars show total time
        spent in each phase per day or month.
  \item \code{"increment"}: cumulative increment, phase 3, over time.
  \item \code{"frequency"}: number of phase events starting per day or month.
  \item \code{"heatmap"}: dominant phase by hour-of-day across dates or months.
  \item \code{"boxplot"}: distribution of selected cycle statistics after
        assembling cycle summaries to the chosen daily or monthly time scale.
}

The \code{"frequency"} plot uses \code{SC_cycle}.
The \code{"heatmap"} plot uses \code{SC_phase}.
The \code{"transition"} plot is especially useful for checking whether
smoothing reduced spurious phase switching.
}
\examples{
\donttest{
# data(gf_nepa17)
# sc <- phase.sc(df = gf_nepa17, TreeNum = 1, smoothing = 12)

# plot(sc)
# plot(sc, type = "ribbon")
# plot(sc, type = "transition")

# Daily within-day phase timing
# plot(sc, type = "balance", temporal = "daily")

# Daily within-day phase timing for selected DOY range
# plot(sc, type = "balance", temporal = "daily",
#      DOY = c(150, 160), Year = 2023)

# Old stacked-duration balance plot
# plot(sc, type = "balance", temporal = "daily",
#      balance_mode = "duration")

# Monthly stacked duration
# plot(sc, type = "balance", temporal = "monthly",
#      balance_mode = "duration")

# Boxplots
# plot(sc, type = "boxplot", stat = "Magnitude", temporal = "monthly")
# plot(sc, type = "boxplot", stat = "Duration_h",
#      temporal = "daily", phase = "Shrinkage")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_mov.cor.R
\name{plot.summary_mov_cor_dm}
\alias{plot.summary_mov_cor_dm}
\title{Plot method for summaries of moving dendrometer-climate correlation}
\usage{
\method{plot}{summary_mov_cor_dm}(
  x,
  y = NULL,
  type = c("peak_corr", "prop_significant", "peak_time"),
  x_axis = c("time", "doy"),
  low_col = "red",
  mid_col = "white",
  high_col = "blue",
  show_window_label = TRUE,
  ...
)
}
\arguments{
\item{x}{Object of class \code{"summary_mov_cor_dm"}.}

\item{y}{Unused.}

\item{type}{Plot type. One of \code{"peak_corr"}, \code{"prop_significant"},
or \code{"peak_time"}.}

\item{x_axis}{X-axis style for \code{type = "peak_time"}. One of
\code{"time"} or \code{"doy"}.}

\item{low_col}{Colour for negative values.}

\item{mid_col}{Colour for zero.}

\item{high_col}{Colour for positive values.}

\item{show_window_label}{Logical. If \code{TRUE}, climate settings are shown
as labels in \code{type = "peak_corr"}.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
S3 plotting method for objects returned by \code{summary.mov_cor_dm()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{dm.na.interpolation}
\alias{dm.na.interpolation}
\title{Detection and interpolation of missing values in dendrometer data}
\usage{
dm.na.interpolation(
  df,
  resolution,
  fill = FALSE,
  method = "spline",
  assess = FALSE,
  assess_lengths_hours = seq(6, 72, by = 6),
  assess_samples_per_length = 10,
  assess_buffer_hours = 6,
  assess_seed = NULL,
  verbose = FALSE
)
}
\arguments{
\item{df}{A data frame with the first column as datetime (\code{"yyyy-mm-dd HH:MM:SS"},
or POSIXct/Date), followed by dendrometer values.}

\item{resolution}{Integer. Temporal resolution in minutes (e.g., 60 for hourly data).}

\item{fill}{Logical. If \code{TRUE}, missing values will be interpolated.}

\item{method}{Character. Interpolation method (only used if \code{fill = TRUE}):
\itemize{
  \item \code{"spline"} --- cubic spline (\code{zoo::na.spline}).
  \item \code{"seasonal"} --- seasonal decomposition (\code{forecast::na.interp}).
}}

\item{assess}{Logical. If \code{TRUE}, run a simulation-based test to evaluate how well
interpolation performs at different gap lengths (on existing data).}

\item{assess_lengths_hours}{Integer vector. The gap durations (in hours) to test during
assessment. Default: \code{seq(6, 72, by = 6)}.}

\item{assess_samples_per_length}{Integer. Number of artificial gaps per gap length
per series. Default: \code{10}.}

\item{assess_buffer_hours}{Integer. Minimum number of hours of valid data required
before and after the artificial gap to allow valid assessment. Default: \code{6}.}

\item{assess_seed}{Integer or \code{NULL}. Random seed to ensure reproducibility
of sampling windows. Default: \code{NULL}.}

\item{verbose}{Logical. If \code{TRUE}, prints messages during filling and testing.}
}
\value{
A list of class \code{"dm_na_interpolation"} with components:
\describe{
  \item{\code{$data}}{Data frame containing the original or gap-filled series.}
  \item{\code{$gap_info}}{Table describing timestamp gaps and internal NA runs.}
  \item{\code{$assessment}}{A tibble summarizing interpolation accuracy for each
  series and gap length, or \code{NULL} when \code{assess = FALSE}.}
  \item{\code{$filled}}{Logical indicating whether interpolation was performed.}
  \item{\code{$assessed}}{Logical indicating whether interpolation assessment was performed.}
  \item{\code{$method}}{Interpolation method used.}
  \item{\code{$resolution}}{Temporal resolution in minutes.}
}
}
\description{
Detects missing timestamps (based on provided resolution), inserts rows with \code{NA},
and optionally interpolates missing values using either cubic spline interpolation
(\code{\link[zoo:na.spline]{na.spline}}) or seasonal decomposition interpolation
(\code{\link[forecast:na.interp]{na.interp}}).

Optionally, the function can test the reliability of interpolation for increasing
gap lengths (e.g., 6 hours to 3 days) using real data. This allows the user to assess
how well a given interpolation method recovers missing values over different durations.
}
\details{
The assessment simulates interpolation over artificial gaps of different durations.
For each dendrometer series and each gap length:
\enumerate{
  \item Random windows (with clean data) are selected.
  \item Data in the window is temporarily set to \code{NA}.
  \item The gap is interpolated using the selected method.
  \item The predicted values are compared to the original (true) values using:
    \itemize{
      \item MAPE --- Mean Absolute Percentage Error.
      \item MdAPE --- Median Absolute Percentage Error.
      \item RMSE_pct --- Root Mean Square Error (\%).
      \item Bias_pct --- Mean Percentage Error (\%).
      \item Max_diff_abs --- Mean absolute difference in daily maximum.
      \item Min_diff_abs --- Mean absolute difference in daily minimum.
      \item Time_max_diff_h --- Mean absolute difference in time of daily maximum (hours).
      \item Time_min_diff_h --- Mean absolute difference in time of daily minimum (hours).
    }
}
}
\examples{
\donttest{
library(dendRoAnalyst)
data(nepa17)

#res0 <- dm.na.interpolation(nepa17[1:1000, ], resolution = 60)
#plot(res0)
#plot(res0, type = "gaps")

#res1 <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "spline"
#)
#plot(res1)
#plot(res1, type = "gaps")
#plot(res1, type = "interpolation", original = nepa17[1:1000, ])

#res2 <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "seasonal",
#  assess = TRUE
# )
#plot(res2)
#plot(res2, type = "assessment", metric = "MdAPE")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean.detrended.dm.R
\name{mean_detrended.dm}
\alias{mean_detrended.dm}
\title{Calculate the mean detrended dendrometer series}
\usage{
mean_detrended.dm(
  detrended_dm,
  series = NULL,
  ac1.remove = TRUE,
  robust.mean = TRUE,
  trim = 0.15,
  seasonal_rescale = TRUE
)
}
\arguments{
\item{detrended_dm}{An object of class \code{"dm_detrended"} returned by
\code{dm.detrend.fit()}, or a data frame in the same wide format as
\code{$detrended_data}.}

\item{series}{Optional character vector of tree/series names to include.
Default is \code{NULL}, meaning all available detrended series are used.}

\item{ac1.remove}{Logical. If \code{TRUE}, removes temporal autocorrelation
from the mean detrended series using \code{forecast::auto.arima()} applied
separately within each season. Default is \code{TRUE}.}

\item{robust.mean}{Logical. If \code{TRUE}, calculates a trimmed mean across
trees at each time step. Default is \code{TRUE}.}

\item{trim}{Proportion to trim from each tail when \code{robust.mean = TRUE}.
Default is \code{0.15}.}

\item{seasonal_rescale}{Logical. If \code{TRUE}, the autocorrelation-removed
series is shifted to non-negative values and rescaled to mean = 1 within
each season. Default is \code{TRUE}.}
}
\value{
A tibble of class \code{"mean_dm_detrended"} containing:
\itemize{
  \item metadata columns copied from the detrended input,
  \item \code{STD_DDM}: the mean detrended series,
  \item \code{RES_DDM}: the autocorrelation-removed mean detrended series
        (returned only when \code{ac1.remove = TRUE}).
}
}
\description{
Computes a mean detrended dendrometer series across multiple trees from the
output of \code{dm.detrend.fit()}.

This is useful for creating one representative detrended series for a species,
site, or treatment group after detrending individual dendrometer series.

Optionally, the function can:
\itemize{
  \item calculate a \strong{robust mean} using a trimmed mean across trees,
  \item remove temporal autocorrelation from the mean detrended series using
  \code{forecast::auto.arima()},
  \item rescale the autocorrelation-removed series so it stays non-negative
  and has mean = 1 within each vegetation season.
}
}
\examples{
\donttest{
fit1 <- dm.growth.fit(
  df = gf_nepa17,
  TreeNum = 1:2,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)

det1 <- dm.detrend.fit(fit1)

m_det <- mean_detrended.dm(det1)
head(m_det, 10)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_detect_two_pulse_pattern}
\alias{dmgfd_detect_two_pulse_pattern}
\title{Detect two-pulse pattern from fitted growth rate}
\usage{
dmgfd_detect_two_pulse_pattern(
  rate,
  x_pred,
  peak_separation_min = 10,
  valley_ratio_max = 0.4,
  min_relative_peak = 0.05
)
}
\arguments{
\item{rate}{Fitted growth-rate vector.}

\item{x_pred}{Prediction season-day values.}

\item{peak_separation_min}{Minimum separation between peaks.}

\item{valley_ratio_max}{Maximum allowed valley-to-peak ratio.}

\item{min_relative_peak}{Minimum relative peak height.}
}
\value{
List describing detected pulse pattern.
}
\description{
Detect two-pulse pattern from fitted growth rate
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_tidy_fit_data}
\alias{dmgp_tidy_fit_data}
\title{Convert growth-fit object to tidy plotting data}
\usage{
dmgp_tidy_fit_data(x, observed_source = c("processed", "original_daily"))
}
\arguments{
\item{x}{An object of class \code{"dm_growth_fit"}.}

\item{observed_source}{Character string. Either \code{"processed"} or
\code{"original_daily"}.}
}
\value{
A tibble containing fitted values, observed values, series labels,
  fit identifiers, and panel identifiers.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to combine fitted and observed
daily dendrometer growth data into a long-format plotting table.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_resolve_x_axis}
\alias{dmgp_resolve_x_axis}
\title{Resolve x-axis mode for dendrometer growth-fit plots}
\usage{
dmgp_resolve_x_axis(type, x_axis = c("default", "season_day", "doy", "date"))
}
\arguments{
\item{type}{Plot type.}

\item{x_axis}{Requested x-axis representation.}
}
\value{
A list with elements \code{var}, \code{label}, and
  \code{timing_mode}.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to determine which column is
used on the x-axis and which timing coordinate system should be used.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{print.summary.clim_twd_stats}
\alias{print.summary.clim_twd_stats}
\title{Print summary of clim.twd.stats output}
\usage{
\method{print}{summary.clim_twd_stats}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.clim_twd_stats"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print summary of clim.twd.stats output
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_climate_phase.R
\name{plot.SC_output_clim}
\alias{plot.SC_output_clim}
\title{Plot climate-augmented stem-cycle output}
\usage{
\method{plot}{SC_output_clim}(
  x,
  y = NULL,
  ...,
  climate_var = NULL,
  climate_vars = NULL,
  numeric_vars = NULL,
  compare = FALSE,
  temporal = NULL
)
}
\arguments{
\item{x}{An object of class \code{SC_output_clim}.}

\item{y}{Optional climate variable name passed as the second argument.}

\item{...}{Additional arguments passed to \code{dm_plot_climate()} or
\code{dm_plot_climate_compare()}.}

\item{climate_var}{Character. Name of one climate variable to plot.}

\item{climate_vars}{Character vector of climate variables for comparison
plots.}

\item{numeric_vars}{Character vector of numeric variables used for correlation
or regression heatmaps.}

\item{compare}{Logical. If \code{TRUE}, \code{dm_plot_climate_compare()} is
called.}

\item{temporal}{Deprecated argument kept for compatibility with older
examples. It is ignored.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
S3 plotting method for objects of class \code{SC_output_clim}. This allows
climate-augmented stem-cycle outputs to be visualized directly with the
generic \code{plot()} function.
}
\examples{
\donttest{
# plot(sc_clim, climate_var = "temp_mean_phase", scale = "cycle",
#      type = "violin")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_dm.detrend.R
\name{plot.dm_detrended}
\alias{plot.dm_detrended}
\title{Plot detrended dendrometer series}
\usage{
\method{plot}{dm_detrended}(
  x,
  y = NULL,
  type = c("compare", "fit", "residual", "detrended", "boxplot"),
  series = NULL,
  seasons = NULL,
  x_axis = c("default", "date", "season_day", "doy"),
  facet_by = c("series", "season", "none"),
  ncol = NULL,
  box_group = c("series", "season"),
  show_observed = TRUE,
  show_fitted = TRUE,
  point_alpha = 0.7,
  line_width = 0.8,
  legend_position = "right",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_detrended"} returned by
[dm.detrend.fit()].}

\item{y}{Unused.}

\item{type}{Plot type. One of:
\describe{
  \item{`"compare"`}{Default three-panel comparison of original daily vs
    fitted, residuals, and detrended standardized series.}
  \item{`"fit"`}{Original daily and fitted original-scale values only.}
  \item{`"residual"`}{Residuals only.}
  \item{`"detrended"`}{Detrended standardized series only.}
  \item{`"boxplot"`}{Distribution of detrended standardized values by series
    or season.}
}}

\item{series}{Optional character vector of dendrometer series to plot.
Default is \code{NULL}, meaning all available series are used.}

\item{seasons}{Optional character vector of vegetation-season labels to plot.
Default is \code{NULL}, meaning all seasons are used.}

\item{x_axis}{Character string controlling the x-axis. One of:
\describe{
  \item{`"default"`}{Uses date for `"compare"` and `"fit"`, and season day
    for `"residual"` and `"detrended"`.}
  \item{`"date"`}{Use actual calendar date.}
  \item{`"season_day"`}{Use vegetation-season day.}
  \item{`"doy"`}{Use calendar day-of-year.}
}}

\item{facet_by}{Character string controlling faceting. One of:
\describe{
  \item{`"series"`}{Facet by dendrometer series.}
  \item{`"season"`}{Facet by vegetation season.}
  \item{`"none"`}{No faceting.}
}}

\item{ncol}{Optional integer giving the number of columns in faceted plots
where [ggplot2::facet_wrap()] is used.}

\item{box_group}{For \code{type = "boxplot"}, grouping variable on the x-axis.
One of \code{"series"} or \code{"season"}.}

\item{show_observed}{Logical. If \code{TRUE}, original daily observations are
shown in plot types where relevant. Default is \code{TRUE}.}

\item{show_fitted}{Logical. If \code{TRUE}, fitted original-scale values are
shown in plot types where relevant. Default is \code{TRUE}.}

\item{point_alpha}{Numeric alpha level used for observed points. Default is
\code{0.7}.}

\item{line_width}{Numeric line width used for fitted, residual, and detrended
lines. Default is \code{0.8}.}

\item{legend_position}{Character string specifying legend position.
Default is \code{"right"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
S3 plotting method for objects returned by [dm.detrend.fit()].

The default plot compares:
\itemize{
  \item original daily dendrometer series,
  \item fitted curve reconstructed on the original daily scale,
  \item residuals (`observed - fitted_original`),
  \item detrended standardized series.
}
}
\examples{
\donttest{
fit1 <- dm.growth.fit(
  df = gf_nepa17,
  TreeNum = 1:2,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)

det1 <- dm.detrend.fit(fit1)

plot(det1)
plot(det1, type = "fit")
plot(det1, type = "residual")
plot(det1, type = "detrended")
plot(det1, type = "boxplot")
plot(det1, type = "compare", facet_by = "series")
plot(det1, type = "compare", facet_by = "season")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_join_subdaily_clim.R
\name{dm_join_subdaily_clim}
\alias{dm_join_subdaily_clim}
\title{Join subdaily climate features to point-level dendrometer output}
\usage{
dm_join_subdaily_clim(x, clim_sub)
}
\arguments{
\item{x}{Object returned by \code{phase.zg()} or \code{phase.sc()}.}

\item{clim_sub}{Climate input. This can be a subdaily climate-feature table,
a standardized climate object returned by \code{read.climate()}, a raw
climate data frame, or a valid climate file path.}
}
\value{
The same phase object with climate features appended to \code{ZG_phase} or
\code{SC_phase}. The returned object has class
\code{c("ZG_output_clim", "ZG_output", ...)} or
\code{c("SC_output_clim", "SC_output", ...)}.
}
\description{
Joins timestamp-level subdaily climate features to point-level output from
\code{phase.zg()} or \code{phase.sc()}.

The climate input can be:
\itemize{
  \item the output of \code{dm_subdaily_clim()}
  \item the output of \code{read.climate()}
  \item a raw climate data frame
  \item a valid climate file path readable by \code{read.climate()}
}

If the supplied climate input is not already a subdaily climate-feature table,
the function attempts to standardize it with \code{read.climate()} and then
joins by exact timestamp using the \code{TIME} column.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)

clim_sub <- dm_subdaily_clim(
  ktm_clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  sum_vars  = c("prec"),
  lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)

zg_point_clim <- dm_join_subdaily_clim(zg, clim_sub)
head(zg_point_clim$ZG_phase)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reso_dm.R
\name{reso_dm}
\alias{reso_dm}
\title{Detect Temporal Resolution and Irregularities in Time Series}
\usage{
reso_dm(input_time)
}
\arguments{
\item{input_time}{A vector of class \code{POSIXct}, \code{Date}, or \code{character}
representing time stamps.}
}
\value{
A single integer: the estimated (rounded) average resolution in minutes.
        If multiple intervals are detected, a warning and index positions are printed.
}
\description{
Determines the average temporal resolution (in minutes) of a time series vector (e.g., dendrometer timestamps)
and detects inconsistencies in time intervals. If the time column is character-formatted, it is automatically
converted to POSIXct. Any inconsistent intervals are flagged and printed.
}
\details{
This function is helpful for checking if a time series (especially dendrometer data) is regularly sampled.
It handles both regular and irregular timestamps and gives feedback if the resolution changes.
}
\examples{
\dontrun{
# Regular 30-minute time sequence
time_seq <- seq.POSIXt(from = as.POSIXct("2023-06-01 00:00:00"),
                       by = "30 min", length.out = 100)
reso_dm(time_seq)  # Should return 30

# With character time input
time_char <- format(time_seq, format = "\%Y-\%m-\%d \%H:\%M:\%S")
reso_dm(time_char)  # Auto converts to POSIXct

# Introduce an irregular step
time_seq[51] <- time_seq[50] + 60  # One-time 1-hour jump
reso_dm(time_seq)  # Should print warning and irregular step index
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_validate_inputs}
\alias{dmgfd_validate_inputs}
\title{Validate inputs for double dendrometer growth fitting}
\usage{
dmgfd_validate_inputs(
  df,
  growth_fraction,
  min_unique_growth,
  custom_veg_season,
  rate_threshold_fraction,
  peak_separation_min,
  valley_ratio_max,
  min_relative_peak,
  fallback_to_single,
  fix_a_to_observed_max,
  fixed_a_multiplier
)
}
\arguments{
\item{df}{Input data frame.}

\item{growth_fraction}{Numeric vector of two growth fractions.}

\item{min_unique_growth}{Minimum number of unique growth values.}

\item{custom_veg_season}{Custom vegetation-season DOY range.}

\item{rate_threshold_fraction}{Fraction of peak growth rate.}

\item{peak_separation_min}{Minimum separation between derivative peaks.}

\item{valley_ratio_max}{Maximum allowed valley-to-peak ratio.}

\item{min_relative_peak}{Minimum relative derivative peak height.}

\item{fallback_to_single}{Logical fallback flag.}

\item{fix_a_to_observed_max}{Logical fixed-asymptote flag.}

\item{fixed_a_multiplier}{Multiplier for fixed total asymptote.}
}
\value{
Invisibly returns \code{TRUE} if checks pass.
}
\description{
Validate inputs for double dendrometer growth fitting
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_add_climate.R
\name{dm_add_climate}
\alias{dm_add_climate}
\title{Add climate information to dendrometer outputs}
\usage{
dm_add_climate(
  x,
  clim,
  scale = c("auto", "daily", "phase", "subdaily"),
  mean_vars = NULL,
  min_vars = NULL,
  max_vars = NULL,
  sum_vars = NULL,
  median_vars = NULL,
  lag_vars = NULL,
  lagmean_vars = NULL,
  lagsum_vars = NULL,
  lag_days = c(1, 3, 7),
  sub_mean_vars = NULL,
  sub_sum_vars = NULL,
  sub_lag_vars = NULL,
  roll_hours = c(3, 6, 24),
  lag_hours = c(1, 3, 6, 24),
  suffix = "_phase"
)
}
\arguments{
\item{x}{Object returned by \code{daily.data()}, \code{phase.zg()}, or \code{phase.sc()}.}

\item{clim}{Climate input. This can be:
\itemize{
  \item a standardized object returned by \code{read.climate()}
  \item a raw climate data frame
  \item a valid climate file path readable by \code{read.climate()}
  \item a daily climate table returned by \code{dm_daily_clim()}
  \item a subdaily climate-feature table returned by \code{dm_subdaily_clim()}
}}

\item{scale}{Character string controlling how climate is attached.
One of \code{"auto"}, \code{"daily"}, \code{"phase"}, or \code{"subdaily"}.
\itemize{
  \item For \code{daily_output}, only \code{"daily"} is supported.
  \item For \code{ZG_output} and \code{SC_output}, \code{"phase"} adds climate
        summaries to \code{ZG_cycle}/\code{SC_cycle}, while \code{"subdaily"}
        adds timestamp-level climate to \code{ZG_phase}/\code{SC_phase}.
}}

\item{mean_vars, min_vars, max_vars, sum_vars, median_vars}{Climate variables to
summarize by mean, minimum, maximum, sum, or median.
These are used by \code{dm_daily_clim()} and \code{dm_join_phase_clim()}.}

\item{lag_vars, lagmean_vars, lagsum_vars}{Variables used by \code{dm_daily_clim()}
to build lagged and antecedent daily climate features.}

\item{lag_days}{Integer vector of lag/antecedent windows in days for
\code{dm_daily_clim()}.}

\item{sub_mean_vars, sub_sum_vars, sub_lag_vars}{Variables used by
\code{dm_subdaily_clim()} to build rolling and lagged subdaily features.}

\item{roll_hours, lag_hours}{Numeric vectors of rolling-window and lag sizes
in hours for \code{dm_subdaily_clim()}.}

\item{suffix}{Suffix appended to climate summaries added by
\code{dm_join_phase_clim()}.}
}
\value{
The same biological object with climate information added.
}
\description{
A single helper function that adds climate information to outputs from
\code{daily.data()}, \code{phase.zg()}, or \code{phase.sc()}.

Depending on the class of \code{x} and the selected \code{scale}, the function:
\itemize{
  \item joins daily climate to \code{daily.data()} output
  \item joins climate summaries to phase windows in \code{ZG_cycle} or \code{SC_cycle}
  \item joins subdaily climate features to point-level \code{ZG_phase} or \code{SC_phase}
}
}
\examples{
\donttest{
data(nepa17)
data(gf_nepa17)
data(ktm_clim_hourly)

# daily.data() output + daily climate
dd <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
dd_clim <- dm_add_climate(
  dd,
  ktm_clim_hourly,
  scale = "daily",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(dd_clim)

# phase.zg() output + phase-window climate
zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
zg_clim <- dm_add_climate(
  zg,
  ktm_clim_hourly,
  scale = "phase",
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("temp", "VPD"),
  sum_vars  = c("prec")
)
head(zg_clim$ZG_cycle)

# phase.sc() output + point-level subdaily climate
sc <- phase.sc(df = gf_nepa17[1:800, ], TreeNum = 1, smoothing = 12)
sc_point <- dm_add_climate(
  sc,
  ktm_clim_hourly,
  scale = "subdaily",
  sub_mean_vars = c("temp", "VPD", "RH"),
  sub_sum_vars  = c("prec"),
  sub_lag_vars  = c("temp", "VPD", "RH"),
  roll_hours = c(1, 3, 6, 24),
  lag_hours  = c(1, 3, 6, 24)
)
head(sc_point$SC_phase)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ZG_output.R
\name{plot.ZG_output}
\alias{plot.ZG_output}
\title{Plot method for zero-growth output}
\usage{
\method{plot}{ZG_output}(
  x,
  y = NULL,
  DOY = NULL,
  Year = NULL,
  view = NULL,
  type = c("gro_twd", "phase_ribbon", "transition", "twd", "abr", "phase_summary",
    "balance", "boxplot"),
  temporal = c("raw", "daily", "monthly"),
  x_axis = c("time", "doy"),
  stat = c("Duration_h", "Magnitude", "rate", "max.twd", "Avg.twd", "STD.twd",
    "AUC.load", "AUC.total", "ABr.value"),
  phase = c("auto", "TWD", "GRO", "all"),
  box_group = c("period", "month_of_year", "doy"),
  twd_fun = c("mean", "max"),
  gro_fun = c("max", "mean"),
  transition_linetype = "dashed",
  transition_alpha = 0.55,
  cols = c(TWD = "red", GRO = "blue"),
  singleton_as_points = TRUE,
  balance_mode = c("time_of_day", "duration"),
  balance_gap = c("carry_forward", "observed_only"),
  ...
)
}
\arguments{
\item{x}{Object of class \code{"ZG_output"} returned by \code{phase.zg()}.}

\item{y}{Unused.}

\item{DOY}{Optional numeric vector of length 2 giving start and end
day-of-year for truncating the plotting window.}

\item{Year}{Optional numeric year used together with \code{DOY}.}

\item{view}{Optional backward-compatible argument. Use \code{"boxplot"}
to force boxplot mode.}

\item{type}{Plot type. One of \code{"gro_twd"}, \code{"phase_ribbon"},
\code{"transition"}, \code{"twd"}, \code{"abr"},
\code{"phase_summary"}, \code{"balance"}, or \code{"boxplot"}.}

\item{temporal}{Temporal scale. One of \code{"raw"}, \code{"daily"},
or \code{"monthly"}.}

\item{x_axis}{X-axis style for time-based plots. One of \code{"time"} or
\code{"doy"}. When \code{"doy"} is used, timestamps are converted to
day-of-year. For \code{temporal = "monthly"}, \code{"doy"} is not
meaningful and time is used instead.}

\item{stat}{For \code{type = "boxplot"}, one of
\code{"Duration_h"}, \code{"Magnitude"}, \code{"rate"},
\code{"max.twd"}, \code{"Avg.twd"}, \code{"STD.twd"},
\code{"AUC.load"}, \code{"AUC.total"}, or \code{"ABr.value"}.}

\item{phase}{For \code{type = "boxplot"}, one of \code{"auto"},
\code{"TWD"}, \code{"GRO"}, or \code{"all"}.}

\item{box_group}{For \code{type = "boxplot"}, grouping mode:
\code{"period"}, \code{"month_of_year"}, or \code{"doy"}.
\code{"period"} uses exact dates/months from the already aggregated
daily or monthly phase summaries; the other two pool values across
repeated seasonal positions. Rows with missing grouping labels are
removed before plotting.}

\item{twd_fun}{Aggregation function for point-level TWD when
\code{temporal != "raw"}. One of \code{"mean"} or \code{"max"}.}

\item{gro_fun}{Aggregation function for point-level GRO when
\code{temporal != "raw"}. One of \code{"max"} or \code{"mean"}.}

\item{transition_linetype}{Line type used for transition markers.}

\item{transition_alpha}{Transparency of transition markers.}

\item{cols}{Vector of two colours for TWD and GRO. It may be named
(\code{c(TWD = "red", GRO = "blue")}) or unnamed length 2.}

\item{singleton_as_points}{Logical. If \code{TRUE}, the boxplot panel
falls back to a point plot when every group contains only one value.}

\item{balance_mode}{For \code{type = "balance"}, one of
\code{"time_of_day"} or \code{"duration"}. \code{"time_of_day"} draws
continuous TWD/GRO intervals along the y-axis from 0 to 24 h, preserving
when phases occurred within each day. \code{"duration"} draws stacked bars
of total hours spent in each phase per day or month.}

\item{balance_gap}{For \code{type = "balance"}, one of
\code{"carry_forward"} or \code{"observed_only"}.
\code{"carry_forward"} assigns gaps to the previous observed phase;
\code{"observed_only"} counts only observed intervals.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object, returned invisibly.
}
\description{
Unified S3 plotting method for objects returned by \code{phase.zg()}.
It supports time-series views of GRO and TWD, phase ribbons and transitions,
TWD-only plots, ABr summaries, phase-balance views, and boxplots of
phase-level statistics. For boxplots, phase summaries are first assembled to
the requested daily or monthly time scale, and rows with missing x-axis
grouping labels are removed before plotting.

The plotting method operates on the two tables returned by
\code{phase.zg()}:
\itemize{
  \item \code{ZG_phase}: point-level time series with \code{TIME},
        \code{dm}, \code{Phases}, \code{TWD}, and \code{GRO};
  \item \code{ZG_cycle}: phase-level summaries including
        \code{max.twd}, \code{AUC.load}, \code{AUC.total}, and
        \code{ABr.value}.
}
}
\details{
\strong{ABr formula shown by this plot method.}

When \code{type = "abr"} or when \code{stat = "ABr.value"} is used in
boxplots, the displayed values come directly from \code{phase.zg()}:

\deqn{
  ABr = \max(TWD) \times \left(\frac{AUC_{load}}{\max(TWD)}\right)^{\beta}
}

where

\deqn{AUC_{load} = \int_{t_{start}}^{t_{max}} TWD(t)\,dt}

and

\deqn{AUC_{total} = \int_{t_{start}}^{t_{end}} TWD(t)\,dt.}

This function does not recompute ABr or AUC values; it only visualizes
the values already stored in the \code{ZG_output} object.

\strong{Aggregation behavior.}

For \code{temporal = "daily"} or \code{"monthly"}, point-level series
are aggregated before plotting. TWD and GRO can be summarized using
\code{"mean"} or \code{"max"}, depending on \code{twd_fun} and
\code{gro_fun}.

For phase-level summaries in \code{ZG_cycle}, the plot method aggregates
statistics by daily or monthly period. In those aggregated summaries:
\itemize{
  \item \code{Duration_h}, \code{Magnitude}, \code{AUC.load},
        \code{AUC.total}, and \code{ABr.value} are summed;
  \item \code{max.twd} is taken as the maximum;
  \item \code{Avg.twd} and \code{STD.twd} are averaged;
  \item \code{rate} is recalculated from aggregated magnitude and duration.
}

\strong{Phase balance.}

For \code{type = "balance"}, two balance styles are available through
\code{balance_mode}. With \code{balance_mode = "time_of_day"}, the plot
draws TWD and GRO intervals on a 0--24 hour y-axis, preserving the order in
which phases occurred within each day, similar to the balance option in
\code{plot.SC_output()}. With \code{balance_mode = "duration"}, the plot
draws stacked bars showing the total number of hours spent in each phase.
Missing or irregular intervals can be handled with \code{balance_gap}.

\strong{Boxplot statistics.}

The following phase-level variables can be visualized in
\code{type = "boxplot"}. For \code{box_group = "period"}, boxplots are
drawn on a continuous date axis spanning the selected multi-annual data
window, so empty days or months remain visible as gaps rather than being
removed from the x-axis:
\itemize{
  \item \code{"Duration_h"}
  \item \code{"Magnitude"}
  \item \code{"rate"}
  \item \code{"max.twd"}
  \item \code{"Avg.twd"}
  \item \code{"STD.twd"}
  \item \code{"AUC.load"}
  \item \code{"AUC.total"}
  \item \code{"ABr.value"}
}

\strong{Phase handling.}

For \code{phase = "auto"}, TWD-specific statistics
(\code{max.twd}, \code{Avg.twd}, \code{STD.twd}, \code{AUC.load},
\code{AUC.total}, \code{ABr.value}) automatically select TWD phases,
whereas GRO-specific variables (\code{Magnitude}, \code{rate}) select GRO
phases.
}
\examples{
\donttest{
# data(gf_nepa17)
# zg <- phase.zg(df = gf_nepa17[1:500, ], TreeNum = 1, beta = 0.1)

# Raw GRO and TWD time series
# plot(zg)

# Daily aggregated series
# plot(zg, temporal = "daily")

# Monthly TWD plot
# plot(zg, temporal = "monthly", type = "twd")

# ABr bar plot
# plot(zg, temporal = "monthly", type = "abr")

# Daily phase timing, similar to plot.SC_output balance
# plot(zg, type = "balance", temporal = "daily",
#      balance_mode = "time_of_day")

# Stacked phase-duration balance
# plot(zg, type = "balance", temporal = "daily",
#      balance_mode = "duration")

# Transition plot in day-of-year coordinates
# plot(zg, type = "transition", x_axis = "doy",
#      DOY = c(50, 100), Year = 2017)

# Boxplots of peak TWD
# plot(zg, type = "boxplot", stat = "max.twd",
#      temporal = "daily", box_group = "doy")

# Boxplots of AUC.load
# plot(zg, type = "boxplot", stat = "AUC.load",
#      temporal = "monthly", box_group = "month_of_year")

# Boxplots of ABr.value
# plot(zg, type = "boxplot", stat = "ABr.value",
#      temporal = "monthly", box_group = "month_of_year")
# plot(zg, view = "boxplot", stat = "ABr.value", temporal = "monthly")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_add_overall_timing}
\alias{dmgfd_add_overall_timing}
\title{Add overall cumulative-growth timing}
\usage{
dmgfd_add_overall_timing(
  params,
  pred_full,
  x_pred,
  growth_fraction,
  season_start_date = as.Date(NA)
)
}
\arguments{
\item{params}{Parameter list.}

\item{pred_full}{Fitted cumulative growth.}

\item{x_pred}{Prediction season-day values.}

\item{growth_fraction}{Growth fractions.}

\item{season_start_date}{Season start date.}
}
\value{
Updated parameter list.
}
\description{
Add overall cumulative-growth timing
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{plot.clim_twd_stats}
\alias{plot.clim_twd_stats}
\title{Plot method for clim.twd.stats output}
\usage{
\method{plot}{clim_twd_stats}(
  x,
  y = NULL,
  type = c("trajectory", "id_metric"),
  ids = NULL,
  groups = NULL,
  band = c("none", "sd", "limit95", "both"),
  metric = c("lag_to_below_zero", "lag_to_max_adverse_decline", "lag_to_return_zero",
    "max_adverse_decline_value", "change_end_adverse", "change_end_full_period",
    "continuous_end_full_period"),
  facet_by = c("IDs", "group", "none"),
  legend_position = "bottom",
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"clim_twd_stats"}.}

\item{y}{Unused.}

\item{type}{One of \code{"trajectory"} or \code{"id_metric"}.}

\item{ids}{Optional numeric vector of IDs to plot.}

\item{groups}{Optional character vector of group labels to plot.}

\item{band}{One of \code{"none"}, \code{"sd"}, \code{"limit95"}, or \code{"both"}.
Used for \code{type = "trajectory"}.}

\item{metric}{Metric column from \code{x$id_summary} for \code{type = "id_metric"}.}

\item{facet_by}{One of \code{"IDs"}, \code{"group"}, or \code{"none"}.}

\item{legend_position}{Legend position.}

\item{main}{Optional title.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plots grouped trajectories or grouped ID-level metrics from
\code{clim.twd.stats()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dm_event_times}
\alias{dm_event_times}
\title{Extract event times from phase.zg() or phase.sc() output}
\usage{
dm_event_times(
  x,
  event = "all",
  phase = NULL,
  min_duration = NULL,
  min_magnitude = NULL,
  min_max_twd = NULL,
  remove_na_times = TRUE
)
}
\arguments{
\item{x}{Object of class \code{"ZG_output"} or \code{"SC_output"}.}

\item{event}{Event type to extract. Use \code{"all"} to return all supported
events. For ZG output, supported events are \code{"TWD_start"},
\code{"TWD_end"}, \code{"GRO_start"}, \code{"GRO_end"},
\code{"MaxTWD"}, \code{"phase_start"}, and \code{"phase_end"}.
For SC output, supported events are \code{"Shrinkage_start"},
\code{"Shrinkage_end"}, \code{"Expansion_start"}, \code{"Expansion_end"},
\code{"Increment_start"}, \code{"Increment_end"}, \code{"phase_start"},
and \code{"phase_end"}.}

\item{phase}{Optional phase filter for generic events \code{"phase_start"}
or \code{"phase_end"}. For ZG output, use \code{"TWD"} or \code{"GRO"}.
For SC output, use \code{"Shrinkage"}, \code{"Expansion"}, or
\code{"Increment"}.}

\item{min_duration}{Optional minimum \code{Duration_h} filter.}

\item{min_magnitude}{Optional minimum \code{Magnitude} filter.}

\item{min_max_twd}{Optional minimum \code{max.twd} filter for ZG output.}

\item{remove_na_times}{Logical. If \code{TRUE}, rows with missing
\code{event_time} are removed.}
}
\value{
A tibble of events with class \code{c("dm_events", ...)}.
}
\description{
Extracts event times from objects returned by \code{phase.zg()} or
\code{phase.sc()}. The resulting table can be passed directly to
[dm_epoch_extract()] for superposed epoch analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_unit_string}
\alias{dmea_unit_string}
\title{Create a lubridate period string}
\usage{
dmea_unit_string(step, unit)
}
\arguments{
\item{step}{Numeric step size.}

\item{unit}{Unit. One of \code{"mins"}, \code{"hours"}, or \code{"days"}.}
}
\value{
A character period string for [lubridate::floor_date()].
}
\description{
Create a lubridate period string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ktm_clim_hourly.R
\docType{data}
\name{ktm_clim_hourly}
\alias{ktm_clim_hourly}
\title{Hourly climate data for Kathmandu derived from ERA5-Land}
\format{
A data frame with hourly observations and 5 variables:
\describe{
  \item{\code{TIME}}{Date-time stamp of the hourly observation.}
  \item{\code{temp}}{Air temperature in degree Celsius.}
  \item{\code{prec}}{Precipitation in millimetres.}
  \item{\code{VPD}}{Vapour pressure deficit in kPa.}
  \item{\code{RH}}{Relative humidity in percent.}
}
}
\source{
Extracted from the Copernicus Climate Change Service (C3S) Climate Data Store
ERA5-Land reanalysis product:

Muñoz-Sabater, J. (2019). ERA5-Land hourly data from 1950 to present.
Copernicus Climate Change Service (C3S) Climate Data Store (CDS).
}
\usage{
ktm_clim_hourly
}
\description{
Hourly near-surface climate data for Kathmandu, Nepal, extracted from the
Copernicus Climate Change Service (C3S) ERA5-Land reanalysis. The dataset is
included to demonstrate climate–dendrometer workflows in
\pkg{dendRoAnalyst}, including joining hourly climate data with dendrometer
observations, event-based analyses, and wavelet-based analyses.
}
\details{
\code{ktm_clim_hourly} contains a single-location hourly climate time series
for Kathmandu derived from ERA5-Land. ERA5-Land is a global land-surface
reanalysis produced by replaying the land component of ERA5 at enhanced
spatial resolution and is widely used for land-surface and ecohydrological
applications.

This dataset is intended as an example climate input for
\pkg{dendRoAnalyst}. It can be used, for example, with functions that join
dendrometer and climate data, calculate phase-climate relations, run event
analyses, or perform wavelet and wavelet-coherence analyses.

Users should check the exact temporal coverage of the object in their local
installation, for example with:
\preformatted{
range(ktm_clim_hourly$TIME)
}
}
\references{
Muñoz-Sabater, J., Dutra, E., Agustí-Panareda, A., Albergel, C.,
Arduini, G., Balsamo, G., Boussetta, S., Choulga, M., Harrigan, S.,
Hersbach, H., Martens, B., Miralles, D. G., Piles, M.,
Rodríguez-Fernández, N. J., Zsoter, E., Buontempo, C., and Thépaut, J.-N.
(2021). ERA5-Land: a state-of-the-art global reanalysis dataset for land
applications. \emph{Earth System Science Data}, 13, 4349--4383.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/climate_functions.R
\name{dm_daily_clim}
\alias{dm_daily_clim}
\title{Daily climate summaries for dendrometer analyses}
\usage{
dm_daily_clim(
  clim_df,
  mean_vars = NULL,
  min_vars = NULL,
  max_vars = NULL,
  sum_vars = NULL,
  median_vars = NULL,
  lag_vars = NULL,
  lagmean_vars = NULL,
  lagsum_vars = NULL,
  lag_days = c(1, 3, 7)
)
}
\arguments{
\item{clim_df}{Climate input. This can be:
\itemize{
  \item a standardized object returned by \code{read.climate()}
  \item a raw data frame with a time column in the first column or in a
        column named \code{TIME}
  \item a valid file path readable by \code{read.climate()}
}}

\item{mean_vars}{Character vector of variables to summarize by mean.}

\item{min_vars}{Character vector of variables to summarize by minimum.}

\item{max_vars}{Character vector of variables to summarize by maximum.}

\item{sum_vars}{Character vector of variables to summarize by sum.}

\item{median_vars}{Character vector of variables to summarize by median.}

\item{lag_vars}{Character vector of summarized daily climate columns for
which simple lagged values should be computed, e.g.
\code{c("VPD_max", "SWC_mean")}.}

\item{lagmean_vars}{Character vector of summarized daily climate columns for
which antecedent means should be computed, e.g.
\code{c("Tair_mean", "VPD_mean")}.}

\item{lagsum_vars}{Character vector of summarized daily climate columns for
which antecedent sums should be computed, e.g.
\code{c("P_sum", "Rad_sum")}.}

\item{lag_days}{Integer vector giving lag/antecedent window sizes in days,
e.g. \code{c(1, 3, 7)}.}
}
\value{
A tibble of class \code{"daily_clim"} with one row per day.
}
\description{
Computes daily climate summaries from climate time series so they can be
related to daily dendrometer summaries from \code{daily.data()}.

The input can be a standardized climate object returned by
\code{read.climate()}, a raw data frame, or a valid file path accepted by
\code{read.climate()}.

In addition to same-day climate summaries, the function can also compute
lagged and antecedent daily climate features from the summarized daily series:
\itemize{
  \item lagged values (e.g. previous 1 or 3 days)
  \item antecedent means over previous \code{n} days
  \item antecedent sums over previous \code{n} days
}
}
\details{
Lagged and antecedent features are calculated from the already summarized
daily climate columns. For example, if \code{VPD} is included in
\code{max_vars}, the daily summary column will be \code{VPD_max}. If this
column is listed in \code{lag_vars} and \code{lag_days = 1}, then the
additional column \code{VPD_max_lag_1d} is created.

Antecedent means and sums exclude the current day. For example:
\deqn{x\_lagmean\_3d(t) = mean(x_{t-3}, x_{t-2}, x_{t-1})}
\deqn{x\_lagsum\_7d(t) = sum(x_{t-7}, \ldots, x_{t-1})}
}
\examples{
\donttest{
data(ktm_clim_hourly)
clim_day <- dm_daily_clim(
  ktm_clim_hourly,
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec"),
  lag_vars = c("VPD_max", "temp_mean"),
  lagmean_vars = c("temp_mean", "VPD_mean", "RH_mean"),
  lagsum_vars = c("prec_sum"),
  lag_days = c(1, 3, 7)
)
head(clim_day, 5)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ktm_rain17.R
\docType{data}
\name{ktm_rain17}
\alias{ktm_rain17}
\title{Daily rainfall data of Kathmandu for 2017.}
\format{
A data frame with 365 rows and 2 variables:
\describe{
  \item{\code{TIME}}{Date in YYYY-MM-DD format.}
  \item{\code{rainfall}}{double rainfall in millimeters}
}
}
\source{
\url{http://www.mfd.gov.np/city?id=31/}
}
\usage{
ktm_rain17
}
\description{
This file contains daily rainfall data of Kathmandu. The source of this data is 'Government of Nepal, Department of Hydrology and Meteorology'.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_fit_model_logistic}
\alias{dmgf_fit_model_logistic}
\title{Fit logistic growth curve}
\usage{
dmgf_fit_model_logistic(
  x,
  y,
  x_pred,
  fixed_a_value = NA_real_,
  fix_a_to_observed_max = FALSE,
  start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v =
    1)
)
}
\arguments{
\item{x}{Observed x values.}

\item{y}{Observed y values.}

\item{x_pred}{Prediction x values.}

\item{fixed_a_value}{Optional fixed asymptote value.}

\item{fix_a_to_observed_max}{Logical.}

\item{start_value_richards_parameters}{Starting values.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit logistic growth curve
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_prepare_climate}
\alias{dmea_prepare_climate}
\title{Prepare climate data for superposed epoch analysis}
\usage{
dmea_prepare_climate(
  clim,
  vars = "all",
  step = 1,
  unit = c("hours", "days", "mins"),
  agg_fun = "mean",
  var_funs = NULL
)
}
\arguments{
\item{clim}{Climate data frame. First column must contain timestamps.}

\item{vars}{Climate variables to use, or \code{"all"}.}

\item{step}{Step size in units.}

\item{unit}{One of \code{"hours"}, \code{"days"}, or \code{"mins"}.}

\item{agg_fun}{Aggregation function applied to all variables when
\code{var_funs} is not supplied.}

\item{var_funs}{Optional named vector or named list of aggregation functions
for individual variables.}
}
\value{
A tibble with aggregated climate data.
}
\description{
Prepare climate data for superposed epoch analysis
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_sample_anchor_one}
\alias{dmea_sample_anchor_one}
\title{Sample one null anchor time}
\usage{
dmea_sample_anchor_one(
  event_time,
  clim_times,
  null = c("same_doy_window", "same_month", "random_time"),
  doy_window = 15,
  match_hour = TRUE,
  match_minute = TRUE,
  unit = c("hours", "days", "mins")
)
}
\arguments{
\item{event_time}{Original event time.}

\item{clim_times}{Candidate climate timestamps.}

\item{null}{Null model. One of \code{"same_doy_window"},
\code{"same_month"}, or \code{"random_time"}.}

\item{doy_window}{DOY window used for \code{null = "same_doy_window"}.}

\item{match_hour}{Logical. For hourly or minute data, keep the same hour
when possible.}

\item{match_minute}{Logical. For minute data, keep the same minute when
possible.}

\item{unit}{One of \code{"hours"}, \code{"days"}, or \code{"mins"}.}
}
\value{
One sampled \code{POSIXct} timestamp.
}
\description{
Sample one null anchor time
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_infer_gompertz_starts}
\alias{dmgfd_infer_gompertz_starts}
\title{Infer single Gompertz starting values}
\usage{
dmgfd_infer_gompertz_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial asymptote.}
}
\value{
Named list with \code{a}, \code{b}, and \code{k}.
}
\description{
Infer single Gompertz starting values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_add_anchor_points}
\alias{dmgf_add_anchor_points}
\title{Add zero-growth anchor points}
\usage{
dmgf_add_anchor_points(x, y, season_id = NULL)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{season_id}{Optional season identifier for pooled fits.}
}
\value{
List with \code{x} and \code{y}.
}
\description{
Add zero-growth anchor points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_validate_fit_growth_inputs}
\alias{dmgf_validate_fit_growth_inputs}
\title{Validate inputs for dendrometer growth fitting}
\usage{
dmgf_validate_fit_growth_inputs(
  df,
  growth_fraction,
  min_unique_growth,
  custom_veg_season,
  rate_threshold_fraction,
  fixed_a_multiplier
)
}
\arguments{
\item{df}{Input data frame.}

\item{growth_fraction}{Numeric vector of two growth fractions.}

\item{min_unique_growth}{Minimum number of unique growth values.}

\item{custom_veg_season}{Custom vegetation season DOY vector.}

\item{rate_threshold_fraction}{Fraction of peak rate used for rate window.}

\item{fixed_a_multiplier}{Multiplier used when fixing asymptote \code{a}.}
}
\value{
Invisibly returns \code{TRUE} if checks pass.
}
\description{
Validate inputs for dendrometer growth fitting
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.interpolation.R
\name{network.interpolation}
\alias{network.interpolation}
\title{Interpolate missing dendrometer values using a site network}
\usage{
network.interpolation(
  df,
  referenceDF,
  niMethod = c("proportional", "linear"),
  n_boot = 1000,
  return_flags = TRUE,
  return_pi = TRUE,
  pi_for_all = FALSE,
  return_fit = FALSE,
  assess = FALSE,
  assess_lengths_steps = c(1, 2, 3, 6, 12, 24),
  assess_samples_per_length = 20,
  assess_buffer_steps = 1,
  assess_seed = NULL,
  assess_use_only_observed = TRUE,
  progress = interactive(),
  correct_gap_jumps = FALSE,
  jump_threshold = NULL,
  jump_adjustment_method = c("window_median", "point_diff"),
  jump_adjust_window = 5,
  jump_set_point_na = FALSE
)
}
\arguments{
\item{df}{Data frame containing focal dendrometer data. The first column must
be datetime (\code{POSIXct}, \code{Date}, or character convertible by
\code{lubridate::ymd_hms}). Remaining columns are focal series.}

\item{referenceDF}{Data frame containing reference dendrometer data. The
first column must be datetime, and remaining columns are reference series.}

\item{niMethod}{Character. Interpolation method. One of
\code{"proportional"} or \code{"linear"}.}

\item{n_boot}{Integer. Number of bootstrap resamples used in proportional
mode.}

\item{return_flags}{Logical. If \code{TRUE}, add a logical column
\code{<series>_interp} indicating which focal values were imputed.}

\item{return_pi}{Logical. If \code{TRUE}, add 95\% prediction-limit columns
\code{<series>_pi_lo} and \code{<series>_pi_hi}.}

\item{pi_for_all}{Logical. If \code{TRUE}, prediction limits are also
attached for non-imputed rows where the model could be formed. If
\code{FALSE}, limits are returned only for imputed rows.}

\item{return_fit}{Logical. If \code{TRUE}, add \code{<series>_fit} columns
containing the model-implied fitted value.}

\item{assess}{Logical. If \code{TRUE}, run synthetic-gap validation.}

\item{assess_lengths_steps}{Integer vector. Artificial gap lengths,
expressed in number of rows/time steps, to test during validation.}

\item{assess_samples_per_length}{Integer. Number of sampled artificial gaps
per gap length and per focal series.}

\item{assess_buffer_steps}{Integer. Number of observed steps required before
and after each artificial gap when
\code{assess_use_only_observed = TRUE}.}

\item{assess_seed}{Integer or \code{NULL}. Random seed for reproducible
validation sampling.}

\item{assess_use_only_observed}{Logical. If \code{TRUE}, synthetic gaps are
sampled only from windows fully observed in the focal series, including
the requested buffer.}

\item{progress}{Logical. If \code{TRUE}, display a single-line progress
indicator.}

\item{correct_gap_jumps}{Logical. If \code{TRUE}, inspect the first observed
point after each imputed run and optionally remove a detected jump.}

\item{jump_threshold}{Numeric. Minimum absolute jump size required before a
post-gap correction is applied. Required if
\code{correct_gap_jumps = TRUE}.}

\item{jump_adjustment_method}{Character. Method used to estimate the
post-gap offset. One of \code{"window_median"} or \code{"point_diff"}.}

\item{jump_adjust_window}{Integer. Window size used by
\code{jump_adjustment_method = "window_median"}.}

\item{jump_set_point_na}{Logical. If \code{TRUE}, the jump point itself is
set to \code{NA} after correction.}
}
\value{
A data frame with class \code{"network_interpolation"}.

The data frame contains the original \code{TIME} column, focal series
columns, and optionally:
\describe{
  \item{\code{<series>_interp}}{Logical flag for imputed rows.}
  \item{\code{<series>_pi_lo}, \code{<series>_pi_hi}}{95\% prediction
  limits.}
  \item{\code{<series>_fit}}{Model-implied fitted values.}
  \item{\code{<series>_jump_correction}}{Numeric jump offset removed at the
  first observed point after an imputed run. \code{NA} means no correction
  was applied.}
}

In addition, the following attributes are attached:
\describe{
  \item{\code{network_original}}{Original focal data on the aligned time
  grid.}
  \item{\code{network_reference}}{Reference data aligned to the focal time
  grid.}
  \item{\code{network_diagnostics}}{Long-format diagnostics for plotting.}
  \item{\code{network_summary}}{Per-series interpolation summary table.}
  \item{\code{network_jump_corrections}}{Table of inspected and corrected
  post-gap jumps.}
  \item{\code{network_validation_raw}}{Per-gap validation summary table, or
  \code{NULL}.}
  \item{\code{network_validation_points}}{Point-level validation results, or
  \code{NULL}.}
  \item{\code{network_validation_summary}}{Summarized validation metrics by
  series and gap length, or \code{NULL}.}
  \item{\code{network_validation_seasonal}}{Seasonal validation diagnostics
  by day of year, or \code{NULL}.}
  \item{\code{network_method}}{Interpolation method used.}
  \item{\code{network_n_boot}}{Number of bootstrap resamples used.}
  \item{\code{network_assessed}}{Logical indicating whether validation was
  run.}
  \item{\code{network_jump_settings}}{List of jump-correction settings.}
}
}
\description{
Fills \code{NA} gaps in one or more focal dendrometer series by borrowing
information from a reference network recorded at the same site and on the
same time grid.

The focal dataset \code{df} is treated as the master time axis. The
reference network is aligned to that axis, so missing timestamps already
inserted into \code{df} are preserved during interpolation.

Two interpolation modes are available:
\itemize{
  \item \code{niMethod = "proportional"} uses the average relative change in
  the reference network between consecutive timestamps and propagates that
  change to the focal series.
  \item \code{niMethod = "linear"} fits a cross-sensor regression between
  reference values at time \eqn{t-1} and \eqn{t}, and predicts the focal
  value at time \eqn{t} from the focal value at time \eqn{t-1}.
}

Optional bootstrap-based uncertainty limits can be attached, synthetic-gap
validation can be used to assess recovery performance, and an optional
post-processing step can detect and correct abrupt post-gap jumps in the
interpolated series.
}
\details{
The function assumes that missing timestamps have already been inserted into
\code{df} and corresponding measurements set to \code{NA}, for example with
\code{\link{dm.na.interpolation}}.

For a focal series \eqn{a}, let \eqn{a_{t-1}} be the last available focal
value before the current timestamp. Let \eqn{b_{t-1}} and \eqn{b_t} be the
vectors of reference values at the previous and current timestamps.

In proportional mode, the relative reference change is
\deqn{
  \delta = \mathrm{mean}\left(\frac{b_t - b_{t-1}}{\max(b_{t-1}, \epsilon)}\right)
}
and the focal estimate is
\deqn{
  \hat{a}_t = a_{t-1}(1 + \delta).
}
Bootstrap resampling of reference sensors is used to estimate a 95\%
interval.

In linear mode, a simple regression of \eqn{b_t} on \eqn{b_{t-1}} is fit
across reference sensors at each step, and the focal value is predicted from
\eqn{a_{t-1}} with a 95\% prediction interval.

If fewer than two valid reference sensors are available at a timestamp pair,
or if the focal previous value is missing, the focal value is left unchanged.

If \code{correct_gap_jumps = TRUE}, the function inspects the first observed
point immediately after each imputed run. If a jump larger than
\code{jump_threshold} is detected, the estimated offset is removed from that
point onward. This is useful when a dendrometer resumes from a shifted
baseline after a gap.

If \code{assess = TRUE}, the function inserts artificial gaps into observed
sections of each focal series, interpolates them with the chosen method, and
compares predictions with the true values. This produces both per-gap
summaries and seasonal diagnostics of interpolation error.

The returned object is a data frame with class
\code{"network_interpolation"}.
}
\section{Interpretation}{

Smaller validation metrics such as \code{MAE}, \code{RMSE}, \code{MAPE}, and
\code{MdAPE} indicate better gap recovery. \code{PI_coverage_95} indicates
how well the nominal 95\% uncertainty interval captures the true values
under synthetic-gap validation. Seasonal diagnostics help identify periods
of the year when network interpolation is more or less reliable.

If many post-gap corrections are applied or \code{max_abs_gap_jump} is
large, it may indicate re-zeroing, sensor reset, or baseline
discontinuities after gaps.
}

\section{Notes}{

\itemize{
  \item The focal dataset \code{df} defines the output time grid.
  \item Reference series are aligned to that grid using the datetime column.
  \item At least two valid reference sensors are required at consecutive
  timestamps for interpolation.
  \item Jump correction is a post-processing step and does not alter the
  core interpolation logic.
}
}

\examples{
\donttest{
#library(dendRoAnalyst)
#data("gf_nepa17")

## Create an artificial focal gap
#df1 <- gf_nepa17
#df1[40:50, "T2"] <- NA

## Build a small reference network
#ref <- cbind(gf_nepa17, gf_nepa17[, 2:3], gf_nepa17[, 2:3])
#colnames(ref) <- c("Time", "T1", "T2", "T3", "T4", "T5", "T6")

## Interpolate with uncertainty limits
#out <- network.interpolation(
#  df1, ref,
#  niMethod = "proportional",
#  n_boot = 100,
#  return_flags = TRUE,
#  return_pi = TRUE
#)

#head(out, 10)

# Interpolate with jump correction and validation
#out2 <- network.interpolation(
#  df1, ref,
#  niMethod = "proportional",
#  n_boot = 100,
#  return_flags = TRUE,
#  return_pi = TRUE,
#  assess = TRUE,
#  assess_lengths_steps = c(1, 2, 4),
#  correct_gap_jumps = TRUE,
#  jump_threshold = 0.05,
#  jump_adjustment_method = "window_median",
#  jump_adjust_window = 5
#)
# Plotting
#plot(out2)
#plot(out2, type = "compare")
#plot(out2, type = "seasonal_error")

# Extracting the information using attr()
#attr(out, "network_validation_summary")
#attr(out, "network_validation_points")
#attr(out, "network_validation_seasonal")
}

}
\seealso{
\code{\link{plot.network_interpolation}},
\code{\link{dm.na.interpolation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_fit_model_loess}
\alias{dmgf_fit_model_loess}
\title{Fit loess growth curve}
\usage{
dmgf_fit_model_loess(x, y, x_pred, loess_span = 0.2, extrapolated = FALSE)
}
\arguments{
\item{x}{Observed x values.}

\item{y}{Observed y values.}

\item{x_pred}{Prediction x values.}

\item{loess_span}{Loess span.}

\item{extrapolated}{Logical flag for extrapolation.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit loess growth curve
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mov.cor.dm.R
\name{print.summary_mov_cor_dm}
\alias{print.summary_mov_cor_dm}
\title{Print method for summaries of running dendrometer-climate correlation objects}
\usage{
\method{print}{summary_mov_cor_dm}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"summary_mov_cor_dm"}.}

\item{...}{Unused.}
}
\value{
The summary object, invisibly.
}
\description{
Print method for summaries of running dendrometer-climate correlation objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{plot.clim_twd_test}
\alias{plot.clim_twd_test}
\title{Plot method for clim.twd.test output}
\usage{
\method{plot}{clim_twd_test}(
  x,
  y = NULL,
  show_points = TRUE,
  facet = TRUE,
  legend_position = "none",
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"clim_twd_test"}.}

\item{y}{Unused.}

\item{show_points}{Logical. If \code{TRUE}, add jittered points.}

\item{facet}{Logical. If \code{TRUE}, facet by stratum when multiple strata exist.}

\item{legend_position}{Legend position.}

\item{main}{Optional plot title.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plots the data used in \code{clim.twd.test()} as grouped boxplots with jittered
observations, faceted by stratum when applicable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_reconstruct.R
\name{print.summary.dm_wavelet_reconstruct}
\alias{print.summary.dm_wavelet_reconstruct}
\title{Print method for summary.dm_wavelet_reconstruct}
\usage{
\method{print}{summary.dm_wavelet_reconstruct}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.dm_wavelet_reconstruct"}.}

\item{digits}{Number of digits for rounded numeric printing.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print method for summary.dm_wavelet_reconstruct
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_reconstruct.R
\name{plot.dm_wavelet_reconstruct}
\alias{plot.dm_wavelet_reconstruct}
\title{Plot method for dm_wavelet_reconstruct objects}
\usage{
\method{plot}{dm_wavelet_reconstruct}(
  x,
  y = NULL,
  series = NULL,
  type = c("compare", "reconstructed", "difference", "filtered"),
  facet = TRUE,
  legend_position = "right",
  line_width = 0.8,
  alpha = 0.7,
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_wavelet_reconstruct"}.}

\item{y}{Unused.}

\item{series}{Optional character vector of series names to plot. If
\code{NULL}, all available reconstructed series are used.}

\item{type}{One of:
\describe{
  \item{`"compare"`}{Original and reconstructed series together.}
  \item{`"reconstructed"`}{Reconstructed component only.}
  \item{`"difference"`}{Original minus reconstructed.}
  \item{`"filtered"`}{Directly plot the returned filtered series. This is
    the extracted component for \code{mode = "extract"} and the
    component-removed series for \code{mode = "remove"}.}
}}

\item{facet}{Logical. If \code{TRUE}, facet by series.}

\item{legend_position}{Legend position passed to ggplot2.}

\item{line_width}{Line width.}

\item{alpha}{Alpha transparency for original series in compare plots.}

\item{main}{Optional title.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plots reconstructed or filtered cycle components extracted by
\code{dm_wavelet_reconstruct()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_coherence.R
\name{plot.dm_wavelet_coherence}
\alias{plot.dm_wavelet_coherence}
\title{Plot method for dm_wavelet_coherence objects}
\usage{
\method{plot}{dm_wavelet_coherence}(
  x,
  y = NULL,
  pair = NULL,
  type = c("coherence", "cross_power", "average_coherence", "average_cross_power",
    "phase", "series"),
  facet = TRUE,
  log_period = TRUE,
  log_power = TRUE,
  clip_quantile = c(0.01, 0.99),
  show_sig = TRUE,
  show_coi = TRUE,
  siglvl = 0.05,
  sig_color = "black",
  sig_size = 0.4,
  coi_fill = "white",
  coi_alpha = 0.45,
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dm_wavelet_coherence"}.}

\item{y}{Unused.}

\item{pair}{Optional pair name(s) to plot. If \code{NULL}, the first pair is used.}

\item{type}{One of \code{"coherence"}, \code{"cross_power"},
\code{"average_coherence"}, \code{"average_cross_power"}, \code{"phase"},
or \code{"series"}.}

\item{facet}{Logical. If \code{TRUE} and multiple pairs are selected, facet them.}

\item{log_period}{Logical.}

\item{log_power}{Logical for raster intensity.}

\item{clip_quantile}{Optional clipping quantiles for raster intensity.}

\item{show_sig}{Logical.}

\item{show_coi}{Logical.}

\item{siglvl}{Significance level.}

\item{sig_color}{Significance contour color.}

\item{sig_size}{Significance contour linewidth.}

\item{coi_fill}{COI fill color.}

\item{coi_alpha}{COI alpha.}

\item{main}{Optional title.}

\item{...}{Unused.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plot method for dm_wavelet_coherence objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_reconstruct.R
\name{dm_wavelet_reconstruct}
\alias{dm_wavelet_reconstruct}
\title{Reconstruct or remove selected cycle components from a dm_wavelet object}
\usage{
dm_wavelet_reconstruct(
  x,
  series = NULL,
  mode = c("extract", "remove"),
  period_hours = NULL,
  lower_hours = NULL,
  upper_hours = NULL,
  lvl = 0,
  only_sig = TRUE,
  siglvl = 0.05,
  only_coi = FALSE,
  only_ridge = FALSE,
  rescale = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{x}{An object of class \code{"dm_wavelet"} returned by
\code{dm_wavelet()}.}

\item{series}{Optional character vector of series names to reconstruct. If
\code{NULL}, all available series are used.}

\item{mode}{One of \code{"extract"} or \code{"remove"}.}

\item{period_hours}{Optional numeric vector of exact periods, in hours, to
reconstruct. If supplied, \code{lower_hours} and \code{upper_hours} are
ignored.}

\item{lower_hours}{Optional lower period bound in hours for band
reconstruction.}

\item{upper_hours}{Optional upper period bound in hours for band
reconstruction.}

\item{lvl}{Minimum wavelet power level to include in the reconstruction.}

\item{only_sig}{Logical. If \code{TRUE}, use wavelet power significance in
reconstruction.}

\item{siglvl}{Significance level used when \code{only_sig = TRUE}.}

\item{only_coi}{Logical. If \code{TRUE}, restrict reconstruction to the cone
of influence.}

\item{only_ridge}{Logical. If \code{TRUE}, reconstruct from the power ridge
only.}

\item{rescale}{Logical. Passed to \code{WaveletComp::reconstruct()}.}

\item{verbose}{Logical. If \code{TRUE}, prints a completion message.}
}
\value{
An object of class \code{"dm_wavelet_reconstruct"} with elements:
\describe{
  \item{call}{Matched function call.}
  \item{series}{Selected series names.}
  \item{mode}{Reconstruction mode, either \code{"extract"} or \code{"remove"}.}
  \item{selection}{A list describing the requested selection in hours and in
  native wavelet units.}
  \item{results}{Named list of \code{WaveletComp::reconstruct()} outputs.}
  \item{reconstructed_long}{Tidy table with columns \code{TIME},
  \code{series}, \code{original}, \code{reconstructed}, \code{difference},
  and \code{filtered}.}
  \item{used_periods}{Per-series table of periods actually used in the
  reconstruction, in native units and in hours.}
  \item{filtered_wide}{Wide table with \code{TIME} in the first column and
  one filtered series column per selected tree/series.}
  \item{parent}{Minimal metadata copied from the parent \code{dm_wavelet}
  object.}
}
}
\description{
Reconstructs a selected oscillatory component, or a selected period band,
from a \code{dm_wavelet} object using \code{WaveletComp::reconstruct()}.

Requested periods are supplied in \strong{hours}. They are internally
converted to the native wavelet period units used when \code{dm_wavelet()}
was computed, then passed to \code{WaveletComp::reconstruct()}.

The function supports two modes:
\itemize{
  \item \code{mode = "extract"} returns the selected cycle or band itself.
  \item \code{mode = "remove"} returns the original series with the selected
  cycle or band removed.
}
}
\examples{
\donttest{
wv <- dm_wavelet(
  x = gf_nepa17,
  TreeNum = 1:2,
  source = "raw",
  make_pval = TRUE,
  verbose = FALSE
)

# extract circadian component
rec_extract <- dm_wavelet_reconstruct(
  wv,
  mode = "extract",
  lower_hours = 20,
  upper_hours = 28,
  only_sig = TRUE
)

# remove circadian component
rec_remove <- dm_wavelet_reconstruct(
  wv,
  mode = "remove",
  lower_hours = 20,
  upper_hours = 28,
  only_sig = TRUE
)

head(rec_extract$filtered_wide)
head(rec_remove$filtered_wide)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{plot_dm_interpolation}
\alias{plot_dm_interpolation}
\title{Plot original and interpolated dendrometer series}
\usage{
plot_dm_interpolation(
  x,
  original = NULL,
  series = NULL,
  start = NULL,
  end = NULL,
  free_y = TRUE,
  ncol = 1
)
}
\arguments{
\item{x}{Object returned by \code{dm.na.interpolation()}.}

\item{original}{Optional original input data frame used in
\code{dm.na.interpolation()}.}

\item{series}{Optional character vector of series names to plot.}

\item{start}{Optional start datetime for plotting subset.}

\item{end}{Optional end datetime for plotting subset.}

\item{free_y}{Logical. If \code{TRUE}, each facet gets its own y-scale.}

\item{ncol}{Integer. Number of facet columns.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plot original and interpolated dendrometer series
}
\examples{
\donttest{
#res <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "spline"
#)
#plot_dm_interpolation(res, original = nepa17[1:1000, ])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_normalize.R
\name{dm_standardize}
\alias{dm_standardize}
\title{Standardize dendrometer series within seasonal years}
\usage{
dm_standardize(
  df,
  season_type = c("NH", "SH", "CS"),
  CS_doys = NULL,
  method = c("center", "amplitude", "robust_amplitude", "minmax", "zscore",
    "robust_zscore", "percentile"),
  ref_type = c("first_value", "first_n_days", "ref_window"),
  ref_n_days = 7,
  ref_doys = NULL,
  q_low = 0.05,
  q_high = 0.95
)
}
\arguments{
\item{df}{A data frame whose first column contains date-time
(\code{yyyy-mm-dd HH:MM:SS}, \code{POSIXct}, or \code{Date}) and all
remaining columns are dendrometer series.}

\item{season_type}{One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.}

\item{CS_doys}{Optional numeric vector of length 2 defining the start and end
DOY for \code{season_type = "CS"}.}

\item{method}{Standardization method. One of:
\itemize{
  \item \code{"center"}: subtract seasonal reference value
  \item \code{"amplitude"}: subtract reference and divide by seasonal range
  \item \code{"robust_amplitude"}: subtract reference and divide by
        \code{Q_high - Q_low}
  \item \code{"minmax"}: rescale to \code{(x - min)/(max - min)}
  \item \code{"zscore"}: seasonal z-score
  \item \code{"robust_zscore"}: robust seasonal z-score using median and MAD
  \item \code{"percentile"}: convert seasonal values to percentiles in \eqn{[0,1]}
}}

\item{ref_type}{Reference-value definition for methods that need a reference
(\code{"center"}, \code{"amplitude"}, \code{"robust_amplitude"}). One of:
\itemize{
  \item \code{"first_value"}: first non-missing value in the seasonal year
  \item \code{"first_n_days"}: mean of the first \code{ref_n_days} unique days
  \item \code{"ref_window"}: mean of values within \code{ref_doys}
}}

\item{ref_n_days}{Number of initial days used when \code{ref_type = "first_n_days"}.}

\item{ref_doys}{Optional numeric vector of length 2 defining a DOY reference
window when \code{ref_type = "ref_window"}.}

\item{q_low}{Lower quantile used by \code{method = "robust_amplitude"}.}

\item{q_high}{Upper quantile used by \code{method = "robust_amplitude"}.}
}
\value{
A list of class \code{"dm_standardized"} containing:
\itemize{
  \item \code{data}: wide data frame with columns
    \code{TIME}, \code{season_year}, \code{in_season}, and standardized dendrometer series
  \item \code{parameters}: tibble with one row per tree and seasonal year,
    summarizing the reference value and scaling denominator used
  \item \code{metadata}: method and seasonal-year settings
}

For \code{season_type = "CS"}, observations outside the custom season are
retained in \code{data}, but their standardized values are set to \code{NA}
and \code{in_season = FALSE}.
}
\description{
Standardizes one or more dendrometer series within seasonal years so that
multiple trees can be brought to a comparable scale while preserving their
within-season temporal pattern.

Seasonal years are defined as:
\itemize{
  \item \code{"NH"}: Northern Hemisphere year, from \strong{01 Jan} to \strong{31 Dec}
  \item \code{"SH"}: Southern Hemisphere year, from \strong{01 Jul} to \strong{30 Jun} of the next year
  \item \code{"CS"}: Custom season defined by \code{CS_doys = c(doy1, doy2)}
}

For \code{season_type = "CS"}:
\itemize{
  \item if \code{CS_doys[1] <= CS_doys[2]}, the season stays within one calendar year
  \item if \code{CS_doys[1] > CS_doys[2]}, the season wraps across years
}

Standardization is applied separately for each dendrometer series and each
seasonal year.
}
\examples{
\donttest{
data(gf_nepa17)

# Northern Hemisphere seasonal-year standardization
out_nh <- dm_standardize(
  df = gf_nepa17,
  season_type = "NH",
  method = "robust_amplitude"
)

# Southern Hemisphere seasonal-year standardization
out_sh <- dm_standardize(
  df = gf_nepa17,
  season_type = "SH",
  method = "center"
)

# Custom season within one year
out_cs1 <- dm_standardize(
  df = gf_nepa17,
  season_type = "CS",
  CS_doys = c(100, 280),
  method = "robust_amplitude"
)

# Custom season wrapping across years
out_cs2 <- dm_standardize(
  df = gf_nepa17,
  season_type = "CS",
  CS_doys = c(250, 120),
  method = "percentile"
)

head(out_nh$data)
head(out_nh$parameters)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daily.data.R
\name{daily.data}
\alias{daily.data}
\title{Daily statistics of dendrometer data}
\usage{
daily.data(df, TreeNum)
}
\arguments{
\item{df}{A data frame with the first column containing date-time stamps
convertible to \code{POSIXct} and subsequent columns containing
dendrometer measurements.}

\item{TreeNum}{Integer. The index of the tree (column) to analyze.
\code{TreeNum = 1} refers to the first dendrometer column after the
time column.}
}
\value{
A tibble of class \code{"daily_output"} containing:
\describe{
  \item{DATE}{Calendar date.}
  \item{Min}{Daily minimum value.}
  \item{Time_min}{Time of day of minimum value.}
  \item{Max}{Daily maximum value.}
  \item{Time_max}{Time of day of maximum value.}
  \item{mean}{Daily mean value.}
  \item{median}{Daily median value.}
  \item{amplitude}{Daily amplitude = Max - Min.}
  \item{Time_min_h}{Time of minimum expressed in decimal hours.}
  \item{Time_max_h}{Time of maximum expressed in decimal hours.}
  \item{lag_h}{Signed difference in hours: Time_max_h - Time_min_h.}
  \item{Remarks}{\code{"*"} if \code{Time_max > Time_min}, otherwise \code{""}.}
  \item{Max_diff}{Difference between today's maximum and the previous day's maximum.}
  \item{Day_status}{\code{"growing"}, \code{"shrinking"}, \code{"stable"}, or \code{NA}.}
}
}
\description{
Computes daily statistics from high-frequency dendrometer time series.
For each day, it extracts the minimum and maximum values, their times
of occurrence, daily mean and median, daily amplitude, the signed lag
between the time of maximum and minimum, the day-to-day change in the
daily maximum, and a daily status indicating whether the day is growing,
shrinking, or stable relative to the previous day.
}
\details{
The function requires a data frame with a time column in the first column
and one or more dendrometer series in the following columns. The user
selects the series using \code{TreeNum}.

The returned object has class \code{"daily_output"}, so it can be plotted
directly with \code{plot()}.

The column \code{Max_diff} is computed as:
\deqn{Max\_diff_t = Max_t - Max_{t-1}}

The column \code{Day_status} is derived from \code{Max_diff}:
\itemize{
  \item \code{"growing"} if \code{Max_diff > 0}
  \item \code{"shrinking"} if \code{Max_diff < 0}
  \item \code{"stable"} if \code{Max_diff = 0}
}

The first day has \code{NA} for \code{Max_diff} and \code{Day_status}.
}
\note{
The object returned by \code{daily.data()} can be plotted using
\code{plot()} because it is assigned class \code{"daily_output"}.
}
\examples{
\donttest{
data(nepa17)
daily_stats <- daily.data(df = nepa17[1:1000, ], TreeNum = 1)
head(daily_stats, 10)
}

}
\references{
King G, Fonti P, Nievergelt D, Büntgen U, Frank D (2013)
Climatic drivers of hourly to yearly tree radius variations along a 6°C natural warming gradient.
\emph{Agricultural and Forest Meteorology} 168:36–46.
\doi{10.1016/j.agrformet.2012.08.002}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_first_non_na}
\alias{dmgf_first_non_na}
\title{Return the first non-missing value}
\usage{
dmgf_first_non_na(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
First non-missing value, or \code{NA_real_}.
}
\description{
Return the first non-missing value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_is_scalar_finite}
\alias{dmgf_is_scalar_finite}
\title{Check whether a value is a finite numeric scalar}
\usage{
dmgf_is_scalar_finite(x)
}
\arguments{
\item{x}{Object to test.}
}
\value{
Logical scalar.
}
\description{
Check whether a value is a finite numeric scalar
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events_phase_plots.R
\name{plot_event_climate_relation}
\alias{plot_event_climate_relation}
\title{Plot event-based climate-response relationships}
\usage{
plot_event_climate_relation(
  event_data,
  climate_var,
  response_var,
  group_var = NULL,
  Year = NULL,
  DOY = NULL,
  add_smooth = TRUE,
  point_alpha = 0.7
)
}
\arguments{
\item{event_data}{Output of \code{dm_event_climate()}.}

\item{climate_var}{Name of the climate-derived column.}

\item{response_var}{Name of the response variable.}

\item{group_var}{Optional grouping variable for color.}

\item{Year}{Optional numeric year or vector of years for filtering.}

\item{DOY}{Optional numeric vector of length 2 for filtering.}

\item{add_smooth}{Logical. If \code{TRUE}, adds linear fit line.}

\item{point_alpha}{Point transparency.}
}
\value{
A \code{ggplot2} object.
}
\description{
Draws a scatterplot between an event-based climate variable and a response
variable and annotates the fitted linear relationship with R², slope,
and p-value.
}
\examples{
\donttest{
data(gf_nepa17)
data(ktm_clim_hourly)

zg <- phase.zg(df = gf_nepa17[1:800, ], TreeNum = 1)
evt_maxtwd <- dm_event_climate(
  zg,
  ktm_clim_hourly,
  event = "max_twd",
  windows = c(0, 3, 6, 12, 24),
  mean_vars = c("temp", "VPD", "RH"),
  max_vars  = c("VPD"),
  sum_vars  = c("prec")
)

if (all(c("VPD_mean_prev_6h", "max.twd") \%in\% names(evt_maxtwd)) &&
    any(is.finite(evt_maxtwd$VPD_mean_prev_6h)) &&
    any(is.finite(evt_maxtwd$max.twd))) {
  plot_event_climate_relation(
    evt_maxtwd,
    climate_var = "VPD_mean_prev_6h",
    response_var = "max.twd",
    group_var = "Phase"
  )
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nepa17.R
\docType{data}
\name{nepa17}
\alias{nepa17}
\title{Dendrometer data of Kathmandu for 2017}
\format{
A data frame with 8753 rows and 3 variables:
\describe{
  \item{\code{Time}}{datetime time of data recording}
  \item{\code{T2}}{double reading for first tree}
  \item{\code{T3}}{double reading for second tree}
}
}
\usage{
nepa17
}
\description{
Dendrometer data from three Chir pine tree collected in hourly resolution for 2017.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.evaluate.R
\name{dm.growth.evaluate}
\alias{dm.growth.evaluate}
\title{Compare dendrometer growth-fitting methods using fit statistics}
\usage{
dm.growth.evaluate(
  df,
  TreeNum = "all",
  methods = c("gam", "gompertz", "logistic", "richards", "loess", "spline",
    "double_gompertz", "double_richards"),
  years = "all",
  year_mode = c("yearly", "pooled"),
  fit_GRO = TRUE,
  site_mode = c("NH", "SH", "CS"),
  custom_veg_season = c(275, 274),
  growth_fraction = c(0.1, 0.9),
  min_unique_growth = 10,
  rate_threshold_fraction = 0.1,
  peak_separation_min = 10,
  valley_ratio_max = 0.4,
  min_relative_peak = 0.05,
  start_value_gompertz_parameters = list(a = NA_real_, b = NA_real_, k = NA_real_),
  start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v =
    1),
  start_value_double_gompertz_parameters = list(a = NA_real_, k = NA_real_, t0 =
    NA_real_, a2 = NA_real_, k2 = NA_real_, t02 = NA_real_),
  start_value_double_richards_parameters = list(a = NA_real_, k = NA_real_, t0 =
    NA_real_, v = 1, a2 = NA_real_, k2 = NA_real_, t02 = NA_real_, v2 = 1),
  loess_span = 0.2,
  spline_df = 10,
  verbose = TRUE
)
}
\arguments{
\item{df}{A data frame or tibble. The first column must be a time stamp and
the remaining selected columns must be numeric dendrometer series.}

\item{TreeNum}{Either `"all"`, a numeric vector selecting dendrometer series
by position, or a character vector of series names.}

\item{methods}{Character vector of fitting methods to evaluate.}

\item{years}{Either `"all"` or a character vector of vegetation-season labels.}

\item{year_mode}{Either `"yearly"` or `"pooled"`.}

\item{fit_GRO}{Logical. If `TRUE`, convert processed daily series to
cumulative growth before fitting.}

\item{site_mode}{Vegetation season definition. One of `"NH"`, `"SH"`, or
`"CS"`.}

\item{custom_veg_season}{Numeric vector of length 2 defining a custom season
in day-of-year coordinates when `site_mode = "CS"`.}

\item{growth_fraction}{Numeric vector of length 2 used by the fitting
functions to estimate cumulative-growth timing.}

\item{min_unique_growth}{Minimum number of unique cumulative-growth values
required before a fit is attempted.}

\item{rate_threshold_fraction}{Numeric scalar between 0 and 1. Passed to the
fitting functions where supported.}

\item{peak_separation_min}{Minimum peak separation in days for
[dm.growth.fit.double()].}

\item{valley_ratio_max}{Maximum allowed valley-to-peak ratio for
[dm.growth.fit.double()].}

\item{min_relative_peak}{Minimum relative peak height for
[dm.growth.fit.double()].}

\item{start_value_gompertz_parameters}{Optional starting values for Gompertz
fits.}

\item{start_value_richards_parameters}{Optional starting values for logistic
and Richards fits.}

\item{start_value_double_gompertz_parameters}{Optional starting values for
double-Gompertz fits.}

\item{start_value_double_richards_parameters}{Optional starting values for
double-Richards fits.}

\item{loess_span}{Span used when `method = "loess"`.}

\item{spline_df}{Degrees of freedom used when `method = "spline"`.}

\item{verbose}{Logical. If `TRUE`, prints progress messages.}
}
\value{
A tibble with one row per fitted series and fit, containing:
\describe{
  \item{method}{Fitting method used for the row.}
  \item{series}{Series name.}
  \item{fit_id}{Vegetation-season label or `"pooled"`.}
  \item{n_compare}{Number of observed days used for comparison.}
  \item{rmse}{Root mean squared error.}
  \item{mae}{Mean absolute error.}
  \item{bias}{Mean fitted minus observed value.}
  \item{r2}{Coefficient of determination.}
  \item{correlation}{Pearson correlation between observed and fitted values.}
  \item{nrmse}{RMSE divided by the observed range.}
  \item{rss}{Residual sum of squares.}
  \item{aic_approx}{Approximate AIC based on RSS and effective parameter count.}
  \item{bic_approx}{Approximate BIC based on RSS and effective parameter count.}
  \item{k_effective}{Effective parameter count used in the approximate
    information criteria.}
  \item{converged}{Logical convergence flag returned by the fitting function.}
  \item{model_note}{Model note, warning, or error message returned by the
    fitting function.}
}
}
\description{
Fits one or more growth models to dendrometer series and returns a compact
table of evaluation statistics only.

This function is intended for method comparison rather than data extraction.
It runs the selected fitting methods, aligns observed and fitted values on
all non-missing observation days, and calculates goodness-of-fit measures for
each fitted series and vegetation-season fit.

Supported single-curve methods are `"gam"`, `"gompertz"`, `"logistic"`,
`"richards"`, `"loess"`, and `"spline"`, which are fitted using
[dm.growth.fit()]. Supported bimodal methods are `"double_gompertz"` and
`"double_richards"`, which are fitted using [dm.growth.fit.double()].

The function returns only an evaluation table and does not return fitted
curves, processed data, or parameter objects. It is therefore useful for
comparing alternative fitting methods across series, years, or sites before
selecting a final modeling approach.
}
\details{
Evaluation statistics are calculated by comparing observed daily values
against fitted daily values on all days where observed values are available.

The returned metrics have the following interpretation:
\itemize{
  \item `rmse`: root mean squared error. Smaller values indicate closer fit.
  \item `mae`: mean absolute error. Smaller values indicate closer fit and are
    less sensitive to large deviations than RMSE.
  \item `bias`: mean fitted minus observed value. Positive values indicate
    overestimation and negative values indicate underestimation.
  \item `r2`: coefficient of determination, computed from residual and total
    sums of squares. Larger values indicate better fit.
  \item `correlation`: Pearson correlation between observed and fitted values.
    Larger values indicate stronger agreement in shape.
  \item `nrmse`: normalized RMSE, calculated as RMSE divided by the observed
    value range. This allows comparison across series with different
    magnitudes.
  \item `rss`: residual sum of squares.
  \item `aic_approx` and `bic_approx`: approximate information criteria based
    on RSS and an effective parameter count rather than the exact likelihood
    returned by each modeling function.
}

The columns `aic_approx` and `bic_approx` are labeled as approximate because
they are not extracted from the original model objects through a unified
likelihood interface. Instead, they are computed from the residual sum of
squares and an effective number of fitted parameters. They are therefore most
useful for relative comparison among methods fitted to the same dataset.

Negative values of `aic_approx` or `bic_approx` are not an error. They occur
naturally when the average residual variance is smaller than 1, because the
logarithmic term in the information-criterion formula becomes negative. In
practice, the absolute sign is not important; only differences among methods
fitted to the same data should be interpreted.

For methods with flexible smoothness, such as GAM or spline, the effective
parameter count is approximate. Consequently, `aic_approx` and `bic_approx`
should be treated as heuristic ranking measures rather than exact
likelihood-based criteria.
}
\seealso{
[dm.growth.fit()], [dm.growth.fit.double()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_parse_time}
\alias{dmea_parse_time}
\title{Parse date-time values for SEA}
\usage{
dmea_parse_time(x)
}
\arguments{
\item{x}{A vector of date-time, date, or character values.}
}
\value{
A \code{POSIXct} vector.
}
\description{
Parse date-time values for SEA
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.detrend.R
\name{dm.detrend.fit}
\alias{dm.detrend.fit}
\title{Detrend and standardize dendrometer series from growth-fit residuals}
\usage{
dm.detrend.fit(
  x,
  series = NULL,
  seasons = NULL,
  epsilon = 1e-06,
  keep_na_rows = FALSE
)
}
\arguments{
\item{x}{An object of class \code{"dm_growth_fit"} returned by
[dm.growth.fit()] or [dm.growth.fit.double()].}

\item{series}{Optional character vector of dendrometer series to retain.
Default is \code{NULL}, meaning all available series are used.}

\item{seasons}{Optional character vector of vegetation-season labels to
retain. Default is \code{NULL}, meaning all available seasons are used.}

\item{epsilon}{Small positive constant added after shifting residuals by their
seasonal minimum. Default is \code{1e-6}. Use \code{0} to allow exact zeros.}

\item{keep_na_rows}{Logical. If \code{TRUE}, rows are retained even when the
observed or fitted value is missing; detrended values remain \code{NA}.
If \code{FALSE} (default), rows with missing observed or fitted values are
removed from the long output before standardization.}
}
\value{
A list of class \code{"dm_detrended"} with elements:
\describe{
  \item{call}{The matched function call.}
  \item{detrended_data}{Wide-format tibble with metadata columns and one
    detrended standardized series column per dendrometer.}
  \item{detrended_long}{Long-format tibble containing original daily values,
    reconstructed fitted values on the original scale, residuals, shifted
    residuals, and detrended standardized values.}
  \item{parameters}{Per-series and per-season summary table containing
    baseline values and residual standardization parameters.}
  \item{source_fit_statistics}{The original \code{x$fit_statistics} table.}
}
}
\description{
Creates detrended standardized dendrometer series from objects returned by
[dm.growth.fit()] or [dm.growth.fit.double()].

The function compares the original daily dendrometer series
(`x$original_daily_data`) with the fitted growth curve from `x$fitted_data`.
Because fitted values are stored on the processed scale used for modelling,
the function first reconstructs fitted values on the original daily scale by
adding back the seasonal baseline (the first non-missing original daily value
of each series within each vegetation season).

Residuals are then calculated as:
\deqn{residual = observed_{daily} - fitted_{original\ scale}}

To obtain a detrended standardized series with mean equal to 1 and no negative
values, residuals are transformed within each \code{series × season_label} as:
\deqn{z = \frac{residual - \min(residual) + \epsilon}{\mathrm{mean}(residual - \min(residual) + \epsilon)}}

where \eqn{\epsilon > 0} is a small constant ensuring strictly positive values.
If all residuals within a season are identical, the standardized series
becomes a constant vector of 1.
}
\details{
The function uses \code{x$original_daily_data}, which must be present in the
input object. If the object was created by an older version of
[dm.growth.fit()] or [dm.growth.fit.double()] that did not store
\code{original_daily_data}, the function will stop with an informative error.

Standardization is carried out separately within each vegetation season. This
means the mean of the detrended standardized series is 1 for each
\code{series × season_label}, not necessarily across the entire dataset.
}
\examples{
\donttest{
data(gf_nepa17)

fit1 <- dm.growth.fit(
  df = gf_nepa17,
  TreeNum = 1:2,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)

det1 <- dm.detrend.fit(fit1)
head(det1$detrended_data)
head(det1$parameters)

fit2 <- dm.growth.fit.double(
  df = gf_nepa17,
  TreeNum = 1,
  method = "gompertz",
  year_mode = "yearly",
  verbose = FALSE
)

det2 <- dm.detrend.fit(fit2)
head(det2$detrended_long)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_fit_single_fallback}
\alias{dmgfd_fit_single_fallback}
\title{Fit single-curve fallback}
\usage{
dmgfd_fit_single_fallback(
  method,
  x,
  y,
  x_pred,
  fixed_a_value = NA_real_,
  fix_a_to_observed_max = FALSE,
  start_value_double_gompertz_parameters,
  start_value_double_richards_parameters
)
}
\arguments{
\item{method}{Original double method.}

\item{x}{Observed season-day values.}

\item{y}{Observed cumulative growth values.}

\item{x_pred}{Prediction season-day values.}

\item{fixed_a_value}{Optional fixed asymptote.}

\item{fix_a_to_observed_max}{Logical fixed-asymptote flag.}

\item{start_value_double_gompertz_parameters}{Double-Gompertz starts.}

\item{start_value_double_richards_parameters}{Double-Richards starts.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit single-curve fallback
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{clim.twd.test}
\alias{clim.twd.test}
\title{Statistical testing for clim.twd output}
\usage{
clim.twd.test(
  x,
  tree_info = NULL,
  parameter = c("lag_to_below_zero", "lag_to_max_adverse_decline", "lag_to_return_zero",
    "max_adverse_decline_value", "change_end_adverse", "change_end_full_period",
    "continuous_end_full_period"),
  compare_by = c("tree", "species", "site", "species_site"),
  within = c("all", "year", "ID"),
  test_method = c("auto", "t_test", "welch_t", "wilcox", "anova", "kruskal"),
  ids = NULL,
  years = NULL,
  months = NULL,
  doy_window = NULL,
  year_window = NULL,
  pairwise = TRUE,
  p_adjust_method = "BH",
  min_n_per_group = 2
)
}
\arguments{
\item{x}{An object of class \code{"clim_twd_output"} returned by
\code{clim.twd()}.}

\item{tree_info}{Optional metadata table describing trees. It must contain a
column named \code{tree}, and may additionally contain \code{species} and
\code{site}. A named character vector is also accepted for species-only
mapping, where names are trees and values are species.}

\item{parameter}{Character. Parameter from \code{x$period_info} to test.
Supported examples include:
\describe{
  \item{`"lag_to_below_zero"`}{Lag from adverse start until relative change
  first drops below zero.}
  \item{`"lag_to_max_adverse_decline"`}{Lag from adverse start until maximum
  adverse decline is reached.}
  \item{`"lag_to_return_zero"`}{Lag from adverse start until relative change
  returns to zero or above.}
  \item{`"max_adverse_decline_value"`}{Most negative relative change during
  the adverse phase.}
  \item{`"change_end_adverse"`}{Relative change at the end of the adverse phase.}
  \item{`"change_end_full_period"`}{Relative change at the end of the full period.}
  \item{`"continuous_end_full_period"`}{Continuous end value across periods.}
}}

\item{compare_by}{Character. Grouping variable to compare. One of
\code{"tree"}, \code{"species"}, \code{"site"}, or \code{"species_site"}.}

\item{within}{Character. Defines the comparison scenario:
\describe{
  \item{`"all"`}{Compare groups across all retained IDs.}
  \item{`"year"`}{Compare groups separately within each adverse-start year.}
  \item{`"ID"`}{Compare groups separately within each ID.}
}}

\item{test_method}{Character. One of \code{"auto"}, \code{"t_test"},
\code{"welch_t"}, \code{"wilcox"}, \code{"anova"}, or \code{"kruskal"}.
With \code{"auto"}, the function uses:
\itemize{
  \item Welch t-test or one-way ANOVA when group distributions look
  approximately normal,
  \item Wilcoxon or Kruskal-Wallis otherwise.
}}

\item{ids}{Optional numeric vector of IDs to retain before testing.}

\item{years}{Optional numeric vector of years to retain based on
\code{adverse_start}.}

\item{months}{Optional numeric vector of months (1--12) to retain based on
\code{adverse_start}.}

\item{doy_window}{Optional numeric vector of length 2 defining the allowed
day-of-year window of \code{adverse_start}. Wrapped windows such as
\code{c(300, 50)} are supported.}

\item{year_window}{Optional numeric vector of length 2 defining the allowed
year range for \code{adverse_start}.}

\item{pairwise}{Logical. If \code{TRUE}, pairwise post-hoc tests are computed
for strata with more than two groups when applicable.}

\item{p_adjust_method}{Character. P-value adjustment method for pairwise tests.
Default is \code{"BH"}.}

\item{min_n_per_group}{Minimum number of non-missing observations required per
group before testing. Default is \code{2}.}
}
\value{
A list of class \code{"clim_twd_test"} containing:
\describe{
  \item{data_used}{Filtered test data used for the analyses.}
  \item{test_results}{Main test results by stratum.}
  \item{pairwise_results}{Pairwise comparison table where available.}
  \item{group_summary}{Descriptive summaries by stratum and group.}
  \item{settings}{Settings used for the analysis.}
}
}
\description{
Performs hypothesis tests on parameters derived from \code{clim.twd()} output.

This function is designed for scenarios such as:
\itemize{
  \item species comparison as a whole,
  \item species comparison by year,
  \item site comparison as a whole,
  \item site comparison by year,
  \item species-within-site comparison,
  \item ID-wise comparisons.
}

The function uses the \code{period_info} table from \code{clim.twd()} and
tests selected parameters across groups such as trees, species, sites, or
species-site combinations.
}
\examples{
\donttest{
rel_out <- clim.twd(
  df = gf_nepa17,
  Clim = ktm_rain17,
  thresholdClim = "<10",
  thresholdDays = ">5",
  showPlot = FALSE
)

tree_info <- data.frame(
  tree = c("T2", "T3"),
  species = c("Pinus", "Pinus"),
  site = c("Ktm", "Ktm")
)

# species comparison across all retained IDs
tst1 <- clim.twd.test(
  rel_out,
  tree_info = tree_info,
  parameter = "max_adverse_decline_value",
  compare_by = "species",
  within = "all"
)

summary(tst1)
plot(tst1)

# species comparison by year
tst2 <- clim.twd.test(
  rel_out,
  tree_info = tree_info,
  parameter = "lag_to_return_zero",
  compare_by = "species",
  within = "year"
)

# site comparison by year
tst3 <- clim.twd.test(
  rel_out,
  tree_info = tree_info,
  parameter = "change_end_full_period",
  compare_by = "site",
  within = "year",
  test_method = "kruskal"
)
}

}
